#include "U8constant.h"
#include "nxgetsocket.h"
#include "nxxdrstream.h"
#include "nx.h"

// fonctions ecritures

int
EcritFichierSocketXdr(nxXdrstream* fichier_socket_xdr)
{
	int ier = 0;

	if(!fichier_socket_xdr || !fichier_socket_xdr->rdbuf())
	{
	    cerr << "EcritFichierSocketXdr> Pb ouverture canal de lecture xdr :"
			" On ne lit pas " << endl;
	    return -1;
	}

	cout << "EcritFichierSocketXdr> fichier_socket_xdr " <<
	    (void*) fichier_socket_xdr << endl;

    // on lit et on ecrit

	XDR* xr = fichier_socket_xdr->xdrread();
	XDR* xw = fichier_socket_xdr->xdrwrite();
	double d1 = 2.;
	int i1 = 5;
	double d2 = 4.;
	int i2 = 7;
	double d3 = 3.6;
	int i3 = 0;
	int re = 0;

	xdr_double(xw,&d1);
	xdr_int(xw,&i1);
	xdrrec_endofrecord (xw, TRUE);

	xdr_int(xr,&re);
	xdrrec_skiprecord (xr);

	xdr_double(xw,&d2);
	xdr_int(xw,&i2);
	xdrrec_endofrecord (xw, TRUE);

	xdr_int(xr,&re);
	xdrrec_skiprecord (xr);

	xdr_double(xw,&d3);
	xdr_int(xw,&i3);
	xdrrec_endofrecord (xw, TRUE);

	xdr_int(xr,&re);
	xdrrec_skiprecord (xr);

	cout << "EcritFichierSocketXdr> re = " << re << endl;

	return ier;
}

int
EcritFichierSocketBinaire(nxstream* fichier_socket_binaire)
{
	int ier = 0;

	if(!fichier_socket_binaire || !fichier_socket_binaire->rdbuf())
	{
	    cerr << "EcritFichierSocketBinaire> Pb ouverture canal de "
		"lecture binaire : On ne lit pas" << endl;
	    return -1;
	}

	cout << "EcritFichierSocketBinaire> fichier_socket_binaire " <<
	    (void*) fichier_socket_binaire << endl;

    // on lit

	double d1 = 2.;
	int i1 = 5;
	double d2 = 4.;
	int i2 = 7;
	int re = 0;

	fichier_socket_binaire->nxstream::write(&d1,1);
	fichier_socket_binaire->nxstream::read(&re,1);
	fichier_socket_binaire->nxstream::write(&i1,1);
	fichier_socket_binaire->nxstream::write(&d2,1);
	fichier_socket_binaire->nxstream::read(&re,1);
	fichier_socket_binaire->nxstream::write(&i2,1);

	cout << "EcritFichierSocketBinaire> re = " << re << endl;

	return ier;
}

// main

int
main(int argc,char** argv)
{
    // test XDR

	const char* machine = getenv("RHOST");
	const char* destxdr = "+xdr+brochard+@ie32";
	const char* destbin = "+binaire@ie32";

	cerr << "Usage : client [port serveur (.nx_unix, .nx_inet ou ...)]" <<
	    endl;

	if(!machine)
	{
	    cerr << "main> Pb $RHOST" << endl;
	    return 1;
	}

	char nomfichxdr[1024];
	char nomfichbin[1024];

	strcpy(nomfichxdr,machine);
	strcat(nomfichxdr,"+");

	if(argc == 2) strcat(nomfichxdr,argv[1]);
	else strcat(nomfichxdr,"0");

	strcat(nomfichxdr,destxdr);

	FILE* client_xdr = fnxopen(nomfichxdr,"r");

	if(!client_xdr) return 0;

	nxXdrstream* nx_client_xdr = new nxXdrstream(client_xdr);

	if(!nx_client_xdr)
	{
	    cerr << "main> Pb ouverture client dans " << __FILE__ << endl;
	    return 1;
	}

	EcritFichierSocketXdr(nx_client_xdr);

	delete nx_client_xdr;

    // test binaire nx

	strcpy(nomfichbin,machine);
	strcat(nomfichbin,"+");

	if(argc == 2) strcat(nomfichbin,argv[1]);
	else strcat(nomfichbin,"0");

	strcat(nomfichbin,destbin);

	FILE* client_binaire = fnxopen(nomfichbin,"r");

	if(!client_binaire) return 0;

	nxstream* nx_client_binaire = new nxstream(client_binaire);

	if(!nx_client_binaire)
	{
	    cerr << "main> Pb ouverture client dans " << __FILE__ << endl;
	    return 1;
	}

	EcritFichierSocketBinaire(nx_client_binaire);

	delete nx_client_binaire;

	return 0;
}
