#include "U8constant.h"
#include "nxgetsocket.h"
#include "nxxdrstream.h"
#include "poulpe/U8serveur.h"
#include "serveur.h"
#include "U8signal.h"

// variables globales (utile pour kill soft seulement)

static nxstream* nx_serveur_binaire_unix = NULL;
static nxstream* nx_serveur_binaire_inet = NULL;
static SocketXdr socket_xdr;
static SocketBinaire socket_binaire;

// handler pour kill soft

void
server_quit(int sig)
{

	if(sig)
	{
	    cout << "server_quit> Signal recu : " << sig << endl;
	}

	delete nx_serveur_binaire_unix;
	delete nx_serveur_binaire_inet;
	exit(0);
}

// fonction lecture

int
LitFichierSocketXdr(nxstream* fichier_socket_xdr,U8InputId input_id)
{
	int ier = 0;

	cout << "LitFichierSocketXdr> input_id " << input_id << endl;

	if(input_id != -1)
	{
	    socket_xdr.RemoveInputDriver(input_id);

	    cout << "LitFichierSocketXdr> nouvelle lecture sur meme "
		"socket" << endl;
	}

	if(!fichier_socket_xdr || !fichier_socket_xdr->rdbuf())
	{
	    cerr << "LitFichierSocketXdr> Pb ouverture canal de "
		"lecture binaire : On ne lit pas " << endl;
	    return -1;
	}

	cout << "LitFichierSocketXdr> fichier_socket_xdr " <<
	    (void*) fichier_socket_xdr << endl;

    // on lit

	XDR* xr = ((nxXdrstream*) fichier_socket_xdr)->xdrread();
	XDR* xw = ((nxXdrstream*) fichier_socket_xdr)->xdrwrite();
	double dd = 0.;
	int cont = 0;
	int re = 438;

    // on s'arrete a un endroit arbitraire avant le end of record
    // et on se reconnecte correctement

	static int recon = 0;

	if(!recon) xdr_double(xr,&dd);
	
	if(dd != 2.)
	{
	    xdr_int(xr,&cont);
	    xdrrec_skiprecord (xr);

	    xdr_int(xw,&re);
	    xdrrec_endofrecord (xw, TRUE);
	    recon = 0;
	}
	else
	{
	    recon = 1;
	    cont = 1;
	}

	cout << "LitFichierSocketXdr> dd = " << dd << '\n';
	cout << "LitFichierSocketXdr> cont = " << cont << endl;

    // si close

	if(cont == 0)
	{
	    delete fichier_socket_xdr;

	    cout << "LitFichierSocketXdr> close fichier_socket_xdr" << endl;
	}
	else
	{
	    socket_xdr.AddInputDriver(fichier_socket_xdr,0);

	    cout << "LitFichierSocketXdr> code fin de fichier = " <<
		cont << endl;
	}

	return ier;
}


int
LitFichierSocketBinaire(nxstream* fichier_socket_binaire,
		U8InputId input_id)
{
	int ier = 0;

	cout << "LitFichierSocketBinaire> input_id " << input_id << endl;

	if(input_id != -1)
	{
	    socket_binaire.RemoveInputDriver(input_id);

	    cout << "LitFichierSocketBinaire> nouvelle lecture sur "
		"meme socket" << endl;
	}

	if(!fichier_socket_binaire || !fichier_socket_binaire->rdbuf())
	{
	    cerr << "LitFichierSocketBinaire> Pb ouverture canal de "
		"lecture binaire : On ne lit pas " << endl;
	    return -1;
	}

	cout << "LitFichierSocketBinaire> fichier_socket_binaire " <<
	    (void*) fichier_socket_binaire << endl;

    // on lit

	double dd = 0.;
	int cont = 0;
	int re = 438;

	fichier_socket_binaire->read(&dd,1);

	if(fichier_socket_binaire->fail()) cerr << "LitFichierSocketBinaire> "
					       "read1: car non lu" << endl;

	fichier_socket_binaire->write(&re,1);
	fichier_socket_binaire->read(&cont,1);

	if(fichier_socket_binaire->fail()) cerr << "LitFichierSocketBinaire> "
					       "read2: car non lu" << endl;

	cout << "LitFichierSocketBinaire> dd = " << dd << '\n';
	cout << "LitFichierSocketBinaire> cont = " << cont << endl;

    // si close

	if(cont == 0)
	{
	    delete fichier_socket_binaire;

	    cout << "LitFichierSocketBinaire> close "
		"fichier_socket_binaire" << endl;
	}
	else
	{
	    socket_binaire.AddInputDriver(fichier_socket_binaire,0);

	    cout << "LitFichierSocketBinaire> code "
		"fin de fichier " << cont << endl;
	}

	return ier;
}

// RootBatch

RootBatch::RootBatch() : ElementInput()
{
}

RootBatch::~RootBatch()
{
}

void
RootBatch::LectureInputCB(int fid,U8InputId)
{
	char handler_name[10];

	memset(handler_name,0,sizeof(handler_name));

	nxstream* const fdti = GetNxstream(fid);
	nxstream* fichier_socket = NULL;
	
	if(fdti)
	{
	    fichier_socket = new nxstream(*fdti,handler_name);
	    cout << "RootBatch> fichier_socket " <<
		(void*) fichier_socket << endl;
	}
	else
	{
	    cerr << "RootBatch> UNKNOWN fid " << fid << endl;
	    return;
	}

	cout << "RootBatch> handler_name " << handler_name << endl;

	if(!strcmp(handler_name,"xdr"))
	{
	    nxXdrstream* fichier_socket_xdr = new nxXdrstream(fichier_socket);
	    cout << "RootBatch> fichier_socket_xdr " <<
		(void*) fichier_socket_xdr << endl;
	    LitFichierSocketXdr(fichier_socket_xdr);
	}
	else if(!strcmp(handler_name,"binaire"))
	{
	    LitFichierSocketBinaire(fichier_socket);
	}
	else
	{
	    LitFichierSocketBinaire(fichier_socket);
	}

}

// xdr

void
SocketXdr::LectureInputCB(int fid,U8InputId input_id)
{
	nxstream* const fdti = GetNxstream(fid);

	if(fdti)
	{
	    LitFichierSocketXdr(fdti,input_id);
	}

}

// binaire

void
SocketBinaire::LectureInputCB(int fid,U8InputId input_id)
{
	nxstream* const fdti = GetNxstream(fid);

	if(fdti)
	{
	    LitFichierSocketBinaire(fdti,input_id);
	}

}

// main

int
main()
{
	fnx_lance_rpc_port();

	NxSocket serveur_binaire_unix =
	    NxGetNxSocket("SERVER+/UNIX/.nx_unix@ie32","r");

	NxSocket serveur_binaire_inet =
	    NxGetNxSocket("SERVER+.nx_inet@ie32","r");

	cout << "main> Port number = " << serveur_binaire_inet.Number << endl;
	cout << "main> Port name = " << serveur_binaire_unix.PortName << endl;

	nx_serveur_binaire_unix = new nxstream(serveur_binaire_unix.PtFile);
	nx_serveur_binaire_inet = new nxstream(serveur_binaire_inet.PtFile);

	if(!nx_serveur_binaire_unix || !nx_serveur_binaire_inet)
	{
	    cerr << "main> Pb ouverture serveur dans " << __FILE__ << endl;
	    delete nx_serveur_binaire_unix;
	    delete nx_serveur_binaire_inet;
	    return 1;
	}

    // gestion du kill soft (reponse au kill -TERM)

	sigset(SIGTERM,SIG_PF(server_quit));

    // lancement de la boucle serveur

	ElementBase* root = new RootBatch();
	ElementInput* const root_input = root->ElInput();

	root_input->AddInputDriver(nx_serveur_binaire_unix,0);
	root_input->AddInputDriver(nx_serveur_binaire_inet,0);

	cout << "main> Serveur test lance" << endl;

	root_input->ScruteEvenement(0);
	return 0;
}
