#include <stdio.h>
#include <malloc.h>

typedef struct
{
	unsigned signe      : 1;
	unsigned exposant   : 8;
	unsigned mantisse_a : 15;
	unsigned mantisse_b : 8;
} _ieee32_type;

typedef struct
{
	unsigned signe      : 1;
	unsigned exposant   : 15;
	unsigned bit_47     : 1;
	unsigned mantisse_a : 15;
	unsigned mantisse_b : 32;
} _cray64_type;

int 
Nxconv_ieee32_To_cray64(_ieee32_type *s, _cray64_type *d, int nb)

{
	int             i;

	for (i = 0; i < nb; i++) 
	{
	    if ((s[i].mantisse_a != 0) || (s[i].mantisse_b != 0) || 
			    (s[i].exposant != 0) || (s[i].signe != 0))
	    {
		d[i].signe = s[i].signe;

		d[i].exposant = ((int)s[i].exposant - 127) + 16384 + 1;

		d[i].bit_47 = 1;

		d[i].mantisse_a = s[i].mantisse_a;
		d[i].mantisse_b = s[i].mantisse_b;
	    } 
	    else
	    {
		d[i].mantisse_a = 0;
		d[i].mantisse_b = 0;
		d[i].bit_47     = 0;
		d[i].exposant   = 0;
		d[i].signe      = 0;
	    };

	};

}

int 
Nxconv_cray64_To_ieee32(_cray64_type *s, _ieee32_type *d, int nb)

{
	int             i;

	printf("Taille _cray64_type : %d octets\n",sizeof(_cray64_type));
	printf("Taille _ieee32_type : %d octets\n",sizeof(_ieee32_type));

	for (i = 0; i < nb; i ++) 
	{
	    if (s[i].bit_47 != 0) 
	    {
		d[i].exposant = ((int)s[i].exposant - 16384) + 127 - 1;
		if (d[i].exposant < 0) 
		{	/* underflow */
		    d[i].mantisse_a = 0;
		    d[i].mantisse_b = 0;
		    d[i].exposant   = 0;
		    d[i].signe      = 0;
		} 
		else if (d[i].exposant > 255) 
		{	/* overflow  */
		    d[i].signe = s[i].signe;            
		    d[i].exposant = 255;	/* max float ieee */
		} 
		else 
		{
		    d[i].signe      = s[i].signe;            
		    d[i].mantisse_a = s[i].mantisse_a;
		    d[i].mantisse_b = s[i].mantisse_b;
		};
	    } 
	    else
	    {
		d[i].mantisse_a = 0;
		d[i].mantisse_b = 0;
		d[i].exposant   = 0;
		d[i].signe      = 0;
	    };

	};

}

main()
{
#define nbval 10
#define fact 3.14159

	int cr,i;
	float tabr[nbval];
	char *buff;
	char *filename1 = "ftest1";
	FILE *fd1,*fopen();

#if (defined CRAY && !defined _CRAYMPP)

	buff = calloc(nbval,sizeof(float)/2);

	fd1 = fopen(filename1,"w");

	for (i=0 ; i<nbval ; tabr[i] = fact*(i+1-nbval/2) , i++);
	for (i=0;i<nbval;printf("%f ",tabr[i++]));
	printf("\n");
	for (i=0;i<nbval;printf("%16x ",tabr[i++]));
	printf("\n");

	Nxconv_cray64_To_ieee32(tabr,buff,nbval);

	fwrite(buff, nbval, sizeof(float)/2, fd1);

	fclose(fd1);

/* ----------------------------------------- */

	for (i=0 ; i<nbval ; tabr[i] = 0.00 , i++);

	fd1 = fopen(filename1,"r");

	fread(buff, nbval, sizeof(float)/2, fd1);

	Nxconv_ieee32_To_cray64(buff,tabr,nbval);

	fclose(fd1);

	for (i=0;i<nbval;printf("%f ",tabr[i++]));
	printf("\n");

/* ----------------------------------------- */

#else

/* ----------------------------------------- */

	buff = calloc(nbval,2*sizeof(float));

	fd1 = fopen(filename1,"w");

	for (i=0 ; i<nbval ; tabr[i] = fact*(i+1-nbval/2) , i++);
	for (i=0;i<nbval;printf("%f ",tabr[i++]));
	printf("\n");
	for (i=0;i<nbval;printf("%08x ",tabr[i++]));
	printf("\n");

	Nxconv_ieee32_To_cray64(tabr,buff,nbval);

	fwrite(buff, nbval, 2*sizeof(float), fd1);

	fclose(fd1);

/* ----------------------------------------- */

	for (i=0 ; i<nbval ; tabr[i] = 0.00 , i++);

	fd1 = fopen(filename1,"r");

	fread(buff, nbval, 2*sizeof(float), fd1);

	Nxconv_cray64_To_ieee32(buff,tabr,nbval);

	fclose(fd1);

	for (i=0;i<nbval;printf("%f ",tabr[i++]));
	printf("\n");

#endif

}
