/*                           N X - A M                                 */
/*                           ---------                                 */
/*                        FORTRAN interface                            */    

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992    MA/AIM";
#endif

/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nx.h"
#include "nxconv.h"
#include "machine_types.h"

#define FTNFILDES nx_desc_table[*fd_index].dp.fd

/* ------------------------------------------------------------------------ */
/*             formes de prototypes possibles                               */
/* ------------------------------------------------------------------------ */

/* Pour generer les appels FORTRAN, IBM et hp ont une structure speciale */
/* oui, mais :                                                           */
/* si on mets l'option -qextname au compilo Fortran, il cherche des noms */
/* avec "_", meme sur IBM                                                */

/* Pour generer les appels FORTRAN, CRAY a une structure speciale */

#ifdef CRAY_PROTO
#   undef CRAY_PROTO
#endif

#if defined(CRAY)
#   define CRAY_PROTO
#endif

/* ------------------------------------------------------------------------ */
/*                    macros utiles                                         */
/* ------------------------------------------------------------------------ */

#define STR_FOR2C(for_name, for_len,c_name,max_len) \
	{ \
	  int _lg ; \
	  _lg = noblkq(for_name,MIN((int)for_len,(int)max_len)) ; \
	  memcpy(c_name,for_name,_lg) ; \
	  c_name[_lg] = '\0' ; \
	}

#define SOCDES nx_desc_table[*sd_index].dp.fd
static FILE* stdfile[3] = { stdin , stdout , stderr } ;
static int noblkq(const char* str,int len) ;
extern DESC nx_desc_table[];

/* ------------------------------------------------------------------------ */

/* ============================================================ */
/*			====>     NXOPEN     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXOPEN(_fcd name,_fcd type,int_type* fd_index,int_type* rc)
{
	char c_name[FILE_MAX_LEN + 1] ;
	char c_type[TYPE_MAX_LEN + 1] ;
	FILE *fd;

	STR_FOR2C(_fcdtocp(name),_fcdlen(name),c_name,FILE_MAX_LEN);
	STR_FOR2C(_fcdtocp(type),_fcdlen(type),c_type,TYPE_MAX_LEN);
	*rc = 0;

	if((fd = fnxopen(c_name,c_type)) == NULL)
	{
	    *rc = -1;
	}
	else
	    *fd_index = (int_type)fileno(fd);
}
#else
void
nxopen_(char* name,char* type,int_type* fd_index,int_type* rc,
        int name_lgth,int type_lgth)
{
	char c_name[FILE_MAX_LEN + 1] ;
	char c_type[TYPE_MAX_LEN + 1] ;
	FILE *fd;

	STR_FOR2C(name,name_lgth,c_name,FILE_MAX_LEN);
	STR_FOR2C(type,type_lgth,c_type,TYPE_MAX_LEN);
	*rc = 0;

	if((fd = fnxopen(c_name,c_type)) == NULL)
	    *rc = -1;
	else
	    *fd_index = (int_type)fileno(fd);

}
void
nxopen(char* name,char* type,int_type* fd_index,int_type* rc,
       int name_lgth,int type_lgth)
{
       nxopen_(name,type,fd_index,rc,name_lgth,type_lgth);
}
#endif

/* ============================================================ */
/*			====>     NXACCEPT     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXACCEPT(int_type* sd_index,int_type* fd_index,int_type* rc)
{
	FILE* fd;

	if (!(fd = fnxaccept(SOCDES)))
	{
	    *rc = -1;
	}
	else
	{
	    *fd_index = (int_type)fileno(fd);
	    *rc = 0;
	}
}
#else
void nxaccept_(int_type* sd_index,int_type* fd_index,int_type* rc)
{
	FILE* fd;

	if (!(fd = fnxaccept(SOCDES)))
	{
	    *rc = -1;
	}
	else
	{
	    *fd_index = (int_type)fileno(fd);
	    *rc = 0;
	}
}

void nxaccept(int_type* sd_index,int_type* fd_index,int_type* rc)
{
        nxaccept_(sd_index,fd_index,rc);
}
#endif

/* ============================================================ */
/*			====>     NXSELECT     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXSELECT(int_type* nb,int_type* tabfd_index,int_type* fd_index,
              int_type* mode,int_type* rc)
{
	FILE* tabfd[NX_MAX_DESC];
	FILE* fd;
	int_type i = 0;

	for (;i<*nb;i++)
	{
	    tabfd[i] = nx_desc_table[tabfd_index[i]].dp.fd;
	}

	fd = fnxselect((int)*nb,tabfd,*mode,(int *)rc);

	if (*rc == 1)
	    *fd_index = (int_type)fileno(fd);

}
#else
void nxselect_(int_type* nb,int_type* tabfd_index,
               int_type* fd_index,int_type* mode,int_type* rc)
{
	FILE* tabfd[NX_MAX_DESC];
	FILE* fd;
	int_type i = 0;

	for (;i<*nb;i++)
	{
	    tabfd[i] = nx_desc_table[tabfd_index[i]].dp.fd;
	}

	fd = fnxselect((int)*nb,tabfd,*mode,(int *)rc);

	if (*rc == 1)
	    *fd_index = (int_type)fileno(fd);

}

void nxselect(int_type* nb,int_type* tabfd_index,
              int_type* fd_index,int_type* mode,int_type* rc)
{
        nxselect_(nb,tabfd_index,fd_index,mode,rc);
}

#endif


/* ============================================================ */
/*			====>     NXCLOSE     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXCLOSE(int_type* fd_index,int_type* rc)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE;
	}

	*rc = (int_type)fnxclose(FTNFILDES) ;
}
#else
void nxclose_(int_type* fd_index,int_type* rc)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE;
	}

	*rc = (int_type)fnxclose(FTNFILDES) ;
}

void nxclose(int_type* fd_index,int_type* rc)
{
        nxclose_(fd_index,rc);
}

#endif


/* ============================================================ */
/*		   ====>  NXLPORT     <====                     */
/* ============================================================ */

#ifdef CRAY_PROTO
void NXLPORT()
{
	fnx_lance_rpc_port();
}
#else
void nxlport_()
{
	fnx_lance_rpc_port();
}

void nxlport()
{
	fnx_lance_rpc_port();
}
#endif

/* ============================================================ */
/*			====>     NXWRITE      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXWRITE(register int_type* fd_index,_fcd buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if(*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	if((*lgecrit = (int_type)fnxwrite(FTNFILDES, _fcdtocp(buffer), (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void nxwrite_(register int_type* fd_index,char* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc,int lb)
{
	*rc = 0;
	
	if(*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if((*lgecrit = (int_type)fnxwrite(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}

void nxwrite(register int_type* fd_index,char* buffer,register int_type* lg,
             register int_type* lgecrit,register int_type* rc,int lb)
{
        nxwrite_(fd_index,buffer,lg,lgecrit,rc,lb);
}
#endif

/* ============================================================ */
/*			====>     NXNBKWI      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXNBKWI(register int_type* fd_index,int_type* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	if ((*lgecrit = (int_type)fnxnbkwi(FTNFILDES, (int *)buffer, (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void nxnbkwi_(register int_type* fd_index,int_type* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lgecrit = (int_type)fnxnbkwi(FTNFILDES, (int *)buffer, (int)*lg)) <= 0)
	    *rc = -1;


}

void nxnbkwi(register int_type* fd_index,int_type* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
        nxnbkwi_(fd_index,buffer,lg,lgecrit,rc);
}

#endif

/* ============================================================ */
/*			====>     NXNBKWF      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXNBKWF(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	if ((*lgecrit = (int_type)fnxnbkwf(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void
nxnbkwf_(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lgecrit = (int_type)fnxnbkwf(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}

void
nxnbkwf(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
	register int_type* lgecrit,register int_type* rc)
{
        nxnbkwf_(fd_index,buffer,lg,lgecrit,rc);
}
#endif

/* ============================================================ */
/*			====>     NXNBKWT      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXNBKWT(register int_type* fd_index,void* buffer,register int_type* lg,
		_fcd icod,_fcd ocod,register int_type* lgecrit,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	if ((*lgecrit = (int_type)fnxnbkwt(FTNFILDES, buffer, (int)*lg,
			_fcdtocp(icod),_fcdtocp(ocod))) <= 0)
	    *rc = -1;

}
#else
void
nxnbkwt_(register int_type* fd_index,void* buffer,register int_type* lg,
	 register char* icod,register char* ocod,register int_type* lgecrit,
	 register int_type* rc,int_type li,int_type lo)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lgecrit = (int_type)fnxnbkwt(FTNFILDES, buffer, (int)*lg,icod,ocod)) <= 0)
	    *rc = -1;

}

void
nxnbkwt(register int_type* fd_index,void* buffer,register int_type* lg,
        register char* icod,register char* ocod,register int_type* lgecrit,
	register int_type* rc,int_type li,int_type lo)
{
        nxnbkwt_(fd_index,buffer,lg,icod,ocod,lgecrit,rc,li,lo);
}
#endif

/* ============================================================ */
/*			====>     NXREAD      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXREAD(register int_type* fd_index,_fcd buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxread(FTNFILDES, _fcdtocp(buffer), (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void
nxread_(register int_type* fd_index,char* buffer,register int_type* lg,
	register int_type* lglus,register int_type* rc,int lb)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxread(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}

void
nxread(register int_type* fd_index,char* buffer,register int_type* lg,
       register int_type* lglus,register int_type* rc,int lb)
{
        nxread_(fd_index,buffer,lg,lglus,rc,lb);
}

#endif

/* ============================================================ */
/*			====>     NXNBKRI      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXNBKRI(register int_type* fd_index,int_type* buffer,register int_type*  lg,
		register int_type*  lglus,register int_type*  rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkri(FTNFILDES, (int *)buffer, (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void
nxnbkri_(register int_type* fd_index,int_type* buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkri(FTNFILDES, (int *)buffer, (int)*lg)) <= 0)
	    *rc = -1;

}

void
nxnbkri(register int_type* fd_index,int_type* buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
        nxnbkri_(fd_index,buffer,lg,lglus,rc);
}
#endif

/* ============================================================ */
/*			====>     NXNBKRF      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXNBKRF(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkrf(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}
#else
void
nxnbkrf_(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkrf(FTNFILDES, buffer, (int)*lg)) <= 0)
	    *rc = -1;

}

void
nxnbkrf(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register int_type* lglus,register int_type* rc)
{
       nxnbkrf_(fd_index,buffer,lg,lglus,rc);
}
#endif

/* ============================================================ */
/*			====>     NXNBKRT      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXNBKRT(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		_fcd icod,_fcd ocod,register int_type* lglus,register int_type* rc)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkrt(FTNFILDES, buffer, (int)*lg,
			_fcdtocp(icod),_fcdtocp(ocod))) <= 0)
	    *rc = -1;

}
#else
void
nxnbkrt_(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register char* icod,register char* ocod,
		register int_type* lglus,register int_type* rc,int_type li,int_type lo)
{
	*rc = 0;
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}
	
	if ((*lglus = (int_type)fnxnbkrt(FTNFILDES, buffer, (int)*lg,icod,ocod)) <= 0)
	    *rc = -1;

}

void
nxnbkrt(register int_type* fd_index,NXFLT* buffer,register int_type* lg,
		register char* icod,register char* ocod,
		register int_type* lglus,register int_type* rc,int_type li,int_type lo)
{
        nxnbkrt_(fd_index,buffer,lg,icod,ocod,lglus,rc,li,lo);
}
#endif

/* ============================================================ */
/*			====>     NXWRITI      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXWRITI(register int_type* fd_index,register int_type* nb,int_type* buffer,register int_type* rc)
#else
void
nxwriti_(register int_type* fd_index, register int_type* nb,int_type* buffer,
		  register int_type* rc)
#endif
{
if (*fd_index < 3) {
	FTNFILDES = stdfile[(int)*fd_index];
	nx_desc_table[*fd_index].ds = _FILE ;
	}

*rc = fnxwrite_int_type(FTNFILDES, (int)*nb, sizeof(int_type),(char *)buffer);
}

#ifndef CRAY_PROTO
void
nxwriti(register int_type* fd_index, register int_type* nb,int_type* buffer,
		  register int_type* rc)
{
        nxwriti_(fd_index,nb,buffer,rc);
}
#endif


/* ============================================================ */
/*			====>     NXREADI      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXREADI(register int_type* fd_index,register int_type* nb,int_type* buffer,register int_type* rc)
#else
void
nxreadi_(register int_type* fd_index,register int_type* nb,int_type* buffer,
		register int_type* rc)
#endif
{
if (*fd_index < 3) {
	FTNFILDES = stdfile[(int)*fd_index];
	nx_desc_table[*fd_index].ds = _FILE ;
	}

*rc = fnxread_int_type(FTNFILDES, (int)*nb, sizeof(int_type),(char *)buffer);
}

#ifndef CRAY_PROTO
void
nxreadi(register int_type* fd_index,register int_type* nb,int_type* buffer,
		register int_type* rc)
{
       nxreadi_(fd_index,nb,buffer,rc);
}
#endif


/* ============================================================ */
/*			====>     NXWRITF      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXWRITF(register int_type* fd_index,register int_type* nb,NXFLT* buffer,register int_type* rc)
#else
void
nxwritf_(register int_type* fd_index,register int_type* nb,NXFLT* buffer,
		register int_type* rc)
#endif
{
if (*fd_index < 3) {
	FTNFILDES = stdfile[(int)*fd_index];
	nx_desc_table[*fd_index].ds = _FILE ;
	}

*rc=(int_type)fnxwrite_float_type(FTNFILDES, (int)*nb, sizeof(float_type),(char *)buffer);
}

#ifndef CRAY_PROTO
void
nxwritf(register int_type* fd_index,register int_type* nb,NXFLT* buffer,
		register int_type* rc)
{
       nxwritf_(fd_index,nb,buffer,rc);
}

#endif


/* ============================================================ */
/*			====>     NXREADF      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXREADF(register int_type* fd_index,register int_type* nb,NXFLT* buffer,register int_type* rc)
#else
void
nxreadf_(register int_type* fd_index,register int_type* nb,NXFLT* buffer,
		register int_type* rc)
#endif
{
if (*fd_index < 3) {
	FTNFILDES = stdfile[(int)*fd_index];
	nx_desc_table[*fd_index].ds = _FILE ;
	}

*rc = (int_type)fnxread_float_type(FTNFILDES, (int)*nb, sizeof(float_type),(char *)buffer);
}

#ifndef CRAY_PROTO
void
nxreadf(register int_type* fd_index,register int_type* nb,NXFLT* buffer,
		register int_type* rc)
{
        nxreadf_(fd_index,nb,buffer,rc);
}
#endif


/* ============================================================ */
/*			====>     NXWRITT      <====		*/
/* ============================================================ */
/* Format & Type conversions                                    */

#ifdef CRAY_PROTO
void
NXWRITT(register int_type* fd_index,register int_type* nb,void* buffer,_fcd icod,
		_fcd ocod,int_type* rc)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	*rc = (int_type)fnxwritt(FTNFILDES,(int)*nb,buffer,_fcdtocp(icod),_fcdtocp(ocod));
}
#else
void
nxwritt_(register int_type* fd_index,register int_type* nb,void* buffer,
		register char* icod,register char* ocod,register int_type* rc,
		int li,int lo)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	*rc = (int_type)fnxwritt(FTNFILDES,(int)*nb,buffer,icod,ocod);
}

void
nxwritt(register int_type* fd_index,register int_type* nb,void* buffer,
		register char* icod,register char* ocod,register int_type* rc,
		int li,int lo)
{
       nxwritt_(fd_index,nb,buffer,icod,ocod,rc,li,lo);
}

#endif

/* ============================================================ */
/*			====>     NXREADT      <====		*/
/* ============================================================ */
/* Format & Type conversions                                    */

#ifdef CRAY_PROTO
void
NXREADT(register int_type* fd_index,register int_type* nb,void* buffer,
		_fcd icod,_fcd ocod,register int_type* rc)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	*rc = (int_type)fnxreadt(FTNFILDES,(int)*nb,buffer,_fcdtocp(icod),_fcdtocp(ocod));
}
#else
void
nxreadt_(register int_type* fd_index,register int_type* nb,void* buffer,
		register char* icod,register char* ocod,register int_type *rc,
		int li,int lo)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	*rc = (int_type)fnxreadt(FTNFILDES,(int)*nb,buffer,icod,ocod);
}

void
nxreadt(register int_type* fd_index,register int_type* nb,void* buffer,
		register char* icod,register char* ocod,register int_type *rc,
		int li,int lo)
{
        nxreadt_(fd_index,nb,buffer,icod,ocod,rc,li,lo);
}

#endif

/* ============================================================ */
/*			====>     NXDIALO     <====   		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXDIALO(int_type* fd_index,long int* question ,int_type* lg,long int* reply,
	int_type* lg_max,int_type* lgrep,int_type* rc)
{
	*rc = (int_type)fnxdialo(FTNFILDES,(char *)question,(int)*lg,(char *)reply,
			         (int)*lg_max,(int *)lgrep);
}
#else
void
nxdialo_(int_type* fd_index,char* question,int_type* lg,char* reply,
	int_type* lgmax,int_type* lgrep,int_type* rc)
{
	*rc = (int_type)fnxdialo(FTNFILDES,(char *)question,(int)*lg,(char *)reply,
			         (int)*lgmax,(int *)lgrep);
}

void
nxdialo(int_type* fd_index,char* question,int_type* lg,char* reply,
	int_type* lgmax,int_type* lgrep,int_type* rc)
{
	*rc = (int_type)fnxdialo(FTNFILDES,(char *)question,(int)*lg,(char *)reply,
			         (int)*lgmax,(int *)lgrep);
}

#endif

/* ============================================================ */
/*			====>     NXFLUSH      <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXFLUSH(int_type* fd_index)
{	
	
	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	fnxflush(FTNFILDES);
}
#else
void
nxflush_(int_type* fd_index)
{

	if (*fd_index < 3) 
	{
	    FTNFILDES = stdfile[(int)*fd_index];
	    nx_desc_table[*fd_index].ds = _FILE ;
	}

	fnxflush(FTNFILDES);
}

void
nxflush(int_type* fd_index)
{
        nxflush_(fd_index);
}
#endif

/* ============================================================ */
/*			====>     NXSEEK       <====		*/ 
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXSEEK(register int_type* fd_index,register int_type * offset,
	register int_type* from,register int_type* rc)
{
	*rc = (int_type)fnxseek(FTNFILDES,(long) *offset,(int)*from);
}
#else
void
nxseek_(register int_type* fd_index,register int_type * offset,
	         register int_type* from,register int_type* rc)
{
	*rc = (int_type)fnxseek(FTNFILDES,(long) *offset,(int)*from);
}

void
nxseek(register int_type* fd_index,register int_type * offset,
	         register int_type* from,register int_type* rc)
{
	*rc = (int_type)fnxseek(FTNFILDES,(long) *offset,(int)*from);
}

#endif

/* ============================================================ */
/*			====>     NXTELL       <====		*/ 
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXTELL(register int_type* fd_index,register int_type* offset,register int_type* rc)
{

	if ((*offset = (int_type)fnxtell(FTNFILDES)) < 0)
	    *rc = -1;
	else
	    *rc = 0;

}
#else
void
nxtell_(register int_type* fd_index,register int_type* offset,register int_type* rc)
{

	if ((*offset = (int_type)fnxtell(FTNFILDES)) < 0)
	    *rc = -1;
	else
	    *rc = 0;

}

void
nxtell(register int_type* fd_index,register int_type* offset,register int_type* rc)
{
        nxtell_(fd_index,offset,rc);
}

#endif

/* ============================================================ */
/*			====>   NXCHFRM       <====		*/ 
/* ============================================================ */
/* Dynamic update of the file format				*/

#ifdef CRAY_PROTO
void
NXCHFRM(register int_type* fd_index,_fcd codfmt,register int_type* cr)
{
	char c_codfmt[5];

	STR_FOR2C(_fcdtocp(codfmt),_fcdlen(codfmt),c_codfmt,5);

	*cr = (int_type)fnxchfrm(nx_desc_table[*fd_index].dp.fd,c_codfmt);
}
#else
void
nxchfrm_(register int_type* fd_index,const char* codfmt,
	register int_type* cr,int lcod)
{
	char c_codfmt[5];

	STR_FOR2C(codfmt,lcod,c_codfmt,5);

	*cr = (int_type)fnxchfrm(nx_desc_table[*fd_index].dp.fd,c_codfmt);
}

void
nxchfrm(register int_type* fd_index,const char* codfmt,
	register int_type* cr,int lcod)
{
        nxchfrm_(fd_index,codfmt,cr,lcod);
}

#endif

/* ------------------------------------------------------------ */
/*                      NXPERROR                                */
/* ------------------------------------------------------------ */

#ifdef CRAY_PROTO
void
NXPERROR(_fcd fcname)
{
	fnxperror(_fcdtocp(fcname));
}
#else
void
nxperror_(char* fcname,int lf)
{
	fnxperror(fcname);
}

void
nxperror(char* fcname,int lf)
{
	fnxperror(fcname);
}

#endif

/* ------------------------------------------------------------------   */
/*                  =====>    NOBLKQ    <====                           */
/* ------------------------------------------------------------------   */

static int
noblkq(const char* str,int len)
{
	int cnt = len, lg = 0;
  
	while(cnt-- && *str != ' ' && *str != '\0')
	{
	    str++; 
	    lg++;
	}

	return lg;
} 

/* ============================================================ */
/*       ====>     NXCONV_REAL_CR64_TO_IE64     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NXCONV_REAL_CR64_TO_IE64(const void *s, void *d, int_type *nb)
{
       (void)nxconv_real_cr64_to_ie64(s, d, *nb);
}
#else
void nxconv_real_cr64_to_ie64_(const void *s, void *d, int_type *nb)
{
       (void)nxconv_real_cr64_to_ie64(s, d, *nb);
}

#endif


/* ============================================================ */
/*		====>   verif type entier       <====		*/ 
/* ============================================================ */

static char nx_inttype_verif_C(void *a1, char *a2)
{
 long ia1, ia2, diff;
 
 ia1 = (long)a1;
 ia2 = (long)a2;

#if defined(CRAY) && !defined(_CRAYMPP) 
 diff = (ia2 - ia1)*sizeof(int);
#else
 diff = ia2 - ia1;
#endif

 if (diff == (long)sizeof(int_type))
    {
     return ('y');
    }
 else
    {
     return ('n');
    }
}


/* ============================================================ */
/*		====>   verif type reel       <====		*/ 
/* ============================================================ */

static char nx_realtype_verif_C(void *a1, void *a2)
{
 long ia1, ia2, diff;
 
 ia1 = (long)a1;
 ia2 = (long)a2;

#if defined(CRAY) && !defined(_CRAYMPP) 
 diff = (ia2 - ia1)*sizeof(int);
#else
 diff = ia2 - ia1;
#endif

 if (diff == (long)sizeof(float_type))
    {
     return ('y');
    }
 else
    {
     return ('n');
    }
}

/* ============================================================ */
/*            ====>     NX_INTTYPE_VERIF     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NX_INTTYPE_VERIF(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_inttype_verif_C(a1, a2);
}
#else
void nx_inttype_verif_(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_inttype_verif_C(a1, a2);
}

void nx_inttype_verif(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_inttype_verif_C(a1, a2);
}

#endif

/* ============================================================ */
/*            ====>     NX_REALTYPE_VERIF     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NX_REALTYPE_VERIF(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_realtype_verif_C(a1, a2);
}
#else
void nx_realtype_verif_(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_realtype_verif_C(a1, a2);
}

void nx_realtype_verif(void *a1, void *a2, char *rc)
{
 *rc = (char)nx_realtype_verif_C(a1, a2);
}

#endif
