/*
 *
 * Heading of function `COL'
 * __________________________________________________________________________
 *
 * .NAME
 * COL - Lecture d'une tables de couleurs.
 *
 * .SYNTAXE
 * COL_ReadColors (FILE *{file}, char *{label},
 *               int *{nd}, int *{nc}, float *{data})
 *
 * .ID: Fichier: COL_ReadColors.c Version: 1.9 Modifie: 93/05/19  CEL-V/DMA/SIE/IG
 *
 * .Auteur: Perros Jean-Pierre (6421)
 * __________________________________________________________________________
 */
#include "COL_P.h"

/*************************************************************************
 * Retourne le nombre de noms de tables trouves.
 * number -> numero de la table a chercher
 * name   -> nom de la table demandee
 *************************************************************************/
static int
Find_ColorName (FILE * file, int number, char *ptr_name)
{
   int             n;
   char            buffer[BUFSIZ];
   n = 0;
   while ((fgets (buffer, BUFSIZ, file)) != NULL)
   {
      if (buffer[0] == '#')	       /* C'est un nom de tables */
      {
	 n++;
	 if (n == number)
	 {
	    buffer[strlen (buffer) - 1] = '\0';	/* suppression du CR */
	    strcpy (ptr_name, &buffer[1]);	/* suppression du # */
	    return (n);
	 }
      }
   }
   return (n);
}

/*************************************************************************
 * Retourne le code du type de definition de couleurs.
 * type <- String correspondant au code du type de couleurs a verifier
 * ncmp -> Nombre de composantes
 *************************************************************************/
static          TypeColorDef
ColorDef (char *type, int *nbr_comp)
{
   int             n;
   TypeColorDef    code;

   if (!strcmp (type, "CHLS"))
   {
      code = COL_CHLS;
      n = 4;
   } else if (!strcmp (type, "CRGB"))
   {
      code = COL_CRGB;
      n = 4;
   } else if (!strcmp (type, "HLS"))
   {
      code = COL_HLS;
      n = 3;
   } else if (!strcmp (type, "RGB"))
   {
      code = COL_RGB;
      n = 3;
   } else
   {
      code = COL_INVALID;
      n = 0;
   }
   *nbr_comp = n;
   return (code);
}

/*************************************************************************
 * Retourne la string correspondant au code du type de definition de couleurs.
 * code <- code du type de couleurs
 * type -> String correspondant au code du type de couleurs
 *************************************************************************/
TypeError
COL_GetColorDef (TypeColorDef code,
		 char *type)
{
   TypeError       status;
   char            mess[BUFSIZ];
   
   status = OK;
   switch (code)
   {
      case COL_CHLS:
	 strcpy (type, "CHLS");
	 break;
      case COL_CRGB:
	 strcpy (type, "CRGB");
	 break;
      case COL_HLS:
	 strcpy (type, "HLS");
	 break;
      case COL_RGB:
	 strcpy (type, "RGB");
	 break;
      default:
	 strcpy (type, "");
	 sprintf (mess, "Code de definition des couleurs non reconnu");
	 ERR_Put_Messag (ERROR, mess);
	 status = ERROR;
	 break;
   }
   return (status);
}

/******************************************************************************
 * Retourne le nombre de table disponibles dans un fichier de couleurs.
 *
 * datafile  <- Nom du fichier contenant la/les table(s)
 * nbrtab    -> Nombre de tables du fichier
 ******************************************************************************/
TypeError
COL_InfoColorFile (char *datafile,     /* Nom du fichier a ouvrir */
		   int *nbrtab)	       /* Pointeur sur le nombre de tables */
{
   FILE           *file;	       /* Identifier du fichier */
   char            buffer[BUFSIZ],
                   mess[BUFSIZ];
   int             n,
                   number = 9999;

   /* Ouverture du fichier */
   if ((file = fopen (datafile, "r")) == NULL)
   {
      sprintf (mess, "Pas d'acces au fichier : %s", datafile);
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   /* Lecture des labels pour retourner le nombre de tables */
   n = Find_ColorName (file, number, buffer);
   if (n < 1)
   {
      sprintf (mess, "Il n'y a pas de tables dans le fichier : %s", datafile);
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   *nbrtab = n;

   /* Fermeture du fichier */
   fclose (file);

   return (OK);
}

/**********************************************************************************
 * Retourne les infos sur une table d'un fichier de couleurs.
 *
 * datafile  <- Nom du fichier contenant la/les table(s)
 * number    <- Numero de la table a chercher
 * name      -> Nom de la table demandee
 * type      -> Type de description des couleurs de la table
 * nval      -> Nombre de valeurs
 * ncmp      -> Nombre de composantes
 **********************************************************************************/
TypeError
COL_InfoColorTable (char *datafile,    /* Nom du fichier a ouvrir */
		    int number,	       /* Numero de la table */
		    char *name,	       /* Pointeur sur le nom de la table */
		    TypeColorDef * type,	/* Type de definition */
		    int *nval,	       /* Nombre de valeurs */
		    int *ncmp)	       /* Nombre de composantes */
{
   FILE           *file;	       /* Identifier du fichier */
   char            mess[BUFSIZ],
                   buffer[BUFSIZ];
   TypeColorDef    typ;		       /* Type de definition */
   int             nt,
                   nv,
                   nc;

   /* Ouverture du fichier */
   if ((file = fopen (datafile, "r")) == NULL)
   {
      sprintf (mess, "Pas d'acces au fichier : %s", datafile);
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   /* Lecture des labels pour rechercher la table numero `number' */
   nt = Find_ColorName (file, number, name);

   if (nt < number)
   {
      sprintf (mess, "Table demandee : %d, Tables du fichier : %d", number, nt);
      ERR_Put_Messag (WARNING, mess);
      return (WARNING);
   }
   /* Lecture du TYPE et du NOMBRE DE COMPOSANTES */
   fgets (buffer, BUFSIZ, file);
   buffer[strlen (buffer) - 1] = '\0'; /* suppression du CR */
   if ((typ = ColorDef (buffer, &nc)) == COL_INVALID)
   {
      sprintf (mess, "Code de definition des couleurs non reconnu");
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   *type = typ;
   *ncmp = nc;

   /* Lecture du NOMBRE DE VALEURS */
   fscanf (file, "%d \n", &nv);
   *nval = nv;

   /* Fermeture du fichier */
   fclose (file);

   return (OK);
}

/**********************************************************************************
 * Retourne le contenu d'une table d'un fichier de couleurs.
 *
 * datafile  <- Nom du fichier contenant la/les table(s)
 * number    <- Numero de la table a chercher
 * values    -> Valeurs des couleurs (dimension [nval*ncmp])
 **********************************************************************************/
TypeError
COL_GetColorValues (char *datafile,    /* Nom du fichier a ouvrir */
		    int number,
		    float *values)
{
   FILE           *file;	       /* Identifier du fichier */
   char            buffer[BUFSIZ],
                   mess[BUFSIZ];
   int             i,
                   nt,
                   nc,
                   nv,
                   n;

   /* Ouverture du fichier */
   if ((file = fopen (datafile, "r")) == NULL)
   {
      sprintf (mess, "Pas d'acces au fichier : %s", datafile);
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   /* Lecture des labels pour rechercher la table numero `number' */
   nt = Find_ColorName (file, number, buffer);

   if (nt < number)
   {
      sprintf (mess, "Table demandee : %d, Tables du fichier : %d", number, nt);
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   /* Lecture du TYPE et du NOMBRE DE COMPOSANTES */
   fgets (buffer, BUFSIZ, file);
   buffer[strlen (buffer) - 1] = '\0'; /* suppression du CR */
   if (ColorDef (buffer, &nc) == COL_INVALID)
   {
      sprintf (mess, "Code de definition des couleurs non reconnu");
      ERR_Put_Messag (ERROR, mess);
      return (ERROR);
   }
   /* Lecture du NOMBRE DE VALEURS */
   fscanf (file, "%d \n", &nv);
   n = nv * nc;

   /* Lecture des VALEURS des couleurs */
   for (i = 0; i < n; i++)
      fscanf (file, "%f \n", &values[i]);

   /* Fermeture du fichier */
   fclose (file);

   return (OK);
}

/*
 * End of function `COL'
 */
