// ************************************************************************
//         Ident
// ************************************************************************

struct SccsIdP4Constant
{
	static const char* const sccs_id;
};

const char* const SccsIdP4Constant::sccs_id = "%W% %G% Pierre Brochard";

// ************************************************************************
//          Includes
// ************************************************************************

#include "P4constant.h"

#if defined(_IBMR2)
#  include<ctype.h>
#endif

// ************************************************************************
//            Variables globales
// ************************************************************************

extern const char* const P4Tabgen_resize_error =
		"Erreur : tentative de reallouer un "
		"TabGEN de type_alloc=P4UNKNOWN";
extern const REEL Emin = log10(Bmin);
extern const REEL Emax = log10(Bmax);

// ************************************************************************
//            Fonctions utiles
// ************************************************************************

void
P4Vrbor(REEL& xmin,REEL& xmax,REEL nbdecades,P4Boolean modlog,int marge)
{

    // *-----------------------------------------------------------*
    //      routine arondissant les bornes d'un intervalle
    //      avec eventuellement passage en logarithme
    // *-----------------------------------------------------------*

	if(xmin > xmax) P4Inv(xmin,xmax);

	if(modlog == FALSE)
	{
	    const REEL dx = xmax-xmin, dxx = dx*0.1;

	    if(marge == 1)
	    {
		xmax += dxx;
		xmin -= dxx;
	    }

	    if(dx == 0.)
	    {
		REEL xx = xmin*.15;

		if(xx == 0.) xx = 1.;

		xmin -= xx;
		xmax += xx;

		return;
	    }

	}
	else
	{

	    if(xmax > 0.)
	    {
		xmax=P4Log(xmax);

		if(xmin <= 0. || marge != 0)
		{
		    int i = (int) xmax; if(xmax > 0.) i++;

		    xmax = (REEL) i;
		    xmin = xmax-nbdecades;

		    if(marge == 100 || xmin <= 0.) return;

		    if(xmin >= xmax-0.2)
		    {
			xmin = xmax-0.2;
			xmax += 0.2;
		    }

		    i = (int) xmin; if(xmin > 0.) i++;

		    xmin = (REEL) i;

		    return;
		}

		xmin=P4Log(xmin);

		if(xmin >= xmax-0.2)
		{
		    xmin = xmax-0.2;
		    xmax += 0.2;
		}

		return;
	    }

	    xmax = Emax;
	    xmin = Emax-1.;

	    return;
	}

}

// *-----------------------------------------------------------*
//     routine calculant le min d'un tableau et son max
//     cas vecteurs lignes, colonnes et matrice
// *-----------------------------------------------------------*

int
P4MinMax(int nx,int ny,int nz,const REEL_S z[],REEL& xmin,REEL& xmax)
{

    // *-----------------------------------------------------------*
    //  si nz vaut 0 z va de 0 a nx-1
    //  si nz vaut -1 z va de 0 a ny-1
    // *-----------------------------------------------------------*

	if(nx == 0 || ny == 0) return -1;

	register REEL_S xmaxs = z[0],xmins = xmaxs;
	register int i = 0;

	switch(nz)
	{
	  case -1 :
	    {
		register const REEL_S* const pzmax = z+ny;

		for(register const REEL_S* pz = z;pz < pzmax;pz++)
		{
		    register REEL_S zij = *pz;

		    xmins = min(xmins,zij);
		    xmaxs = max(xmaxs,zij);
		}

	    }
	    break;

	  case 0 :
	    {
		register const REEL_S* const pzmax = z+nx;

		for(register const REEL_S* pz = z;pz < pzmax;pz++)
		{
		    register REEL_S zij = *pz;

		    xmins = min(xmins,zij);
		    xmaxs = max(xmaxs,zij);
		}

	    }
	    break;

	  default :

	    for(;i < ny;i++)
	    {
		register const REEL_S* const pzmax = z+nx+nz*i;
		register const REEL_S* pz = z+nz*i;
		register REEL_S zij = 0.0;

		switch(nx%8)
		{
		  case 0 : do
		  {
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 7:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 6:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 5:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 4:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 3:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 2:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  case 1:
		      zij = *pz++;xmins = min(xmins,zij);xmaxs = max(xmaxs,zij);
		  } while(pz < pzmax);

		}

	    }

	}

	xmin = xmins;
	xmax = xmaxs;

	return 0;
}

void
P4Dspace(REEL& h1,REEL& h2,const P4Boolean logh,const REEL nbredecades)
{
	const REEL habs = h1 > h2 ? h1 - h2 : h2 - h1;

	if(habs <= Bmin)
	{

	    if(logh) h1=1.2*h1,h2=.8*h2;
	    else     h1=h1-1. ,h2=h2+1.;

	}

	P4Vrbor(h1,h2,nbredecades,logh,0);
}

#if defined(_IBMR2)
int
strcasecmp(const char* s1, const char* s2)
{
	register const unsigned tai1 = strlen(s1);
	register const unsigned tai2 = strlen(s2);

	char* s1temp = new char[tai1+1];
	char* s2temp = new char[tai2+1];
	register unsigned i;

	for(i=0;i<tai1;i++)
	{
	    s1temp[i] = tolower(s1[i]);
	}

	s1temp[tai1] = '\0';

	for(i=0;i<tai2;i++)
	{
	    s2temp[i] = tolower(s2[i]);
	}

	s2temp[tai2] = '\0';

	const int rtn = strcmp(s1temp,s2temp);

	delete[] s1temp;
	delete[] s2temp;

	return rtn;
}
#endif

/* Impression infos constantes utiles */

void
P4ImpConstantes()
{
	cout << "P4ImpConstantes : Impression constantes utiles :\n";
	cout << "    Bmin = " << Bmin << '\n';
	cout << "    Bmax = " << Bmax << '\n';
	cout << "    Emin = " << Emin << '\n';
	cout << "    Emax = " << Emax << '\n';
	cout << "    Imin = " << Imin << '\n';
	cout << "    Imax = " << Imax << '\n';
	cout << flush;
}

/* version sans template */

#if !defined(P4VERSION_AVEC_TEMPLATE) || defined(__GNUC__)
#   define GEN unsigned
#   include "P4constant_gen.h"
#   undef GEN

#   define GEN int
#   include "P4constant_gen.h"
#   undef GEN
#endif
