// ident

struct SccsIdP4TabUnsigned
{
	static const char* const sccs_id;
};

const char* const SccsIdP4TabUnsigned::sccs_id = "%W% %G% Pierre Brochard";

// includes

#include "dinosaure/P4tabunsigne.h"
#include "P4constant.h"

// implementation

int
ptrunsigned::operator < (const ptrunsigned& t) const
{

    // operateur permettant le tri de tableaux d'entiers entre eux
    // pour ensuite eliminer des tableaux identiques.
    // l'operateur < donne seulement une relation d'ordre
    // et n'est pas forcement identique a l'idee intuitive du inferieur.
    // en particulier sur machines little endian c'est plutot un >,
    // qui convient a l'usage pour lequel cet operateur existe.

	if(_taille < t._taille) return TRUE;
	else if(_taille > t._taille) return FALSE;
	else // on tri les numeros par ordre croissant puis on compare
	{
	    static unsigned* temp1 = NULL;
	    static unsigned* temp2 = NULL;
	    static unsigned taille_temp = 0;

	    if(_taille > taille_temp)
	    {
		delete[] temp1;delete[] temp2;

		temp1 = new unsigned[_taille];
		temp2 = new unsigned[_taille];

		if(!temp1 || !temp2)
		{
		    cerr << "Pb allocation " << __FILE__ <<
			" ligne : " << __LINE__ << endl;
		    taille_temp = 0;

		    return -1;
		}

		taille_temp = _taille;
	    }

	    const unsigned tai = _taille*sizeof(unsigned);

	    memcpy(temp1,_tab,tai);
	    memcpy(temp2,t._tab,tai);

	    register unsigned debut = 0;
	    register const unsigned fin = _taille - 1;

	    while(debut < fin)
	    {
		register unsigned posmin = debut;
		register unsigned minim = temp1[debut];

		for(register unsigned j=debut+1;j<=fin;j++)
		{

		    if(temp1[j] < minim)
		    {
			posmin = j;
			minim = temp1[j];
		    }

		}

		if(posmin != debut)
		{
		    temp1[posmin] = temp1[debut];
		    temp1[debut] = minim;
		}

		debut++;
	    }

	    debut = 0;

	    while(debut < fin)
	    {
		register unsigned posmin = debut;
		register unsigned minim = temp2[debut];

		for(register unsigned j=debut+1;j<=fin;j++)
		{

		    if(temp2[j] < minim)
		    {
			posmin = j;
			minim = temp2[j];
		    }

		}

		if(posmin != debut)
		{
		    temp2[posmin] = temp2[debut];
		    temp2[debut] = minim;
		}

		debut++;
	    }

	    return memcmp(temp1,temp2,tai) < 0 ? TRUE : FALSE;
	}

}

int
ptrunsigned::operator == (const ptrunsigned& t) const
{

	if(_taille != t._taille) return FALSE;
	else // on tri les numeros par ordre croissant puis on compare
	{
	    static unsigned* temp1 = NULL;
	    static unsigned* temp2 = NULL;
	    static unsigned taille_temp = 0;

	    if(_taille > taille_temp)
	    {
		delete[] temp1;delete[] temp2;

		temp1 = new unsigned[_taille];
		temp2 = new unsigned[_taille];

		if(!temp1 || !temp2)
		{
		    cerr << "Pb allocation " << __FILE__ <<
			" ligne : " << __LINE__ << endl;
		    taille_temp = 0;

		    return -1;
		}

		taille_temp = _taille;
	    }

	    const unsigned tai = _taille*sizeof(unsigned);

	    memcpy(temp1,_tab,tai);
	    memcpy(temp2,t._tab,tai);

	    register unsigned debut = 0;
	    register const unsigned fin = _taille - 1;

	    while(debut < fin)
	    {
		register unsigned posmin = debut;
		register unsigned minim = temp1[debut];

		for(register unsigned j=debut+1;j<=fin;j++)
		{

		    if(temp1[j] < minim)
		    {
			posmin = j;
			minim = temp1[j];
		    }

		}

		if(posmin != debut)
		{
		    temp1[posmin] = temp1[debut];
		    temp1[debut] = minim;
		}

		debut++;
	    }

	    debut = 0;

	    while(debut < fin)
	    {
		register unsigned posmin = debut;
		register unsigned minim = temp2[debut];

		for(register unsigned j=debut+1;j<=fin;j++)
		{

		    if(temp2[j] < minim)
		    {
			posmin = j;
			minim = temp2[j];
		    }

		}

		if(posmin != debut)
		{
		    temp2[posmin] = temp2[debut];
		    temp2[debut] = minim;
		}

		debut++;
	    }

	    return memcmp(temp1,temp2,tai) == 0 ? TRUE : FALSE;
	}

}











