#ifndef P4GRAF3_H
#define P4GRAF3_H

// %W% %G% Pierre Brochard

#include "minotaure/P4graf2.h"

enum FilLisse {P4USETAB = -1,P4LIS = 0,P4LIS_FIL = 1,P4FIL = 2,P4FAC = 3,
		P4FAC_FIL = 4,P4NIV = 5,P4FIL_NIV = 6,
		P4FAC_NIV = 7,P4FAC_FIL_NIV = 8};
enum Coupe {P4COUPE_TAB = -1,P4COUPE_NON = 0,P4COUPE_OUI = 1};

class Tronc;

#if defined(P4VERSION_AVEC_TEMPLATE) /* version avec template */
   class GeomPure4;
   typedef GeomPure4* ptrGeomPure4;

#  ifndef P4TABGEN_H
      template<class GEN> class TabGEN;
#  endif

#  if P4VERSION_AVEC_TEMPLATE == 4
      typedef TabGEN<ptrGeomPure4> TabGeomPure4;
      typedef TabGEN<int> Tabint;
#  else
#     define TabGeomPure4 TabGEN<ptrGeomPure4>
#     define Tabint TabGEN<int>
#  endif

#else // version sans template
   class TabGeomPure4;
   class Tabint;
#endif // version sans template

// *******************************************************************
//  le classe P4Graf3 traite plus specifiquement du multibloc
// *******************************************************************

class P4Graf3 : public P4Graf2
{
   protected:

        TabGeomPure4& _tabgeo;
        Tabint&       _type_donnee; // var de noeud ou de mailles
        Tronc&        _tronc_save; // pour non troncature si bornes auto
        TabGeomPure4& _tab_geo_coupe;
        Tabint&       _ico_haut; // tableau de _icoh pour multibloc
        Tabint&       _ico_bas; // tableau de _icob pour multibloc
	Tabint&       _fil_lisse; // tableau de fil_lisse pour multibloc
	Tabint&       _coupe; // tableau de coupe pour multibloc
	Tabint&       _fil_opt;
	Tabint&       _fac_opt;
	Tabint&       _niv_opt;
	Tabint&	      _lis_opt;
	Tabint&	      _masq_opt;
	Tabint&       _trace_fil_opt;
	Tabint&       _marq_opt;
	TabREEL_S&    _coupe_a;
	TabREEL_S&    _coupe_b;
	TabREEL_S&    _coupe_c;
	TabREEL_S&    _coupe_d;
	TabREEL_S&    _coupe_e;
	P4Boolean     _bornes_auto;
	Opt34d	      _pix;
        P4Boolean     _log_zh;
        P4Boolean     _log_ch;

    public:

        P4Graf3();
        ~P4Graf3();

        void P4Ecriture(P4Boolean ecriture = TRUE);

        void P4InitPix34d(Opt34d pix) {_pix = pix;};

    // icob et icoh sont initialises dans P4AjouteGeom pour leur
    // version tableau. Les parametres tableaux sont utilises si
    // le parametre scalaire correspondant vaut -1.
    // Si on veut les modifier sans recharger les geometries car
    // ce sont des parametres de trace independants de la geometrie :

	void CouleurHaut(const int icoh[],unsigned taille);
	void CouleurBas(const int icob[],unsigned taille);

    // Pour fil_lisse en tableau = actif si fil_lisse vaut P4USETAB :

	void P4FilLisse(const FilLisse fil_lisse[],unsigned taille);

    // Pour coupe en tableau = actif si coupe vaut P4COUPE_TAB :

	void P4Coupe(const int coupe[],unsigned taille);

    // Pour *_opt en tableau = actif si fil vaut OPT_NONE

	void P4FilOpt(const Opt34d fil_opt[],unsigned taille);
	void P4FacOpt(const Opt34d fac_opt[],unsigned taille);
	void P4NivOpt(const Opt34d niv_opt[],unsigned taille);
	void P4LisOpt(const Opt34d lis_opt[],unsigned taille);
	void P4MasqOpt(const int masq_opt[],unsigned taille);
	void P4MarqOpt(const int marq_opt[],unsigned taille);
	void P4TraceFilOpt(const int trace_fil_opt[],unsigned taille);

    // Pour coupe_* en tableau = actif si coupe vaut P4COUPE_TAB

	void P4CoupeA(const REEL_S coupe_a[],unsigned taille);
	void P4CoupeB(const REEL_S coupe_b[],unsigned taille);
	void P4CoupeC(const REEL_S coupe_c[],unsigned taille);
	void P4CoupeD(const REEL_S coupe_d[],unsigned taille);
	void P4CoupeE(const REEL_S coupe_e[],unsigned taille);

    // Gestion echelles de hauteurs

        int P4CalcHauteurs(REEL_S hauteurs[/*nbrehauteurs*/],
			unsigned nbrehauteurs,REEL hmin,REEL hmax,
			Opt34d cz,P4Boolean modlog,
			P4Boolean bornes_faites = FALSE);
        int P4InitHauteurs(REEL_S hauteurs[],unsigned nbrehauteurs,
			REEL hmin,REEL hmax,Opt34d cz,unsigned couleurmin,
			unsigned couleurmax,P4Boolean modlog,
			P4Boolean logresu = TRUE,
			P4Boolean bornes_faites = FALSE);

    // Bornes

        int P4Bornes();
        int P4BornesLogiques();
        int P4BornesRapide();
        void P4BornesAuto(P4Boolean bornes_auto = TRUE)
	{_bornes_auto = bornes_auto;};

    // Ajout de geometries

        int P4AjouteGeom(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,unsigned mailles[],unsigned nmailles,
			int dimmailles,TypeDonnee4d td = DONNEE_NOEUD);
        int P4AjouteGeom(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,int nz,unsigned mailles[],
			unsigned nmailles,int dimmailles,
			TypeDonnee4d td = DONNEE_NOEUD);
        int P4AjouteGeom(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,int nz,const unsigned mailles[],
			unsigned nmailles,int dimmailles,
			const unsigned volumes[],unsigned nvolumes,
			int dimvolumes,TypeDonnee4d td = DONNEE_NOEUD);

        int P4VideGeom();
        int P4VideGeomCoupe();

    // Traces

    // pixel
        int P4TraceGeom(P4Boolean restore_pt_vue = FALSE);
    // filaire + facettes + niveaux + lissage + coupes physiques
        int P4TraceGeom(const FilLisse fil_lisse,const Coupe coupe = P4COUPE_NON,
		REEL a = 1.,REEL b = 0.,REEL c = 0.,REEL d = 0.,REEL e = 0.);

        int P4Coordonnees(REEL& x,REEL& y,REEL& z,
			unsigned& num_maille,unsigned& num_bloc,
			REEL xp = -1.,REEL yp = -1.);

        friend ostream& operator << (ostream& ost,const P4Graf3& p23);

    private:

	int P4Pixel(const GeomPure4& geo);
	int Icoh(unsigned num_bloc) const;
	int Icob(unsigned num_bloc) const;
	FilLisse P4FilLisse(FilLisse fil_lisse,unsigned num_bloc) const;
	Coupe P4Coupe(Coupe coupe,unsigned num_bloc) const;
	Opt34d P4FilOpt(unsigned num_bloc) const;
	Opt34d P4FacOpt(unsigned num_bloc) const;
	Opt34d P4NivOpt(unsigned num_bloc) const;
	Opt34d P4LisOpt(unsigned num_bloc) const;
	REEL P4CoupeA(Coupe coupe,REEL a,unsigned num_bloc) const;
	REEL P4CoupeB(Coupe coupe,REEL b,unsigned num_bloc) const;
	REEL P4CoupeC(Coupe coupe,REEL c,unsigned num_bloc) const;
	REEL P4CoupeD(Coupe coupe,REEL d,unsigned num_bloc) const;
	REEL P4CoupeE(Coupe coupe,REEL e,unsigned num_bloc) const;
	Masque P4MasqOpt(unsigned num_bloc) const;
	int P4MarqOpt(unsigned num_bloc) const;
	TypeTraceFil P4TraceFilOpt(unsigned num_bloc) const;
};

#endif
