#ifndef NX_H
#define NX_H

/* %W% %G% pierre.brochard.1982@m4x.org */
/* revision Pierre Brochard et Alain Fontaine */

#include <stdio.h>
#include "nxconv.h"

/********************************************************************/
/*  API nx                                                          */
/********************************************************************/

#ifdef __cplusplus
extern "C" {
#   define NXVOID
#else
#   define NXVOID void
#endif

/* enums */

typedef enum
{
    NX_PIPE=0,
    NX_SOCKET=1,
    NX_FILE=2,
    NX_RPC=3
} NX_Fd_Type;

/********************************************************************/
/*  Macros utiles                                                   */
/********************************************************************/

#if defined(__GLIBC__)
#  if __GLIBC_PREREQ(2,28)
#    include <bits/types/struct_FILE.h>
#    define _STDIO_USES_IOSTREAM
#    define _G_USING_THUNKS
#    define _IO_off64_t __off64_t
typedef struct _IO_FILE _IO_FILE;
struct _IO_FILE_plus;
extern void _IO_file_init (struct _IO_FILE_plus *);
#  endif
#endif

#if defined(NX_LONGLONG)
#   undef NX_LONGLONG
#endif

#if (!defined (CRAY) || !defined (__cplusplus)) && \
	(!defined(__STDC__) || __STDC__ != 1) || \
	(defined(IRIX) && defined(_LONGLONG)) || \
	(defined(__hppa) && defined(__STDC_EXT__)) || \
	(defined(_IBMR2) && defined(_LONG_LONG)) || \
	(defined(__GNUC__) && !defined(__STRICT_ANSI__))
#   define NX_LONGLONG
#endif

/********************************************************************/
/* declarations des fcs de l'interface C publique                   */
/********************************************************************/

/* ouverture d'un fichier disque, d'une socket, d'un pipe ou de RPCs ou */
/* d'une socket d'ecoute (= serveur nx) */

FILE* fnxopen(const char* name,const char* type);

/* name = "|..." => le fichier est un pipe UNIX ouvert avec popen(...)

   ==========================

   name = "machine+port+handler+user@format" => le fichier
   est une socket INET ou UNIX ou une connexion RPC

   + signifie +, : ou blanc

   name = "nom different des formes precedentes@format" => le fichier
   est une fichier disque ordinaire

   @format est facultatif : par defaut c'est le format supporte
   par la machine conrante sans conversion

   machine = SERVER => on lance un serveur socket sur la machine ou tourne
   le process en cours (machine courante).

   machine = "nom de machine" => on cherche a se connecter sur un
   serveur tournant sur la machine designee. Le nom de machine est au
   format internet. Le defaut est la machine courante.

   ==========================

   port = 0 < numero < 100000 => socket INET.
   Si on est client pas de detection automatique de port.
   Si on est serveur alors le demon n'est pas lance ou verifie. Par contre
   les caracteristiques du serveur sont sauvegardees systematiquement dans
   /tmp/.nx (voir ci-dessous).

   port = 0 et client => detection automatique du port

   port = numero >= 100000 => RPC

   port = 0 et si machine = SERVER => serveur socket avec choix automatique
   du numero de port socket par le systeme

   port = "nom de service" => socket ou RPC avec mapping service <-> numero
   de port socket

   port = "/UNIX/" et si machine = SERVER => socket UNIX avec choix du fichier
   automatique pour bind

   port = "chemin absolu du nom d'un fichier" => socket UNIX

   ==========================

   port = port.name.numero avec port = numero de port socket ou nom
   absolu de fichier entre "" ou '', name une chaine
   de caractere identifiant le programme (ex "Psyche $DISPLAY"),

   numero est alors un numero d'ordre si on lance plusieurs programme
   avec la meme chaine name.

   Si port = 0 et que l'on est client alors
   on passe en mode de recherche automatique du serveur.

   Si port est omis (.name.numero) alors port = 0.
   Si port est different de 0 et que name ou
   numero existent alors la recherche, si on est client, est aussi automatique
   et si le mecanisme automatique ne trouve pas le serveur alors port
   est essaye quand meme (port=2101 => manuel, port=2101. => auto + defaut).

   Si numero est omis pour le client alors numero = 1.
   Si numero = 0 le client se connecte au dernier programme lance avec name.

   Pour le serveur le numero d'ordre des programmes ayant la meme chaine
   est choisi automatiquement. numero est donc ignore.

   Le defaut de name qui peut-etre ignore est "nx". (Le serveur inscrit
   ses caracteristiques dans /tmp/.nx et lance un demon RPC s'il ne tourne pas
   deja).

   ==========================

   handler => chaine de caracteres utilisable par l'utilisateur de nx

   ==========================

   user => nom de l'utilisateur au nom duquel on lance un serveur, qui doit
   etre l'utilisateur courant a prori (on verifie avec U8GetUser et on ne
   lance pas le serveur si different),
   ou de proprietaire du serveur a atteindre si on est client.
   Le defaut est l'utilisateur courant.

   */

/* reouverture d'un fichier disque, d'une socket, d'un pipe ou de RPCs
   deja ouvert par fopen ou fdopen */

int fnxreopen(
     FILE* fd, /* IN : pointeur de fichier, rendu par fopen */
     NX_Fd_Type fd_type /* IN : type de "fichier" */
    );

/* encapsulation du select pour serveurs nx */

FILE* fnxselect(const int nb,FILE* tabfd[],const int mode,int* rc);
int fnxselectnum(const int nb,FILE* tabfd[],const int mode,int* rc);

/* accept d'une connexion socket (pour serveurs nx) */

FILE* fnxaccept(FILE* sd);
FILE* fnxaccept2(FILE* sd,char handler_name[]);

/* Fermeture d'un fichier disque, d'une socket, d'un pipe ou de RPCs
   ou d'un serveur nx. */

int fnxclose(FILE* fd);
int fnxclose_shutdown(FILE* fd,int shutdown);

/* Routine de recherche de port sous lequel tourne le serveur
   pour les clients nx. Cette routine est appelee automatiquement
   par fnxopen si port = 0 ou est de la forme numero.[...] ou numero > 0
   et numero < 100000. Reservee aux inities !! */

int fnx_find_port(const char* machine_serveur,const char* user_name,
		const char* port_service,int port_which,
		int* process,int* port,char* port_name,int* lport_name,
		int* port_which_reel);

/* Lance le demon rpc de recherche de ports sockets apres avoir
   verifie s'il n'existait pas deja. Une fois lance ce demon rend service
   a tout le monde et survit jusqu'a ce que son proprietaire le tue
   ou que l'on reboote la machine. Il n'est en effet pas utile d'avoir
   un demon par utilisateur. Le bon endroit pour appeler cette
   routine peut-etre au debut du main de maniere systematique pour
   les programmes du type serveur socket.
   En FORTRAN on peut utiliser : call nxlport */

void fnx_lance_rpc_port(NXVOID);

/* test pour savoir si le buffer en lecture contient des donnees a lire
   renvoit 1 si oui et 0 sinon */

int fnx_data_in_buffer(FILE* fd);

/* Fonctions de renvoi des vrai FILE* en lecture et en ecriture pour
   utilisation en dehors de nx. */

FILE* fnx_write_desc(FILE* fd);
FILE* fnx_read_desc(FILE* fd);

/* Fonction de changement de format de fichier. */

int fnxchfrm(FILE* fd,const char* codfmt);

/* codfmt est une chaine de caractere qui designe le format sous lequel
   les nombres doivent etre stockes. Les codes sont les memes que la
   chaine format mentionnee ci-dessus (description du parametre name
   de fnxopen).

   codfmt peut prendre les valeurs avec les tailles en bits suivantes :
                                                               |   ocod
         type | int | long | long long | float | double | real | cp | sp | dp
   ie32  ieee   32    32     32          32      32       32     32   32   64
   li32  ieee   32    32     32          32      32       32     32   32   64
   vms32 ieee   32    32     32          32      32       32     32   32   64
   ie64  ieee   64    64     64          64      64       64     64   64   128
   cr64  cray   64    64     64          64      64       64     64   64   128
   li64  ieee   64    64     64          64      64       64     64   64   128
   xdr   xdr */

/* Envoi d'une question et retour d'une reponse en mode non bloquant
   pour une connexion socket ou RPC sinon pas d'action. */

int fnxdialo(FILE* fd,char* question,const int lg,char* reply,const int lgmax,
		int* lgrep);

/* flush sur le canal en ecriture */

int fnxflush(FILE* fd);

/* fseek dans le cas d'un fichier sinon pas d'action */

int fnxseek(FILE* fd,const long offset,const int from);

/* ftell dans le cas d'un fichier sinon pas d'action */

int fnxtell(FILE* fd);

/* pas d'impression de message d'erreur par les routines nx
   si noprint different de 0. NXERRNO reste positionne.
   on retourne l'ancienne valeur du parametre d'impression. */

int fnx_noprint_error(const int noprint);

/* impression de l'erreur de type nx en cours */
/* avec la chaine fcname devant */
/* pour certains types d'erreurs leur nombre est limite a */
/* MAX_ERROR_CNT pour l'impression */

void fnxperror(const char* fcname);

/* fnxsyserror : impression de l'erreur systeme errno avec devant la chaine
   prealablement initialisee avec fnxsetsyserror et encore devant fcname */

void fnxsyserror(const char* fcname);
void fnxsetsyserror(const char* fcname);

/* on supprime la bufferisation des FILE* (a appeler avant tout
   appel a fread ou fwrite : */

void fnxunbuf(FILE* fd);

/* routines de lecture, ecriture pour caracteres */

int fnxwrite(FILE* fd,const char* buffer,const int lg);
int fnxread(FILE* fd,char* buffer,const int lg);

#ifndef NXP_H_IMPLEMENT /* prototypes publics */
#   define NX_TYPE_I int
#   define NX_TYPE_L long
#   define NX_TYPE_LL long long
#   define NX_TYPE_C NXFLT
#   define NX_TYPE_F float
#   define NX_TYPE_D double
#   define NX_TYPE_T void
#else /* prototypes prives */
#   define NX_TYPE_I char
#   define NX_TYPE_L char
#   define NX_TYPE_LL char
#   define NX_TYPE_C char
#   define NX_TYPE_F char
#   define NX_TYPE_D char
#   define NX_TYPE_T char
#endif /* fin prototypes prives */

/* routines de lecture, ecriture pour entiers
   ...nbk... <=> non bloquant sinon bloquant */

int fnxwriti(FILE* fd,const int nb,const NX_TYPE_I* buffer);
int fnxreadi(FILE* fd,const int nb,NX_TYPE_I* buffer);
int fnxwrite_int(FILE* fd,const int nb,const NX_TYPE_I* buffer);
int fnxread_int(FILE* fd,const int nb,NX_TYPE_I* buffer);
int fnxwrite_long(FILE* fd,const int nb,const NX_TYPE_L* buffer);
int fnxread_long(FILE* fd,const int nb,NX_TYPE_L* buffer);

#if defined(NX_LONGLONG)
int fnxwrite_longlong(FILE* fd,const int nb,const NX_TYPE_LL* buffer);
int fnxread_longlong(FILE* fd,const int nb,NX_TYPE_LL* buffer);
#endif

int fnxwrite_int_type(FILE* fd,const int nb,const int type_size,
		const NX_TYPE_T* buffer);
int fnxread_int_type(FILE* fd,const int nb,const int type_size,
		NX_TYPE_T* buffer);

int fnxnbkwi(FILE* fd,const NX_TYPE_I* buffer,const int lg);
int fnxnbkri(FILE* fd,NX_TYPE_I* buffer,const int lg);
int fnxnbkw_int(FILE* fd,const NX_TYPE_I* buffer,const int lg);
int fnxnbkr_int(FILE* fd,NX_TYPE_I* buffer,const int lg);
int fnxnbkw_long(FILE* fd,const NX_TYPE_L* buffer,const int lg);
int fnxnbkr_long(FILE* fd,NX_TYPE_L* buffer,const int lg);

#if defined(NX_LONGLONG)
int fnxnbkw_longlong(FILE* fd,const NX_TYPE_LL* buffer,const int lg);
int fnxnbkr_longlong(FILE* fd,NX_TYPE_LL* buffer,const int lg);
#endif

/* routines de lecture, ecriture pour reels simple precision de type FORTRAN
   ...nbk... <=> non bloquant sinon bloquant */

int fnxwritf(FILE* fd,const int nb,const NX_TYPE_C* buffer);
int fnxreadf(FILE* fd,const int nb,NX_TYPE_C* buffer);
int fnxwrite_float(FILE* fd,const int nb,const NX_TYPE_F* buffer);
int fnxread_float(FILE* fd,const int nb,NX_TYPE_F* buffer);
int fnxwrite_double(FILE* fd,const int nb,const NX_TYPE_D* buffer);
int fnxread_double(FILE* fd,const int nb,NX_TYPE_D* buffer);

int fnxwrite_float_type(FILE* fd,const int nb,const int type_size,
		const NX_TYPE_T* buffer);
int fnxread_float_type(FILE* fd,const int nb,const int type_size,
		NX_TYPE_T* buffer);

int fnxnbkwf(FILE* fd,const NX_TYPE_C* buffer,const int lg);
int fnxnbkrf(FILE* fd,NX_TYPE_C* buffer,const int lg);
int fnxnbkw_float(FILE* fd,const NX_TYPE_F* buffer,const int lg);
int fnxnbkr_float(FILE* fd,NX_TYPE_F* buffer,const int lg);
int fnxnbkw_double(FILE* fd,const NX_TYPE_D* buffer,const int lg);
int fnxnbkr_double(FILE* fd,NX_TYPE_D* buffer,const int lg);

/* icod = type des donnees de buffer
   ocod = type des donnees ecrites ou lues
   valeurs :
   "cp" <=> float de type C
   "sp" <=> reel simple precision de type FORTRAN
   "dp" <=> reel double precision */

int fnxwritt(FILE* fd,const int nb,const NX_TYPE_T* buffer,const char* icod,
		const char* ocod);
int fnxreadt(FILE* fd,const int nb,NX_TYPE_T* buffer,const char* icod,
		const char* ocod);
int fnxnbkwt(FILE* fd,const NX_TYPE_T* buffer,const int lg,const char* icod,
		const char* ocod);
int fnxnbkrt(FILE* fd,NX_TYPE_T* buffer,const int lg,const char* icod,
		const char* ocod);

#ifdef __cplusplus
}
#endif

#endif /* fin NX_H */
