#ifndef NXSTREAMSTDIO_H
#define NXSTREAMSTDIO_H

// %W% %G% Pierre Brochard, pierre.brochard.1982@m4x.org

#include "U8machcomp.h"
#include "U8signal.h"
#include "nxstreamiocompat.h"

#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

// utiliser la classe indirectbuf de GNU ou norme ANSI des que possible
// Pour GNU voir sources stdiobuf : fonctions buffered a definir
// Dans le cas defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
// un filebuf a des chances d etre compatible C/C++ ie FILE est
// implemente de maniere compatible avec filebuf !

#if !defined(NAR_SUN_ISO) && !defined(NAR_GNU_ISO) && !defined(NAR_INTEL_ISO)
#   include <stdiostream.h>
#else

// Cas compilateur SUN ISO C++ : a creuser
// Il faut que les fonctions de iostream marchent !
// Certaines fonctions sont a surdefinir.

#  if defined(NAR_GNU_ISO)
     using std::streamsize;
     using std::ios_base;
     using std::char_traits;
     typedef char_traits<char> traits;
#  else
     using std::streamsize;
     using std::ios_base;
#    include <rw/traits>
     using std::char_traits;
     typedef char_traits<char> traits;

#    include "nx.h"

#    if defined(_LP64)
#      include <stdio_ext.h>
#    endif

#  endif

// Pour les patchs SUN !!

#if defined(_RWSTD_VER) && _RWSTD_VER < 0x020101
    typedef int nx_streamsize;
#elif defined(_RWSTD_VER) || defined(__INTEL_COMPILER) || defined(NAR_GNU_ISO)
    typedef streamsize nx_streamsize;
#else
    typedef int nx_streamsize;
#endif

// Les methodes put devraient etre pas mal pour des ecritures ostream
// non nxstream.

// Les methodes get sont a tester avec des lectures ostream
// non nxstream.

class stdiobuf : public streambuf
{
    public:

	stdiobuf(FILE* const file) : streambuf(),_file(file),
				     _filer(fnx_read_desc(_file)),
				     _filew(fnx_write_desc(_file))
		{

			if(!_file)
			    return;

			streambuf::setbuf(NULL,0);
			mygsync();
			mypsync();
		}
	~stdiobuf() {}

	FILE* stdiofile() const {return _file;}

    // A revoir
	streambuf* setbuf(char_type* s, streamsize n)
		{

			if(s)
			{
			    const streamsize n2 = n/2;
			    const streamsize n21 = n - n2;

			    setvbuf(_filer, s, _IOFBF, n2);
			    setvbuf(_filew, s + n2, _IOFBF, n21);
			    mygsync();
			    mypsync();
			}

			return this;
		}

    // A revoir
	streamsize xsgetn(char_type* const s,const streamsize n)
		{

			if(n == 1)
			{
			    register int ch;

			    U8GETC(_filer,ch);
			    mygsync();

			    if(ch == EOF)
				return 0;

			    *s = char_type(ch);

			    return 1;
			}
			else
			{
			    size_t lus;

			    U8FREAD(s,n,_filer,lus);
			    mygsync();

			    return lus;
			}

		}

    // A revoir
	int_type underflow()
		{
			register int ch;

			U8GETC(_filer,ch);

			if(ch != EOF)
			    ungetc(ch,_filer);

			mygsync();

			if(ch == EOF)
			    return traits::eof();
			else
			    return ch;

		}

    // A revoir
	int_type uflow()
		{
		    /*const int_type ch = underflow();

			if(ch == traits::eof())
			    return traits::eof();
			else
			{
			    gbump(1);
			    _filer->_ptr++;
			    _filer->_cnt--;

			    return *gptr();
			    }*/

			return streambuf::uflow();
		}

    // A revoir
	nx_streamsize showmanyc()
		{
#if defined(SUNOS) && defined(_STDIO_EXT_H)
			const nx_streamsize show =
			    nx_streamsize(_filer->__pad[3]);

			return show;
#elif defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			return _filer->_IO_read_end - _filer->_IO_read_ptr;
#elif defined(WIN32)
			return _filer->_endr - _filer->_next;
#else
			return _filer->_cnt;
#endif
		}

	streamsize xsputn(const char_type* s, streamsize n)
		{
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			const int delta = pptr() - (char*) _filew->_IO_write_ptr;
#elif defined(WIN32)
			const int delta = pptr() - (char*) _filew->_next;
#else
			const int delta = pptr() - (char*) _filew->_ptr;
#endif

			if(delta > 0)
			{
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			    _filew->_IO_write_ptr += delta;
#elif defined(WIN32)
			    _filew->_next += delta;
#else
			    _filew->_ptr += delta;
#endif
			}

			size_t count;

			U8FWRITE(s,n,_filew,count);
			mypsync();

			return count;
		}

	int_type overflow(int_type c = traits::eof())
		{

			if(sync())
			    return traits::eof();

			if(c != traits::eof())
			{
			    int count;

			    U8PUTC(c,_filew,count);

			    if(count == EOF)
				return traits::eof();

			}

			mypsync();

			return c;
		}

	int sync()
		{

			if(pbase())
			{
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			    const int delta =
				pptr() - (char*) _filew->_IO_write_ptr;
#elif defined(WIN32)
			    const int delta = pptr() - (char*) _filew->_next;
#else
			    const int delta = pptr() - (char*) _filew->_ptr;
#endif

			    if(delta > 0)
			    {
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
				_filew->_IO_write_ptr += delta;
#elif defined(WIN32)
				_filew->_next += delta;
#else
				_filew->_ptr += delta;
#endif
			    }

			    int rc;

			    U8FFLUSH(_filew,rc);
			    mypsync();

			    if(rc == EOF)
				return -1;

			}

		    // A revoir
		    /*if(gptr())
			{
			    int rc;

			    U8FFLUSH(_filer,rc);

#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			    setg((char*) _filer->_IO_read_base,
					    (char*) _filer->_IO_read_ptr,
					    (char*) _filer->_IO_read_end);
#else
			    setg((char*) _filer->_base,(char*) _filer->_ptr,
					    (char*) _filer->_ptr + _filer->_cnt);
#endif

			    if(rc == EOF)
				return -1;

				}*/

			return 0;
		}

    // A revoir
	pos_type seekoff(off_type off,
			ios_base::seekdir way, ios_base::openmode which = 
			ios_base::in | ios_base::out)
		{
			const int fway = (way == ios::beg) ? SEEK_SET :
			    (way == ios::cur) ? SEEK_CUR :
			    (way == ios::end) ? SEEK_END : way;
			int ret;

			while((ret = fseek(_file,off,fway)) == -1 &&
					errno == EINTR);

			return ret;
		}

    // A revoir
	pos_type seekpos(pos_type sp,
			ios_base::openmode which = 
			ios_base::in | ios_base::out)
		{
			const int fway = SEEK_SET;
			int ret;

			while((ret = fseek(_file,sp,fway)) == -1 &&
					errno == EINTR);

			return ret;
		}

    // A revoir
	int_type pbackfail(int_type c = traits::eof())
		{
			const int rc = ungetc(c,_filer);

			mygsync();

			if(rc == EOF)
			    return traits::eof();
			else
			    return rc;

		}

    private:
	void mypsync()
		{
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			setp((char*) _filew->_IO_write_base,
					(char*) _filew->_IO_write_end);
			pbump(_filew->_IO_write_ptr - _filew->_IO_write_base);
#elif defined(WIN32)
			setp((char*) _filew->_data,(char*) _filew->_endw);
			pbump(_filew->_next - _filew->_data);
#else
			setp((char*) _filew->_base,
					(char*) (_filew->_ptr+_filew->_cnt));
			pbump(_filew->_ptr - _filew->_base);
#endif
		}

	void mygsync()
		{
#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
			setg((char*) _filer->_IO_read_base,
					(char*) _filer->_IO_read_ptr,
					(char*) _filer->_IO_read_end);
#elif defined(WIN32)
			setg((char*) _filer->_data,
					(char*) _filer->_next,
					(char*) _filer->_endr);
#else
			setg((char*) _filer->_base,(char*) _filer->_ptr,
					(char*) _filer->_ptr+_filer->_cnt);
#endif
		}

    private:

	FILE* _file;
	FILE* _filer;
	FILE* _filew;
};

#endif

#endif /* fin NXSTREAMSTDIO_H */
