#ifndef NXXDR_H
#define NXXDR_H

// %W% %G% P. Brochard & M. Chineaud

// *********************************************************************
// fichier pour unifier les differences entre implementations de XDR
// *********************************************************************

#if defined(OSF1)
extern "C" {
#endif

#include <rpc/types.h>

#if defined(OSF1)
}
#endif


#if (!defined(_IBMR2) && !defined(CRAY) && !defined(OSF1)) || \
	!defined(__cplusplus) 
#   include <rpc/xdr.h>
#elif (defined(_IBMR2) || defined(OSF1)) && defined(__cplusplus)
#   define __XDR_HEADER__

/* le rpc/xdr.h d'IBM ne marche pas correctement en C++ */

extern "C" {

enum xdr_op {
	XDR_ENCODE=0,
	XDR_DECODE=1,
	XDR_FREE=2
};

#define BYTES_PER_XDR_UNIT	(4)
#define RNDUP(x)  ((((x) + BYTES_PER_XDR_UNIT - 1) / BYTES_PER_XDR_UNIT) \
		    * BYTES_PER_XDR_UNIT)

typedef	bool_t (*xdrproc_t)();

typedef struct {
	enum xdr_op	x_op;		/* operation; fast additional param */
	struct xdr_ops {
		bool_t	(*x_getlong)(...);
		bool_t	(*x_putlong)(...);	/* put a long to " */
		bool_t	(*x_getbytes)(...);/* get some bytes from " */
		bool_t	(*x_putbytes)(...);/* put some bytes to " */
		u_int	(*x_getpostn)(...);/* returns bytes off from beginning */
		bool_t  (*x_setpostn)(...);/* lets you reposition the stream */
		long *	(*x_inline)(...);
		void	(*x_destroy)(...);
	} *x_ops;
	caddr_t 	x_public;	/* users' data */
	caddr_t		x_private;	/* pointer to private data */
	caddr_t 	x_base;		/* private used for position info */
	int		x_handy;	/* extra private word */
} XDR;

#define XDR_GETLONG(xdrs, longp)			\
	(*(xdrs)->x_ops->x_getlong)(xdrs, longp)
#define xdr_getlong(xdrs, longp)			\
	(*(xdrs)->x_ops->x_getlong)(xdrs, longp)

#define XDR_PUTLONG(xdrs, longp)			\
	(*(xdrs)->x_ops->x_putlong)(xdrs, longp)
#define xdr_putlong(xdrs, longp)			\
	(*(xdrs)->x_ops->x_putlong)(xdrs, longp)

#define XDR_GETBYTES(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)
#define xdr_getbytes(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)

#define XDR_PUTBYTES(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)
#define xdr_putbytes(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)

#define XDR_GETPOS(xdrs)				\
	(*(xdrs)->x_ops->x_getpostn)(xdrs)
#define xdr_getpos(xdrs)				\
	(*(xdrs)->x_ops->x_getpostn)(xdrs)

#define XDR_SETPOS(xdrs, pos)				\
	(*(xdrs)->x_ops->x_setpostn)(xdrs, pos)
#define xdr_setpos(xdrs, pos)				\
	(*(xdrs)->x_ops->x_setpostn)(xdrs, pos)

#define	XDR_INLINE(xdrs, len)				\
	(*(xdrs)->x_ops->x_inline)(xdrs, len)
#define	xdr_inline(xdrs, len)				\
	(*(xdrs)->x_ops->x_inline)(xdrs, len)

#define	XDR_DESTROY(xdrs)				\
	(*(xdrs)->x_ops->x_destroy)(xdrs)
#define	xdr_destroy(xdrs) XDR_DESTROY(xdrs)

#define NULL_xdrproc_t ((xdrproc_t)0)
struct xdr_discrim {
	int	value;
	xdrproc_t proc;
};

#define IXDR_GET_LONG(buf)		((long)ntohl((u_long)*(buf)++))
#define IXDR_PUT_LONG(buf, v)		(*(buf)++ = (long)htonl((u_long)v))

#define IXDR_GET_BOOL(buf)		((bool_t)IXDR_GET_LONG(buf))
#define IXDR_GET_ENUM(buf, t)		((t)IXDR_GET_LONG(buf))
#define IXDR_GET_U_LONG(buf)		((u_long)IXDR_GET_LONG(buf))
#define IXDR_GET_SHORT(buf)		((short)IXDR_GET_LONG(buf))
#define IXDR_GET_U_SHORT(buf)		((u_short)IXDR_GET_LONG(buf))

#define IXDR_PUT_BOOL(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_ENUM(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_LONG(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_SHORT(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_SHORT(buf, v)	IXDR_PUT_LONG((buf), ((long)(v)))

extern bool_t	xdr_void();
extern bool_t	xdr_int(XDR*, int*);
extern bool_t	xdr_u_int(XDR*, u_int*);
extern bool_t	xdr_long(XDR*, long*);
extern bool_t	xdr_u_long(XDR*, u_long*);
extern bool_t	xdr_short(XDR*, short*);
extern bool_t	xdr_u_short(XDR*, u_short*);
extern bool_t	xdr_bool(XDR*, bool_t*);
extern bool_t	xdr_enum(XDR*, int*);
extern bool_t	xdr_array(XDR*, char **, u_int *, u_int, u_int, xdrproc_t);
extern bool_t	xdr_bytes(XDR*, char **, u_int *, u_int);
extern bool_t	xdr_opaque(XDR*, char *, u_int);
extern bool_t	xdr_string(XDR*, char **, u_int);
extern bool_t	xdr_union(XDR*, int *, char *, struct xdr_discrim *,
			  xdrproc_t *);
extern bool_t	xdr_char(XDR*, char *);
extern bool_t	xdr_u_char(XDR*, u_char *);
extern bool_t	xdr_vector(XDR*, char*, u_int, u_int, xdrproc_t);
extern bool_t	xdr_float(XDR*, float*);
extern bool_t	xdr_double(XDR*, double*);
extern bool_t	xdr_reference(XDR*, char **, u_int, xdrproc_t);
extern bool_t	xdr_pointer(XDR*, char **, u_int, xdrproc_t);
extern bool_t	xdr_wrapstring(XDR*, char **);
extern void   xdrmem_create(XDR*, char*, u_int, enum xdr_op);
extern void   xdrstdio_create(XDR*, FILE*, enum xdr_op);
extern void   xdrrec_create(XDR*, u_int, u_int, char *,
			    int (*read)(int, void *, size_t),
			    int (*write)(int, const void *, size_t));
extern bool_t xdrrec_endofrecord(XDR*, bool_t);
extern bool_t xdrrec_skiprecord(XDR*);
extern bool_t xdrrec_eof(XDR*);

#define MAX_NETOBJ_SZ 1024 
struct netobj {
	u_int	n_len;
	char	*n_bytes;
};
typedef struct netobj netobj;

extern bool_t   xdr_netobj(XDR*, struct netobj *);
}

#else /* CRAY et __cplusplus */
#   define __XDR_HEADER__

/* le rpc/xdr.h de CRAY ne marche pas correctement en C++ */

enum xdr_op {
        XDR_ENCODE=0,
        XDR_DECODE=1,
        XDR_FREE=2
};

#define BYTES_PER_XDR_UNIT      (4)
#define RNDUP(x)  ((((x) + BYTES_PER_XDR_UNIT - 1) / BYTES_PER_XDR_UNIT) \
                    * BYTES_PER_XDR_UNIT)

#include <sys/cdefs.h>

typedef bool_t (*xdrproc_t)(_DOTDOTDOT);

typedef struct {
        enum xdr_op     x_op;           /* operation; fast additional param */
        struct xdr_ops {
                bool_t  (*x_getlong)(_DOTDOTDOT);
                bool_t  (*x_putlong)(_DOTDOTDOT); /* put a long to " */
                bool_t  (*x_getbytes)(_DOTDOTDOT);/* get some bytes from " */
                bool_t  (*x_putbytes)(_DOTDOTDOT);/* put some bytes to " */
                u_int   (*x_getpostn)(_DOTDOTDOT);
                bool_t  (*x_setpostn)(_DOTDOTDOT);
#ifdef _CRAY
                inline_t *(*x_inline)(_DOTDOTDOT);
#else
                long *  (*x_inline)(_DOTDOTDOT);
#endif
                void    (*x_destroy)(_DOTDOTDOT);
        } *x_ops;
        caddr_t         x_public;       /* users' data */
        caddr_t         x_private;      /* pointer to private data */
        caddr_t         x_base;         /* private used for position info */
        int             x_handy;        /* extra private word */
        int             x_size;         /* yet another */
} XDR;

#define XDR_GETLONG(xdrs, longp)                        \
        (*(xdrs)->x_ops->x_getlong)(xdrs, longp)
#define xdr_getlong(xdrs, longp)                        \
        (*(xdrs)->x_ops->x_getlong)(xdrs, longp)

#define XDR_PUTLONG(xdrs, longp)                        \
        (*(xdrs)->x_ops->x_putlong)(xdrs, longp)
#define xdr_putlong(xdrs, longp)                        \
        (*(xdrs)->x_ops->x_putlong)(xdrs, longp)

#define XDR_GETBYTES(xdrs, addr, len)                   \
        (*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)
#define xdr_getbytes(xdrs, addr, len)                   \
        (*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)

#define XDR_PUTBYTES(xdrs, addr, len)                   \
        (*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)
#define xdr_putbytes(xdrs, addr, len)                   \
        (*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)

#define XDR_GETPOS(xdrs)                                \
        (*(xdrs)->x_ops->x_getpostn)(xdrs)
#define xdr_getpos(xdrs)                                \
        (*(xdrs)->x_ops->x_getpostn)(xdrs)

#define XDR_SETPOS(xdrs, pos)                           \
        (*(xdrs)->x_ops->x_setpostn)(xdrs, pos)
#define xdr_setpos(xdrs, pos)                           \
        (*(xdrs)->x_ops->x_setpostn)(xdrs, pos)

#define XDR_INLINE(xdrs, len)                           \
        (*(xdrs)->x_ops->x_inline)(xdrs, len)
#define xdr_inline(xdrs, len)                           \
        (*(xdrs)->x_ops->x_inline)(xdrs, len)

#define XDR_DESTROY(xdrs)                               \
        (*(xdrs)->x_ops->x_destroy)(xdrs)
#define xdr_destroy(xdrs) XDR_DESTROY(xdrs)

#define NULL_xdrproc_t ((xdrproc_t)0)
struct xdr_discrim {
        int     value;
        xdrproc_t proc;
};

#ifdef  _CRAY

#define IXDR_GET_LONG(buf) \
      ( \
        (buf += BYTES_PER_XDR_UNIT), \
        (ntohl( \
                (buf == (inline_t *)(word64 *)buf) ? \
                        (   ((word64 *)(buf-4))->lower | \
                           -(long)(((word64 *)(buf-4))->lower & 0x80000000) \
                        ) \
                : \
                        (   ((word64 *)buf)->upper | \
                           -(long)(((word64 *)buf)->upper & 0x80000000) \
                        ) \
              ) \
        ) \
      )

#define IXDR_GET_RU_LONG(buf)\
        ( \
                (buf += BYTES_PER_XDR_UNIT), \
                (ntohl( \
                    (buf == (inline_t *)(word64 *)buf) ? \
                        ((word64 *)(buf-4))->lower : ((word64 *)buf)->upper \
                      ) \
                ) \
        )

#define IXDR_GET_U_LONG(buf)    ((u_long)IXDR_GET_LONG(buf))
#define IXDR_PUT_LONG(buf, v)   ((buf == (inline_t *)(word64 *)buf ? \
                                    (((word64 *)buf)->upper = htonl(v)) : \
                                    (((word64 *)buf)->lower = htonl(v))), \
                                 buf += BYTES_PER_XDR_UNIT)
#else /* _CRAY */
#define IXDR_GET_LONG(buf)              ((long)ntohl((u_long)*(buf)++))
#define IXDR_GET_U_LONG(buf)            ((u_long)IXDR_GET_LONG(buf))
#define IXDR_PUT_LONG(buf, v)           (*(buf)++ = (long)htonl((u_long)v))
#endif
#define IXDR_GET_BOOL(buf)              ((bool_t)IXDR_GET_LONG(buf))
#define IXDR_GET_ENUM(buf, t)           ((t)IXDR_GET_LONG(buf))
#define IXDR_GET_SHORT(buf)             ((short)IXDR_GET_LONG(buf))
#define IXDR_GET_U_SHORT(buf)           ((u_short)IXDR_GET_U_LONG(buf))
 
#define IXDR_PUT_BOOL(buf, v)           IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_ENUM(buf, v)           IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_LONG(buf, v)         IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_SHORT(buf, v)          IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_SHORT(buf, v)        IXDR_PUT_LONG((buf), ((long)(v)))

__BEGIN_DECLS
extern bool_t xdr_array __((XDR *_Xdrs, caddr_t *_Addrp, u_int *_Sizep,
        u_int _Maxsize, u_int _Elsize, xdrproc_t _Elproc));
extern bool_t xdr_bool __((XDR *_Xdrs, bool_t *_Bp));
extern bool_t xdr_bytes __((XDR *_Xdrs, char **_Cpp, u_int *_Sizep,
        u_int _Maxsize));
extern bool_t xdr_char __((XDR *_Xdrs, char *_Cp));
extern bool_t xdr_double __((XDR *_Xdrs, double *_Dp));
extern bool_t xdr_enum __((XDR *_Xdrs, enum_t *_Ep));
extern bool_t xdr_float __((XDR *_Xdrs, float *_Fp));
extern bool_t xdr_int __((XDR *_Xdrs, int *_Ip));
extern bool_t xdr_long __((XDR *_Xdrs, long *_Lp));
extern bool_t xdr_opaque __((XDR *_Xdrs, caddr_t _Cp, u_int _Cnt));
extern bool_t xdr_pointer __((XDR *_Xdrs, char **_Objpp, u_int _Obj_size,
        xdrproc_t _Xdr_obj));
extern bool_t xdr_reference __((XDR *_Xdrs, caddr_t *_Pp, u_int _Size,
        xdrproc_t _Proc));
extern bool_t xdr_short __((XDR *_Xdrs, short *_Sp));
extern bool_t xdr_string __((XDR *_Xdrs, char **_Cpp, u_int _Maxsize));
extern bool_t xdr_u_char __((XDR *_Xdrs, char *_Cp));
extern bool_t xdr_u_int __((XDR *_Xdrs, u_int *_Up));
extern bool_t xdr_u_long __((XDR *_Xdrs, u_long *_Ulp));
extern bool_t xdr_u_short __((XDR *_Xdrs, u_short *_Usp));
extern bool_t xdr_union __((XDR *_Xdrs, enum_t *_Dscmp, char *_Unp,
        struct xdr_discrim *_Choices, xdrproc_t _Dfault));
extern bool_t xdr_vector __((XDR *_Xdrs, char *_Basep, u_int _Nelem,
        u_int _Elemsize, xdrproc_t _Xdr_elem));
extern bool_t xdr_void __((void));
extern bool_t xdr_wrapstring __((XDR *_Xdrs, char **_Cpp));
__END_DECLS

#define MAX_NETOBJ_SZ 1024
struct netobj {
        u_int   n_len;
        char    *n_bytes;
};
typedef struct netobj netobj;
extern bool_t   xdr_netobj();

#ifndef KERNEL
#include <stdio.h>
#endif  
__BEGIN_DECLS
extern void xdrmem_create __((XDR *_Xdrs, caddr_t _Addr, u_int _Size,
        enum xdr_op _Op));
#ifndef KERNEL
extern void xdrstdio_create __((XDR *_Xdrs, FILE *_File, enum xdr_op _Op));
#endif  
extern void xdrrec_create __((XDR *_Xdrs, u_int _Sendsize, u_int _Recvsize,
        u_long _Tcp_handle, int (*_Readit)(), int (*_Writeit)()));
extern bool_t xdrrec_endofrecord __((XDR *_Xdrs, bool_t _Sendnow));
extern bool_t xdrrec_eof __((XDR *_Xdrs));
extern bool_t xdrrec_skiprecord __((XDR *_Xdrs));
__END_DECLS

#endif /* fin _IBMR2 et CRAY */

#endif /* fin NXXDR_H */
