#ifndef U8PGML_H
#define U8PGML_H

// %W% %G% Pierre Brochard

#include <stdio.h>
#include "poulpe/U8driver.h"

// **********************************************************
//     classe U8PGML : Driver PGML (draft W3C)
//     Pour interfacer avec Sanglier (bibliotheque L3) :
//
//     U8PGML* pgml = new U8PGML;
//     pgml->NomDebutFichier("debut_nom_fichier");
//     U8PGML::Driver(pgml);
//     L3AddNewDriver(U8PGML::L3PGMLSanglier,"PGML",TRUE);
//     L3ChoixDriver("PGML");
//
//     L3FinChoixDriver("PGML");
//     delete pgml;
// **********************************************************

class U8PGML : public U8DriverL3
{
    public:

        U8PGML();
	~U8PGML();

	void L3Open(const int bloc_l3[]);
	void L3Close(const int bloc_l3[]);
	void L3Texte(const int bloc_l3[]);
	void L3Colormap(const int bloc_l3[]);
	void L3Couleur(const int bloc_l3[]);
	void L3Pixel(const int bloc_l3[]);
	void L3Cadre(const int bloc_l3[]);
	void L3Segment(const int bloc_l3[]);
	void L3Marqueur(const int bloc_l3[]);
	void L3Polygone(const int bloc_l3[]);
	void L3Flush(const int bloc_l3[]);
	void L3Default(const int bloc_l3[]);

    // pour interface avec sanglier

	inline static void L3PGMLSanglier(int codel3[]);
	inline static void Driver(U8PGML* const);
	static U8PGML* Driver();

    // couleurs

	static const int _defncolors; // taille de _defaultmap

    // init du debut du nom de fichier (se termine par XXXX.pgml ou
    // XXXX est un compteur sur 4 chiffres commencant a 0000)

	void NomDebutFichier(const char* filedebut);

    private:

    // pour interfacer avec sanglier

	static U8PGML* _pgml;

    // open/close

	int _opengroup;
	char* _filedebut;
	char* _filename;
	FILE* _filenx;
	FILE* _file;
	static int _numfile;

    // pour bounding box

	double _factor;
	int _xmin;
	int _xmax;
	int _ymin;
	int _ymax;

    // coefs de changement d echelle

	double _a;
	double _b;
	double _c;
	double _d;

    // couleurs

	static const int _maxcolors;
	static const int _defaultmap[];
	int _ncolors;
	int _color;
	int* _red;
	int* _green;
	int* _blue;
};

inline void
U8PGML::L3PGMLSanglier(int codel3[])
{

	if(_pgml) _pgml->L3Driver(codel3);

}

inline void
U8PGML::Driver(U8PGML* const pgml)
{
	_pgml = pgml;
}

inline U8PGML*
U8PGML::Driver()
{
	return _pgml;
}

#endif /* fin U8PGML_H */
