#ifndef U8SVG_H
#define U8SVG_H

// %W% %G% Pierre Brochard

#include <stdio.h>
#include "poulpe/U8driver.h"

// **********************************************************
//     classe U8SVG : Driver SVG (draft W3C)
//     Pour interfacer avec Sanglier (bibliotheque L3) :
//
//     U8SVG* svg = new U8SVG;
//     svg->NomDebutFichier("debut_nom_fichier");
//     U8SVG::Driver(svg);
//     L3AddNewDriver(U8SVG::L3SVGSanglier,"SVG",TRUE);
//     L3ChoixDriver("SVG");
//
//     L3FinChoixDriver("SVG");
//     delete svg;
// **********************************************************

class U8SVG : public U8DriverL3
{
    public:

        U8SVG();
	~U8SVG();

	void L3Open(const int bloc_l3[]);
	void L3Close(const int bloc_l3[]);
	void L3Texte(const int bloc_l3[]);
	void L3Colormap(const int bloc_l3[]);
	void L3Couleur(const int bloc_l3[]);
	void L3Pixel(const int bloc_l3[]);
	void L3Cadre(const int bloc_l3[]);
	void L3Segment(const int bloc_l3[]);
	void L3Marqueur(const int bloc_l3[]);
	void L3Polygone(const int bloc_l3[]);
	void L3Flush(const int bloc_l3[]);
	void L3Default(const int bloc_l3[]);

    // pour interface avec sanglier

	inline static void L3SVGSanglier(int codel3[]);
	inline static void Driver(U8SVG* const);
	static U8SVG* Driver();

    // couleurs

	static const int _defncolors; // taille de _defaultmap

    // init du debut du nom de fichier (se termine par XXXX.svg ou
    // XXXX est un compteur sur 4 chiffres commencant a 0000)

	void NomDebutFichier(const char* filedebut);

    private:

    // pour interfacer avec sanglier

	static U8SVG* _svg;

    // couleurs

	char* _coul;

    // open/close

	char* _filedebut;
	char* _filename;
	FILE* _filenx;
	FILE* _file;
	static int _numfile;

    // pour bounding box

	double _factor;
	int _xmin;
	int _xmax;
	int _ymin;
	int _ymax;

    // coefs de changement d echelle

	double _a;
	double _b;
	double _c;
	double _d;
	double _scaley;

    // couleurs

	static const int _maxcolors;
	static const char* _defaultmap[];
	int _ncolors;
	int _color;

    // open/close

	int _opengroup;
};

inline void
U8SVG::L3SVGSanglier(int codel3[])
{

	if(_svg) _svg->L3Driver(codel3);

}

inline void
U8SVG::Driver(U8SVG* const svg)
{
	_svg = svg;
}

inline U8SVG*
U8SVG::Driver()
{
	return _svg;
}

#endif /* fin U8SVG_H */
