/* %W% %G% */
#ifndef SANGLIER_H
#define SANGLIER_H

/*-------------------------------------------------------------------*
 *                            Partie publique                        *
 *-------------------------------------------------------------------*/

#ifndef PROTO_H
#define PROTO_H

#ifdef __STDC__
#define Proto(argliste) argliste
#else
#ifdef __cplusplus
#define Proto(argliste) argliste
#else
#define Proto(argliste) ()
#endif
#endif

#endif

/*-------------------------------------------------------------------*/

/*-----------------------------------------------*
 * Type general pour eviter les problemes futurs *
 *-----------------------------------------------*/

/* types REEL_S et REEL identiques a NXFLT et NXDBLE de nx */
/* sauf pour CRAY ou REEL est pris a double et non a long double */

#if defined(CRAY) && !defined(_CRAYMPP)
    typedef float REEL;
    typedef float REEL_S;
#elif defined(_CRAYMPP)
    typedef double REEL;
    typedef double REEL_S;
#else
    typedef double REEL;
    typedef float REEL_S;
#endif

/*-------------------------------------------------------------------*/

#ifndef VRAI
#define VRAI (1==1)
#define FAUX (1==0)
#endif

/*----------------------------------------------*
 * Definition des types de police de caracteres *
 *----------------------------------------------*/

#define POLICEHARD 0
#define POLICESOFT 1

/*------------------------------------------------------------*
 * Definition des caracteres de controle des modes d'ecriture *
 * du generateur de texte                                     *
 *------------------------------------------------------------*/

#define MODNORMAL    	'\1'
#define MODTEXTE     	'\2'
#define MODGRAPHIQUE 	'\3'
#define MODTEXTLOCAL	'\4'
#define MODGRAPHLOCAL	'\5'
#define MODINDICE    	'\6'
#define MODEXPOSANT  	'\7'
#define MODINDICEINF	'\10'
#define MODEXPOSANTSUP  '\11'
#define MODREDUIT	'\12'
#define MODAGRANDI	'\13'
#define MODDECALEINF	'\14'
#define MODDECALESUP	'\15'
#define MODRECUL	'\16'
#define MODINDPREC	'\17'
#define MODINDPRECRECUL	'\20'
#define MODFRACTION	'\21'
#define MODNUMDEN	'\22'
#define MODFINFRACTION	'\23'
#define MODDEBHAB	'\24'
#define MODFINHAB	'\25'
#define MODDEBINTEG	'\26'
#define MODSEPINTEG	'\27'
#define MODFININTEG	'\30'

typedef enum
{
    RACINE = 1, VECTEUR, SURLIGNE, SOULIGNE, CROCHET_CARRE,
    CROCHET, PARENTHESE, ACCOLADE
} type_habillage;

typedef enum
{
    INTEGRALE = 1, SOMME, PRODUIT
} type_encapsulage;

typedef enum
{
    SIMPLE = 1, DOUBLE, TRIPLE
} rang_integrale;

/*----------------------------------------------------------*
 * Definition du positionnement des labels dans les grilles *
 *----------------------------------------------------------*/

#define BAS    0
#define HAUT   1
#define GAUCHE 2
#define DROITE 3

/*----------------------------*
 * Definition des types d'axe *
 *----------------------------*/

#define AXE_PLEIN     0
#define AXE_CREUX     1
#define AXE_SEMICREUX 2
#define AXE_VIDE      3
#define AXE_ABSENT    4

/*-------------------------------------------*
 * Definition du mode logarithmique des axes *
 *-------------------------------------------*/

#define LINEAIRE 0
#define XCOLOG   1
#define YCOLOG   2
#define XAXLOG   4
#define YAXLOG   8

/*-------------------------------------------------------------------------*
 * Definition du choix automatique du nombre de graduations intermediaires *
 *-------------------------------------------------------------------------*/

#define AUTOMATIQUE -2

/*----------------------------------------------------*
 * Definition des types de factorisation sur les axes *
 *----------------------------------------------------*/

#define AUCUNE   0
#define MINIMALE 1
#define MOYENNE  2
#define MAXIMALE 3

/*-------------------------------------------------*
 * Definition de la position de cadrage d'un texte *
 *-------------------------------------------------*/

#define POSGB 0
#define POSGM 1
#define POSGH 2
#define POSMB 3
#define POSMM 4
#define POSMH 5
#define POSDB 6
#define POSDM 7
#define POSDH 8

/*------------------------------------*
 * Definition des modes de troncature *
 *------------------------------------*/

#define CLIPPINGACTIF   1
#define CLIPPINGINACTIF 2

/*------------------------------------------------------------*
 * Definition des positions d'un point dans une projection 2D *
 *------------------------------------------------------------*/

#define DEVANTOEIL   1
#define DERRIEREOEIL 2

/*---------------------------------------------------*
 * Definition des cartes de couleurs                 *
 *---------------------------------------------------*/

typedef enum
{
    ABSENTE = 0, PRESENTE
} flag_carte_de_couleurs;

typedef enum
{
    ABSOLUTE = 0, BINARY, BLUEGREEN, DEFAULT, NEGATIVE, POSITIVE,
    RAINBOW, RAINBOWHLS, RANDOM, REDBLUE, REDGREEN, SHIFTED, STANDARDL3,
    DLPP1, XCODE, INVDEFAULT
} nom_de_carte;

typedef struct
{
   
	int             r;
	int             g;
	int             b;
   
} RGB_INT;

typedef struct
{
	int             n;
	int            *r;
	int            *g;
	int            *b;
} carte_de_couleurs;

typedef struct
{
	REEL r;
	REEL g;
	REEL b;
} RGB ;

typedef struct
{
	REEL h;
	REEL s;
	REEL v;
} HSV ;

typedef struct
{
	REEL h;
	REEL l;
	REEL s;
} HLS ;

typedef struct
{
	REEL c;
	REEL m;
	REEL y;
} CMY ;

typedef struct
{
	REEL r;
	REEL b;
	REEL w;
} RBW ;

/* ---------------------------------------------------*/
/* Definition des modes de gestion de la copie        */
/* ---------------------------------------------------*/

typedef enum
{
    REMPLACEMENT, INCREMENTAL
} mode_gestion_copie;

/* ------------------------------------------------------*/
/* Interface de gestion des options du driver PostScript */
/* ------------------------------------------------------*/

typedef enum
{
    ENCOULEUR,
    NOIRETBLANC,
    TOUTNOIR,
    AUTEUR,
    LOGO,
    FICHIER,
    PRINTER,
    CHRONO,
    ABSOLU,
    REVERSEVIDEO,
    NOINITGRAPHICS,
    FICHIERDEBUT,
    TEXTESNOIR,
    TITRE,
    LANDSCAPE
} Option;

/* ------------------------------------------------------*/
/*    Interface C                                        */
/* ------------------------------------------------------*/

#ifdef __cplusplus
extern          "C" {
#endif

    void L3SetOptPost(const void* const arg,
		    Option opt);
    void L3ChoixDriver(const char *nom
		/* IN : negs, visu, post, lwc, lwct, T4014 */
		    );
    void L3FinChoixDriver(const char *nom
		/* IN : negs, visu, post, lwc, lwct, T4014 */
		    );
    void L3QuestionDriver(void);
    void L3SetModeCopie(int mode /* IN : TRUE / FALSE */ );
    void L3SetGestionCopie(mode_gestion_copie Mode /* IN */ );
    void L3Brillance(double brillance /* IN : 0. <= x <= 2. */ );
    void L3Cadre(double base /* IN : en cm */, 
		    double hauteur /* IN */ );
    void L3DetruitCopie(void);
    void L3UndoCopie(void);
    void L3AfficheCopie(void);
    void L3SetSeparateur(void);
    void L3Couleur(int couleur /* IN : < 256 */ );
    void L3Croix(int sousimage /* IN */ ,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    int npoints /* IN */ ,
		    REEL taille) /* IN : 0<=x<=1. fraction de la page totale */ ;
    void L3SaveCurseur(void);
    void L3RestoreCurseur(void);
    void L3DefPoliceCaractere(int typepolice/* IN : POLICEHARD / POLICESOFT */);
    void L3DefSpace(int sousimage /* IN */ ,
		    REEL xu	/* IN : point inferieur droit */,
		    REEL yu,
		    REEL xv	/* point superieur
				   gauche */  ,
		    REEL yv);
    void L3DefSpaceLog(int sousimage /* IN */ ,
		    REEL xu /* IN */ ,
		    REEL yu /* IN */ ,
		    REEL xv /* IN */ ,
		    REEL yv /* IN */ ,
		    int xcoordlog	/* IN : VRAI / FAUX mode
					 * logarithmique suivant
					 cet axe */ ,
		    int ycoordlog	/* IN : VRAI / FAUX mode
					 * logarithmique suivant
					 cet axe */ );
   
    void L3DefTexte(int sousimage /* IN */ ,
		    int ncaracligne /* IN */ ,
		    double angle	/* IN : en degre par
					 * rapport a
					 l'horizontal */ );
    void L3FenetreEcran(int type /* IN */ );
    void L3FlushBuffers(void);
    void L3Projection2D(int sousimage /* IN */ ,
		    REEL x3D /* IN */ ,
		    REEL y3D /* IN */ ,
		    REEL z3D /* IN */ ,
		    REEL * p_x2D /* OUT : [-1.0, 1.0] */ ,
		    REEL * p_y2D /* OUT : [-1.0, 1.0] */ ,
		    int *p_position	/* OUT : DEVANTOEIL /
					   DERRIEREOEIL */ );
    void L3DefSpace3D(int sousimage /* IN */ ,
		    REEL xoeil /* IN */ ,
		    REEL yoeil /* IN */ ,
		    REEL zoeil /* IN */ ,
		    REEL xvise /* IN */ ,
		    REEL yvise /* IN */ ,
		    REEL zvise /* IN */ ,
		    double angle	/* IN : champ de visee
					   en radians */ );
    void L3PointCourant3D(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    REEL z /* IN */ );
    void L3LigneBrisee3D(int sousimage /* IN */ ,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    REEL z[] /* IN */ ,
		    int nlignes /* IN */ );
    void L3Segment3D(int sousimage /* IN */ ,
		    REEL x1s[] /* IN */ ,
		    REEL y1s[] /* IN */ ,
		    REEL z1s[] /* IN */ ,
		    REEL x2s[] /* IN */ ,
		    REEL y2s[] /* IN */ ,
		    REEL z2s[] /* IN */ ,
		    int nsegments /* IN */ );
    void L3Marqueur3D(int sousimage,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    REEL z[] /* IN */ ,
		    int npoints /* IN */ ,
		    int typemarqueur /* IN */ );
    void L3ModeTexte(int mode /* IN */ ,
		    char *chaine /* INOUT */ );
    void L3GenTextePos(int sousimage /* IN */ ,
		    char texte[] /* IN */ ,
		    REEL xutil /* IN */ ,
		    REEL yutil /* IN */ ,
		    double echelle	/* IN : facteur
					 * d'echelle par rapport
					 * a la taille des
					 * caracteres definie
					 par L3DefTexte */ ,
		    double facteurz	/* IN : rapport d'aspect
					   du caractere */ ,
		    double angledegre	/* IN : par rapport a
					   l'horizontal */ ,
		    int modeprop /* IN : VRAI / FAUX */ ,
		    int position	/* IN : POSGB / POSGM /
					 * POSGH / POSMB / POSMM
					 * / POSMH / POSDB /
					 POSDM / POSDH */ );
    void L3GenTexte(int sousimage /* IN */ ,
		    const char texte[] /* IN */ ,
		    REEL xutil /* IN */ ,
		    REEL yutil /* IN */ ,
		    double echelle	/* IN : facteur
					 * d'echelle par rapport
					 * a la taille des
					 * caracteres definie
					 par L3DefTexte */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN : VRAI / FAUX */ );
    void L3DefTitreAxe(int sousimage /* IN */ ,
		    int nom_axe /* IN */ ,
		    char titre[] /* IN */ ,
		    char unite[] /* IN */ );
    void L3DefModeLog(int sousimage /* IN */ ,
		    int xcoordlog /* IN */ ,
		    int ycoordlog /* IN */ );
    void L3DefGrille(int sousimage /* IN */ ,
		    int axe /* IN */ ,
		    int typeaxe /* IN */ ,
		    int label /* IN */ ,
		    int gradinter /* IN */ ,
		    int factorisation	/* IN : AUCUNE /
					 * MINIMALE / MOYENNE /
					 MAXIMALE */ );
    void L3RestaureGrille(int sousimage /* IN */ );
    void L3Grille(int sousimage /* IN */ ,
		    REEL xd /* IN */ ,
		    REEL yd /* IN */ ,
		    REEL xf /* IN */ ,
		    REEL yf /* IN */ ,
		    int nx /* IN */ ,
		    int ny /* IN */ ,
		    int modes /* IN */ );
    void L3Ingraf(void);
    void L3ReinitGraf(int nimg /* IN */ );
    void L3Label(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    char texte[] /* IN */ ,
		    REEL valeur /* IN */ ,
		    int type /* 'I' ou 'E' ou'F' */ ,
		    int ncaracteres /* IN */ ,
		    int ndecimales /* IN */ );
    void L3LabelPos(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    char texte[] /* IN */ ,
		    REEL valeur /* IN */ ,
		    int type /* IN */ ,
		    int ncaracteres /* IN */ ,
		    int ndecimales /* IN */ ,
		    int position /* IN */ );
    void L3BeauLabel(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    char texte[] /* IN */ ,
		    REEL valeur /* IN */ ,
		    int nsignif /* IN */ );
    void L3BeauLabelPos(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    char texte[] /* IN */ ,
		    REEL valeur /* IN */ ,
		    int nsignif /* IN */ ,
		    int position /* IN */ );
    void L3BeauLabelPur(REEL valeur /* IN */ ,
		    int nsignif /* IN */ ,
		    char textesortie[] /* OUT */ ,
		    int Impose /* IN */
		    );
    void L3LectureCoordonnees(int sousimage /* IN */ ,
		    REEL * p_x,
		    REEL * p_y);
    void L3LigneBrisee(int sousimage /* IN */ ,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    int nlignes /* IN */ );
    void L3Marqueur(int sousimage /* IN */ ,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    int npoints /* IN */ ,
		    int typemarqueur /* IN */ );
    void L3MiseEnPage(int sousimage /* IN */ ,
		    REEL xa /* IN */ ,
		    REEL ya /* IN */ ,
		    REEL xb /* IN */ ,
		    REEL yb /* IN */ ,
		    REEL xp /* IN */ ,
		    REEL yp /* IN */ ,
		    REEL xq /* IN */ ,
		    REEL yq /* IN */ );
    void L3ModeAlphanum(void);
    void L3ModeGraphique(int i /* IN */ );
    void L3NegsEcran(int i /* IN */ );
    void L3ConversionCoordonnees(REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    int *p_xpage /* OUT */ ,
		    int *p_ypage /* OUT */ );
    void L3ConversionCoordonneesLog(REEL x /* IN */ ,
		    REEL y /* IN */ ,
		    int *p_xpage /* OUT */ ,
		    int *p_ypage /* OUT */ ,
		    int xcoordlog /* IN */ ,
		    int ycoordlog /* IN */ );
    void L3ConvCoordL3VersUtil(int xl3 /* IN */ ,
		    int yl3 /* IN */ ,
		    REEL * p_xutil /* OUT */ ,
		    REEL * p_yutil /* OUT */ );
    void L3ConvCoordL3VersUtilLog(int xl3 /* IN */ ,
		    int yl3 /* IN */ ,
		    REEL * p_xutil /* OUT */ ,
		    REEL * p_yutil /* OUT */ ,
		    int xcoordlog /* IN */ ,
		    int ycoordlog /* IN */ );
    void L3DialogueGETPOS(int codel3[] /* IN */ ,
		    int typecroix /* IN */ ,
		    int typeelastique /* IN */ ,
		    int typemarque /* IN */ ,
		    int typetrace /* IN */ );
    void L3PointCourant(int sousimage /* IN */ ,
		    REEL x /* IN */ ,
		    REEL y /* IN */ );
    void L3Polygone(int sousimage /* IN */ ,
		    REEL x[] /* IN */ ,
		    REEL y[] /* IN */ ,
		    int nsommets /* IN */ );
    void L3SautDePage(void);
    void L3Segment(int sousimage /* IN */ ,
		    REEL x1s[] /* IN */ ,
		    REEL y1s[] /* IN */ ,
		    REEL x2s[] /* IN */ ,
		    REEL y2s[] /* IN */ ,
		    int nsegments /* IN */ );
    void L3SegmentLog(int sousimage /* IN */ ,
		    REEL x1s[] /* IN */ ,
		    REEL y1s[] /* IN */ ,
		    REEL x2s[] /* IN */ ,
		    REEL y2s[] /* IN */ ,
		    int nsegments /* IN */ ,
		    int xcoordlog /* IN */ ,
		    int ycoordlog /* IN */ );
    void L3SegmentL3(int sousimage /* IN */ ,
		    int x1s[] /* IN */ ,
		    int y1s[] /* IN */ ,
		    int x2s[] /* IN */ ,
		    int y2s[] /* IN */ ,
		    int nsegments /* IN */ );
    void L3Texte(int sousimage /* IN */ ,
		    const char texte[] /* IN */ );
    void L3Tmgraf(void);
    void L3Troncature(int sousimage /* IN */ ,
		    int modtronc /* IN */ );
    void L3HSBColor(float h /* IN */ ,
		    float s /* IN */ ,
		    float b /* IN */ );
    void L3GenTexteL3(int sousimage /* IN */ ,
		    const char texte[] /* IN */ ,
		    int xl3 /* IN */ ,
		    int yl3 /* IN */ ,
		    double echelle /* IN */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN : VRAI / FAUX */ );
    void L3GenTexteEval(int sousimage /* IN */ ,
		    const char texte[] /* IN */ ,
		    double echelle /* IN */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN : VRAI / FAUX */ ,
		    int *p_hdeltaxl3 /* OUT */ ,
		    int *p_hdeltayl3 /* OUT */ ,
		    int *p_vdeltaxl3 /* OUT */ ,
		    int *p_vdeltayl3 /* OUT */ ,
		    int *p_offsetxl3 /* OUT */ ,
		    int *p_offsetyl3 /* OUT */ );
    void L3GenerationCaractere(int carascii /* IN */ ,
		    int xl3 /* IN */ ,
		    int yl3 /* IN */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN : VRAI / FAUX */ ,
		    int modgraf /* IN : VRAI / FAUX */ ,
		    double taille /* IN */ ,
		    int deltaxindl3 /* IN */ ,
		    int deltayindl3 /* IN */ ,
		    int *p_deltaxl3 /* OUT */ ,
		    int *p_deltayl3 /* OUT */ ,
		    int *p_nsegments /* OUT */ ,
		    int xdebut[] /* OUT */ ,
		    int ydebut[] /* OUT */ ,
		    int xfin[] /* OUT */ ,
		    int yfin[] /* OUT */ ,
		    int *p_erreur /* OUT */ );
    void L3GenerationCaractereEval(int carascii /* IN */ ,
		    int modeprop /* IN : VRAI / FAUX */ ,
		    int modgraf /* IN : VRAI / FAUX */ ,
		    double htaille /* IN */ ,
		    double vtaille /* IN */ ,
		    double v_offset /* IN */ ,
		    int *p_deltaxl3 /* OUT */ ,
		    int *p_deltayl3 /* OUT */ ,
		    int *p_offsetyl3 /* OUT */ ,
		    int *p_erreur /* OUT */ );
   
    void L3Generateur(int car /* caractere que l'on veut dessiner */ ,
		    int *p_npoint /* OUT */ ,
		    int xpoint[] /* OUT espace [0-7] */ ,
		    int ypoint[] /* OUT espace [0-7] */ ,
		    int modeprop /* IN : VRAI / FAUX */ ,
		    int modgraf /* IN : VRAI / FAUX */ ,
		    int *p_largeur	/* OUT : largeur du carac */,
		    int *p_erreur /* OUT : OK / NOK */ );
    void L3TranscodeChaine(char *source /* IN */ ,
		    char *cible /* OUT */ );
    void L3EncadreTexte(void);
    void L3CadreTexteEvalExact(int x[ /* 4 */ ] /* OUT */ ,
		    int y[ /* 4 */ ] /* OUT */ );
    void L3CadreTexteEvalEnglob(int *xmin /* OUT */ ,
		    int *xmax /* OUT */ ,
		    int *ymin /* OUT */ ,
		    int *ymax /* OUT */ );
    void L3BoiteTexteExacte(int sousimage /* IN */ ,
		    const char texte[] /* IN */ ,
		    double echelle /* IN */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN */ ,
		    int x[ /* 4 */ ] /* OUT */ ,
		    int y[ /* 4 */ ] /* OUT */ );
    void L3BoiteTexteEnglobante(int sousimage /* IN */ ,
		    const char texte[] /* IN */ ,
		    double echelle /* IN */ ,
		    double facteurz /* IN */ ,
		    double angledegre /* IN */ ,
		    int modeprop /* IN */ ,
		    int *deltax /* OUT */ ,
		    int *deltay /* OUT */ );
    void L3AfficheTitres(int image /* IN */ ,
		    char *texte /* IN */ ,
		    double ecart /* IN */ ,
		    int position /* IN */ );
    int L3ChoixCartesDeCouleurs(nom_de_carte nomcarte /* IN */,
		    int nombredecouleurs /* IN */,
		    RGB *fond,
		    RGB *antifond);
    carte_de_couleurs L3LitCartesDeCouleurs();
    void L3EcritCartesDeCouleurs(carte_de_couleurs);
    carte_de_couleurs L3SelectionCartesDeCouleurs(nom_de_carte nomcarte /*IN*/,
		    int nombredecouleurs /*IN*/,
		    RGB *fond,
		    RGB *antifond);
    void L3AddNewDriver(void (*NewDriver) (
		    int codel3[]
		    ) /* IN pt d'entree du driver */,
		    char *Nom /* du driver */,
		    int interactif /* TRUE si interactif */);
    void L3BarreErreur(int ssim,
		    REEL x[], REEL y[], /* courbe support */
		    REEL miny[], REEL maxy[], /* erreur absolue min et max */
		    int nbpts); 
    HSV L3RgbToHsv(
		    RGB rgb);
    RGB L3HsvToRgb(
		    HSV hsv);
    HLS L3RgbToHls(
		    RGB rgb);
    RGB L3HlsToRgb(
		    HLS hls);
    CMY L3RgbToCmy(
		    RGB rgb);
    RGB L3CmyToRgb(
		    CMY cmy);
    HSV L3RbwToHsv(
		    RBW rbw);
    RBW L3HsvToRbw(
		    HSV hsv);
    void L3EpaisseurDeLigne(
		    int epai);

#ifdef __cplusplus
}
#endif				/* __cplusplus */

#endif				/* SANGLIER_H */
