#ifndef P4GEOM_H
#define P4GEOM_H

// %W% %G% Pierre Brochard

#include "P4maillest4.h"
#include "P4maillens.h"
#include "P4maillensf.h"
#include "P4maillens3f.h"
#include "dinosaure/P4tabgenbase.h"
#include "P4proj.h"
#include "dinosaure/T7triter.h"


#if defined(P4VERSION_AVEC_TEMPLATE) && defined(P4TABSOMGRI_IMPLEMENT)
/* version avec template */

#  if P4VERSION_AVEC_TEMPLATE == 4
      typedef TabGEN<Sommet> TabSommet;
#  else
#     define TabSommet TabGEN<Sommet>
#  endif

#else // version sans template
   class TabSommet;
#endif // fin version sans template

class GeomPure
{	   
    public:

	GeomPure();
	GeomPure(const GeomPure& g);
	GeomPure(const TabREEL_S& x,const TabREEL_S& y,
			const TabREEL_S& z,const MailleSt4& mail);
	GeomPure(const TabREEL_S& x,const TabREEL_S& y,const TabREEL_S& z,
			const MailleNS& mail);
	virtual ~GeomPure();

	void Ecriture(P4Boolean ecriture = TRUE);

	unsigned NombrePoints() const {return _x;};

	const TabREEL_S& X() const {return _x;};
	const TabREEL_S& Y() const {return _y;};
	const TabREEL_S& Z() const {return _z;};

	void AjoutePoint(REEL point_x,REEL point_y,REEL point_z) ;

	void AjoutePoints(const TabREEL_S& x,const TabREEL_S& y,
			const TabREEL_S& z) ;
	int AjoutePoints(const int nx,const int ny,const int dimx,
			const int dimy,const int dimz,
			const REEL_S x[],const REEL_S y[],const REEL_S z[]);
	int AjoutePoints(const int nx,const int ny,const int dimx,
			const int dimy,const int dimz,
			const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const Bool3& lxyz);

	void ChercheMailles(const REEL xm,const REEL ym,
			TabSommet& ts,Tabunsigned& tm,
			const P4Boolean timer=FALSE) const;

	int InvProjete(const GeomPure& geo_projete,const Bool3& lxyz,
			const Sommet& orig,const Proj& proj);

	int Projete(const int nx,const int ny,const int dimx,
			const int dimy,const int dimz,const REEL_S x[],
			const REEL_S y[],const REEL_S z[],
			const Bool3& lxyz,const Sommet& orig,const Proj& proj) ;

	int Projete2d(const Sommet& orig,const Proj& proj) ;

	virtual void Enleve() ;

	friend ostream& operator << (ostream& ost,GeomPure g) ;
	void operator = (GeomPure& g) 
			{_x.Lier(g._x);_y.Lier(g._y);_z.Lier(g._z);
			delete ptma;
			ptma = g.ptma->NewCopy();}

    protected:

	P4Boolean _ecriture;          // lie au break negs
	TabREEL_S _x;
	TabREEL_S _y;
	TabREEL_S _z;     //les n de sommets commencent a 0 par convention
	MailleQcq* ptma;  //les n de mailles commencent  0 par convention
};

class Geom : public GeomPure, public T7TriTer
{
    public :

	Geom() : GeomPure(),T7TriTer() {};
	Geom(GeomPure& g) : GeomPure(g),T7TriTer() {};
	Geom(GeomPure& g,Tabunsigned& ord) : GeomPure(g),T7TriTer(ord) {};
	Geom(TabREEL_S& x,TabREEL_S& y,TabREEL_S& z,MailleSt4& mail,
		Tabunsigned& ord) : GeomPure(x,y,z,mail),T7TriTer(ord) {};
	Geom(TabREEL_S& x,TabREEL_S& y,TabREEL_S& z,MailleNS& mail,
		Tabunsigned& ord) : GeomPure(x,y,z,mail),T7TriTer(ord) {};
	~Geom() {};

	Test Compare(unsigned,unsigned,P4Boolean init=FALSE);

	void Compminx();
	void Compminy();
	void Delminx();
	void Delminy();

	int Break();    
};

inline void GeomPure::Ecriture(P4Boolean ecriture)
{
	_ecriture = ecriture;
}

#endif
