#ifdef P4GEOMCONV
// %W% %G% Pierre Brochard
// *******************************************************************
//    2 options : - pour 4eme tableau c grandeur aux noeuds
//                - pour 4eme tableau c grandeurs mailles
//    P4TYPE_DONNEE = 0 -> noeuds
//                  = 1 -> mailles
// *******************************************************************/

int
GeomPure4::P4GEOMCONV(GeomPure4& geo_convexe,Tabunsigned& ord,
	const P4Boolean timer,const P4Boolean propag_vol) const
{

    // *------------------------------------------------------------*
    // *   convexification de mailles                               *
    // *   valable pour des mailles a nombre de cotes quelconques   *
    // *   et non planaires                                         *
    // *   -> geo_convexe contiendra la geometrie modifiee          *
    // *   au depart elle doit contenir les points _x _y _z _c      *
    // *   de this, on y rajoute les points crees ainsi que le      *
    // *   nouveau tableau de mailles.                              *
    // *   -> ord sera rempli avec le nouvel ordre des mailles      *
    // *   en considerant qu'il contenait deja un ordre             *
    // *   le calcul du nouvel ord n'est pas fait s'il contient     *
    // *   Dep en premiere valeur                                   *
    // *   -> timer donne ou non une impression de temps d'execution*
    // *------------------------------------------------------------*

	if(timer) TimerDetInit();

	const REEL tolerance = 0.;
	const unsigned TRACE_FIL_NON_U = TRACE_FIL_NON;
	const unsigned TRACE_FIL_OUI_U = TRACE_FIL_OUI;
	const unsigned nma = ptma->Nmailles();

	if(!nma)
	{

	    if(timer)
	    {
		TimerDetReport(1,nma,"Temps calcul convexification");
	    }

	    return 0;
	}

	Tabunsignedbuf nu;
	Tabcharbuf ltf;
	TabREELbuf xnew,ynew,znew;
#if P4TYPE_DONNEE == 0
	TabREELbuf tnew;
#endif
	TabREELbuf anew,bnew,cnew;
	Tabunsignedbuf seg_debut,seg_fin,seg_origine,seg_contour;
	unsigned nma_new = 0;
	unsigned taille_prec = 0;
	Tabunsignedbuf* sommets = NULL;
	Tabunsignedbuf* contours = NULL;
	unsigned a_tai = 0;

    // ---------------------------------------------------------------
    //   on alloue de grands buffers pour eviter les reallocations
    //   trop frequentes
    // ---------------------------------------------------------------

	Tabcharbuf trace_fil(unsigned(5*nma));trace_fil.Enleve();
	Tabunsignedbuf nombre_ord(nma);nombre_ord.Enleve();
	Tabunsignedbuf mailles_new_1(unsigned(1.2*nma));mailles_new_1.Enleve();
	Tabunsignedbuf mailles_new_2(unsigned(5*nma));mailles_new_2.Enleve();
	TabREEL_Sbuf temp_x(unsigned(0.2*nma));temp_x.Enleve();
	TabREEL_Sbuf temp_y(unsigned(0.2*nma));temp_y.Enleve();
	TabREEL_Sbuf temp_z(unsigned(0.2*nma));temp_z.Enleve();
#if P4TYPE_DONNEE == 0
	TabREEL_Sbuf temp_c(unsigned(0.2*nma));temp_c.Enleve();
#else
	TabREEL_Sbuf cmailles(unsigned(1.2*nma));cmailles.Enleve();
#endif

    // *************************************************
    // -------------------------------------------------
    //        boucle generale sur les mailles
    // -------------------------------------------------
    // *************************************************

	for(unsigned num_maille=0;num_maille<nma;num_maille++)
	{

	    if(num_maille%500 == 0) RootWin::ExpIHM();

	    if(_ecriture == FALSE)
	    {
		delete[] sommets;
		delete[] contours;

		if(timer)
		{
		    TimerDetReport(1,nma,"Temps calcul convexification");
		}

		return -1;
	    }

    // -------------------------------------------------
    //        on convexifie chaque maille
    // -------------------------------------------------

	    ptma->DonneSommets(nu,num_maille);
	    ptma->LitTraceFil(ltf,num_maille);

	    const unsigned nnu = nu;

	    if(nnu <= 3)
	    {
		trace_fil.Ajoute(ltf);
		nombre_ord.Ajoute(1u);
		nma_new++;
		taille_prec += nnu;
		mailles_new_1.Ajoute(taille_prec);
#if P4TYPE_DONNEE == 1
		cmailles.Ajoute(_c(num_maille));
#endif
		mailles_new_2.Ajoute(nu);

		continue;
	    }

	    xnew.Enleve();
	    ynew.Enleve();
	    znew.Enleve();
#if P4TYPE_DONNEE == 0
	    tnew.Enleve();
#endif
	    seg_debut.Enleve();
	    seg_fin.Enleve();
	    seg_origine.Enleve();
	    seg_contour.Enleve();

	    unsigned j;

	    for (j=0;j<nnu;j++)
	    {
		const unsigned l = nu(j);

		xnew.Ajoute(REEL(_x(l)));
		ynew.Ajoute(REEL(_y(l)));
		znew.Ajoute(REEL(_z(l)));
#if P4TYPE_DONNEE == 0
		tnew.Ajoute(REEL(_c(l)));
#endif
		seg_debut.Ajoute(j);
		seg_origine.Ajoute(j);
		seg_contour.Ajoute(unsigned(ltf(j)));

		const unsigned k = (j+1)%nnu;

		seg_fin.Ajoute(k);
	    }

	// -------------------------------------------------------------------
	//  calculs des parametres de segments (equation de la droite support)
	// -------------------------------------------------------------------

	    GeomUtil::SegmentsEquatFromPoints(anew,bnew,cnew,xnew,ynew,
			    seg_debut,seg_fin);

	// ------------------------------------------
	//    calcul points et segments a rajouter
	// ------------------------------------------

	    unsigned seg_orig_etat = nnu;
	    const unsigned nnu2 = nnu-2;
	    unsigned i1;

	    for(i1=0;i1<nnu2;i1++)
	    {
		const unsigned j1t=(i1+1)%nnu;
		const unsigned nnu3 = i1 == 0 ? nnu-1 : nnu;
		const REEL xnewi1 = xnew(i1);
		const REEL xnewj1 = xnew(j1t);
		const REEL ynewi1 = ynew(i1);
		const REEL ynewj1 = ynew(j1t);
		const REEL anewi1 = anew(i1);
		const REEL bnewi1 = bnew(i1);
		const REEL cnewi1 = cnew(i1);
		const REEL distk1 = abs(xnewj1-xnewi1) + abs(ynewj1-ynewi1);

		for(unsigned i2 = i1+2;i2<nnu3;i2++)
		{
		    const unsigned j2 = (i2+1)%nnu;
		    const REEL xnewi2 = xnew(i2);
		    const REEL xnewj2 = xnew(j2);
		    const REEL ynewi2 = ynew(i2);
		    const REEL ynewj2 = ynew(j2);
		    register const int elim =
			(xnewj1 == xnewi2 && ynewj1 == ynewi2) ||
			(xnewi1 == xnewi2 && ynewi1 == ynewi2) ||
			(xnewi1 == xnewj2 && ynewi1 == ynewj2) ||
			(xnewj1 == xnewj2 && ynewj1 == ynewj2);

		    if(elim) continue;

		    const REEL anewi2 = anew(i2);
		    const REEL bnewi2 = bnew(i2);
		    const REEL den = anewi2*bnewi1-anewi1*bnewi2;

		    if(den == 0.) continue;

		    const REEL cnewi2 = cnew(i2);
		    const REEL xm1 = (cnewi1*bnewi2-cnewi2*bnewi1)/den;
		    const REEL ym1 = (anewi1*cnewi2-anewi2*cnewi1)/den;
		    const REEL disti1 = abs(xnewi1-xm1) + abs(ynewi1-ym1);
		    const REEL distj1 = abs(xnewj1-xm1) + abs(ynewj1-ym1);
		    const REEL disti2 = abs(xnewi2-xm1) + abs(ynewi2-ym1);
		    const REEL distj2 = abs(xnewj2-xm1) + abs(ynewj2-ym1);
		    const REEL distk2 = abs(xnewj2-xnewi2) + abs(ynewj2-ynewi2);

		    if(disti1 <= distk1 && distj1 <= distk1 &&
				    disti2 <= distk2 && distj2 <= distk2)
		    {
		    // cas point interieur-interieur

			register const REEL al2 = distk1 != 0. ?
			    disti1/distk1 : 0.;
			register const REEL al1 = 1.-al2;
			register const REEL al4 = distk2 != 0. ?
			    disti2/distk2 : 0.;
			register const REEL al3 = 1.-al4;
			const REEL zm1 = al1*znew(i1)+al2*znew(j1t);
			const REEL zm2 = al3*znew(i2)+al4*znew(j2);
#if P4TYPE_DONNEE == 0
			const REEL cm1 = al1*tnew(i1)+al2*tnew(j1t);
			const REEL cm2 = al3*tnew(i2)+al4*tnew(j2);
#endif
			const unsigned ip = xnew;
			unsigned ips1 = ip;
			unsigned ips2 = ip;
			unsigned is;

			for(is=0;is<ip;is++)
			{

			    if(GeomUtil::CarreDiffVecteurs3D(xnew(is),
					    ynew(is),znew(is),xm1,ym1,zm1) <=
					    tolerance)
			    {
				ips1 = is;
				break;
			    }

			}

			if(is==ip)
			{
			    ips2 = ip + 1;
			    xnew.Ajoute(xm1);
			    ynew.Ajoute(ym1);
			    znew.Ajoute(zm1);
#if P4TYPE_DONNEE == 0
			    tnew.Ajoute(cm1);
#endif
			}

			for(is=0;is<ip;is++)
			{

			    if(GeomUtil::CarreDiffVecteurs3D(xnew(is),
					    ynew(is),znew(is),xm1,ym1,zm2) <=
					    tolerance)
			    {
				ips2 = is;
				break;
			    }

			}

			if(is==ip)
			{
			    xnew.Ajoute(xm1);
			    ynew.Ajoute(ym1);
			    znew.Ajoute(zm2);
#if P4TYPE_DONNEE == 0
			    tnew.Ajoute(cm2);
#endif
			}

			const unsigned ik = seg_origine;

			seg_debut.Ajoute(ips1);
			seg_fin.Ajoute(ips2);
			seg_origine.Ajoute(seg_orig_etat);
			seg_contour.Ajoute(TRACE_FIL_NON_U);

			seg_orig_etat++;

			for (unsigned k=0;k<ik;k++)
			{

			    if(seg_origine(k)==i1 && ips1!=seg_debut(k) &&
					    ips1!=seg_fin(k))
			    {
				seg_debut.Ajoute(seg_debut(k));
				seg_fin.Ajoute(ips1);
				seg_origine.Ajoute(i1);
				seg_contour.Ajoute(seg_contour(i1));
				seg_debut.Ajoute(ips1);
				seg_fin.Ajoute(seg_fin(k));
				seg_origine.Ajoute(i1);
				seg_contour.Ajoute(seg_contour(i1));
			    }
			    else if(seg_origine(k)==i2 && ips2!=seg_debut(k) &&
					    ips2!=seg_fin(k))
			    {
				seg_debut.Ajoute(seg_debut(k));
				seg_fin.Ajoute(ips2);
				seg_origine.Ajoute(i2);
				seg_contour.Ajoute(seg_contour(i2));
				seg_debut.Ajoute(ips2);
				seg_fin.Ajoute(seg_fin(k));
				seg_origine.Ajoute(i2);
				seg_contour.Ajoute(seg_contour(i2));
			    }

			}

		    }
		    else
		    {

		    // cas point exterieur-interieur
		    // criteres a ajouter : au + 2 intersections par seg
		    // avec la distance la + courte de chaque cote
		    // nouveau seg a l'interieur de la maille.
		    // Ceci est realise avec :
		    // L'elimination des segments englobants
		    // Apres elimination verification si le segment est exterieur
		    // a la maille.
		    // Il reste des bouts de segments
		    // (=> non optimum mais pas grave)
			int type = 0;

			if((distj1 > distk1 || disti1 >= distk1) &&
					distj2 <= distk2 && disti2 < distk2)
			{
			    type = 1;
			}
			else if((distj2 >= distk2 || disti2 > distk2) &&
					distj1 < distk1 && disti1 <= distk1)
			{
			    type = 2;
			}

			if(!type) continue;

			REEL zm1 = 0.;
#if P4TYPE_DONNEE == 0
			REEL cm1 = 0.;
#endif
			unsigned choixint = TRACE_FIL_OUI_U;
			unsigned notchoixint = TRACE_FIL_NON_U;

			if(type == 2)
			{
			    register const REEL al2 = distk1 != 0. ?
				disti1/distk1 : 0.;
			    register const REEL al1 = 1.-al2;

			    zm1 = al1*znew(i1)+al2*znew(j1t);
#if P4TYPE_DONNEE == 0
			    cm1 = al1*tnew(i1)+al2*tnew(j1t);
#endif
			    choixint = seg_contour(i1);
			    notchoixint = TRACE_FIL_NON_U;
			}
			else if(type == 1)
			{
			    register const REEL al2 = distk2 != 0. ?
				disti2/distk2 : 0.;
			    register const REEL al1 = 1.-al2;

			    zm1 = al1*znew(i2)+al2*znew(j2);
#if P4TYPE_DONNEE == 0
			    cm1 = al1*tnew(i2)+al2*tnew(j2);
#endif
			    choixint = TRACE_FIL_NON_U;
			    notchoixint = seg_contour(i2);
			}

			const unsigned ip = xnew;
			unsigned ips = ip;
			unsigned is;

			for(is=0;is<ip;is++)
			{

			    if(GeomUtil::CarreDiffVecteurs3D(xnew(is),
					    ynew(is),znew(is),xm1,ym1,zm1) <=
					    tolerance)
			    {
				ips = is;
				break;
			    }

			}

			if(is==ip)
			{
			    xnew.Ajoute(xm1);
			    ynew.Ajoute(ym1);
			    znew.Ajoute(zm1);
#if P4TYPE_DONNEE == 0
			    tnew.Ajoute(cm1);
#endif
			}

			const unsigned ik = seg_origine;

			for (unsigned k=0;k<ik;k++)
			{

			    if(ips==seg_debut(k) || ips==seg_fin(k))
				continue;

			    if(seg_origine(k)==i1)
			    {
				seg_debut.Ajoute(seg_debut(k));
				seg_fin.Ajoute(ips);
				seg_origine.Ajoute(i1);
				seg_contour.Ajoute(choixint);
				seg_debut.Ajoute(ips);
				seg_fin.Ajoute(seg_fin(k));
				seg_origine.Ajoute(i1);
				seg_contour.Ajoute(choixint);
			    }
			    else if(seg_origine(k)==i2)
			    {
				seg_debut.Ajoute(seg_debut(k));
				seg_fin.Ajoute(ips);
				seg_origine.Ajoute(i2);
				seg_contour.Ajoute(notchoixint);
				seg_debut.Ajoute(ips);
				seg_fin.Ajoute(seg_fin(k));
				seg_origine.Ajoute(i2);
				seg_contour.Ajoute(notchoixint);
			    }

			} // fin boucle k

		    } // fin test ext-int

		} // fin boucle i2

	    } // fin boucke i1

	    unsigned nnu_nouveau = nnu;

	// -------------------------------------------------------------
	//  Si il y a eu de nouveaux segments : on elimine les segments
	//  englobant de plus petits, les segments en double
	//  puis les segments externes.
	// -------------------------------------------------------------

	    const P4Boolean finelimine =
		nnu!=unsigned(seg_debut) ? TRUE : FALSE ;

	    if(finelimine)
	    {
		GeomUtil::ElimineSegments(xnew,ynew,seg_debut,seg_fin,
				seg_origine,seg_contour);
		GeomUtil::ElimineSegmentsExt(xnew,ynew,seg_debut,seg_fin,
				seg_origine,seg_contour,nnu);

	    // ---------------------------------------------------------------
	    //  s'il y avait de nouveaux segments
	    //  on cherche si il y a de nouvelles intersections internes =>
	    //  points + segments supplementaires (3 eme cas d'intersection =
	    //  intersection externes-externes a l'interieur de la maille <==>
	    //  interne-interne sur segment internes-externes generes
	    //  au-dessus)
	    // ---------------------------------------------------------------

		const unsigned tai = seg_debut;

		if(tai > nnu)
		{
		    GeomUtil::SegmentsEquatFromPoints(anew,bnew,cnew,xnew,ynew,
				    seg_debut,seg_fin);

		    register const unsigned tai1 = tai - 1;

		    for (unsigned i1seg = 0;i1seg < tai1;i1seg++)
		    {
			const unsigned i1t = seg_debut(i1seg);
			const unsigned j1t = seg_fin(i1seg);
			const unsigned k1 = seg_origine(i1seg);
			const REEL xnewi1 = xnew(i1t);
			const REEL xnewj1 = xnew(j1t);
			const REEL ynewi1 = ynew(i1t);
			const REEL ynewj1 = ynew(j1t);
			const REEL anewi1 = anew(i1seg);
			const REEL bnewi1 = bnew(i1seg);
			const REEL cnewi1 = cnew(i1seg);
			const REEL distk1 = abs(xnewj1-xnewi1) +
			    abs(ynewj1-ynewi1);

			for (unsigned i2seg = i1seg+1;i2seg < tai;i2seg++)
			{
			    const unsigned i2 = seg_debut(i2seg);
			    const unsigned j2 = seg_fin(i2seg);
			    const unsigned k2 = seg_origine(i2seg);
			    register const int elim = k1==k2 || i1t==i2 ||
				i1t==j2 || j1t==i2 || j1t==j2;

			    if(elim) continue;

			    const REEL xnewi2 = xnew(i2);
			    const REEL xnewj2 = xnew(j2);
			    const REEL ynewi2 = ynew(i2);
			    const REEL ynewj2 = ynew(j2);
			    register const int elim2 =
				((xnewi1 == xnewi2) && (ynewi1 == ynewi2)) ||
				((xnewi1 == xnewj2) && (ynewi1 == ynewj2)) ||
				((xnewj1 == xnewi2) && (ynewj1 == ynewi2)) ||
				((xnewj1 == xnewj2) && (ynewj1 == ynewj2));

			    if(elim2) continue;

			    const REEL anewi2 = anew(i2seg);
			    const REEL bnewi2 = bnew(i2seg);
			    const REEL den = anewi2*bnewi1-anewi1*bnewi2;

			    if(den == 0.) continue;

			    const REEL cnewi2 = cnew(i2seg);
			    const REEL xm1 = (cnewi1*bnewi2-cnewi2*bnewi1)/den;
			    const REEL ym1 = (anewi1*cnewi2-anewi2*cnewi1)/den;
			    const REEL disti1 = abs(xnewi1-xm1)+abs(ynewi1-ym1);
			    const REEL distj1 = abs(xnewj1-xm1)+abs(ynewj1-ym1);
			    const REEL disti2 = abs(xnewi2-xm1)+abs(ynewi2-ym1);
			    const REEL distj2 = abs(xnewj2-xm1)+abs(ynewj2-ym1);
			    const REEL distk2 = abs(xnewj2-xnewi2) +
				abs(ynewj2-ynewi2);

			    if(disti1 > distk1 || distj1 > distk1 ||
					    disti2 > distk2 || distj2 > distk2)
				continue;

			// cas point interieur-interieur

			    register const REEL al2 = distk1 != 0. ?
				disti1/distk1 : 0.;
			    register const REEL al1 = 1.-al2;
			    register const REEL al4 = distk2 != 0. ?
				disti2/distk2 : 0.;
			    register const REEL al3 = 1.-al4;
			    const REEL zm2 = al3*znew(i2)+al4*znew(j2);
			    const REEL zm1 = (al1*znew(i1t)+al2*znew(j1t)+
					    zm2)*.5;
#if P4TYPE_DONNEE == 0
			    const REEL cm2 = al3*tnew(i2)+al4*tnew(j2);
			    const REEL cm1 = (al1*tnew(i1t)+al2*tnew(j1t)+
					    cm2)*.5;
#endif

			    const unsigned ip = xnew;
			    unsigned ips = ip;
			    unsigned is;

			    for(is=0;is<ip;is++)
			    {

				if(GeomUtil::CarreDiffVecteurs3D(xnew(is),
						ynew(is),znew(is),xm1,ym1,
						zm1) <= tolerance)
				{
				    ips=is;
				    break;
				}

			    }

			    if(is==ip)
			    {
				xnew.Ajoute(xm1);
				ynew.Ajoute(ym1);
				znew.Ajoute(zm1);
#if P4TYPE_DONNEE == 0
				tnew.Ajoute(cm1);
#endif
			    }

			    const unsigned ik = seg_origine;

			    for (unsigned k=0;k<ik;k++)
			    {

				if(ips==seg_debut(k) || ips==seg_fin(k))
				    continue;

				if(seg_origine(k)==k1)
				{
				    seg_debut.Ajoute(seg_debut(k));
				    seg_fin.Ajoute(ips);
				    seg_origine.Ajoute(k1);
				    seg_contour.Ajoute(TRACE_FIL_NON_U);
				    seg_debut.Ajoute(ips);
				    seg_fin.Ajoute(seg_fin(k));
				    seg_origine.Ajoute(k1);
				    seg_contour.Ajoute(TRACE_FIL_NON_U);
				}
				else if(seg_origine(k)==k2)
				{
				    seg_debut.Ajoute(seg_debut(k));
				    seg_fin.Ajoute(ips);
				    seg_origine.Ajoute(k2);
				    seg_contour.Ajoute(TRACE_FIL_NON_U);
				    seg_debut.Ajoute(ips);
				    seg_fin.Ajoute(seg_fin(k));
				    seg_origine.Ajoute(k2);
				    seg_contour.Ajoute(TRACE_FIL_NON_U);
				}

			    } // fin boucle ik

			} // fin boucle i2seg

		    } // fin boucle i1seg

		} // fin tai > nnu

	    // --------------------------------------------------------------
	    //  ecriture des points supplementaires dans l'objet geo_convexe
	    // --------------------------------------------------------------

		nnu_nouveau = xnew;

		for (unsigned jj=nnu;jj<nnu_nouveau;jj++)
		{
		    temp_x.Ajoute(REEL_S(xnew(jj)));
		    temp_y.Ajoute(REEL_S(ynew(jj)));
		    temp_z.Ajoute(REEL_S(znew(jj)));
#if P4TYPE_DONNEE == 0
		    temp_c.Ajoute(REEL_S(tnew(jj)));
#endif
		}

	// --------------------------------------------------------------
	//     elimination des segments supplementaires englobants
	// --------------------------------------------------------------

		const unsigned tai2 = seg_debut;
		const P4Boolean finelimin2 = (tai2 > tai) ? TRUE : FALSE;

		if(finelimin2)
		{
		    GeomUtil::ElimineSegments(xnew,ynew,seg_debut,seg_fin,
				    seg_origine,seg_contour);
		}

	    } // fin finelimine

	// -----------------------------------------------------------------
	//      fin calcul segments et points
	// -----------------------------------------------------------------

	// -----------------------------------------------------------------
	//            calcul des mailles a partir des segments
	// -----------------------------------------------------------------

	    const unsigned tai = seg_debut;
	    const unsigned tai_reduit = tai == nnu ? 1 : 2*tai;

	    if(tai_reduit > a_tai)
	    {
		delete[] sommets;delete[] contours;
		a_tai = tai_reduit;
		sommets = new Tabunsignedbuf[tai_reduit];
		contours = new Tabunsignedbuf[tai_reduit];

		if(!sommets || !contours)
		{
		    cerr << "pb allocation dans GeomPure4::Convexe" << endl;
		    delete[] sommets;delete[] contours;
		    sommets = NULL;
		    contours = NULL;
		    a_tai = 0;

		    if(timer)
		    {
			TimerDetReport(1,nma,"Temps calcul convexification");
		    }

		    return -1;
		}

	    }

	    if(tai == nnu)
	    {

	    // cas ou la maille etait deja convexe

		sommets[0].Ajoute(seg_debut);
		contours[0].Ajoute(seg_contour);
	    }
	    else
	    {
		GeomUtil::SegmentsToMailles(xnew,ynew,seg_debut,seg_fin,
				seg_contour,sommets,contours);
	    }

	// on elimine les mailles en double

	    unsigned nombre_mailles = 0;

	    GeomUtil::MaillesElimineDoubles(sommets,contours,tai_reduit,
			    num_maille,nombre_mailles);
	    nma_new += nombre_mailles;

	// On garde la meme orientation de maille que pour la maille originelle

	    const unsigned nnu1 = nnu-1;
	    const int srot = (xnew(1)-xnew(0))*(ynew(nnu1)-ynew(0))-
		(ynew(1)-ynew(0))*(xnew(nnu1)-xnew(0)) > 0. ? 1 : 0;
	    unsigned i1seg;

	    for (i1seg = 0;i1seg < tai_reduit;i1seg++)
	    {
		const unsigned s_tai = sommets[i1seg];

		if(s_tai <= 2) continue;

		const unsigned s_tai1 = s_tai-1;
		unsigned* const pt_sommets = sommets[i1seg];
		const unsigned nu0 = pt_sommets[0];
		const unsigned nu1 = pt_sommets[1];
		const unsigned nu2 = pt_sommets[s_tai1];

		const int srotn = (xnew(nu1)-xnew(nu0))*(ynew(nu2)-ynew(nu0))-
		    (ynew(nu1)-ynew(nu0))*(xnew(nu2)-xnew(nu0)) > 0. ? 1 : 0;

		if(srotn != srot)
		{
		    unsigned* const pt_contours = contours[i1seg];

		    for(unsigned i2seg=0;i2seg<s_tai;i2seg++)
		    {
			const unsigned sommet = pt_sommets[i2seg];
			const unsigned contour = pt_contours[i2seg];
			const unsigned i2segm = s_tai-i2seg-1;

			pt_sommets[i2seg] = pt_sommets[i2segm];
			pt_sommets[i2segm] = sommet;
			pt_contours[i2seg] = pt_contours[i2segm];
			pt_contours[i2segm] = contour;
		    }

		}

	    }

	// on stocke les contours dans trace_fil

	    for (i1seg = 0;i1seg < tai_reduit;i1seg++)
	    {
		const unsigned s_tai = contours[i1seg];
		const unsigned* const pt_contours = contours[i1seg];

		for(unsigned i2seg=0;i2seg<s_tai;i2seg++)
		{
		    trace_fil.Ajoute((unsigned char) pt_contours[i2seg]);
		}

	    }

	// utile pour ord

	    nombre_ord.Ajoute(nombre_mailles);

	// on stocke les mailles dans mailles_new_1 pour les nbre de sommets
	// de la maille et mailles_new_2 pour les numeros de sommets de la 
	// maille

	    for (i1seg = 0;i1seg < tai_reduit;i1seg++)
	    {
		const unsigned s_tai = sommets[i1seg];

		if(s_tai > 0)
		{
		    taille_prec += s_tai;
		    mailles_new_1.Ajoute(taille_prec);
#if P4TYPE_DONNEE == 1
		    cmailles.Ajoute(_c(num_maille));
#endif
		}

	    }

	    const unsigned nombre_points_init = geo_convexe.NombrePoints() +
			unsigned(temp_x) - nnu_nouveau;

	    for (i1seg = 0;i1seg < tai_reduit;i1seg++)
	    {
		const unsigned s_tai = sommets[i1seg];

		for(unsigned i2seg=0;i2seg<s_tai;i2seg++)
		{
		    register const unsigned st = sommets[i1seg](i2seg);

		    if(st < nnu)
		    {
			mailles_new_2.Ajoute(nu(st));
		    }
		    else
		    {
			mailles_new_2.Ajoute(nombre_points_init+st);
		    }

		}

	    }

	// on nettoie les tableaux temporaires

	    for (i1seg = 0;i1seg < a_tai;i1seg++)
	    {
		sommets[i1seg].Enleve();
		contours[i1seg].Enleve();
	    }

	} // fin boucle sur les mailles

    // **************************************************************
    //     nettoyage
    // **************************************************************

	delete[] sommets;
	delete[] contours;

	nu.Vide();
	ltf.Vide();
	xnew.Vide();ynew.Vide();znew.Vide();
#if P4TYPE_DONNEE == 0
	tnew.Vide();
#endif
	anew.Vide();bnew.Vide();cnew.Vide();
	seg_debut.Vide();seg_fin.Vide();seg_origine.Vide();seg_contour.Vide();

    // **************************************************************
    //     stockage des points supplementaires dans geo_convexe
    // **************************************************************

#if P4TYPE_DONNEE == 0
	geo_convexe.AjoutePoints(temp_x,temp_y,temp_z,temp_c);
	temp_x.Vide();
	temp_y.Vide();
	temp_z.Vide();
	temp_c.Vide();
#else
	geo_convexe.GeomPure::AjoutePoints(temp_x,temp_y,temp_z);
	temp_x.Vide();
	temp_y.Vide();
	temp_z.Vide();
#endif

    // **************************************************************
    //     stockage de contours trace_fil dans geoconvexe
    // **************************************************************

	geo_convexe.ModTraceFil(trace_fil);
	trace_fil.Vide();

    // **************************************************************
    //     stockage de mailles dans geo_convexe
    // **************************************************************

	mailles_new_1.Ajoute(mailles_new_2);
	mailles_new_2.Vide();
	geo_convexe.ModMailles(nma_new,mailles_new_1);
	mailles_new_1.Vide();

    // **************************************************************
    //     stockage du 4eme tableau c dans geo_convexe
    // **************************************************************

#if P4TYPE_DONNEE == 1
	geo_convexe.EnleveC();
	geo_convexe.AjouteC(cmailles);
	cmailles.Vide();
#endif

    // **************************************************************
    //     calcul du nouveau tableau de volume si propag_vol
    //     utile pour les coupes avec regeneration de mailles
    //     par exemple
    // **************************************************************

	if(propag_vol && ptma->Nvolumes())
	{
	    Tabunsignedbuf vol_temp(nma);vol_temp.Enleve();
	    unsigned num_maille_new = 0;
	    register unsigned i=0;

	    for(;i<nma;i++)
	    {
		vol_temp.Ajoute(num_maille_new);
		num_maille_new += nombre_ord(i);
	    }

	    vol_temp.Ajoute(num_maille_new);

	    register const unsigned nvol = ptma->Nvolumes();
	    unsigned nb_mailles = 0;
	    Tabunsignedbuf nv;
	    Tabunsignedbuf volume_1;
	    Tabunsignedbuf volume_2;

	    for(i=0;i<nvol;i++)
	    {
		ptma->DonneMailles(nv,i);

		register const unsigned nnv = nv;

		for(register unsigned j=0;j<nnv;j++)
		{
		    register const unsigned nvt = nv(j);

		    nb_mailles += vol_temp(nvt+1) - vol_temp(nvt);

		    for(unsigned k=vol_temp(nvt);k<vol_temp(nvt+1);k++)
		    {
			volume_2.Ajoute(k,nvol);
		    }

		}

		volume_1.Ajoute(nb_mailles,nvol);
	    }

	    volume_1.Ajoute(volume_2);
	    volume_2.Vide();
	    geo_convexe.ModVolumes(nvol,volume_1);
	}

    // **************************************************************
    //     calcul de ord nouveau
    // **************************************************************

	if(ord(0) < Dep-1)
	{
	    register unsigned nbre_ord_max = 0;
	    register unsigned i = 0;

	    for(;i<nma;i++)
	    {
		register const unsigned nbre_ord = nombre_ord(ord(i));

		if(nbre_ord > nbre_ord_max) nbre_ord_max = nbre_ord;

	    }

	    if(log(double(nma)) + log(double(nbre_ord_max)) >= 32)
	    {
		cerr << "Pb maillage trop gros fichier : " << __FILE__ <<
			", ligne : " << __LINE__ << endl;
		delete[] sommets;
		delete[] contours;

		if(timer)
		{
		    TimerDetReport(1,nma,"Temps calcul convexification");
		}

		return -1;
	    }

	    Tabunsignedbuf ord_temp(nma_new);ord_temp.Enleve();

	    for(i=0;i<nma;i++)
	    {
		const unsigned num_ord = ord(i);
		const unsigned nbre_ord = nombre_ord(num_ord);

		for(unsigned j=0;j<nbre_ord;j++)
		{
		    ord_temp.Ajoute(num_ord*nbre_ord_max+j);
		}

	    }

	    const unsigned tai_temp = ord_temp;
	    Tabunsigned ord_rang(tai_temp);

	    quicksort((const unsigned*) ord_temp,ord_rang,tai_temp);

	    for(unsigned l=0;l<tai_temp;l++)
	    {
		ord_temp.ModVal(l,ord_rang(l));
	    }

	    ord.ModTab(ord_temp);
	}
	else if(ord(0) == Dep-1)
	{

	// stockage anciens numeros de mailles dans ord
	// pour correspondance nouveaux numeros -> anciens numeros
	// utile pour calcul de coordonnes et de numeros de mailles
	// par picking

	    Tabunsignedbuf ord_temp(nma_new);ord_temp.Enleve();

	    for(unsigned i=0;i<nma;i++)
	    {
		ord_temp.Agrand(i,nombre_ord(i));
	    }

	    ord.ModTab(ord_temp);
	}

    // **************************************************************
    //     impressions temps
    // **************************************************************

	if(timer)
	{
	    TimerDetReport(1,nma_new,"Temps calcul convexification");
	}

	return 0;
}
#endif /* fin P4GEOMCONV */
