//
// Ident.
//

struct SccsIdP4Graf2
{
	static const char* const sccs_id;
};

const char* const SccsIdP4Graf2::sccs_id = "%W% %G% Pierre Brochard";

//
// Includes
//

#include "poulpe/IHM.h"
#define P4TABSOMGRI_IMPLEMENT
#include "minotaure/P4graf2.h"
#include "sanglierP.h"
#include "P4sommet.h"
#include "P4grille.h"
#include "P4geom4.h"
#include "P4tabgengr.h"
#include "P4int3.h"
#include "P4hauteurs.h"
#include "P4tronc.h"
#include "P4tabsommet.h"
#include "dinosaure/P4quicksort.h"

// ************************************************************************
// *  Rappel de bonne programmation en c++ :                              *
// *  Pour rajouter des fonctions membres de cette classe                 *
// *  utiliser l'heritage et ne pas modifier P4graf2.C !!                 *
// ************************************************************************

P4Graf2::P4Graf2() :
		P4Graf1(),
		_borne_min(*new Sommet4),
		_borne_max(*new Sommet4),
		_log(*new Int4),
		_tabgrillesimple(*new TabGrilleSimple),
		_tabgrilleauto(*new TabGrilleAuto),
		_fil(OPT_4D),
		_fac(OPT_4D),
		_niv(OPT_4D),
		_lis(OPT_4D),
		_type_trace_fil(TRACE_EN_XY),
		_td4d(DONNEE_NOEUD),
		_cherche_mailles(TRUE),
		_marqueur(5),
		pt_geo_temp(NULL)
{
}

P4Graf2::~P4Graf2()
{
	P4AnnuleGrilles();

	TabGrilleSimple* ptgstmp = &_tabgrillesimple;
	TabGrilleAuto* ptgatmp = &_tabgrilleauto;
	delete ptgstmp;
	delete ptgatmp;

	Sommet4* s4min = &_borne_min;
	Sommet4* s4max = &_borne_max;
	delete s4min;
	delete s4max;

	Int4* int4log = &_log;
	delete int4log;
}

ostream&
operator << (ostream& ost,const P4Graf2& p23)
{
	ost << " Impression des membres statiques de P4Graf2\n";
	ost << (const P4Graf1&) p23;
	ost << "  tabgrillesimple :\n" << p23._tabgrillesimple;
	ost << "  tabgrilleauto :\n" << p23._tabgrilleauto;
	ost << "  fil :\n" << p23._fil << endl;
	ost << "  fac :\n" << p23._fac << endl;
	ost << "  niv :\n" << p23._niv << endl;
	ost << "  lis :\n" << p23._lis << endl;
	ost << "  cherche mailles :\n" << p23._cherche_mailles << endl;
	ost << endl;

	return ost;
}

void
P4Graf2::P4DefEchelles(const REEL xmin,const REEL ymin,const REEL zmin,
		const REEL cmin,const REEL xmax,const REEL ymax,const REEL zmax,
		const REEL cmax,const int logx,const int logy,const int logz,
		const int logc)
{
	_borne_min.X(xmin);
	_borne_min.Y(ymin);
	_borne_min.Z(zmin);
	_borne_min.C(cmin);
	_borne_max.X(xmax);
	_borne_max.Y(ymax);
	_borne_max.Z(zmax);
	_borne_max.C(cmax);
	_log.I(logx);
	_log.J(logy);
	_log.K(logz);
	_log.L(logc);
}

void
P4Graf2::P4DefCminmax(const REEL cmin,const REEL cmax)
{
	_borne_min.C(cmin);
	_borne_max.C(cmax);
}

void
P4Graf2::P4LitEchelles(REEL& xmin,REEL& ymin,REEL& zmin,
		REEL& cmin,REEL& xmax,REEL& ymax,REEL& zmax,
		REEL& cmax,int& logx,int& logy,int& logz,int& logc)
{
	xmin=_borne_min.X();
	ymin=_borne_min.Y();
	zmin=_borne_min.Z();
	cmin=_borne_min.C();
	xmax=_borne_max.X();
	ymax=_borne_max.Y();
	zmax=_borne_max.Z();
	cmax=_borne_max.C();
	logx=_log.I();
	logy=_log.J();
	logz=_log.K();
	logc=_log.L();
}

int
P4Graf2::P4LigneBrisee(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int npoints,P4Boolean trace_lent,
		int nombre_cesures)
{
    // *------------------------------------------------------------*
    // *  trace de ligne 3d coloree                                 *
    // *  si *min=*max=0 => calcul automatique des bornes           *
    // *  cmin = cmax : couleur _icoh                               *
    // *  sinon on se sert de _hauteurs                             *
    // *  en particulier si cmin=cmax != 0 : toute la ligne est     *
    // *  de la couleur _icoh   (couleur dessus surface)            *
    // *------------------------------------------------------------*

	if(_time) TimerInit();

	GeomPure4Dim2 geo(x,y,z,c,npoints,npoints,npoints,npoints,1,
			npoints,0u,NULL,0);

	int ier = P4Bornes(geo);

	P4TraceGrilles();
	ier = P4LigneBrisee(geo,trace_lent,nombre_cesures);

	if(_time)
	{
	    cout << "Temps calcul P4LigneBrisee 4d" << endl;
	    TimerReport(1,npoints);
	}

	return ier;
}

int
P4Graf2::P4LigneBrisee(const GeomPure4& geo,P4Boolean trace_lent,
		int nombre_cesures)
{  
	int ier = -1;

	const int npoints = geo.Ny();

	const REEL_S* const x = geo.X();
	const REEL_S* const y = geo.Y();
	const REEL_S* const z = geo.Z();
	const REEL_S* const c = geo.C();

	nombre_cesures = max(1,nombre_cesures);
	ptr_hauteurs = _fil==OPT_4D ? &_hauteurs_c : &_hauteurs_z;

	const int nys10 = npoints / nombre_cesures + 1;

	for(int i=0;i<nombre_cesures;i++)
	{
	    const int ifirst = max(i*nys10,1) - 1;
	    const int ilast = min((i+1)*nys10,npoints) - 1;

	    for(register int j=ifirst;j<ilast;j++)
	    {
		int ierv = -1;

		if(j%500 == 0) RootWin::ExpIHM();

		if(!_ecriture) return ierv;

		register const int jp = j+1;

		if(_fil == OPT_MONO || ptr_hauteurs->P4NombreHauteurs() == 0)
		{
		    ierv = P4VecteurColor(x[j],y[j],z[j],x[jp],y[jp],z[jp],
				    _icoh);
		}
		else if(_fil == OPT_4D)
		{
		    ierv = P4VecteurColor(x[j],y[j],z[j],c[j],x[jp],y[jp],
				    z[jp],c[jp]);
		}
		else
		{
		    ierv = P4VecteurColor(x[j],y[j],z[j],z[j],x[jp],y[jp],
				    z[jp],z[jp]);
		}

		if(ierv==-1) ier = -1;

	    }

	    L3FlushBuffers();

	    if(trace_lent) sleep(1);

	}

	return ier;
}

int
P4Graf2::P4LigneBrisee(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		int npoints,P4Boolean trace_lent,int nombre_cesures)
{
    // *------------------------------------------------------------*
    // *  ligne 3d monocouleur (couleur courante)                   *
    // *  si *min=*max=0 => calcul automatique des bornes           *
    // *  ligne mono couleur dans la couleur en cours (P4Dcolor)    *
    // *  pour l'instant, comme L3LigneBrisee donne des items L3    *
    // *  de type segment, cette fonction membre est un cas         *
    // *  particulier de la precedente (avec cmin=cmax!=0)          *
    // *------------------------------------------------------------*

	if(_time) TimerInit();
    
	int ier = -1;
	REEL xmin = _borne_min.X();
	REEL ymin = _borne_min.Y();
	REEL zmin = _borne_min.Z();
	REEL xmax = _borne_max.X();
	REEL ymax = _borne_max.Y();
	REEL zmax = _borne_max.Z();
	const int logx = _log.I();
	const int logy = _log.J();
	const int logz = _log.K();

	nombre_cesures=max(1,nombre_cesures);

	if(xmin == 0. && xmax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,x,xmin,xmax);
	}

	if(ymin == 0. && ymax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,y,ymin,ymax);
	}

	if(zmin == 0. && zmax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,z,zmin,zmax);
	}

	_borne_min.X(xmin);
	_borne_min.Y(ymin);
	_borne_min.Z(zmin);
	_borne_max.X(xmax);
	_borne_max.Y(ymax);
	_borne_max.Z(zmax);

	P4DefSpace(xmin,ymin,zmin,xmax,ymax,zmax,logx,logy,logz ) ;
	P4TraceGrilles();

	const int nys10 = npoints / nombre_cesures + 1;
	const int tnys10 = 2*nys10+2;

	REEL* xx = NULL;REEL* yy = NULL;REEL* zz = NULL;

	if(!(xx=new REEL[tnys10]) || !(yy=new REEL[tnys10]) || 
			!(zz=new REEL[tnys10]))
	{
	    cerr << " Probleme memoire dans P4LigneBrisee " << endl;
	    return -1;
	}

	for(int i=0;i<nombre_cesures;i++)
	{
	    const int ifirst = max(i*nys10,1)-1;
	    const int ilast = min((i+1)*nys10,npoints)-1;
	    int k = -1;
	    P4Boolean horsa=FALSE;

	    for(int j=ifirst;j<ilast;j++)
	    {

		if(j%500 == 0) RootWin::ExpIHM();

		if(!_ecriture)
		{
		    delete[] xx;delete[] yy;delete[] zz;
		    return -1;
		}

		const int jp=j+1;
		REEL xd=x[j];
		REEL yd=y[j];
		REEL zd=z[j];
		REEL xf=x[jp];
		REEL yf=y[jp];
		REEL zf=z[jp];

		int ierv=P4CorrigeVecteur(xd,yd,zd,xf,yf,zf);

		if(ierv==0)
		{
		    horsa=TRUE;
		}
		else
		{

		    if(ierv==1 || horsa || k==-1)
		    {
			k++;
			xx[k]=xd;yy[k]=yd;zz[k]=zd;
		    }

		    k++;
		    xx[k]=xf;yy[k]=yf;zz[k]=zf;
		    horsa=FALSE;
		}

	    }

	    k++;

	    if(k>tnys10)
	    {
		cerr << " Probleme memoire dans P4LigneBrisee " << endl;
		delete[] xx;delete[] yy;delete[] zz;
		return -1;
	    }

	    switch (_modetrace)
	    {
		case L32DROT :

		    {
			Sommet xyz;
			Sommet uvda;
			Bool3  lxyz(FALSE,FALSE,FALSE);

			for(int l=0;l<k;l++)
			{
			    xyz(xx[l],yy[l],zz[l]);
			    uvda = _projh.Projete(lxyz,_orig,xyz);
			    xx[l] = uvda.X();
			    yy[l] = uvda.Y();
			}

			L3PointCourant(0,xx[0],yy[0]);
			L3LigneBrisee(0,xx,yy,k);
	    		L3FlushBuffers();
		    }

		    break;

		case L33DROT :

		    {
			Sommet xyz;
			Sommet uvda;
			Bool3  lxyz(FALSE,FALSE,FALSE);

			for(int l=0;l<k;l++)
			{
			    xyz(xx[l],yy[l],zz[l]);
			    uvda = _projh.Projete(lxyz,_orig,xyz);
			    xx[l] = uvda.X();
			    yy[l] = uvda.Y();
			    zz[l] = uvda.Z();
			}

			// L3PointCourantP4(0,xx[0],yy[0],zz[0]); (a faire)
			// L3LigneBriseeP4(0,xx,yy,zz,k); (a faire)
	    		L3FlushBuffers();
		    }

		    break;

		case L33DSROT :

		    break;

	    }

	    if(trace_lent) sleep(1);

	}

	delete[] xx;delete[] yy;delete[] zz;

	if(_time) TimerReport(1,npoints);

	return ier;
}

int
P4Graf2::P4Polygone(REEL_S x[],REEL_S y[],REEL_S z[],int npoints)
{
    // *------------------------------------------------------------*
    // *  si *min=*max=0 => calcul automatique des bornes           *
    // *  Polygone dans la couleur en cours (P4Dcolor)              *
    // *------------------------------------------------------------*

	if(_time) TimerInit();
    
	int ier = -1;
	REEL xmin=_borne_min.X();
	REEL ymin=_borne_min.Y();
	REEL zmin=_borne_min.Z();
	REEL xmax=_borne_max.X();
	REEL ymax=_borne_max.Y();
	REEL zmax=_borne_max.Z();
	const int logx=_log.I();
	const int logy=_log.J();
	const int logz=_log.K();

	if(xmin == 0. && xmax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,x,xmin,xmax);
	}

	if(ymin == 0. && ymax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,y,ymin,ymax);
	}

	if(zmin == 0. && zmax == 0.)
	{
	    ier=P4MinMax(npoints,1,npoints,z,zmin,zmax);
	}

	_borne_min.X(xmin);
	_borne_min.Y(ymin);
	_borne_min.Z(zmin);
	_borne_max.X(xmax);
	_borne_max.Y(ymax);
	_borne_max.Z(zmax);

	P4DefSpace(xmin,ymin,zmin,xmax,ymax,zmax,logx,logy,logz ) ;
	P4TraceGrilles();

	const int tnys10 = 2*npoints+4;
	REEL* xx = new REEL[tnys10];
	REEL* yy = new REEL[tnys10];
	REEL* zz = new REEL[tnys10];

	if(!xx || !yy || !zz)
	{
	    delete[] xx; delete[] yy; delete[] zz;
	    cerr << "Probleme memoire dans P4Polygone" << endl;
	    return -1;
	}

	int k = -1;
	P4Boolean horsa = FALSE;

	for(int j=0;j<npoints;j++)
	{
	    const int jp=(j+1)%npoints;
	    REEL xd=x[j];
	    REEL yd=y[j];
	    REEL zd=z[j];
	    REEL xf=x[jp];
	    REEL yf=y[jp];
	    REEL zf=z[jp];

	    const int ierv=P4CorrigeVecteur(xd,yd,zd,xf,yf,zf);

	    if(ierv==0) horsa=TRUE;
	    else
	    {

		if(ierv==1 || horsa || k==-1)
		{
		    k++;
		    xx[k]=xd;yy[k]=yd;zz[k]=zd;
		}

		k++;
		xx[k]=xf;yy[k]=yf;zz[k]=zf;
		horsa=FALSE;
	    }

	}

	k++;

	if(k>tnys10)
	{
	    cerr << " Probleme memoire dans P4Polygone " << endl;
	    delete[] xx;delete[] yy;delete[] zz;
	    return -1;
	}

	switch (_modetrace)
	{
	    case L32DROT :

		{
		    Sommet xyz;
		    Sommet uvda;
		    Bool3  lxyz(FALSE,FALSE,FALSE);

		    for(int l=0;l<k;l++)
		    {
			xyz(xx[l],yy[l],zz[l]);
			uvda = _projh.Projete(lxyz,_orig,xyz);
			xx[l] = uvda.X();
			yy[l] = uvda.Y();
		    }

		    L3Polygone(0,xx,yy,k);
	    	    L3FlushBuffers();
		}

		break;

	    case L33DROT : 

		{
		    Sommet xyz;
		    Sommet uvda;
		    Bool3  lxyz(FALSE,FALSE,FALSE);

		    for(int l=0;l<k;l++)
		    {
			xyz(xx[l],yy[l],zz[l]);
			uvda = _projh.Projete(lxyz,_orig,xyz);
			xx[l] = uvda.X();
			yy[l] = uvda.Y();
			zz[l] = uvda.Z();
		    }

		    // L3PointCourantP4(0,xx[0],yy[0],zz[0]); (a faire)
		    // L3PolygoneP4(0,xx,yy,zz,k); (a faire)
	    	    L3FlushBuffers();
		}

		break;

	    case L33DSROT :

		break;
	}

	delete[] xx;delete[] yy;delete[] zz;

	if(_time) TimerReport(1,npoints);

	return ier;
}

void
P4Graf2::P4AnnuleGrilles()
{
	unsigned tai = _tabgrillesimple;
	unsigned i;

	for(i=0;i<tai;i++)
	{
		GrilleSimple& gs = _tabgrillesimple[i];

#if !defined(__GNUC__)
		gs.GrilleSimple::~GrilleSimple();
#else
		gs.~GrilleSimple();
#endif
	}

	tai = _tabgrilleauto;

	for(i=0;i<tai;i++)
	{
		GrilleAuto& ga = _tabgrilleauto[i];

#if !defined(__GNUC__)
		ga.GrilleAuto::~GrilleAuto();
#else
		ga.~GrilleAuto();
#endif
	}

	_tabgrillesimple.Enleve();
	_tabgrilleauto.Enleve();
}

void
P4Graf2::P4AnnuleGrillesSimple()
{
	unsigned tai = _tabgrillesimple;

	for(unsigned i=0;i<tai;i++)
	{
		GrilleSimple& gs = _tabgrillesimple[i];

#if !defined(__GNUC__)
		gs.GrilleSimple::~GrilleSimple();
#else
		gs.~GrilleSimple();
#endif
	}

	_tabgrillesimple.Enleve();
}

void
P4Graf2::P4AnnuleGrillesAuto()
{
	unsigned tai = _tabgrilleauto;

	for(unsigned i=0;i<tai;i++)
	{
		GrilleAuto& ga = _tabgrilleauto[i];

#if !defined(__GNUC__)
		ga.GrilleAuto::~GrilleAuto();
#else
		ga.~GrilleAuto();
#endif
	}

	_tabgrilleauto.Enleve();
}

void
P4Graf2::P4TraceGrilles()
{
	unsigned tai = _tabgrillesimple;
	unsigned i;

	for(i=0;i<tai;i++)
	{
	    Face face = _tabgrillesimple(i).Fac();
	    REEL decal = _tabgrillesimple(i).Decal();

	    switch (face)
	    {
		case B_DESSUS :
		    P4Dcoin(0.,0.,decal);
		    _tabgrillesimple[i].Trace(_boite_proj[5],_boite_proj[6],
		    _boite_proj[7],_boite_proj[4],_omin.X(),_omax.X(),
		    _omin.Y(),_omax.Y(),_lxyz.Bx(),_lxyz.By());
		    break;

		case B_DESSOUS :
		    P4Dcoin(0.,0.,decal);
		    _tabgrillesimple[i].Trace(_boite_proj[0],_boite_proj[1],
		    _boite_proj[2],_boite_proj[3],_omin.X(),_omax.X(),
		    _omin.Y(),_omax.Y(),_lxyz.Bx(),_lxyz.By());
		    break;

		case B_DEVANT :
		    P4Dcoin(0.,decal,0.);
		    _tabgrillesimple[i].Trace(_boite_proj[0],_boite_proj[1],
		    _boite_proj[6],_boite_proj[5],_omin.X(),_omax.X(),
		    _omin.Z(),_omax.Z(),_lxyz.Bx(),_lxyz.Bz());
		    break;

		case B_DERRIERE :
		    P4Dcoin(0.,decal,0.);
		    _tabgrillesimple[i].Trace(_boite_proj[2],_boite_proj[3],
		    _boite_proj[4],_boite_proj[7],_omin.X(),_omax.X(),
		    _omin.Z(),_omax.Z(),_lxyz.Bx(),_lxyz.Bz());
		    break;

		case B_GAUCHE :
		    P4Dcoin(decal,0.,0.);
		    _tabgrillesimple[i].Trace(_boite_proj[3],_boite_proj[0],
		    _boite_proj[5],_boite_proj[4],_omin.Y(),_omax.Y(),
		    _omin.Z(),_omax.Z(),_lxyz.By(),_lxyz.Bz());
		    break;

		case B_DROITE :
		    P4Dcoin(decal,0.,0.);
		    _tabgrillesimple[i].Trace(_boite_proj[1],_boite_proj[2],
		    _boite_proj[7],_boite_proj[6],_omin.Y(),_omax.Y(),
		    _omin.Z(),_omax.Z(),_lxyz.By(),_lxyz.Bz());
		    break;
	    }

	}

	tai = _tabgrilleauto;

	for(i=0;i<tai;i++)
	{
	    Sommet decal=_tabgrilleauto(i).Decal();

	    P4Dcoin(decal.X(),decal.Y(),decal.Z());
	    _tabgrilleauto[i].Trace(_boite_proj,_boite,_lxyz);
	}

}

void
P4Graf2::P4NouvelleGrilleSimple()
{
	GrilleSimple gs;
	_tabgrillesimple.AjouteClean(gs);
}

void
P4Graf2::P4NouvelleGrilleAuto()
{
	GrilleAuto go;
	_tabgrilleauto.AjouteClean(go);
}

void
P4Graf2::P4DefTitreSimple(const char* titre_axe_bas,const char* titre_axe_haut,
		const char* titre_axe_droite,const char* titre_axe_gauche,
		const char* unite_axe_bas,const char* unite_axe_haut,
		const char* unite_axe_droite,const char* unite_axe_gauche)
{
	GrilleSimple gs;
	unsigned tai=_tabgrillesimple;

	if(tai != 0) gs=_tabgrillesimple[tai-1];

	gs.DefTitre(_icolor,
		titre_axe_bas,titre_axe_haut,titre_axe_droite,titre_axe_gauche,
		unite_axe_bas,unite_axe_haut,unite_axe_droite,unite_axe_gauche);

	if(tai != 0)
	    _tabgrillesimple.ModVal(gs,tai-1);
	else
	    _tabgrillesimple.AjouteClean(gs);

}

void
P4Graf2::P4DefCaracGrille(int taille_caract_titres,
		int taille_caract_echelles)
{
	_karv_titre = taille_caract_titres;
	_karv_echelle = taille_caract_echelles;
}

void
P4Graf2::P4DefGrilleSimple(Face face,REEL decalx,P4Boolean texte_horizontal,
		int grad_axex,int grad_axey,
		int* type_axe,P4Boolean* label_axe,
		int* grad_inter_axe, int* factorisation_axe)
{
	GrilleSimple gs;
	unsigned tai=_tabgrillesimple;

	if(tai != 0) gs=_tabgrillesimple[tai-1];

	gs.DefGrille(face,decalx,texte_horizontal,grad_axex,grad_axey,
		_icolor,_karv_titre,_karv_echelle,
		type_axe,label_axe,
		grad_inter_axe,factorisation_axe);

	if(tai != 0)
	    _tabgrillesimple.ModVal(gs,tai-1);
	else
	    _tabgrillesimple.AjouteClean(gs);

}

void
P4Graf2::P4DefTitreAuto(const char* titre_x,const char* titre_y,
		const char* titre_z,const char* unite_x,const char* unite_y,
		const char* unite_z)
{
	GrilleAuto go;
	unsigned tai=_tabgrilleauto;

	if(tai != 0) go=_tabgrilleauto[tai-1];

	go.DefTitre(_icolor,
		titre_x,titre_y,titre_z,
		unite_x,unite_y,unite_z);

	if(tai != 0)
	    _tabgrilleauto.ModVal(go,tai-1);
	else
	    _tabgrilleauto.AjouteClean(go);

}

void
P4Graf2::P4DefGrilleAuto(REEL decalx,REEL decaly,REEL decalz,
		P4Boolean texte_horizontal,int grad_axex,int grad_axey,
		int grad_axez,int type_grilles,
		int* grad_inter)
{
	GrilleAuto go;
	unsigned tai=_tabgrilleauto;

	if(tai != 0) go=_tabgrilleauto[tai-1];

	go.DefGrille(decalx, decaly, decalz,
		texte_horizontal,grad_axex,grad_axey,grad_axez,_icolor,
		_karv_titre,_karv_echelle,type_grilles,
		grad_inter);

	if(tai != 0)
	    _tabgrilleauto.ModVal(go,tai-1);
	else
	    _tabgrilleauto.AjouteClean(go);

}

int
P4Graf2::P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
		int nx,int ny)
{
    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d sans masque et en maillage    *
    // *  structure                                                 *
    // *  si *min=*max=0 => calcul automatique des bornes           *
    // *  au-dessus de cmax et en dessous de cmin : couleur _icolor *
    // *  sinon on se sert de _hauteurs                             *
    // *  en particulier si cmin=cmax != 0 : toute la ligne est     *
    // *  de la couleur _icoh (couleur dessus surface)              *
    // *------------------------------------------------------------*

	if(_time) TimerInit();

	GeomPure4Dim2 geo(x,y,z,c,dimx,dimy,dimz,dimc,nx,ny,0u,NULL,0);

	int ier = P4Bornes(geo);

	P4TraceGrilles();
	ier = P4Filaire(geo);

	if(_time)
	{
	    cout << " Temps calcul P4Filaire sans masque structure" << endl;
	    TimerReport(1,nx*ny);
	}

	return ier;
}

int
P4Graf2::P4Filaire(const GeomPure4& geo)
{

    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d sans masque et en maillage    *
    // *  structure                                                 *
    // *------------------------------------------------------------*

	int ier = 0;

	const int nx = geo.Nx();
	const int ny = geo.Ny();
	const int dimx = geo.Dimx();
	const int dimy = geo.Dimy();

	const REEL_S* const x = geo.X();
	const REEL_S* const y = geo.Y();
	const REEL_S* const z = geo.Z();
	const REEL_S* const c = geo.C();

	switch (_modetrace)
	{
	    case L32DROT :

		if(_fil == OPT_MONO)
		{

		    if(_type_trace_fil == TRACE_EN_Y ||
			_type_trace_fil == TRACE_EN_XY)
		    {

			for(int i=0;i<ny;i++)
			{
			    RootWin::ExpIHM();

			    if(!_ecriture) return -1;

			    P4Dcolo(_icoh);

			    if(dimx == 0 && dimy == -1)
			    {
				ier = P4Graf1::P4LigneBrisee(x,y[i],&z[i*nx],nx);
			    }
			    else
			    {
				ier = P4Graf1::P4LigneBrisee(&x[i*nx],&y[i*nx],
						&z[i*nx],nx);
			    }

			}

		    }

		    if(_type_trace_fil == TRACE_EN_X ||
			_type_trace_fil == TRACE_EN_XY)
		    {

			for(int i=0;i<nx;i++)
			{
			    RootWin::ExpIHM();

			    if(!_ecriture) return -1;

			    P4Dcolo(_icoh);

			    if(dimx == 0 && dimy == -1)
			    {
				ier = P4Graf1::P4LigneBrisee(x[i],y,&z[i],ny,nx);
			    }
			    else
			    {
				ier = P4Graf1::P4LigneBrisee(&x[i],&y[i],&z[i],
						ny,nx);
			    }

			}

		    }

		}
		else
		{

		    if(_fil==OPT_4D)
		    {
			ptr_hauteurs=&_hauteurs_c;

			if(_type_trace_fil == TRACE_EN_Y ||
				_type_trace_fil == TRACE_EN_XY)
			{

			    for(int i=0;i<ny;i++)
			    {
				RootWin::ExpIHM();

				if(!_ecriture) return -1;

				if(dimx == 0 && dimy == -1)
				{
				    ier = P4Graf1::P4LigneBrisee(x,y[i],
					&z[i*nx],&c[i*nx],nx);
				}
				else
				{
				    ier = P4Graf1::P4LigneBrisee(&x[i*nx],
					&y[i*nx],&z[i*nx],&c[i*nx],nx);
				}

			    }

			}


			if(_type_trace_fil == TRACE_EN_X ||
				_type_trace_fil == TRACE_EN_XY)
			{

			    for(int i=0;i<nx;i++)
			    {
				RootWin::ExpIHM();

				if(!_ecriture) return -1;

				if(dimx == 0 && dimy == -1)
				{
				    ier = P4Graf1::P4LigneBrisee(x[i],y,
					&z[i],&c[i],ny,nx);
				}
				else
				{
				    ier = P4Graf1::P4LigneBrisee(&x[i],&y[i],
					&z[i],&c[i],ny,nx);
				}

			    }

			}

		    }
		    else
		    {
			ptr_hauteurs=&_hauteurs_z;

			if(_type_trace_fil == TRACE_EN_Y ||
				_type_trace_fil == TRACE_EN_XY)
			{

			    for(int i=0;i<ny;i++)
			    {
				RootWin::ExpIHM();

				if(!_ecriture) return -1;

				if(dimx == 0 && dimy == -1)
				{
				    ier = P4Graf1::P4LigneBrisee(x,y[i],
					&z[i*nx],&z[i*nx],nx);
				}
				else
				{
				    ier = P4Graf1::P4LigneBrisee(&x[i*nx],
					&y[i*nx],&z[i*nx],&z[i*nx],nx);
				}

			    }

			}

			if(_type_trace_fil == TRACE_EN_X ||
				_type_trace_fil == TRACE_EN_XY)
			{

			    for(int i=0;i<nx;i++)
			    {
				RootWin::ExpIHM();

				if(!_ecriture) return -1;

				if(dimx == 0 && dimy == -1)
				{
				    ier = P4Graf1::P4LigneBrisee(x[i],y,
					&z[i],&z[i],ny,nx);
				}
				else
				{
				    ier = P4Graf1::P4LigneBrisee(&x[i],&y[i],
					&z[i],&z[i],ny,nx);
				}

			    }

			}

		    }

		}

		break;

	    case L33DROT :

		break;

	    case L33DSROT :

		break;
	}

	return ier;
}

int
P4Graf2::P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
		int nx,int ny,unsigned mailles[],unsigned nmailles,
		int dimmailles,const Fil opt_fil)
{

    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d  cas general                  *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *------------------------------------------------------------*

	GeomPure4Dim2 geo(x,y,z,c,dimx,dimy,dimz,dimc,
		nx,ny,nmailles,mailles,dimmailles);

	return P4Filaire(geo,opt_fil);
}

int
P4Graf2::P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
		int nx,int ny,int mailles[],int nmailles,
		int dimmailles,const Fil opt_fil)
{

    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d  cas general (avec numeros    *
    // *  de mailles codes pour le fortran (entre 1 et nmailles)    *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *------------------------------------------------------------*

	GeomPure4Dim2 geo(x,y,z,c,dimx,dimy,dimz,dimc,
		nx,ny,nmailles,mailles,dimmailles);

	return P4Filaire(geo,opt_fil);
}

int
P4Graf2::P4Filaire(const int nbrnoeuds,const int nbrmailles,
		const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],const int mailles[][3],const Fil opt_fil)
{

    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d  cas general                  *
    // *  pour maillage non structure triangulaire                  *
    // *------------------------------------------------------------*

	GeomPure4 geo(nbrnoeuds,nbrmailles,x,y,z,c,mailles);

	return P4Filaire(geo,opt_fil);
}

int
P4Graf2::P4Bornes(const GeomPure4& geo)
{

    // *----------------------------------------------------*
    // *  Calcul des bornes si mode auto (bornes = 0.)      *
    // *----------------------------------------------------*

	int ier = -1;

	REEL xmin = _borne_min.X();
	REEL ymin = _borne_min.Y();
	REEL zmin = _borne_min.Z();
	REEL cmin = _borne_min.C();
	REEL xmax = _borne_max.X();
	REEL ymax = _borne_max.Y();
	REEL zmax = _borne_max.Z();
	REEL cmax = _borne_max.C();
	const int logx = _log.I();
	const int logy = _log.J();
	const int logz = _log.K();
	const int logc = _log.L();

	const int nx = geo.Nx();
	const int ny = geo.Ny();
	const int dimx = geo.Dimx();
	const int dimy = geo.Dimy();
	const int dimz = geo.Dimz();
	const int dimc = geo.Dimc();

	if(xmin == 0. && xmax == 0.)
	{
	    ier=P4MinMax(nx,ny,dimx,geo.X(),xmin,xmax);
	}

	if(ymin == 0. && ymax == 0.)
	{
	    ier=P4MinMax(nx,ny,dimy,geo.Y(),ymin,ymax);
	}

	if(zmin == 0. && zmax == 0.)
	{
	    ier=P4MinMax(nx,ny,dimz,geo.Z(),zmin,zmax);
	}

	if(dimc != 0. && cmin == 0. && cmax == 0.)
	{
	    ier=P4MinMax(nx,ny,dimc,geo.C(),cmin,cmax);
	}
	else if(dimc == 0. && cmin == 0. && cmax == 0.)
	{
	    cmin = zmin;
	    cmax = zmax;
	}

	_borne_min.X(xmin);
	_borne_min.Y(ymin);
	_borne_min.Z(zmin);
	_borne_min.C(cmin);
	_borne_max.X(xmax);
	_borne_max.Y(ymax);
	_borne_max.Z(zmax);
	_borne_max.C(cmax);

	P4DefSpace(xmin,ymin,zmin,xmax,ymax,zmax,logx,logy,logz ) ;

	if(logc != ECH3DINC) _lxyz.Bc(logc==ECH3DLOG);
	
	return ier;
}

int
P4Graf2::P4Filaire(const GeomPure4& geo,const Fil opt_fil)
{

    // *-------------------------------------------------------------*
    // *  trace de surface filaire 4d  cas general                   *
    // *  si *min=*max=0 => calcul automatique des bornes            *
    // *  au-dessus de cmax et en dessous de cmin : couleur _icolor  *
    // *  sinon on se sert de _hauteurs                              *
    // *  en particulier si cmin=cmax != 0 : toute la ligne est      *
    // *  de la couleur _icoh ou _icob (couleurs dessus dessous surf)*
    // *-------------------------------------------------------------*

	if(_time) TimerInit();

	int ier = P4Bornes(geo);

	P4TraceGrilles();
	ier = P4TraceGeom(geo,opt_fil);

	if(_time)
	{
	    cout << " Temps calcul P4Filaire " << endl;
	    TimerReport(1,geo.Nx()*geo.Ny());
	}

	return ier;
}

int
P4Graf2::P4TraceGeom(const GeomPure4& geo,const Fil opt_fil)
{

    // *------------------------------------------------------------*
    // *  trace de surface filaire 4d  cas general                  *
    // *------------------------------------------------------------*

	int ier = 0;

	const int nx = geo.Nx();
	const int ny = geo.Ny();
	const int dimx = geo.Dimx();
	const int dimy = geo.Dimy();
	const int dimz = geo.Dimz();
	const int dimc = geo.Dimc();

	switch (_modetrace)
	{
	    case L32DROT :

		{
		    switch(_masq)
		    {
			case MASQUE_TRI :
			{
			    GeomPure4 geo_initial_projete(geo);

			    geo_initial_projete.EnlevePoints();

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_initial_projete.Projete(nx,ny,dimx,dimy,
						dimz,dimc,geo.X(),geo.Y(),
						geo.Z(),geo.C(),_lxyz,_orig,
						_projh);
			    }
			    else
			    {
				geo_initial_projete.GeomPure::Projete(nx,ny,
						dimx,dimy,dimz,geo.X(),geo.Y(),
						geo.Z(),_lxyz,_orig,_projh);
			    }

			    GeomPure4 geo_convexe_projete;
			    Tabunsigned ordt;

			    ordt.ModValn(unsigned(Dep),1);
			    geo_convexe_projete.AjoutePoints(
					    geo_initial_projete);
			    pt_geo_temp = &geo_initial_projete;

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_initial_projete.Convexe(geo_convexe_projete,
						ordt,_time_geom);
			    }
			    else
			    {
				geo_initial_projete.AjouteC(geo.C(),_lxyz);
				geo_initial_projete.ConvexeMaille(
						geo_convexe_projete,ordt,
						_time_geom);
			    }

			    pt_geo_temp = NULL;
			    geo_initial_projete.Enleve();

			    ordt.ModValn(1,geo_convexe_projete.NombreMailles());

			    Geom geo_tri(geo_convexe_projete,ordt);

			    RootWin::ExpIHM();

			    if(_ecriture == FALSE) break;

			    pt_geo_temp = &geo_tri;
			    ier = geo_tri.TriTer(_time_geom);
			    pt_geo_temp = NULL;

			    if(ier == -1) break;

			    GeomPure4 geo_convexe;

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_convexe.AjoutePoints(nx,
						ny,dimx,dimy,dimz,dimc,geo.X(),
						geo.Y(),geo.Z(),geo.C(),_lxyz);
			    }
			    else
			    {
				geo_convexe.GeomPure::AjoutePoints(nx,
						ny,dimx,dimy,dimz,
						geo.X(),geo.Y(),geo.Z(),_lxyz);
			    }

			    Bool4 lxyz = _lxyz;

			    _lxyz = Bool4();

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_convexe.InvProjete(geo_convexe_projete,
						_lxyz,_orig,_projh);
			    }
			    else
			    {
				geo_convexe.AjouteC(geo_convexe_projete.C(),
						_lxyz);
				geo_convexe.GeomPure::InvProjete(
						geo_convexe_projete,
						_lxyz,_orig,_projh);
			    }

			    geo_convexe.ModMailles(geo_convexe_projete);
			    geo_convexe.ModTraceFil(geo_convexe_projete);

			    geo_convexe_projete.Enleve();
			    pt_geo_temp = &geo_convexe;

			    if(_td4d == DONNEE_NOEUD && opt_fil != FIL &&
					    opt_fil != FAC && opt_fil != FAC_FIL)
			    {
				geo_convexe.Convexe(geo_convexe,ordt,
						_time_geom);
			    }
			    else if(opt_fil != FIL && opt_fil != FAC &&
					    opt_fil != FAC_FIL)
			    {
				geo_convexe.ConvexeMaille(geo_convexe,ordt,
						_time_geom);
			    }

			    pt_geo_temp = NULL;
			    RootWin::ExpIHM();

			    if(_ecriture == FALSE) break;

			    ier = P4Fil(geo_convexe,ordt,opt_fil);
			    _lxyz = lxyz;

			    geo_convexe.Enleve();
			    ordt.Enleve();
			}
			    break;

			case MASQUE_SIMPLE_MIN :
			case MASQUE_SIMPLE_MAX : 
			{
			    Tabunsigned ordt;

			    P4CalcOrdreSimple(geo,ordt);

			    GeomPure4 geo_initial(geo);

			    geo_initial.EnlevePoints();

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_initial.AjoutePoints(nx,ny,dimx,dimy,dimz,
						dimc,geo.X(),geo.Y(),geo.Z(),
						geo.C(),_lxyz);
			    }
			    else
			    {
				geo_initial.GeomPure::AjoutePoints(nx,
						ny,dimx,dimy,dimz,
						geo.X(),geo.Y(),geo.Z(),_lxyz);
			    }

			    Bool4 lxyz = _lxyz;

			    _lxyz = Bool4();

			    if(opt_fil != FIL && opt_fil != FAC &&
					    opt_fil != FAC_FIL)
			    {
				GeomPure4 geo_convexe;

				geo_convexe.AjoutePoints(geo_initial);
				pt_geo_temp = &geo_initial;

				if(_td4d == DONNEE_NOEUD)
				{
				    geo_initial.Convexe(geo_convexe,ordt,
						    _time_geom);
				}
				else
				{
				    geo_initial.AjouteC(geo.C(),_lxyz);
				    geo_initial.ConvexeMaille(geo_convexe,
						    ordt,_time_geom);
				}

				pt_geo_temp = NULL;
				RootWin::ExpIHM();

				if(_ecriture == FALSE) break;

				geo_initial.Enleve();

				ier = P4Fil(geo_convexe,ordt,opt_fil);

				geo_convexe.Enleve();
			    }
			    else
			    {

				if(_td4d == DONNEE_MAILLE)
				{
				    geo_initial.AjouteC(geo.C(),_lxyz);
				}

				if(opt_fil == FAC && _modetrace == L32DROT &&
						_lxyz.Bool3::AllFalse() &&
						_tronc.Notron())
				{
				    geo_initial.Projete2d(_orig,_projh);
				    ier = P4Facette(geo_initial,ordt);
				}
				else
				{
				    ier = P4Fil(geo_initial,ordt,opt_fil);
				}

				geo_initial.Enleve();
			    }

			    _lxyz = lxyz;
			}
			    break;

			case MASQUE_ABSENT :
			{
			    Tabunsigned ordt;
			    GeomPure4 geo_initial(geo);

			    geo_initial.EnlevePoints();

			    if(_td4d == DONNEE_NOEUD)
			    {
				geo_initial.AjoutePoints(nx,
					ny,dimx,dimy,dimz,dimc,
					geo.X(),geo.Y(),geo.Z(),geo.C(),_lxyz);
			    }
			    else
			    {
				geo_initial.GeomPure::AjoutePoints(nx,
					ny,dimx,dimy,dimz,
					geo.X(),geo.Y(),geo.Z(),_lxyz);
			    }

			    Bool4 lxyz = _lxyz;

			    _lxyz = Bool4();

			    if(opt_fil != FIL && opt_fil != FAC &&
					    opt_fil != FAC_FIL)
			    {
				GeomPure4 geo_convexe;

				geo_convexe.AjoutePoints(geo_initial);
				ordt.ModValn(unsigned(Dep),1);
				pt_geo_temp = &geo_initial;

				if(_td4d == DONNEE_NOEUD)
				{
				    geo_initial.Convexe(geo_convexe,ordt,
					_time_geom);
				}
				else
				{
				    geo_initial.AjouteC(geo.C(),_lxyz);
				    geo_initial.ConvexeMaille(geo_convexe,
					ordt,_time_geom);
				}

				pt_geo_temp = NULL;
				RootWin::ExpIHM();

				if(_ecriture == FALSE) break;

				geo_initial.Enleve();
				ordt.Enleve();

				unsigned nma = geo_convexe.NombreMailles();
				Tabunsignedbuf temp(nma);

				temp.Enleve();

				for(unsigned i=0;i<nma;i++) temp.Ajoute(i);

				ordt.Ajoute(temp);
				temp.Vide();

			 	ier=P4Fil(geo_convexe,ordt,opt_fil);

				geo_convexe.Enleve();
			    }
			    else
			    {
				unsigned nma = geo_initial.NombreMailles();
				Tabunsignedbuf temp(nma);

				temp.Enleve();

				for(unsigned i=0;i<nma;i++) temp.Ajoute(i);

				ordt.Ajoute(temp);
				temp.Vide();

				if(_td4d == DONNEE_MAILLE)
				{
				    geo_initial.AjouteC(geo.C(),_lxyz);
				}

				if(opt_fil == FAC && _modetrace == L32DROT &&
						_lxyz.Bool3::AllFalse() &&
						_tronc.Notron())
				{
				    geo_initial.Projete2d(_orig,_projh);
				    ier = P4Facette(geo_initial,ordt);
				}
				else
				{
				    ier = P4Fil(geo_initial,ordt,opt_fil);
				}

				geo_initial.Enleve();
			    }

			    _lxyz=lxyz;
			}
			break;
		      default:
			break;
		    }

		}

		break;

	    case L33DROT :

		break;

	    case L33DSROT :

		break;
	}

	return ier;
}

int
P4Graf2::P4Lissage(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
		int nx,int ny,unsigned mailles[],unsigned nmailles,
		int dimmailles,const Lisse opt_lisse)
{

    // *------------------------------------------------------------*
    // *  trace de surface lissage 4d  cas general                  *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *------------------------------------------------------------*

	GeomPure4Dim2 geo(x,y,z,c,dimx,dimy,dimz,dimc,
		nx,ny,nmailles,mailles,dimmailles);

	return P4Lissage(geo,opt_lisse);
}

int
P4Graf2::P4Lissage(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
		int nx,int ny,int mailles[],int nmailles,
		int dimmailles,const Lisse opt_lisse)
{

    // *------------------------------------------------------------*
    // *  trace de surface lissage 4d  cas general (avec numeros    *
    // *  de mailles codes pour le fortran (entre 1 et nmailles)    *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *------------------------------------------------------------*

	GeomPure4Dim2 geo(x,y,z,c,dimx,dimy,dimz,dimc,
		nx,ny,nmailles,mailles,dimmailles);

	return P4Lissage(geo,opt_lisse);
}

int
P4Graf2::P4Lissage(const int nbrnoeuds,const int nbrmailles,
		const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const REEL_S c[],const int mailles[][3],const Lisse opt_lisse)
{

    // *------------------------------------------------------------*
    // *  trace de surface lissage 4d  cas general                  *
    // *  pour maillage non structure triangulaire                  *
    // *------------------------------------------------------------*

	GeomPure4 geo(nbrnoeuds,nbrmailles,x,y,z,c,mailles);

	return P4Lissage(geo,opt_lisse);
}

int
P4Graf2::P4Lissage(const GeomPure4& geo,const Lisse opt_lisse)
{

    // *---------------------------------------------------------------*
    // *  trace de surface lisse   4d                                  *
    // *  si *min=*max=0 => calcul automatique des bornes              *
    // *  au-dessus de cmax et en dessous de cmin : couleur _icolor    *
    // *  sinon on se sert de _hauteurs                                *
    // *  en particulier si cmin=cmax != 0 : toute la ligne est        *
    // *  de la couleur _icoh ou _icob (couleurs dessus dessous surf)  *
    // *---------------------------------------------------------------*

	if(_time) TimerInit();

	int ier = P4Bornes(geo);

	P4TraceGrilles();
	ier = P4TraceGeom(geo,opt_lisse);

	if(_time)
	{
	    cout << "Temps calcul P4Lissage" << endl;
	    TimerReport(1,geo.Nx()*geo.Ny());
	}

	return ier;
}

int
P4Graf2::P4TraceGeom(const GeomPure4& geo,const Lisse opt_lisse)
{

    // *------------------------------------------------------------*
    // *  trace de surface lisse   4d                               *
    // *------------------------------------------------------------*

	int ier = 0;

	const int nx = geo.Nx();
	const int ny = geo.Ny();
	const int dimx = geo.Dimx();
	const int dimy = geo.Dimy();
	const int dimz = geo.Dimz();
	const int dimc = geo.Dimc();

	switch (_modetrace)
	{
	  case L32DROT :
	    {

		switch(_masq)
		{
		  case MASQUE_TRI :
		    {
			GeomPure4 geo_initial_projete(geo);

			geo_initial_projete.EnlevePoints();

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial_projete.Projete(nx,ny,dimx,dimy,
					    dimz,dimc,geo.X(),geo.Y(),geo.Z(),
					    geo.C(),_lxyz,_orig,_projh);
			}
			else
			{
			    geo_initial_projete.GeomPure::Projete(nx,ny,
					    dimx,dimy,dimz,geo.X(),
					    geo.Y(),geo.Z(),
					    _lxyz,_orig,_projh);
			}

			GeomPure4 geo_convexe_projete;
			Tabunsigned ordt;

			ordt.ModValn(unsigned(Dep),1);
			geo_convexe_projete.AjoutePoints(geo_initial_projete);
			pt_geo_temp = &geo_initial_projete;

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial_projete.Convexe(geo_convexe_projete,
					    ordt,_time_geom);
			}
			else
			{
			    geo_initial_projete.AjouteC(geo.C(),_lxyz);
			    geo_initial_projete.ConvexeMaille(
					    geo_convexe_projete,ordt,_time_geom);
			}

			pt_geo_temp = NULL;
			RootWin::ExpIHM();

			if(_ecriture == FALSE) break;

			geo_initial_projete.Enleve();

			ordt.ModValn(1,geo_convexe_projete.NombreMailles());

			Geom geo_tri(geo_convexe_projete,ordt);

			RootWin::ExpIHM();

			if(_ecriture == FALSE) break;

			pt_geo_temp = &geo_tri;
			ier = geo_tri.TriTer(_time_geom);
			pt_geo_temp = NULL;

			if(ier == -1) break;

			GeomPure4 geo_convexe;

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_convexe.AjoutePoints(nx,
					    ny,dimx,dimy,dimz,dimc,
					    geo.X(),geo.Y(),geo.Z(),geo.C(),
					    _lxyz);
			}
			else
			{
			    geo_convexe.GeomPure::AjoutePoints(nx,
					    ny,dimx,dimy,dimz,
					    geo.X(),geo.Y(),geo.Z(),_lxyz);
			}

			const Bool4 lxyz = _lxyz;

			_lxyz = Bool4();

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_convexe.InvProjete(geo_convexe_projete,
					    _lxyz,_orig,_projh);
			}
			else
			{
			    geo_convexe.AjouteC(geo_convexe_projete.C(),_lxyz);
			    geo_convexe.GeomPure::InvProjete(
					    geo_convexe_projete,_lxyz,_orig,
					    _projh);
			}

			geo_convexe.ModMailles(geo_convexe_projete);
			geo_convexe.ModTraceFil(geo_convexe_projete);

			if(_cherche_mailles == FALSE)
			{
			    geo_convexe_projete.Enleve();
			}

			pt_geo_temp = &geo_convexe;

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_convexe.Convexe(geo_convexe,ordt,
					    _time_geom);
			}
			else
			{
			    geo_convexe.ConvexeMaille(geo_convexe,ordt,
					    _time_geom);
			}

			pt_geo_temp = NULL;
			RootWin::ExpIHM();

			if(_ecriture == FALSE) break;

			ier = P4Lisse(geo_convexe,ordt,opt_lisse);

			if(_cherche_mailles == TRUE)
			{
			    int codel3[14];
			    REEL xpos,ypos;

			    while(TRUE)
			    {
				L3SelectionSousImage(0);
				L3DialogueGETPOS(codel3, GRANDECROIX,
						SANSELASTIQUE,SANSMARQUE,
						SANSTRACE);
				L3ConvCoordL3VersUtil(codel3[7],codel3[8],
						&xpos,&ypos);
				
				if(codel3[9]==102) break;

				TabSommet ts;
				Tabunsigned tmai;

				geo_convexe_projete.ChercheMailles(xpos,
						ypos,ts,tmai);
			    }

			    geo_convexe_projete.Enleve();
			}

			_lxyz=lxyz;

			geo_convexe.Enleve();
			ordt.Enleve();
		    }
		    break;

		  case MASQUE_SIMPLE_MIN :
		  case MASQUE_SIMPLE_MAX : 
		    {
			Tabunsigned ordt;

			P4CalcOrdreSimple(geo,ordt);

			GeomPure4 geo_initial(geo);

			geo_initial.EnlevePoints();

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial.AjoutePoints(nx,
					    ny,dimx,dimy,dimz,dimc,
					    geo.X(),geo.Y(),geo.Z(),geo.C(),
					    _lxyz);
			}
			else
			{
			    geo_initial.GeomPure::AjoutePoints(nx,
					    ny,dimx,dimy,dimz,
					    geo.X(),geo.Y(),geo.Z(),_lxyz);
			}

			const Bool4 lxyz = _lxyz;
			GeomPure4 geo_convexe;

			_lxyz = Bool4();
			geo_convexe.AjoutePoints(geo_initial);
			pt_geo_temp = &geo_initial;

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial.Convexe(geo_convexe,ordt,
					    _time_geom);
			}
			else
			{
			    geo_initial.AjouteC(geo.C(),_lxyz);
			    geo_initial.ConvexeMaille(geo_convexe,
					    ordt,_time_geom);
			}

			pt_geo_temp = NULL;
			RootWin::ExpIHM();

			if(_ecriture == FALSE) break;

			geo_initial.Enleve();

			ier = P4Lisse(geo_convexe,ordt,opt_lisse);
			_lxyz = lxyz;

			geo_convexe.Enleve();
		    }
		    break;

		  case MASQUE_ABSENT :
		    {
			Tabunsigned ordt;
			GeomPure4 geo_initial(geo);

			geo_initial.EnlevePoints();

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial.AjoutePoints(nx,
					    ny,dimx,dimy,dimz,dimc,
					    geo.X(),geo.Y(),geo.Z(),geo.C(),
					    _lxyz);
			}
			else
			{
			    geo_initial.GeomPure::AjoutePoints(nx,
					    ny,dimx,dimy,dimz,
					    geo.X(),geo.Y(),geo.Z(),_lxyz);
			}

			const Bool4 lxyz = _lxyz;
			GeomPure4 geo_convexe;

			_lxyz = Bool4();
			ordt.ModValn(unsigned(Dep),1);
			geo_convexe.AjoutePoints(geo_initial);
			pt_geo_temp = &geo_initial;

			if(_td4d == DONNEE_NOEUD)
			{
			    geo_initial.Convexe(geo_convexe,ordt,
					    _time_geom);
			}
			else
			{
			    geo_initial.AjouteC(geo.C(),_lxyz);
			    geo_initial.ConvexeMaille(geo_convexe,
					    ordt,_time_geom);
			}

			pt_geo_temp = NULL;
			RootWin::ExpIHM();

			if(_ecriture == FALSE) break;

			geo_initial.Enleve();
			ordt.Enleve();

			const unsigned nma = geo_convexe.NombreMailles();
			Tabunsignedbuf temp(nma);

			temp.Enleve();

			for(unsigned i=0;i<nma;i++) temp.Ajoute(i);

			ordt.Ajoute(temp);
			temp.Vide();

			ier=P4Lisse(geo_convexe,ordt,opt_lisse);
			_lxyz=lxyz;

			geo_convexe.Enleve();
		    }
		    break;
		  default:
		    break;
		}

	    }

	    break;

	  case L33DROT :

	    break;

	  case L33DSROT :

	    break;
	}

	return ier;
}

// ********************************************************************
//    coupes generales par un hyperplan ax + by +cz +dc + e = 0
//    traces en filaire mono 3d ou 4d
// ********************************************************************

int
P4Graf2::P4CoupePlan(const REEL_S x[],const REEL_S y[],const REEL_S z[],
		int dimx,int dimy,int dimz,int nx,int ny,
		unsigned mailles[],unsigned nmailles,int dimmailles,
		REEL a,REEL b,REEL c,REEL d)
{

    // *------------------------------------------------------------*
    // *  calcul de coupe par un plan  cas general                  *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *------------------------------------------------------------*

	GeomPure4Dim2 geo(x,y,z,dimx,dimy,dimz,
		nx,ny,nmailles,mailles,dimmailles);

	return P4CoupePlan(geo,a,b,c,0,d);
}

int
P4Graf2::P4CoupePlan(const GeomPure4& geo,
		REEL a,REEL b,REEL c,REEL d,REEL e)
{

    // *------------------------------------------------------------*
    // *  calcul de coupe par un plan  cas general                  *
    // *------------------------------------------------------------*

	if(_time) TimerInit();

	int ier = P4Bornes(geo);

	P4TraceGrilles();
	ier = P4TraceGeom(geo,a,b,c,d,e);

	if(_time)
	{
	    cout << "Temps calcul P4CoupePlan" << endl;
	    TimerReport(1,geo.Nx()*geo.Ny());
	}

	return ier;
}

int
P4Graf2::P4TraceGeom(const GeomPure4& geo,
		REEL a,REEL b,REEL c,REEL d,REEL e,
		GeomPure4* const pt_geo_coupe)
{
    // *------------------------------------------------------------*
    // *  calcul de coupe par un plan  cas general                  *
    // *  pour maillage structure (i,j)                             *
    // *  ou non structure mailles a nombre de cote quelconques     *
    // *  non planaires                                             *
    // *  save_geom : TRUE => si le maillage est volumique          *
    // *  alors on reconstitue une maillage GeomPure4               *
    // *  que l'on pourra repasser dans les methodes de lissages    *
    // *  ou filaires                                               *
    // *------------------------------------------------------------*

	int ier = 0;

	int nx = geo.Nx();
	int ny = geo.Ny();
	int dimx = geo.Dimx();
	int dimy = geo.Dimy();
	int dimz = geo.Dimz();
	int dimc = geo.Dimc();

	switch (_modetrace)
	{
	  case L32DROT :
	    {
		Tabunsigned ordt;
		GeomPure4 geo_initial(geo);

		geo_initial.EnlevePoints();

		ptr_hauteurs = _fil==OPT_4D ? &_hauteurs_c : &_hauteurs_z;

		if(_td4d == DONNEE_NOEUD)
		{
		    geo_initial.AjoutePoints(nx,
				    ny,dimx,dimy,dimz,dimc,
				    geo.X(),geo.Y(),geo.Z(),geo.C(),_lxyz);
		}
		else
		{
		    geo_initial.GeomPure::AjoutePoints(nx,
				    ny,dimx,dimy,dimz,
				    geo.X(),geo.Y(),geo.Z(),_lxyz);
		}

		Bool4 lxyz = _lxyz;

		_lxyz = Bool4();

		GeomPure4 geo_convexe;
		const P4Boolean save_geom = P4Boolean(pt_geo_coupe != NULL);

		geo_convexe.AjoutePoints(geo_initial);
		ordt.ModValn(unsigned(Dep),1);

		if(_td4d == DONNEE_NOEUD)
		{
		    geo_initial.Convexe(geo_convexe,ordt,_time_geom,save_geom);
		}
		else
		{
		    geo_initial.AjouteC(geo.C(),_lxyz);
		    geo_initial.ConvexeMaille(geo_convexe,ordt,_time_geom,
				    save_geom);
		}

		geo_initial.Enleve();
		ordt.Enleve();

		ier = P4Coupe(geo_convexe,a,b,c,d,e,pt_geo_coupe);

		geo_convexe.Enleve();

		_lxyz = lxyz;
	    }

	    break;

	  case L33DROT :

	    break;

	  case L33DSROT :

	    break;
	}

	return ier;
}

// *********************************************************
//     routine de calcul d'ordre des mailles dans les
//     cas simples
// *********************************************************

void
P4Graf2::P4CalcOrdreSimple(const GeomPure4& geo,Tabunsigned& ordt)
{

    // -------------------------------------------------------
    //  calcul de l'ordre des mailles dans le cas tri simple
    // -------------------------------------------------------

	if(strcmp(geo.TypeMailles(),"MailleSt4") != 0)
	{
	    const REEL_S* const x = geo.X();
	    const REEL_S* const y = geo.Y();
	    const REEL_S* const z = geo.Z();

	    const unsigned nma = geo.NombreMailles();
	    TabREEL_Sbuf temp(nma);temp.Enleve();
	    Tabunsignedbuf nu;

	    for(register unsigned i=0;i<nma;i++)
	    {
                geo.DonneSommets(nu,i);
    
                register const unsigned nnu = nu;

		if(!nnu)
		{
		    temp.Ajoute(0.0f);
		    continue;
		}

		register unsigned indice = nu(0);
		register REEL_S xm = x[indice];
		register REEL_S ym = y[indice];
		register REEL_S zm = z[indice];

		for(register unsigned j=1;j<nnu;j++)
		{
		    indice = nu(j);
		    xm += x[indice];
		    ym += y[indice];
		    zm += z[indice];
		}

		xm /= nnu;
		ym /= nnu;
		zm /= nnu;

		Sommet s(xm,ym,zm);
		REEL_S prof = - _projh.ProjeteZ(_lxyz,_orig,s);;

		temp.Ajoute(prof);
	    }

	    unsigned* num = new unsigned[nma];

	    quicksort((const REEL_S*) temp,num,nma);
	    temp.Vide();
	    ordt.Ajoute(num,nma);

	    delete[] num;

	    return;
	}

	const int nx = geo.Nx();
	const int ny = geo.Ny();
	const int dimx = geo.Dimx();
	const int dimy = geo.Dimy();
	const int dimz = geo.Dimz();

	REEL_S xc[4],yc[4],zc[4];

	xc[0] = geo.X()(0);
	yc[0] = geo.Y()(0);
	zc[0] = geo.Z()(0);
	xc[1] = geo.X()(nx-1);
	zc[1] = geo.Z()(nx-1);
	xc[2] = geo.X()(nx-1 + dimx*(ny-1));
	xc[3] = geo.X()(dimx*(ny-1));

	if(dimy == -1)
	{
	    yc[1] = geo.Y()(0);
	    yc[2] = geo.Y()(ny-1);
	    yc[3] = geo.Y()(ny-1);
	}
	else
	{
	    yc[1] = geo.Y()(nx-1);
	    yc[2] = geo.Y()(nx-1 + dimy*(ny-1));
	    yc[3] = geo.Y()(dimy*(ny-1));
	}

	zc[2] = geo.Z()(nx-1 + dimz*(ny-1));
	zc[3] = geo.Z()(dimz*(ny-1));

	GeomPure geo_red;

	geo_red.Projete(4,1,4,4,4,xc,yc,zc,_lxyz,_orig,_projh);

	int p = 0;

	if(_masq==MASQUE_SIMPLE_MIN)	p = P4Imin(geo_red.Z());
	else              		p = P4Imax(geo_red.Z());

	int imin = 0,isig = 1;
	int jmin = 0,jsig = 1;

	if(p==0) {imin=0;jmin=0;isig=1;jsig=1;}

	if(p==1) {imin=nx-2;jmin=0;isig=-1;jsig=1;}

	if(p==2) {imin=nx-2;jmin=ny-2;isig=-1;jsig=-1;}

	if(p==3) {imin=0;jmin=ny-2;isig=1;jsig=-1;}

	Tabunsignedbuf temp((nx-1)*(ny-1));temp.Enleve();

	for(int j=0;j<ny-1;j++)
	{

	    for(int i=0;i<nx-1;i++)
	    {
		temp.Ajoute(unsigned(i*isig+imin+(nx-1)*(j*jsig+jmin)));
	    }

	}

	ordt.Ajoute(temp);
}

int
P4Graf2::P4Coordonnees(const GeomPure4& geo,REEL xpos,REEL ypos,
		TabSommet& ts,Tabunsigned& tmm)
{

    // *------------------------------------------------------------*
    // *  calcul de coordonnees 3d dans l'espace projete            *
    // *------------------------------------------------------------*

	int ier = 0;

	if(_td4d != DONNEE_NOEUD)
	{
	    cerr << "P4Graf2> Calcul de coordonnees pour donnees aux mailles "
		"non implemente" << endl;
	    return ier;
	}

	int nx = geo.Nx();
	int ny = geo.Ny();
	int dimx = geo.Dimx();
	int dimy = geo.Dimy();
	int dimz = geo.Dimz();
	int dimc = geo.Dimc();

	GeomPure4 geo_initial_projete(geo);

	geo_initial_projete.EnlevePoints();
	geo_initial_projete.Projete(nx,ny,dimx,dimy,
			dimz,dimc,geo.X(),geo.Y(),geo.Z(),geo.C(),
			_lxyz,_orig,_projh);

	GeomPure4 geo_convexe_projete;
	Tabunsigned ordt;

	ordt.ModValn(unsigned(Dep-1),1);
	geo_convexe_projete.AjoutePoints(geo_initial_projete);
	geo_initial_projete.Convexe(geo_convexe_projete,ordt,_time_geom);

	geo_initial_projete.Enleve();

	unsigned taille_tm = tmm;

	geo_convexe_projete.ChercheMailles(xpos,ypos,ts,tmm);

	unsigned taille_tm2 = tmm;

	for(register unsigned i=taille_tm;i<taille_tm2;i++)
	{
	    tmm[i] = ordt(tmm(i));
	}

	return ier;
}
