#ifndef P4SOMMET_H
#define P4SOMMET_H

// %W% %G% Pierre Brochard

/* Includes */

#include "U8machcomp.h"
#include "S6compat.h"
#include "P4objet.h"

/* Declarations des classes */

class Sommet : public Objet
{
    protected :

        REEL _x;
        REEL _y;
        REEL _z;

    public :

        Sommet();
        Sommet(const REEL x,const REEL y,const REEL z);
        Sommet(const Sommet& s);
        virtual ~Sommet();

	void* operator new(size_t);
	void* operator new(size_t,void*);
#ifndef P4ANSI_DELETE
	inline void operator delete(void* ptr,size_t);
#else
	inline void operator delete(void* ptr);
	inline void operator delete(void* ptr,void*);
#endif

        void operator()(REEL x,REEL y,REEL z);

        REEL X() const {return _x;};
        REEL Y() const {return _y;};
        REEL Z() const {return _z;};

        void X(const REEL x) {_x=x;};
        void Y(const REEL y) {_y=y;};
        void Z(const REEL z) {_z=z;};

        void  S(const Sommet& s) {_x=s._x;_y=s._y;_z=s._z;};
        void  S(REEL x,REEL y,REEL z) {_x=x;_y=y;_z=z;};

        void Permute(Sommet& s1,Sommet& s2,Sommet& s3);

        void Echange(Sommet& s);

	unsigned sizeobjet() {return sizeof(Sommet);}; 

        int operator == (const Sommet& s) const
            {return (_x==s._x)&&(_y==s._y)&&(_z==s._z);};
        int operator != (const Sommet& s) const
            {return (_x!=s._x)&&(_y!=s._y)&&(_z!=s._z);};
        void operator = (const Sommet& s) {_x=s._x;_y=s._y;_z=s._z;};
        void operator ++ () {_x=_x+1.;_y=_y+1.;_z=_z+1.;};
#if (defined(_IBMR2) || defined(SUNOS) || (defined(IRIX) && \
		!defined(IRIX_4))) && (!defined(SC1_0) || !defined(SUNOS_4))
	void operator ++ (int) {_x=_x+1.;_y=_y+1.;_z=_z+1.;};
#endif

	friend inline ostream& operator << (ostream& ost,Sommet s);
};

class Sommet4 : public Sommet
{
    private :

        REEL _c;

    public :

        Sommet4();
        Sommet4(const REEL x,const REEL y,const REEL z,const REEL c);
        Sommet4(const Sommet4& s);
        ~Sommet4();

	void* operator new(size_t);
	void* operator new(size_t,void*);
#ifndef P4ANSI_DELETE
	inline void operator delete(void* ptr,size_t);
#else
	inline void operator delete(void* ptr);
	inline void operator delete(void* ptr,void*);
#endif

        void operator()(REEL x,REEL y,REEL z,REEL c);

        REEL C() const {return _c;};

        void C(const REEL c) {_c=c;};

        void  S(const Sommet4& s) {_x=s._x;_y=s._y;_z=s._z;_c=s._c;};
        void  S(REEL x,REEL y,REEL z,REEL c) {_x=x;_y=y;_z=z;_c=c;};

        void Permute(Sommet4& s1,Sommet4& s2,Sommet4& s3);

        void Echange(Sommet4& s);

        unsigned sizeobjet() {return sizeof(Sommet4);}; 

        int operator == (const Sommet4& s) const
            {return (_x==s._x)&&(_y==s._y)&&(_z==s._z)&&(_c==s._c);};
        int operator != (const Sommet4& s) const
            {return (_x!=s._x)&&(_y!=s._y)&&(_z!=s._z)&&(_c!=s._c);};
        void operator = (const Sommet4& s)
            {_x=s._x;_y=s._y;_z=s._z;_c=s._c;};

        friend inline ostream& operator << (ostream& ost,Sommet4 s);
};

// ******************************************************
//  fonctions membres inline
// ******************************************************

// ---------------------------
//    Sommet
// ---------------------------

inline void*
Sommet::operator new(size_t size)
{
	return ::operator new(size);
}

inline void*
Sommet::operator new(size_t,void* buf)
{
	return buf;
}

#ifndef P4ANSI_DELETE
inline void
Sommet::operator delete(void* ptr,size_t)
{
	::operator delete(ptr);
}
#else
inline void
Sommet::operator delete(void* ptr)
{
	::operator delete(ptr);
}

inline void
Sommet::operator delete(void*,void*)
{
}
#endif

inline
Sommet::Sommet() : Objet()
{
	_x = 0.;
	_y = 0.;
	_z = 0.;
}

inline
Sommet::Sommet(const REEL x,const REEL y,const REEL z) : Objet()
{
	_x = x;
	_y = y;
	_z = z;
}

inline
Sommet::Sommet(const Sommet& s) : Objet()
{
	_x = s._x;
	_y = s._y;
	_z = s._z;
}

inline
Sommet::~Sommet()
{
}

inline void
Sommet::operator()(REEL x,REEL y,REEL z)
{
	_x = x;
	_y = y;
	_z = z;
}

inline void
Sommet::Permute(Sommet& s1,Sommet& s2,Sommet& s3)
{
	Sommet si(s1);

	s1._x = _x; s1._y = _y; s1._z = _z;
	_x = s3._x; _y = s3._y; _z= s3._z;
	s3._x = s2._x; s3._y = s2._y; s3._z = s2._z;
	s2._x = si._x; s2._y = si._y; s2._z = si._z;
}

inline void
Sommet::Echange(Sommet& s)
{
	Sommet si(s);

	s._x=_x;s._y=_y;s._z=_z;
	_x=si._x;_y=si._y;_z=si._z;
}

inline ostream&
operator << (ostream& ost,Sommet s) 
{
	ost << " sommet " << s._x << " " << s._y << " " << s._z << endl;

	return ost;
}

// ---------------------------
//    Sommet4
// ---------------------------

inline void*
Sommet4::operator new(size_t size)
{
	return ::operator new(size);
}

inline void*
Sommet4::operator new(size_t,void* buf)
{
	return buf;
}

#ifndef P4ANSI_DELETE
inline void
Sommet4::operator delete(void* ptr,size_t)
{
	::operator delete(ptr);
}
#else
inline void
Sommet4::operator delete(void* ptr)
{
	::operator delete(ptr);
}

inline void
Sommet4::operator delete(void*,void*)
{
}
#endif

inline
Sommet4::Sommet4() : Sommet()
{
	_c = 0.;
}

inline
Sommet4::Sommet4(const REEL x,const REEL y,const REEL z,const REEL c) :
            Sommet(x,y,z)
{
	_c = c;
}


inline
Sommet4::Sommet4(const Sommet4& s) : Sommet(s)
{
	_c = s._c;
}

inline
Sommet4::~Sommet4()
{
}

inline void
Sommet4::operator()(REEL x,REEL y,REEL z,REEL c)
{
	_x = x;
	_y = y;
	_z = z;
	_c = c;
}

inline void
Sommet4::Permute(Sommet4& s1,Sommet4& s2,Sommet4& s3)
{
	Sommet4 si(s1);

	s1._x = _x; s1._y = _y; s1._z = _z; s1._c = _c;
	_x = s3._x; _y = s3._y; _z= s3._z; _c = s3._c;
	s3._x = s2._x; s3._y = s2._y; s3._z = s2._z; s3._c = s2._c;
	s2._x = si._x; s2._y = si._y; s2._z = si._z; s2._c = si._c;
}

inline void
Sommet4::Echange(Sommet4& s)
{
	Sommet4 si(s);

	s._x=_x;s._y=_y;s._z=_z;s._c=_c;
	_x=si._x;_y=si._y;_z=si._z;_c=si._c;
}

inline ostream&
operator << (ostream& ost,Sommet4 s) 
{
	ost << " sommet4 " << s._x << " " << s._y << " " <<
		s._z << " " << s._c << endl;

	return ost;
}

// ******************************************************
//  fonctions inline
// ******************************************************

inline Sommet
P4Log(const Sommet& s)
{
	Sommet s2;
	if(s.X()>0.) s2.X(log10(s.X())); else s2.X(s.X());
	if(s.Y()>0.) s2.Y(log10(s.Y())); else s2.Y(s.Y());
	if(s.Z()>0.) s2.Z(log10(s.Z())); else s2.Z(s.Z());
	return s2;
}

inline Sommet
P4Cvmgt(const Sommet& x,const Sommet& y,const P4Boolean t)
{
	return t == TRUE ? x : y;
}

#endif



