struct SccsIdP4Trace
{
	static const char* const sccs_id;
};

const char* const SccsIdP4Trace::sccs_id = "%W% %G% Pierre Brochard";

#include "P4constant.h"
#include "minotaure/P4sanglier.h"
#include "P4trace.h"
#include "minotaure/P4graf2.h"

static P4Graf2 graf;
const REEL AspectTexte = 1.5;

int FinChaine(const char carac[],int long_init)
{
	carac = carac+long_init;

	while((*--carac == ' ' || *carac == '\0') && --long_init>0);

	return long_init;
}

int DebutChaine(const char carac[],int long_init)
{
	int longueur = 0;

	while((*carac == ' ' || *carac == '\0') && longueur++<long_init)
		carac++;

	return longueur;
}

REEL Xphytolog(REEL zmin,REEL zmax,int izech,REEL z)
{
	REEL xphytolog = 0.;

	if(izech==0)     // cas lineaire
	{
	    xphytolog = ( z -zmin ) / ( zmax - zmin );
	}
	else             // cas logarithmique
	{
	   REEL zzmin = P4Log ( zmin );
           REEL zzmax = P4Log ( zmax );
           REEL zz    = P4Log ( z    );
           xphytolog   = ( zz -zzmin ) / ( zzmax - zzmin );
	}

	return xphytolog;

}

void TraceEchellePolyg(const int n,const REEL y[],const int icol[],
		const REEL xg,const REEL xd)
{
	REEL xp[4],yp[4];

	xp[0]=xg;
	xp[1]=xg;
	xp[2]=xd;
	xp[3]=xd;

	for(int i=0;i<n;i++)
	{       
	    yp[0]=y[i];
	    yp[3]=yp[0];
	    yp[1]=y[i+1];
	    yp[2]=yp[1];

	    L3Couleur(icol[i]);
	    L3PointCourant(2,xp[0],yp[0]);
	    L3Polygone(2,xp,yp,4);
	}

}

void TraceEchelleSegment(const int n,const REEL y[],const int icol[],
		const REEL xg,const REEL xd)
{
	REEL yp[4];

	for(int i=0;i<n;i++)
	{       
	    yp[0]=y[i];
	    yp[3]=yp[0];
	    yp[1]=y[i+1];
	    yp[2]=yp[1];

	    L3Couleur(icol[i]);
	    REEL xde1=xg+(xd-xg)*.1,xde2=xd-(xd-xg)*.1;
	    REEL yde=(y[i]+y[i+1])*.5;
	    L3Segment(2,&xde1,&yde,&xde2,&yde,1);
	}

}

extern "C"
{
    void p4reouvrel3_()
    {
	graf.P4ReOuvreL3();
    }

    void p4ecriture_()
    {
	graf.P4Ecriture();
    }

    inline int p4mini_(const int& a,const int& b)
    {
    // Sur OSF1_V4 avec template, l'instanciation de min est refusee a
    // cause du extern "C" !!
	return a<=b ? a : b;
    }

    inline REEL p4min_(const REEL& a,const REEL& b)
    {
    // Sur OSF1_V4 avec template, l'instanciation de min est refusee a
    // cause du extern "C" !!
	return a<=b ? a : b;
    }

    inline REEL p4max_(const REEL& a,const REEL& b)
    {
    // Sur OSF1_V4 avec template, l'instanciation de max est refusee a
    // cause du extern "C" !!
	return a>=b ? a : b;
    }

    inline int p4maxi_(const int& a,const int& b)
    {
    // Sur OSF1_V4 avec template, l'instanciation de max est refusee a
    // cause du extern "C" !!
	return a>=b ? a : b;
    }

    inline REEL p4abs_(const REEL& a)
    {
    // Sur OSF1_V4 avec template, l'instanciation de abs est refusee a
    // cause du extern "C" !!
	return a>0 ? a : -a;
    }

    inline REEL p4exp10_(const REEL& a)
    {
    // Sur OSF1_V4 avec template, l'instanciation de P4Exp10 est refusee a
    // cause du extern "C" !!
	return REEL(pow(10.,double(a)));
    }

    REEL_S p4log10_(const REEL_S& x)
    {
    // Avec cxx sous OSF1_V4 avec template, l'instanciation de P4Log
    // est refusee a cause du extern "C" !!
        REEL lx = x > 0 ? log10(REEL(x)) : Emin;

	return REEL_S(lx);
    }

    void p4vrbor_(REEL_S& xmin,REEL_S& xmax,const REEL_S& nbdecades,
		const P4Boolean& modlog,const int& marge)
    {
	REEL xmi=(REEL) xmin,xma=(REEL) xmax;

	P4Vrbor(xmi,xma,(REEL) nbdecades,modlog,marge);

	xmin=(REEL_S) xmi,xmax=(REEL_S) xma;
    }

    void p4calchauteurs_(REEL_S hauteurs[],const int& nbrehauteurs,
		const REEL_S& hmin,const REEL_S& hmax,const REEL_S z[],
		const int& nx,const int& ny,const int& nz,
		const P4Boolean& modlog)
    {
	graf.P4CalcHauteurs(hauteurs,nbrehauteurs,hmin,hmax,
		z,nx,ny,nz,modlog);
    }

    void p4trace_(P4Boolean& iseul,REEL_S x[],REEL_S y[],REEL_S z[],REEL_S c[],
                int& nx,int& ny,int nxsto[],
		REEL_S hauteurs_z[],int& n_hauteurs_z,REEL_S hauteurs_c[],
		int& n_hauteurs_c,P4Boolean& log_zh,P4Boolean& log_ch,
		char ctith[],char ctitg[],char ctitd[],char ctitb[],
		char texte[][132],int& nbtex,
		char tit1[],char tit2[],char tit3[],
		P4Boolean& logx,P4Boolean& logy,P4Boolean& logz,P4Boolean& logc,
		TypeTrace& i3d,int& i2d,int& icho32,
		REEL_S& site,REEL_S& azimu,REEL_S& dista,REEL_S& redrv,
		REEL_S& redrh,REEL_S& brilla,
		int icolot[],int& icoloh,int& icolob,int& icolog,
		int icolote[],int& icoloe,int& igrille,int ititr[],
		int kartr[],int icourbe[],int icoloc[],char ilabel[][136],
		char label3d[][136],const int& nlabe3d, 
		int& icolo7z,int& icolo7c,int& icolo8,int itext[],int ktext[],
		REEL_S xpostex[],REEL_S ypostex[],REEL_S anglet[],
		REEL_S& xmin,REEL_S& xmax,REEL_S& ymin,REEL_S& ymax,
		REEL_S& zmin,REEL_S& zmax,REEL_S& cmin,REEL_S& cmax,
		int& ikart,int& ikare,REEL_S& dcx,REEL_S& dcy,Masque& masque,
		REEL_S& xpos5i,REEL_S& ypos5i,REEL_S& xpos5a,REEL_S& ypos5a,
		REEL_S& xpos2i,REEL_S& ypos2i,REEL_S& xpos2a,REEL_S& ypos2a,
		REEL_S& x_ech_min_z,REEL_S& y_ech_min_z,REEL_S& x_ech_max_z,
		REEL_S& y_ech_max_z,REEL_S& x_ech_min_c,REEL_S& y_ech_min_c,
		REEL_S& x_ech_max_c,REEL_S& y_ech_max_c,REEL_S& x_leg_min_c,
		REEL_S& y_leg_min_c,REEL_S& x_leg_max_c,REEL_S& y_leg_max_c,
		REEL_S& x_leg_min,REEL_S& y_leg_min,REEL_S& x_leg_max,
		REEL_S& y_leg_max,
		REEL_S& xming1,REEL_S& xmaxg1,REEL_S& yming1,REEL_S& ymaxg1,
		REEL_S& x1b,REEL_S& x2b,REEL_S& y1b,REEL_S& y2b,
		REEL_S& x1,REEL_S& x2,REEL_S& y1t,REEL_S& y2,
		int& izzz,int& idede,int& ifondopt)
    {

    // ----------------------------------------------------------
    // sous-programme d'appels graphiques pour Psyche++ 3.0
    // ----------------------------------------------------------

	static int num_courbe=0;

	int nnx=10,nny=10,nnz=5,ninter=4;

    // -calcul des parametres de boite-----------------------------------

	P4Boolean auto_boite = (P4Boolean) (xpos5i==0. && ypos5i==0. &&
		xpos5a==0. && ypos5a==0.);

	REEL_S xpos6i = p4min_(xpos5i,xpos5a);
	xpos5a = p4max_(xpos5i,xpos5a);
	xpos5i = xpos6i;
	REEL_S ypos6i = p4min_(ypos5i,ypos5a);
	ypos5a = p4max_(ypos5i,ypos5a);
	ypos5i = ypos6i;

	REEL xcen=.5,ycen=.55,scalx=.75,scaly=.75;

	if(!auto_boite && icho32!=1)
	{
	    xcen = (xpos5i+xpos5a)*.5;
	    ycen = (ypos5i+ypos5a)*.5;
	    scalx = p4abs_(REEL(xpos5a-xpos5i));
	    scaly = p4abs_(REEL(ypos5a-ypos5i));
	}

	REEL xpost1=0.,ypost1=0.;
	REEL xpost2=0.,ypost2=0.;
	REEL xming=15.,yming=15.;
	REEL xmaxg=90.,ymaxg=90.;

	if(auto_boite && icho32==1)
	{
	    REEL xkkk=(FinChaine(tit1)-DebutChaine(tit1)+1)/ikart;
	    REEL ykkk=5./((REEL) ikare);
	    xpost1=p4max_(REEL(.01),REEL(89./99.-xkkk));
	    ypost1=p4max_(REEL(.01),REEL(14./99.-ykkk));
	    xkkk=3./((REEL) ikare);
	    ykkk=5./((REEL) ikare);
	    xpost2 = p4max_(REEL(.01),REEL(14./99.-xkkk));
	    ypost2 = p4min_(REEL(89./99.+ykkk),REEL(.99));
	}
	else if(icho32==1)
	{
	    xming=xpos5i*99.+1.;
	    xmaxg=xpos5a*99.+1.;
	    yming=ypos5i*99.+1.;
	    ymaxg=ypos5a*99.+1.;
	    REEL xkkk=(FinChaine(tit1)-DebutChaine(tit1)+1)/ikart;
	    REEL ykkk=5./((REEL) ikare);
	    xpost1=p4max_(REEL(.01),xpos5a-xkkk);
	    ypost1=p4max_(REEL(.01),ypos5i-ykkk);
	    xkkk=3./((REEL) ikare);
	    ykkk=5./((REEL) ikare);
	    xpost2 = p4max_(REEL(.01),xpos5i-xkkk);
	    ypost2 = p4min_(ypos5a+ykkk,REEL(.99));
	}

    // -initialisations, mise en page----------------------------------

	num_courbe++;

	cout << "\n TRACE NUMERO " << num_courbe << endl;

	L3DefPoliceCaractere(POLICESOFT);
	L3Cadre((double) dcx,(double) dcy);

	L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.);

	if(icho32==1)
	{
	    L3MiseEnPage(1,xming,yming,xmaxg,ymaxg,1.,1.,100.,100.);
	}

	L3MiseEnPage(2,0.,0.,1.,1.,0.,0.,1.,1.);
	L3DefSpace(2,0.,0.,1.,1.);

	L3Brillance((double) brilla);

    // -trace d'un carre pour forcer le fond a la couleur de fond----------

	if(ifondopt==1)
	{
	    REEL xpag[4],ypag[4];
	    xpag[0]=0.;
	    xpag[1]=1.;
	    xpag[2]=1.;
	    xpag[3]=0.;
	    ypag[0]=1.;
	    ypag[1]=1.;
	    ypag[2]=0.;
	    ypag[3]=0.;
	    L3Couleur(-1);
	    L3Polygone(2,xpag,ypag,4);
	}

    // -trace des echelles de couleur pour le 3d-4d------------------------

	if((icolo7z>=1 || icolo7c>=1) && icho32==0)
	{
	    REEL_S xtemp = p4min_(x_ech_min_z,x_ech_max_z);
	    x_ech_max_z = p4max_(x_ech_min_z,x_ech_max_z);
	    x_ech_min_z = xtemp;
	    REEL_S ytemp = p4min_(y_ech_min_z,y_ech_max_z);
	    y_ech_max_z = p4max_(y_ech_min_z,y_ech_max_z);
	    y_ech_min_z = ytemp;

	    xtemp = p4min_(x_ech_min_c,x_ech_max_c);
	    x_ech_max_c = p4max_(x_ech_min_c,x_ech_max_c);
	    x_ech_min_c = xtemp;
	    ytemp = p4min_(y_ech_min_c,y_ech_max_c);
	    y_ech_max_c = p4max_(y_ech_min_c,y_ech_max_c);
	    y_ech_min_c = ytemp;

	    int n_hauteurs = p4maxi_(n_hauteurs_z,n_hauteurs_c);

	    REEL* yh = NULL;yh = new REEL[n_hauteurs];
	    int*  ch = NULL;ch = new int[n_hauteurs+1];

	    if(!yh || !ch)
	    {
		delete[] yh;delete[] ch;
		yh=NULL,ch=NULL;
		cerr << " pb allocation memoire dans P4Trace " << endl;
		return;
	    }

	    if(icolo7z>=1)
	    {
		int i;

		for(i=0;i<n_hauteurs_z;i++)
		{
		    yh[i]=Xphytolog(hauteurs_z[0],hauteurs_z[n_hauteurs_z-1],
			log_zh,hauteurs_z[i]);
		    yh[i]=(y_ech_max_z-y_ech_min_z)*yh[i]+y_ech_min_z;
		}

		int tempi = 54;
		REEL xc=54./p4min_(n_hauteurs_z,tempi);

		for(i=0;i<n_hauteurs_z+1;i++)
		{
		    ch[i]=((int) (xc*i+.1))%55+8;
		}

		TraceEchellePolyg(n_hauteurs_z-1,yh,&ch[1],x_ech_min_z,
			x_ech_max_z);

		int modes=0;

		if(log_zh)  modes=modes+YCOLOG;

		L3MiseEnPage(1,x_ech_min_z,y_ech_min_z,x_ech_max_z,y_ech_max_z,
			0.,0.,1.,1.);
		L3DefGrille(1,GAUCHE,AXE_CREUX,TRUE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,DROITE,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,BAS,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,HAUT,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefTexte(1,ikare,0.);
		L3Couleur(icolog);
		L3Grille(1,0.,hauteurs_z[0],1.,hauteurs_z[n_hauteurs_z-1],10,10,
			modes);
	    }

	    if(icolo7c>=1)
	    {
		int i;

		for(i=0;i<n_hauteurs_c;i++)
		{
		    yh[i]=Xphytolog(hauteurs_c[0],hauteurs_c[n_hauteurs_c-1],
			log_ch,hauteurs_c[i]);
		    yh[i]=(y_ech_max_c-y_ech_min_c)*yh[i]+y_ech_min_c;
		}

		int tempi = 54;
		REEL xc=54./p4min_(n_hauteurs_c,tempi);

		for(i=0;i<n_hauteurs_c+1;i++)
		{
		    ch[i]=((int) (xc*i+.1))%55+8;
		}

		TraceEchellePolyg(n_hauteurs_c-1,yh,&ch[1],x_ech_min_c,
			x_ech_max_c);

		int modes=0;

		if(log_ch)  modes=modes+YCOLOG;

		L3MiseEnPage(1,x_ech_min_c,y_ech_min_c,x_ech_max_c,y_ech_max_c,
			0.,0.,1.,1.);
		L3DefGrille(1,GAUCHE,AXE_CREUX,TRUE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,DROITE,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,BAS,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(1,HAUT,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefTexte(1,ikare,0.);
		L3Couleur(icolog);
		L3Grille(1,0.,hauteurs_c[0],1.,hauteurs_c[n_hauteurs_c-1],10,10,
			modes);
	    }

	    delete[] yh;delete[] ch;
	}

    // -legendes des courbes de niveaux 4d------------------------------

	if(nlabe3d > 0 && icho32==0)
	{
	    REEL_S xtemp = p4min_(x_leg_min_c,x_leg_max_c);
	    x_leg_max_c = max(x_leg_min_c,x_leg_max_c);
	    x_leg_min_c = xtemp;
	    REEL_S ytemp = p4min_(y_leg_min_c,y_leg_max_c);
	    y_leg_max_c = p4max_(y_leg_min_c,y_leg_max_c);
	    y_leg_min_c = ytemp;

	    REEL* yh=NULL;yh=new REEL[nlabe3d+1];
	    int*  ch=NULL;ch=new int[nlabe3d+1];

	    if(!yh || !ch)
	    {
		delete[] yh;delete[] ch;
		yh=NULL,ch=NULL;
		cerr << " pb allocation memoire dans P4Trace " << endl;
		return;
	    }

	    int tempi = 54;
	    int tempi2 = nlabe3d;
	    REEL xc = 54./p4min_(tempi2,tempi);
	    int i;

	    for(i=0;i<nlabe3d;i++)
	    {
		yh[i]=Xphytolog(1.,(REEL) (nlabe3d+1),
			0,(REEL) (i+1));
		yh[i]=(y_leg_max_c-y_leg_min_c)*yh[i]+y_leg_min_c;
		ch[i]=((int) (xc*i+.1))%55+8;
	    }

	    yh[nlabe3d]=y_leg_max_c;

	    TraceEchelleSegment(nlabe3d,yh,ch,x_leg_min_c,x_leg_max_c);

	    int llstn=0;
	    char bufchar[256];
	    char decode[256];

	    for(i=0;i<nlabe3d;i++)
	    {
		L3Couleur(ch[i]);
		L3DefTexte(2,100,0.);
		REEL xpo9=x_leg_max_c;
		REEL ypo9=(yh[i+1]+yh[i])*.5-.005;
		L3PointCourant(2,xpo9,ypo9);
		int iin=DebutChaine(label3d[i],136);
		int jjn=FinChaine(label3d[i],136);
		int lln=jjn-iin+1;
		label3d[i][jjn]='\0';

		if(lln > 0)
		{
		    llstn = p4maxi_(llstn,lln);
		    bufchar[0]=' ',bufchar[1]='\0';
		    L3TranscodeChaine(strcat(bufchar,&label3d[i][iin]),
				decode);
		    L3Texte(2,decode);
		}

	    }

	    L3Couleur(0);

	    REEL xx1=x_leg_min_c-.01,
		yy1=y_leg_min_c-.01,
		xx2=x_leg_max_c+.01*(llstn+1),
		yy2=y_leg_min_c-.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_min_c-.01,
		yy1=y_leg_max_c+.01,
		xx2=x_leg_max_c+.01*(llstn+1),
		yy2=y_leg_max_c+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_min_c-.01,
		yy1=y_leg_min_c-.01,
		xx2=x_leg_min_c-.01,
		yy2=y_leg_max_c+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_max_c+.01*(llstn+1),
		yy1=y_leg_min_c-.01,
		xx2=x_leg_max_c+.01*(llstn+1),
		yy2=y_leg_max_c+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

	    delete[] yh;delete[] ch;
	}


    // -legende des courbes 2d-------------------------------------------

	if(icolo8 >= 1 && icho32==1)
	{
	    REEL_S xtemp = p4min_(x_leg_min,x_leg_max);
	    x_leg_max = p4max_(x_leg_min,x_leg_max);
	    x_leg_min = xtemp;
	    REEL_S ytemp = p4min_(y_leg_min,y_leg_max);
	    y_leg_max = p4max_(y_leg_min,y_leg_max);
	    y_leg_min = ytemp;

	    int courbes_a_tracer=0;
	    int i;

	    for(i=0;i<ny;i++)
	    {

		if(icourbe[i] != -1) courbes_a_tracer++;

	    }

	    REEL* yh=NULL;yh=new REEL[ny+1];
	    int*  jc=NULL;jc=new int[ny+1];

	    if(!yh || !jc)
	    {
		delete[] yh;delete[] jc;
		yh=NULL,jc=NULL;
		cerr << " pb allocation memoire dans P4Trace " << endl;
		return;
	    }

	    int j = -1;

	    for(i=0;i<ny;i++)
	    {

		if(icourbe[i] != -1)
		{
		    j++;
		    yh[j]=Xphytolog(1.,(REEL) (courbes_a_tracer+1),0,
			(REEL) (j+1));
		    yh[j]=(y_leg_max-y_leg_min)*yh[j]+y_leg_min;
		    jc[j]=icoloc[i];
		}

	    }

	    yh[courbes_a_tracer]=y_leg_max;

	    TraceEchelleSegment(courbes_a_tracer,yh,jc,x_leg_min,x_leg_max);

	    j=-1;
	    int llstn=0;
	    char bufchar[256];
	    char decode[256];

	    for(i=0;i<ny;i++)
	    {

		if(icourbe[i] != -1)
		{
		    j++;
		    L3Couleur(icoloc[i]);
		    L3DefTexte(2,100,0.);
		    REEL xpo9=x_leg_max;
		    REEL ypo9=(yh[j+1]+yh[j])*.5-.005;
		    L3PointCourant(2,xpo9,ypo9);
		    int iin=DebutChaine(ilabel[i],136);
		    int jjn=FinChaine(ilabel[i],136);
		    int lln=jjn-iin+1;
		    ilabel[i][jjn]='\0';

		    if(lln > 0)
		    {
			llstn = p4maxi_(llstn,lln);
			bufchar[0]=' ',bufchar[1]='\0';
			L3TranscodeChaine(strcat(bufchar,&ilabel[i][iin]),
				decode);
			L3Texte(2,decode);
		    }

		}

	    }

	    L3Couleur(0);

	    REEL xx1=x_leg_min-.01,
		yy1=y_leg_min-.01,
		xx2=x_leg_max+.01*(llstn+1),
		yy2=y_leg_min-.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_min-.01,
		yy1=y_leg_max+.01,
		xx2=x_leg_max+.01*(llstn+1),
		yy2=y_leg_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_min-.01,
		yy1=y_leg_min-.01,
		xx2=x_leg_min-.01,
		yy2=y_leg_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=x_leg_max+.01*(llstn+1),
		yy1=y_leg_min-.01,
		xx2=x_leg_max+.01*(llstn+1),
		yy2=y_leg_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

	    delete[] yh;delete[] jc;
	}

    // -TITRES------------------------------------------------------- 

	if(!iseul)
	{
	    char decode[256];
	    float ecart = -1.;

	    if(ititr[0] >= 1)
	    {
		int ii0=DebutChaine(ctith);
		int jj0=FinChaine(ctith);
		int ll0=jj0-ii0+1;

		if(ll0 > 0)
		{
		    ctith[jj0]='\0';
		    L3Couleur(icolot[0]);
		    L3DefTexte(0,kartr[0],0.);
		    L3TranscodeChaine(&ctith[ii0],decode);
		    L3AfficheTitres(0,decode,ecart, HAUT);

		    if(ititr[0] == 2) L3EncadreTexte();

		}

	    }

	    if(ititr[1] >= 1)
	    {
		int ii1=DebutChaine(ctitg);
		int jj1=FinChaine(ctitg);
		int ll1=jj1-ii1+1;

		if(ll1 > 0)
		{
		    ctitg[jj1]='\0';
		    L3Couleur(icolot[1]);
		    L3DefTexte(0,kartr[1],0.);
		    L3TranscodeChaine(&ctitg[ii1],decode);
		    L3AfficheTitres(0,decode,ecart, GAUCHE);

		    if(ititr[1] == 2) L3EncadreTexte();

		}

	    }

	    if(ititr[2] >= 1)
	    {
		int ii2=DebutChaine(ctitd);
		int jj2=FinChaine(ctitd);
		int ll2=jj2-ii2+1;

		if(ll2 > 0)
		{
		    ctitd[jj2]='\0';
		    L3Couleur(icolot[2]);
		    L3DefTexte(0,kartr[2],0.);
		    L3TranscodeChaine(&ctitd[ii2],decode);
		    L3AfficheTitres(0,decode,ecart, DROITE);

		    if(ititr[2] == 2) L3EncadreTexte();

		}

	    }

	    if(ititr[3] >= 1)
	    {
		int ii3=DebutChaine(ctitb);
		int jj3=FinChaine(ctitb);
		int ll3=jj3-ii3+1;

		if(ll3 > 0)
		{
		    ctitb[jj3]='\0';
		    L3Couleur(icolot[3]);
		    L3DefTexte(0,kartr[3],0.);
		    L3TranscodeChaine(&ctitb[ii3],decode);
		    L3AfficheTitres(0,decode,ecart, BAS);

		    if(ititr[3] == 2) L3EncadreTexte();

		}

	    }

	    L3Couleur(icolog);
	}

    // -3d-4d------------------------------------------------------------

	if(icho32 != 1 && i3d != PAS_DE_TRACE)
	{
	    REEL zsoc=.1;

	    graf.P4Time(TRUE);
	    graf.P4TimeGeom(TRUE);
	    graf.P4ChercheMaillesInit(FALSE);
	    graf.P4Dcolo(icoloh);

	    if(idede == 0)
	    {
		graf.P4Defpa(BAS_TRACE,icolob);
	    }
	    else
	    {
		graf.P4Defpa(BAS_TRACE,icoloh);
	    }

	    int type_masque=(int) masque;

	    graf.P4Def2(type_masque,2,zsoc);
	    graf.P4Dtron(TRONC3D_GAUDROI);

	// init point de vue

	    graf.P4PtvueScxy(site,azimu,dista,xcen,ycen,scalx,scaly,
			    redrv,redrh);

	// init hauteurs

	    graf.P4InitHauteurs(hauteurs_z,n_hauteurs_z,
			    OPT_3D,8,62,log_zh);
	    graf.P4InitHauteurs(hauteurs_c,n_hauteurs_c,
			    OPT_4D,8,62,log_ch);

	// init grilles

	    if(iseul || igrille == 0)
	    {
		graf.P4AnnuleGrilles();
	    }
	    else if(!iseul && igrille != 0)
	    {
		int type_axe=AXE_ABSENT;

		if(igrille == 1) type_axe=AXE_CREUX;

		if(igrille == 2) type_axe=AXE_SEMICREUX;

		if(igrille == 3) type_axe=AXE_PLEIN;

		graf.P4Dcolo(icoloe);

		graf.P4DefCaracGrille(ikart,ikare);

		int ii7=DebutChaine(tit1);
		int jj7=FinChaine(tit1);
		tit1[jj7]='\0';
		int ll7=jj7-ii7+1;
		int ii8=DebutChaine(tit2);
		int jj8=FinChaine(tit2);
		int ll8=jj8-ii8+1;
		tit2[jj8]='\0';
		int ii9=DebutChaine(tit3);
		int jj9=FinChaine(tit3);
		int ll9=jj9-ii9+1;
		tit3[jj9]='\0';

		if(ll7 <= 0) {ii7=0;tit1[0]=' ';tit1[1]='\0';}

		if(ll8 <= 0) {ii8=0;tit2[0]=' ';tit2[1]='\0';}

		if(ll9 <= 0) {ii9=0;tit3[0]=' ';tit3[1]='\0';}

		char decodx[256],decody[256],decodz[256];

		L3TranscodeChaine(&tit1[ii7],decodx);
		L3TranscodeChaine(&tit2[ii8],decody);
		L3TranscodeChaine(&tit3[ii9],decodz);

		graf.P4DefTitreAuto(decodx,decody,decodz,"","","");

		graf.P4Dcolo(icolog);

		int grad_inter[6];

		grad_inter[0]=ninter;
		grad_inter[1]=ninter;
		grad_inter[2]=ninter;
		grad_inter[3]=ninter;
		grad_inter[4]=ninter;
		grad_inter[5]=ninter;

		graf.P4DefGrilleAuto(0.,0.,zsoc,FALSE,nnx,nny,nnz,type_axe,
		grad_inter);
	    }

	// trace ligne monocouleur, 3d ou 4d

	    graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
		(REEL) zmin,(REEL) cmin,(REEL) xmax,(REEL) ymax,
		(REEL) zmax,(REEL) cmax,logx,logy,logz,logc);

	    if(icho32 == 3)
	    {

		switch (i3d)
		{

		    case FILMONO :

			graf.P4LigneBrisee(x,y,z,ny);
			break;

		    case FIL3D :

			graf.P4Init34d(OPT_3D);
			graf.P4LigneBrisee(x,y,z,c,ny);
			break;

		    case FIL4D :

			graf.P4Init34d(OPT_4D);
			graf.P4LigneBrisee(x,y,z,c,ny);
			break;

		    default :

			cerr << " Option non implementee " << endl;
			break;
		}

	    }

	// trace surface monocouleur, 3d ou 4d

	    else
	    {

		switch (i3d)
		{

		    case PAS_DE_TRACE :

			break;

		    case FILMONO :

			cout << "\n cas filaire monocouleur " << endl;
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);

			if(masque==MASQUE_ABSENT)
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny);
			}
			else
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					    NULL,0u,0,FIL);
			}

			break;

		    case FIL3D :

			cout << "\n cas filaire 3d " << endl;
			graf.P4Init34d(OPT_3D);

			if(masque==MASQUE_ABSENT)
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny);
			}
			else
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					    NULL,0u,0,FIL);
			}

			break;

		    case FIL4D :

			cout << "\n cas filaire 4d " << endl;
			graf.P4Init34d(OPT_4D);

			if(masque==MASQUE_ABSENT)
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny);
			}
			else
			{
			    graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					    NULL,0u,0,FIL);
			}

			break;

		    case FAC3D :

			cout << "\n cas facettes 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC);
			break;

		    case FAC4D :

			cout << "\n cas facettes 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC);
			break;

		    case NIV3D :

			cout << "\n cas niveaux 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,NIV);
			break;

		    case NIV4D :

			cout << "\n cas niveaux 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,NIV);
			break;

		    case LIS3D :

			cout << "\n cas lissage 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_3D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS);
			break;

		    case LIS4D :

			cout << "\n cas lissage 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_4D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS);
			break;

		    case FIL_FAC3D :

			cout << "\n cas filaire monocouleur + facettes 3d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_3D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL_FAC4D :

			cout << "\n cas filaire monocouleur + facettes 4d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_4D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL3D_FAC3D :

			cout << "\n cas filaire 3d + facettes 3d " << endl;
			graf.P4Init34d(OPT_3D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL3D_FAC4D :

			cout << "\n cas filaire 3d + facettes 4d " << endl;
			graf.P4Init34d(OPT_3D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL4D_FAC3D :

			cout << "\n cas filaire 4d + facettes 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL4D_FAC4D :

			cout << "\n cas filaire 4d + facettes 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL);
			break;

		    case FIL_NIV3D :

			cout << "\n cas filaire monocouleur + niveaux 3d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FIL_NIV4D :

			cout << "\n cas filaire monocouleur + niveaux 4d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FIL3D_NIV3D :

			cout << "\n cas filaire 3d + niveaux 3d " << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FIL3D_NIV4D :

			cout << "\n cas filaire 3d + niveaux 4d " << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FIL4D_NIV3D :

			cout << "\n cas filaire 4d + niveaux 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FIL4D_NIV4D :

			cout << "\n cas filaire 4d + niveaux 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FIL_NIV);
			break;

		    case FAC3D_NIV3D :

			cout << "\n cas facettes 3d + niveaux 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_NIV);
			break;

		    case FAC3D_NIV4D :

			cout << "\n cas facettes 3d + niveaux 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_NIV);
			break;

		    case FAC4D_NIV3D :

			cout << "\n cas facettes 4d + niveaux 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_NIV);
			break;

		    case FAC4D_NIV4D :

			cout << "\n cas facettes 4d + niveaux 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_NIV);
			break;

		    case FIL_LIS3D :

			cout << "\n cas filaire monocouleur + lissage 3d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_3D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL_LIS4D :

			cout << "\n cas filaire monocouleur + lissage 4d "
				<< endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_4D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL3D_LIS3D :

			cout << "\n cas filaire 3d + lissage 3d " << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_4D,OPT_3D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL3D_LIS4D :

			cout << "\n cas filaire 3d + lissage 4d " << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_4D,OPT_4D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL4D_LIS3D :

			cout << "\n cas filaire 4d + lissage 3d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_3D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL4D_LIS4D :

			cout << "\n cas filaire 4d + lissage 4d " << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D,OPT_4D);
			graf.P4Lissage(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,LIS_FIL);
			break;

		    case FIL_FAC3D_NIV3D :

			cout << "\n cas filaire monocouleur + facettes 3d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_3D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL_FAC3D_NIV4D :

			cout << "\n cas filaire monocouleur + facettes 3d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_4D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL_FAC4D_NIV3D :

			cout << "\n cas filaire monocouleur + facettes 4d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL_FAC4D_NIV4D :

			cout << "\n cas filaire monocouleur + facettes 4d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4DefEchelles((REEL) xmin,(REEL) ymin,
			(REEL) zmin,1.,(REEL) xmax,(REEL) ymax,
			(REEL) zmax,1.,logx,logy,logz,logc);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL3D_FAC3D_NIV3D :

			cout << "\n cas filaire 3d + facettes 3d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_3D,OPT_3D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL3D_FAC3D_NIV4D :

			cout << "\n cas filaire 3d + facettes 3d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_3D,OPT_3D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL3D_FAC4D_NIV3D :

			cout << "\n cas filaire 3d + facettes 4d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL3D_FAC4D_NIV4D :

			cout << "\n cas filaire 3d + facettes 4d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_3D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL4D_FAC3D_NIV3D :

			cout << "\n cas filaire 4d + facettes 3d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL4D_FAC3D_NIV4D :

			cout << "\n cas filaire 4d + facettes 3d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_4D,OPT_3D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL4D_FAC4D_NIV3D :

			cout << "\n cas filaire 4d + facettes 4d + "
				<< "niveaux 3d" << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_3D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    case FIL4D_FAC4D_NIV4D :

			cout << "\n cas filaire 4d + facettes 4d + "
				<< "niveaux 4d" << endl;
			graf.P4Init34d(OPT_4D,OPT_4D,OPT_4D);
			graf.P4Filaire(x,y,z,c,nx,nx,nx,nx,nx,ny,
					NULL,0u,0,FAC_FIL_NIV);
			break;

		    default :

			cerr << " Option non implementee " << endl;
			break;
		}

	    }

	}

    // -2d + coupes 3d-4d-------------------------------------------------

	else if(icho32 == 1 && i2d >= 0 && i2d <= 2)
	{
	    REEL_S* xi = NULL, *yi = NULL;

	// cas courbes integrales

	    if(i2d == 1)
	    {
		xi=new REEL_S[nx*ny];
		yi=new REEL_S[nx*ny];
	
		if(!xi || !yi)
		{
		    cerr << " pas assez de place memoire dans P4trace" << endl;
		    cerr << " on ne trace pas la courbe integrale " << endl;
		    delete[] xi;delete[] yi;
		    return;
		}

		int i;

		for(i=0;i<ny;i++)
		{

		    for(int j=0;j<nxsto[i];j++)
		    {
			xi[j+i*nx]=x[j+i*nx];
			yi[j+i*nx]=y[j+i*nx];
		    }

		}

		int j;

		for(j=0;j<ny;j++)
		{

		    for(i=1;i<nxsto[j];i++)
		    {
			y[i+j*nx]=y[i-1+j*nx]+(yi[i-1+j*nx]+
				yi[i+j*nx])*(x[i+j*nx]-x[i-1+j*nx])*.5;
		    }

		}

		for(j=0;j<ny;j++)
		{

		    for(i=0;i<nxsto[j];i++)
		    {
			y[i+j*nx]=p4max_(p4min_(y[i+j*nx],REEL_S(Bmax)),
				REEL_S(-Bmax));
		    }

		}

	    }

	// cas courbes derivees

	    else if(i2d == 2)
	    {
		xi=new REEL_S[nx*ny];
		yi=new REEL_S[nx*ny];
	
		if(!xi || !yi)
		{
		    cerr << " pas assez de place memoire dans P4trace" << endl;
		    cerr << " on ne trace pas la courbe derivee " << endl;
		    delete[] xi;delete[] yi;
		    return;
		}

		int i = 0;

		for(;i<ny;i++)
		{

		    for(int j=0;j<nxsto[i];j++)
		    {
			xi[j+i*nx]=x[j+i*nx];
			yi[j+i*nx]=y[j+i*nx];
		    }

		}

		for(int j=0;j<ny;j++)
		{
		    REEL dx=x[1+j*nx]-x[j*nx];
		    P4Boolean init1=FALSE;

		    if(dx != 0.)
		    {
			y[j*nx]=(yi[1+j*nx]-yi[j*nx])/dx;
			init1=FALSE;
		    }
		    else
		    {
			init1=TRUE;
		    }

		    for(i=1;i<nxsto[j];i++)
		    {
			dx=x[p4mini_(i+1,nxsto[j])+j*nx]-x[i-1+j*nx];

			if(dx != 0.)
			{
			    y[i+j*nx]=(yi[p4mini_(i+1,nxsto[j])+j*nx]
				-yi[i-1+j*nx])/dx;
			}
			else
			{
			    y[i+j*nx]=y[i-1+j*nx];
			}

		    }

		    if(init1)
		    {

			if(nxsto[j] >= 2)
			{
			    y[j*nx]=y[1+j*nx];
			}
			else
			{
			    y[j*nx]=0.;
			}

		    }

		    for(i=0;i<nxsto[j];i++)
		    {
			y[i+j*nx] = p4max_(p4min_(y[i+j*nx],REEL_S(Bmax)),
				REEL_S(-Bmax));
		    }

		}

	    }

	// cas general

	    // cas non zoome

	    if(izzz == 0)
	    {
		int jst=0;
		int j;

		for(j=0;j<ny;j++)
		{

		    if(icourbe[j] >= 0)
		    {
			jst=j;
			break;
		    }

		}

		x1=x[jst*nx];
		x2=x[jst*nx];
		y1t=y[jst*nx];
		y2=y[jst*nx];

		for(j=0;j<ny;j++)
		{

		    if(icourbe[j] >= 0)
		    {

			for(int i=0;i<nxsto[j];i++)
			{
			    x1=p4min_(x1,x[i+j*nx]);
			    x2=p4max_(x2,x[i+j*nx]);
			    y1t=p4min_(y1t,y[i+j*nx]);
			    y2=p4max_(y2,y[i+j*nx]);

			}
			
		    }

		}

		P4Boolean auto2d = FALSE;
		auto2d = (P4Boolean) (xmin == 0. && xmax == 0. &&
				ymin == 0. && ymax == 0.);

		if(!auto2d)
		{

		    if(xmin != 0. || xmax != 0.)
		    {
			x1=xmin;
			x2=xmax;
		    }

		    if(ymin != 0. || ymax != 0.)
		    {
			y1t=ymin;
			y2=ymax;
		    }

		}

	    }

	    // cas avec zoom

	    else
	    {
		REEL xpos3i = p4min_(xpos2i,xpos2a)*99.+1.;
		REEL xpos3a = p4max_(xpos2i,xpos2a)*99.+1.;
		REEL ypos3i = p4min_(ypos2i,ypos2a)*99.+1.;
		REEL ypos3a = p4max_(ypos2i,ypos2a)*99.+1.;

		REEL x1a=x1b;
		REEL x2a=x2b;
		REEL y1a=y1b;
		REEL y2a=y2b;

		if(x1b == 0. && x2b == 0. && y1b == 0. && y2b == 0.)
		{
		    int jst=0;
		    int j;

		    for(j=0;j<ny;j++)
		    {

			if(icourbe[j] >= 0)
			{
			    jst=j;
			    break;
			}

		    }

		    x1a=x[jst*nx];
		    x2a=x[jst*nx];
		    y1a=y[jst*nx];
		    y2a=y[jst*nx];

		    for(j=0;j<ny;j++)
		    {

			if(icourbe[j] >= 0)
			{

			    for(int i=0;i<nxsto[j];i++)
			    {
				x1a = p4min_(x1a,REEL(x[i+j*nx]));
				x2a = p4max_(x2a,REEL(x[i+j*nx]));
				y1a = p4min_(y1a,REEL(y[i+j*nx]));
				y2a = p4max_(y2a,REEL(y[i+j*nx]));
			    }
			
			}

		    }

		}

		P4Vrbor(x1a,x2a,7.,logx,0);
		P4Vrbor(y1a,y2a,7.,logy,0);

		x1=(x2a-x1a)/(xmaxg1-xming1)*(xpos3i-xming1)+x1a;
		x2=(x2a-x1a)/(xmaxg1-xming1)*(xpos3a-xming1)+x1a;
		y1t=(y2a-y1a)/(ymaxg1-yming1)*(ypos3i-yming1)+y1a;
		y2=(y2a-y1a)/(ymaxg1-yming1)*(ypos3a-yming1)+y1a;

		cout << " x1b " << x1b << " x2b " << x2b << "\n";
		cout << " y1b " << y1b << " y2b " << y2b << "\n";
		cout << " x1a " << x1a << " x2a " << x2a << "\n";
		cout << " y1a " << y1a << " y2a " << y2a << "\n";
		cout << " xming1 " << xming1 << " xmaxg1 " << xmaxg1 << "\n"; 
		cout << " xpos2i " << xpos2i << " xpos2a " << xpos2a << "\n";
		cout << " xpos3i " << xpos3i << " xpos3a " << xpos3a << "\n";
		cout << " x1 " << x1 << " x2 " << x2 << " y1 " << y1t 
			<< " y2 " << y2 << endl;

		if(logx)
		{
		    x1=p4exp10_(x1);
		    x2=p4exp10_(x2);
		}

		if(logy)
		{
		    y1t=p4exp10_(y1t);
		    y2=p4exp10_(y2);
		}

		cout << " x1 " << x1 << " x2 " << x2 << " y1 " << y1t 
			<< " y2 " << y2 << endl;
	    // -fin differences pour zoom--------------------------

	    }

	// -suite cas general (on arrondi x1,x2,y1t,y2 et on ecarte ---
	// - s'ils sont egaux)----------------------------------------

	    REEL xp1=x1,xp2=x2,yp1=y1t,yp2=y2;

	    P4Vrbor(xp1,xp2,7.,logx,0);
	    P4Vrbor(yp1,yp2,7.,logy,0);

	    x1=xp1,x2=xp2,y1t=yp1,y2=yp2;

	    if(logx)
	    {
		x1=p4exp10_(x1);
		x2=p4exp10_(x2);
	    }

	    if(logy)
	    {
		y1t=p4exp10_(y1t);
		y2=p4exp10_(y2);
	    }

	// -trace de la grille------------------------------------

	    L3DefSpaceLog(1,x1,y1t,x2,y2,logx,logy);

	    if(igrille && !iseul)
	    {
		int modes=0;

		if(logx)  modes=modes+XCOLOG;

		if(logy)  modes=modes+YCOLOG;

		int type_axe=AXE_VIDE;

		if(igrille == 1) type_axe=AXE_CREUX;

		if(igrille == 2) type_axe=AXE_SEMICREUX;

		if(igrille == 3) type_axe=AXE_PLEIN;

		L3DefTexte(1,ikare,0.);
		L3Couleur(icolog);
		L3DefGrille(1,GAUCHE,type_axe,TRUE,ninter,AUCUNE);
		L3DefGrille(1,DROITE,AXE_VIDE,TRUE,ninter,AUCUNE);
		L3DefGrille(1,BAS,type_axe,TRUE,ninter,AUCUNE);
		L3DefGrille(1,HAUT,AXE_VIDE,TRUE,ninter,AUCUNE);
		L3Grille(1,x1,y1t,x2,y2,nnx,nny,modes);
	    }

	// -trace des courbes 2d------------------------------------

	    REEL* xd;REEL* yd;
	    xd = new REEL[2*nx-1];
	    yd = new REEL[2*nx-1];

	    if(!xd || !yd)
	    {
		delete[] xd;delete[] yd;
		cerr << " pb allocation memoire dans p4trace " << endl;
		return;
	    }

	    for(int j=0;j<ny;j++)
	    {

		if(icourbe[j] >= 0)
		{
		    L3Couleur(icoloc[j]);

		    char decode[256];
		    int i;

		    switch (icourbe[j])
		    {
			case 0 :

			    for(i=0;i<nxsto[j];i++)
			    {
				xd[i]=x[i+j*nx];
				yd[i]=y[i+j*nx];
			    }

			    L3PointCourant(1,*xd,*yd);
			    L3LigneBrisee(1,xd,yd,nxsto[j]);
			    break;

			case 1 :

			    for(i=0;i<nxsto[j];i++)
			    {
				xd[2*i]=x[i+j*nx];
				yd[2*i]=y[i+j*nx];

				if(i != nxsto[j]-1)
				{
				    xd[2*i+1]=x[i+1+j*nx];
				    yd[2*i+1]=y[i+j*nx];
				}

			    }

			    L3PointCourant(1,*xd,*yd);
			    L3LigneBrisee(1,xd,yd,2*nxsto[j]-1);
			    break;

			default :

			    for(i=0;i<nxsto[j];i++)
			    {
				xd[i]=x[i+j*nx];
				yd[i]=y[i+j*nx];
			    }

			    L3PointCourant(1,*xd,*yd);
			    L3Marqueur(1,xd,yd,nxsto[j],icourbe[j]-1);
		    }

		// label de courbes

		    L3DefTexte(1,100,0.);

		    int iin = DebutChaine(ilabel[j]);
		    int jjn = FinChaine(ilabel[j]);
		    int lln = jjn-iin+1;

		    if(lln > 0 && icolo8 == 0)
		    {
			int isto=nxsto[j]-1;

			for(i=nxsto[j]-1;i>=0;i--)
			{

			    if(x[i+j*nx] <= x2 && x[i+j*nx] >= x1 &&
				y[i+j*nx] <= y2 && y[i+j*nx] >= y1t)
			    {
				isto=i;
				break;
			    }

			}

			const int isto1 = p4mini_(isto+1,nxsto[j]-1);

			isto = p4maxi_(0,isto);

			REEL yla = y[isto+j*nx];

			if(y[isto1+j*nx] > y2) yla=y2;

			if(y[isto1+j*nx] < y1t) yla=y1t;

			L3PointCourant(1,x2,yla);
			L3Troncature(1,CLIPPINGINACTIF);
			ilabel[j][jjn]='\0';
			L3TranscodeChaine(&ilabel[j][iin],decode);
			L3Texte(1,decode);
			L3Troncature(1,CLIPPINGACTIF);
		    }

		}

	    }

	    delete[] xd;delete[] yd;

	// -titres des textes echelles x et y---------------------------

	    if(!iseul)
	    {
		char decode[256];

		L3Couleur(icoloe);

		if(ikart == 300) ikart=100;

		L3DefTexte(2,ikart,0.);
		L3PointCourant(2,xpost1,ypost1);

		int ii7=DebutChaine(tit1);
		int jj7=FinChaine(tit1);
		int ll7=jj7-ii7;

		if(ll7 > 0)
		{
		    tit1[jj7]='\0';
		    L3TranscodeChaine(&tit1[ii7],decode);
		    L3Texte(2,decode);
		}

		L3PointCourant(2,xpost2,ypost2);

		int ii8=DebutChaine(tit2);
		int jj8=FinChaine(tit2);
		int ll8=jj8-ii8;

		if(ll8 > 0)
		{
		    tit2[jj8]='\0';
		    L3TranscodeChaine(&tit2[ii8],decode);
		    L3Texte(2,decode);
		}

	    }

	// remise a zero dans le cas integral et derivees

	    if(i2d == 1 || i2d == 2)
	    {

		for(int jj=0;jj<ny;jj++)
		{

		    for(int i=0;i<nxsto[jj];i++)
		    {
			x[i+jj*nx]=xi[i+jj*nx];
			y[i+jj*nx]=yi[i+jj*nx];
		    }

		}

		delete[] xi;delete[] yi;
	    }

	}

    // -textes------------------------------------------------------------

	if(!iseul)
	{
	    char decode[256];

	    for(int i=0;i<nbtex;i++)
	    {

		if(itext[i] >= 1)
		{
		    int iim=DebutChaine(texte[i]);
		    int jjm=FinChaine(texte[i]);
		    int llm=jjm-iim+1;

		    if(llm > 0)
		    {
			texte[i][jjm]='\0';
			L3Couleur(icolote[i]);
			double angle_texte=anglet[i];

			if(angle_texte > 180.) angle_texte-=360.;

			L3DefTexte(2,ktext[i],angle_texte);
			L3TranscodeChaine(&texte[i][iim],decode);
			L3GenTextePos(2,decode,(REEL) xpostex[i],
				(REEL) ypostex[i],1.,AspectTexte,angle_texte,
				VRAI,POSGB);

			if(itext[i] == 2) L3EncadreTexte();

		   }

		}

	    }

	}

	L3Couleur(1);    // pour creation d'une image meme vide
	L3DefTexte(2,2,0.);
	L3PointCourant(2,0.,0.);
	L3Texte(2," ");

	cout << " FIN TRACE " << endl;

	L3SautDePage();

    }

}
