// ident

struct SccsIdS6Base
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Base::sccs_id = "%W% %G% Pierre Brochard";

// includes

#include "P4constant.h"
#include "S6base.h"
#include "sanglier.h"
#include "sanglierP.h"
#include "L3ListeDesCodes.h"
#include "S6trace.h"
#include "S6gereconfig.h"
#include "S6flagssup.h"

// ************************************************************
//   Point
// ************************************************************

S6Base2DPoint::S6Base2DPoint() : S6Base2D()
{
	_x1 = 0.0;
	_y1 = 0.0;
}

S6Base2DPoint::S6Base2DPoint(const S6Base2DPoint& b) : S6Base2D(b)
{
	_x1 = b._x1;
	_y1 = b._y1;
}

S6Base2DPoint::S6Base2DPoint(REEL_S x,REEL_S y,unsigned couleur) :
		S6Base2D(couleur)
{
	_x1 = x;
	_y1 = y;
}

void
S6Base2DPoint::Trace()
{
	REEL x = _x1;
	REEL y = _y1;

	L3Couleur(_couleur);
	L3Marqueur(4,&x,&y,1,1);
}

void
S6Base2DPoint::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = xmax = _x1;
	ymin = ymax = _y1;
}

// ************************************************************
//   Tableau de Points
// ************************************************************

S6Base2DTabPoint::S6Base2DTabPoint() : S6Base2DPolyline()
{
}

S6Base2DTabPoint::S6Base2DTabPoint(const S6Base2DTabPoint& b) :
		S6Base2DPolyline(b)
{
}

S6Base2DTabPoint::S6Base2DTabPoint(REEL_S* x,REEL_S* y,unsigned nb,
		unsigned couleur) : S6Base2DPolyline(x,y,nb,couleur)
{
}

S6Base2DTabPoint::~S6Base2DTabPoint()
{
}

void
S6Base2DTabPoint::Trace()
{
	REEL* x = new REEL[_nb];
	REEL* y = new REEL[_nb];

	if(!x || !y)
	{
	    delete[] x;
	    delete[] y;

	    cerr << "Pb alloc " << __FILE__ << " ligne " << __LINE__ << endl;

	    return;
	}

	for(register unsigned i=0;i<_nb;i++)
	{
	    x[i] = _x[i];
	    y[i] = _y[i];
	}

	L3Couleur(_couleur);
	L3Marqueur(4,x,y,_nb,1);

	delete[] x;delete[] y;
}

// ************************************************************
//   Segment
// ************************************************************

S6Base2DSegment::S6Base2DSegment() : S6Base2DPoint()
{
	_x2 = 0.0;
	_y2 = 0.0;
}

S6Base2DSegment::S6Base2DSegment(const S6Base2DSegment& b) :
		S6Base2DPoint(b)
{
	_x2 = b._x2;
	_y2 = b._y2;
}

S6Base2DSegment::S6Base2DSegment(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
		unsigned couleur) : S6Base2DPoint(x1,y1,couleur)
{
	_x2 = x2;
	_y2 = y2;
}

void
S6Base2DSegment::Trace()
{
	L3SelectionSousImage(4);

	int codel3[6];

	codel3[0] = 6;
	codel3[1] = CODEL3SEGMENT;
	L3ConversionCoordonneesLog(_x1,_y1,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x2,_y2,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	if(L3ClippingSegment(codel3[2],
			codel3[3],codel3[4],codel3[5],&codel3[2]))
	{
	    L3Couleur(_couleur);
	    L3AppelDrivers(codel3);
	}

}

void
S6Base2DSegment::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{

	if(_x1<_x2)
	{
	    xmin = _x1;xmax = _x2;
	}
	else
	{
	    xmin = _x2;xmax = _x1;
	}

	if(_y1<_y2)
	{
	    ymin = _y1;ymax = _y2;
	}
	else
	{
	    ymin = _y2;ymax = _y1;
	}

}

// ************************************************************
//   Tableau de segments
// ************************************************************

S6Base2DTabSegment::S6Base2DTabSegment() : S6Base2DTabPoint()
{
	_x2 = NULL;
	_y2 = NULL;
}

S6Base2DTabSegment::S6Base2DTabSegment(const S6Base2DTabSegment& b) :
		S6Base2DTabPoint(b)
{
	_x2 = new REEL_S[_nb];
	_y2 = new REEL_S[_nb];

	if(!_x2 || !_y2)
	{
	    cerr << "pb alloc S6Base2DTabSegment" << endl;
	    delete[] _x; delete[] _y; delete[] _x2; delete[] _y2;
	    _x = _y = _x2 = _y2 = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x2,b._x2,_nb*sizeof(REEL_S));
	memcpy(_y2,b._y2,_nb*sizeof(REEL_S));
}

S6Base2DTabSegment::S6Base2DTabSegment(REEL_S* x1,REEL_S* y1,
		REEL_S* x2,REEL_S* y2,unsigned nb,
		unsigned couleur) : S6Base2DTabPoint(x1,y1,nb,couleur)
{
	_x2 = new REEL_S[_nb];
	_y2 = new REEL_S[_nb];

	if(!_x2 || !_y2)
	{
	    cerr << "pb alloc S6Base2DTabSegment" << endl;
	    delete[] _x; delete[] _y; delete[] _x2; delete[] _y2;
	    _x = _y = _x2 = _y2 = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x2,x2,_nb*sizeof(REEL_S));
	memcpy(_y2,y2,_nb*sizeof(REEL_S));
}

void
S6Base2DTabSegment::Trace()
{
	L3SelectionSousImage(4);
	L3Couleur(_couleur);

	int codel3[6];

	codel3[0] = 6;
	codel3[1] = CODEL3SEGMENT;

	for(register unsigned i=0;i<_nb;i++)
	{
	    L3ConversionCoordonneesLog(_x[i],_y[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x2[i],_y2[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	    if(L3ClippingSegment(codel3[2],
			    codel3[3],codel3[4],codel3[5],&codel3[2]))
	    {
		L3Couleur(_couleur);
		L3AppelDrivers(codel3);
	    }

	}

}

void
S6Base2DTabSegment::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = ymin = Bmax,xmax = ymax = -Bmax;

	register unsigned nb = _nb;

	for(register unsigned i=0;i<nb;i++)
	{
	    xmin = min(xmin,min(REEL(_x[i]),REEL(_x2[i])));
	    xmax = max(xmax,max(REEL(_x[i]),REEL(_x2[i])));
	    ymin = min(ymin,min(REEL(_y[i]),REEL(_y2[i])));
	    ymax = max(ymax,max(REEL(_y[i]),REEL(_y2[i])));
	}

}

// ************************************************************
//   Arc
// ************************************************************

S6Base2DArc::S6Base2DArc() : S6Base2DSegment()
{
	_x3 = 0.0;
	_y3 = 0.0;
}

S6Base2DArc::S6Base2DArc(const S6Base2DArc& b) : S6Base2DSegment(b)
{
	_x3 = b._x3;
	_y3 = b._y3;
}

S6Base2DArc::S6Base2DArc(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
		REEL_S x3,REEL_S y3,unsigned couleur) :
		S6Base2DSegment(x1,y1,x2,y2,couleur)
{
	_x3 = x3;
	_y3 = y3;
}

void
S6Base2DArc::Trace()
{
	L3SelectionSousImage(4);

	int codel3[8];

	codel3[0] = 8;
	codel3[1] = CODEL3ARC;
	L3ConversionCoordonneesLog(_x1,_y1,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x2,_y2,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x3,_y3,
			&codel3[6],&codel3[7],LINEAIRE,LINEAIRE);

    /*if(L3ClippingArc(codel3[2],
			codel3[3],codel3[4],codel3[5],
			codel3[6],codel3[7],&codel3[2]))
	{
	    L3Couleur(_couleur);
	    L3AppelDrivers(codel3);
	}*/

}

void
S6Base2DArc::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = min(_x1,min(_x2,_x3));
	xmax = max(_x1,max(_x2,_x3));
	ymin = min(_y1,min(_y2,_y3));
	ymax = max(_y1,max(_y2,_y3));
}

// ************************************************************
//   Tableau d'arcs
// ************************************************************

S6Base2DTabArc::S6Base2DTabArc() : S6Base2DTabSegment()
{
	_x3 = NULL;
	_y3 = NULL;
}

S6Base2DTabArc::S6Base2DTabArc(const S6Base2DTabArc& b) :
		S6Base2DTabSegment(b)
{
	_x3 = new REEL_S[_nb];
	_y3 = new REEL_S[_nb];

	if(!_x3 || !_y3)
	{
	    cerr << "pb alloc S6Base2DTabArc" << endl;
	    delete[] _x; delete[] _y;
	    delete[] _x2; delete[] _y2;
	    delete[] _x3; delete[] _y3;
	    _x = _y = _x2 = _y2 = _x3 = _y3 = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x3,b._x3,_nb*sizeof(REEL_S));
	memcpy(_y3,b._y3,_nb*sizeof(REEL_S));
}

S6Base2DTabArc::S6Base2DTabArc(REEL_S* x1,REEL_S* y1,
		REEL_S* x2,REEL_S* y2,REEL_S* x3,REEL_S* y3,unsigned nb,
		unsigned couleur) : S6Base2DTabSegment(x1,y1,x2,y2,nb,couleur)
{
	_x3 = new REEL_S[_nb];
	_y3 = new REEL_S[_nb];

	if(!_x3 || !_y3)
	{
	    cerr << "pb alloc S6Base2DTabArc" << endl;
	    delete[] _x; delete[] _y;
	    delete[] _x2; delete[] _y2;
	    delete[] _x3; delete[] _y3;
	    _x = _y = _x2 = _y2 = _x3 = _y3 = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x3,x3,_nb*sizeof(REEL_S));
	memcpy(_y3,y3,_nb*sizeof(REEL_S));
}

void
S6Base2DTabArc::Trace()
{
	L3SelectionSousImage(4);
	L3Couleur(_couleur);

	int codel3[8];

	codel3[0] = 8;
	codel3[1] = CODEL3ARC;

	for(register unsigned i=0;i<_nb;i++)
	{
	    L3ConversionCoordonneesLog(_x[i],_y[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x2[i],_y2[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x3[i],_y3[i],
			    &codel3[6],&codel3[7],LINEAIRE,LINEAIRE);

	/*if(L3ClippingArc(codel3[2],
			    codel3[3],codel3[4],codel3[5],
			    codel3[6],codel3[7],&codel3[2]))
	    {
		L3Couleur(_couleur);
		L3AppelDrivers(codel3);
	    }*/

	}

}

void
S6Base2DTabArc::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = ymin = Bmax,xmax = ymax = -Bmax;

	register unsigned nb = _nb;

	for(register unsigned i=0;i<nb;i++)
	{
	    xmin = min(xmin,min(REEL(_x[i]),min(REEL(_x2[i]),REEL(_x3[i]))));
	    xmax = max(xmax,max(REEL(_x[i]),max(REEL(_x2[i]),REEL(_x3[i]))));
	    ymin = min(ymin,min(REEL(_y[i]),min(REEL(_y2[i]),REEL(_y3[i]))));
	    ymax = max(ymax,max(REEL(_y[i]),max(REEL(_y2[i]),REEL(_y3[i]))));
	}

}

// ************************************************************
//   Rectangle
// ************************************************************

S6Base2DRectangle::S6Base2DRectangle() : S6Base2DSegment()
{
}

S6Base2DRectangle::S6Base2DRectangle(const S6Base2DRectangle& b) :
		S6Base2DSegment(b)
{
}

S6Base2DRectangle::S6Base2DRectangle(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
		unsigned couleur) :
		S6Base2DSegment(x1,y1,x2,y2,couleur)
{
}

void
S6Base2DRectangle::Trace()
{
	L3Couleur(_couleur);
	L3SelectionSousImage(4);

	int codel3[6];

	codel3[0] = 6;
	codel3[1] = CODEL3SEGMENT;
	L3ConversionCoordonneesLog(_x1,_y1,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x1,_y2,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	if(L3ClippingSegment(codel3[2],
			codel3[3],codel3[4],codel3[5],&codel3[2]))
	{
	    L3AppelDrivers(codel3);
	}

	L3ConversionCoordonneesLog(_x1,_y1,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x2,_y1,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	if(L3ClippingSegment(codel3[2],
			codel3[3],codel3[4],codel3[5],&codel3[2]))
	{
	    L3AppelDrivers(codel3);
	}

	L3ConversionCoordonneesLog(_x2,_y2,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x2,_y1,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	if(L3ClippingSegment(codel3[2],
			codel3[3],codel3[4],codel3[5],&codel3[2]))
	{
	    L3AppelDrivers(codel3);
	}

	L3ConversionCoordonneesLog(_x2,_y2,
			&codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	L3ConversionCoordonneesLog(_x1,_y2,
			&codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	if(L3ClippingSegment(codel3[2],
			codel3[3],codel3[4],codel3[5],&codel3[2]))
	{
	    L3AppelDrivers(codel3);
	}

}

// ************************************************************
//   Tableau de rectangles
// ************************************************************

S6Base2DTabRectangle::S6Base2DTabRectangle() : S6Base2DTabSegment()
{
}

S6Base2DTabRectangle::S6Base2DTabRectangle(const S6Base2DTabRectangle& b) :
		S6Base2DTabSegment(b)
{
}

S6Base2DTabRectangle::S6Base2DTabRectangle(REEL_S* x1,REEL_S* y1,
		REEL_S* x2,REEL_S* y2,unsigned nb,
		unsigned couleur) : S6Base2DTabSegment(x1,y1,x2,y2,nb,couleur)
{
}

void
S6Base2DTabRectangle::Trace()
{
	L3SelectionSousImage(4);
	L3Couleur(_couleur);

	int codel3[6];

	codel3[0] = 6;
	codel3[1] = CODEL3SEGMENT;

	for(register unsigned i=0;i<_nb;i++)
	{
	    L3ConversionCoordonneesLog(_x[i],_y[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x[i],_y2[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	    if(L3ClippingSegment(codel3[2],
			    codel3[3],codel3[4],codel3[5],&codel3[2]))
	    {
		L3Couleur(_couleur);
		L3AppelDrivers(codel3);
	    }

	    L3ConversionCoordonneesLog(_x[i],_y[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x2[i],_y[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	    if(L3ClippingSegment(codel3[2],
			    codel3[3],codel3[4],codel3[5],&codel3[2]))
	    {
		L3AppelDrivers(codel3);
	    }

	    L3ConversionCoordonneesLog(_x2[i],_y2[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x2[i],_y[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	    if(L3ClippingSegment(codel3[2],
			    codel3[3],codel3[4],codel3[5],&codel3[2]))
	    {
		L3AppelDrivers(codel3);
	    }

	    L3ConversionCoordonneesLog(_x2[i],_y2[i],
			    &codel3[2],&codel3[3],LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(_x[i],_y2[i],
			    &codel3[4],&codel3[5],LINEAIRE,LINEAIRE);

	    if(L3ClippingSegment(codel3[2],
			    codel3[3],codel3[4],codel3[5],&codel3[2]))
	    {
		L3AppelDrivers(codel3);
	    }

	}

}

// ************************************************************
//   Polyline
// ************************************************************

S6Base2DPolyline::S6Base2DPolyline() : S6Base2D()
{
	_x = NULL;
	_y = NULL;
	_nb = 0;
}

S6Base2DPolyline::S6Base2DPolyline(const S6Base2DPolyline& b) :
		S6Base2D(b)
{
	_nb = b._nb;
	_x = new REEL_S[_nb];
	_y = new REEL_S[_nb];

	if(!_x || !_y)
	{
	    cerr << "pb alloc S6Base2DPolyline" << endl;
	    _x = _y = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x,b._x,_nb*sizeof(REEL_S));
	memcpy(_y,b._y,_nb*sizeof(REEL_S));
}

S6Base2DPolyline::S6Base2DPolyline(REEL_S* x,REEL_S* y,unsigned nb,
		unsigned couleur) :
		S6Base2D(couleur)
{
	_nb = nb;
	_x = new REEL_S[_nb];
	_y = new REEL_S[_nb];

	if(!_x || !_y)
	{
	    cerr << "pb alloc S6Base2DPolyline" << endl;
	    delete[] _x; delete[] _y;
	    _x = _y = NULL;
	    _nb = 0;

	    return;
	}

	memcpy(_x,x,_nb*sizeof(REEL_S));
	memcpy(_y,y,_nb*sizeof(REEL_S));
}

S6Base2DPolyline::~S6Base2DPolyline()
{
	delete[] _x;
	delete[] _y;
}

void
S6Base2DPolyline::Trace()
{
	REEL* x = new REEL[_nb];
	REEL* y = new REEL[_nb];

	if(!x || !y)
	{
	    delete[] x;
	    delete[] y;

	    cerr << "Pb alloc " << __FILE__ << " ligne " << __LINE__ << endl;

	    return;
	}

	for(register unsigned i=0;i<_nb;i++)
	{
	    x[i] = _x[i];
	    y[i] = _y[i];
	}

	L3Couleur(_couleur);
	L3PointCourant(4,*x,*y);
	L3LigneBrisee(4,x,y,_nb);

	delete[] x; delete[] y;
}

void
S6Base2DPolyline::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = ymin = Bmax,xmax = ymax = -Bmax;

	register unsigned nb = _nb;

	for(register unsigned i=0;i<nb;i++)
	{
	    xmin = min(xmin,REEL(_x[i]));
	    xmax = max(xmax,REEL(_x[i]));
	    ymin = min(ymin,REEL(_y[i]));
	    ymax = max(ymax,REEL(_y[i]));
	}

}

// ************************************************************
//   Polygone
// ************************************************************

S6Base2DPolygone::S6Base2DPolygone() : S6Base2DPolyline()
{
}

S6Base2DPolygone::S6Base2DPolygone(const S6Base2DPolygone& b) :
		S6Base2DPolyline(b)
{
}

S6Base2DPolygone::S6Base2DPolygone(REEL_S* x,REEL_S* y,unsigned nb,
		unsigned couleur) :
		S6Base2DPolyline(x,y,nb,couleur)
{
}

void
S6Base2DPolygone::Trace()
{
	REEL* x = new REEL[_nb];
	REEL* y = new REEL[_nb];

	if(!x || !y)
	{
	    delete[] x;
	    delete[] y;

	    cerr << "Pb alloc " << __FILE__ << " ligne " << __LINE__ << endl;

	    return;
	}

	for(register unsigned i=0;i<_nb;i++)
	{
	    x[i] = _x[i];
	    y[i] = _y[i];
	}

	L3Couleur(_couleur);
	L3Polygone(4,x,y,_nb);

	delete[] x; delete[] y;
}

// ************************************************************
//   Polygone contoured
// ************************************************************

S6Base2DPolygonecont::S6Base2DPolygonecont() : S6Base2DPolyline()
{
}

S6Base2DPolygonecont::S6Base2DPolygonecont(const S6Base2DPolygonecont& b) :
		S6Base2DPolyline(b)
{
	_couleurcont = b._couleurcont;
}

S6Base2DPolygonecont::S6Base2DPolygonecont(REEL_S* x,REEL_S* y,unsigned nb,
		unsigned couleur,unsigned couleurcont) :
		S6Base2DPolyline(x,y,nb,couleur)
{
	_couleurcont = couleurcont;
}

void
S6Base2DPolygonecont::Trace()
{
	REEL* x = new REEL[_nb];
	REEL* y = new REEL[_nb];

	if(!x || !y)
	{
	    delete[] x;
	    delete[] y;

	    cerr << "Pb alloc " << __FILE__ << " ligne " << __LINE__ << endl;

	    return;
	}

	for(register unsigned i=0;i<_nb;i++)
	{
	    x[i] = _x[i];
	    y[i] = _y[i];
	}

	L3Couleur(_couleur);
	L3Polygone(4,x,y,_nb);
	L3Couleur(_couleurcont);
	L3PointCourant(4,x[_nb-1],y[_nb-1]);
	L3LigneBrisee(4,x,y,_nb);

	delete[] x; delete[] y;
}

// ************************************************************
//   Texte
// ************************************************************

S6Base2DTexte::S6Base2DTexte() : S6Base2DPoint()
{
	_texte = NULL;
}

S6Base2DTexte::S6Base2DTexte(const S6Base2DTexte& t) : S6Base2DPoint(t)
{
	register unsigned ll = strlen(t._texte) + 1;

	_texte = new char[ll];

	if(!_texte)
	{
	    cerr << "pb alloc S6Base2DTexte" << endl;
	    return;
	}

	memcpy(_texte,t._texte,ll);
}

S6Base2DTexte::S6Base2DTexte(const char* const texte,REEL_S x,REEL_S y,
		unsigned couleur) :
		S6Base2DPoint(x,y,couleur)
{
	register unsigned ll = strlen(texte) + 1;

	_texte = new char[ll];

	if(!_texte)
	{
	    cerr << "pb alloc S6Base2DTexte" << endl;
	    return;
	}

	memcpy(_texte,texte,ll);
}

S6Base2DTexte::~S6Base2DTexte()
{
	delete[] _texte;
}

void
S6Base2DTexte::Trace()
{
}

void
S6Base2DTexte::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = _x1-1.;
	xmax = _x1+1.;
	ymin = _y1-1.;
	ymax = _y1+1.;
}

// ************************************************************
//   Objets composes
// ************************************************************

#if !defined(P4VERSION_AVEC_TEMPLATE) /* version sans template */
#   define GEN ptrS6Base2D
#   define TabGEN TabptrS6Base2D
#   define TabGENbuf TabptrS6Base2Dbuf
#   include "dinosaure/P4tabgen.c.h"
#   undef GEN
#   undef TabGEN
#   undef TabGENbuf

#   define GEN ptrS6ContBase2D
#   define TabGEN TabptrS6ContBase2D
#   define TabGENbuf TabptrS6ContBase2Dbuf
#   include "dinosaure/P4tabgen.c.h"
#   undef GEN
#   undef TabGEN
#   undef TabGENbuf
#elif defined(__GNUC__)
#   include "dinosaure/P4tabgen.c.h"

    template class TabptrS6Base2D;
    template class TabptrS6Base2Dbuf;
    template class TabptrS6ContBase2D;
    template class TabptrS6ContBase2Dbuf;
#endif // fin version sans template

// ensemble de pointeurs sur Base2D

S6ContBase2D::~S6ContBase2D()
{

	for(register unsigned i=0;i<_taille;i++)
	{
	    delete _tab[i];
	}

}

void
S6ContBase2D::Trace()
{

	for(register unsigned i=0;i<_taille;i++)
	{
	    _tab[i]->Trace();
	}

}

void
S6ContBase2D::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = ymin = Bmax,xmax = ymax = -Bmax;

	for(register unsigned i=0;i<_taille;i++)
	{
	    REEL xmint,xmaxt,ymint,ymaxt;
   
	    _tab[i]->Bornes(xmint,xmaxt,ymint,ymaxt);
	    xmin = min(xmint,xmin);
	    xmax = max(xmaxt,xmax);
	    ymin = min(ymint,ymin);
	    ymax = max(ymaxt,ymax);
	}

}

// ensemble de pointeurs sur ensemble de pointeurs sur Base2D

S6ContContBase2D::~S6ContContBase2D()
{

	if(_desalloue)
	{

	    for(register unsigned i=0;i<_taille;i++)
	    {
		delete _tab[i];
	    }

	}

}

void
S6ContContBase2D::Trace(P4Boolean trace_last)
{
	int xmin_old,ymin_old,xmax_old,ymax_old;
	P4Boolean tlb = P4Boolean(_trace_last && trace_last && _taille);

	if(tlb)
	{
	    L3SelectionSousImage(4);
	    L3ConversionCoordonneesLog(0,0,&xmin_old,&ymin_old,
			    LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(1,1,&xmax_old,&ymax_old,
			    LINEAIRE,LINEAIRE);
	}

	REEL xming = 14./99.,yming = 14./99.;
	REEL xmaxg = 89./99.,ymaxg = 89./99.;

	if(boite_cur_x_min != 0. || boite_cur_y_min != 0. ||
			boite_cur_x_max != 0. || boite_cur_y_max != 0.)
	{
	    xming = boite_cur_x_min;
	    xmaxg = boite_cur_x_max;
	    yming = boite_cur_y_min;
	    ymaxg = boite_cur_y_max;
	}

	if(zoom_gra_x_min != 0. || zoom_gra_x_max != 0. ||
			zoom_gra_y_min != 0. || zoom_gra_y_max != 0.)
	{
	    xming = xming*(zoom_gra_x_max-zoom_gra_x_min) + zoom_gra_x_min;
	    xmaxg = xmaxg*(zoom_gra_x_max-zoom_gra_x_min) + zoom_gra_x_min;
	    yming = yming*(zoom_gra_y_max-zoom_gra_y_min) + zoom_gra_y_min;
	    ymaxg = ymaxg*(zoom_gra_y_max-zoom_gra_y_min) + zoom_gra_y_min;
	}

	if(zoom_actif == 0)
	{

	    if(axe_x_min == 0. && axe_x_max == 0. &&
			    axe_y_min == 0. && axe_y_max == 0.)
	    {
		Bornes(axe_x_min_reel,axe_x_max_reel,axe_y_min_reel,
				axe_y_max_reel);
	    }
	    else
	    {

		if(axe_x_min != 0. || axe_x_max != 0.)
		{
		    axe_x_min_reel = axe_x_min;
		    axe_x_max_reel = axe_x_max;
		}

		if(axe_y_min != 0. || axe_y_max != 0.)
		{
		    axe_y_min_reel = axe_y_min;
		    axe_y_max_reel = axe_y_max;
		}

	    }

	}
	else
	{
	    REEL xpos3i = min(zoom_cur_x_min,zoom_cur_x_max);
	    REEL xpos3a = max(zoom_cur_x_min,zoom_cur_x_max);
	    REEL ypos3i = min(zoom_cur_y_min,zoom_cur_y_max);
	    REEL ypos3a = max(zoom_cur_y_min,zoom_cur_y_max);

	    REEL x1a = S6FlagsSup::axe_x_min_reel_anc;
	    REEL x2a = S6FlagsSup::axe_x_max_reel_anc;
	    REEL y1a = S6FlagsSup::axe_y_min_reel_anc;
	    REEL y2a = S6FlagsSup::axe_y_max_reel_anc;

	    if(x1a == 0. && x2a == 0. && y1a == 0. && y2a == 0.)
	    {
		Bornes(x1a,x2a,y1a,y2a);
	    }

	    P4Vrbor(x1a,x2a,7.,FALSE,0);
	    P4Vrbor(y1a,y2a,7.,FALSE,0);

	    axe_x_min_reel = (x2a-x1a)/(xmaxg-xming)*(xpos3i-xming)+x1a;
	    axe_x_max_reel = (x2a-x1a)/(xmaxg-xming)*(xpos3a-xming)+x1a;
	    axe_y_min_reel = (y2a-y1a)/(ymaxg-yming)*(ypos3i-yming)+y1a;
	    axe_y_max_reel = (y2a-y1a)/(ymaxg-yming)*(ypos3a-yming)+y1a;
	}

	P4Vrbor(axe_x_min_reel,axe_x_max_reel,7.,FALSE,0);
	P4Vrbor(axe_y_min_reel,axe_y_max_reel,7.,FALSE,0);

	if(!tlb)
	{
	    S6Trace::Init();
	}

	L3MiseEnPage(4,xming,yming,xmaxg,ymaxg,0.,0.,1.,1.);
	L3DefSpace(4,axe_x_min_reel,axe_y_min_reel,axe_x_max_reel,
			axe_y_max_reel);

	if(tlb)
	{
	    int xmin,ymin,xmax,ymax;

	    L3ConversionCoordonneesLog(0,0,&xmin,&ymin,LINEAIRE,LINEAIRE);
	    L3ConversionCoordonneesLog(1,1,&xmax,&ymax,LINEAIRE,LINEAIRE);

	    if(xmin != xmin_old || xmax != xmax_old ||
			    ymin != ymin_old || ymax != ymax_old)
	    {
		tlb = FALSE;
		S6Trace::Init();

		L3MiseEnPage(4,xming,yming,xmaxg,ymaxg,0.,0.,1.,1.);
		L3DefSpace(4,axe_x_min_reel,axe_y_min_reel,axe_x_max_reel,
			axe_y_max_reel);
	    }

	}

	if(!tlb)
	{

	    for(register unsigned i=0;i<_taille;i++)
	    {
		register S6ContBase2D* const scb = _tab[i];

		if(scb) scb->Trace();

	    }

	}
	else
	{
	    register S6ContBase2D* const scb = _tab[_last];

	    if(scb) scb->Trace();

	}

	L3FlushBuffers();
}

void
S6ContContBase2D::Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax)
{
	xmin = ymin = Bmax,xmax = ymax = -Bmax;

	for(register unsigned i=0;i<_taille;i++)
	{
   	    register S6ContBase2D* const scb = _tab[i];

	    if(scb)
	    {
		REEL xmint,xmaxt,ymint,ymaxt;

		scb->Bornes(xmint,xmaxt,ymint,ymaxt);
		xmin = min(xmint,xmin);
		xmax = max(xmaxt,xmax);
		ymin = min(ymint,ymin);
		ymax = max(ymaxt,ymax);
	    }

	}

}
