#ifndef S6BASE_H
#define S6BASE_H

// %W% %G% P. Brochard

// *************************************************************
//   gestion des objets graphiques de base de Psyche
// *************************************************************

#include "P4types.h"

// objet de base virtuel pur

class S6Base2D
{
    public:

        S6Base2D() {_couleur = 0;};
        S6Base2D(const S6Base2D& b) {_couleur = b._couleur;};
        S6Base2D(unsigned couleur) {_couleur = couleur;};
        virtual ~S6Base2D() {};

	virtual void Trace() = 0;
	virtual void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax) = 0;

    protected:

	unsigned _couleur;
};

// objets elementaires

class S6Base2DPoint : public S6Base2D
{
    public:

        S6Base2DPoint();
        S6Base2DPoint(const S6Base2DPoint&);
        S6Base2DPoint(REEL_S x,REEL_S y,unsigned couleur);
        ~S6Base2DPoint() {};

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S _x1;
	REEL_S _y1;
};

class S6Base2DSegment : public S6Base2DPoint
{
    public:

        S6Base2DSegment();
        S6Base2DSegment(const S6Base2DSegment&);
        S6Base2DSegment(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
			unsigned couleur);
	~S6Base2DSegment() {};

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S _x2;
	REEL_S _y2;
};

class S6Base2DArc : public S6Base2DSegment
{
    public:

        S6Base2DArc();
        S6Base2DArc(const S6Base2DArc&);
        S6Base2DArc(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
			REEL_S x3,REEL_S y3,unsigned couleur);
	~S6Base2DArc() {};

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S _x3;
	REEL_S _y3;
};

class S6Base2DRectangle : public S6Base2DSegment
{
    public:

        S6Base2DRectangle();
        S6Base2DRectangle(const S6Base2DRectangle&);
        S6Base2DRectangle(REEL_S x1,REEL_S y1,REEL_S x2,REEL_S y2,
			unsigned couleur);
	~S6Base2DRectangle() {};

	void Trace();
};

class S6Base2DPolyline : public S6Base2D
{
    public:

        S6Base2DPolyline();
        S6Base2DPolyline(const S6Base2DPolyline&);
        S6Base2DPolyline(REEL_S* x,REEL_S* y,unsigned nb,unsigned couleur);
	~S6Base2DPolyline();

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S* _x;
	REEL_S* _y;
	unsigned _nb;
};

class S6Base2DPolygone : public S6Base2DPolyline
{
    public:

        S6Base2DPolygone();
        S6Base2DPolygone(const S6Base2DPolygone&);
        S6Base2DPolygone(REEL_S* x,REEL_S* y,unsigned nb,unsigned couleur);
	~S6Base2DPolygone() {};

	void Trace();
};

class S6Base2DPolygonecont : public S6Base2DPolyline
{
    public:

        S6Base2DPolygonecont();
        S6Base2DPolygonecont(const S6Base2DPolygonecont&);
        S6Base2DPolygonecont(REEL_S* x,REEL_S* y,unsigned nb,unsigned couleur,
			unsigned couleurcont);
	~S6Base2DPolygonecont() {};

	void Trace();

    private:

	unsigned _couleurcont;
};

class S6Base2DTexte : public S6Base2DPoint
{
    public:

        S6Base2DTexte();
        S6Base2DTexte(const S6Base2DTexte&);
        S6Base2DTexte(const char* const texte,REEL_S x,REEL_S y,
			unsigned couleur);
	~S6Base2DTexte();

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    private:

	char* _texte;
};

class S6Base2DTabPoint : public S6Base2DPolyline
{
    public:

        S6Base2DTabPoint();
        S6Base2DTabPoint(const S6Base2DTabPoint&);
        S6Base2DTabPoint(REEL_S* x,REEL_S* y,unsigned nb,unsigned couleur);
        ~S6Base2DTabPoint();

	void Trace();
};

class S6Base2DTabSegment : public S6Base2DTabPoint
{
    public:

        S6Base2DTabSegment();
        S6Base2DTabSegment(const S6Base2DTabSegment&);
        S6Base2DTabSegment(REEL_S* x1,REEL_S* y1,
			REEL_S* x2,REEL_S* y2,unsigned nb,unsigned couleur);
        ~S6Base2DTabSegment() {};

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S* _x2;
	REEL_S* _y2;
};

class S6Base2DTabArc : public S6Base2DTabSegment
{
    public:

        S6Base2DTabArc();
        S6Base2DTabArc(const S6Base2DTabArc&);
        S6Base2DTabArc(REEL_S* x1,REEL_S* y1,
			REEL_S* x2,REEL_S* y2,
			REEL_S* x3,REEL_S* y3,unsigned nb,unsigned couleur);
        ~S6Base2DTabArc() {};

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

    protected:

	REEL_S* _x3;
	REEL_S* _y3;
};

class S6Base2DTabRectangle : public S6Base2DTabSegment
{
    public:

        S6Base2DTabRectangle();
        S6Base2DTabRectangle(const S6Base2DTabRectangle&);
        S6Base2DTabRectangle(REEL_S* x1,REEL_S* y1,
			REEL_S* x2,REEL_S* y2,unsigned nb,unsigned couleur);
        ~S6Base2DTabRectangle() {};

	void Trace();
};

// objets composes

class S6ContBase2D;

typedef S6Base2D* ptrS6Base2D;
typedef S6ContBase2D* ptrS6ContBase2D;

#include "dinosaure/P4tabgenbasec.h"

#define INCR22

#if defined(P4VERSION_AVEC_TEMPLATE) /* version avec template */

#   if P4VERSION_AVEC_TEMPLATE == 4
        typedef TabGEN<ptrS6Base2D> TabptrS6Base2D;
        typedef TabGENbuf<ptrS6Base2D> TabptrS6Base2Dbuf;
        typedef TabGEN<ptrS6ContBase2D> TabptrS6ContBase2D;
        typedef TabGENbuf<ptrS6ContBase2D> TabptrS6ContBase2Dbuf;
#   else
#       define TabptrS6Base2D TabGEN<ptrS6Base2D>
#       define TabptrS6Base2Dbuf TabGENbuf<ptrS6Base2D>
#       define TabptrS6ContBase2D TabGEN<ptrS6ContBase2D>
#       define TabptrS6ContBase2Dbuf TabGENbuf<ptrS6ContBase2D>
#   endif

#else // version sans template
#   define GEN ptrS6Base2D
#   define TabGEN TabptrS6Base2D
#   define TabGENbuf TabptrS6Base2Dbuf
#   include "dinosaure/P4tabgen.h"
#   undef GEN
#   undef TabGEN
#   undef TabGENbuf

#   define GEN ptrS6ContBase2D
#   define TabGEN TabptrS6ContBase2D
#   define TabGENbuf TabptrS6ContBase2Dbuf
#   include "dinosaure/P4tabgen.h"
#   undef GEN
#   undef TabGEN
#   undef TabGENbuf
#endif // fin version sans template

#undef INCR22

class S6ContBase2D : public TabptrS6Base2D
{
    public:

	S6ContBase2D();
	~S6ContBase2D();

	void Trace();
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);
};

class S6ContContBase2D : public TabptrS6ContBase2D
{
    public:

	S6ContContBase2D();
	S6ContContBase2D(const S6ContContBase2D& tc);
	~S6ContContBase2D();

	void Trace(P4Boolean trace_last = FALSE);
	void Bornes(REEL& xmin,REEL& xmax,REEL& ymin,REEL& ymax);

        P4Boolean TraceLast() const;
        void TraceLast(P4Boolean tlb);
        void Last(unsigned lb);

    private:

	P4Boolean _desalloue;
	P4Boolean _trace_last;
	unsigned _last;
};

inline
S6ContBase2D::S6ContBase2D() : TabptrS6Base2D()
{
}

inline
S6ContContBase2D::S6ContContBase2D() :
		TabptrS6ContBase2D(),
		_desalloue(TRUE),
		_trace_last(FALSE),
		_last(0)
{
}

inline
S6ContContBase2D::S6ContContBase2D(const S6ContContBase2D& tc) :
		TabptrS6ContBase2D(tc),
		_desalloue(FALSE),
		_trace_last(tc._trace_last),
		_last(tc._last)
{
}

inline P4Boolean
S6ContContBase2D::TraceLast() const
{
	return _trace_last;
}

inline void
S6ContContBase2D::TraceLast(P4Boolean tlb)
{
	_trace_last = tlb;
}

inline void
S6ContContBase2D::Last(unsigned lb)
{
	_last = lb;
}

#endif /* S6BASE_H */
