// ******************************************************************
//   Identification
// ******************************************************************

struct SccsIdS6Boutgen
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Boutgen::sccs_id = "%W% %G% Pierre Brochard";

// ******************************************************************
//   Includes
// ******************************************************************

#include "S6boutgen.h"
#include "poulpe/U8affx.h"
#include "S6traduc.h"
#include "U8constant.h"

// ******************************************************************
//   classes de bouttons generaux independants du toolkit
// ******************************************************************

// ********************************************************
// choix verticaux ........................................
// ********************************************************

ChoixVertical::ChoixVertical(const Widget& parent, const char* name,
		const char* label,const char** const val,int taille,
		int nbcol,int col) :
		ConteneurColonne(parent,name,col),
		_val(_widgetid,label,"MENTP",nbcol),
		_lab(_widgetid,val[0]),
		_bou(new ChoixVerticalPDE[taille]),
		_taille(taille),
		_choix(0)
{

	if(_bou)
	{

	    for(int i=0;i<taille;i++)
	    {
		_bou[i].Init(_val.GetId(),val[i],i);
		_bou[i].Attach(this);
	    }

	}
	else
	    _taille = 0;

}

ChoixVertical::~ChoixVertical()
{
	delete[] _bou; 
}

void
ChoixVertical::operator()(const char** val,int taille,int nbcol,int col)
{
	_val.Lignes(nbcol);
	Colonnes(col);

	delete[] _bou; 

	_taille = taille;
	_choix = _choix >= taille ? taille-1 : _choix;
	_bou = new ChoixVerticalPDE[taille];

	for(int i=0;i<taille;i++)
	{
	    _bou[i].Init(_val.GetId(),val[i],i);
	    _bou[i].Attach(this);
	}

}

void
ChoixVertical::Couleur(int taille)
{

	if(taille == int(_taille)) return;

	if(taille == 64)
	{
	    const char** val = S6Traduc::couleur;

	    (*this)(val,taille,taille/8,1);
	}
	else
	{
	    char** val = new char*[taille];
	    register int i;

	    for(i=0;i<taille;i++)
	    {
		ostrstream os;

		os << i << ends;

		val[i] = os.str();
	    }

	    (*this)(val,taille,taille/8,1);

	    for(i=0;i<taille;i++)
	    {
		delete[] val[i];
	    }

	    delete[] val;
	}

	Couleur();
}

void
ChoixVertical::Couleur()
{
	Pixel* pix = new Pixel[_taille];
	int nb_pix_used = 0;

	U8AfficheurX* const uax = U8AfficheurX::Driver();

	if(uax)
	{
	    uax->l3_init_pix(_taille,2,NULL,NULL);
	    uax->l3_donne_pix(pix,_taille,&nb_pix_used);
	}

	Couleur(pix,nb_pix_used);

	delete[] pix;
}

void
ChoixVertical::Couleur(Pixel pix[],unsigned nb_pix)
{

	if(nb_pix != _taille)
	{
	   cout << " warning : " << __FILE__ << " line " << __LINE__ <<
		" nb_pix != _taille" << endl;
	}

	unsigned taille = nb_pix > _taille ? _taille : nb_pix;

	for(unsigned i=0;i<taille;i++)
	    _bou[i].Couleur(pix[i]);

}

void
ChoixVertical::Change(int choix)
{
	Choix(choix);
	ChangeLabel(_bou[choix].Lab());
}

// boutons pour choix verticaux

void
ChoixVerticalPDE::SelectCB(XtPointer call_data)
{
	PullDownEntry::SelectCB(call_data);

	if(_pto)
	{
	    _pto->ChangeLabel(_label);
	    _pto->Choix(_choix);
	    _pto->ChargeInv();
	}

}

// *************************************************************
//     choix verticaux ou le meme panneau peut etre commun
//     a plusieurs cascade boutons
// *************************************************************

// boutons pour choix verticaux panel

void
ChoixVerticalPanelPDE::SelectCB(XtPointer call_data)
{
	PullDownEntry::SelectCB(call_data);

	if(_pto)
	{
	    _pto->ChangeLabel(_label);
	    _pto->Choix(_choix);
	    _pto->ChargeInv();
	}

}

// panel pour ChoixVerticalCascade

ChoixVerticalPanel::ChoixVerticalPanel(const Widget& parent, 
		const char** const val,int taille,int nbcol) :
		PullDownTopPanel(parent,"MENTP",nbcol)
{
	_taille = taille;
	_bou = new ChoixVerticalPanelPDE[taille];
	_iscolor = 0;

	for(int i=0;i<taille;i++)
	    _bou[i].Init(GetId(),val[i],i);

}

ChoixVerticalPanel::~ChoixVerticalPanel()
{
	delete[] _bou; 
}

char*
ChoixVerticalPanel::Label(int choix)
{

	if(!_taille)
	    return NULL;
	else
	{

	    if(choix < 0) choix = 0;
	
	    if(choix >= int(_taille)) choix = _taille-1;
	
	    return _bou[choix].Lab();
	}
	  
}

void
ChoixVerticalPanel::Attach(ChoixVerticalCascade* pto)
{

	for(unsigned i=0;i<_taille;i++)
	    _bou[i].Attach(pto);

        if(_taille && _iscolor)
	  Couleur();

}

void
ChoixVerticalPanel::operator()(const char** val,int taille,int nbcol)
{
	Lignes(nbcol);

	ChoixVerticalPanelPDE* const tbou = _bou;

	_taille = taille;
	_bou = new ChoixVerticalPanelPDE[taille];

	for(int i=0;i<taille;i++)
	    _bou[i].Init(GetId(),val[i],i);

	delete[] tbou; 
}

void
ChoixVerticalPanel::Couleur(int taille)
{

	if(taille == int(_taille)) return;

	if(taille == 64)
	{
	    const char** val = S6Traduc::couleur;

	    (*this)(val,taille,taille/8);
	}
	else
	{
	    char** val = new char*[taille];
	    register int i;

	    for(i=0;i<taille;i++)
	    {
		ostrstream os;

		os << i << ends;
		val[i] = os.str();
	    }

	    (*this)(val,taille,taille/8);

	    for(i=0;i<taille;i++)
		delete[] val[i];

	    delete[] val;
	}

	Couleur();
}

void
ChoixVerticalPanel::Couleur()
{
	Pixel* pix = new Pixel[_taille];
	int nb_pix_used = 0;

	U8AfficheurX* const uax = U8AfficheurX::Driver();

	if(uax)
	{
	    uax->l3_init_pix(_taille,2,NULL,NULL);
	    uax->l3_donne_pix(pix,_taille,&nb_pix_used);
	}

	Couleur(pix,nb_pix_used);

	delete[] pix;
	_iscolor = 1;
}

void
ChoixVerticalPanel::Couleur(Pixel pix[],unsigned nb_pix)
{

	if(nb_pix != _taille)
	{
	   cout << " warning : " << __FILE__ << " line " << __LINE__ <<
		" nb_pix != _taille" << endl;
	}

	unsigned taille = nb_pix > _taille ? _taille : nb_pix;

	for(unsigned i=0;i<taille;i++)
	    _bou[i].Couleur(pix[i]);

}

// choix verticaux avec panel venant de l'exterieur....................

ChoixVerticalCascade::ChoixVerticalCascade(const Widget& parent,
		ChoixVerticalPanel& panel,const char* const name,
		const char* const label,const int col,PopUp* const popupbloque) :
		ConteneurColonne(parent,name,col),
		_pdtc(_widgetid,panel.GetId(),label,"MENTP"),
		_lab(_widgetid,""),
		_panel(panel),
		_popupbloque(popupbloque),
		_choix(0)
{
	_pdtc.Attach(this);
	ChangeLabel(_panel.Label(_choix));
}

ChoixVerticalCascade::~ChoixVerticalCascade()
{
}

void
ChoixVerticalCascade::ChangeLabel(const char* const label)
{

	if(_popupbloque)
	    _popupbloque->LibereGeom();

	_lab.Change(label);

	if(_popupbloque)
	    _popupbloque->BloqueGeom();

}

void
ChoixVerticalCascade::Attach()
{
	_panel.Attach(this);
}

void
ChoixVerticalPullDownTopCascade::SelectCB(XtPointer)
{

	if(_pto) _pto->Attach();

	Montre();
}

void
ChoixVerticalCascade::Change(int choix)
{

	if(choix == Choix()) return;

	Choix(choix);
	ChangeLabel(_panel.Label(choix));
}

void
ChoixVerticalCascade::Change()
{
	ChangeLabel(_panel.Label(_choix));
}
