// ******************************************************************
//   Identification
// ******************************************************************

struct SccsIdS6Bouttons
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Bouttons::sccs_id = "%W% %G% Pierre Brochard";

// ******************************************************************
//   Includes
// ******************************************************************

#include "S6bouttons.h"
#include "S6constant.h"
#include "nxstream.h"
#include "S6trace.h"
#include "S6gerefenet.h"
#include "S6gerefich.h"
#include "S6points.h"
#include "S6gereenv.h"
#include "S6gereconfig.h"
#include "toto.h"
#include "nx.h"
#include "S6namecomp.h"
#include "S6traduc.h"
#include "S6initcourbe.h"
#include "S6outils.h"
#include <ctype.h>
#include "poulpe/WidAffX.h"
#include "poulpe/U8affx.h"
#include "S6flagssup.h"
#include "dinosaure/S6string.h"
#include <sys/utsname.h>

// ******************************************************************
//		classes bouttons de Psyche
// ******************************************************************

// fonction Quit generale ...................................

void
Quit(int purge_fenet) 
{
	S6Trace::Termine();

	delete S6GereFichier::serveur_binaire;
	delete S6GereFichier::serveur_unix;
	delete S6GereFichier::serveur_ascii;
	delete S6GereFichier::serveur_ucd;

	if(S6GereFichier::pt_points) S6GereFichier::pt_points->Enleve();

	if(S6FlagsSup::pt_points_save) S6FlagsSup::pt_points_save->Enleve();

	delete S6GereFichier::pt_points;S6GereFichier::pt_points = NULL;
	delete S6FlagsSup::pt_points_save;S6FlagsSup::pt_points_save = NULL;

	unlink(S6NameComp::fichier_temp_1);
	unlink(S6NameComp::fichier_temp_2);
	unlink(S6NameComp::fichier_provi_1);
	unlink(S6NameComp::fichier_provi_2);
	unlink(S6NameComp::imp_dest_fichtaille);

	S6GereFenet::PurgeIdent();
	S6GereEnv::PurgeEnv();

	if(purge_fenet)
	{
	    S6GereFenet::PurgeFenet();
	}

	cout << "Quit> Quit trappe" << endl;
}

extern "C"
{
    void
    S6Exit(int status)
    {
    // sur SunOS 5.4 bug sur exit avec std io streams

	struct utsname utsn;

	if(uname(&utsn) >= 0 && !strcmp(utsn.sysname,"SunOS") &&
			!strcmp(utsn.release,"5.4"))
	{
	    _exit(status);
	}
	else
	{
	    exit(status);
	}

    }

}

// fenetre principale de Psyche avec IHM ...............................

RootPsyche::RootPsyche(int argc, char** argv,int choix,const char* my_visual) :
		RootWin(argc,argv,choix,my_visual)
{
}

RootPsyche::~RootPsyche()
{
}

void
RootPsyche::LectureInputCB(int fid,U8InputId)
{

    // gestion des fichiers sockets

	S6GereFichier::LitFichierSocket(fid);
}

// fenetre principale de Psyche sans IHM ...............................

RootPsycheBatch::RootPsycheBatch() : ElementInput()
{
}

RootPsycheBatch::~RootPsycheBatch()
{
}

void
RootPsycheBatch::LectureInputCB(int fid,U8InputId)
{

    // gestion des fichiers sockets

	S6GereFichier::LitFichierSocket(fid);
}

// icone .............................................

void
NewIcone::SelectCB(XtPointer) 
{
	RootWin* root_win = (RootWin*) S6GereFenet::root->ElInput();

	root_win->ChangeIcone((char*) toto_bits,toto_width,toto_height);
}

// bouton quit ................................................

void
QuitMenu::SelectCB(XtPointer) 
{
	//_quit = True;

	Quit();
	S6Exit(0);
}

// calcul image .......................................................

void
CalculImage::SelectCB(XtPointer call_data) 
{
	Bouton::SelectCB(call_data);
	S6Trace::Trace();
}

// Info Psyche .......................................................

void
InfoPsyche::SelectCB(XtPointer call_data) 
{
	Bouton::SelectCB(call_data);

	if(S6GereFenet::bienvenue) S6GereFenet::bienvenue->Montre();

}

// sous menus options 3d ................................................

Options3dChoixAutres::Options3dChoixAutres(const Widget& root,
		const Widget& parent,const char* label,
		const int& mini, const int& maxi,
		const int& ValeurInitiale,Options3D& opt3d) :
		PullDownEntry(parent,label),
		_pop(root,label,100,100),
		_tout(_pop.GetId(),"OPT_3D_TOUT",-1),
		_joli(_tout.GetId()),
		_col(_joli.GetId(),"OPT3D_CONT_COL",-1),
		_sli(_col.GetId(),"",mini,maxi,ValeurInitiale,opt3d)
{
}

Options3D::Options3D(const Widget& root,const Widget& parent,const char* label) :
		_options3d(parent, label,"MENSB_3D_OPTIONS"),

		_pixel(_options3d.GetId(),S6Traduc::traduc[351]),
		_niveau(_options3d.GetId(),S6Traduc::traduc[18]),
		_facette(_options3d.GetId(),S6Traduc::traduc[19]),
		_filaire(_options3d.GetId(),S6Traduc::traduc[20]),
		_lissage(_options3d.GetId(),S6Traduc::traduc[21]),
		_trace_fil(_options3d.GetId(),S6Traduc::traduc[233]),
		_masque(_options3d.GetId(),S6Traduc::traduc[31]),
		_concatene(_options3d.GetId(),S6Traduc::traduc[39],
			"MENSB_3D_CONCATENE"),

		_p3d(_pixel.GetId(),S6Traduc::traduc[305]),
		_p4d(_pixel.GetId(),S6Traduc::traduc[306]),

		_n3d(_niveau.GetId(),S6Traduc::traduc[22]),
		_n4d(_niveau.GetId(),S6Traduc::traduc[23]),

		_f3d(_facette.GetId(),S6Traduc::traduc[24]),
		_f4d(_facette.GetId(),S6Traduc::traduc[25]),

		_fimo(_filaire.GetId(),S6Traduc::traduc[26]),
		_fi3d(_filaire.GetId(),S6Traduc::traduc[27]),
		_fi4d(_filaire.GetId(),S6Traduc::traduc[28]),

		_l3d(_lissage.GetId(),S6Traduc::traduc[29]),
		_l4d(_lissage.GetId(),S6Traduc::traduc[30]),

		_sansm(_masque.GetId(),S6Traduc::traduc[32]),
		_masq0(_masque.GetId(),S6Traduc::traduc[33]),
		_masq1(_masque.GetId(),S6Traduc::traduc[34]),
		_masq2(_masque.GetId(),S6Traduc::traduc[35]),

		_oui(_concatene.GetId(),S6Traduc::traduc[43]),
		_non(_concatene.GetId(),S6Traduc::traduc[44]),
		_fin(_concatene.GetId(),S6Traduc::traduc[313]),
		_autres(root,_concatene.GetId(),S6Traduc::traduc[314],0,100,
			2,*this),

		_trace_x(_trace_fil.GetId(),S6Traduc::traduc[234]),
		_trace_y(_trace_fil.GetId(),S6Traduc::traduc[235]),
		_trace_xy(_trace_fil.GetId(),S6Traduc::traduc[236])
{
	cout << "Options3D> Options 3d cree " << endl;

	_p3d.Attach(this);
	_p4d.Attach(this);

	_n3d.Attach(this);
	_n4d.Attach(this);

	_f3d.Attach(this);
	_f4d.Attach(this);

	_fimo.Attach(this);
	_fi3d.Attach(this);
	_fi4d.Attach(this);

	_l3d.Attach(this);
	_l4d.Attach(this);

	_trace_x.Attach(this);
	_trace_y.Attach(this);
	_trace_xy.Attach(this);

	_sansm.Attach(this);
	_masq0.Attach(this);
	_masq1.Attach(this);
	_masq2.Attach(this);

	_oui.Attach(this);
	_non.Attach(this);

	_sansm.Change(TRUE);
}

int
Options3D::CalculOption3d()
{
	int opt = 0;

	opt = _p4d.IsSelected();
	opt = opt << 1 | _p3d.IsSelected();
	opt = opt << 1 | _l4d.IsSelected();
	opt = opt << 1 | _l3d.IsSelected();
	opt = opt << 1 | _n4d.IsSelected();
	opt = opt << 1 | _n3d.IsSelected();
	opt = opt << 1 | _f4d.IsSelected();
	opt = opt << 1 | _f3d.IsSelected();
	opt = opt << 1 | _fi4d.IsSelected();
	opt = opt << 1 | _fi3d.IsSelected();
	opt = opt << 1 | _fimo.IsSelected();
	opt--;

	option_3d = opt;
	p_option_3d = option_3d;
	S6GereFenet::ReConfigOpt3DPanel();

	cout << "Options3D> opt " << opt << endl;

	return opt;
}

void
Options3D::ChangeConcatene(int& choix)
{
	_non.Change(FALSE);
	_oui.Change(FALSE);
	_autres.Change(0,100,2);

	switch(choix)
	{
	    case 0 :
		_non.Change(TRUE);
		_autres.Change(0,100,0);
		break;
	    case 1 :
		_oui.Change(TRUE);     
		_autres.Change(0,100,1);
		break;
	    default :

		if(choix < 2) choix = 2;

		if(choix > 100) choix = 100;

		_autres.Change(0,100,choix);
		break;
	}

}

void
Options3dChoixP3d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeFimo(0);
	_pto->ChangeFi3d(0);
	_pto->ChangeFi4d(0);
	_pto->ChangeN3d(0);
	_pto->ChangeN4d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeF3d(0);
	_pto->ChangeF4d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixP3d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixP4d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeFimo(0);
	_pto->ChangeFi3d(0);
	_pto->ChangeFi4d(0);
	_pto->ChangeN3d(0);
	_pto->ChangeN4d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeF3d(0);
	_pto->ChangeF4d(0);
	_pto->ChangeP3d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixP4d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixN3d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeN4d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixN3d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixN4d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeN3d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixN4d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixF3d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeF4d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixF3d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixF4d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeF3d(0);
	_pto->ChangeL3d(0);
	_pto->ChangeL4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixF4d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixFimo::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeFi3d(0);
	_pto->ChangeFi4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixFimo::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixFi3d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeFimo(0);
	_pto->ChangeFi4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixFi3d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	_pto->CalculOption3d();
}

void
Options3dChoixFi4d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeFimo(0);
	_pto->ChangeFi3d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixFi4d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixL3d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeL4d(0);
	_pto->ChangeN3d(0);
	_pto->ChangeN4d(0);
	_pto->ChangeF3d(0);
	_pto->ChangeF4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixL3d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dChoixL4d::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeL3d(0);
	_pto->ChangeN3d(0);
	_pto->ChangeN4d(0);
	_pto->ChangeF3d(0);
	_pto->ChangeF4d(0);
	_pto->ChangeP3d(0);
	_pto->ChangeP4d(0);
	_pto->CalculOption3d();
}

void
Options3dChoixL4d::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->CalculOption3d();
}

void
Options3dTraceX::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeTraceY(0);
	_pto->ChangeTraceXY(0);
	option_3d_trace_fil = 0;
}

void
Options3dTraceX::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeTraceX(1);
	_pto->ChangeTraceY(0);
	_pto->ChangeTraceXY(0);
	option_3d_trace_fil = 0;
}

void
Options3dTraceY::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeTraceX(0);
	_pto->ChangeTraceXY(0);
	option_3d_trace_fil = 1;
}

void
Options3dTraceY::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeTraceX(0);
	_pto->ChangeTraceY(1);
	_pto->ChangeTraceXY(0);
	option_3d_trace_fil = 1;
}

void
Options3dTraceXY::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeTraceX(0);
	_pto->ChangeTraceY(0);
	option_3d_trace_fil = 2;
}

void
Options3dTraceXY::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeTraceX(0);
	_pto->ChangeTraceY(0);
	_pto->ChangeTraceXY(1);
	option_3d_trace_fil = 2;
}

void
Options3dChoixSansm::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq1(0);
	_pto->ChangeMasq2(0);
	option_3d_masque = 0;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixSansm::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	_pto->ChangeSansm(1);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq1(0);
	_pto->ChangeMasq2(0);

	option_3d_masque = 0;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq0::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq1(0);
	_pto->ChangeMasq2(0);
	option_3d_masque = 1;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq0::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq0(1);
	_pto->ChangeMasq1(0);
	_pto->ChangeMasq2(0);
	option_3d_masque = 1;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq1::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq2(0);
	option_3d_masque = 2;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq1::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq1(1);
	_pto->ChangeMasq2(0);
	option_3d_masque = 2;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq2::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq1(0);
	option_3d_masque = 3;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixMasq2::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);
	_pto->ChangeSansm(0);
	_pto->ChangeMasq0(0);
	_pto->ChangeMasq1(0);
	_pto->ChangeMasq2(1);
	option_3d_masque = 3;
	p_option_3d_masque = option_3d_masque;
	S6GereFenet::ReConfigOpt3DPanel();
}

void
Options3dChoixOui::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int opt3dc = 1;

	_pto->ChangeConcatene(opt3dc);
	option_3d_concatene = opt3dc;
}

void
Options3dChoixNon::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int opt3dc = 0;

	_pto->ChangeConcatene(opt3dc);
	option_3d_concatene = opt3dc;
}

void
Options3dChoixFin::SelectCB(XtPointer call_data)
{
	PullDownEntry::SelectCB(call_data);

	S6Points* const s6p = S6GereFichier::pt_points;

	s6p->Enleve();
	s6p->NbFichiersConcat(0);
}

void
Options3dChoixAutres::SelectCB(XtPointer call_data)
{
	PullDownEntry::SelectCB(call_data);

	_pop.Montre();
}

void
Options3dChoixAutresSlider::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);

	int opt3dc = GetValue();

	_opt3d.ChangeConcatene(opt3dc);
	option_3d_concatene = opt3dc;
}

// sous menus options 2d ..........................................

Options2D::Options2D(const Widget& parent,const char* label) :
		_options2d(parent, label,"MENSB_2D_OPTIONS"),

		_trace(_options2d.GetId(),S6Traduc::traduc[38],
			"MENSB_2D_TRACE"),
		_concatene(_options2d.GetId(),S6Traduc::traduc[39],
			"MENSB_2D_CONCATENE"),

		_cour(_trace.GetId(),S6Traduc::traduc[40],"BTCX_2d_courbe"),
		_inte(_trace.GetId(),S6Traduc::traduc[41],"BTCX_2d_integrale"),
		_deri(_trace.GetId(),S6Traduc::traduc[42],"BTCX_2d_derivee"),
		_oui(_concatene.GetId(),S6Traduc::traduc[43],"BTCX_2d_oui"),
		_non(_concatene.GetId(),S6Traduc::traduc[44],"BTCX_2d_non")
{
	cout << "Options2D> Options 2d cree " << endl;

	_cour.Attach(this);
	_inte.Attach(this);
	_deri.Attach(this);
	_oui.Attach(this);
	_non.Attach(this);

	_cour.Change(TRUE);
	_non.Change(TRUE);
}

void Options2dChoixCour::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeInte(0);
	_pto->ChangeDeri(0);

	option_2d = 0;
}

void Options2dChoixCour::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	option_2d = -1;
}

void Options2dChoixInte::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeCour(0);
	_pto->ChangeDeri(0);

	option_2d = 1;
}

void Options2dChoixInte::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	option_2d = -1;
}

void Options2dChoixDeri::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeCour(0);
	_pto->ChangeInte(0);

	option_2d = 2;
}

void Options2dChoixDeri::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	option_2d = -1;
}

void Options2dChoixOui::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeNon(0);

	option_2d_concatene = 1;
}

void Options2dChoixOui::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	_pto->ChangeOui(1);
	_pto->ChangeNon(0);

	option_2d_concatene = 1;
}

void Options2dChoixNon::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeOui(0);

	option_2d_concatene = 0;
}

void Options2dChoixNon::UnSelectCB(XtPointer call_data)
{
	BoutonDeChoix::UnSelectCB(call_data);

	_pto->ChangeOui(0);
	_pto->ChangeNon(1);

	option_2d_concatene = 0;
}

// choix verticaux ........................................

void ChoixVerticalCouleurTitres::ChargeInv()
{

	switch(p_titre_numero)
	{

	    case 0 :

		p_titre_couleur_haut = Choix();

		if(!parametre_couleur_fond && !p_titre_couleur_haut)
			p_titre_couleur_haut = 1;

		break;

	    case 1 :

		p_titre_couleur_gauche = Choix();

		if(!parametre_couleur_fond && !p_titre_couleur_gauche)
			p_titre_couleur_gauche = 1;

		break;

	    case 2 :

		p_titre_couleur_droit = Choix();

		if(!parametre_couleur_fond && !p_titre_couleur_droit)
			p_titre_couleur_droit = 1;

		break;

	    case 3 :

		p_titre_couleur_bas = Choix();

		if(!parametre_couleur_fond && !p_titre_couleur_bas)
			p_titre_couleur_bas = 1;

		break;

	}

}

void ChoixVerticalTailleTitres::ChargeInv()
{

	switch(p_titre_numero)
	{

	    case 0 :

		p_titre_taille_haut = max((Choix() + 1) * 8,10);
		break;

	    case 1 :

		p_titre_taille_gauche = max((Choix() + 1) * 8,10);
		break;

	    case 2 :

		p_titre_taille_droit = max((Choix() + 1) * 8,10);
		break;

	    case 3 :

		p_titre_taille_bas = max((Choix() + 1) * 8,10);
		break;

	}

}

// titres ................................................

Titres::Titres(const Widget& root, const Widget& parent,
		const char *label) : PullDownEntry(parent, label),
		_pere(root,S6Traduc::traduc[49],200,100),
		_tout(_pere.GetId(),"CONT_TITRE_TOUT",-1),

		_titre_j(_tout.GetId(),"CONT_TITRE_TITRE_J"),

		_titre(_titre_j.GetId(),"CONT_TITRE_TITRE"),
		_titre_place_en(_titre.GetId(),S6Traduc::traduc[50]),
		_haut(_titre.GetId(),S6Traduc::traduc[52],"BTCX_titre_haut"),
		_gauche(_titre.GetId(),S6Traduc::traduc[53],
				"BTCX_titre_gauche"),
		_droit(_titre.GetId(),S6Traduc::traduc[54],"BTCX_titre_droit"),
		_bas(_titre.GetId(),S6Traduc::traduc[55],"BTCX_titre_bas"),

		_param_j(_tout.GetId(),"CONT_TITRE_PARAM_J"),
		_param_l(_param_j.GetId(),"CONT_TITRE_PARAM_L",1),

		_affichage(_param_l.GetId(),"CONT_TITRE_AFFICHAGE"),
		_label_affichage(_affichage.GetId(),S6Traduc::traduc[51]),
		_non(_affichage.GetId(),S6Traduc::traduc[56],"BTCX_titre_non"),
		_sans(_affichage.GetId(),S6Traduc::traduc[57],
				"BTCX_titre_sans"),
		_avec(_affichage.GetId(),S6Traduc::traduc[58],
				"BTCX_titre_avec"),

		_texte(_param_l.GetId(),"texteti",2),
		_titre_mes(_texte.GetId(),S6Traduc::traduc[69]),
		_titre_valeur(_texte.GetId(),""),

		_params(_param_l.GetId(),"CONT_TITRE_PARAMS"),
		_couleur(_params.GetId(),*(S6GereFenet::couleur),
				"TITRE_COULEUR",S6Traduc::traduc[59]),
		_taille(_params.GetId(),*(S6GereFenet::taille),
				"TITRE_TAILLE",S6Traduc::traduc[70]),

		_standard_j(_tout.GetId(),"TITRE_STANDARD_J"),

		_titre_standard(_standard_j.GetId(),"TITRE_STANDARD",1),
		_titre_ok(_titre_standard.GetId(),
				S6Traduc::traduc[227],"BTN_OAA"),
		_titre_annuler(_titre_standard.GetId(),S6Traduc::traduc[228],
				"BTN_OAA"),
		_titre_aide(_titre_standard.GetId(),S6Traduc::traduc[229],
				"BTN_OAA")
{
	_haut.Attach(this);
	_gauche.Attach(this);
	_droit.Attach(this);
	_bas.Attach(this);

	_non.Attach(this);
	_sans.Attach(this);
	_avec.Attach(this);

	_haut.Change(TRUE);
	_non.Change(TRUE);

	_titre_ok.Attach(this);

	_pere.BloqueGeom();
}

void
Titres::ChangeNumero(int& choix)
{
	_haut.Change(FALSE);
	_gauche.Change(FALSE);
	_bas.Change(FALSE);
	_droit.Change(FALSE);

	switch(choix)
	{
	    case 0 :
		_haut.Change(TRUE);
		break;

	    case 1 :
		_gauche.Change(TRUE);
		break;

	    case 2 :
		_droit.Change(TRUE);
		break;

	    case 3 :
		_bas.Change(TRUE);
		break;

	    default :
		choix = 0;
		_haut.Change(TRUE);
		break;
	}

}

void
Titres::SelectCB(XtPointer call_data)
{
	ModeTrace(S6Output::Output());
	PullDownEntry::SelectCB(call_data);
	_pere.Montre();
}

void
Titres::Change(const char* titre_valeur)
{	
	_titre_valeur.Change(titre_valeur);
}

void
Titres::Charge(int titre_num)
{

    // chargement des valeurs de config dans le popup Titres

	p_titre_numero = titre_num;

	switch(titre_num)
	{

	    case 0 :

		ChangeNon(p_titre_affiche_haut == 0);
		ChangeSans(p_titre_affiche_haut == 1);
		ChangeAvec(p_titre_affiche_haut == 2);
		Change(p_titre_valeur_haut);
		_couleur.Change(p_titre_couleur_haut);
		_taille.Change(min(15,max(0,p_titre_taille_haut/8-1)));
		break;

	    case 1 :

		ChangeNon(p_titre_affiche_gauche == 0);
		ChangeSans(p_titre_affiche_gauche == 1);
		ChangeAvec(p_titre_affiche_gauche == 2);
		Change(p_titre_valeur_gauche);
		_couleur.Change(p_titre_couleur_gauche);
		_taille.Change(min(15,max(0,p_titre_taille_gauche/8-1)));
		break;

	    case 2 :

		ChangeNon(p_titre_affiche_droit == 0);
		ChangeSans(p_titre_affiche_droit == 1);
		ChangeAvec(p_titre_affiche_droit == 2);
		Change(p_titre_valeur_droit);
		_couleur.Change(p_titre_couleur_droit);
		_taille.Change(min(15,max(0,p_titre_taille_droit/8-1)));
		break;

	    case 3 :

		ChangeNon(p_titre_affiche_bas == 0);
		ChangeSans(p_titre_affiche_bas == 1);
		ChangeAvec(p_titre_affiche_bas == 2);
		Change(p_titre_valeur_bas);
		_couleur.Change(p_titre_couleur_bas);
		_taille.Change(min(15,max(0,p_titre_taille_bas/8-1)));
		break;

	}

}

void
Titres::ChargeInv()
{

    // lecture des valeurs de config a partir du popup Titres

	switch(p_titre_numero)
	{

	    case 0 :

		p_titre_affiche_haut = _sans.IsSelected() +
			_avec.IsSelected()*2;
		break;

	    case 1 :

		p_titre_affiche_gauche = _sans.IsSelected() +
			_avec.IsSelected()*2;
		break;

	    case 2 :

		p_titre_affiche_droit = _sans.IsSelected() +
			_avec.IsSelected()*2;
		break;

	    case 3 :

		p_titre_affiche_bas = _sans.IsSelected() +
			_avec.IsSelected()*2;
		break;

	}

}

void
TitreOK::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // lecture de tous les champs textes

	_pto->ChangeValeur();

    // on prend en compte les valeurs de la config partielles lues
    // precedemment

	S6GereConfig::_config = S6GereConfigP::_p_titre_config;

	if(calcul_ok_cache) S6GereFenet::titres->Cache();

}

void
TitreAnnuler::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on annule les valeurs lues dans la config partielle

	S6GereConfigP::_p_titre_config = S6GereConfig::_config;
	S6GereFenet::ReConfigTitres();
	S6GereFenet::titres->Cache();
}

TitreAide::TitreAide(const Widget& parent, const char *label, 
		const char* classe) :
		Bouton(parent,label,classe)
{
	RegisterCol(_widgetid);

	U8AfficheurX* const uax = U8AfficheurX::Driver();

	if(uax)
	{
	    uax->l3_tabwid_init(_widgetcol);
	}

}

void
TitreAide::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(!S6GereFenet::titres_help)
	{
	    cout << "TitreAide> creation pop-up-help Titres " << endl;

	    S6GereFenet::titres_help = new ScrHelp(GetId(),
			    S6Traduc::traduc[239],"TITRES_HELP");
	}

	S6GereFenet::titres_help->Montre();
}

void
TitresSaisie::ChangeValeur()
{

    // lecture systematique du titre si on change de numero de titre

	switch(p_titre_numero)
	{
	  case 0:
	    {
		delete[] p_titre_valeur_haut;

		S6String ss(GetText());

		p_titre_valeur_haut = ss;
	    }
	    break;
	  case 1:
	    {
		delete[] p_titre_valeur_gauche;

		S6String ss(GetText());

		p_titre_valeur_gauche = ss;
	    }
	    break;
	  case 2:
	    {
		delete[] p_titre_valeur_droit;

		S6String ss(GetText());

		p_titre_valeur_droit = ss;
	    }
	    break;
	  case 3:
	    {
		delete[] p_titre_valeur_bas;

		S6String ss(GetText());

		p_titre_valeur_bas = ss;
	    }
	    break;
	}

}

void
TitresSaisie::SelectCB(XtPointer call_data)
{
	Saisie::SelectCB(call_data);

	ChangeValeur();
}

void
TitresChoixHaut::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int numero = 0;

	_pto->ChangeNumero(numero);
	_pto->ChangeValeur();
	_pto->Charge(0);
}

void
TitresChoixGauche::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int numero = 1;

	_pto->ChangeNumero(numero);
	_pto->ChangeValeur();
	_pto->Charge(1);
}

void
TitresChoixDroit::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int numero = 2;

	_pto->ChangeNumero(numero);
	_pto->ChangeValeur();
	_pto->Charge(2);
}

void
TitresChoixBas::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int numero = 3;

	_pto->ChangeNumero(numero);
	_pto->ChangeValeur();
	_pto->Charge(3);
}

void
TitresChoixNon::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeSans(0);
	_pto->ChangeAvec(0);
	_pto->ChargeInv();
}

void
TitresChoixSans::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeNon(0);
	_pto->ChangeAvec(0);
	_pto->ChargeInv();
}

void
TitresChoixAvec::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	_pto->ChangeNon(0);
	_pto->ChangeSans(0);
	_pto->ChargeInv();
}

// textes ..................................................

TextesAppel::TextesAppel(const Widget& parent,const char* label) :
		PullDownEntry(parent, label)
{
}

void
TextesAppel::SelectCB(XtPointer call_data)
{
	ModeTrace(S6Output::Output());
	PullDownEntry::SelectCB(call_data);

	if(S6GereFenet::textes == NULL)
	{
	    S6GereFenet::textes = new Textes(S6GereFenet::root->ElVisuel()->
		GetId(),S6Traduc::traduc[96],300,100);
	    S6GereFenet::ReConfigTextes();
	}	

	S6GereFenet::textes->Montre();
}

Textes::Textes(const Widget& root,const char* titre,int nx,int ny) :
		PopUp(root,titre,nx,ny),
		_tout(GetId(),"TEXTE_TOUT",-1),

		_texte_j(_tout.GetId(),"CONT_TEXTE_TEXTE_J"),
		_texte_l(_texte_j.GetId(),"CONT_TEXTE_TEXTE_L",-1),

		_nombregen(_texte_l.GetId(),"texte_nombregen",-1),

		_nombre(_nombregen.GetId(),"texte_nombre",-1),
		_nombre_label(_nombre.GetId(),S6Traduc::traduc[87]),
		_nombre_saisie(_nombre.GetId(),""),

		_generateur(_nombregen.GetId(),"generateur"),
		_generateur_label(_generateur.GetId(),S6Traduc::traduc[310]),
		_hard(_generateur.GetId(),S6Traduc::traduc[311]),
		_soft(_generateur.GetId(),S6Traduc::traduc[312]),

		_numero(_texte_l.GetId(),"texte_numero",-1),
		_numero_label(_numero.GetId(),S6Traduc::traduc[88]),
		_numero_slider(_numero.GetId(),"",0,1,0),

		_param_j(_tout.GetId(),"CONT_TEXTE_PARAM_J"),
		_param_l(_param_j.GetId(),"CONT_TEXTE_PARAM_L",-1),

		_affichage(_param_l.GetId(),"AFFICHAGE"),
		_affichage_label(_affichage.GetId(),S6Traduc::traduc[89]),
		_non(_affichage.GetId(),S6Traduc::traduc[90],"texte_non"),
		_sans(_affichage.GetId(),S6Traduc::traduc[91],"texte_sans"),
		_avec(_affichage.GetId(),S6Traduc::traduc[92],"texte_avec"),

		_texte(_param_l.GetId(),"texte_texte",-1),
		_texte_label(_texte.GetId(),S6Traduc::traduc[93]),
		_texte_saisie(_texte.GetId(),""),

		_params(_param_l.GetId(),"TEXTE_PARAMS",-1),
		_couleur(_params.GetId(),*(S6GereFenet::couleur),
			"TEXTE_COULEUR",S6Traduc::traduc[59],-1),
		_taille(_params.GetId(),*(S6GereFenet::taille),
			"TEXTE_TAILLE",S6Traduc::traduc[70],-1),

		_position(_params.GetId(),"TEXTE_POSITION",-1),
		_position_bouton(_position.GetId(),S6Traduc::traduc[94]),
		_position_posx(_position.GetId(),"texte_pos_x"),
		_position_posy(_position.GetId(),"texte_pos_y"),

		_angle(_param_l.GetId(),"TEXTE_ANGLE",-1),
		_angle_label(_angle.GetId(),S6Traduc::traduc[95]),
		_angle_slider(_angle.GetId(),"",0,360,0),

		_standard_j(_tout.GetId(),"TEXTE_STANDARD_J"),

		_standard(_standard_j.GetId(),"TEXTE_STANDARD",1),
		_ok(_standard.GetId(),S6Traduc::traduc[227],"BTN_OAA"),
		_annuler(_standard.GetId(),S6Traduc::traduc[228],"BTN_OAA"),
		_aide(_standard.GetId(),S6Traduc::traduc[229],"BTN_OAA")
{
	_texte_nombre = 0;

	_numero_slider.Attach(this);
	_nombre_saisie.Attach(this);
	_position_bouton.Attach(this);

	_non.Attach(this);
	_sans.Attach(this);
	_avec.Attach(this);

	_non.Change(TRUE);
	
	_ok.Attach(this);

	_hard.Change(FALSE);
	_soft.Change(TRUE);

	BloqueGeom();
}

void
Textes::ChargeInv()
{
    // lecture des valeurs de config a partir du popup Textes

	p_texte_affiche[p_texte_numero] = _sans.IsSelected() +
			_avec.IsSelected()*2;
}

void
Textes::ChangeValeur(int taille)
{
	((C5ConfigTabChar&) p_texte_valeur_config)(taille,"");
	((C5ConfigTabInt&) p_texte_affiche_config)(taille,1);
	((C5ConfigTabInt&) p_texte_couleur_config)(taille,1);
	((C5ConfigTabInt&) p_texte_taille_config)(taille,90);
	((C5ConfigTabReel&) p_texte_pos_x_config)(taille,.01,0.);

	REEL_S coef = REEL_S(.5/max(1,taille));

	((C5ConfigTabReel&) p_texte_pos_y_config)(taille,coef,coef);
	((C5ConfigTabReel&) p_texte_angle_config)(taille,0.,0.);
}

void
Textes::Charge(int texte_num)
{

    // chargement des valeurs de config dans le popup Textes

	cout << "Textes> texte_nombre " << _texte_nombre << endl;

	if(texte_num >= _texte_nombre) texte_num = _texte_nombre - 1;

	if(texte_num < 0) return;

	p_texte_numero = texte_num;
	ChangeType(p_texte_affiche[texte_num]);
	Change(p_texte_valeur[texte_num]);
	Change(0,_texte_nombre - 1,texte_num);
	_couleur.Change(p_texte_couleur[texte_num]);
	_taille.Change(min(15,max(0,p_texte_taille[texte_num]/8-1)));
	_angle_slider.Change(0,360,int(p_texte_angle[texte_num]));

	{
	    S6String ss(p_texte_pos_x[texte_num]);
	    char* ptos = ss;

	    _position_posx.Change(ptos);

	    delete[] ptos;
	}

	{
	    S6String ss(p_texte_pos_y[texte_num]);
	    char* ptos = ss;

	    _position_posy.Change(ptos);

	    delete[] ptos;
	}

}

void
Textes::Change(REEL xpos,REEL ypos)
{
	cout << "Textes> xpos " << xpos << " ypos " << ypos << endl;

	const int num = p_texte_numero;

	if(num >= _texte_nombre || num < 0) return;

	p_texte_pos_x[p_texte_numero] = REEL_S(xpos);
	p_texte_pos_y[p_texte_numero] = REEL_S(ypos);

	{
	    S6String ss(xpos);
	    char* ptos = ss;

	    _position_posx.Change(ptos);

	    delete[] ptos;
	}

	{
	    S6String ss(ypos);
	    char* ptos = ss;

	    _position_posy.Change(ptos);

	    delete[] ptos;
	}

}

void
Textes::Change(const char* texte_saisie)
{
	_texte_saisie.Change(texte_saisie);
}

void
Textes::Change(int vmin,int vmax,int pos)
{
	_texte_nombre = vmax + 1;
	_numero_slider.Change(vmin,max(vmax,vmin+1),pos);
}

void
Textes::ChangeNombre()
{
	int texte_nombre = 1;

	texte_nombre = max(texte_nombre,p_texte_valeur_taille);
	texte_nombre = max(texte_nombre,p_texte_angle_taille);
	texte_nombre = max(texte_nombre,p_texte_pos_y_taille);
	texte_nombre = max(texte_nombre,p_texte_pos_x_taille);
	texte_nombre = max(texte_nombre,p_texte_taille_taille);
	texte_nombre = max(texte_nombre,p_texte_couleur_taille);
	texte_nombre = max(texte_nombre,p_texte_affiche_taille);

	ChangeValeur(texte_nombre);
	_texte_nombre = texte_nombre;

	S6String ss(_texte_nombre);
	char* ptos = ss;

	_nombre_saisie.Change(ptos);

	delete[] ptos;
}

void
Textes::ChangeGenerateur(int& choix)
{
	_hard.Change(FALSE);
	_soft.Change(FALSE);

	switch(choix)
	{
	    case 0 :
		_hard.Change(TRUE);
		break;

	    case 1 :
		_soft.Change(TRUE);
		break;

	    default :
		choix = 1;
		_soft.Change(TRUE);
		break;
	}

}

void
Textes::ChangeType(int& choix)
{
	_non.Change(FALSE);
	_sans.Change(FALSE);
	_avec.Change(FALSE);

	switch(choix)
	{
	    case 0 :
		_non.Change(TRUE);
		break;

	    case 1 :
		_sans.Change(TRUE);
		break;

	    case 2 :
		_avec.Change(TRUE);
		break;

	    default :
		choix = 1;
		_sans.Change(TRUE);
		break;
	}

}

void
TexteOK::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // lecture de tous les champs textes

	_pto->ChangeValeur();

    // on prend en compte les valeurs de la config partielles lues
    // precedemment

	S6GereConfig::_config = S6GereConfigP::_p_texte_config;

	if(calcul_ok_cache) S6GereFenet::textes->Cache();

}

void
TexteAnnuler::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on annule les valeurs lues dans la config partielle

	S6GereConfigP::_p_texte_config = S6GereConfig::_config;
	S6GereFenet::ReConfigTextes();
	S6GereFenet::textes->Cache();
}

void
TexteAide::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(S6GereFenet::textes_help == NULL)
	{
	    cout << "TexteAide> creation pop-up-help Textes " << endl;

	    S6GereFenet::textes_help = new ScrHelp(GetId(),
		S6Traduc::traduc[240],"TEXTES_HELP");
	}	

	S6GereFenet::textes_help->Montre();
}

void
TexteGenerateurHard::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	p_texte_generateur = 0;
}

void
TexteGenerateurSoft::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);
	p_texte_generateur = 1;
}

void
NombreTextesSaisie::SelectCB(XtPointer call_data)
{
	Saisie::SelectCB(call_data);

	char* nombre = GetText();
	istrstream is(nombre);
	int taille = 0;
	is >> taille;

	cout << "NombreTextesSaisie> nombre de textes " << taille << endl;

	if(taille >= 100 || taille <= 0)
	{
	    taille = max(min(99,taille),1);

	    S6String ss(taille);
	    char* ptos = ss;

	    Change(ptos);

	    delete[] ptos;
	}

	ChangeValeur(taille);
}

void
NombreTextesSaisie::ChangeValeur(int nombre)
{
	_pto->ChangeValeur(nombre);
	_pto->Change(0,nombre-1,0);
}

void
NumeroTextesSlider::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	_pto->ChangeValeur();
	_pto->Charge(GetValue());
}

void
TextesBoutonPosition::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	REEL xpos,ypos;

	L3GetPosition(2,xpos,ypos);
	_pto->Change(xpos,ypos);
}

void
AngleTextesSlider::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_texte_angle[p_texte_numero] = (REEL_S) GetValue();
}

void
TextesSaisie::SelectCB(XtPointer call_data)
{
	Saisie::SelectCB(call_data);
	ChangeValeur();
}

void
TextesSaisie::ChangeValeur()
{

    // lecture systematique du texte si on change de numero de texte

	delete[] p_texte_valeur[p_texte_numero];

	S6String ss(GetText());

	p_texte_valeur[p_texte_numero] = ss;
}

void
ChoixVerticalCouleurTextes::ChargeInv()
{
	p_texte_couleur[p_texte_numero] = Choix();

	if(!parametre_couleur_fond && !p_texte_couleur[p_texte_numero])
			p_texte_couleur[p_texte_numero] = 1;

}

void
ChoixVerticalTailleTextes::ChargeInv()
{
	p_texte_taille[p_texte_numero] = max((Choix() + 1) * 8,10);
}

void
TextesChoixNon::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int type = 0;

	_pto->ChangeType(type);
	_pto->ChargeInv();
}

void
TextesChoixSans::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int type = 1;

	_pto->ChangeType(type);
	_pto->ChargeInv();
}

void
TextesChoixAvec::SelectCB(XtPointer call_data)
{
	BoutonDeChoix::SelectCB(call_data);

	int type = 2;

	_pto->ChangeType(type);
	_pto->ChargeInv();
}
