// ******************************************************************
// identification du source
// ******************************************************************

struct SccsIdS6Bouttons5
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Bouttons5::sccs_id = "%W% %G% Pierre Brochard";

// ******************************************************************
// includes
// ******************************************************************

#include "U8constant.h"
#include "S6bouttons5.h"
#include "S6constant.h"
#include "S6gerefenet.h"
#include "S6gereconfig.h"
#include "S6gereenv.h"
#include "S6traduc.h"
#include "poulpe/Help.h"

// ******************************************************************
//		classes bouttons (suite) de Psyche
// ******************************************************************

// Point de vue .....................................................

PointDeVue::PointDeVue(const Widget& root, const Widget& parent,
		const char* label,const char* classe) :
		PullDownEntry(parent, label,classe),
		_pere(root,S6Traduc::traduc[114],500,100),
		_tout(_pere.GetId(),"CONT_POINT_DE_VUE_TOUT",-1),

		_azimuth_cont(_tout.GetId(),"CONT_POINT_DE_VUE_AZIM"),
		_azimuth_label(_azimuth_cont.GetId(),S6Traduc::traduc[15]),
		_azimuth(_azimuth_cont.GetId(),"",0,360,45,
			"SLIDER_point_de_vue_azimuth"),

		_site_cont(_tout.GetId(),"CONT_POINT_DE_VUE_SITE"),
		_site_label(_site_cont.GetId(),S6Traduc::traduc[16]),
		_site(_site_cont.GetId(),"",0,360,45,
			"SLIDER_point_de_vue_site"),

		_roulis_cont(_tout.GetId(),"CONT_POINT_DE_VUE_ROULIS"),
		_roulis_label(_roulis_cont.GetId(),S6Traduc::traduc[269]),
		_roulis(_roulis_cont.GetId(),"",0,360,45,
			"SLIDER_point_de_vue_roulis"),

		_distance_cont(_tout.GetId(),"CONT_POINT_DE_VUE_DISTANCE"),
		_distance_label(_distance_cont.GetId(),S6Traduc::traduc[115]),
		_distance(_distance_cont.GetId(),"",0,20,0,
			"SLIDER_point_de_vue_distance"),

		_redrh_cont(_tout.GetId(),"CONT_POINT_DE_VUE_REDRH"),
		_redrh_label(_redrh_cont.GetId(),S6Traduc::traduc[116]),
		_redrh(_redrh_cont.GetId(),"",0,360,0,
			"SLIDER_point_de_vue_redrh"),

		_redrv_cont(_tout.GetId(),"CONT_POINT_DE_VUE_REDRV"),
		_redrv_label(_redrv_cont.GetId(),S6Traduc::traduc[117]),
		_redrv(_redrv_cont.GetId(),"",0,360,0,
			"SLIDER_point_de_vue_redrv"),

		_standard(_tout.GetId(),"POINT_DE_VUE_STANDARD",1),
		_ok(_standard.GetId(),S6Traduc::traduc[227],"BTN_OAA"),
		_annuler(_standard.GetId(),S6Traduc::traduc[228],"BTN_OAA"),
		_aide(_standard.GetId(),S6Traduc::traduc[229],"BTN_OAA")
{
	_pere.BloqueGeom();
}

void PointDeVue::SelectCB(XtPointer call_data)
{
	ModeTrace(S6Output::Output());
	PullDownEntry::SelectCB(call_data);
	_pere.Montre();
}

void PointDeVue::Change()
{
	_azimuth.Change(0,360,int(p_ptvue_azimuth+.1));
	_site.Change(0,360,int(p_ptvue_site+.1));
	_roulis.Change(0,360,int(p_ptvue_roulis+.1));
	_distance.Change(0,20,int(p_ptvue_distance+.1));
	_redrh.Change(0,360,int(p_ptvue_redress_hori+.1));
	_redrv.Change(0,360,int(p_ptvue_redress_vert+.1));
}

void PointDeVue::Montre2d()
{
	PullDownEntry::Cache();
	_pere.Cache();
}

void PointDeVue::Cache()
{
	_pere.Cache();
}

void PointDeVueOK::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on prend en compte les valeurs de la config partielles lues
    // precedemment

	S6GereConfig::_config = S6GereConfigP::_p_point_de_vue_config;

	if(calcul_ok_cache) S6GereFenet::ptvue->Cache();

}

void PointDeVueAnnuler::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on annule les valeurs lues dans la config partielle

	S6GereConfigP::_p_point_de_vue_config = S6GereConfig::_config;
	S6GereFenet::ReConfigPtvue();
	S6GereFenet::ptvue->Cache();
}

void PointDeVueAide::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(S6GereFenet::ptvue_help == NULL)
	{
	    cout << " creation pop-up-help Point de vue " << endl;

	    S6GereFenet::ptvue_help = new Help(GetId(),S6Traduc::traduc[270],
		"POINT_DE_VUE_HELP");
	}	

	S6GereFenet::ptvue_help->Montre();
}

void SliderAzimuth::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_azimuth = (REEL) GetValue();
}

void SliderSite::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_site = (REEL) GetValue();
}

void SliderRoulis::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_roulis = (REEL) GetValue();
}

void SliderDistance::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_distance = (REEL) GetValue();
}

void SliderRedrh::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_redress_hori = (REEL) GetValue();
}

void SliderRedrv::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_ptvue_redress_vert = (REEL) GetValue();
}

// Animation .....................................................

Animation::Animation(const Widget& root, const Widget& parent,
		const char* label,const char* classe) :
		PullDownEntry(parent, label,classe),
		_pere(root,S6Traduc::traduc[156],500,100),
		_tout(_pere.GetId(),"ANIMATION_TOUT",-1),

		_pas_azimuth_cont(_tout.GetId(),"ANIMATION_AZIMUTH"),
		_pas_azimuth_label(_pas_azimuth_cont.GetId(),
			S6Traduc::traduc[157]),
		_pas_azimuth(_pas_azimuth_cont.GetId(),"",0,360,0,
			"animation_azimuth"),

		_pas_site_cont(_tout.GetId(),"ANIMATION_SITE"),
		_pas_site_label(_pas_site_cont.GetId(),S6Traduc::traduc[158]),
		_pas_site(_pas_site_cont.GetId(),"",0,360,0,
			"animation_site"),

		_nombre_anim_cont(_tout.GetId(),"ANIMATION_NOMBRE"),
		_nombre_anim_label(_nombre_anim_cont.GetId(),
			S6Traduc::traduc[159]),
		_nombre_anim(_nombre_anim_cont.GetId(),"",0,500,1,
			"animation_nombre"),

		_standard(_tout.GetId(),"ANIMATION_STANDARD",1),
		_ok(_standard.GetId(),S6Traduc::traduc[227],"BTN_OAA"),
		_annuler(_standard.GetId(),S6Traduc::traduc[228],"BTN_OAA"),
		_aide(_standard.GetId(),S6Traduc::traduc[229],"BTN_OAA")
{
	_pere.BloqueGeom();
}

void Animation::SelectCB(XtPointer call_data)
{
	ModeTrace(S6Output::Output());
	PullDownEntry::SelectCB(call_data);
	_pere.Montre();
}

void Animation::Change()
{
	_pas_azimuth.Change(0,360,int(p_animation_pas_azimuth+0.1));
	_pas_site.Change(0,360,int(p_animation_pas_site+0.1));
	_nombre_anim.Change(0,500,int(p_animation_nombre+0.1));
}

void Animation::Montre2d()
{
	PullDownEntry::Cache();
	_pere.Cache();
}

void Animation::Cache()
{
	_pere.Cache();
}

void AnimationOK::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on prend en compte les valeurs de la config partielles lues
    // precedemment

	S6GereConfig::_config = S6GereConfigP::_p_animation_config;

	if(calcul_ok_cache) S6GereFenet::animation->Cache();

}

void AnimationAnnuler::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on annule les valeurs lues dans la config partielle

	S6GereConfigP::_p_animation_config = S6GereConfig::_config;
	S6GereFenet::animation->Change();
	S6GereFenet::animation->Cache();
}

void AnimationAide::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(S6GereFenet::animation_help == NULL)
	{
	    cout << " creation pop-up-help Animation " << endl;

	    S6GereFenet::animation_help = new Help(GetId(),
		S6Traduc::traduc[272],"ANIMATION_HELP");
	}	

	S6GereFenet::animation_help->Montre();
}

void SliderPasAzimuth::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_animation_pas_azimuth = GetValue();
}

void SliderPasSite::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_animation_pas_site = GetValue();
}

void SliderNombreAnim::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);
	p_animation_nombre = GetValue();
}

// Gestion des couleurs ................................................

GestionCouleur::GestionCouleur(const Widget& root, const Widget& parent,
		const char* label,const char* classe) :
		PullDownEntry(parent, label,classe),
		_pere(root,S6Traduc::traduc[339],500,100),
		_tout(_pere.GetId(),"GESTIONCOULEUR_TOUT",-1),

		_icol_cont(_tout.GetId(),"GESTIONCOULEUR_ICOL"),
		_icol(_icol_cont.GetId(),S6Traduc::traduc[340],
				"BTN_colors"),

		_base_cont(_tout.GetId(),"GESTIONCOULEUR_BASE"),
		_base_label(_base_cont.GetId(),S6Traduc::traduc[341]),
		_base(_base_cont.GetId(),"",1,256,1,"gestioncouleur_base"),

		_palette_cont(_tout.GetId(),"GESTIONCOULEUR_PALETTE"),
		_palette_label(_palette_cont.GetId(),S6Traduc::traduc[342]),
		_palette(_palette_cont.GetId(),"",1,256,1,
				"gestioncouleur_palette"),

		_standard(_tout.GetId(),"GESTIONCOULEUR_STANDARD",1),
		_ok(_standard.GetId(),S6Traduc::traduc[227],"BTN_OAA"),
		_annuler(_standard.GetId(),S6Traduc::traduc[228],"BTN_OAA"),
		_aide(_standard.GetId(),S6Traduc::traduc[229],"BTN_OAA")
{
	_ok.Attach(this);
	_annuler.Attach(this);

	_pere.BloqueGeom();
}

void
GestionCouleur::SelectCB(XtPointer call_data)
{
	ModeTrace(S6Output::Output());
	PullDownEntry::SelectCB(call_data);
	_pere.Montre();
}

void
GestionCouleur::Change()
{
	_base.Change(1,256,p_parametre_map_col_t/3);
	_palette.Change(1,256,p_parametre_map_pal_t/3);
}

void
GestionCouleur::Cache()
{
	_pere.Cache();
}

void
GestionCouleurOK::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on prend en compte les valeurs de la config partielle lues
    // precedemment

	S6GereConfig::_config = S6GereConfigP::_p_pcco_config;

	if(hauteur_z_taille >= parametre_map_pal_t/3)
	{
	    register int taille = parametre_map_pal_t/3 - 1;
	    register REEL_S* temp_2 = new REEL_S[taille];

	    memcpy(temp_2,hauteur_z,taille*sizeof(REEL_S));

	    delete[] hauteur_z;

	    hauteur_z = temp_2;
	    hauteur_z_taille = taille;
	}

	if(hauteur_c_taille >= parametre_map_pal_t/3)
	{
	    register int taille = parametre_map_pal_t/3 - 1;
	    register REEL_S* temp_2 = new REEL_S[taille];

	    memcpy(temp_2,hauteur_c,taille*sizeof(REEL_S));

	    delete[] hauteur_c;

	    hauteur_c = temp_2;
	    hauteur_c_taille = taille;
	}

	S6GereConfigP::_p_echelle_config = S6GereConfig::_config;
	S6GereFenet::ReConfigEchelles();
	S6GereFenet::ReConfigCouleurs();

	if(calcul_ok_cache && _pto) _pto->Cache();

}

void
GestionCouleurAnnuler::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

    // on annule les valeurs lues dans la config partielle

	S6GereConfigP::_p_pcco_config = S6GereConfig::_config;

	if(_pto)
	{
	    _pto->Change();
	    _pto->Cache();
	}

}

void
GestionCouleurAide::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(!S6GereFenet::gestioncouleur_help)
	{
	    cout << " creation pop-up-help GestionCouleur" << endl;

	    S6GereFenet::gestioncouleur_help = new Help(GetId(),
		S6Traduc::traduc[343],"GESTIONCOULEUR_HELP");
	}	

	S6GereFenet::gestioncouleur_help->Montre();
}

void
GestionCouleurIcol::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	cerr << "On ouvre Colors : patientez !!" << endl;

	char* ptos = NULL;
	int ier = 0;

	{
	    ostrstream os;

	    os << psyche_sp3 << "/Colors &" << ends;
	    ptos = os.str();ier = system(ptos);
	}

	delete[] ptos;

	if(ier)
	{
	    cerr << "GestionCouleurIcol> Pb system dans " __FILE__ << endl;
	}

}

void
SliderCoulBase::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);

	delete[] p_parametre_map_col;

	const int tai = GetValue();
	register int* pmf = new int[3*tai];

	p_parametre_map_col = pmf;
	p_parametre_map_col_t = 3*tai;
	memset(pmf,0,3*tai*sizeof(int));
}

void
SliderCoulPalette::SelectCB(XtPointer call_data)
{
	Slider::SelectCB(call_data);

	delete[] p_parametre_map_pal;

	const int tai = GetValue();
	register int* pmf = new int[3*tai];

	p_parametre_map_pal = pmf;
	p_parametre_map_pal_t = 3*tai;
	memset(pmf,0,3*tai*sizeof(int));
}
