// *********************************************************************
//  operateurs de lecture de fichiers .spx (binaires Psyche)
// *********************************************************************

struct SccsIdS6Points
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Points::sccs_id = "%W% %G% Pierre Brochard";

// includes

#ifdef CRAY
#   include <locale.h>
#endif

#ifdef _IBMR2
#   define _BSD
#endif

#include <ctype.h>
#include <netinet/in.h>

#include "S6gerefenet.h"
#include "S6bouttons4.h"
#include "P4constant.h"
#include "S6bouttons3.h"
#include "S6boutfich.h"
#include "nxstream.h"
#include "S6points.h"
#include "S6namecomp.h"
#include "S6gereconfig.h"
#include "S6gereenv.h"
#include "S6flagssup.h"
#include "S6initcourbe.h"
#include "S6traduc.h"
#include "dinosaure/S6timer.h"
#include "S6trace.h"
#include "minotaure/P4graf3.h"
#include "S6bouttons9.h"

#if !defined(P4VERSION_AVEC_TEMPLATE) /* version sans template */
#   define GEN ptrS6DonneeAbstraite
#   define TabGEN TabS6DonneeAbstraite
#   define TabGENbuf TabS6DonneeAbstraitebuf
#   include "dinosaure/P4tabgen.c.h"
#   undef GEN
#   undef TabGEN
#   undef TabGENbuf
#elif defined(__GNUC__)
#   include "dinosaure/P4tabgen.c.h"

    template class TabGEN<ptrS6DonneeAbstraite>;
    template class TabGENbuf<ptrS6DonneeAbstraite>;
#endif /* fin version sans template */

// *******************************************************
//  init variables globales et statiques de classe
// *******************************************************

const char* const S6Points::type_fichier_name[] =
{
	"S6PHYSIQUE_4D_NS_BRAS",
	"S6RECT_3D_MAILLE",
	"S6RECT_4D_MAILLE",
	"S6PHYSIQUE_3D_MAILLE",
	"S6PHYSIQUE_4D_MAILLE",
	"S6BASE_2D",
	"S6PARRECT_4D",
	"S6RECT_3D",
	"S6RECT_4D",
	"S6OPEN",
	"S6SYNCRO",
	"S6PHYSIQUE_4D_NS_MAILLE",
	"S6PHYSIQUE_4D_NS",
	"S6CONFIG_SEULE",
	"S6LOGIQUE_3D",
	"S6PHYSIQUE_2D",
	"S6PHYSIQUE_3D",
	"S6PHYSIQUE_4D"
};

int S6Points::num_fich_temp = 0;
int S6Points::_decal_type_fichier = 14; // pour impression du type de fichier

//*********************************************************
//   implementation de la classe S6DonneeAbstraite
//*********************************************************

void
S6DonneeAbstraite::Labels(const char* const labels)
{
	delete[] _labels;_labels = NULL;

	if(labels)
	{
	    const size_t llabels = strlen(labels)+1;

	    _labels = new char[llabels];
	    memcpy(_labels,labels,llabels);
	}

}

void
S6DonneeAbstraite::Unites(const char* const unites)
{
	delete[] _unites;_unites = NULL;

	if(unites)
	{
	    const size_t lunites = strlen(unites)+1;

	    _unites = new char[lunites];
	    memcpy(_unites,unites,lunites);
	}

}

//*********************************************************
//   implementation de la classe S6Points
//*********************************************************

S6Points::S6Points(const S6Points& sp) :
		_type(sp._type),_type_fin(sp._type_fin),
		_donnee_abstraite(sp._donnee_abstraite),
		_config_lue_socket(sp._config_lue_socket,TRUE),
		_nb_fichiers_concat(sp._nb_fichiers_concat),
		_donnee_abstraite_base(sp._donnee_abstraite_base),
		_recherche_base(sp._recherche_base)
{
}

//***************************************************************
//     operateur d' ecriture fichiers binaire Psyche
//***************************************************************

nxstream&
operator << (nxstream& ost,const S6Points& p)
{
	p << ost;
	return ost;
}

nxstream&
S6Points::operator << (nxstream& ost) const
{
	TimerDetInit();

	FILE* const tf = ost.rdbuf()->stdiofile();

    // -----------------------------------------------------------------
    //   sauvegarde du dernier bloc en 3d et de tous les blocs en 2d
    // -----------------------------------------------------------------

	unsigned nbbloc = _donnee_abstraite;nbbloc--;
	S6DonneeAbstraite* sda = _donnee_abstraite[nbbloc];
	TypeFichier type = sda->Typefichier();

	const int type_temp = (int) type;

	cout << "S6Points> type_temp " << type_temp << endl;

	fnxwriti(tf,1,&type_temp);

	int nx = 0;
	int ny = 0;
	Tabint nxsto;

	int i = 0, nmailles = 0, dimmailles = 0;

    // ----------------------------------------------------
    //   ecriture donnees abstraites
    // ----------------------------------------------------

	switch(type)
	{
	    case S6OPEN :
	    case S6SYNCRO :
	    case S6CONFIG_SEULE :
	    case S6BASE_2D :
	    case S6PHYSIQUE_4D_NS_BRAS :

		break;

	    case S6LOGIQUE_3D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx*ny,sda->Z());

		break;

	    case S6PHYSIQUE_2D :

		ny = _donnee_abstraite;
		fnxwriti(tf,1,&ny);

		for(i=0;i<ny;i++)
		{
		    sda = _donnee_abstraite[i];
		    nx = sda->Nx();

		    fnxwriti(tf,1,&nx);
		}

		for(i=0;i<ny;i++)
		{
		    sda = _donnee_abstraite[i];

		    fnxwritf(tf,sda->Nx(),sda->X());
		    fnxwritf(tf,sda->Nx(),sda->Y());
		}

		break;

	    case S6RECT_3D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx,sda->X());
		fnxwritf(tf,ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());

		break;

	    case S6RECT_3D_MAILLE :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx,sda->X());
		fnxwritf(tf,ny,sda->Y());
		fnxwritf(tf,(nx-1)*(ny-1),sda->C());

		break;

	    case S6PHYSIQUE_3D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,&nxsto[0]);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());

		break;

	    case S6PHYSIQUE_3D_MAILLE :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,&nxsto[0]);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,(nx-1)*(ny-1),sda->C());

		break;

	    case S6RECT_4D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx,sda->X());
		fnxwritf(tf,ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());
		fnxwritf(tf,nx*ny,sda->C());

		break;

	    case S6RECT_4D_MAILLE :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx,sda->X());
		fnxwritf(tf,ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());
		fnxwritf(tf,(nx-1)*(ny-1),sda->C());

		break;

	    case S6PHYSIQUE_4D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());
		fnxwritf(tf,nx*ny,sda->C());

		break;

	    case S6PHYSIQUE_4D_MAILLE :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());
		fnxwritf(tf,(nx-1)*(ny-1),sda->C());

		break;

	    case S6PHYSIQUE_4D_NS :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
		nmailles = sda->Nmailles();
		dimmailles = sda->Mailles();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());
		fnxwritf(tf,nx*ny,sda->C());

		fnxwriti(tf,1,&nmailles);
		fnxwriti(tf,1,&dimmailles);

		fnxwriti(tf,dimmailles,(int*) (unsigned*) sda->Mailles());

		break;

	    case S6PHYSIQUE_4D_NS_MAILLE :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();
		nmailles = sda->Nmailles();
		dimmailles = sda->Mailles();
	 	nxsto.ModValn(ny);

		for(i=0;i<ny;i++)
		{
		    nxsto[i] = nx;
		}

		fnxwriti(tf,1,&ny);
		fnxwriti(tf,ny,nxsto);
		fnxwritf(tf,nx*ny,sda->X());
		fnxwritf(tf,nx*ny,sda->Y());
		fnxwritf(tf,nx*ny,sda->Z());

		fnxwriti(tf,1,&nmailles);
		fnxwriti(tf,1,&dimmailles);
		fnxwritf(tf,nmailles,sda->C());
		fnxwriti(tf,dimmailles,(int*) (unsigned*) sda->Mailles());

		break;

	    case S6PARRECT_4D :

		sda = _donnee_abstraite[nbbloc];
		nx = sda->Nx();
		ny = sda->Ny();

		{
		    const int nz = sda->Nz();

		    fnxwriti(tf,1,&nx);
		    fnxwriti(tf,1,&ny);
		    fnxwriti(tf,1,&nz);

		    fnxwritf(tf,nx,sda->X());

		    register const REEL_S* y = sda->Y();

		    for(i=0;i<ny;i++)
		    {
			fnxwritf(tf,1,y);
			y += nx;
		    }

		    register const REEL_S* z = sda->Z();

		    for(i=0;i<nz;i++)
		    {
			fnxwritf(tf,1,z);
			z += nx*ny;
		    }

		    const int npoints = nx*ny*nz;

		    fnxwritf(tf,npoints,sda->C());
		}

		break;

	}   // fin ecriture _donnee_abstraite

	int type_fin = _type_fin;

	if(type_fin !=  10 && type_fin != 11) type_fin = 0;

	fnxwriti(tf,1,&type_fin);

	cout << "S6Points> code config ecriture " << _type_fin << endl;

    // ecriture titres et textes et labels axes spx

	if(_type_fin == 11)
	{
	    int llst[11];

	    memset(llst,0,11*sizeof(int));

	    llst[0] = strlen(axe_valeur_x);
	    llst[1] = strlen(axe_valeur_y);
	    llst[2] = strlen(axe_valeur_z);
	    llst[3] = strlen(titre_valeur_haut);
	    llst[4] = strlen(titre_valeur_gauche);
	    llst[5] = strlen(titre_valeur_bas);
	    llst[6] = strlen(titre_valeur_droit);

	    register const int tvt = texte_valeur_taille;

	    if(tvt > 0) llst[7] = strlen(texte_valeur[0]);

	    if(tvt > 1) llst[8] = strlen(texte_valeur[1]);

	    if(tvt > 2) llst[9] = strlen(texte_valeur[2]);

	    if(tvt > 3) llst[10] = strlen(texte_valeur[3]);

	    fnxwriti(tf,_type_fin,llst);
	    fnxwrite(tf,axe_valeur_x,llst[0]);
	    fnxwrite(tf,axe_valeur_y,llst[1]);
	    fnxwrite(tf,axe_valeur_z,llst[2]);
	    fnxwrite(tf,titre_valeur_haut,llst[3]);
	    fnxwrite(tf,titre_valeur_gauche,llst[4]);
	    fnxwrite(tf,titre_valeur_bas,llst[5]);
	    fnxwrite(tf,titre_valeur_droit,llst[6]);

	    if(tvt > 0) fnxwrite(tf,texte_valeur[0],llst[7]);

	    if(tvt > 1) fnxwrite(tf,texte_valeur[1],llst[8]);

	    if(tvt > 2) fnxwrite(tf,texte_valeur[2],llst[9]);

	    if(tvt > 3) fnxwrite(tf,texte_valeur[3],llst[10]);

	}
	else if(_type_fin == 10)
	{
	    int llst[10];

	    memset(llst,0,10*sizeof(int));

	    llst[0] = strlen(axe_valeur_x);
	    llst[1] = strlen(axe_valeur_y);
	    llst[2] = strlen(titre_valeur_haut);
	    llst[3] = strlen(titre_valeur_gauche);
	    llst[4] = strlen(titre_valeur_bas);
	    llst[5] = strlen(titre_valeur_droit);

	    register const int tvt = texte_valeur_taille;

	    if(tvt > 0) llst[6] = strlen(texte_valeur[0]);

	    if(tvt > 1) llst[7] = strlen(texte_valeur[1]);

	    if(tvt > 2) llst[8] = strlen(texte_valeur[2]);

	    if(tvt > 3) llst[9] = strlen(texte_valeur[3]);

	    fnxwriti(tf,_type_fin,llst);
	    fnxwrite(tf,axe_valeur_x,llst[0]);
	    fnxwrite(tf,axe_valeur_y,llst[1]);
	    fnxwrite(tf,titre_valeur_haut,llst[2]);
	    fnxwrite(tf,titre_valeur_gauche,llst[3]);
	    fnxwrite(tf,titre_valeur_bas,llst[4]);
	    fnxwrite(tf,titre_valeur_droit,llst[5]);

	    if(tvt > 0) fnxwrite(tf,texte_valeur[0],llst[6]);

	    if(tvt > 1) fnxwrite(tf,texte_valeur[1],llst[7]);

	    if(tvt > 2) fnxwrite(tf,texte_valeur[2],llst[8]);

	    if(tvt > 3) fnxwrite(tf,texte_valeur[3],llst[9]);

	}

	const char aro = '@';

	fnxwrite(tf,&aro,1);
	fnxflush(tf);
	TimerDetReport(1,1,"Temps calcul ecriture binaire");

    // fin ecriture binaire

	return ost;
}

// **************************************************************
//   operateur de lecture fichiers binaire Psyche
// **************************************************************

nxstream& operator >> (nxstream& ist,S6Points& p)
{
	p >> ist;
	return ist;
}

nxstream& S6Points::operator >> (nxstream& ist)
{
	TimerDetInit();

    // ------------------------------------------------
    //     initialisations
    // ------------------------------------------------

	const int ihm_existe = S6GereEnv::ihm && S6GereFenet::root;
	const int ihm_misajour = ihm_existe && calcul_ihm;
	int type = 0;
	FILE* const tf = ist.stdiofile();
	int ier = fnxnbkri(tf,&type,1);

	if(ier < 1)
	{
	    S6FlagsSup::socket = FALSE;
	    ist.clear(ios::failbit | ist.rdstate());

	    TimerDetReport("Fin lecture ou fermeture de connection differee");

	    return ist;
	}

	_type = TypeFichier(type);

	switch(_type)
	{
	    case S6BASE_2D :
	    case S6PARRECT_4D :
	    case S6RECT_3D :
	    case S6RECT_4D :
	    case S6RECT_3D_MAILLE :
	    case S6RECT_4D_MAILLE :
	    case S6SYNCRO :
	    case S6CONFIG_SEULE :
	    case S6LOGIQUE_3D :
	    case S6PHYSIQUE_2D :
	    case S6PHYSIQUE_3D :
	    case S6PHYSIQUE_4D :
	    case S6PHYSIQUE_3D_MAILLE :
	    case S6PHYSIQUE_4D_MAILLE :
	    case S6PHYSIQUE_4D_NS :
	    case S6PHYSIQUE_4D_NS_MAILLE :
	    case S6PHYSIQUE_4D_NS_BRAS :
	        cout << "S6Points> type " <<
		    type_fichier_name[type+_decal_type_fichier] << endl;
		break;

	    case S6OPEN :
		S6FlagsSup::socket = FALSE;
	        cout << "S6Points> type " <<
		    type_fichier_name[type+_decal_type_fichier] << endl;
		TimerDetReport(1,1,"Temps calcul lecture binaire");

		return ist;

	    default :
		S6FlagsSup::socket = FALSE;
	        cout << "S6Points> type " << type << ' ' << ntohl(type) << endl;
		cerr << "Erreur : pb lecture (mauvais format) : " <<
		    __FILE__ << "  ligne : " << __LINE__ << endl;
		fnxclose(tf);
		ist.clear(ios::failbit | ist.rdstate());
		TimerDetReport(1,1,"Temps calcul lecture binaire");

		return ist;
	}

	if(S6FlagsSup::socket && _type == S6SYNCRO)
	{
	    int ret = 0;

	    fnxwriti(tf,1,&ret);
	    fnxflush(tf);
	    S6FlagsSup::socket = FALSE;

	    TimerDetReport(1,1,"S6Points> Synchronisation faite\n"
			    "Temps calcul lecture binaire");

	    return ist;
	}

	if(ihm_misajour) S6GereFenet::info->Change(S6Traduc::traduc[6]);

	type = 0;

	const int test_base = _type == S6BASE_2D;

	if(!test_base)
	{
	    ier = fnxreadi(tf,1,&type);

	    cout << "S6Points> ny " << type << endl;

	    if(ier != 0)
	    {

		if(ihm_misajour) S6GereFenet::info->Change(S6Traduc::traduc[4]);

		cerr << "Erreur : pb lecture : " << __FILE__ << "  ligne : " <<
		    __LINE__ << endl;

		fnxclose(tf);
		ist.clear(ios::failbit | ist.rdstate());
		TimerDetReport(1,1,"Temps calcul lecture binaire");

		return ist;
	    }

	}

	char* fichier_temp = NULL;
	FILE* file_fichier_temp = NULL;
	P4Boolean lopen = TRUE;
	P4Boolean utilisation_fichier_temp = FALSE;
	int nx = 0;

	if(!test_base)
	{
	    fichier_temp = FichierTemp();
	    EnleveBase();
	}

	if(!test_base && !S6FlagsSup::lit_config && (S6FlagsSup::socket ||
		(_type == S6PHYSIQUE_2D && option_2d_concatene != 0)))
	{
	    delete[] S6NameComp::fichier_config_attach;

	    const size_t lfca = strlen(fichier_temp) + 3;

	    S6NameComp::fichier_config_attach = new char[lfca];
	    memcpy(S6NameComp::fichier_config_attach,fichier_temp,lfca-3);
	    memcpy(S6NameComp::fichier_config_attach+lfca-3,"rc",3);
	    utilisation_fichier_temp = TRUE;
	}

	if(_type == S6CONFIG_SEULE)
	{
	    _type_fin = type;
	    lopen = FALSE;

	    if(S6FlagsSup::socket && ihm_existe && config_sauvegarde)
	    {
		S6NameComp::AjoutePileFichiersPsy(
			S6NameComp::fichier_config_attach);
		S6NameComp::num_fichier_psy =
			S6NameComp::taille_pile_psy-1;
		S6GereFenet::nom_fichier->
			Change(S6NameComp::fichier_config_attach);
		S6GereFenet::gestion_fichier->
			ChangePsy(S6NameComp::fichier_config_attach);
		S6GereFenet::gestion_fichier->
			ChangeNumPsy(S6NameComp::num_fichier_psy + 1);
	    }

	}
	else   // lecture _donnee_abstraite
	{
	    register const int test_purge1 =
		_type == S6PHYSIQUE_2D && (option_2d_concatene == 0 ||
		S6FlagsSup::choix_trace != S6FlagsSup::TRACE_2D);
	    register const int test_purge2 =
		_type != S6PHYSIQUE_2D && (option_3d_concatene == 0 ||
		S6FlagsSup::choix_trace == S6FlagsSup::TRACE_2D ||
		option_3d_concatene > 1 &&
		int(_nb_fichiers_concat) == option_3d_concatene);

	    if(test_purge1 || test_purge2 || test_base)
	    {
		Enleve();
		_nb_fichiers_concat = 0;
	    }

	    if(!S6FlagsSup::socket && (_type != S6PHYSIQUE_2D ||
		option_2d_concatene == 0)) lopen = FALSE;

	    if(test_base) lopen = FALSE;

	    if(lopen)
	    {
		const size_t lfca = strlen(fichier_temp) + 5;
		char* const fca = new char[lfca];

		memcpy(fca,fichier_temp,lfca-5);

		if(!strcmp(fichier_sauvegarde,"binaire"))
		{
		    memcpy(fca+lfca-5,".spx",5);
		}
		else if(!strcmp(fichier_sauvegarde,"ascii"))
		{
		    memcpy(fca+lfca-5,".sp3",5);
		}
		else if(!strcmp(fichier_sauvegarde,"ucd"))
		{
		    memcpy(fca+lfca-5,".inp",5);
		}
		else
		{
		    *fca = '\0';
		    lopen = FALSE;
		}

		if(lopen)
		{
		    delete[] fichier_temp;

		    fichier_temp = fca;
		    utilisation_fichier_temp = TRUE;

		    cout << "S6Points> fichier_temp " << fichier_temp << endl;

		    file_fichier_temp = fnxopen(fichier_temp,"w");

		    if(!file_fichier_temp)
		    {
			cerr << "Erreur : pb ouverture fichier .temp.pr , "
			    "pas de sauvegarde" << endl;
			lopen = FALSE;
		    }
		    else
		    {
			ier = fnxchfrm(file_fichier_temp,"ie32");
		    }

		}

		if(S6FlagsSup::socket && lopen)
		{
		    delete[] S6NameComp::fichier_config_attach;

		    const size_t lfca2 = strlen(fichier_temp) + 3;

		    S6NameComp::fichier_config_attach = new char[lfca2];
		    memcpy(S6NameComp::fichier_config_attach,
				fichier_temp,lfca2-3);
		    memcpy(S6NameComp::fichier_config_attach+lfca2-3,"rc",3);
		}

	    }

	    int ny = type;
	    int nz = 0;
	    Tabint nxsto;

	    if(!test_base)
	    {

		if(_type != S6PARRECT_4D)
		{
		    nxsto.Agrand(0,ny);
		    ier = fnxreadi(tf,ny,&nxsto[0]);
		}
		else
		{
		    nx = ny;
		    ier = fnxreadi(tf,1,&ny);
		    ier = fnxreadi(tf,1,&nz);
		}

		if(ier)
		{

		    if(ihm_misajour)
			S6GereFenet::info->Change(S6Traduc::traduc[4]);

		    cerr << "Erreur : pb lecture : " << __FILE__ <<
			" ligne : " << __LINE__ << endl;

		    if(lopen) fnxclose(file_fichier_temp);

		    delete[] fichier_temp;

		    fnxclose(tf);
		    ist.clear(ios::failbit | ist.rdstate());
		    TimerDetReport(1,1,"Temps calcul lecture binaire");

		    return ist;
		}

		int testieee = _type == 0 && ny == 0;

		if(int(nxsto))
		{
		    nx = nxsto[0];

		    for(int i=1;i<ny;i++)
		    {
			nx = max(nx,nxsto[i]);
		    }

		    cout << "S6Points> nx = " << nx << ", ny = " << ny << endl;
		    cout << "S6Points> type " << _type << endl;

		    testieee = _type == 0 && nxsto[0] == 0;
		}

		if(testieee)
		{

		    if(ihm_misajour)
			S6GereFenet::info->Change(S6Traduc::traduc[4]);

		    delete[] fichier_temp;

		    cerr << "Erreur : votre fichier spx est-il "
			"au format ieee 32 bits " << __FILE__ <<
			" ligne " << __LINE__ << endl;

		    fnxclose(tf);
		    ist.clear(ios::failbit | ist.rdstate());
		    TimerDetReport(1,1,"Temps calcul lecture binaire");

		    return ist;
		}

	    }

	// ------------------------------------------------
	//     lecture des donnees abstraites
	// ------------------------------------------------

	    S6DonneeAbstraite* sda = NULL;
	    int nmailles = 0,dimmailles = 0;
	    int i = 0;

	    switch(_type)
	    {
	      case S6OPEN :
	      case S6SYNCRO :
	      case S6CONFIG_SEULE :

		break;

	      case S6BASE_2D :

		LectureBase(tf);

		break;

	      case S6LOGIQUE_3D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->Typemailles(S6RECT);
		sda->X().ModValn(nx);
		sda->Y().ModValn(ny);
		sda->Z().ModValn(nx*ny);

		ier = fnxreadf(tf,nx*ny,sda->Z());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;
		sda->C().Lier(sda->Z());

		for(i=0;i<nx;i++)
		{
		    sda->X()[i] = i;
		}

		for(i=0;i<ny;i++)
		{
		    sda->Y()[i] = i;
		}

		break;

	      case S6PHYSIQUE_2D :

		for(i=0;i<ny;i++)
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(_type);
		    sda->Nx(nxsto[i]);
		    sda->Ny(1);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_NOEUD);
		    sda->X().ModValn(nxsto[i]);
		    sda->Y().ModValn(nxsto[i]);

		    ier = fnxreadf(tf,nxsto[i],sda->X());
		    ier = fnxreadf(tf,nxsto[i],sda->Y());

		    _donnee_abstraite.Ajoute(sda);
		}

		if(ny>0)
		{
		    _nb_fichiers_concat++;
		}

		break;

	      case S6RECT_3D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->Typemailles(S6RECT);
		sda->X().ModValn(nx);
		sda->Y().ModValn(ny);
		sda->Z().ModValn(nx*ny);

		ier = fnxreadf(tf,nx,sda->X());
		ier = fnxreadf(tf,ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;
		sda->C().Lier(sda->Z());

		break;

	      case S6RECT_3D_MAILLE :
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(_type);
		    sda->Nx(nx);
		    sda->Ny(ny);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_MAILLE);
		    sda->Typemailles(S6RECT);
		    sda->X().ModValn(nx);
		    sda->Y().ModValn(ny);

		    register const int nxym = (nx-1) * (ny-1);

		    sda->C().ModValn(nxym);

		    ier = fnxreadf(tf,nx,sda->X());
		    ier = fnxreadf(tf,ny,sda->Y());
		    ier = fnxreadf(tf,nxym,sda->C());

		    sda->Z().ModValn(nx*ny);

		    register REEL_S* const z = sda->Z();
		    register const REEL_S* const c = sda->C();
		    register const int nx1 = nx-1;

		    for(register int k=0;k<nx;k++)
		    {
			register const int kb = min(k,nx-2);
			register const int kb1 = max(k-1,0);

			for(register int kk=0;kk<ny;kk++)
			{
			    register const int kkb = min(kk,ny-2);
			    register const int kkb1 = max(kk-1,0);

			    z[k+nx*kk] = (c[kb+nx1*kkb]+c[kb1+nx1*kkb]+
					    c[kb+nx1*kkb1]+c[kb1+nx1*kkb1])*.25;
			}

		    }

		    _donnee_abstraite.Ajoute(sda);
		    _nb_fichiers_concat++;
		}

		break;

	      case S6PHYSIQUE_3D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->X().ModValn(nx*ny);
		sda->Y().ModValn(nx*ny);
		sda->Z().ModValn(nx*ny);

		ier = fnxreadf(tf,nx*ny,sda->X());
		ier = fnxreadf(tf,nx*ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;
		sda->C().Lier(sda->Z());

		break;

	      case S6PHYSIQUE_3D_MAILLE :
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(_type);
		    sda->Nx(nx);
		    sda->Ny(ny);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_MAILLE);
		    sda->X().ModValn(nx*ny);
		    sda->Y().ModValn(nx*ny);

		    register const int nxym = (nx-1) * (ny-1);

		    sda->C().ModValn(nxym);

		    ier = fnxreadf(tf,nx*ny,sda->X());
		    ier = fnxreadf(tf,nx*ny,sda->Y());
		    ier = fnxreadf(tf,nxym,sda->C());

		    sda->Z().ModValn(nx*ny);

		    register REEL_S* const z = sda->Z();
		    register const REEL_S* const c = sda->C();
		    register const int nx1 = nx-1;

		    for(register int k=0;k<nx;k++)
		    {
			register const int kb = min(k,nx-2);
			register const int kb1 = max(k-1,0);

			for(register int kk=0;kk<ny;kk++)
			{
			    register const int kkb = min(kk,ny-2);
			    register const int kkb1 = max(kk-1,0);

			    z[k+nx*kk] = (c[kb+nx1*kkb]+c[kb1+nx1*kkb]+
					    c[kb+nx1*kkb1]+c[kb1+nx1*kkb1])*.25;
			}

		    }

		    _donnee_abstraite.Ajoute(sda);
		    _nb_fichiers_concat++;
		}

		break;

	      case S6RECT_4D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->Typemailles(S6RECT);
		sda->X().ModValn(nx);
		sda->Y().ModValn(ny);
		sda->Z().ModValn(nx*ny);
		sda->C().ModValn(nx*ny);

		ier = fnxreadf(tf,nx,sda->X());
		ier = fnxreadf(tf,ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());
		ier = fnxreadf(tf,nx*ny,sda->C());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;

		break;

	      case S6RECT_4D_MAILLE :
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(_type);
		    sda->Nx(nx);
		    sda->Ny(ny);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_MAILLE);
		    sda->Typemailles(S6RECT);
		    sda->X().ModValn(nx);
		    sda->Y().ModValn(ny);
		    sda->Z().ModValn(nx*ny);

		    register const int nxym = (nx-1) * (ny-1);

		    sda->C().ModValn(nxym);

		    ier = fnxreadf(tf,nx,sda->X());
		    ier = fnxreadf(tf,ny,sda->Y());
		    ier = fnxreadf(tf,nx*ny,sda->Z());
		    ier = fnxreadf(tf,nxym,sda->C());

		    _donnee_abstraite.Ajoute(sda);
		    _nb_fichiers_concat++;
		}

		break;

	      case S6PHYSIQUE_4D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->X().ModValn(nx*ny);
		sda->Y().ModValn(nx*ny);
		sda->Z().ModValn(nx*ny);
		sda->C().ModValn(nx*ny);

		ier = fnxreadf(tf,nx*ny,sda->X());
		ier = fnxreadf(tf,nx*ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());
		ier = fnxreadf(tf,nx*ny,sda->C());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;

		break;

	      case S6PHYSIQUE_4D_MAILLE :
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(_type);
		    sda->Nx(nx);
		    sda->Ny(ny);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_MAILLE);
		    sda->X().ModValn(nx*ny);
		    sda->Y().ModValn(nx*ny);
		    sda->Z().ModValn(nx*ny);

		    register const int nxym = (nx-1) * (ny-1);

		    sda->C().ModValn(nxym);

		    ier = fnxreadf(tf,nx*ny,sda->X());
		    ier = fnxreadf(tf,nx*ny,sda->Y());
		    ier = fnxreadf(tf,nx*ny,sda->Z());
		    ier = fnxreadf(tf,nxym,sda->C());

		    _donnee_abstraite.Ajoute(sda);
		    _nb_fichiers_concat++;
		}

		break;

	      case S6PHYSIQUE_4D_NS :

	    // Lecture donnees non structures 4D surfacique

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_NOEUD);
		sda->X().ModValn(nx*ny);
		sda->Y().ModValn(nx*ny);
		sda->Z().ModValn(nx*ny);
		sda->C().ModValn(nx*ny);

		ier = fnxreadf(tf,nx*ny,sda->X());
		ier = fnxreadf(tf,nx*ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());
		ier = fnxreadf(tf,nx*ny,sda->C());

		ier = fnxreadi(tf,1,&nmailles);
		ier = fnxreadi(tf,1,&dimmailles);

		cout << "S6Points> nmailles = " << nmailles <<
		    ", dimmailles = " << dimmailles << endl;

		sda->Nmailles(unsigned(nmailles));
		sda->Mailles().ModValn(unsigned(dimmailles));

		ier = fnxreadi(tf,dimmailles,(int*)
				(unsigned*) sda->Mailles());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;

	    // Appel correcteur du tableau mailles
		CorrectMailles(sda->Mailles(),nmailles,nx*ny);
		break;

	      case S6PHYSIQUE_4D_NS_MAILLE :

	    // Lecture donnees non structures 4D surfacique
	    // ou C sont des donnees valeurs aux mailles

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(1);
		sda->Typec(S6DONNEE_MAILLE);
		sda->X().ModValn(nx*ny);
		sda->Y().ModValn(nx*ny);
		sda->Z().ModValn(nx*ny);

		ier = fnxreadf(tf,nx*ny,sda->X());
		ier = fnxreadf(tf,nx*ny,sda->Y());
		ier = fnxreadf(tf,nx*ny,sda->Z());

		ier = fnxreadi(tf,1,&nmailles);
		ier = fnxreadi(tf,1,&dimmailles);

		sda->C().ModValn(nmailles);

		ier = fnxreadf(tf,nmailles,sda->C());

		sda->Nmailles(unsigned(nmailles));
		sda->Mailles().ModValn(unsigned(dimmailles));

		ier = fnxreadi(tf,dimmailles,(int*) (unsigned*) sda->Mailles());

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;

	    // Appel correcteur du tableau mailles
		CorrectMailles(sda->Mailles(),nmailles,nx*ny);
		break;

	      case S6PHYSIQUE_4D_NS_BRAS :
		{
		    sda = new S6DonneeAbstraite;
		    sda->Typefichier(S6PHYSIQUE_4D_NS);
		    sda->Nx(nx);
		    sda->Ny(ny);
		    sda->Nz(1);
		    sda->Typec(S6DONNEE_NOEUD);

		    const int nxy = nx*ny;

		    sda->X().ModValn(nxy);
		    sda->Y().ModValn(nxy);
		    sda->Z().ModValn(nxy);
		    sda->C().ModValn(nxy);

		// mailles - noeuds
		// bras - noeuds

		    ier = fnxreadf(tf,nxy,sda->X());
		    ier = fnxreadf(tf,nxy,sda->Y());
		    ier = fnxreadf(tf,nxy,sda->Z());

		    int nbras = 0;

		    ier = fnxreadi(tf,1,&nbras);

		    TabREEL_S cbras(nbras);

		    ier = fnxreadf(tf,nbras,cbras);
		    ier = fnxreadi(tf,1,&nmailles);
		    ier = fnxreadi(tf,1,&dimmailles);
		    sda->Nmailles(unsigned(nmailles));
		    sda->Mailles().ModValn(unsigned(dimmailles));
		    ier = fnxreadi(tf,dimmailles,
				    (int*) (unsigned*) sda->Mailles());

		    int* bras = new int[2*nbras];

		    ier = fnxreadi(tf,2*nbras,bras);
		    DonneBrasSommetNS(cbras,sda->C(),bras);

		    delete[] bras;

		    _donnee_abstraite.Ajoute(sda);
		    _nb_fichiers_concat++;

		// Appel correcteur du tableau mailles

		    CorrectMailles(sda->Mailles(),nmailles,nx*ny);
		}
		break;

	      case S6PARRECT_4D :

		sda = new S6DonneeAbstraite;
		sda->Typefichier(_type);
		sda->Nx(nx);
		sda->Ny(ny);
		sda->Nz(nz);
		sda->Typec(S6DONNEE_NOEUD);
		sda->Typemailles(S6RECT);
		sda->X().ModValn(nx*ny*nz);
		sda->Y().ModValn(nx*ny*nz);
		sda->Z().ModValn(nx*ny*nz);
		sda->C().ModValn(nx*ny*nz);

		{
		    register REEL_S* const x = sda->X();

		    ier = fnxreadf(tf,nx,x);

		    for(i=1;i<ny;i++)
		    {
			REEL_S* const xi = x + i*nx;

			memcpy(xi,x,nx*sizeof(REEL_S));
		    }

		    for(i=1;i<nz;i++)
		    {
			REEL_S* const xi = x + i*nx*ny;

			memcpy(xi,x,nx*ny*sizeof(REEL_S));
		    }

		    register REEL_S* const y = sda->Y();

		    ier = fnxreadf(tf,ny,y);

		    for(i=ny-1;i>=0;i--)
		    {
			register const REEL_S yi = y[i];
			register REEL_S* const yinx = y + i*nx;

			for(register int j=0;j<nx;j++)
			{
			    yinx[j] = yi;
			}

		    }

		    for(i=1;i<nz;i++)
		    {
			REEL_S* const yi = y + i*nx*ny;

			memcpy(yi,y,nx*ny*sizeof(REEL_S));
		    }

		    register REEL_S* const z = sda->Z();

		    ier = fnxreadf(tf,nz,z);

		    for(i=nz-1;i>=0;i--)
		    {
			register const REEL_S zi = z[i];
			register REEL_S* const zinx = z + i*nx*ny;

			for(register int j=0;j<nx*ny;j++)
			{
			    zinx[j] = zi;
			}

		    }

		    ier = fnxreadf(tf,nx*ny*nz,sda->C());
		}

		_donnee_abstraite.Ajoute(sda);
		_nb_fichiers_concat++;

		break;
	    }

	    _type_fin = -1;

	    if(_type != S6BASE_2D)
	    {
		ier = fnxreadi(tf,1,&_type_fin);
	    }

	    if(_type == S6PHYSIQUE_4D_NS_BRAS)
	    {
		_type = S6PHYSIQUE_4D_NS;
	    }

	    cout << "S6Points> code config lecture " << _type_fin << endl;
	} // fin lecture _donnee_abstraite

	if(utilisation_fichier_temp) num_fich_temp++;

    // lecture config

	P4Boolean lopen2 = FALSE;
	P4Boolean config_transmise = FALSE;

	if(_type_fin == 10 || _type_fin == 11)
	{
	    int llst[11];

	    ier = fnxreadi(tf,_type_fin,llst);
	    ier = LectureTexte(tf,axe_valeur_x,llst[0]);
	    ier = LectureTexte(tf,axe_valeur_y,llst[1]);

	    int il = 2;

	    if(_type_fin == 11)
	    {
		ier = LectureTexte(tf,axe_valeur_z,llst[2]);il++;
	    }

	    ier = LectureTexte(tf,titre_valeur_haut,llst[il]);il++;
	    ier = LectureTexte(tf,titre_valeur_gauche,llst[il]);il++;
	    ier = LectureTexte(tf,titre_valeur_bas,llst[il]);il++;
	    ier = LectureTexte(tf,titre_valeur_droit,llst[il]);il++;

	    if(texte_valeur_taille < 4)
	    {
		char** val = texte_valeur;

		texte_valeur = new char*[4];

		memcpy(texte_valeur,val,texte_valeur_taille*sizeof(char*));
		memset(&texte_valeur[texte_valeur_taille],0,
				(4-texte_valeur_taille)*sizeof(char*));

		delete[] val;

		texte_valeur_taille = 4;
	    }

	    for(register int nt=0;nt<4;nt++)
	    {
		ier = LectureTexte(tf,texte_valeur[nt],llst[il+nt]);
	    }

	}
	else if(_type_fin > 100)
	{
	    FILE* file_config_temp = NULL;
	    P4Boolean first = config_sauvegarde ? TRUE : FALSE;
	    int nb_mots_cles = _type_fin - 100;
	    int ityy[2];

	    if(nb_mots_cles > 10000) // valeurs abberantes
	    {
		cerr << "Erreur : trop de mots cles " << __FILE__ << endl;

		nb_mots_cles = 0;
	    }

	    for(int i=0;i<nb_mots_cles;i++)
	    {
		ier = fnxreadi(tf,2,ityy);

		if(ier != 0) break;

		if(first)
		{
		    file_config_temp =
			fnxopen(S6NameComp::fichier_config_attach,"w");
		    lopen2 = TRUE;

		    if(!file_config_temp)
		    {
			cerr << "Erreur :  pb ouverture fichier $IMAGES_SP3, " <<
				"pas de sauvegarde dans " <<
				S6NameComp::fichier_config_attach << endl;
			lopen2 = FALSE;
		    }

		    first = FALSE;
		}

		char* texi = NULL;
		int ival = 0;
		REEL_S xval = 0;
		int lval = 0;
		char* chara = NULL;

		if(ityy[1] < 0) break;

		texi = new char[ityy[1]+1];
		texi[ityy[1]] = '\0';
		ier = fnxread(tf,texi,ityy[1]);

		if(ityy[0] == 0)
		{
		    ier = fnxreadi(tf,1,&ival);
		}
		else if(ityy[0] == 1)
		{
		    ier = fnxreadf(tf,1,&xval);
		}
		else if(ityy[0] == 2)
		{
		    ier = fnxreadi(tf,1,&lval);
		    chara = new char[lval+1];
		    chara[lval] = '\0';
		    ier = fnxread(tf,chara,lval);
		}
		else
		{
		    delete[] texi;
		    break;
		}

		config_transmise = TRUE;

		if(lopen2)
		{
		    const size_t ldentif = strlen(S6GereConfig::_identif);
		    const size_t ltexi = strlen(texi);
		    const size_t lptos = ldentif+ltexi+4;
		    char* ptos = new char[lptos];

		    memcpy(ptos,S6GereConfig::_identif,ldentif);
		    ptos[ldentif] = '.';
		    memcpy(ptos+ldentif+1,texi,ltexi);
		    memcpy(ptos+lptos-3,":  ",3);

		    ier = fnxwrite(file_config_temp,ptos,lptos);
		    fnxflush(file_config_temp);
		   
		    delete[] ptos;
		}

		char* ptos = NULL;

	        {
		    ostrstream os;

		    if(ityy[0] == 0)
		    {
			C5Config* c = new C5ConfigInt(S6GereConfig::_identif,
					texi,ival);

			_config_lue_socket.Ajoute(c);
			os << ival << ends;
		    }
		    else if(ityy[0] == 1)
		    {
			C5Config* c = new C5ConfigReel(
				     S6GereConfig::_identif,texi,xval);

			_config_lue_socket.Ajoute(c);
			os << xval << ends;
		    }
		    else if(ityy[0] == 2)
		    {

			if(chara[0] == '[')
			{
			    os << chara << ends;
			    chara[strlen(chara)-1] = '\0';

			    C5Config* c = new C5ConfigChar(
					 S6GereConfig::_identif,texi,chara+1);

			    _config_lue_socket.Ajoute(c);
			}
			else
			{
			    os << "\"" << chara << "\"" << ends;

			    C5Config* c = new C5ConfigChar(
					 S6GereConfig::_identif,texi,chara);

			    _config_lue_socket.Ajoute(c);
			}

		    }
		    else
		    {
			delete[] texi;

			break;
		    }

		    ptos = os.str();
		}

		delete[] texi;delete[] chara;

		if(lopen2)
		{
		    const char nl = '\n';

		    ier = fnxwrite(file_config_temp,ptos,strlen(ptos));
		    ier = fnxwrite(file_config_temp,&nl,1);
		}

		delete[] ptos;
	    }

	    if(lopen2)
	    {
		ier = fnxclose(file_config_temp);
	    }

	}
	else if(_type_fin == 0 && !S6FlagsSup::socket)
	{
	    char bid[1];

	    ier = fnxread(tf,bid,1); // cas 0@ de la fin des fichiers lib
	}

    // fin lecture fichier binaire -> _donnee_abstraite

    // ecriture _donnee_abstraite -> fichier binaire ou ascii

	if(lopen)
	{
	    Sauve(fichier_temp,file_fichier_temp);
	    ier = fnxclose(file_fichier_temp);
	}

	delete[] fichier_temp;

    // initialisations

	Init(config_transmise,nx);

	TimerDetReport(1,1,"Temps calcul lecture binaire");

	return ist;
}

// ********************************************************
//    initialisations
// ********************************************************

void
S6Points::Init(P4Boolean config,int nx)
{
    // init diverses

#if !defined(IRIX_4)
	S6FlagsSup::S6ChoixTrace choix_trace_old = S6FlagsSup::choix_trace;
#else
	S6ChoixTrace choix_trace_old = S6FlagsSup::choix_trace;
#endif

	if(_type == S6PHYSIQUE_2D)
	{
	    S6FlagsSup::choix_trace = S6FlagsSup::TRACE_2D;
	}
	else if(_type == S6BASE_2D)
	{
	    S6FlagsSup::choix_trace = S6FlagsSup::TRACE_BASE_2D;
	}
	else if(_type != S6CONFIG_SEULE)
	{

	    if(S6FlagsSup::choix_trace != S6FlagsSup::TRACE_COUPE)
	    {
		S6FlagsSup::choix_trace = S6FlagsSup::TRACE_3D;
	    }

	    if(nx == 1)
	    {
		S6FlagsSup::choix_trace = S6FlagsSup::TRACE_TRAJ_3D;
	    }

	}

	const int ihm_existe = S6GereEnv::ihm && S6GereFenet::root;
	const int ihm_misajour = ihm_existe && calcul_ihm;

	if(ihm_misajour) S6GereFenet::info->Change(S6Traduc::traduc[4]);

    // S6FlagsSup::socket : si socket et non fichier
    // S6FlagsSup::lit_config : si fichier config associe existe
    // config : si _config_lue_socket contient quelque chose

	if(config || S6FlagsSup::lit_config)
	{
	    const char* const fca = S6NameComp::fichier_config_attach;

	    if(fca)
	    {
		const size_t taille = strlen(fca) + 1;
		char* fc = new char[taille];

		memcpy(fc,fca,taille);

		delete[] S6NameComp::fichier_config;

		S6NameComp::fichier_config = fc;
	    }

	    if(ihm_existe)
		S6GereFenet::config_p->Change(S6NameComp::fichier_config);

	    if(config && (S6FlagsSup::socket || !S6FlagsSup::lit_config))
	    {
		S6GereConfig::Charge(_config_lue_socket);
		_config_lue_socket.Enleve();

		if(ihm_misajour) S6GereFenet::ReConfig();

	    }
	    else if(S6FlagsSup::lit_config)
	    {
		S6GereConfig::Charge(S6NameComp::fichier_config);

		if(ihm_misajour) S6GereFenet::ReConfig();

	    }

	}
	else
	{
	    S6GereConfigP::_p_axe_config = S6GereConfig::_config;
	    S6GereConfigP::_p_titre_config = S6GereConfig::_config;
	    S6GereConfigP::_p_texte_config = S6GereConfig::_config;

	    if(ihm_misajour) S6GereFenet::ReConfigTTA();

	}

    // conversion eventuelle

	if(option_3d_conv_mu_mo) ConvMultiMono();

	if(S6FlagsSup::choix_trace == S6FlagsSup::TRACE_COUPE)
	{
	    S6FlagsSup::coupe_regenere = TRUE;

	    delete[] S6FlagsSup::axe_valeur_x_save;

	    {
		const size_t taille = strlen(axe_valeur_x) + 1;

		S6FlagsSup::axe_valeur_x_save = new char[taille];
		memcpy(S6FlagsSup::axe_valeur_x_save,axe_valeur_x,taille);
	    }

	    S6FlagsSup::axe_log_x_save = axe_log_x;
	    S6FlagsSup::axe_x_min_save = axe_x_min;
	    S6FlagsSup::axe_x_max_save = axe_x_max;

	    delete[] S6FlagsSup::axe_valeur_y_save;

	    {
		const size_t taille = strlen(axe_valeur_y) + 1;

		S6FlagsSup::axe_valeur_y_save = new char[taille];
		memcpy(S6FlagsSup::axe_valeur_y_save,axe_valeur_y,taille);
	    }

	    S6FlagsSup::axe_log_y_save = axe_log_y;
	    S6FlagsSup::axe_y_min_save = axe_y_min;
	    S6FlagsSup::axe_y_max_save = axe_y_max;

	    if(ihm_misajour && S6GereFenet::coupelogique)
	    {
		S6GereFenet::coupelogique->Change();
		S6GereFenet::ChangeCoupeLogiqueValeurs();
	    }
	    else
	    {
		S6GereFenet::ChangeCoupeLogiqueValeurs();
	    }

	}
	else if(S6FlagsSup::choix_trace == S6FlagsSup::TRACE_2D)
	{
	    S6InitCourbe::Init();

	    if(ihm_misajour)
	    {
		S6GereFenet::ReConfigCourbes();

		if(S6GereFenet::courbes)
		{
		    unsigned nbcourbes = _donnee_abstraite;

		    S6GereFenet::courbes->Change(nbcourbes);
		}

	    }

	}

	if(ihm_misajour) S6GereFenet::info->Change(S6Traduc::traduc[5]);

	if(choix_trace_old != S6FlagsSup::choix_trace)
	{
	    S6GereFenet::CacheExpose();
	}

}

// ********************************************************
//         Nettoyage
// ********************************************************

void
S6Points::Enleve()
{
	register const unsigned taille = _donnee_abstraite;

	for(register unsigned i=0;i<taille;i++)
	{
	    delete _donnee_abstraite[i];
	}

	_donnee_abstraite.Enleve();

	if(S6Trace::graf)
	{
	    S6Trace::graf->P4VideGeom();
	    S6Trace::graf->P4VideGeomCoupe();
	}

}

void
S6Points::EnleveBase()
{
	register unsigned taille = _donnee_abstraite_base;

	for(register unsigned i=0;i<taille;i++)
	{
	    delete _donnee_abstraite_base[i];
	}

	_donnee_abstraite_base.Enleve();
	_recherche_base(0);
	_donnee_abstraite_base.TraceLast(FALSE);
}

void
S6Points::Sauve(const char* const nomfich,FILE* file_nomfich)
{
	const int ihm_existe = S6GereEnv::ihm && S6GereFenet::root;
	nxstream fisobi(file_nomfich);

	if(!strcmp(fichier_sauvegarde,"binaire"))
	{
	// temporaire explicite car bug dans GNU
	    S6Points s6ptemp = *this;

	    fisobi << s6ptemp;
	    fisobi.flush();

	    if(ihm_existe)
	    {
		S6NameComp::AjoutePileFichiersPsy(nomfich);
		S6NameComp::num_fichier_psy = S6NameComp::taille_pile_psy-1;
		S6GereFenet::nom_fichier->Change(nomfich);
		S6GereFenet::gestion_fichier->ChangePsy(nomfich);
		S6GereFenet::gestion_fichier->
		    ChangeNumPsy(S6NameComp::num_fichier_psy + 1);
	    }

	}
	else if(!strcmp(fichier_sauvegarde,"ascii"))
	{
	// temporaire explicite car bug dans GNU
	    S6PointsAscii s6ptemp = *this;

	    fisobi << s6ptemp;
	    fisobi.flush();

	    char* ptos = NULL;

	    {
		ostrstream os;

		os << "compress -f " << nomfich << ends;
		ptos = os.str();
		system(ptos);
	    }

	    delete[] ptos;

	    {
		ostrstream os;

		os << nomfich << ".Z" << ends;
		ptos = os.str();

		if(ihm_existe)
		{
		    S6NameComp::AjoutePileFichiersPsy(ptos);
		    S6NameComp::num_fichier_psy = S6NameComp::taille_pile_psy-1;
		    S6GereFenet::nom_fichier->Change(ptos);
		    S6GereFenet::gestion_fichier->ChangePsy(ptos);
		    S6GereFenet::gestion_fichier->
			ChangeNumPsy(S6NameComp::num_fichier_psy + 1);
		}

	    }

	    delete[] ptos;
	}
	else if(!strcmp(fichier_sauvegarde,"ucd"))
	{
	// temporaire explicite car bug dans GNU
	    S6PointsUcd s6ptemp = *this;

	    fisobi << s6ptemp;
	    fisobi.flush();

	    char* ptos = NULL;

	    {
		ostrstream os;

		os << "compress -f " << nomfich << ends;
		ptos = os.str();
		system(ptos);
	    }

	    delete[] ptos;

	    {
		ostrstream os;

		os << nomfich << ".Z" << ends;
		ptos = os.str();

		if(ihm_existe)
		{
		    S6NameComp::AjoutePileFichiersPsy(ptos);
		    S6NameComp::num_fichier_psy = S6NameComp::taille_pile_psy-1;
		    S6GereFenet::nom_fichier->Change(ptos);
		    S6GereFenet::gestion_fichier->ChangePsy(ptos);
		    S6GereFenet::gestion_fichier->
			ChangeNumPsy(S6NameComp::num_fichier_psy + 1);
		}

	    }

	    delete[] ptos;
	}
	else
	{
	    cout << "S6Points> pas de sauvegarde demandee" << endl;
	}

}

// ************************************************************************
//    genere un fichier de sauvegarde dans $IMAGES_SP3/[.temp.pr]
// ************************************************************************

void
S6Points::Sauve()
{
	P4Boolean lopen = TRUE;
	char* fichier_temp = FichierTemp();
	const size_t lfca = strlen(fichier_temp) + 5;
	char* fca = new char[lfca];

	memcpy(fca,fichier_temp,lfca-5);

	if(!strcmp(fichier_sauvegarde,"binaire"))
	{
	    memcpy(fca+lfca-5,".spx",5);
	}
	else if(!strcmp(fichier_sauvegarde,"ascii"))
	{
	    memcpy(fca+lfca-5,".sp3",5);
	}
	else if(!strcmp(fichier_sauvegarde,"ucd"))
	{
	    memcpy(fca+lfca-5,".inp",5);
	}
	else
	{
	    *fca = '\0';
	    lopen = FALSE;
	}

	if(lopen)
	{
	    delete[] fichier_temp;

	    fichier_temp = fca;

	    cout << "S6Points> fichier_temp = " << fichier_temp << endl;

	    FILE* const file_fichier_temp = fnxopen(fichier_temp,"w");

	    if(!file_fichier_temp)
	    {
		cerr << "S6Points> Erreur ouverture fichier_temp => "
		    "pas de sauvegarde" << endl;
	    }
	    else
	    {
		fnxchfrm(file_fichier_temp,"ie32");
		Sauve(fichier_temp,file_fichier_temp);
		fnxclose(file_fichier_temp);
	    }

	    num_fich_temp++;
	}

	delete[] fichier_temp;
}
