#ifdef CRAY
# include <locale.h>
#endif

#ifdef _IBMR2
#  define _BSD
#endif

#include <ctype.h>
#include "S6gerefenet.h"
#include "S6bouttons4.h"
#include "S6bouttons5.h"
#include "poulpe/WidAffX.h"
#include "S6bouttons8.h"
#include "S6boutfich.h"
#include "nxstream.h"
#include "S6pointsmtv.h"
#include "S6namecomp.h"
#include "S6gereconfig.h"
#include "S6gereenv.h"
#include "S6flagssup.h"
#include "S6traduc.h"
#include "dinosaure/S6timer.h"
#include <float.h>

struct SccsIdS6PointsMtv
{
	static const char* const sccs_id;
};

const char* const SccsIdS6PointsMtv::sccs_id = "%W% %G% P. Brochard";

//*********************************************************
//   implementation de la classe S6PointsMtv
//*********************************************************

//***************************************************************
//     operateur d'ecriture fichiers mtv de RAYSHADE
//***************************************************************

nxstream&
S6PointsMtv::operator << (nxstream& ost) const
{
	TimerDetInit();

	cerr << "Ecriture mtv : non implemente" << endl;

	TimerDetReport(1,1,"Temps calcul ecriture mtv");

    // fin ecriture mtv

	return ost;
}

// **************************************************************
//   operateur de lecture fichiers mtv de RAYSHADE
// **************************************************************

nxstream&
S6PointsMtv::operator >> (nxstream& ist)
{
	TimerDetInit();

	const int ihm_existe = S6GereEnv::ihm && S6GereFenet::root;
	const int ihm_misajour = ihm_existe && calcul_ihm;
	char nxyc[32];

	ist.get(nxyc,32,'\n');
	nxyc[ist.gcount()] = '\0';

	if(ist.fail())
	{
	    S6FlagsSup::socket = FALSE;

	    cout << "S6PointsMtv> "
		"Pb lecture ou fermeture de connection differee" << endl;

	    TimerDetReport(1,1,"Temps calcul lecture mtv");

	    return ist;
	}

	int nx = 0;
	int ny = 0;

	sscanf(nxyc,"%d %d",&nx,&ny);
	_type = S6RECT_3D_MAILLE;

	int nx1 = nx,ny1 = ny;

	nx++;ny++; // passage mailles -> noeuds

	if(ihm_misajour) S6GereFenet::info->Change(S6Traduc::traduc[6]);

	cout << "S6PointsMtv> nx = " << nx << ", ny = " << ny << endl;

	FILE* file_fichier_temp = NULL;
	P4Boolean lopen = TRUE;
	P4Boolean utilisation_fichier_temp = FALSE;
	char* fichier_temp = FichierTemp();

	if(!S6FlagsSup::lit_config && (S6FlagsSup::socket))
	{
	    ostrstream os;

	    os << fichier_temp << "rc" << ends;

	    delete[] S6NameComp::fichier_config_attach;

	    S6NameComp::fichier_config_attach = os.str();
	    utilisation_fichier_temp = TRUE;
	}

	if(option_3d_concatene == 0 ||
	   S6FlagsSup::choix_trace == S6FlagsSup::TRACE_2D ||
	   option_3d_concatene > 1 &&
	   int(_nb_fichiers_concat) == option_3d_concatene)
	{
	    Enleve();
	    _nb_fichiers_concat = 0;
	}

	if(!S6FlagsSup::socket && option_2d_concatene == 0) lopen = FALSE;

	if(lopen)
	{
	    ostrstream os;

	    if(strcmp(fichier_sauvegarde,"binaire") == 0)
	    {
		os << fichier_temp << ".spx" << ends;
	    }
	    else if(strcmp(fichier_sauvegarde,"ascii") == 0)
	    {
		os << fichier_temp << ".sp3" << ends;
	    }
	    else if(strcmp(fichier_sauvegarde,"ucd") == 0)
	    {
		os << fichier_temp << ".inp" << ends;
	    }
	    else
	    {
		lopen = FALSE;
	    }

	    if(lopen)
	    {
		delete[] fichier_temp;

		fichier_temp = os.str();
		utilisation_fichier_temp = TRUE;

		cout << "fichier_temp " << fichier_temp << endl;

		file_fichier_temp = fnxopen(fichier_temp,"w");

		if(!file_fichier_temp)
		{
		    cerr << "pb ouverture fichier .temp.pr ,"
			"pas de sauvegarde " << endl;
		    lopen = FALSE;
		}
		else
		{
		    fnxchfrm(file_fichier_temp,"ie32");
		}

	    }

	}

	if(S6FlagsSup::socket && lopen)
	{
	    delete[] S6NameComp::fichier_config_attach;

	    int lfca = strlen(fichier_temp) + 3;

	    S6NameComp::fichier_config_attach = new char[lfca];
	    memcpy(S6NameComp::fichier_config_attach,fichier_temp,lfca-3);
	    memcpy(S6NameComp::fichier_config_attach+lfca-3,"rc",3);
	}

	S6DonneeAbstraite* sda = new S6DonneeAbstraite;

	sda->Typefichier(_type);
	sda->Nx(nx);
	sda->Ny(ny);
	sda->Nz(1);
	sda->Typec(S6DONNEE_MAILLE);
	sda->Typemailles(S6RECT);

	register int nxym = nx1 * ny1;

	sda->C().ModValn(nxym);

	REEL_S* const c = sda->C();

	int tnxy = 3*nx1*ny1;
	unsigned char* data_temp = new unsigned char[tnxy];
	FILE* tf = ist.stdiofile();

	fnxread(tf,(char*) data_temp,tnxy);

    /* sauvegarde en sun raster :

       FILE* ff = fnxopen("/tmp/sunrast","w");
       int mag = 0x59a66a95;
       int res = 24;
       int zer = 0;

       fnxwriti(ff,1,&mag);
       fnxwriti(ff,1,&nx1);
       fnxwriti(ff,1,&ny1);
       fnxwriti(ff,1,&res);
       fnxwriti(ff,1,&tnxy);
       fnxwriti(ff,1,&zer);
       fnxwriti(ff,1,&zer);
       fnxwriti(ff,1,&zer);
       fnxwrite(ff,(char*) data_temp,tnxy);
       fnxclose(ff);
       */

	const int nbstdcol = 255;
	const int nbpal = parametre_map_pal_t / 3;
	register int nbred = int(pow(nbpal/2.,1./3.)+.1);
	register int nbred1 = nbred-1;
	register int nbred2 = 2*nbred;
	register int nbred21 = 2*nbred-1;
	register int nbredc = nbred2*nbred;
	register int nbred3 = nbredc*nbred;
	register int nbred6 = 2*nbred3;
	register int nx21 = 2*nx1;
	register int nxym1 = nxym - nx1;
	register int i3 = 0;
	int i = 0;

    // calcul de C (depend de la taille de la colormap)

	for(i=0;i<nxym;i++)
	{
	    register int aa = int((double(nbred1)/nbstdcol)*data_temp[i3++]);
	    register int bb = int((double(nbred21)/nbstdcol)*data_temp[i3++]);
	    register int cc = int((double(nbred1)/nbstdcol)*data_temp[i3++]);
	    register int k = i + nxym1 - (i/nx1)*nx21;

	    c[k] = aa*nbredc + bb*nbred + cc;
	}

	delete[] data_temp;

    // calcul colormap

	delete[] parametre_map_pal;

	register int* pmp = new int[3*nbred3];

	for(i=0;i<nbred3;i++)
	{
	    register REEL aa = i%nbred;
	    register REEL bb = (i/nbred)%nbred2;
	    register REEL cc = i/nbredc;

	    pmp[i] = int((nbstdcol*cc)/nbred1);
	    pmp[i+nbred3] = int((nbstdcol*bb)/nbred21);
	    pmp[i+nbred6] = int((nbstdcol*aa)/nbred1);
	}

	parametre_map_pal = pmp;
	parametre_map_pal_t = 3*nbred3;
	parametre_map = -1;
	S6GereConfig::InitConfig();
	_donnee_abstraite.Ajoute(sda);
	_nb_fichiers_concat++;

    // calcul de X et Y

	sda->X().ModValn(nx);

	REEL_S* const x = sda->X();

	for(i=0;i<nx;i++)
	{
	    x[i] = i;
	}
	
	sda->Y().ModValn(ny);

	REEL_S* const y = sda->Y();

	for(int j=0;j<ny;j++)
	{
	    y[j] = j;
	}

    // calcul de Z au noeud sachant que C est aux mailles et Z = C

	register int nxy = nx*ny;

	sda->Z().ModValn(nxy);

	REEL_S* const z = sda->Z();

	for(register int k=0;k<nx;k++)
	{
	    register int kb = min(k,nx-2);
	    register int kb1 = max(k-1,0);

	    for(register int kk=0;kk<ny;kk++)
	    {
		register int kkb = min(kk,ny-2);
		register int kkb1 = max(kk-1,0);

		z[k+nx*kk] = (c[kb+nx1*kkb]+c[kb1+nx1*kkb]+
				c[kb+nx1*kkb1]+c[kb1+nx1*kkb1])*.25;
	    }

	}

	if(utilisation_fichier_temp) num_fich_temp++;

    // ecriture _donnee_abstraite -> fichier binaire ou ascii

	if(lopen)
	{
	    Sauve(fichier_temp,file_fichier_temp);
	    fnxclose(file_fichier_temp);
	}

	delete[] fichier_temp;

    //   initialisations

	Init(FALSE,nx);
	S6GereFenet::ReConfigEchelles();
	S6GereFenet::ReConfigCouleurs();

	if(S6GereFenet::mapdecouleur)
	    S6GereFenet::mapdecouleur->Change(parametre_map);

	if(S6GereFenet::gestioncouleur)
	    S6GereFenet::gestioncouleur->Change();

	TimerDetReport(1,1,"Temps calcul lecture mtv");

	return ist;
}
