/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992    MA/AIM";
#endif

/***************************************************************************
  etendu le 5/9/96 par P. Brochard pour autoriser les sockets AF_UNIX
  etendu le 6/11/96 par P. Brochard pour le calcul automatique du port socket
 ***************************************************************************/

#include "nxP.h"
#include "U8machine_types.h"

#include <regex.h>
#include <poll.h>

#include "nxcode.h"
#include "nxerror.h"
#include "nxfonc.h"
#include "nxsystem.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"
#include "nxoptim.h"
#include "nx_free.h"

#if defined(EPIX) && defined(_SYSTYPE_BSD43)
    typedef int mode_t;
#endif

/*****************************************************/
/* macros utiles */
/*****************************************************/

#define NX_LFORMAT 6

#if defined(NX_SOCKET_BSD) /* BSD */
#   define NX_ISSOCKET(mode) (((mode)&S_IFMT) == S_IFSOCK)
#else /* SYSV */
#   define NX_ISSOCKET(mode) (((mode)&S_IFMT) == S_IFIFO)
#endif

/*****************************************************/
/* variables statiques */
/*****************************************************/

static struct
{
	char format[NX_LFORMAT];
	int fmt; 
} nxtabfrm[NX_MAX_DESC] = {0};

struct form
{
	char* typfor;
};

static struct form formats[] = {{"cr64"},{"ie32"},{"li32"},{"ie64"},{"li64"},
				{"vms32"},{"xdr"}};
static const int maxform = sizeof(formats)/sizeof(struct form);

/* correspondance code - nb d'octets - type general */

static struct nxcofor nxtypc[] =  
                             {{"stdf", NX_SIZE_INT, NEUTRE}, /* format machine */
                              {"cr64", 8, REEL},
                              {"ie32", 4, ENTIER},
			      {"li32", 4, ENTIER}, /* little endian ieee32 */
			      {"ie64", 8, REEL}, /* CRAY mpp */
			      {"li64", 8, REEL}, /* little endian ieee64 */
			      {"vms32", 4, REEL}, /* VAX/VMS */
                              {"xdr", xdrform, NEUTRE}};
static const int maxcodtyp = sizeof(nxtypc)/sizeof(nxtypc[0]);

/* pour stocker les noms de fichiers infos des serveurs */

static char* serverinfo[NX_MAX_DESC] = {0};

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

static int initform(char* filename,int* format);
static char* strtok2(const char* str,const char* sep,const char* anti_sep,
		const char** psto_pos,char** psto_str);
static void stroteg(char* str);
static int net_addr(const char* buf_name,char** host_name,char** user_name,
		int* port_number,int* port_defaut,char** port_name,
		char** port_service,int* port_which,char** handler_name);
static int clientop(const char* host_name,const char* user_name,int port,
		int port_defaut,const char* port_name,const char* port_service,
		int port_which,const char* handler_name,DESC* desc,int* sock);
static int serverop(const char* user_name,int port,
		const char* port_name,const char* port_service,
		DESC* desc,int* sock);
static int servernumdel(const char* file_info_der,const int num_info_del[],
		unsigned num_info_del_taille);
static void attentemicrosec(const long delay);
static int setfilelock(const char* file_info_lock,int repeat);
static int clearfilelock(const char* file_info_lock);

/* ============================================================ */
/*			====>    FNXOPEN     <====		*/
/* ============================================================ */

FILE*
fnxopen(const char* name,const char* type)
{
	char buf_name[FILE_MAX_LEN + 1];
	int fd_index = -1;
	int fmt = 0;
	int cr = 0;
	FILE* fd = NULL;
	DESC desc;

	errno = 0;
	NXERRNO = 0; /* no error */
	fnxsetsyserror(NULL);
	memset(buf_name,0,FILE_MAX_LEN + 1);

	if(name)
	    strcpy(buf_name,name);
	    
	if(initform(buf_name,&fmt))
	{  
	    NXERRNO = 48;
	    return NULL;
	}

    /* PIPE ou ... */

	if (*buf_name == '|')
	{
	    desc.dp.fd = popen(buf_name + 1, "w") ;
	    fd = desc.dp.fd;
	    desc.ds = NXP_PIPE;
	}
	else
	{
	/*     SOCKET OR RPC	*/
	/* forme ([...] <=> facultatif) :
	   [host_name]+
	   [["]port["][.[["]port_service["]][.[port_which]]]][+[handler_name][+
	   [user_name]]]

	   + signifie +, : ou blanc
	   " signifie " ou '
	   port signifie port_number ou port_name
	   */

	    char* host_name;
	    char* port_name;
	    int port_number;
	    int port_defaut;
	    char* port_service;
	    int port_which;
	    char* handler_name;
	    char* user_name;

	    if((cr = net_addr(buf_name,&host_name,&user_name,&port_number,
			    &port_defaut,&port_name,&port_service,
			    &port_which,&handler_name)) == 1)
	    {
	    /*	  FICHIER		*/
	    /* si l'extension existe, elle aura ete eliminee ds initform */
		U8FOPEN(buf_name,type,desc.dp.fd);
		desc.ds = NXP_FILE;
		cr = 0;
	    }
	    else if(!cr)
	    {
	    /*    SOCKET                */

		if (!strcmp(host_name,"SERVER"))
		    cr = serverop(user_name,port_number,port_name,
				    port_service,&desc,&fd_index);
		else
		    cr = clientop(host_name,user_name,port_number,port_defaut,
				    port_name,port_service,port_which,
				    handler_name,&desc,&fd_index);

	    }
	    else
	    {
		cr = 2;
	    }

#if defined(DEBUG)
	    printf("fnxopen : Resultat du parsing du nom de fichier :\n");

	    if(host_name) printf(" host_name = %s\n",host_name);
	    else printf(" host_name = NULL\n");

	    if(user_name) printf(" user_name = %s\n",user_name);
	    else printf(" user_name = NULL\n");

	    if(handler_name) printf(" handler_name = %s\n",handler_name);
	    else printf(" handler_name = NULL\n");

	    if(port_name) printf(" port_name = %s\n",port_name);
	    else printf(" port_name = NULL\n");

	    if(port_service) printf(" port_service = %s\n",port_service);
	    else printf(" port_service = NULL\n");

	    printf(" port_number = %d\n",port_number);
	    printf(" port_defaut = %d\n",port_defaut);
	    printf(" port_which = %d\n",port_which);
	    printf(" buf_name = %s\n",buf_name);
#endif
	    if(host_name) free(host_name);
	    if(user_name) free(user_name);
	    if(handler_name) free(handler_name);
	    if(port_name) free(port_name);
	    if(port_service) free(port_service);

	    if(cr >= 1)
	    {

		if(NXERRNO) fnxperror("fnxopen");
		else fnxsyserror("fnxopen");

		if(cr == 1) NXERRNO = 1;

		return NULL;
	    }

	} 

	fd = desc.dp.fd;

	if(!fd && !desc.dp.clnt)
	{
	    NXERRNO = 1;
	    return NULL;
	}

	switch(desc.ds) 
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_index = fileno(fd);
	    break;
	  default :
	    break;
	}

	if (fd_index >= NX_MAX_DESC)
	{
	    NXERRNO = 2;
	    return NULL;
	}

	nx_desc_table[fd_index].ds = desc.ds;
	nx_desc_table[fd_index].dp = desc.dp;
	nxtabfrm[fd_index].fmt = fmt;
	nx_init_access(fd_index,fmt);

	return fd;
} 

/* ============================================================ */
/*			====>    FNXCLOSE     <====		*/
/* ============================================================ */

int
fnxclose(FILE* fd)
{
	return fnxclose_shutdown(fd,2);
}

/* Dans le cas ou on forke, le pere ne doit pas faire un shutdown */
/* Extension pour XediX */

int
fnxclose_shutdown(FILE* fd,int myshutdown)
{
	int rc = 0;
	int fd_index;

	if(!fd) return EOF;

	fd_index = fileno(fd);

#ifdef DEBUG
	printf("fnxclose : appel pour fd = %d\n",fd_index);
#endif

	if(fd_index >= NX_MAX_DESC || fd_index < 0)
	{
	    NXERRNO = 15;
	    return EOF;
	}

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE:
	    rc = pclose(fd);
	    break;
	  case NXP_FILE:
	    U8FCLOSE(fd,rc);
	    break;
	  case NXP_SOCKET:
	    {
		char* const sinfo = serverinfo[fd_index];

#ifdef DEBUG
		if(sinfo)
		    printf("fnxclose : serverinfo = %s\n",sinfo);
		else
		    printf("fnxclose : serverinfo = NULL\n");
#endif

		if(nx_fdin_table[fd_index])
		{
		    FILE* const fd_lect = nx_fdin_table[fd_index];

		    if(myshutdown >= 0)
		        rc = shutdown(fileno(fd_lect),myshutdown);

		    U8FCLOSE(fd_lect,rc);
		    nx_fdin_table[fd_index] = NULL;
		}

		if(nx_fdout_table[fd_index])
		{
		    FILE* const fd_writ = nx_fdout_table[fd_index];

		    U8FFLUSH(fd_writ,rc);

		    if(myshutdown >= 0)
		        rc = shutdown(fileno(fd_writ),myshutdown);

		    U8FCLOSE(fd_writ,rc);
		    nx_fdout_table[fd_index] = NULL;
		}

		if(myshutdown >= 0)
		    if(sinfo && !memcmp(sinfo,"/tmp/.nx-unix-",14))
		    /* si sockets AF_UNIX */
			unlink(sinfo);
		    else if(sinfo)
		    {
		    /* purge infos : */
			char* const info = strrchr(sinfo,'.');

			if(info)
			{
			    int num_info[1];
			    int fdinfo;

			    num_info[0] = atoi(info+1);

			/* ecriture d'un fichier lock de nom
			   /tmp/.nx/.user_name_service_name_lock */

			    memcpy(info-4,"lock",4);
			    *info = '\0';
			    setfilelock(sinfo,100);

			/* purge du fichier de bind du serveur 
			   si sockets AF_UNIX */

			    memcpy(info-4,"info",4);
			    *info = '.';
			    U8OPEN(sinfo,O_RDONLY,0,fdinfo);

			    if(fdinfo != -1)
			    {
				char buf[1040];
				ssize_t lus = 0;
				int ier = 0;

				U8READ(fdinfo,buf,1039,lus);

				if(lus > 0)
				{
				    int pid_old = 0;
				    int number_old = 0;

				    buf[lus] = '\0';
				    sscanf(buf,"%d %d",&number_old,&pid_old);

				    if(!number_old)
				    {
					long int i = lus-1;

					while(i >= 0 && buf[i] != ' ') i--;

					while(i >= 0 && buf[i] == ' ') i--;

					buf[i+1] = '\0';
					unlink(buf);

					while(i >= 0 && buf[i] != '/') i--;

					buf[i] = '\0';
					rmdir(buf);
				    }

				}

				U8CLOSE(fdinfo,ier);
				errno = 0;
			    }

			/* mise a jour de
			   /tmp/.nx/.user_name_service_name_info
			   et clear de
			   /tmp/.nx/.user_name_service_name_info.num_info */

			    *info = '\0';
			    servernumdel(sinfo,num_info,1);
			    *info = '.';
			    unlink(sinfo);

			/* clear du fichier lock */

			    memcpy(info-4,"lock",4);
			    *info = '\0';
			    clearfilelock(sinfo);
			}

		    }

		if(sinfo)
		{
		/* liberation ressources */
		    free(sinfo);
		    serverinfo[fd_index] = NULL;
		}

	    }
	    break;
	  case NXP_RPC:
	    clnt_destroy(nx_desc_table[fd_index].dp.clnt);
	    nx_desc_table[fd_index].dp.clnt = NULL;
	    break;
	  default :
	    break;
	}

	nx_desc_table[fd_index].ds = 0;
	nxtabfrm[fd_index].format[0] = '\0';
	nxtabfrm[fd_index].fmt = 0;
	nx_desc_table[fd_index].dp.fd = NULL;

	return rc;
}

/* ============================================================ */
/*	          ====>    FNXREOPEN     <====		        */
/* ============================================================ */

int
fnxreopen(FILE* fd,NX_Fd_Type fd_type)
{
    /* travail sur un fichier deja ouvert par fopen                 
       ou sur une socket deja ouverte */

	int fd_index;
	int fmt;
	DESC desc;

	if(!fd) return EOF;

	fd_index = fileno(fd);

	if(fd_index >= NX_MAX_DESC || fd_index < 0) 
	{
	    NXERRNO = 2;
	    return EOF;
	}

	nxtabfrm[fd_index].format[0] = '\0';

	if((nxtabfrm[fd_index].fmt < 0) || (nxtabfrm[fd_index].fmt >= maxform))
	{
	    fmt = 0; /* host format */
	}
	else
	{
	    fmt = nxtabfrm[fd_index].fmt;
	}

	desc.dp.fd = fd;

	switch(fd_type)
	{
	  case NX_PIPE:
	    desc.ds = NXP_PIPE;
	    break;
	  case NX_SOCKET:
	    desc.ds = NXP_SOCKET;
	    break;
	  case NX_RPC:
	    desc.ds = NXP_RPC;
	    break;
	  case NX_FILE:
	  default:
	    desc.ds = NXP_FILE;
	}

	nx_desc_table[fd_index].ds = desc.ds;
	nx_desc_table[fd_index].dp = desc.dp;
	nxtabfrm[fd_index].fmt = fmt;
	nx_init_access(fd_index, fmt);

	return 0;
}

/* ============================================================ */
/*			====>     CLIENTOP     <====		*/ 
/* ============================================================ */

static int
clientop(const char* host_name,const char* user_name,int port,int port_defaut,
		const char* port_name,const char* port_service,int port_which,
		const char* handler_name,DESC* desc,int* sock)
{  
	struct sockaddr_in server_addr_inet;
	struct sockaddr_un server_addr_unix;
	struct sockaddr* server_addr = NULL;
	int lserver_addr = 0;
	int family = 0;

	struct Header_S header_s;
	char h_name[MAXHOSTNAMELEN];
	char port_name_reel[1024];
	char* port_temp = NULL;

    /* recherche automatique du port si possible */

	if(port_defaut || !port)
	{
	    int port_reel = -1;
	    int lport_name = 1024;
	    int process = -1;
	    int port_which_reel = 0;

	    port_name_reel[0] = '\0';

	    if(fnx_find_port(host_name,user_name,port_service,port_which,
			    &process,&port_reel,port_name_reel,&lport_name,
			    &port_which_reel))
	    {

		if(!port_defaut)
		{
		    NXERRNO = 71;
		    return 1;
		}

	    }
	    else
	    {
		port = port_reel;

		if(lport_name > 1) port_name = port_name_reel;

	    }

#ifdef DEBUG
	    printf("clientop : port = %d port_name = %s lport_name = %d\n",port,
			    port_name ? port_name : "NULL",lport_name);
#endif
	}

    /* inits server_addr_* */

	memset((char*) &server_addr_inet,0,sizeof(server_addr_inet));
	memset((char*) &server_addr_unix,0,sizeof(server_addr_unix));

    /* nom de la machine locale = h_name */

	if(U8GetHostname(h_name,sizeof(h_name)) == -1 || !strlen(h_name))
	{
	    NXERRNO = 49;
	    return 1;
	}

    /*	structure d'identification d'un stream */ 

	memset((char*) &header_s, 0, SIZ_HDX);
	sprintf(header_s.lg,"%.*d",(int) sizeof(header_s.lg)-1,SIZ_HDX);
	sprintf(header_s.version,"%.*d",(int) sizeof(header_s.version)-1,
			SOCK_VERSION);
	strncpy(header_s.host_name,h_name,(int) sizeof(header_s.host_name)-1);
	strncpy(header_s.user_name,user_name,(int) sizeof(header_s.user_name)-1);
	strncpy(header_s.handler_name,handler_name,
			(int) sizeof(header_s.handler_name)-1);

	if(!port_name) /* sockets inet */
	{
	/* Get inet address */
	    struct hostent* const host_info = U8GetHostbyname(host_name);

	    if(!host_info)
	    {
		NXERRNO = 16;
		return 1;
	    }

	    if(host_info->h_addrtype != AF_INET)
	    {
		NXERRNO = 50;
		return 1;
	    }

	    family = AF_INET;
	    server_addr_inet.sin_family = AF_INET;
	    server_addr_inet.sin_port = htons(port);
	    memcpy((caddr_t)&server_addr_inet.sin_addr,host_info->h_addr,
			    host_info->h_length);
	    memset(server_addr_inet.sin_zero,0,
			    sizeof(server_addr_inet.sin_zero));
	    server_addr = (struct sockaddr*) &server_addr_inet;
	    lserver_addr = sizeof(server_addr_inet);
	}
	else /* sockets unix */
	{

	    if(strlen(port_name) > sizeof(server_addr_unix.sun_path))
	    {
		errno = ENAMETOOLONG;
		return 1;
	    }

	    family = AF_UNIX;
	    server_addr_unix.sun_family = AF_UNIX;
	    strcpy(server_addr_unix.sun_path,port_name);
	    server_addr = (struct sockaddr*) &server_addr_unix;
	    lserver_addr = sizeof(server_addr_unix);
	}

	if(!port_name && port > 1000000) /* RPC */
	{
	    const unsigned long appli = (unsigned long) port;

	    *sock = RPC_ANYSOCK;
	    server_addr_inet.sin_port = 0; /* somebody do it for you */

	    if(!(desc->dp.clnt = clnttcp_create(&server_addr_inet, appli, 
			    1, sock, 0, 0)))
	    {
		clnt_pcreateerror("clntcp_create");
		nxpioerror("gethostname");
		return 1;
	    }

	    desc->ds = NXP_RPC;
	}
	else					/* SOCKET */
	{
	    const char* message = "";

	/* Create a socket, try to connect it to the server. and then 
	   send an identification item. */

	    if((*sock  = socket(family,SOCK_STREAM,0)) < 0)
	    {
		fnxsetsyserror("socket");
		return 1;
	    }

	    if(family == AF_UNIX)
	    {

	    /* pour les sockets AF_UNIX on binde explicitement la socket
	       cliente */

		struct sockaddr_un server_addr_unix_clnt;
		struct sockaddr* server_addr_clnt = NULL;
		int lserver_addr_clnt = 0;
		struct stat sbuf;

	    /* choix automatique d'un nom de fichier port socket AF_UNIX */
	    /* pour binder le client */

		const char* const port_tmp = "/tmp/.nx-unix-clnt.XXXXXX";
		const size_t lport_tmp = strlen(port_tmp);
		int j = 0;

		if(lport_tmp > sizeof(server_addr_unix_clnt.sun_path))
		{
		    int ier = 0;

		    errno = ENAMETOOLONG;
		    U8CLOSE(*sock,ier);
		    return 1;
		}

		port_temp = (char*) malloc(lport_tmp + 1);

		if(!port_temp)
		{
		    int ier = 0;

		    NXERRNO = 51;
		    U8CLOSE(*sock,ier);
		    return 1;
		}

		for(;j<10;j++)
		{
		    memcpy(port_temp,port_tmp,lport_tmp + 1);
		    mktemp(port_temp);

		    if(port_temp[0] == '\0')
		    {
			int ier = 0;

			NXERRNO = 52;
			free(port_temp);
			U8CLOSE(*sock,ier);
			return 1;
		    }

		    if(stat(port_temp,&sbuf))
			if(errno == ENOENT)
			    errno = 0;
			else
			    continue;
		    else
			if(NX_ISSOCKET(sbuf.st_mode))
			{
			    if(unlink(port_temp))
				continue;
			}
			else
			    continue;

		    family = AF_UNIX;
		    server_addr_unix_clnt.sun_family = AF_UNIX;
		    memcpy(server_addr_unix_clnt.sun_path,port_temp,lport_tmp+1);
		    server_addr_clnt = (struct sockaddr*) &server_addr_unix_clnt;
		    lserver_addr_clnt = sizeof(server_addr_unix_clnt);

		    if(!bind(*sock,server_addr_clnt,lserver_addr_clnt))
			break;

		}

		if(j == 10)
		{
		    int ier = 0;

		    free(port_temp);
		    U8CLOSE(*sock,ier);
		    fnxsetsyserror("bind");
		    return 1;
		}

		if(serverinfo[*sock])
		{
		    int ier = 0;

		    NXERRNO = 74;
		    free(port_temp);
		    U8CLOSE(*sock,ier);
		    return 1;
		}

		serverinfo[*sock] = port_temp;
	    }

	/* connection socket a celle du serveur */
	/* En cas d'interruption, on ne peut pas juste relancer (voir POSIX). */
	    if(connect(*sock,server_addr,lserver_addr) == -1)
	    {
		struct pollfd pfd;
		int optval = 0;
		socklen_t optlen = sizeof(optval);

		message = "clientop : connect";

		if(errno != EINTR)
		    goto errconn;

		pfd.fd = *sock;
		pfd.events = POLLOUT;

		while(poll(&pfd,1,-1) == -1)
		    if(errno != EINTR)
			goto errconn;

		if(getsockopt(*sock,SOL_SOCKET,SO_ERROR,&optval,&optlen) == -1)
		    goto errconn;

		if(optval)
		    goto errconn;

	    }

	    unlink(port_temp);

	/* Ecriture header dans la socket que l'on vient de connecter */
	    {
		ssize_t ecrit = 0;
		u_int_4 xs = 0;
		u_int_4 nxs = 0;
		ssize_t lg = 0;

		message = "clientop : can't write header";
		U8WRITE(*sock,&header_s,SIZ_HDSL,ecrit);

		if(ecrit < SIZ_HDSL)
		    goto errconn;

		xs = (u_int_4) strlen(header_s.host_name) + 1;
		nxs = htonl(xs);
		U8WRITE(*sock,&nxs,4,lg);

		if(lg < 4)
		    goto errconn;

		U8WRITE(*sock,header_s.host_name,xs,lg);

		if(lg < xs)
		    goto errconn;

		xs = (u_int_4) strlen(header_s.user_name) + 1;
		nxs = htonl(xs);
		U8WRITE(*sock,&nxs,4,lg);

		if(lg < 4)
		    goto errconn;

		U8WRITE(*sock,header_s.user_name,xs,lg);

		if(lg < xs)
		    goto errconn;

		U8WRITE(*sock,header_s.port_name,L3,lg);

		if(lg < L3)
		    goto errconn;

		U8WRITE(*sock,header_s.handler_name,L3,lg);

		if(lg < L3)
		    goto errconn;

	    }

	/* creation d'un pointeur sur FILE */
	    if(!(desc->dp.fd = fdopen(*sock,"w")))
	    {
		message = "clientop : fdopen";
		goto errconn;
	    }

	/* stockages */
	    nx_fdout_table[*sock] = desc->dp.fd;
	    desc->ds = NXP_SOCKET;

	    return 0;

    errconn:{
		int ier = 0;

		shutdown(*sock,2);
		U8CLOSE(*sock,ier);
		fnxsetsyserror(message);

		if(port_temp)
		{
		    unlink(port_temp);
		    free(port_temp);
		    serverinfo[*sock] = NULL;
		}

		return 1;
	    }

	}

	return 0;
}

/* ============================================================ */
/*			====>     SERVERECRI      <====		*/ 
/*  ecriture infos (port et numero process)                     */
/*  s'inspire de S6GereFenet::InitServeurs() de Narcisse.       */
/*  Narcisse (Psyche au CEA) est nom depose du CEA.             */
/*  le nom des fichiers est de la forme :                       */
/*  /tmp/.nx/.${USER}_{port_service}_info.n                     */
/* ============================================================ */

static int
serverecri(const char* file_info,size_t lfile_info,const char* port_name,
		int port,int pid,int fdinfo,int ns)
{
	char buf[1040];
	size_t lbuf;
	ssize_t ecrit = 0;
	int ier = 0;

	if(port_name)
	    sprintf(buf,"%s %d",port_name,pid);
	else
	    sprintf(buf,"%d %d",port,pid);

	lbuf = strlen(buf);
	buf[lbuf] = '\n';
	U8WRITE(fdinfo,buf,lbuf + 1,ecrit);
	U8CLOSE(fdinfo,ier);

	if(ecrit == -1)
	{
	    fnxsetsyserror("serverecri : can't write info");
	    return 1;
	}

	if(serverinfo[ns])
	{
	    NXERRNO = 66;
	    return 1;
	}

	serverinfo[ns] = (char*) malloc(lfile_info);

	if(!serverinfo[ns])
	{
	    fnxsetsyserror("serverecri : can't alloc memory");
	    return 1;
	}

	memcpy(serverinfo[ns],file_info,lfile_info);

	return 0;
}

/* ============================================================ */
/*			====>     SERVERNUMDEL      <====	*/ 
/*  enleve la liste des numeros des services nx qui ne          */
/*  tournent plus et enleve les numeros en double.              */
/*  le nom du fichier est de la forme :                         */
/*  /tmp/.nx/.${USER}_{port_service}_info                       */
/* ============================================================ */

static int
servernumdel(const char* file_info_der,const int num_info_del[],
		unsigned num_info_del_taille)
{
	ssize_t lus,ecrit;
	char* buf;
	size_t lbuf = 0;
	int ier = 0;
	int err = 0;
	struct stat sbuf;

    /* ouverture en lecture et pas erreur si fichier n'existe pas */

	int fdinfo_der;

	U8OPEN(file_info_der,O_RDONLY|O_CREAT,
			S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo_der);

	if(fdinfo_der == -1)
	{
	    fnxsetsyserror("servernumdel : can't open info_der");
	    return -1;
	}

    /* pb sur epix_2 : fstat efface file_info_der !!!!! */

	if(!fstat(fdinfo_der,&sbuf))
	    lbuf = sbuf.st_size;

	buf = (char*) malloc(lbuf+1);

	if(!buf)
	{
	    fnxsetsyserror("servernumdel : can't alloc memory");
	    U8CLOSE(fdinfo_der,err);
	    return -1;
	}

	U8READ(fdinfo_der,buf,lbuf,lus);

	if(lus == -1)
	{
	    fnxsetsyserror("servernumdel : can't read info_der");
	    U8CLOSE(fdinfo_der,err);
	    free(buf);
	    return -1;
	}

	U8CLOSE(fdinfo_der,err);
	buf[lus] = '\0';

    /* on elimine tous les nombres contenus dans num_info_del et ceux non
       contenus dans l'ensemble des valeurs absolues des num_info_del negatifs
       si cet ensemble est non vide.
       Si num_info_del_taille = 0, on renvoit le num_info_del_max */

	if(!num_info_del_taille)
	{
	    char* buftemp = buf;
	    char* bufnl = strchr(buftemp,'\n');
	    int num_info_del_taille_max = 0;

	    while(bufnl)
	    {
		*bufnl = '\0';
		num_info_del_taille_max = MAX(atoi(buftemp),
				num_info_del_taille_max);
		*bufnl = '\n';
		buftemp = bufnl + 1;
		bufnl = strchr(buftemp,'\n');
	    }

	    ier = num_info_del_taille_max;
	}
	else
	{
	    char* buftemp = buf;
	    char* bufnl = strchr(buftemp,'\n');
	    int neg = FALSE;
	    unsigned li = 0;

	    for(;li<num_info_del_taille;li++)
	    {

		if(num_info_del[li] < 0)
		{
		    neg = TRUE;
		    break;
		}

	    }

	    while(bufnl)
	    {
		int decal = TRUE;
		int decal_del = 0;

		*bufnl = '\0';

		for(li=0;li<num_info_del_taille;li++)
		{
		    register int num_info = num_info_del[li];

		    num_info = num_info > 0 ? num_info : -num_info;

		    if(atoi(buftemp) == num_info)
		    {
			decal = FALSE;
			decal_del = num_info_del[li];
			break;
		    }

		}

		*bufnl = '\n';

		if((decal || decal_del < 0) && (!neg || !decal))
		{
		    buftemp = bufnl + 1;
		}
		else
		{
		    memmove(buftemp,bufnl+1,strlen(bufnl));
		}

		bufnl = strchr(buftemp,'\n');
	    }

	}

    /* on elimine tous les nombres en double sauf leurs dernieres occurences */

	{
	    char* buftemp = buf;
	    char* bufnl = strchr(buftemp,'\n');

	    while(bufnl)
	    {
		int decal = TRUE;
		char* buftemp2 = bufnl+1;
		char* bufnl2 = strchr(buftemp2,'\n');

		*bufnl = '\0';

		while(bufnl2)
		{
		    *bufnl2 = '\0';

		    if(!memcmp(buftemp,buftemp2,strlen(buftemp)))
		    {
			decal = FALSE;
			*bufnl2 = '\n';
			break;
		    }

		    *bufnl2 = '\n';
		    buftemp2 = bufnl2+1;
		    bufnl2 = strchr(buftemp2,'\n');
		}

		*bufnl = '\n';

		if(decal)
		{
		    buftemp = bufnl + 1;
		}
		else
		{
		    memmove(buftemp,bufnl+1,strlen(bufnl));
		}

		bufnl = strchr(buftemp,'\n');
	    }

	}

	lbuf = strlen(buf);

    /* Reecriture du nouveau fichier info contenant la liste des numeros
       de services nx pour port_service et user_name donnes, dans l'ordre
       de lancement, et purge s'il est de taille nulle. */

	if(lbuf)
	{
	    U8OPEN(file_info_der,O_WRONLY|O_CREAT|O_TRUNC,
			    S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo_der);

	    if(fdinfo_der == -1)
	    {
		fnxsetsyserror("servernumdel : can't open info_der for writing");
		free(buf);
		return -1;
	    }

	    U8WRITE(fdinfo_der,buf,lbuf,ecrit);

	    if(ecrit == -1)
	    {
		fnxsetsyserror("servernumdel : can't write info_der");
		ier = -1;
	    }

	    U8CLOSE(fdinfo_der,err);
	    free(buf);
	}
	else
	{
	    free(buf);
	    unlink(file_info_der);
	}

	return ier;
}

/* ============================================================ */
/*			====>     SERVERNUM      <====		*/ 
/*  enregistre la liste des numeros des services nx pour        */
/*  port_service et user_name donnes dans l'ordre ou ils sont   */
/*  lances dans /tmp/.nx                                        */
/*  s'inspire de S6GereFenet::InitServeurs() de Narcisse.       */
/*  Narcisse (Psyche au CEA) est nom depose du CEA.             */
/*  le nom des fichiers est de la forme :                       */
/*  /tmp/.nx/.${USER}_{port_service}_info                       */
/* ============================================================ */

static int
servernum(const char* file_info_der,int num_info)
{
	int fdinfo_der;
	size_t lbuf;
	ssize_t ecrit = 0;
	char buf[32];
	int err  = 0;

    /* ouverture en ecriture et pas erreur si fichier
       n'existe pas */

	 U8OPEN(file_info_der,O_WRONLY|O_CREAT|O_APPEND,
			S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo_der);

	if(fdinfo_der == -1)
	{
	    fnxsetsyserror("servernum : can't open info_der");
	    return 1;
	}

	sprintf(buf,"%d",num_info);
	lbuf = strlen(buf);
	buf[lbuf] = '\n';
	U8WRITE(fdinfo_der,buf,lbuf + 1,ecrit);

	if(ecrit == -1)
	{
	    fnxsetsyserror("servernum : can't write info_der");
	    U8CLOSE(fdinfo_der,err);
	    return 1;
	}

	U8CLOSE(fdinfo_der,err);
	return 0;
}

/* ============================================================ */
/*			====>     ATTENTEMICROSEC      <====	*/
/* inspire de routines developpees par G. Colin de Verdieres    */
/* ============================================================ */

static void
attentemicrosec(const long delay)
{
	struct timeval timeout;

	timeout.tv_sec = 0;
	timeout.tv_usec = delay;
	select(0, NULL, NULL, NULL, &timeout);
}

/* ============================================================ */
/*			====>     SETFILELOCK      <====	*/
/* inspire de routines developpees par G. Colin de Verdieres    */
/* ============================================================ */

static int
setfilelock(const char* file_info_lock,int repeat)
{
	int fdl;

    /* O_SYNC va forcer l'ecriture physique des donnees du write */
	U8OPEN(file_info_lock,O_WRONLY|O_CREAT|O_EXCL|O_SYNC,
			S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdl);

	while(fdl == -1 && repeat--)
	{

	    if (errno != EEXIST)
	    {
		fnxsetsyserror("setfilelock : can't open file");
		return 1;
	    }

	    attentemicrosec(10000);
	    U8OPEN(file_info_lock,O_WRONLY|O_CREAT|O_EXCL|O_SYNC,
			    S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdl);
	}
   
	if(fdl != -1)
	{
	    int ier = 0;

	    U8CLOSE(fdl,ier);
	}

	return 0;
}

/* ============================================================ */
/*			====>     CLEARFILELOCK      <====	*/
/* inspire de routines developpees par G. Colin de Verdieres    */
/* ============================================================ */

static int
clearfilelock(const char* file_info_lock)
{
	int ier = unlink(file_info_lock);

	return ier;
}

/* ============================================================ */
/*			====>     SERVERREG      <====		*/ 
/*  enregistre le port et le numero de process du serveur       */
/*  dans /tmp/.nx                                               */
/*  s'inspire de S6GereFenet::InitServeurs() de Narcisse.       */
/*  Narcisse (Psyche au CEA) est nom depose du CEA.             */
/*  le nom des fichiers est de la forme :                       */
/*  /tmp/.nx/.${USER}_{port_service}_info (contient le numero   */
/*  du dernier serveur de nom port_service lance),              */
/*  /tmp/.nx/.${USER}_{port_service}_info.n (contient les       */
/*  numeros de port et de process du nieme serveur de nom       */
/*  port_service).                                              */
/*  on teste si les process ayant donnes naissance aux fichiers */
/*  sont encore la. Sinon on purge les fichiers correspondant.  */
/*  Le numero de serveur est le premier libre (on bouche les    */
/*  trous).                                                     */
/* ============================================================ */

static int
serverreg(int port,const char* port_name,const char* port_service,
		const char* user_name,int ns)
{
	const mode_t old_umask = umask(0);
	int ier = 0;

	int ecrit_info = TRUE;
	int num_info = 1;

	pid_t pid = getpid();

	int num_info_max;
	unsigned lnum_info_del = 10;
	unsigned num_info_del_taille = 0;
	const size_t lfile_info_der =
	    strlen(user_name) + strlen(port_service) + 20;
	int* num_info_del = (int*) malloc(lnum_info_del*sizeof(int));
	char* const file_info_der = (char*) malloc(lfile_info_der);
	char* const file_info_lock = (char*) malloc(lfile_info_der);

    /* Test si user_name est bien egal a ce que renvoit U8GetUser sinon
       on ne stocke pas les infos car c'est suspect. */

	if(!file_info_der || !num_info_del || !file_info_lock)
	{

	    if(file_info_der) free(file_info_der);

	    if(num_info_del) free(num_info_del);

	    if(file_info_lock) free(file_info_lock);

	    fnxsetsyserror("serverreg : can't alloc memory");
	    umask(old_umask);
	    return 1;
	}

    /* U8GetUser est defini dans nxutil.c et declare dans l'include public
       nxsystem.h */

	{
	    const char* const username = U8GetUser(NULL);
	    const size_t lusername = strlen(username);

	    errno = 0;

	    if(!lusername || strcmp(user_name,username))
	    {
		NXERRNO = 65;
		ier = 1;
		goto sorti2serverreg;
	    }

	}

	sprintf(file_info_lock,"/tmp/.nx/.%s_%s_lock",user_name,port_service);
	sprintf(file_info_der,"/tmp/.nx/.%s_%s_info",user_name,port_service);

#if defined(DEBUG)
	if(port_name)
	{
	    printf("serverreg :\n port = %d\n port_name = %s\n",port,port_name);
	}
	else
	{
	    printf("serverreg :\n port = %d\n port_name = NULL\n",port);
	}
#endif

    /* creation du repertoire /tmp/.nx permission 1777 */

	ier = mkdir("/tmp/.nx",S_IRWXU|S_IRWXG|S_IRWXO);

	if(ier == -1 && errno != EEXIST)
	{
	    fnxsetsyserror("serverreg : can't create /tmp/.nx");
	    ier = 1;
	    goto sorti2serverreg;
	}

	if(!errno)
	{
	    ier = chmod("/tmp/.nx",S_IRWXU|S_IRWXG|S_IRWXO|S_ISVTX);

	    if(ier == -1)
	    {
		fnxsetsyserror("serverreg : can't chmod /tmp/.nx");
		ier = 1;
		goto sorti2serverreg;
	    }

	}
	else
	{
	    errno = 0;
	}

	setfilelock(file_info_lock,100);
	num_info_max = servernumdel(file_info_der,NULL,0);

	if(num_info_max == -1)
	{
	    ier = 1;
	    goto sortieserverreg;
	}

    /* exploration des fichiers /tmp/.nx/.${USER}_{port_service}_info.n,
       purge de ceux inutiles et recherche du premier trou */

	while(1)
	{
	    int fdinfo;
	    int num_info_ecrit = 0;
	    size_t lfile_info = strlen(user_name) + strlen(port_service) + 30;
	    char* file_info = (char*) malloc(lfile_info);

	    if(!file_info)
	    {
		fnxsetsyserror("serverreg : can't alloc memory");
		ier = 1;
		goto sortieserverreg;
	    }

	    sprintf(file_info,"/tmp/.nx/.%s_%s_info.%d",
			    user_name,port_service,num_info);
	    lfile_info = strlen(file_info) + 1;

	    if(ecrit_info)
	    {
	    /* ouverture en ecriture et erreur si fichier existe */
		U8OPEN(file_info,O_WRONLY|O_EXCL|O_CREAT,
				S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo);
	    }
	    else
	    {
	    /* ouverture en lecture et erreur si fichier n'existe pas */
		U8OPEN(file_info,O_RDONLY,0,fdinfo);

		if(fdinfo != -1)
		{
		    char buf[1040];
		    int pid_old = 0;
		    int number_old = 0;
		    ssize_t lus = 0;
		    int err = 0;

		    U8READ(fdinfo,buf,1039,lus);

		    if(lus > 0)
		    {
			buf[lus] = '\0';
			sscanf(buf,"%d %d",&number_old,&pid_old);

			if(!number_old)
			{
			/* cas sockets UNIX */
			    long int i = lus-1;

			    while(i >= 0 && buf[i] != ' ') i--;

			    sscanf(buf+i+1,"%d",&pid_old);

			    while(i >= 0 && buf[i] == ' ') i--;

			    buf[i+1] = '\0';
			}

		    }

		    U8CLOSE(fdinfo,err);

		    if(!pid_old || kill(pid_old,0))
		    {
			errno = 0;
			unlink(file_info);

			if(!number_old && pid_old)
			{
			    unlink(buf);
			}

			errno = 0;
			num_info_ecrit = num_info;
		    }
		    else
		    {
			num_info_ecrit = -num_info;
		    }

		}
		else if(num_info > num_info_max)
		{
		    errno = 0;
		    free(file_info);
		    break;
		}
		else
		{
		    num_info_ecrit = num_info;
		}

	    }

	    if(ecrit_info && fdinfo != -1)
	    {
	    /* ecriture infos (port et numero process) */

		if(serverecri(file_info,lfile_info,port_name,port,pid,fdinfo,ns))
		{
		    ier = 1;
		    goto sortieserverreg;
		}

		ecrit_info = FALSE;

	    /* stockage du numero du dernier Psyche ouvert */

		if(servernum(file_info_der,num_info))
		{
		    ier = 1;
		    goto sortieserverreg;
		}

		num_info_ecrit = -num_info;
	    }
	    else if(ecrit_info)
	    {

	    /* ouverture en lecture et pas erreur si fichier
	       n'existe pas */
		errno = 0;
		U8OPEN(file_info,O_RDONLY|O_CREAT,
				S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo);

		if(fdinfo != -1)
		{
		    char buf[1040];
		    int pid_old = 0;
		    int number_old = 0;
		    ssize_t lus = 0;

		    U8READ(fdinfo,buf,1039,lus);

		    if(lus > 0)
		    {
			buf[lus] = '\0';
			sscanf(buf,"%d %d",&number_old,&pid_old);

			if(!number_old)
			{
			/* cas sockets UNIX */
			    long int i = lus-1;

			    while(buf[i] != ' ') i--;

			    sscanf(buf+i+1,"%d",&pid_old);

			    while(i >= 0 && buf[i] == ' ') i--;

			    buf[i+1] = '\0';
			}

		    }

		    if(!pid_old || kill(pid_old,0))
		    {
		    /* ecriture infos (port et numero process) */
			int err = 0;

			U8CLOSE(fdinfo,err);

			if(!number_old && pid_old)
			{
			    unlink(buf);
			}

			errno = 0;
			U8OPEN(file_info,O_WRONLY|O_CREAT|O_TRUNC,
					S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH,fdinfo);

			if(fdinfo == -1)
			{
			    fnxsetsyserror("serverreg : can't open file");
			    ier = 1;
			    goto sortieserverreg;
			}

			if(serverecri(file_info,lfile_info,port_name,port,
					pid,fdinfo,ns))
			{
			    ier = 1;
			    goto sortieserverreg;
			}

			ecrit_info = FALSE;

		    /* stockage du numero du dernier Psyche ouvert */

			if(servernum(file_info_der,num_info))
			{
			    ier = 1;
			    goto sortieserverreg;
			}

		    }

		    num_info_ecrit = -num_info;
		}
		else
		{
		    fnxsetsyserror("serverreg : can't open file");
		    ier = 1;
		    goto sortieserverreg;
		}

	    }

	    if(num_info_ecrit)
	    {

		if(num_info_del_taille >= lnum_info_del)
		{
		    lnum_info_del += 10;
		    num_info_del = (int*) realloc(num_info_del,
				    lnum_info_del*sizeof(int));

		    if(!lnum_info_del)
		    {
			fnxsetsyserror("serverreg : can't alloc memory");
			ier = 1;
			goto sortieserverreg;
		    }

		}

		num_info_del[num_info_del_taille] = num_info_ecrit;
		num_info_del_taille++;
	    }

	    num_info++;
	    free(file_info);
	}

	ier = -servernumdel(file_info_der,num_info_del,num_info_del_taille);

sortieserverreg:
	clearfilelock(file_info_lock);
sorti2serverreg:
	free(num_info_del);
	free(file_info_der);
	free(file_info_lock);
	umask(old_umask);

	return ier;
}

/* ============================================================ */
/*			====>     SERVEROP     <====		*/ 
/*  marche avec sockets AF_INET et AF_UNIX                      */
/*  le port permet la discrimination entre AF_INET et AF_UNIX   */
/*  voir net_addr                                               */
/* ============================================================ */

static int
serverop(const char* user_name,int port,const char* port_name,
		const char* port_service,DESC* desc,int* sock)
{
	struct sockaddr_in server_addr_inet;
	struct sockaddr_un server_addr_unix;
	struct sockaddr* server_addr = NULL;
	int lserver_addr = 0;
	int family = 0;

	memset((char*) &server_addr_inet,0,sizeof(server_addr_inet));
	memset((char*) &server_addr_unix,0,sizeof(server_addr_unix));

	if(!port_name) /* sockets inet */
	{
	    family = AF_INET;
	    server_addr_inet.sin_family = AF_INET;
	    server_addr_inet.sin_port = htons(port);
	    server_addr_inet.sin_addr.s_addr = INADDR_ANY;
	    memset(server_addr_inet.sin_zero,0,
			    sizeof(server_addr_inet.sin_zero));
	    server_addr = (struct sockaddr*) &server_addr_inet;
	    lserver_addr = sizeof(server_addr_inet);
	}
	else /* sockets unix */
	{
	    struct stat sbuf;

	    if(strlen(port_name) > sizeof(server_addr_unix.sun_path))
	    {
		errno = ENAMETOOLONG;
		return 1;
	    }

	    if(stat(port_name,&sbuf))
	    {

		if(errno == ENOENT) errno = 0;
		else return 1;

	    }
	    else
	    {

		if(NX_ISSOCKET(sbuf.st_mode))
		{

		    if(unlink(port_name)) return 1;

		}
		else
		{
		    errno = EBUSY;
		    return 1;
		}

	    }

	    family = AF_UNIX;
	    server_addr_unix.sun_family = AF_UNIX;
	    strcpy(server_addr_unix.sun_path,port_name);
	    server_addr = (struct sockaddr*) &server_addr_unix;
	    lserver_addr = sizeof(server_addr_unix);
	}

	if((*sock  = socket(family,SOCK_STREAM,0)) < 0)
	{
	    fnxsetsyserror("socket");
	    return 1;
	}

	{
	    int on = 1;

	    if(setsockopt(*sock,SOL_SOCKET,SO_REUSEADDR,
			    (const char*) &on,sizeof(on)) < 0)
	    {
		fnxsetsyserror("socket SO_REUSEADDR");
		return 1;
	    }

	}

	if(bind(*sock,server_addr,lserver_addr))
	{
	    const int maxt = 4;
	    int t = 1;

	    perror("bind ");

	    for (;t<=maxt;t++)
	    {
		fprintf(stderr,"waiting for 15 s");

		if (t > 1) fprintf(stderr," again");

		fprintf(stderr,"\n");
		sleep(15);

		if(bind(*sock,server_addr,lserver_addr))
		{

		    if (t == maxt)
		    {
			int ier = 0;

			U8CLOSE(*sock,ier);
			perror("bind ");
			return 1;
		    }

		}
		else
		{
		    fprintf(stderr,"OK for bind\n");
		    break;
		}

	    }

	}

	if(port_service)
	{
	/* enregistrement des caracteristiques du serveur dans /tmp/.nx */

	    int port_temp = -1;
	    char* port_name_temp = NULL;
	    NX_GSN_SIZE len = lserver_addr;

	    if(getsockname(*sock,server_addr,&len))
	    {
		int ier = 0;

		NXERRNO = 63;
		U8CLOSE(*sock,ier);
		return 1;
	    }

	    if(!port_name) /* sockets inet */
	    {
		port_temp = ntohs(server_addr_inet.sin_port);
	    }
	    else /* sockets unix */
	    {
		const size_t lport_name =
		    strlen(server_addr_unix.sun_path) + 1;

		port_name_temp = (char*) malloc(lport_name);

		if(!port_name_temp)
		{
		    int ier = 0;

		    NXERRNO = 62;
		    U8CLOSE(*sock,ier);
		    return 1;
		}

		memcpy(port_name_temp,server_addr_unix.sun_path,lport_name);
	    }

	    if(serverreg(port_temp,port_name_temp,port_service,user_name,*sock))
	    {
	    /* on continue quand meme mais rien n'est enregistre sur /tmp/.nx */

		if(NXERRNO) fnxperror("serverop");
		else fnxsyserror("serverop");

	    }

	    if(port_name_temp) free(port_name_temp);

	    errno = 0;
	}

	if (listen(*sock,5))
	{
	    int ier = 0;

	    U8CLOSE(*sock,ier);
	    fnxsetsyserror("serverop : listen");
	    return 1;
	}

	desc->ds = NXP_SOCKET;

	if(!(desc->dp.fd = fdopen(*sock,"r")))
	{
	    int ier = 0;

	    U8CLOSE(*sock,ier);
	    fnxsetsyserror("serverop : fdopen");
	    return 1;
	}

	nx_fdin_table[*sock] = desc->dp.fd;

    /* => histoire d'avoir un fd valide, mais inutilise, 
       et qui contient le descripteur. */

	return 0;
}

/* ============================================================ */
/*			====>    FNXACCEPT     <====		*/ 
/* ============================================================ */

FILE*
fnxaccept(FILE* sd)
{
	struct Header_S header_s;
	char handler_name[sizeof(header_s.handler_name)];

	return fnxaccept2(sd,handler_name);
}

/* ============================================================ */
/*			====>    FNXACCEPT2    <====		*/
/*  recopie le handler_name dans handler_name                   */
/*  handler_name doit avoir ete alloue auparavant               */
/*  marche avec sockets AF_INET et AF_UNIX                      */
/* ============================================================ */

static void
nxerrorcon(const char* const r_host,const char* const nxr_host,
		const char* const husername,const char* const h_name,
		const char* const username)
{
	fprintf(stderr,"connexion non autorisee (.rhosts) :\n");
	fprintf(stderr,"machine remote = %s\n",r_host);
	fprintf(stderr,"machine nxremote = %s\n",nxr_host);
	fprintf(stderr,"utilisateur nxremote = %s\n",husername);
	fprintf(stderr,"machine = %s\n",h_name);
	fprintf(stderr,"utilisateur = %s\n\n",username);
}

static int
nxuserok(const char* const r_host,const char* const husername,
		const struct passwd* const pp)
{
	const char* const userhome = pp->pw_dir;

	if(userhome && !cacherhosts)
	{
	/* La premiere fois. Ensuite c'est en cache dans cacherhosts. */
	    struct stat sbuf;
	    int fd_index;
	    char userh[L_U8GetHome];

	    strcpy(userh,userhome);
	    strcat(userh,"/.rhosts");
	    U8OPEN(userh,O_RDONLY,0,fd_index);

	    if(fd_index < 0 || fstat(fd_index,&sbuf) ||
			    (sbuf.st_mode&S_IWGRP) ||
			    (sbuf.st_mode&S_IWOTH) ||
			    sbuf.st_uid != pp->pw_uid ||
			    sbuf.st_gid != pp->pw_gid)
	    {
		fprintf(stderr,"Ne peut pas ouvrir %s %d\n",userh,fd_index);

		if(fd_index >= 0)
		{
		    int ier = 0;

		    U8CLOSE(fd_index,ier);
		}

		return -1;
	    }

	    {
		const size_t lbuf = sbuf.st_size;
		char* const buf = cacherhosts = (char*) malloc(lbuf+1);

		if(!buf)
		{
		    int ier = 0;

		    U8CLOSE(fd_index,ier);
		    return -1;
		}

		{
		    ssize_t lus = 0;
		    int ier = 0;

		    U8READ(fd_index,buf,lbuf,lus);
		    U8CLOSE(fd_index,ier);

		    if(lus <= 0)
		    {
			free(buf);cacherhosts = NULL;return -1;
		    }

		    buf[lus] = '\0';
		}

	    }

	}

	{
	    const char* const buf = cacherhosts;
	    const char* const username = pp->pw_name;
	    const size_t lr_host_red = strcspn(r_host,".");
	    const size_t lr_host = strlen(r_host);
	    const size_t lr_hostre = lr_host + strlen(husername) + 22 +
		lr_host_red;
	    char* const r_hostre = (char*) malloc(lr_hostre);
	    regex_t cregex;

	    if(!r_hostre)
		return -1;

	/* r_host, + ou r_host_red dans ~/.rhosts */

	    if(!strcmp(husername,username))
	    {
		strcpy(r_hostre,"(^|\n) *(\\+|");
		strcat(r_hostre,r_host);
		strcat(r_hostre,"|");
		memcpy(r_hostre + lr_host + 12,r_host,lr_host_red);
		memcpy(r_hostre + lr_host + lr_host_red + 12,") *(\n|$)",9);

		if(!regcomp(&cregex,r_hostre,REG_EXTENDED|REG_NOSUB))
		{
		    const int ret =
			!regexec(&cregex,buf,0,(regmatch_t *) NULL,0);

		    regfree(&cregex);

		    if(ret)
		    {
			free(r_hostre);
			return 0;
		    }

		}

	    }

	/* r_host husername ou r_host_red husername dans ~/.rhosts */

	    strcpy(r_hostre,"(^|\n) *(");
	    strcat(r_hostre,r_host);
	    strcat(r_hostre,"|");
	    memcpy(r_hostre + lr_host + 9,r_host,lr_host_red);
	    r_hostre[lr_host + lr_host_red + 9] = '\0';
	    strcat(r_hostre,") *");
	    strcat(r_hostre,husername);
	    strcat(r_hostre," *(\n|$)");

	    if(!regcomp(&cregex,r_hostre,REG_EXTENDED|REG_NOSUB))
	    {
		const int ret = !regexec(&cregex,buf,0,(regmatch_t *) NULL,0);

		regfree(&cregex);

		if(ret)
		{
		    free(r_hostre);
		    return 0;
		}

	    }

	    free(r_hostre);
	    return -1;
	}

}

static int
nxtestrhost(const int ns,const char* const r_host,const char* const h_name,
		const char* const nxr_host,const char* const nxusername)
{
	struct passwd* const pp = U8GetPasswd();
	const char* const username = pp ? pp->pw_name : NULL;

	if(!username || !strlen(username))
	{
	    int ier = 0;

	    NXERRNO = 64;
	    shutdown(ns,2);
	    U8CLOSE(ns,ier);
	    return 1;
	}

	{
	    const size_t lr_host = strlen(r_host);
	    const size_t lr_host_red = strcspn(r_host,".");
	    const size_t lh_name = strlen(h_name);
	    const size_t lh_name_red = strcspn(h_name,".");
	    const size_t lhost_name = strlen(nxr_host);
	    const size_t lhost_name_red = strcspn(nxr_host,".");
	    const int test_nosamehost =
		(((lr_host != lh_name && lr_host_red != lh_name_red) ||
				memcmp(h_name,r_host,lr_host_red)) ||
				strcmp(username,nxusername)) &&
		nxuserok(r_host,nxusername,pp) < 0;
	    const int test_triche = (lhost_name != lr_host &&
			    lhost_name_red != lr_host_red) ||
		memcmp(nxr_host,r_host,lr_host_red);

	    if(test_nosamehost || test_triche)
	    {
		int ier = 0;

		shutdown(ns,2);
		U8CLOSE(ns,ier);
		nxerrorcon(r_host,nxr_host,nxusername,h_name,username);
		return 1;
	    }

	}

	return 0;
}

FILE*
fnxaccept2(FILE* sd,char handler_name[])
{
	FILE* fd = NULL;
	union {struct sockaddr_in _in;struct sockaddr_un _un;} fsin;
	struct sockaddr* fsin_gen = (struct sockaddr*) &fsin;
	int ns = 0,ind = 0;
	const int sock = fileno(sd);
	NX_GSN_SIZE fromlen = sizeof(fsin);
	struct Header_S header_s;

	memset((char*) &fsin,0,sizeof(fsin));

	while((ns = accept(sock,fsin_gen,&fromlen)) < 0 && errno == EINTR);

	if(ns < 0)
	{
	    perror("fnxaccept2 : accept ");
	    return NULL;
	}

	if(ns >= NX_MAX_DESC)
	{
	    int ier = 0;

	    NXERRNO = 60;
	    fnxperror("");
	    shutdown(ns,2);
	    U8CLOSE(ns,ier);
	    return NULL;
	}

	{
	    char r_host[MAXHOSTNAMELEN]; /* adresse du client */
	    char h_name[MAXHOSTNAMELEN];
	    const char* const localname = "localhost";
	    ssize_t lg = 0;

	/* adresse du serveur qui appelle cette routine : */

	    if(U8GetHostname(h_name,sizeof(h_name)) == -1 || !strlen(h_name))
	    {
		int ier = 0;

		perror("fnxaccept2 : gethostname ");
		NXERRNO = 53;
		shutdown(ns,2);
		U8CLOSE(ns,ier);
		return NULL;
	    }

	    if(fsin_gen->sa_family == AF_INET)
	    {
		char* pt_addr = (char*) &fsin._in.sin_addr;
		const int taille_addr = sizeof(fsin._in.sin_addr);
		struct hostent* const hste = gethostbyaddr(pt_addr,taille_addr,
				fsin_gen->sa_family);

		if(strcmp(hste->h_name,localname))
		{

		    if(strlen(hste->h_name) >= sizeof(r_host))
		    {
			int ier = 0;

			perror("fnxaccept2 : gethostbyaddr ");
			NXERRNO = 53;
			shutdown(ns,2);
			U8CLOSE(ns,ier);
			return NULL;
		    }

		    strcpy(r_host,hste->h_name);
		}
		else
		    strcpy(r_host,h_name);

	    }
	    else if(fsin_gen->sa_family == AF_UNIX) /* on est en local */
		strcpy(r_host,h_name);
	    else
	    {
		int ier = 0;

		perror("fnxaccept2 : protocol ");
		NXERRNO = 54;
		shutdown(ns,2);
		U8CLOSE(ns,ier);
		return NULL;
	    }

	/* lecture header convention nx */

	    U8READ(ns,&header_s,SIZ_HDSL,lg);

	    if(lg < SIZ_HDSL)
		goto errheader2;

	    {
		int xsize = 0;

		sscanf(header_s.lg,"%d",&xsize);

		if(xsize == SIZ_HDS)
		{
		    U8READ(ns,header_s.host_name,L2,lg);

		    if(lg < L2)
			goto errheader2;

		    U8READ(ns,header_s.user_name,L2,lg);

		    if(lg < L2)
			goto errheader2;

		}
		else
		{
		    u_int_4 xs = 0;

		    U8READ(ns,&xs,4,lg);
		    xs = ntohl(xs);

		    if(lg < 4 || xs > L2X)
			goto errheader2;

		    U8READ(ns,header_s.host_name,xs,lg);

		    if(lg < xs)
			goto errheader2;

		    U8READ(ns,&xs,4,lg);
		    xs = ntohl(xs);

		    if(lg < 4 || xs > L2X)
			goto errheader2;

		    U8READ(ns,header_s.user_name,xs,lg);

		    if(lg < xs)
			goto errheader2;

		}

	    }

	/* controle d'acces aux conventions /etc/.rhosts */
	    if(nxtestrhost(ns,r_host,h_name,header_s.host_name,
					    header_s.user_name))
		return NULL;

	/* suite lecture header convention nx */

	    U8READ(ns,header_s.port_name,L3,lg);

	    if(lg < L3)
		goto errheader2;

	    U8READ(ns,header_s.handler_name,L3,lg);

	    if(lg < L3)
		goto errheader2;

	}

	if(!(fd = fdopen(ns,"r")))
	{
	    int ier = 0;

	    perror("fnxaccept2 : fdopen ");
	    shutdown(ns,2);
	    U8CLOSE(ns,ier);
	    return NULL;
	}

	nx_desc_table[ns].dp.fd = nx_fdin_table[ns] = fd;
	nx_desc_table[ns].ds = NXP_SOCKET;

    /* par defaut format = celui de la socket d'ecoute (serveur) : */

	nxtabfrm[ns].fmt = ind = nxtabfrm[sock].fmt; 
	nx_init_access(ns,ind);
	strcpy(handler_name,header_s.handler_name);

#ifdef DEBUG
	printf("fnxaccept2 : good for accept, fileno = %d\n",ns);
#endif

	return fd;

 errheader2:{
	    int ier = 0;

	    perror("fnxaccept2 : header ");
	    shutdown(ns,2);
	    U8CLOSE(ns,ier);

	    return NULL;
	}

}

/* =================================================================== */
/* NXSELECT : tabfd etant un tableau de nb descripteurs I/O            */
/* on renvoie fd, le 1er descripteur pret pour l'operation demandee :  */
/* bouclage sur la liste des descripteurs, tant que l'un d'eux n'est   */
/* pas selectionne.                                                    */
/* lecture si mode = 0; ecriture si mode = 1.                          */
/* rc = -1 -> erreur                                                   */
/*    =  0 -> pas de descripteur                                       */
/*    =  1 -> ok                                                       */
/* =================================================================== */

FILE*
fnxselect(const int nb,FILE* tabfd[],const int mode,int* rc)
{
	FILE* fd = NULL;
	int i = 0;
	int width = 0;
	fd_set readfds,writefds;
	struct timeval timeout;
	int tabnf[NX_MAX_DESC];
	int isto = 0;

    /* on teste d'abord s'il y a des donnees dans l'un des buffers
       en lecture des tabfd */

	*rc = 0;

	for (i = 0; i < nb; i++)
	{

	    if(fnx_data_in_buffer(tabfd[i]))
	    {
		isto = i;
		*rc = 1;
		break;
	    }

	}

	for (i=0 ; i<nb ; tabnf[i]=fileno(tabfd[i]),i++);

	if(*rc > 0)
	{
	    fd = nx_desc_table[tabnf[isto]].dp.fd;

	    return fd;
	}

    /* sinon creation des readfds et writefds pour select */

	FD_ZERO(&readfds);
	FD_ZERO(&writefds);

	if(mode == 0)
	{

	    for (i=0; i < nb ; i++)           
	    {
		FD_SET(tabnf[i],&readfds);
	    }

	}
	else
	{

	    for (i=0; i < nb ; i++)           
	    {
		FD_SET(tabnf[i],&writefds);
	    }

	}

    /* appel de select */

	for (;;)
	{
	    timeout.tv_sec = 3;    
	    timeout.tv_usec = 0;   

	    width = 32;

	    if (mode == 0)
	    {
#if defined(HP_UX) && !defined(_XPG4_EXTENDED)
		*rc = select(width,(int*) &readfds,NULL,NULL,&timeout);
#else
		*rc = select(width,&readfds,NULL,NULL,&timeout);
#endif

		if (*rc == 0) return NULL;

		if (*rc > 0)
		{

		    for (i=0; i < nb; i++)
		    {

			if (FD_ISSET(tabnf[i],&readfds))
			{
			    fd = nx_desc_table[tabnf[i]].dp.fd;
			    return fd;
			}

			fd = NULL;
			
		    }

		}
		else return NULL;
	    }
	    else if (mode == 1)
	    {
#if defined(HP_UX) && !defined(_XPG4_EXTENDED)
		*rc = select(width,NULL,(int*) &writefds,NULL,&timeout);
#else
		*rc = select(width,NULL,&writefds,NULL,&timeout);
#endif

		if (*rc == 0) return NULL;

		if (*rc > 0)
		{

		    for (i=0; i < nb; i++)
		    {

			if (FD_ISSET(tabnf[i],&writefds))
			{
			    fd = nx_desc_table[tabnf[i]].dp.fd;
			    return fd;
			}

			fd = NULL;
			
		    }

		}
		else return NULL;
	    }

	}

}

/***********************************************************/
/*   variante qui renvoit le numero dans le tableau tabfd  */
/***********************************************************/

int
fnxselectnum(const int nb,FILE* tabfd[],const int mode,int* rc)
{
	int i = 0;
	int width = 0;
	fd_set readfds,writefds;
	struct timeval timeout;
	int tabnf[NX_MAX_DESC];
	int isto = 0;

    /* on teste d'abord s'il y a des donnees dans l'un des buffers
       en lecture des tabfd */

	*rc = 0;

	for (i = 0; i < nb; i++)
	{

	    if(fnx_data_in_buffer(tabfd[i]))
	    {
		isto = i;
		*rc = 1;
		break;
	    }

	}

	if(*rc > 0)
	{
	    return isto;
	}

    /* sinon creation des readfds et writefds pour select */

	for (i=0 ; i<nb ; tabnf[i]=fileno(tabfd[i]),i++);

	FD_ZERO(&readfds);
	FD_ZERO(&writefds);

	if(mode == 0)
	{

	    for (i=0; i < nb ; i++)           
	    {
		FD_SET(tabnf[i],&readfds);
	    }

	}
	else
	{

	    for (i=0; i < nb ; i++)           
	    {
		FD_SET(tabnf[i],&writefds);
	    }

	}

    /* appel de select */

	for (;;)
	{
	    timeout.tv_sec = 3;    
	    timeout.tv_usec = 0;   
	    width = 32;

	    if (mode == 0)
	    {
#if defined(HP_UX) && !defined(_XPG4_EXTENDED)
		*rc = select(width,(int*) &readfds,NULL,NULL,&timeout);
#else
		*rc = select(width,&readfds,NULL,NULL,&timeout);
#endif

		if (*rc == 0) return 0;

		if (*rc > 0)
		{

		    for (i=0; i < nb; i++)
		    {

			if (FD_ISSET(tabnf[i],&readfds))
			{
		            return i;
			}

	            }

		}
		else return 0;

	    }
	    else if (mode == 1)
	    {
#if defined(HP_UX) && !defined(_XPG4_EXTENDED)
		*rc = select(width,NULL,(int*) &writefds,NULL,&timeout);
#else
		*rc = select(width,NULL,&writefds,NULL,&timeout);
#endif

		if (*rc == 0) return 0;

		if (*rc > 0)
		{

		    for (i=0; i < nb; i++)
		    {

			if (FD_ISSET(tabnf[i],&writefds))
			{
		            return i;
			}

	            }

		}
		else return 0;

	    }

	}

}

/************************************************************************/
/*		         ====>    NET_ADDR    <====	     	        */
/************************************************************************/

static int
net_addr(const char* buf_name,char** host_name,char** user_name,
		int* port_number,int* port_defaut,char** port_name,
		char** port_service,int* port_which,char** handler_name)
{
	char* ptr = NULL;
	int nb_token = 0, rc = 0;
	const char* sto_pos = NULL;
	char* sto_str = NULL;
	const char separator[] = "+: ";
	const char separator2[] = ".";
	const char anti_separator[] = "'\"";

    /* champs : machine+port+handler+user (+ peut etre remplace par : ou blanc)
       si les 4 champs sont de longueur non nulle et si un + : ou blanc
       est en tete alors il est elimine (ancien comportement maintenu), sinon
       + : ou blanc en tete signifie maintenant que la machine doit-etre
       choisie par defaut et egale a la machine locale. Dans ce cas
       on ne l'elimine donc plus.

       plus precisement ([...] <=> facultatif) :
       [host_name]+
       [["]port["][.[["]port_service["]][.[port_which]]]][+[handler_name][+
       [user_name]]]

       ou si pas de + :
       host_name = nom de fichier ordinaire

       + signifie +, : ou blanc
       " signifie " ou '
       port signifie port_number ou port_name

       host_name <= machine
       port_* <= port
       handler_name <= handler
       user_name <= user

       voir nx.h pour la signification de machine, ...
       */

	*host_name = NULL;
	*user_name = NULL;
	*handler_name = NULL;
	*port_number = -1;
	*port_defaut = 0;
	*port_name = NULL;
	*port_service = NULL;
	*port_which = -1;

	NXERRNO = 0;
	errno = 0;

    /********************************/
    /* recherche du champ machine : */
    /********************************/

	if((ptr = strtok2(buf_name, separator,NULL,&sto_pos,&sto_str)))
	{
	    *host_name = ptr;
	    nb_token++;

	/********************************/
	/* recherche du champ port :    */
	/********************************/

	    if((ptr = strtok2(NULL,separator,anti_separator,&sto_pos,&sto_str)))
	    {
	    /* port_name ou port_number */

		const char* sto_pos2 = NULL;
		char* sto_str2 = NULL;
		char* ptr2 = strtok2(ptr,separator2,anti_separator,&sto_pos2,
				&sto_str2);
		char* ptr_port = ptr2;

		stroteg(ptr_port);
		*port_which = 1;

	    /* port_service */

		if((ptr2 = strtok2(NULL,separator2,anti_separator,&sto_pos2,
				&sto_str2)))
		{
		/* le port_number ou port_name deviendra le port par defaut : */

		    *port_defaut = 1;

		    if(strlen(ptr2))
		    {
			*port_service = ptr2;
			stroteg(*port_service);
		    }
		    else
		    {
			free(ptr2);
		    }

		/* port_which */

		    if((ptr2 = strtok2(NULL,separator2,NULL,&sto_pos2,
				    &sto_str2)))
		    {

			if((int) strlen(ptr2) > 0 && (int) strlen(ptr2) <= 10)
			{
			    *port_which = atoi(ptr2);
			}

			free(ptr2);
		    }
 
		    if(sto_str2) free(sto_str2);

		}

		if(!(*port_service))
		{
		    ptr2 = (char*) malloc(3);
		    memcpy(ptr2,"nx",3);
		    *port_service = ptr2;
		}

		free(ptr);

	    /* port_name ou port_number */
		const size_t lptr_port = strlen(ptr_port);

		if(!lptr_port)
		{
		    *port_number = 0;

		    free(ptr_port);
		}
		else if(*ptr_port == '/')
		{
		/* port_name pour sockets unix */
		/* commence par / */
		/* si /UNIX/ => choix automatique */

		    if(lptr_port == 6 && !memcmp(ptr_port,"/UNIX/",6))
		    {
			const char* const port_tmp = "/tmp/.nx-unix-serv.XXXXXX";
			const size_t lport_tmp = strlen(port_tmp);
			int j = 0;
			char* port_temp = (char*) malloc(lport_tmp + 4);

			free(ptr_port);

			for(;j<10;j++)
			{
			    memcpy(port_temp,port_tmp,lport_tmp + 1);
			    *port_name = mktemp(port_temp);
			    
			    if(mkdir(port_temp,S_IRWXU | S_IRWXG | S_IROTH) != -1)
				break;

			}

			*port_defaut = 1;

			if(j == 10 || (*port_name)[0] == '\0')
			{
			    NXERRNO = 55;
			}

			if(strcmp(*host_name,"SERVER"))
			{
			    NXERRNO = 72;
			}

			if(!NXERRNO)
			    memcpy(port_temp + lport_tmp,"/nx",4);

		    }
		    else
			*port_name = ptr_port;

		}
		else if (isalpha((int) (*ptr_port)))
		{
		    struct servent* servent;

		    if(!(servent = getservbyname(ptr_port, NULL)))
		    {
			NXERRNO = 56;
		    }
		    else
		    {
			*port_number = ntohs(servent->s_port);
		    }

		    free(ptr_port);
		}
		else if((int) lptr_port <= 10)
		{
		    *port_number = atoi(ptr_port);
		    free(ptr_port);
		}

		nb_token++;

	    /********************************/
	    /* recherche du champ handler : */
	    /********************************/

		if((ptr = strtok2(NULL,separator,NULL,&sto_pos,&sto_str)))
		{
		    *handler_name = ptr;
		    nb_token++;

		/********************************/
		/* recherche du champ user :    */
		/********************************/

		    if((ptr = strtok2(NULL,separator,NULL,&sto_pos,&sto_str)))
		    {
			*user_name = ptr;
			nb_token++;

		    /*************************************/
		    /* test si champ en trop => erreur : */
		    /*************************************/

			if(sto_pos && strlen(sto_pos)) nb_token++;

			if(sto_str) free(sto_str);

		    }

		}

	    }

	}

    /* il faut au moins un separateur + : ou blanc car c'est comme cela
       que l'on decide que le fichier est une socket =>
       dans le cas 1 c'est un fichier : */

	if(nb_token == 1)
	{
	    return 1;
	}

    /* si 0 separateur ou plus de 4 separateurs => On essai d'enlever le 1er
       separateur s'il est en 1ere position et on recommence sinon erreur : */

	if(!nb_token || nb_token > 4)
	{

	    if(!strcspn(buf_name,separator))
	    {
		const char* const buf_namer = buf_name + 1;

		if(*host_name) free(*host_name);
		if(*user_name) free(*user_name);
		if(*handler_name) free(*handler_name);
		if(*port_name) free(*port_name);
		if(*port_service) free(*port_service);

		return net_addr(buf_namer,host_name,user_name,port_number,
				port_defaut,port_name,port_service,
				port_which,handler_name);
	    }

	    NXERRNO = 3;
	    return -1;
	}

    /* si une erreur au dessus */

	if(NXERRNO)
	{
	    return -1;
	}

    /* tests sur host_name */

	ptr = *host_name;

	if((int) strlen(ptr) > MAXHOSTNAMELEN)
	{
	    NXERRNO = 3;
	    return -1;
	}

	if(!strlen(ptr))
	{
	/* on cherche la machine locale */
	    char h_name[MAXHOSTNAMELEN];
	    size_t lh_name;

	    if(U8GetHostname(h_name,sizeof(h_name)) == -1 ||
			    !(lh_name = strlen(h_name)))
	    {
		NXERRNO = 57;
		return -1;
	    }

	    lh_name++;
	    free(ptr);

	    if(!(ptr = (char*) malloc(lh_name)))
	    {
		NXERRNO = 58;
		return -1;
	    }

	    memcpy(ptr,h_name,lh_name);
	    *host_name = ptr;
	}

    /* si pas de handler */

	if(!(*handler_name))
	{

	    if(!(*handler_name = (char*) malloc(1)))
	    {
		NXERRNO = 58;
		return -1;
	    }

	    *handler_name[0] = '\0';
	}

    /* si le user n'est pas donne, on le prend user du process par defaut */

	if(!(*user_name) || !strlen(*user_name))
	{
	    const char* const u_name = U8GetUser(NULL);
	    size_t lu_name = strlen(u_name);

	    if(!lu_name)
	    {
		NXERRNO = 59;
		return -1;
	    }

	    lu_name++;

	    if(*user_name) free(*user_name);

	    if(!(*user_name = (char*) malloc(lu_name)))
	    {
		NXERRNO = 58;
		return -1;
	    }

	    memcpy(*user_name,u_name,lu_name);
	}

	return rc;
}

/* -----------------------------------------------------*/
/*              ====>    STROTEG     <====              */
/* -----------------------------------------------------*/

static void
stroteg(char* str)
{
    /* si str est de la forme "str" ou 'str', stroteg renvoit str
       le resultat est laisse dans str
       */

	const size_t lstr = strlen(str);

	if(str && lstr >= 2)
	{
	    register char str0 = str[0];
	    register char strl = str[lstr-1];

	    if((str0 == '"' || str0 == '\'') && (strl == '"' || strl == '\''))
	    {
		memmove(str,str+1,lstr-2);
		str[lstr-2] = '\0';
	    }

	}

}

/* -----------------------------------------------------*/
/*              ====>    STRSAS     <====               */
/* -----------------------------------------------------*/

static size_t
strsas(const char* str,const char* sep,const char* anti_sep)
{

    /* Retourne successivement le nombre de caracteres consecutifs
       a partir du debut de str ne faisant pas partie de sep si ceux-ci
       ne sont pas encadres par 2 caracteres identiques de anti_sep.
       */

	size_t i = strcspn(str,sep);
	size_t lstr = strlen(str);

    /* traitement de anti_sep */

	if(anti_sep && i < lstr)
	{
	    size_t jmin = strcspn(str,anti_sep);

	    if(jmin < lstr)
	    {
		register size_t jm = jmin + 1;
		register size_t jmax = strcspn(str+jm,anti_sep) + jm;

		while(i>jmax && i>jmin && jmin < lstr && jmax < lstr)
		{
		    jm = jmax + 1;
		    jmin = strcspn(str + jm,anti_sep) + jm;
		    jm = jmin + 1;

		    if(jmin < lstr)
			jmax = strcspn(str+jm,anti_sep) + jm;

		}

		while(i>=jmin && i<= jmax && i<lstr && jmax < lstr)
		{
		    const char* str_temp;

		    jm = jmax + 1;
		    str_temp = str + jm;
		    i = strcspn(str_temp,sep) + jm;
		    jmin = strcspn(str_temp,anti_sep) + jm;
		    jm = jmin + 1;

		    if(jmin < lstr)
		    {
			jmax = strcspn(str + jm,anti_sep) + jm;

			while(i>jmax && i>jmin && jmin < lstr &&
					jmax < lstr)
			{
			    jm = jmax + 1;
			    jmin = strcspn(str + jm,anti_sep) + jm;
			    jm = jmin + 1;

			    if(jmin < lstr)
				jmax = strcspn(str + jm,anti_sep) + jm;

			}

		    }

		}

	    }

	}

	if(i>=lstr)
	{
	    i = lstr;
	}

	return i;		
}

/* -----------------------------------------------------*/
/*              ====>    STRTOK2     <====              */
/* -----------------------------------------------------*/

static char*
strtok2(const char* str,const char* sep,const char* anti_sep,
		const char** psto_pos,char** psto_str)
{
    /* La variable de retour est alloue dans cette fonction par calloc */
    /* a l'utilisateur de la desallouer. */
    /* Retourne successivement les champs de str separes par les caracteres
       de sep sachant que les caracteres de sep se trouvant entre 2
       caracteres de meme type de anti_sep ne sont pas consideres.
       Un caractere de anti_sep precede de \ est considere quand meme (a voir).
       */
    /* On donne str au premier appel de strtok2 et on obtient le premier
       champ. On rappelle strtok2 avec str=NULL pour avoir les champs
       suivants jusqu'a ce que strtok2 renvoie NULL.
       */
    /* psto_pos et psto_str sont des pointeurs de travail utilises par strtok2
       qui doivent rester identiques pour str donne et initialises a NULL.
       */

	const char* sto_pos = *psto_pos;
	char* sto_str = *psto_str;
	char* str2 = NULL;
	size_t i;
	size_t lsto_pos;

	if(str)
	{

	    if(sto_str)
	    {
		free(sto_str);
	    }

	    sto_str = (char*) calloc(1,strlen(str)+1);
	    strcpy(sto_str,str);
	    sto_pos = sto_str;
	}
	else if(!sto_str)
	{
	    return NULL;
	}

	lsto_pos = strlen(sto_pos);
	i = strsas(sto_pos,sep,anti_sep);

	str2 = (char*) calloc(1,i+1);
	memcpy(str2,sto_pos,i);

	if(i<lsto_pos)
	{
	    sto_pos += i+1;
	}
	else
	{
	    free(sto_str);
	    sto_str = NULL;
	    sto_pos = NULL;
	}

	*psto_pos = sto_pos;
	*psto_str = sto_str;

	return str2;		
}

/* ============================================================ */
/*			====>     INITFORM     <====		*/ 
/* ============================================================ */

static int
initform(char* filename,int* format)
{

    /* the filename gives an indication on its format
       access functions structures are initialized
       filename is returned without @... 
       default format is the host one
       this function returns 0 if ok or -1 on error */

	static char separ[] = "@";
	char* ptc = strpbrk(filename,separ);
	int cr=0,i=0;

	if(ptc)
	{
	    *ptc = '\0';
	    ptc++;

	    while(strcmp(formats[i].typfor, ptc) && ++i < maxform);

	    if(i == maxform)
	    {
		NXERRNO = 17;
		fnxperror("");
		cr = -1;   
	    }
	    else
	    {
		*format = i + 1;
	    }

	    ptc = strtok(filename,separ);
	}
	else
	{
	    *format = 0;  /* no @ => host format */
	}

	return cr;
}

/* ============================================================ */
/*			====>   NXCHFRM       <====		*/ 
/* ============================================================ */
/* Dynamic update of the file format				*/
/* ============================================================ */

int
fnxchfrm(FILE* fd,const char* codfmt)
{
	register int i = 0;
	register const int nf = fileno(fd);
	register const size_t lcod = MIN(NX_LFORMAT,strlen(codfmt));

	if(!strncmp(nxtabfrm[nf].format,codfmt,lcod)) return (0);

	while(strncmp(nxtypc[i].code,codfmt,lcod) && ++i<maxcodtyp);

	if (i == maxcodtyp)
	{
	    NXERRNO = 30;
	    return -1;   
	}
	else
	{
	    strncpy(nxtabfrm[nf].format,codfmt,lcod);
	    nxtabfrm[nf].format[NX_LFORMAT-1] = '\0';
	    nxtabfrm[nf].fmt = i;
	    return nx_init_access(nf,i);
	}

}
