/*                           N X - A M                                 */
/*                           ---------                                 */
/*                        FORTRAN interface                            */    

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992    MA/AIM";
#endif

/***************************************************************************/

#include "nxP.h"
#include "nx.h"
#include "nxnarcisseP.h"
#include "nxfrmt.h"
#include "U8machine_types.h"

/* ------------------------------------------------------------------------ */
/*                    macros utiles                                         */
/* ------------------------------------------------------------------------ */

#define FTNFILDES nx_desc_table[*fd_index].dp.fd

#define NX_LFORMAT 6

#define STR_FOR2C(for_name, for_len,c_name,max_len)\
	{\
	  int _lg = noblkq(for_name,MIN(for_len,max_len));\
          _lg = MIN(_lg,max_len-1);\
	  memcpy(c_name,for_name,_lg);\
	  c_name[_lg] = '\0';\
	}

#define SOCDES nx_desc_table[*sd_index].dp.fd

/* ------------------------------------------------------------------------ */
/*                    fonctions static utiles                               */
/* ------------------------------------------------------------------------ */

static int noblkq(const char* str,int len);

static void
initstdfiles(int fd_index)
{

	switch (fd_index)
	{
 	/*  you can use stdin, stderr, sdout without any nxopen() */

	  case 0 :

	    nx_desc_table[0].dp.fd = stdin;
	    nx_desc_table[0].ds = NXP_FILE;
	    break;

	  case 1 :

	    nx_desc_table[1].dp.fd = stdout;
	    nx_desc_table[1].ds = NXP_FILE;
	    break;

	  case 2 :

	    nx_desc_table[2].dp.fd = stderr;
	    nx_desc_table[2].ds = NXP_FILE;
	    break;
	}

}

/* ------------------------------------------------------------------------ */

/* ============================================================ */
/*			====>     NXOPEN     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXOPEN(_fcd name,_fcd type,int* fd_index,int* rc)
{
	char c_name[FILE_MAX_LEN + 1] ;
	char c_type[TYPE_MAX_LEN + 1] ;
	FILE* fd;

	STR_FOR2C(_fcdtocp(name),_fcdlen(name),c_name,FILE_MAX_LEN);
	STR_FOR2C(_fcdtocp(type),_fcdlen(type),c_type,TYPE_MAX_LEN);
	*rc = 0;

	if((fd = fnxopen(c_name,c_type)) == NULL)
	{
	    *rc = -1;
	}
	else
	    *fd_index = (int) fileno(fd);
}
#else
void
FUNCDECL(nxopen)(char* name,char* type,int* fd_index,int* rc,
		int name_lgth,int type_lgth)
{
	char c_name[FILE_MAX_LEN + 1] ;
	char c_type[TYPE_MAX_LEN + 1] ;
	FILE* fd;

	STR_FOR2C(name,name_lgth,c_name,FILE_MAX_LEN);
	STR_FOR2C(type,type_lgth,c_type,TYPE_MAX_LEN);
	*rc = 0;

	if((fd = fnxopen(c_name,c_type)) == NULL)
	    *rc = -1;
	else
	    *fd_index = (int) fileno(fd);

}
#endif

/* ============================================================ */
/*			====>     NXACCEPT     <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXACCEPT(int* sd_index,int* fd_index,int* rc)
#else
FUNCDECL(nxaccept)(int* sd_index,int* fd_index,int* rc)
#endif
{
	FILE* fd;

	if (!(fd = fnxaccept(SOCDES)))
	{
	    *rc = -1;
	}
	else
	{
	    *fd_index = (int) fileno(fd);
	    *rc = 0;
	}
      
}

/* ============================================================ */
/*			====>     NXSELECT     <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXSELECT(int* nb,int* tabfd_index,int* fd_index,int* mode,
	int* rc)
#else
FUNCDECL(nxselect)(int* nb,int* tabfd_index,int* fd_index,
	int* mode,int* rc)
#endif
{
	FILE* tabfd[NX_MAX_DESC];
	FILE* fd;
	int i = 0;

	for (;i<*nb;i++)
	{
	    tabfd[i] = nx_desc_table[tabfd_index[i]].dp.fd;
	}

	fd = fnxselect(*nb,tabfd,*mode,rc);

	if (*rc == 1)
	    *fd_index = (int) fileno(fd);

}

/* ============================================================ */
/*			====>     NXCLOSE     <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXCLOSE(int* fd_index,int* rc)
#else
FUNCDECL(nxclose)(int* fd_index,int* rc)
#endif
{
	initstdfiles(*fd_index);
	*rc = (int) fnxclose(FTNFILDES) ;
}

/* ============================================================ */
/*		   ====>  NXLPORT     <====                     */
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXLPORT()
#else
FUNCDECL(nxlport)()
#endif
{
	fnx_lance_rpc_port();
}

/* ============================================================ */
/*			====>     NXWRITE      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXWRITE(register int* fd_index,_fcd buffer,register int* lg,
		register int* lgecrit,register int* rc)
{
	const char* const buf = _fcdtocp(buffer);
	const int lbuf = MIN(_fcdlen(buffer),*lg);
#else
FUNCDECL(nxwrite)(register int* fd_index,char* buffer,
		register int* lg,register int* lgecrit,
		register int* rc,int lb)
{
	const char* const buf = buffer;
	const int lbuf = MIN(lb,*lg);
#endif

	*rc = 0;
	initstdfiles(*fd_index);

	if((*lgecrit = (int) fnxwrite(FTNFILDES, buf, lbuf)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKWI      <====		*/
/* ============================================================ */

void 
#ifdef CRAY_PROTO
NXNBKWI(register int* fd_index,int* buffer,register int* lg,
		register int* lgecrit,register int* rc)
#else
FUNCDECL(nxnbkwi)(register int* fd_index,int* buffer,
		register int* lg,register int* lgecrit,
		register int* rc)
#endif
{
	*rc = 0;
	initstdfiles(*fd_index);

	if ((*lgecrit = (int) fnxnbkwi(FTNFILDES, buffer, *lg)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKWF      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXNBKWF(register int* fd_index,NXFLT* buffer,register int* lg,
		register int* lgecrit,register int* rc)
#else
FUNCDECL(nxnbkwf)(register int* fd_index,NXFLT* buffer,
		register int* lg,register int* lgecrit,
		register int* rc)
#endif
{
	*rc = 0;
	initstdfiles(*fd_index);

	if ((*lgecrit = (int) fnxnbkwf(FTNFILDES, buffer, *lg)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKWT      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXNBKWT(register int* fd_index,void* buffer,register int* lg,
		_fcd icod,_fcd ocod,register int* lgecrit,
		register int* rc)
{
	const char* const icodt = _fcdtocp(icod);
	const char* const ocodt = _fcdtocp(ocod);
#else
FUNCDECL(nxnbkwt)(register int* fd_index,void* buffer,
		register int* lg,register char* icod,
		register char* ocod,register int* lgecrit,
		register int* rc,int li,int lo)
{
	const char* const icodt = icod;
	const char* const ocodt = ocod;
#endif

	*rc = 0;
	initstdfiles(*fd_index);

	if ((*lgecrit =
		(int) fnxnbkwt(FTNFILDES, buffer, *lg,icodt,ocodt)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXREAD      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXREAD(register int* fd_index,_fcd buffer,register int* lg,
		register int* lglus,register int* rc)
{
	char* const buf = _fcdtocp(buffer);
	const int lbuf = MIN(_fcdlen(buffer),*lg);
#else
FUNCDECL(nxread)(register int* fd_index,char* buffer,register int* lg,
		register int* lglus,register int* rc,int lb)
{
        char* const buf = buffer;
	const int lbuf = MIN(lb,*lg);
#endif

	*rc = 0;
	initstdfiles(*fd_index);
	
	if ((*lglus = (int) fnxread(FTNFILDES, buf, lbuf)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKRI      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXNBKRI(register int* fd_index,int* buffer,register int*  lg,
		register int*  lglus,register int*  rc)
#else
FUNCDECL(nxnbkri)(register int* fd_index,int* buffer,
		register int* lg,register int* lglus,
		register int* rc)
#endif
{
	*rc = 0;
	initstdfiles(*fd_index);
	
	if ((*lglus = (int) fnxnbkri(FTNFILDES, buffer, *lg)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKRF      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXNBKRF(register int* fd_index,NXFLT* buffer,register int* lg,
		register int* lglus,register int* rc)
#else
FUNCDECL(nxnbkrf)(register int* fd_index,NXFLT* buffer,
		register int* lg,register int* lglus,
		register int* rc)
#endif
{
	*rc = 0;
	initstdfiles(*fd_index);
	
	if ((*lglus = (int) fnxnbkrf(FTNFILDES, buffer, *lg)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXNBKRT      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXNBKRT(register int* fd_index,NXFLT* buffer,register int* lg,
		_fcd icod,_fcd ocod,register int* lglus,
		register int* rc)
{
	const char* const icodt = _fcdtocp(icod);
	const char* const ocodt = _fcdtocp(ocod);
#else
FUNCDECL(nxnbkrt)(register int* fd_index,NXFLT* buffer,
		register int* lg,register char* icod,register char* ocod,
		register int* lglus,register int* rc,int li,int lo)
{
	const char* const icodt = icod;
	const char* const ocodt = ocod;
#endif

	*rc = 0;
	initstdfiles(*fd_index);
	
	if ((*lglus =
		(int) fnxnbkrt(FTNFILDES, buffer, *lg,icodt,ocodt)) <= 0)
	    *rc = -1;

}

/* ============================================================ */
/*			====>     NXWRITI      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXWRITI(register int* fd_index,register int* nb,int* buffer,
		register int* rc)
#else
FUNCDECL(nxwriti)(register int* fd_index,register int* nb,
		int* buffer,register int* rc)
#endif
{
	initstdfiles(*fd_index);
	*rc = (int) fnxwriti(FTNFILDES, *nb, buffer);
}

/* ============================================================ */
/*			====>     NXREADI      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXREADI(register int* fd_index,register int* nb,
		int* buffer,register int* rc)
#else
FUNCDECL(nxreadi)(register int* fd_index,register int* nb,
		int* buffer,register int* rc)
#endif
{
	initstdfiles(*fd_index);
	*rc = (int) fnxreadi(FTNFILDES, *nb, buffer);
}

/* ============================================================ */
/*			====>     NXWRITF      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXWRITF(register int* fd_index,register int* nb,NXFLT* buffer,
		register int* rc)
#else
FUNCDECL(nxwritf)(register int* fd_index,register int* nb,
		NXFLT* buffer,register int* rc)
#endif
{
	initstdfiles(*fd_index);
	*rc = (int) fnxwritf(FTNFILDES, *nb, buffer);
}

/* ============================================================ */
/*			====>     NXREADF      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXREADF(register int* fd_index,register int* nb,NXFLT* buffer,
		register int* rc)
#else
FUNCDECL(nxreadf)(register int* fd_index,register int* nb,
		NXFLT* buffer,register int* rc)
#endif
{
	initstdfiles(*fd_index);
	*rc = (int) fnxreadf(FTNFILDES, *nb, buffer);
}

/* ============================================================ */
/*			====>     NXWRITT      <====		*/
/* ============================================================ */
/* Format & Type conversions                                    */

void
#ifdef CRAY_PROTO
NXWRITT(register int* fd_index,register int* nb,void* buffer,
		_fcd icod,_fcd ocod,int* rc)
{
	const char* const icodt = _fcdtocp(icod);
	const char* const ocodt = _fcdtocp(ocod);
#else
FUNCDECL(nxwritt)(register int* fd_index,register int* nb,
		void* buffer,register char* icod,register char* ocod,
		register int* rc,int li,int lo)
{
	const char* const icodt = icod;
	const char* const ocodt = ocod;
#endif

	initstdfiles(*fd_index);
	*rc = (int) fnxwritt(FTNFILDES,*nb,buffer,icodt,ocodt);
}

/* ============================================================ */
/*			====>     NXREADT      <====		*/
/* ============================================================ */
/* Format & Type conversions                                    */

void
#ifdef CRAY_PROTO
NXREADT(register int* fd_index,register int* nb,void* buffer,
		_fcd icod,_fcd ocod,register int* rc)
{
	const char* const icodt = _fcdtocp(icod);
	const char* const ocodt = _fcdtocp(ocod);
#else
FUNCDECL(nxreadt)(register int* fd_index,register int* nb,
		void* buffer,register char* icod,register char* ocod,
		register int* rc,int li,int lo)
{
	const char* const icodt = icod;
	const char* const ocodt = ocod;
#endif

	initstdfiles(*fd_index);
	*rc = (int) fnxreadt(FTNFILDES,*nb,buffer,icodt,ocodt);
}

/* ============================================================ */
/*			====>     NXDIALO     <====   		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXDIALO(int* fd_index,long int* question ,int* lg,long int* reply,
	int* lg_max,int* lgrep,int* rc)
#else
FUNCDECL(nxdialo)(int* fd_index,char* question,int* lg,char* reply,
	int* lg_max,int* lgrep,int* rc)
#endif
{
	*rc = (int) fnxdialo(FTNFILDES,(char *)question,*lg,(char *)reply,
			*lg_max,lgrep);
}

/* ============================================================ */
/*			====>     NXFLUSH      <====		*/
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXFLUSH(int* fd_index)
#else
FUNCDECL(nxflush)(int* fd_index)
#endif
{	
	initstdfiles(*fd_index);
	fnxflush(FTNFILDES);
}

/* ============================================================ */
/*			====>     NXSEEK       <====		*/ 
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXSEEK(register int* fd_index,register int* offset,
		register int* from,register int* rc)
#else
FUNCDECL(nxseek)(register int* fd_index,register int* offset,
		register int* from,register int* rc)
#endif
{
	*rc = (int) fnxseek(FTNFILDES,(long) *offset,(int) *from);
}

/* ============================================================ */
/*			====>     NXTELL       <====		*/ 
/* ============================================================ */

void
#ifdef CRAY_PROTO
NXTELL(register int* fd_index,register int* offset,
		register int* rc)
#else
FUNCDECL(nxtell)(register int* fd_index,register int* offset,
		register int* rc)
#endif
{

	if ((*offset = (int) fnxtell(FTNFILDES)) < 0)
	    *rc = -1;
	else
	    *rc = 0;

}

/* ============================================================ */
/*			====>   NXCHFRM       <====		*/ 
/* ============================================================ */
/* Dynamic update of the file format				*/

void
#ifdef CRAY_PROTO
NXCHFRM(register int* fd_index,_fcd codfmt,register int* cr)
{
	const char* const codfmtt = _fcdtocp(codfmt);
	const int lcodfmtt = _fcdlen(codfmt);
#else
FUNCDECL(nxchfrm)(register int* fd_index,const char* codfmt,
		register int* cr,int lcod)
{
        const char* const codfmtt = codfmt;
	const int lcodfmtt = lcod;
#endif

	char c_codfmt[NX_LFORMAT];

	STR_FOR2C(codfmtt,lcodfmtt,c_codfmt,NX_LFORMAT);

	*cr = (int) fnxchfrm(nx_desc_table[*fd_index].dp.fd,c_codfmt);
}

/* ------------------------------------------------------------ */
/*                      NXPERROR                                */
/* ------------------------------------------------------------ */

void
#ifdef CRAY_PROTO
NXPERROR(_fcd fcname)
{
	const char* const fcnam = _fcdtocp(fcname);
	const int lfcnam = _fcdlen(fcname);
#else
FUNCDECL(nxperror)(char* fcname,int lf)
{
        const char* const fcnam = fcname;
	const int lfcnam = lf;
#endif
	char* cnam_temp = malloc(lfcnam+1);

	if(cnam_temp)
	{
	    STR_FOR2C(fcnam,lfcnam,cnam_temp,lfcnam+1);
	    fnxperror(cnam_temp);
	    free(cnam_temp);
	}

}

/* ------------------------------------------------------------------   */
/*                  =====>    NOBLKQ    <====                           */
/* ------------------------------------------------------------------   */

static int
noblkq(const char* str,int len)
{
	int cnt = len, lg = 0;
  
	while(cnt-- && *str != ' ' && *str != '\0')
	{
	    str++; 
	    lg++;
	}

	return lg;
}

/* ============================================================ */
/*       ====>     NXCONV_REAL_CR64_TO_IE64     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void
NXCONV_REAL_CR64_TO_IE64(const void *s, void *d, int* nb)
{
       nxconv_real_cr64_to_ie64(s, d, (int) *nb);
}
#elif defined(SUN_PROTO)
void
FUNCDECL(nxconv_real_cr64_to_ie64)(const void *s, void *d, int* nb)
{
       nxconv_real_cr64_to_ie64(s, d, (int) *nb);
}
#endif

/* ============================================================ */
/*		====>   verif type entier       <====		*/ 
/* ============================================================ */

static char nx_inttype_verif_C(void *a1, char *a2)
{
 long ia1, ia2, diff;
 
 ia1 = (long)a1;
 ia2 = (long)a2;

#if defined(CRAY) && !defined(_CRAYMPP) 
 diff = (ia2 - ia1)*sizeof(int);
#else
 diff = ia2 - ia1;
#endif

 if (diff == (long)sizeof(int_type))
    {
     return ('y');
    }
 else
    {
     return ('n');
    }
}


/* ============================================================ */
/*		====>   verif type reel       <====		*/ 
/* ============================================================ */

static char nx_realtype_verif_C(void *a1, void *a2)
{
 long ia1, ia2, diff;
 
 ia1 = (long)a1;
 ia2 = (long)a2;

#if defined(CRAY) && !defined(_CRAYMPP) 
 diff = (ia2 - ia1)*sizeof(int);
#else
 diff = ia2 - ia1;
#endif

 if (diff == (long)sizeof(float_type))
    {
     return ('y');
    }
 else
    {
     return ('n');
    }
}

/* ============================================================ */
/*            ====>     NX_INTTYPE_VERIF     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NX_INTTYPE_VERIF(void *a1, void *a2, char *rc)
{
 *rc = nx_inttype_verif_C(a1, a2);
}
#else
void nx_inttype_verif_(void *a1, void *a2, char *rc)
{
 *rc = nx_inttype_verif_C(a1, a2);
}

void nx_inttype_verif(void *a1, void *a2, char *rc)
{
 *rc = nx_inttype_verif_C(a1, a2);
}
#endif

/* ============================================================ */
/*            ====>     NX_REALTYPE_VERIF     <====		*/
/* ============================================================ */

#ifdef CRAY_PROTO
void NX_REALTYPE_VERIF(void *a1, void *a2, char *rc)
{
 *rc = nx_realtype_verif_C(a1, a2);
}
#else
void nx_realtype_verif_(void *a1, void *a2, char *rc)
{
 *rc = nx_realtype_verif_C(a1, a2);
}

void nx_realtype_verif(void *a1, void *a2, char *rc)
{
 *rc = nx_realtype_verif_C(a1, a2);
}
#endif
