/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Pierre Brochard";
#endif

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxcode.h"
#include "nxerror.h"
#include "nxfonc.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"

/* sources pour machines ieee 32 bits little endian */

#if defined(IEEE) && defined(NX_LITTLEENDIAN) 
#include "nxconvP.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

/* sur machines IEEE 32 bits little endian :
   int est li32
   type est li32 ou li64
   float est li32 = NXFLT
   double est li64 = NXDBLE
   long double est li128 (non fait)
   */

/* littleendian - ieee */

static void nx_int_li32_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_li64_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_li32_from_ie32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_int_nblk_li64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_type_from_ie32_ie64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie32_ie64(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_float_li32_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_li64_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_li32_from_ie32(FILE* fd,char* buff,int nb,
		int* nblus,int* rc);
static void nx_float_nblk_li64_from_ie32(FILE* fd,char* buff,int nb,
		int* nblus,int* rc);

static void nx_int_li32_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_li64_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_li32_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_int_nblk_li64_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_type_to_ie32_ie64(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);

static void nx_float_li32_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_li64_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_li32_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_float_nblk_li64_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

/* littleendian - cray */

static void nx_int_li32_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_li64_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_li32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_int_nblk_li64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_float_li32_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_li64_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_li32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_float_nblk_li64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_li32_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_li64_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_li32_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_int_nblk_li64_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_type_to_cr64_cr128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_float_li32_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_li64_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_li32_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_float_nblk_li64_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

/* littleendian - mpp */

static void nx_int_li32_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_li64_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_li32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_int_nblk_li64_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_type_from_ie64_ie128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie64_ie128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_float_li32_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_li64_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_li32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_float_nblk_li64_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_li32_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_li64_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_li32_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_int_nblk_li64_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_type_to_ie64_ie128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);

static void nx_float_li32_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_li64_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_li32_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_float_nblk_li64_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

/* ------------------------------------------------------------- */
/*  init_access for little endian fonctions */
/*  on lit au format mpp (ieee 64 bits), cray 64 bits ou */
/*  ieee 32 bits et on converti au format little endian ieee 32 bits */
/*  sur machines little endian seulement */
/* ------------------------------------------------------------- */

const ACCESS*
nx_init_access_little_ie32(void)
{

	static ACCESS access =
	{ 
	    NULL,
	    nx_int_li32_from_ie32,
#ifdef NX_LONG_IS_64
	    nx_int_li64_from_ie32,
#else
	    nx_int_li32_from_ie32,
#endif
	    nx_int_li64_from_ie32,
	    nx_float_li32_from_ie32,
	    nx_float_li64_from_ie32,
	    nx_type_from_ie32_ie64,
	    nx_int_nblk_li32_from_ie32,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_from_ie32,
#else
	    nx_int_nblk_li32_from_ie32,
#endif
	    nx_int_nblk_li64_from_ie32,
	    nx_float_nblk_li32_from_ie32,
	    nx_float_nblk_li64_from_ie32,
	    nx_type_nblk_from_ie32_ie64,
	    NULL,
	    nx_int_li32_to_ie32,
#ifdef NX_LONG_IS_64
	    nx_int_li64_to_ie32,
#else
	    nx_int_li32_to_ie32,
#endif
	    nx_int_li64_to_ie32,
	    nx_float_li32_to_ie32,
	    nx_float_li64_to_ie32,
	    nx_type_to_ie32_ie64,
	    nx_int_nblk_li32_to_ie32,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_to_ie32,
#else
	    nx_int_nblk_li32_to_ie32,
#endif
	    nx_int_nblk_li64_to_ie32,
	    nx_float_nblk_li32_to_ie32,
	    nx_float_nblk_li64_to_ie32,
	    nx_type_nblk_to_ie32_ie64
	};

	return &access;
}

const ACCESS*
nx_init_access_little_cr64(void)
{

	static ACCESS access =
	{ 
	    NULL,
	    nx_int_li32_from_cr64,
#ifdef NX_LONG_IS_64
	    nx_int_li64_from_cr64,
#else
	    nx_int_li32_from_cr64,
#endif
	    nx_int_li64_from_cr64,
	    nx_float_li32_from_cr64,
	    nx_float_li64_from_cr64,
	    nx_type_from_cr64_cr128,
	    nx_int_nblk_li32_from_cr64,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_from_cr64,
#else
	    nx_int_nblk_li32_from_cr64,
#endif
	    nx_int_nblk_li64_from_cr64,
	    nx_float_nblk_li32_from_cr64,
	    nx_float_nblk_li64_from_cr64,
	    nx_type_nblk_from_cr64_cr128,
	    NULL,
	    nx_int_li32_to_cr64,
#ifdef NX_LONG_IS_64
	    nx_int_li64_to_cr64,
#else
	    nx_int_li32_to_cr64,
#endif
	    nx_int_li64_to_cr64,
	    nx_float_li32_to_cr64,
	    nx_float_li64_to_cr64,
	    nx_type_to_cr64_cr128,
	    nx_int_nblk_li32_to_cr64,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_to_cr64,
#else
	    nx_int_nblk_li32_to_cr64,
#endif
	    nx_int_nblk_li64_to_cr64,
	    nx_float_nblk_li32_to_cr64,
	    nx_float_nblk_li64_to_cr64,
	    nx_type_nblk_to_cr64_cr128
	};

	return &access;
}

const ACCESS*
nx_init_access_little_ie64(void)
{

	static ACCESS access =
	{
	    NULL,
	    nx_int_li32_from_ie64,
#ifdef NX_LONG_IS_64
	    nx_int_li64_from_ie64,
#else
	    nx_int_li32_from_ie64,
#endif
	    nx_int_li64_from_ie64,
	    nx_float_li32_from_ie64,
	    nx_float_li64_from_ie64,
	    nx_type_from_ie64_ie128,
	    nx_int_nblk_li32_from_ie64,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_from_ie64,
#else
	    nx_int_nblk_li32_from_ie64,
#endif
	    nx_int_nblk_li64_from_ie64,
	    nx_float_nblk_li32_from_ie64,
	    nx_float_nblk_li64_from_ie64,
	    nx_type_nblk_from_ie64_ie128,
	    NULL,
	    nx_int_li32_to_ie64,
#ifdef NX_LONG_IS_64
	    nx_int_li64_to_ie64,
#else
	    nx_int_li32_to_ie64,
#endif
	    nx_int_li64_to_ie64,
	    nx_float_li32_to_ie64,
	    nx_float_li64_to_ie64,
	    nx_type_to_ie64_ie128,
	    nx_int_nblk_li32_to_ie64,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_to_ie64,
#else
	    nx_int_nblk_li32_to_ie64,
#endif
	    nx_int_nblk_li64_to_ie64,
	    nx_float_nblk_li32_to_ie64,
	    nx_float_nblk_li64_to_ie64,
	    nx_type_nblk_to_ie64_ie128
	};

	return &access;
}

/* ----------------------------------------------------------------------- */
/* 		         LITTLEENDIAN      IEEE                            */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI32_FROM_IE32  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_int_li32_from_ie32(FILE* fd,int nb,char* buff,int* rc)
{
	NX_READ(fd,buff,NX_SIZE_32,nb,"nx_int_li32_from_ie32",rc);
	nxconv_little_big32(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI64_FROM_IE32  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_int_li64_from_ie32(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_ie32_to_cr64(buff2,buff,nbt);
			nxconv_little_big64(buff,buff,nbt),
			"nx_int_li64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_FROM_IE32  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_32,nb,nblus,
			"nx_int_nblk_li32_from_ie32",rc);
	nxconv_little_big32(buff,buff,*nblus);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_FROM_IE32  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_ie32_to_cr64(buff2,buff,n);
			nxconv_little_big64(buff,buff,n),
			nblus,"nx_int_nblk_li64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI32_FROM_IE32  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li32_from_ie32(FILE* fd, int nb, char* buff, int* rc)
{
	NX_READ(fd,buff,NX_SIZE_32,nb,"nx_float_li32_from_ie32",rc);
	nxconv_little_big32(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI64_FROM_IE32  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li64_from_ie32(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_li64(buff2,buff,nbt),
			"nx_float_li64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>     NX_FLOAT_NBLK_LI32_FROM_IE32      <====          */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_32,nb,nblus,
			"nx_float_nblk_li32_from_ie32",rc);
	nxconv_little_big32(buff,buff,*nblus);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_NBLK_LI64_FROM_IE32  <====              */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_li64(buff2,buff,n),
			nblus,"nx_float_nblk_li64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_IE32_IE64   <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_ie32_ie64(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float little endian */
    /* conversion float ieee - double little endian */
    /* conversion double ieee - float little endian */
    /* conversion double ieee - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_READBUF(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					nb,nxconv_little_big32(buff,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_li64(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_ie64_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,nxconv_little_big64(buff,buff,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_IE32_IE64   <====               */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie32_ie64(FILE* fd, char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float little endian */
    /* conversion float ieee - double little endian */
    /* conversion double ieee - float little endian */
    /* conversion double ieee - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_READBUF_NBLK(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					nb,nxconv_little_big32(buff,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_li64(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_ie64_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,nxconv_little_big64(buff,buff,n),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI32_TO_IE32    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li32_to_ie32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			"nx_int_li32_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI64_TO_IE32    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li64_to_ie32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_li64_to_ie32"),
			"nx_int_li64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_TO_IE32  <=====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_to_ie32(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_li32_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_NBLK_LI64_TO_IE32    <====              */
/* ----------------------------------------------------------------------- */

static void 
nx_int_nblk_li64_to_ie32(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_nblk_ie64_to_li32"),
			nbecrits,"nx_int_nblk_li64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI32_TO_IE32    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li32_to_ie32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			"nx_float_li32_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI64_TO_IE32    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li64_to_ie32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_li64_to_ie32"),
			"nx_float_li64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_TO_IE32  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li32_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_NBLK_LI64_TO_IE32    <====            */
/* ----------------------------------------------------------------------- */

static void 
nx_float_nblk_li64_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_nblk_li64_to_ie32"),
			nbecrits,"nx_float_nblk_li64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_IE32_IE64   <====                      */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_ie32_ie64(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float ieee  */
    /* conversion double little endian - float ieee */
    /* conversion float little endian - double ieee */
    /* conversion double little endian - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_little_big32(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_li64_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_li32_to_ie64(buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,
					nxconv_little_big64(buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_IE32_IE64   <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod, int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float ieee  */
    /* conversion double little endian - float ieee */
    /* conversion float little endian - double ieee */
    /* conversion double little endian - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_little_big32(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_32,
					nxconv_real_li64_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_li32_to_ie64(buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_little_big64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		         LITTLEENDIAN       CRAY                           */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_LI32_FROM_CR64 <====                       */
/* ----------------------------------------------------------------------- */

static void
nx_int_li32_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_li32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_int_li32_from_cr64"),
			"nx_int_li32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_LI64_FROM_CR64 <====                       */
/* ----------------------------------------------------------------------- */

static void
nx_int_li64_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	NX_READ(fd,buff,NX_SIZE_64,nb,"nx_int_li64_from_cr64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_FROM_CR64 <====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_li32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_int_nblk_li32_from_cr64"),
			nblus,"nx_int_nblk_li32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_FROM_CR64 <====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_64,nb,nblus,
			"nx_int_nblk_li64_from_cr64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_LI32_FROM_CR64  <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_float_li32_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_li32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_li32_from_cr64"),
			"nx_float_li32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_LI32_FROM_CR64  <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_float_li64_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_li64(buff2,buff,nbt),
			"nx_float_li64_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_FROM_CR64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_li32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_li32_from_cr64"),
			nblus,"nx_float_nblk_li32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_FROM_CR64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
			NX_BUF_SIZE_64,
			nxconv_real_cr64_to_li64(buff2,buff,n),
			nblus,"nx_float_nblk_li64_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_CR64_CR128   <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float little endian */
    /* conversion float cray - double little endian */
    /* conversion double cray - float little endian */
    /* conversion double cray - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_li64(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_li64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_CR64_CR128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float little endian */
    /* conversion float cray - double little endian */
    /* conversion double cray - float little endian */
    /* conversion double cray - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_li64(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_cr128_to_li64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_LI32_TO_CR64    <====                      */
/* ----------------------------------------------------------------------- */

static void
nx_int_li32_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt),
			"nx_int_li32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_LI64_TO_CR64    <====                      */
/* ----------------------------------------------------------------------- */

static void
nx_int_li64_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			"nx_int_li64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_TO_CR64    <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_li32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_TO_CR64    <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_li64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_LI32_TO_CR64    <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_float_li32_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_cr64(buff,buff2,nbt),
			"nx_float_li32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_LI64_TO_CR64    <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_float_li64_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			"nx_float_li64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_TO_CR64    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_TO_CR64    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_CR64_CR128    <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_cr64_cr128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float cray  */
    /* conversion double little endian - float cray */
    /* conversion float little endian - double cray */
    /* conversion double little endian - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_li64_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_li32_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_li64_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_CR64_CR128    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float cray  */
    /* conversion double little endian - float cray */
    /* conversion float little endian - double cray */
    /* conversion double little endian - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_real_li64_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_li32_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_li64_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		         LITTLEENDIAN       MPP                            */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI32_FROM_IE64  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_int_li32_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_li32(buff2,buff,nbt,&cr);
			if (cr) fnxperror("nx_int_li32_from_ie64"),
			"nx_int_li32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI64_FROM_IE64  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_int_li64_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	NX_READ(fd,buff,NX_SIZE_64,nb,"nx_int_li64_from_ie64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_FROM_IE64  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_li32(buff2,buff,n,&cr);
			if (cr) fnxperror("nx_int_nblk_li32_from_ie64"),
			nblus,"nx_int_nblk_li32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_FROM_IE64  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_64,nb,nblus,
			"nx_int_nblk_li64_from_ie64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI32_FROM_IE64  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li32_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_li32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_li32_from_ie64"),
			"nx_float_li32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI64_FROM_IE64  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li64_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	NX_READ(fd,buff,NX_SIZE_64,nb,"nx_float_li64_from_ie64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_FROM_IE64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_li32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_li32_from_ie64"),
			nblus,"nx_float_nblk_li32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_FROM_IE64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_64,nb,nblus,
			"nx_int_nblk_li64_from_ie64",rc);
	nxconv_little_big64(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_IE64_IE128   <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_ie64_ie128(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float little endian */
    /* conversion float mpp - double little endian */
    /* conversion double mpp - float little endian */
    /* conversion double mpp - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
		        NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,nxconv_little_big64(buff,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_li64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_IE64_IE128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie64_ie128(FILE* fd, char* buff,int nb, const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float little endian */
    /* conversion float mpp - double little endian */
    /* conversion double mpp - float little endian */
    /* conversion double mpp - double little endian */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
		        NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,nxconv_little_big64(buff,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_ie128_to_li64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI32_TO_IE64    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li32_to_ie64(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt),
			"nx_int_li32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI64_TO_IE64    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li64_to_ie64(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			"nx_int_li64_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_TO_IE64  <=====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_li32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_TO_IE64  <=====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_li64_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI32_TO_IE64    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li32_to_ie64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_ie64(buff,buff2,nbt),
			"nx_float_li32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI64_TO_IE64    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li64_to_ie64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			"nx_float_li64_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_TO_IE64  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_ie64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_TO_IE64  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_little_big64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li64_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_IE64_IE128   <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_ie64_ie128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float mpp */
    /* conversion double little endian - float mpp */
    /* conversion float little endian - double mpp */
    /* conversion double little endian - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_ie64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_little_big64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_li32_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_li64_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_IE64_IE128   <====                */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float little endian - float mpp */
    /* conversion double little endian - float mpp */
    /* conversion float little endian - double mpp */
    /* conversion double little endian - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_ie64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_little_big64(buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_li32_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_li64_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif /* fin IEEE et LITTLEENDIAN */
