/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992    MA/AIM";
#endif

/* cree le 6/11/96 par Pierre Brochard pour diminuer la taille de nx.c */
/* Ajouts de fonctions 1998 Pierre Brochard */

/***************************************************************************/

#include "nxP.h"
#include "nxrw.h"
#include "nxcode.h"
#include "nxerror.h"
#include "nxfonc.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

static FILE* fnxfileinsoc(FILE* fd);
static FILE* fnxfileoutsoc(FILE* fd);

/* ============================================================ */
/*			====>    FNXUNBUF     <====		*/
/* ============================================================ */
/* on supprime la bufferisation des FILE* (a appeler avant tout */
/* appel a fread ou fwrite)                                     */
/* ============================================================ */

void
fnxunbuf(FILE* fd)
{
	int fd_index = fileno(fd);
	FILE* fd_lect;
	FILE* fd_writ;

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	  case NXP_RPC    :

	    setbuf(fd,NULL);

	    return;

	  case NXP_SOCKET :

	    if((fd_lect = fnxfileinsoc(fd))) setbuf(fd_lect,NULL);

	    if((fd_writ = fnxfileoutsoc(fd))) setbuf(fd_writ,NULL);

	    return;
	}

}

/* ============================================================ */
/*			====>    FNXFILEINSOC     <===          */
/* ============================================================ */

static FILE*
fnxfileinsoc(FILE* fd)
{
	int fd_index = fileno(fd);
	FILE* fd_lect = nx_fdin_table[fd_index];

	if(!fd_lect)
	{
	    int fdi2 = dup(fd_index);

	    if(!(fd_lect = fdopen(fdi2,"r")))
	    {
		NXERRNO = 34;
		return NULL;
	    }

	    nx_fdin_table[fd_index] = fd_lect;
	}

	return fd_lect;
}

/* ============================================================ */
/*			====>     FNX_READ_DESC   <===          */
/* ============================================================ */

FILE*
fnx_read_desc(FILE* fd)
{
	int fd_index;

	if(!fd) return NULL;

	fd_index = fileno(fd);
   
	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_SOCKET :

	    return fnxfileinsoc(fd);

	  default :

	    return fd;
	}

}

/* ============================================================ */
/*			====>    FNXGETFILEOUTSOC     <===      */
/* ============================================================ */

static FILE*
fnxfileoutsoc(FILE* fd)
{
	int fd_index = fileno(fd);
	FILE* fd_writ = nx_fdout_table[fd_index];
   
	if(!fd_writ)
	{
	    int fdi2 = dup(fd_index);

	    if(!(fd_writ = fdopen(fdi2,"w")))
	    {
		NXERRNO = 34;
		return NULL;
	    }

	    nx_fdout_table[fd_index] = fd_writ;
	}

	return fd_writ;
} 

/* ============================================================ */
/*			====>   FNX_WRITE_DESC     <===         */
/* ============================================================ */

FILE*
fnx_write_desc(FILE* fd)
{
	int fd_index;

	if(!fd) return NULL;

	fd_index = fileno(fd);
   
	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_SOCKET :

	    return fnxfileoutsoc(fd);

	  default :

	    return fd;
	}

}

/* ============================================================ */
/*			====>   FNX_DATA_IN_BUFFER   <===       */
/* ============================================================ */

int
fnx_data_in_buffer(FILE* fd)
{
    /* test pour savoir si le buffer en lecture contient des donnees a lire
       renvoit 1 si oui et 0 si non et -1 si erreur (fd NULL). */
	FILE* const filein = fnx_read_desc(fd);

	if(!filein) return -1;

#if defined(__GNUC__) && defined(_STDIO_USES_IOSTREAM)
    /* Cas GNU. */
	return filein->_IO_read_end > filein->_IO_read_ptr ? 1 : 0;
#elif defined(SUNOS) && defined(_STDIO_EXT_H)
    /* L interface 64 bits __fpending() est curieuse. Le vrai nombre
       d octets dans le buffer est filein->__pad[3] */
	return filein->__pad[3] > 0 ? 1 : 0;
#elif defined(WIN32)
	return filein->_endr > filein->_next ? 1 : 0;
#else
    /* Cas "classique". */
	return filein->_cnt > 0 ? 1 : 0;
#endif
}

/* -----------------------------------------------------*/
/*		====>    ISITOPEN    <====		*/
/* -----------------------------------------------------*/

static int
isitopen(int fd_index)
{

	switch (fd_index)
	{
 	/*  you can use stdin, stderr, sdout without any nxopen() */

	  case 0 :

	    if(!nx_desc_table[0].dp.fd)
	    {
		nx_desc_table[0].dp.fd = stdin;
		nx_desc_table[0].ds = NXP_FILE;
	    }

	    break;

	  case 1 :

	    if(!nx_desc_table[1].dp.fd)
	    {
		nx_desc_table[1].dp.fd = stdout;
		nx_desc_table[1].ds = NXP_FILE;
	    }

	    break;

	  case 2 :

	    if(!nx_desc_table[2].dp.fd)
	    {
		nx_desc_table[2].dp.fd = stderr;
		nx_desc_table[2].ds = NXP_FILE;
	    }

	    break ;

	  default:

	    if (fd_index >= NX_MAX_DESC || fd_index < 0)
	    {
		NXERRNO = 4;
		fnxperror("isitopen");
		return 0;
	    }

	    if ( nx_desc_table[fd_index].ds == 0)
	    {
		NXERRNO = 5;
		fnxperror("isitopen");
		return 0;
	    }

	}

	return 1;
}

/* ============================================================ */
/*			====>    FNXWRITE      <====		*/
/* ============================================================ */

int
fnxwrite(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index);
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    WRITE_CHAR(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_CHAR(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		rc = -1;
		arg.data = (char*) buffer;
		arg.lg = lg;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL,NULL,total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		lgecrit = lg;
		rc = 0;
	    }

	}

	if (rc == 0) 
	    return lgecrit;
	else       
	    return rc;

}

/* ============================================================ */
/*			====>    FNXNBKW_INT      <====		*/
/* ============================================================ */

int
fnxnbkw_int(FILE* fd,const char* buffer,const int lg)
{
	return fnxnbkwi(fd,buffer,lg);
}

/* ============================================================ */
/*			====>    FNXNBKWI      <====		*/
/* ============================================================ */

int
fnxnbkwi(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index) ;
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_INT_NBLK(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_INT_NBLK(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		rc = -1 ;
	    
		arg.data = (char*) buffer;
		arg.lg = lg;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		lgecrit = lg;
		rc = 0;
	    }

	}

	if (rc == 0) return lgecrit;
	else return rc;

}

/* ============================================================ */
/*			====>    FNXNBKW_LONG      <====	*/
/* ============================================================ */

int
fnxnbkw_long(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index) ;
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_LONG_NBLK(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_LONG_NBLK(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		rc = -1 ;
	    
		arg.data = (char*) buffer;
		arg.lg = lg;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		lgecrit = lg;
		rc = 0;
	    }

	}

	if (rc == 0) return lgecrit;
	else return rc;

}

/* ============================================================ */
/*			====>    FNXNBKW_LONGLONG      <====	*/
/* ============================================================ */

#if defined(NX_LONGLONG)
int
fnxnbkw_longlong(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index) ;
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_LONGLONG_NBLK(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_LONGLONG_NBLK(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		rc = -1;

		arg.data = (char*) buffer;
		arg.lg = lg;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		lgecrit = lg;
		rc = 0;
	    }

	}

	if (rc == 0) return lgecrit;
	else return rc;

}
#endif

/* ============================================================ */
/*			====>    FNXNBKWF      <====		*/
/* ============================================================ */

int
fnxnbkwf(FILE* fd,const char* buffer,const int lg)
{
    /* fnxnbkwf est pour le REAL FORTRAN */

#if defined(CRAY) && defined(_CRAYMPP)
	return fnxnbkw_double(fd,buffer,lg);
#else
	return fnxnbkw_float(fd,buffer,lg);
#endif
}

/* ============================================================ */
/*			====>    FNXNBKW_FLOAT      <====	*/
/* ============================================================ */

int
fnxnbkw_float(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index);
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_FLOAT_NBLK(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_FLOAT_NBLK(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer ;
		arg.lg = lg;
		total_timeout.tv_sec = 0 ;
		total_timeout.tv_usec = 0 ;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout) ;

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc") ;
		    exit(1) ;
		}

		lgecrit = lg;
		rc = 0 ;
	    }

	}

	if (rc == 0) return(lgecrit);
	else return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKW_DOUBLE      <====	*/
/* ============================================================ */

int
fnxnbkw_double(FILE* fd,const char* buffer,const int lg)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index);
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_DOUBLE_NBLK(fd,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_DOUBLE_NBLK(fd_writ,buffer,lg,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer ;
		arg.lg = lg;
		total_timeout.tv_sec = 0 ;
		total_timeout.tv_usec = 0 ;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout) ;

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc") ;
		    exit(1) ;
		}

		lgecrit = lg;
		rc = 0 ;
	    }

	}

	if (rc == 0) return(lgecrit);
	else return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKWT      <====		*/
/* ============================================================ */

int
fnxnbkwt(FILE* fd,const char* buffer,const int lg,const char* icod,
		const char* ocod)
{
	int fd_index = fileno(fd);
	int lgecrit = 0 ,rc = 0;
	FILE* fd_writ;

	if (!lg) return 0;

	ISITOPEN(fd_index);
   
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_TYPE_NBLK(fd,buffer,lg,icod,ocod,&lgecrit,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		return -1;
	    }

	    WRITE_TYPE_NBLK(fd_writ,buffer,lg,icod,ocod,&lgecrit,&rc);
	    break;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = lg;
		total_timeout.tv_sec = 0 ;
		total_timeout.tv_usec = 0 ;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout) ;

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 6;
		    clnt_perror(clnt, "rpc") ;
		    exit(1) ;
		}

		lgecrit = lg;
		rc = 0 ;
	    }

	}

	if (rc == 0) return(lgecrit);
	else return(rc);

}

/* ============================================================ */
/*		====>    FNXWRITE_INT      <====		*/
/* ============================================================ */

int
fnxwrite_int(FILE* fd,const int nb,const char* buffer)
{
	return fnxwriti(fd,nb,buffer);
}

/* ============================================================ */
/*			====>    FNXWRITI      <====		*/
/* ============================================================ */

int
fnxwriti(FILE* fd,const int nb,const char* buffer)
{
	const int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ = fd;

	if (!nb) return rc;

	ISITOPEN(fd_index);
 
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_INT(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 35;
		return -1;
	    }

	    WRITE_INT(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout) ;

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 7;
		    clnt_perror(clnt, "rpc");
		    exit(1) ;
		}

		rc = 0;
	    }

	}

	return (rc);
}

/* ============================================================ */
/*		====>    FNXWRITE_LONG      <====		*/
/* ============================================================ */

int
fnxwrite_long(FILE* fd,const int nb,const char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ = fd;

	if (!nb) return rc;

	ISITOPEN(fd_index);
 
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_LONG(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 35;
		return -1;
	    }

	    WRITE_LONG(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 7;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		rc = 0;
	    }

	}

	return (rc);
}

/* ============================================================ */
/*		====>    FNXWRITE_LONGLONG      <====		*/
/* ============================================================ */

#if defined(NX_LONGLONG)
int
fnxwrite_longlong(FILE* fd,const int nb,const char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ = fd;

	if (!nb) return rc;

	ISITOPEN(fd_index);
 
	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_LONGLONG(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 35;
		return -1;
	    }

	    WRITE_LONGLONG(fd_writ, nb, buffer, &rc);
	    break ;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 7;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		rc = 0;
	    }

	}

	return (rc);
}
#endif

/* ============================================================ */
/*			====>    FNXWRITF      <====		*/
/* ============================================================ */

int
fnxwritf(FILE* fd,const int nb,const char* buffer)
{
    /* fnxwritf est pour le REAL FORTRAN */

#if defined(CRAY) && defined(_CRAYMPP)
	return fnxwrite_double(fd,nb, buffer);
#else
	return fnxwrite_float(fd,nb,buffer);
#endif
}

/* ============================================================ */
/*		====>    FNXWRITE_FLOAT      <====		*/
/* ============================================================ */

int
fnxwrite_float(FILE* fd,const int nb,const char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ;

	if(!nb) return rc;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_FLOAT(fd,nb,buffer,&rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 36;
		return -1;
	    }

	    WRITE_FLOAT(fd_writ,nb,buffer,&rc);
	    break ;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout);

		if(clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 8;
		    clnt_perror(clnt, "rpc") ;
		    exit(1);
		}

		rc = 0;
	    }

	}

	return rc;
}

/* ============================================================ */
/*		====>    FNXWRITE_DOUBLE      <====		*/
/* ============================================================ */

int
fnxwrite_double(FILE* fd,const int nb,const char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ;

	if(!nb) return rc;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE   :
	  case NXP_FILE   :
	    WRITE_DOUBLE(fd,nb,buffer,&rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 36;
		return -1;
	    }

	    WRITE_DOUBLE(fd_writ,nb,buffer,&rc);
	    break ;
	  case NXP_RPC    :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout) ;

		if(clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 8;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		rc = 0;
	    }

	}

	return rc;
}

/* ============================================================ */
/*			====>     NXWRITT      <====		*/
/* ============================================================ */
/* Format & Type conversions                                    */

int
fnxwritt(FILE* fd,const int nb,const char* buffer,const char* icod,
		const char* ocod)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_writ = fd;

	if(!nb) return rc;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :

	    if((fd_index < 3) && (nx_ptio[fd_index] == NULL))
		nx_init_access(fd_index,0);

	    WRITE_TYPE(fd_writ,nb,buffer,icod,ocod,&rc);
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 37;
		return -1;
	    }

	    WRITE_TYPE(fd_writ,nb,buffer,icod,ocod,&rc);
	    break;
	  case NXP_RPC :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct NxTexte  arg;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		arg.data = (char*) buffer;
		arg.lg = nb;
		total_timeout.tv_sec = 0;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, RPCWRITE,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				NULL, NULL,
				total_timeout);

		if(clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 9;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		rc = 0;
	    }

	}

	return rc;
}

/* ============================================================ */
/*			====>    FNXDIALO     <====   		*/
/* ============================================================ */

int
fnxdialo(FILE* fd,char* question,const int lg,char* reply,const int lgmax,
		int* lgrep)
{
	size_t lg_ecris ;
	int rc = 0;
	int fd_index = fileno(fd);
	int ret = 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    break ;
	  case NXP_SOCKET :
	    lg_ecris = fwrite(question,1,lg,fd) ;

	    if(lg_ecris != (size_t) lg)
	    {
		rc = errno;
		nxpioerror("nxdialo");
		*lgrep = 0;
		break;
	    } 
 
	    U8FFLUSH(fd,ret);
	    *lgrep = fnxread(fd,reply,lgmax);
	    break;
	  case NXP_RPC :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct timeval total_timeout;
		struct NxTexte  arg, arg_rep;
		enum clnt_stat clnt_stat;

		total_timeout.tv_sec = STD_TO;
		total_timeout.tv_usec = 0;
		arg.lg = lg;
		arg.data = question;
		arg_rep.lg = lgmax;
		arg_rep.data = reply;
		clnt_stat = clnt_call(clnt, RPCDIALOG,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg,
				(xdrproc_t) xdr_nxtexte,(caddr_t) &arg_rep,
				total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    NXERRNO = 10;
		    rc = -1;
		    clnt_perror(clnt, "rpc");
		    exit(1);
		}

		*lgrep = arg_rep.lg;
		break;
	    }
	  default :
	    rc = -1;
	    break ;
	}

	return rc;
}

/* ============================================================ */
/*			====>    FNXREAD      <====		*/
/* ============================================================ */

int
fnxread(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_CHAR(fd_lect,buffer,lg,&lglus,&rc);

	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_CHAR(fd_lect,buffer,lg,&lglus,&rc);

	    break ;

	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKR_INT      <====		*/
/* ============================================================ */

int
fnxnbkr_int(FILE* fd,char* buffer,const int lg)
{
	return fnxnbkri(fd,buffer,lg);
}

/* ============================================================ */
/*			====>    FNXNBKRI      <====		*/
/* ============================================================ */

int
fnxnbkri(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_INT_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_INT_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;
	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);

	return(rc);
}

/* ============================================================ */
/*			====>    FNXNBKR_LONG      <====	*/
/* ============================================================ */

int
fnxnbkr_long(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_LONG_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_LONG_NBLK(fd_lect,buffer,lg,&lglus,&rc);

	    break ;

	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKR_LONGLONG      <====	*/
/* ============================================================ */

#if defined(NX_LONGLONG)
int
fnxnbkr_longlong(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_LONGLONG_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_LONGLONG_NBLK(fd_lect,buffer,lg,&lglus,&rc);

	    break ;

	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}
#endif

/* ============================================================ */
/*			====>    FNXNBKRF      <====		*/
/* ============================================================ */

int
fnxnbkrf(FILE* fd,char* buffer,const int lg)
{
    /* fnxwritf est pour le REAL FORTRAN */

#if defined(CRAY) && defined(_CRAYMPP)
	return fnxnbkr_double(fd,buffer,lg);
#else
	return fnxnbkr_float(fd,buffer,lg);
#endif
}

/* ============================================================ */
/*			====>    FNXNBKR_FLOAT      <====    	*/
/* ============================================================ */

int
fnxnbkr_float(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_FLOAT_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_FLOAT_NBLK(fd_lect,buffer,lg,&lglus,&rc);

	    break ;

	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKR_DOUBLE      <====    	*/
/* ============================================================ */

int
fnxnbkr_double(FILE* fd,char* buffer,const int lg)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_DOUBLE_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_DOUBLE_NBLK(fd_lect,buffer,lg,&lglus,&rc);
	    break ;
	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}

/* ============================================================ */
/*			====>    FNXNBKRT      <====		*/
/* ============================================================ */

int
fnxnbkrt(FILE* fd,char* buffer,const int lg,const char* icod,const char* ocod)
{
	int rc = 0, lglus = 0;
	int fd_index = fileno(fd);
	FILE* fd_lect;

	if(!lg) return 0;

	errno = 0;
	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd ;
	    READ_TYPE_NBLK(fd_lect,buffer,lg,icod,ocod,&lglus,&rc);
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 11;
		return -1;
	    }

	    READ_TYPE_NBLK(fd_lect,buffer,lg,icod,ocod,&lglus,&rc);

	    break ;

	  default :
	    return rc;
	}

	if (rc == 0)
	    return(lglus);
	else
	    return(rc);

}

/* ============================================================ */
/*			====>    FNXREAD_INT     <====		*/
/* ============================================================ */

int
fnxread_int(FILE* fd,const int nb,char* buffer)
{
	return fnxreadi(fd,nb,buffer);
}

/* ============================================================ */
/*			====>    FNXREADI     <====		*/
/* ============================================================ */

int
fnxreadi(FILE* fd,const int nb,char* buffer)
{
	const int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd;  
	    READ_INT(fd_lect,nb,buffer,&rc);                         
	    break ;
	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 12;
		return -1;
	    }

	    READ_INT(fd_lect,nb,buffer,&rc);
	    break ;
	  default :
	    return rc ;
	}

	return (rc);
}
/* ============================================================ */
/*			====>    FNXREAD_LONG     <====		*/
/* ============================================================ */

int
fnxread_long(FILE* fd,const int nb,char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd;  
	    READ_LONG(fd_lect,nb,buffer,&rc);                         
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 12;
		return -1;
	    }

	    READ_LONG(fd_lect,nb,buffer,&rc);
	    break ;

	  default :
	    return rc ;
	}

	return (rc);
}

/* ============================================================ */
/*		====>    FNXREAD_LONGLONG     <====		*/
/* ============================================================ */

#if defined(NX_LONGLONG)
int
fnxread_longlong(FILE* fd,const int nb,char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd;  
	    READ_LONGLONG(fd_lect,nb,buffer,&rc);                         
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 12;
		return -1;
	    }

	    READ_LONGLONG(fd_lect,nb,buffer,&rc);
	    break ;

	  default :
	    return rc ;
	}

	return (rc);
}
#endif

/* ============================================================ */
/*			====>     FNXREADF     <====		*/
/* ============================================================ */

int
fnxreadf(FILE* fd,const int nb,char* buffer)
{
    /* fnxwritf est pour le REAL FORTRAN */

#if defined(CRAY) && defined(_CRAYMPP)
	return fnxread_double(fd, nb, buffer);
#else
	return fnxread_float(fd, nb, buffer);
#endif
}

/* ============================================================ */
/*		====>     FNXREAD_FLOAT     <====		*/
/* ============================================================ */

int
fnxread_float(FILE* fd,const int nb,char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd; 
	    READ_FLOAT(fd_lect,nb,buffer,&rc);                        
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 13;
		return -1;
	    }

	    READ_FLOAT(fd_lect,nb,buffer,&rc);
	    break ;

	  default :
	    return (-1);
	}

	return (rc);
}

/* ============================================================ */
/*			====>   FNXREAD_DOUBLE   <====	        */
/* ============================================================ */

int
fnxread_double(FILE* fd,const int nb,char* buffer)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    fd_lect = fd; 
	    READ_DOUBLE(fd_lect,nb,buffer,&rc);                        
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 13;
		return -1;
	    }

	    READ_DOUBLE(fd_lect,nb,buffer,&rc);
	    break ;

	  default :
	    return (-1);
	}

	return (rc);
}

/* ============================================================ */
/*			====>     FNXREADT      <====		*/
/* ============================================================ */

int
fnxreadt(FILE* fd,const int nb,char* buffer,const char* icod,const char* ocod)
{
	int fd_index = fileno(fd);
	int rc = 0;
	FILE* fd_lect;

	if(!nb) return 0;

	ISITOPEN(fd_index);

	switch(nx_desc_table[fd_index].ds)
	{
	  case NXP_PIPE :
	  case NXP_FILE :

	    if ((fd_index < 3) && (nx_ptio[fd_index] == NULL))
		nx_init_access(fd_index,0);

	    fd_lect = fd;
	    READ_TYPE(fd_lect,nb,buffer,icod,ocod,&rc);                         
	    break ;

	  case NXP_SOCKET :

	    if(!(fd_lect = fnxfileinsoc(fd)))
	    {
		NXERRNO = 14;
		return -1;
	    }

	    READ_TYPE(fd_lect,nb,buffer,icod,ocod,&rc);                         

	    break ;

	  default :
	    return  (-1);
	}

	return (rc);
}

/* ============================================================ */
/*			====>    FNXFLUSH      <====		*/
/* ============================================================ */

int
fnxflush(FILE* fd)
{
	int fd_index = fileno(fd);
	FILE* fd_writ;
	int ret = 0;

	switch ( nx_desc_table[fd_index].ds )
	{
	  case NXP_PIPE :
	  case NXP_FILE :
	    U8FFLUSH(fd,ret);                          
	    break;
	  case NXP_SOCKET :

	    if(!(fd_writ = fnxfileoutsoc(fd)))
	    {
		NXERRNO = 36;
		return -1;
	    }

	    U8FFLUSH(fd_writ,ret);
	    break;
	  case NXP_RPC :
	    {
		CLIENT* clnt = nx_desc_table[fd_index].dp.clnt;
		struct timeval total_timeout;
		enum clnt_stat clnt_stat;

		total_timeout.tv_sec = STD_TO;
		total_timeout.tv_usec = 0;
		clnt_stat = clnt_call(clnt, NULLPROC,
				(xdrproc_t) xdr_void, NULL,
				(xdrproc_t) xdr_void , 0, total_timeout);

		if (clnt_stat != RPC_SUCCESS)	
		{
		    clnt_perror(clnt, "nxflush");
		    exit(1);
		}

	    }

	}

	return ret;
}

/* ============================================================ */
/*			====>    FNXSEEK       <====		*/ 
/* ============================================================ */

int
fnxseek(FILE* fd,const long offset,const int from)
{
	const int nf = fileno(fd);
	int rc = 0;

	switch (nx_desc_table[nf].ds)
	{
	  case NXP_FILE: 
	    {
		rc = fseek(fd,offset,from);

		if (rc)
		{
		    rc = -1;
		    NXERRNO = 32;
		}

		break;
	    }

	  default :
	    rc = -1;
	    NXERRNO = 31;
	    break;
	}

	return(rc);
}

/* ============================================================ */
/*			====>    FNXTELL       <====		*/ 
/* ============================================================ */

int
fnxtell(FILE* fd)
{
	const int nf = fileno(fd);

	switch (nx_desc_table[nf].ds)
	{
	  case NXP_FILE: 
	    {
		return (int) ftell(fd);
	    }

	  default :
	    {
		NXERRNO = 33;
		return (-1);
	    }

	}

}

/* ============================================================ */
/*		====>    FNXWRITE_INT_TYPE       <====		*/ 
/* ============================================================ */

int
fnxwrite_int_type(FILE* fd,const int nb,const int type_size,
		const char* buffer)
{

	if (type_size == sizeof(int))
	{
	    return fnxwrite_int(fd, nb, buffer);
	}   
	else if (type_size == sizeof(long))
	{
	    return fnxwrite_long(fd, nb, buffer);
	}
#if defined(NX_LONGLONG)
	else if (type_size == sizeof(long long))
	{
	    return fnxwrite_longlong(fd, nb, buffer);
	}
#endif
	else
	{
	    NXERRNO = 80;
	    return (-1);
	}

}

/* ============================================================ */
/*		====>    FNXREAD_INT_TYPE       <====		*/ 
/* ============================================================ */

int
fnxread_int_type(FILE* fd,const int nb,const int type_size,char* buffer)
{

	if (type_size == sizeof(int))
	{
	    return fnxread_int(fd, nb, buffer);
	}   
	else if (type_size == sizeof(long))
	{
	    return fnxread_long(fd, nb, buffer);
	}
#if defined(NX_LONGLONG)
	else if (type_size == sizeof(long long))
	{
	    return fnxread_longlong(fd, nb, buffer);
	}
#endif
	else
	{
	    NXERRNO = 80;
	    return (-1);
	}

}


/* ============================================================ */
/*	====>    FNXWRITE_FLOAT_TYPE       <====		*/ 
/* ============================================================ */

int
fnxwrite_float_type(FILE* fd,const int nb,const int type_size,
		const char* buffer)
{

	if (type_size == sizeof(float))
	{
	    return fnxwrite_float(fd,nb,buffer);
	}   
	else if (type_size == sizeof(double))
	{
	    return fnxwrite_double(fd,nb,buffer);
	}
	else
	{
	    NXERRNO = 80;
	    return (-1);
	}  
 
}

/* ============================================================ */
/*		====>    FNXREAD_FLOAT_TYPE       <====		*/ 
/* ============================================================ */

int
fnxread_float_type(FILE* fd,const int nb,const int type_size,char* buffer)
{

	if (type_size == sizeof(float))
	{
	    return fnxread_float(fd,nb,buffer);
	}   
	else if (type_size == sizeof(double))
	{
	    return fnxread_double(fd,nb,buffer);
	}
	else
	{
	    NXERRNO = 80;
	    return (-1);
	}  

}
