/*************************************************************************/
/*  Identification du source */
/*************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Pierre Brochard";
#endif

/*************************************************************************/
/* Pour LINUX */
/*************************************************************************/

#include "U8machcomp.h"

#if defined(LINUX)

/* Pour avoir une instance de stat et fstat non inline pour certaines
   librairies precompilees comme motif sous Linux ou normalement stat et fstat
   sont des fonctions static inline comprises par le compilateur gcc */

#   define static
#   include <sys/stat.h>
#   undef static

#else /* ! LINUX */
    void
    U8StatBidon()
    {
    }
#endif /* LINUX */

/*************************************************************************/
/* Routines pour recuperer le nom et le home de l'utilisateur conformes
   POSIX.1c */
/*************************************************************************/

#include "nx.h"
#include "nxsystem.h"

#if defined(_SYSTYPE_SVR3)
#   include <bsd/sys/types.h>
#else
#   include <sys/types.h>
#endif

#include <sys/socket.h>

#include <stddef.h>
#include <malloc.h>
#include <pwd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/param.h>
#include <netdb.h>

#ifndef MIN
#   define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#include "nxoptim.h"
#include "nx_free.h"

DLL_LOCAL THREAD_LOCAL char* cacherhosts = NULL;
DLL_LOCAL THREAD_LOCAL char* nxsys_err = NULL;
DLL_LOCAL THREAD_LOCAL char* ppbuf = NULL;
DLL_LOCAL THREAD_LOCAL char* hibuf = NULL;

void
nx_free()
{

	if(cacherhosts)
	{
	    free(cacherhosts);cacherhosts = NULL;
	}

	if(nxsys_err)
	{
	    free(nxsys_err);nxsys_err = NULL;
	}

	if(ppbuf)
	{
	    free(ppbuf);ppbuf = NULL;
	}

	if(hibuf)
	{
	    free(hibuf);hibuf = NULL;
	}

}

/* ============================================================ */
/*			====>   U8GetUser      <====		*/
/*      Recheche du nom de l'utilisateur                        */
/*      Alternative a cuserid                                   */
/*  Si s est fourni de taille L_U8GetUser, on met le nom        */
/*  dedans et on renvoit un pointeur dessus,                    */
/*  sinon, on utilise un espace static et on renvoit le         */
/*  pointeur dessus.                                            */
/*  Si retour NULL ou si chaine nulle dans s si s non NULL,     */
/*  alors erreur.                                               */
/* ============================================================ */

/* Modif PB 19/09/2008 : suppression de getlogin et ajout cache. */

struct passwd*
U8GetPasswd(void)
{
	static THREAD_LOCAL struct passwd mypp;
	struct passwd* pp = NULL;

	if(!ppbuf)
	{
#ifdef WIN32
	    const size_t lppbuf = 2048;
#else
	    const size_t lppbuf = sysconf(_SC_GETPW_R_SIZE_MAX);
#endif

	    ppbuf = (char*) malloc(lppbuf);

	    if(ppbuf)
		getpwuid_r(getuid(),&mypp,ppbuf,lppbuf,&pp);

	}
	else
	    pp = &mypp;

	return pp;
}

char*
U8GetUser(char* const s)
{
	struct passwd* const pp = U8GetPasswd();

	if(s)
	{

	    if(pp)
	    {
		const unsigned ll = strlen(pp->pw_name) + 1;
		const unsigned lll = MIN(L_U8GetUser,ll);

		memcpy(s,pp->pw_name,lll);
		s[lll-1] = '\0';
	    }
	    else
		s[0] = '\0';

	    return s;
	}
	else
	    if(pp)
		return pp->pw_name;
	    else
		return NULL;

}

/* ============================================================ */
/*			====>   U8GetHome      <====		*/
/*      Recheche du home de l'utilisateur                       */
/*  Si s est fourni de taille L_U8GetHome, on met le repertoire */
/*  home dedans et on renvoit un pointeur dessus,               */
/*  sinon, on alloue un espace avec malloc et on renvoit le     */
/*  pointeur sur la zone allouee.                               */
/*  Si retour NULL ou si chaine nulle dans s si s non NULL,     */
/*  alors erreur.                                               */
/* ============================================================ */

char*
U8GetHome(char s[/* L_U8GetHome */])
{
	struct passwd* const pp = U8GetPasswd();

	if(s)
	{

	    if(pp)
	    {
		const unsigned ll = strlen(pp->pw_dir) + 1;
		const unsigned lll = MIN(L_U8GetHome,ll);

		memcpy(s,pp->pw_dir,lll);
		s[lll-1] = '\0';
	    }
	    else
		s[0] = '\0';

	    return s;
	}
	else
	    if(pp)
		return pp->pw_dir;
	    else
		return NULL;

}

/* ============================================================ */
/*			====>   U8GetHostname     <====		*/
/*      Recheche du hostname avec cache                         */
/* ============================================================ */

int
U8GetHostname(char* const s,const size_t lg)
{
	static THREAD_LOCAL char h_name[MAXHOSTNAMELEN];
	static THREAD_LOCAL size_t lh_name = 0;

	if(!lh_name)
	{
	    const int ier = gethostname(h_name,sizeof(h_name));

	    if(ier == -1)
		return -1;

	    lh_name = strlen(h_name);

	    if(!lh_name)
		return -1;

	}

	if(s)
	{
	    const unsigned lll = MIN(MAXHOSTNAMELEN,lg);

	    memcpy(s,h_name,lll);
	    s[lll-1] = '\0';
	    return 0;
	}
	else
	    return -1;

}

/* ============================================================ */
/*			====>   U8GetHostbyname     <====    	*/
/*      Recheche des IPs de hostname avec cache                 */
/* ============================================================ */

struct hostent* 
U8GetHostbyname(const char* const hostname)
{
	static THREAD_LOCAL struct hostent myhi;

	if(!hibuf || strcmp(hostname,hibuf))
	{
	    const size_t nalign = sizeof(void*) - 1;
	    struct hostent* const host_info = gethostbyname(hostname);
	    size_t linfo = 0;
	    size_t laliases = 0;
	    size_t laddrs = 0;
	    char** p;
	    char** q;
	    size_t ind;

	    if(!host_info)
		return NULL;

	    if(host_info->h_addrtype != AF_INET)
		return NULL;

	    linfo += strlen(hostname) + 1;
	    linfo = (linfo + nalign) & ~nalign;

	    linfo += strlen(host_info->h_name) + 1;
	    linfo = (linfo + nalign) & ~nalign;

	    for (q = host_info->h_aliases;*q;q++)
	    {
		laliases += sizeof(char*);
		linfo += strlen(*q) + 1;
		linfo = (linfo + nalign) & ~nalign;
	    }

	    laliases += sizeof(char*);

	    for (p = host_info->h_addr_list;*p;p++)
	    {
		laddrs += sizeof(char*);
		linfo += host_info->h_length;
		linfo = (linfo + nalign) & ~nalign;
	    }

	    laddrs += sizeof(char*);
	    linfo += laliases + laddrs;

	    if(hibuf)
		free(hibuf);

	    hibuf = (char*) malloc(linfo);

	    if(!hibuf)
		return NULL;

	    myhi.h_addrtype = host_info->h_addrtype;
	    myhi.h_length = host_info->h_length;

	/* hostname. */
	    linfo = strlen(hostname) + 1;
	    memcpy(hibuf,hostname,linfo);
	    linfo = (linfo + nalign) & ~nalign;

	/* h_name. */
	    myhi.h_name = hibuf + linfo;

	    {
		const size_t lname = strlen(host_info->h_name) + 1;

		memcpy(myhi.h_name,host_info->h_name,lname);
		linfo += lname;
		linfo = (linfo + nalign) & ~nalign;
	    }

	/* h_aliases. */
	    myhi.h_aliases = (char**) (hibuf + linfo);
	    linfo += laliases;

	    for (ind = 0,q = host_info->h_aliases;*q;q++,ind++)
	    {
		const size_t lq = strlen(*q) + 1;

		myhi.h_aliases[ind] = hibuf + linfo;
		memcpy(myhi.h_aliases[ind],*q,lq);
		linfo += lq;
		linfo = (linfo + nalign) & ~nalign;
	    }

	    myhi.h_aliases[ind] = NULL;

	/* h_addr_list. */
	    myhi.h_addr_list = (char**) (hibuf + linfo);
	    linfo += laddrs;

	    for (ind = 0,p = host_info->h_addr_list;*p;p++,ind++)
	    {
		const size_t lp = host_info->h_length;

		myhi.h_addr_list[ind] = hibuf + linfo;
		memcpy(myhi.h_addr_list[ind],*p,lp);
		linfo += lp;
		linfo = (linfo + nalign) & ~nalign;
	    }

	    myhi.h_addr_list[ind] = NULL;
	}

	return &myhi;
}

/* ============================================================ */
/*			====>    CUSERID     <====		*/
/* Pour les machines qui ne l'ont pas                           */
/* ============================================================ */

#ifdef NX_NOCUSERID
char*
cuserid(char* s)
{
	return U8GetUser(s);
}
#endif
