// ***********************************************************
// *    partie X11 de l'afficheur X11                        *
// ***********************************************************

struct SccsIdU8getpos
{
	static const char* const sccs_id;
};

const char* const SccsIdU8getpos::sccs_id = "%W% %G% Pierre Brochard";

#include "U8constant.h"
#include "U8signal.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#include "L3ListeDesCodes.h"

#if XlibSpecificationRelease < 5
    typedef char* XPointer;
#endif

#include "poulpe/U8getpos.h"

// ******************************************
// * define divers                          *
// ******************************************

#define U8SIGN(a) (((a) > 0) ? 1 : -1)

// ******************************************
// * implementation classe U8Motion
// ******************************************

void
U8Motion::AddButtonPress(Widget widget)
{

	if(widget)
	{
	    XtAddEventHandler(widget,ButtonPressMask,False,
			    MyInputButtonPress,XtPointer(this));
	    _is_buttonpress = True;
	}

}

void
U8Motion::AddButtonRelease(Widget widget)
{

	if(widget)
	{
	    XtAddEventHandler(widget,ButtonReleaseMask,False,
			    MyInputButtonRelease,XtPointer(this));
	    _is_buttonrelease = True;
	}

}

void
U8Motion::AddPointerMotion(Widget widget)
{

	if(widget)
	{
	    XtAddEventHandler(widget,PointerMotionMask,False,
			    MyInputPointerMotion,XtPointer(this));
	    _is_pointermotion = True;
	}

}

void
U8Motion::RemoveButtonPress(Widget widget)
{

	if(widget)
	{
	    XtRemoveEventHandler(widget,ButtonPressMask,False,
			    MyInputButtonPress,XtPointer(this));
	    _is_buttonpress = False;
	}

}

void
U8Motion::RemoveButtonRelease(Widget widget)
{

	if(widget)
	{
	    XtRemoveEventHandler(widget,ButtonReleaseMask,False,
			    MyInputButtonRelease,XtPointer(this));
	    _is_buttonrelease = False;
	}

}

void
U8Motion::RemovePointerMotion(Widget widget)
{

	if(widget)
	{
	    XtRemoveEventHandler(widget,PointerMotionMask,False,
			    MyInputPointerMotion,XtPointer(this));
	    _is_pointermotion = False;
	}

}

// ********************************************************************
//    Callbacks (appeles avec XtAddEventHandler par API ci-dessus)
// ********************************************************************

void
U8Motion::MyInputButtonPress(Widget,XtPointer clientdata,
		XEvent* event,Boolean*)
{
	U8Motion* const uax = (U8Motion*) clientdata;

	uax->SelectButtonPress(XtPointer(event));
}

void
U8Motion::MyInputButtonRelease(Widget,XtPointer clientdata,
		XEvent* event,Boolean*)
{
	U8Motion* const uax = (U8Motion*) clientdata;

	uax->SelectButtonRelease(XtPointer(event));
}

void
U8Motion::MyInputPointerMotion(Widget,XtPointer clientdata,
		XEvent* event,Boolean*)
{
	U8Motion* const uax = (U8Motion*) clientdata;

	uax->SelectPointerMotion(XtPointer(event));
}

// ********************************************************************
//    fonctions membres Callbacks (appeles avec MyInput*)
// ********************************************************************

void
U8Motion::SelectButtonPress(XtPointer calldata)
{
	XEvent* const cdtevent = (XEvent*) calldata;

	_getpos.BoutonPress(cdtevent);
}

void
U8Motion::SelectButtonRelease(XtPointer calldata)
{
	XEvent* const cdtevent = (XEvent*) calldata;

	_getpos.BoutonRelease(cdtevent);
}

void
U8Motion::SelectPointerMotion(XtPointer calldata)
{
	XEvent* const cdtevent = (XEvent*) calldata;

	_getpos.PointeurMotion(cdtevent);
}

// ******************************************
// * implementation classe U8GetPos
// ******************************************

U8GetPos::U8GetPos(U8Motion* motion) :
		_l3Appcontext(NULL),
		_disp(NULL),
		_widget(NULL),
		_win(0),
		_curseur_gc(NULL),
		_curseur_dial(None),
		_pixel(0),
		_motion(motion),
		_x_cur_save(-1),
		_y_cur_save(-1),
		_x_cur_min_save(-1),
		_y_cur_min_save(-1),
		_x_cur_max_save(-1),
		_y_cur_max_save(-1),
		_picking_grandecroix(FALSE),
		_picking_elastique(SANSELASTIQUE),
		_picking_carre(FALSE),
		_getpos_possible(TRUE),
		_Alire(FALSE),
		_Disponible(FALSE),
		_x(0),
		_y(0)
{
}

U8GetPos::~U8GetPos()
{
	del_dial();
	DeleteGC();
}

void
U8GetPos::init(XtAppContext appc,Widget widget)
{

	if(_curseur_dial != None)
	{
	    XFreeCursor(_disp, _curseur_dial);
	    XDefineCursor(_disp, _win, None);
	}
	
	_l3Appcontext = appc;
	_widget = widget;
	_win = XtWindow(widget);
	_disp = XtDisplay(widget);
	_curseur_dial = XCreateFontCursor(_disp, XC_crosshair);
}

void
U8GetPos::del_dial()
{

	if(_curseur_dial != None)
	{
	    XFreeCursor(_disp, _curseur_dial);
	    XDefineCursor(_disp, _win, None);
	    _curseur_dial = None;
	}

}

// *****************************************
//         fonction Dialogue
// *****************************************

void
U8GetPos::Dialogue(int typecroix,int typeelastique)
{

    // Un rectangle elastique est cree par 2 appels de L3Dialogue
    // Le premier avec typeelastique = SANSELASTIQUE,
    // le 2eme avec typeelastique = RECTANGLEELASTIQUE

	if(!_getpos_possible)
	{
	    fprintf(stderr,"erreur : U8GetPos : L3Dialogue deja en cours\n");
	    return;
	}

	_getpos_possible = FALSE;

	int motion_handler = FALSE;
	int buttonpress_handler = FALSE;
	int buttonrelease_handler = FALSE;

	_Alire = TRUE;
	_Disponible = FALSE;

	if(typecroix == GRANDECROIX)
	{
	    _picking_grandecroix = TRUE;
	    XDefineCursor(_disp,_win, None);
	    _picking_elastique = typeelastique;
	    XSetForeground(_disp,_curseur_gc,_pixel);

	    if(_motion)
	    {

		if(!_motion->IsButtonPress())
		{
		    _motion->AddButtonPress(_widget);
		    buttonpress_handler = TRUE;
		}

		if(!_motion->IsButtonRelease())
		{
		    _motion->AddButtonRelease(_widget);
		    buttonrelease_handler = TRUE;
		}

		if(!_motion->IsPointerMotion())
		{
		    _motion->AddPointerMotion(_widget);
		    motion_handler = TRUE;
		}

	    }

	    switch(typeelastique)
	    {
	      case SANSELASTIQUE:
	    // affichage d'une grande croix initiale

		if(_x_cur_save != -1)
		{
		    XDrawLine(_disp, _win, _curseur_gc,0,_y_cur_save,
				    1200,_y_cur_save);
		    XDrawLine(_disp, _win, _curseur_gc,_x_cur_save,0,
				    _x_cur_save,1200);
		}

		{
		    Window root_return;
		    Window child_return;
		    int root_x_return;
		    int root_y_return;
		    int win_x_return;
		    int win_y_return;
		    unsigned int mask_return;

		    if(XQueryPointer(_disp,_win,&root_return,
				    &child_return,&root_x_return,
				    &root_y_return,&win_x_return,
				    &win_y_return,&mask_return))
		    {
			_x_cur_save = win_x_return;
			_y_cur_save = win_y_return;

			XDrawLine(_disp, _win, _curseur_gc, 0,_y_cur_save,
					1200, _y_cur_save);
			XDrawLine(_disp, _win, _curseur_gc, _x_cur_save,0,
					_x_cur_save, 1200);
		    }

		}
		break;
	      default:
		break;
	    }

	}
	else
	{
	    _picking_grandecroix = FALSE;
	    XDefineCursor(_disp, _win, _curseur_dial);
	}

	while(!_Disponible)
	{
	    XEvent event_return;
	    XtAppNextEvent(_l3Appcontext, &event_return);
	    XtDispatchEvent(&event_return);
	}

	if(_motion)
	{

	    if(buttonpress_handler) _motion->RemoveButtonPress(_widget);

	    if(buttonrelease_handler) _motion->RemoveButtonRelease(_widget);

	    if(motion_handler) _motion->RemovePointerMotion(_widget);

	    XDefineCursor(_disp, _win, _motion->Curseur());
	}

	if(typecroix == GRANDECROIX)
	{
	    _picking_grandecroix = FALSE;

	    switch(typeelastique)
	    {
	      case SANSELASTIQUE:
	    // effacement de la grande croix

		if(_x_cur_save != -1)
		{
		    XDrawLine(_disp, _win, _curseur_gc,
				    0,_y_cur_save,1200, _y_cur_save);
		    XDrawLine(_disp, _win, _curseur_gc,
				    _x_cur_save,0,_x_cur_save, 1200);
		}

		_x_cur_min_save = _x_cur_save;
		_y_cur_min_save = _y_cur_save;
		_x_cur_save = -1;
		_y_cur_save = -1;
		break;
	      case RECTANGLEELASTIQUE:
	    // effacement du rectangle elastique

		if(_x_cur_max_save != -1)
		{
		    XDrawLine(_disp, _win, _curseur_gc,
				    _x_cur_min_save,_y_cur_min_save,
				    _x_cur_max_save, _y_cur_min_save);
		    XDrawLine(_disp, _win, _curseur_gc,
				    _x_cur_max_save,_y_cur_min_save,
				    _x_cur_max_save, _y_cur_max_save);
		    XDrawLine(_disp, _win, _curseur_gc,
				    _x_cur_max_save,_y_cur_max_save,
				    _x_cur_min_save, _y_cur_max_save);
		    XDrawLine(_disp, _win, _curseur_gc,
				    _x_cur_min_save,_y_cur_max_save,
				    _x_cur_min_save, _y_cur_min_save);
		}

		_x_cur_min_save = -1;
		_y_cur_min_save = -1;
		_x_cur_max_save = -1;
		_y_cur_max_save = -1;
		break;
	      default:
		break;
	    }

	}

	_Alire = FALSE;
	_getpos_possible = TRUE;
}

void
U8GetPos::BoutonPress(const XEvent* cdtevent)
{

	if (_Alire)
	{
	    _x = cdtevent->xbutton.x;
	    _y = cdtevent->xbutton.y;

	    if(_picking_elastique == SANSELASTIQUE)
	    {

		if(_x_cur_save != -1)
		{
		    XDrawLine(_disp, _win, _curseur_gc,0,_y_cur_save,
				    1200, _y_cur_save);
		    XDrawLine(_disp, _win, _curseur_gc,_x_cur_save,0,
				    _x_cur_save, 1200);
		}

		_x_cur_save = _x;
		_y_cur_save = _y;

		XDrawLine(_disp, _win, _curseur_gc, 0,
				_y_cur_save, 1200, _y_cur_save);
		XDrawLine(_disp, _win, _curseur_gc, _x_cur_save,
				0, _x_cur_save, 1200);
	    }

	    if(_picking_elastique && _picking_carre)
	    {
		register int deltx = _x - _x_cur_min_save;
		register int delty = _y - _y_cur_min_save;

		if(abs(deltx) > abs(delty))
		{
		    _y = _y_cur_min_save + abs(deltx)*U8SIGN(delty);
		}
		else
		{
		    _x = _x_cur_min_save + abs(delty)*U8SIGN(deltx);
		}

	    }

	    _Alire = FALSE;
	}
	else
	{
	    _Disponible = TRUE;
	}

}

void
U8GetPos::BoutonRelease(const XEvent* cdtevent)
{

	if (_Alire)
	{
	    _x = cdtevent->xbutton.x;
	    _y = cdtevent->xbutton.y;

	    if(_picking_elastique && _picking_carre)
	    {
		register int deltx = _x - _x_cur_min_save;
		register int delty = _y - _y_cur_min_save;

		if(abs(deltx) > abs(delty))
		{
		    _y = _y_cur_min_save + abs(deltx)*U8SIGN(delty);
		}
		else
		{
		    _x = _x_cur_min_save + abs(delty)*U8SIGN(deltx);
		}

	    }

	    _Alire = FALSE;
	}

	_Disponible = TRUE;
}

void
U8GetPos::PointeurMotion(const XEvent* cdtevent)
{

	if(_picking_grandecroix)
	{

	    switch(_picking_elastique)
	    {
	      case SANSELASTIQUE:

		if(_x_cur_save != -1)
		{
		    XDrawLine(_disp, _win, _curseur_gc,0,_y_cur_save,
				    1200, _y_cur_save);
		    XDrawLine(_disp, _win, _curseur_gc,_x_cur_save,0,
				    _x_cur_save, 1200);
		}

		_x_cur_save = cdtevent->xmotion.x;
		_y_cur_save = cdtevent->xmotion.y;

		XDrawLine(_disp, _win, _curseur_gc, 0,
				_y_cur_save, 1200, _y_cur_save);
		XDrawLine(_disp, _win, _curseur_gc, _x_cur_save,
				0, _x_cur_save, 1200);

		if(!_Alire) _Disponible = True;

		break;
	      case RECTANGLEELASTIQUE:

		if(_x_cur_max_save != -1)
		{
		    XDrawLine(_disp,_win,_curseur_gc,
				    _x_cur_min_save,_y_cur_min_save,
				    _x_cur_max_save,_y_cur_min_save);
		    XDrawLine(_disp,_win,_curseur_gc,
				    _x_cur_max_save,_y_cur_min_save,
				    _x_cur_max_save,_y_cur_max_save);
		    XDrawLine(_disp,_win,_curseur_gc,
				    _x_cur_max_save,_y_cur_max_save,
				    _x_cur_min_save,_y_cur_max_save);
		    XDrawLine(_disp,_win,_curseur_gc,
				    _x_cur_min_save,_y_cur_max_save,
				    _x_cur_min_save,_y_cur_min_save);
		}

		_x_cur_max_save = cdtevent->xmotion.x;
		_y_cur_max_save = cdtevent->xmotion.y;

		if(_picking_carre)
		{
		    register int deltx = _x_cur_max_save - _x_cur_min_save;
		    register int delty = _y_cur_max_save - _y_cur_min_save;

		    if(abs(deltx) > abs(delty))
		    {
			_y_cur_max_save = _y_cur_min_save +
			    abs(deltx)*U8SIGN(delty);
		    }
		    else
		    {
			_x_cur_max_save = _x_cur_min_save +
			    abs(delty)*U8SIGN(deltx);
		    }

		}

		XDrawLine(_disp,_win,_curseur_gc,
				_x_cur_min_save,_y_cur_min_save,
				_x_cur_max_save,_y_cur_min_save);
		XDrawLine(_disp,_win,_curseur_gc,
				_x_cur_max_save,_y_cur_min_save,
				_x_cur_max_save,_y_cur_max_save);
		XDrawLine(_disp,_win,_curseur_gc,
				_x_cur_max_save,_y_cur_max_save,
				_x_cur_min_save,_y_cur_max_save);
		XDrawLine(_disp,_win,_curseur_gc,
				_x_cur_min_save,_y_cur_max_save,
				_x_cur_min_save,_y_cur_min_save);

		if(!_Alire) _Disponible = True;

		break;
	      default:
		break;
	    }

	}

}

void
U8GetPos::Clear()
{
	_x_cur_save = -1;
	_y_cur_save = -1;
	_x_cur_max_save = -1;
	_y_cur_max_save = -1;
}
