// ***********************************************************
// *    classe Driver Xfig (modele de driver L3 en C++)
// ***********************************************************

struct SccsIdU8Xfig
{
	static const char* const sccs_id;
};

const char* const SccsIdU8Xfig::sccs_id = "%W% %G% Pierre Brochard";

#include "U8constant.h"
#include "poulpe/U8xfig.h"
#include "sanglier.h"
#include "L3ListeDesCodes.h"
#include "nx.h"
#include "dinosaure/S6string.h"

#ifndef TRUE
#  define TRUE 1
#endif

#ifndef FALSE
#  define FALSE 0
#endif

#define MAX_STRING   256

// *************************************************************************
// * implementation classe U8Xfig (driver pour xfig : freeware logiciel
// * de dessin sous X-Window)
// *************************************************************************

int U8Xfig::_numfile = 0;
U8Xfig* U8Xfig::_Xfig = NULL;
const int U8Xfig::_maxcolors = 256;
const int U8Xfig::_defncolors = 16;
const char* U8Xfig::_defaultmap[U8Xfig::_defncolors] = 
{
   "#ffffff","#0000ff","#00ff00","#ffff00",
   "#ff8000","#ff0000","#ff00ff","#c8c8c8",
   "#8c8c8c","#00ffff","#74bf40","#8c8000",
   "#ffaa91","#ffc8c8","#bf00bf","#000000"
};

U8Xfig::U8Xfig() :
		_filedebut(NULL),
		_filename(NULL),
		_filenx(NULL),
		_file(NULL),
		_factor(.13),
		_xmin(-32768),
		_xmax(32767),
		_ymin(-32768),
		_ymax(32767),
		_a(_factor),
		_b(-_factor*_xmin),
		_c(-_factor),
		_d(_factor*_ymax),
		_ncolors(0),
		_color(-2),
		_depth(999),
		_texte_soft(FALSE),
		_axe(FALSE),
		_first_vect(FALSE)
{
}

U8Xfig::~U8Xfig()
{
	delete[] _filedebut;
	delete[] _filename;
}

void
U8Xfig::NomDebutFichier(const char* filedebut)
{
	delete _filedebut;

	S6String ss(filedebut);

	_filedebut = ss;
}

// *****************************************
//         fonction L3Dialogue
// *****************************************

void
U8Xfig::L3Dialogue(int bloc_l3[])
{
}

void
U8Xfig::L3Open(const int bloc_l3[])
{

	{
	    ostrstream os;

	    os << _filedebut << setfill('0') << setw(4) << _numfile <<
		".fig" << ends;
	    _filename = os.str();
	}

	_filenx = fnxopen(_filename,"w");
	_file = fnx_write_desc(_filenx);
	fprintf(_file,"#FIG 3.2\nPortrait\nCenter\nMetric\nA4\n100.00\n");
	fprintf(_file,"Single\n-2\n1200 2\n");

    // init table de couleurs

	_ncolors = _defncolors;
	_ncolors = _ncolors > _maxcolors ? _maxcolors : _ncolors;

	for(int i=0;i<_ncolors;i++)
	{
	    fprintf(_file,"0 %d %s\n",i+32,_defaultmap[i]);
	}

    // Init profondeur

	_depth = 999;

    // init texte soft

	_texte_soft = FALSE;
	_first_vect = FALSE;

    // Reinit des coef de page (ils peuvent avoir change avec CODEL3CADRE)

	_a = _factor;
	_c = -_factor;
}

void
U8Xfig::L3Close(const int bloc_l3[])
{

	if(imagevide)
	{
	    fnxclose(_filenx);
	    unlink(_filename);
	}
	else
	{
	    fnxclose(_filenx);
	    _numfile++;
	}

	_filenx = NULL;
	_file = NULL;
	delete _filename;_filename = NULL;
}

void
U8Xfig::L3Texte(const int bloc_l3[])
{
	char tstring[MAX_STRING];
	register const int* const p = bloc_l3 + 1;
	const int len = *(p + 5);

	if(len > MAX_STRING - 1)
	{
	    fprintf(stderr, "Chaine trop longue non traitee\n");
	    return;
	}

	const char* chaine = (char*) (p + 6);

	for(register int ii = 0;ii<len;ii++)
	{
	    tstring[ii] = chaine[ii+2*(ii/2)+2];
	}

	tstring[len] = '\0';

	const double cx = _a/_factor;
	const double cy = -_c/_factor;
	const double cr = sqrt((cx*cx+cy*cy)*.5);
	const double ang = double(p[4]) / 1024.0;
	const double atang = atan(cx != 0. ? tan(ang)*cy/cx : 0.);
	const double sang = cos(ang) >= 0. ? atang : atang + M_PI;
    // 18.5 (cm) / 2.5 (=> pouces) * 72 (pt/pouce) = 504 puis * 2 <=> soft
    // car la taille de la fonte est une hauteur. On considere que la largeur
    // est la moitie.
	const float ch_he = 1008.*cr/p[3];
	const int color = _color >= 0 ? _color + 32 : 32;
	const int xl3 = int(_a * double(p[1]) + _b);
	const int yl3 = int(_c * double(p[2]) + _d);

	fprintf(_file,"4 0 %d %d -1 0 %f %f 2 500 500 %d %d %s\\001\n",
			color,_depth,ch_he,sang,xl3,yl3,tstring);
	_depth--;_depth = _depth >= 0 ? _depth : 0;
}

void
U8Xfig::L3TexteSoft(const int bloc_l3[])
{
	_texte_soft = _texte_soft ? FALSE : TRUE;

	if(_texte_soft)
	{
	    _first_vect = TRUE;
	}
	else if(!_first_vect)
	{
	    fprintf(_file,"-6\n");

	    if(!_axe)
	    {
		_depth--;_depth = _depth >= 0 ? _depth : 0;
	    }

	}

}

void
U8Xfig::L3Colormap(const int bloc_l3[])
{

	if (bloc_l3[0] < 8) return;

	_ncolors = bloc_l3[2];
	_ncolors = _ncolors > _maxcolors ? _maxcolors : _ncolors;

	char coul[8];

	for(int i=0,j=3;i<_ncolors;i++,j+=3)
	{
	    sprintf(coul,"#%2.2x%2.2x%2.2x",bloc_l3[j],bloc_l3[j+1],
			    bloc_l3[j+2]);
	    fprintf(_file,"0 %d %s\n",i+32,coul);
	}

}

void
U8Xfig::L3Couleur(const int bloc_l3[])
{
	const int color = bloc_l3[2];

	_color = (color < _ncolors) ? color : _ncolors - 1; // Couleur courante
}

void
U8Xfig::L3Brillance(const int bloc_l3[])
{
}

void
U8Xfig::L3Pixel(const int bloc_l3[])
{
}

void
U8Xfig::L3Cadre(const int bloc_l3[])
{
	register const int cadrex = bloc_l3[2];
	register const int cadrey = bloc_l3[3];

	_a = _factor * double(cadrex / 185.);
	_c = -_factor * double(cadrey / 185.);
}

void
U8Xfig::L3Segment(const int bloc_l3[])
{
	register const int npo = bloc_l3[0]/2 - 1;

	if(npo < 2) return;

	register const int color = _color >= 0 ? _color + 32 : 32;
	register const int* p = bloc_l3 + 2;
	register const int* const pmax = p + 2*npo - 2;
	register const int ix1 = int(_a * double(p[0]) + _b);
	register const int iy1 = int(_c * double(p[1]) + _d);
	register const int ix2 = int(_a * double(pmax[0]) + _b);
	register const int iy2 = int(_c * double(pmax[1]) + _d);

	if(_first_vect)
	{
	/* Coordonnees de la boite code 6 = Compound object */
	    fprintf(_file,"6 %d %d %d %d\n",ix1,iy1,ix2,iy2);
	    _first_vect = FALSE;
	}

	fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n",
			color,_depth,npo);
	fprintf(_file,"%d %d ",ix1,iy1);p+=2;

	if(!_axe && !_texte_soft)
	{
	    _depth--;_depth = _depth >= 0 ? _depth : 0;
	}

	while(p<pmax)
	{
	    register const int ix = int(_a * double(*p++) + _b);
	    register const int iy = int(_c * double(*p++) + _d);

	    fprintf(_file,"%d %d ",ix,iy);
	}

	fprintf(_file,"%d %d\n",ix2,iy2);
}

void
U8Xfig::L3Marqueur(const int bloc_l3[])
{
	const int* const p = bloc_l3 + 1;
	const int Xm = int(_a * p[1] + _b);
	const int Ym = int(_c * p[2] + _d);
	const int clip = int(_taillel3*_factor);

    /* test du clipping */

	if ((Xm < 0) || (Xm > clip)) return;

	if ((Ym < 0) || (Ym > clip)) return;

	const int color = _color >= 0 ? _color + 32 : 32;
	const int d1 = 16;
	const int d2 = 32;

    /*
      Sauf pour les lettres, on dessine directement le marqueur
      en pixels via le buffer de segments
    */

	switch (p[3])
	{
	  case 1:

	/* '+' */
	    fprintf(_file,"6 %d %d %d %d\n",Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm,Ym-d2,Xm,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm-d2,Ym,Xm+d2,Ym);
	    fprintf(_file,"-6\n");
	    break;
	  case 2:
	/* '*' */
	    fprintf(_file,"6 %d %d %d %d\n",Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm,Ym-d2,Xm,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm-d2,Ym,Xm+d2,Ym);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,
			    Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,
			    Xm-d2,Ym+d2,Xm+d2,Ym-d2);
	    fprintf(_file,"-6\n");
	    break;
	  case 3:
	/* 'o' */
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d %d %d %d %d %d %d\n",color,_depth,5,
			    Xm-d2,Ym-d2,Xm-d2,Ym+d2,Xm+d2,Ym+d2,
			    Xm+d2,Ym-d2,Xm-d2,Ym-d2);
	    break;
	  case 4:
	/* 'x' */
	    fprintf(_file,"6 %d %d %d %d\n",Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm-d2,Ym-d2,
			    Xm+d2,Ym+d2);
	    fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
			    "%d %d %d %d\n",color,_depth,2,Xm-d2,Ym+d2,
			    Xm+d2,Ym-d2);
	    fprintf(_file,"-6\n");
	    break;
	  default:
	/* '.' ou lettre (centree) */

	    if (*p > 256)
	    {
		const char* const temp = ((char*) p) + sizeof(int) - 2;
		char tstring[2];

		tstring[0] = temp[0];
		tstring[1] = '\0';
		fprintf(_file,"4 1 %d %d -1 0 %f %f 2 500 500 %d %d %s\\001\n",
				color,_depth,12.,0.,Xm,Xm,tstring);
	    }
	    else // gros point
	    {
		fprintf(_file,"2 1 -1 1 %d -1 %d -1 -1 0 0 0 -1 0 0 %d\n"
				"%d %d %d %d %d %d %d %d %d %d\n",color,_depth,5,
				Xm-d1,Ym-d1,Xm-d1,Ym+d1,Xm+d1,Ym+d1,
				Xm+d1,Ym-d1,Xm-d1,Ym-d1);
	    } /* if */

	}

	_depth--;_depth = _depth >= 0 ? _depth : 0;
}

void
U8Xfig::L3Polygone(const int bloc_l3[])
{
	register const int npo = bloc_l3[0]/2 - 1;
	register const int color = _color >= 0 ? _color + 32 : 32;

	fprintf(_file,"2 3 -1 1 %d %d %d -1 20 0 0 0 -1 0 0 %d\n",
			color,color,_depth,npo+1);
	_depth--;_depth = _depth >= 0 ? _depth : 0;

	register const int* p = bloc_l3 + 2;
	register const int* const pmax = p + 2*npo;
	register const int ix1 = int(_a * double(*p++) + _b);
	register const int iy1 = int(_c * double(*p++) + _d);

	fprintf(_file,"%d %d ",ix1,iy1);

	while(p<pmax)
	{
	    register const int ix = int(_a * double(*p++) + _b);
	    register const int iy = int(_c * double(*p++) + _d);

	    fprintf(_file,"%d %d ",ix,iy);
	}

	fprintf(_file,"%d %d\n",ix1,iy1);
}

void
U8Xfig::L3Axe(const int bloc_l3[])
{
	_axe = _axe ? FALSE : TRUE;

	if(_axe)
	{
	    _first_vect = TRUE;
	}
	else if(!_first_vect)
	{
	    fprintf(_file,"-6\n");
	    _depth--;_depth = _depth >= 0 ? _depth : 0;
	}

}

void
U8Xfig::L3Flush(const int bloc_l3[])
{
}

void
U8Xfig::L3Default(const int bloc_l3[])
{
}

void
U8Xfig::L3End(const int bloc_l3[])
{
}
// Fin de Fichier
