/********************/
/* L3AppelDrivers.c */
/********************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "L3Environnement.h"
#include "L3AppelDrivers.h"


/* identification du source */

#ifndef lint
    static const char sccs_id[] = "%W% %G% G. Colin de Verdiere";
#endif

/* Auteur G. Colin de Verdiere DMA/SIE/IG 7/21/93 */

typedef struct
{
	int selectionne;
	void (*driver) (int codel3[]);
	char *nom;
	int Interactif; /* Le driver est-il interactif ? */
} Drivers;


/* Creation d'un driver bidon pour extension par l'utilisateur */
static void L3DummyDriver(int codel3[]) { /* vide */ }

static Drivers  driver[] =
{
	{FALSE, L3NegsDriver, "negs", TRUE},
	{FALSE, L3T4014Driver, "t4014", TRUE},
	{FALSE, L3VisuDriver, "visu", FALSE},
	{FALSE, L3PostscriptDriver, "post", FALSE},
	{FALSE, L3PostscriptDriverLwc, "lwc", FALSE},
	{FALSE, L3PostscriptDriverLwct, "lwct", FALSE},
	{FALSE, L3DummyDriver, "dummy", FALSE} /* driver utilisateur */
};

static void (*driver_en_cours) (int codel3[]) = NULL;

static int nbdrivers = sizeof(driver) / sizeof(Drivers);

int codel3close[] = {2, CODEL3CLOSE};
int codel3open[] = {2, CODEL3OPEN};
int codel3cadre[] = {4, CODEL3CADRE, 185, 185};
int codel3couleur[] = {3, CODEL3COULEUR, 0};
int codel3brillance[] = {3, CODEL3BRILLANCE, 1};
int codel3cmap[1030];
int codel3epaisseur[] = {3, CODEL3EPAISSEUR, 1};

/*
 * Le mode copie n'est pas un driver en soi car la copie peut reinjecter
 * l'image cree dans un veritable driver
 * ce mode sert surtout a l'edition des images (pour faire le UNDO)
 * l'option choisie est de garder les codes L3 en memoire ; cela pourrait
 * tout aussi bien etre sur disque (mais ce serait plus lent).
 */

static int ModeCopie = FALSE;

void
L3AddNewDriver(void (*NewDriver) (int codel3[]), char* Nom, int interactif)
{
   char* LeNom = malloc(strlen(Nom)+1);
   driver[nbdrivers - 1].Interactif = interactif;
   driver[nbdrivers - 1].driver = NewDriver;
   strcpy(LeNom, Nom);

   if(strcmp(driver[nbdrivers - 1].nom,"dummy") != 0)
	free(driver[nbdrivers - 1].nom);

   driver[nbdrivers - 1].nom = LeNom;
}

void
L3SetModeCopie(int mode)
{
   ModeCopie = mode;

   if(ModeCopie)
   {
      driver_en_cours = NULL;
   }
   else
   {
      int compteur_driver = 0;
      int i = 0;

      for (; i < nbdrivers; i++)
      {

         if (driver[i].selectionne)
         {
            compteur_driver++;
            driver_en_cours = driver[i].driver;
         }

      }

      if(compteur_driver != 1)
      {
         driver_en_cours = NULL;
      }

   }

}

static void
L3InitDesDrivers(void)
{
   L3NegsInitialize();
}

void
L3ChoixDriver(const char* nom)
{
	int i = 0;
	static int Init = FALSE;
	char* LeNom = NULL;
	int compteur_driver = 0;
	int ok = 0;

	if (!Init)
	{
	    L3InitDesDrivers();
	    Init = TRUE;
	}

    /* Detournement par l'exterieur de la destination si necessaire */
	if (!getenv("TERML3"))
	{
	/* destination par programme */
	    LeNom = malloc(strlen(nom) + 1);
	    strcpy(LeNom, nom);
	}
	else
	{

	    if (strlen(getenv("TERML3")))
	    {
	    /* destination par l'environnement */
		LeNom = malloc(strlen(getenv("TERML3")) + 1);
		strcpy(LeNom, getenv("TERML3"));
	    /* Conversion en minuscule */
		i = 0;

		while(LeNom[i])
		{
		    LeNom[i] = tolower(LeNom[i]);
		    i++;
		}

	    /* Gestion des equivalences */
		if (!strncmp(LeNom, "su", 2))
		{
		    free(LeNom);
		    LeNom = (char*) malloc(3);
		    strcpy(LeNom, "negs");
		}
		if (!strncmp(LeNom, "xt", 2)) {
		    free(LeNom);
		    LeNom = (char *) malloc(3);
		    strcpy(LeNom, "t4014");
		}
		if (!strncmp(LeNom, "pe", 2)) {
		    free(LeNom);
		    LeNom = (char *) malloc(3);
		    strcpy(LeNom, "t4014");
		}
		if (!strncmp(LeNom, "fa", 2)) {
		    free(LeNom);
		    LeNom = (char *) malloc(3);
		    strcpy(LeNom, "t4014");
		}
		if (!strncmp(LeNom, "pe", 2)) {
		    free(LeNom);
		    LeNom = (char *) malloc(3);
		    strcpy(LeNom, "t4014");
		}

	    }
	    else
	    {
	    /* destination par programme */
		LeNom = malloc(strlen(nom) + 1);
		strcpy(LeNom, nom);
	    }

	}

    /* Conversion en minuscule */
	i = 0;

	while(LeNom[i])
	{
	    LeNom[i] = tolower(LeNom[i]);
	    i++;
	}

	for (i = 0; i < nbdrivers; i++)
	{

	    if (strcmp(LeNom, driver[i].nom) == 0)
	    {
		driver[i].selectionne = TRUE;
		ok = 1;

		if (L3CMap)
		{
		/* passage de la carte au driver */
		    (*driver[i].driver) (codel3cmap);
		    (*driver[i].driver) (codel3cadre);
		    (*driver[i].driver) (codel3couleur);
		    (*driver[i].driver) (codel3epaisseur);
		/* passage de la brillance au driver inutile ! */
		}
		else
		{
		    (*driver[i].driver) (codel3cadre);
		    (*driver[i].driver) (codel3couleur);
		    (*driver[i].driver) (codel3brillance);
		    (*driver[i].driver) (codel3epaisseur);
		}

	    }

	    if (driver[i].selectionne)
	    {
		compteur_driver++;
		driver_en_cours = driver[i].driver;
	    }

	}

	if (!ok)
	{
	    fprintf(stderr, "Driver demande inconnu : %s\n", LeNom);
	}

	if(compteur_driver != 1 || ModeCopie)
	{
	    driver_en_cours = NULL;
	}

	free(LeNom);
}

void
L3FinChoixDriver(const char* nom)
{
   int i;
   int compteur_driver = 0;

   for (i = 0; i < nbdrivers; i++)
   {

      if (strcmp(nom, driver[i].nom) == 0)
      {
	 driver[i].selectionne = FALSE;
      }

      if (driver[i].selectionne)
      {
         compteur_driver++;
         driver_en_cours = driver[i].driver;
      }

   }

   if(compteur_driver != 1 || ModeCopie)
   {
      driver_en_cours = NULL;
   }

}

void
L3AppelDrivers(int codel3[])
{

   if(driver_en_cours)
   {
      (*driver_en_cours) (codel3);
   }
   else
   {
      int i = 0;

      for (; i < nbdrivers; i++)
      {

         if (driver[i].selectionne) (*driver[i].driver) (codel3);

      }

      if (ModeCopie) L3CopieDriver(codel3);

   }

}

void
L3QuestionDriver(void)
{
   int             i, j;
   char            bidon[256];
   char            nom[256];

   (void) printf("Choix de votre terminal ? \n");
   (void) printf("Vous pouvez choisir entre exit (ou ^D) et : \n");

   for (i = 0; i < nbdrivers; i++) {
      if (driver[i].Interactif)
	printf("    %s\n", driver[i].nom);
   }

   while (!feof(stdin)) {
      /* Detournement par l'exterieur de la destination si necessaire */
      if (getenv("TERML3") != NULL) {
	 /* destination imposee par l'exterieur on ne pose pas la question */
	 strcpy(bidon, getenv("TERML3"));
      } else {
	 (void) printf("Votre reponse : ");
	 (void) fflush(stdout);
	 (void) fgets(bidon, 256, stdin);
      }   
      /* elimination des eventuels blancs en tete de chaine */
      j=0;
      for (i = 0; i<256; i++) {
	 if (bidon[i] != ' ') {
	    nom[j] = bidon[i];
	    if (bidon[i] == '\0') {
	       break;
	    }
	    j++;
	 }
      }
      
      if (strncmp(nom, "ex", 2) == 0) {
	 fprintf(stderr, " Abandon Sanglier L3QuestionDriver\n");
	 exit(1);
      }
      for (i = 0; i < nbdrivers; i++) {
	 if (strncmp(nom, driver[i].nom, 2) == 0) {
	    (void) fprintf(stderr, " Driver %s choisi\n",
			   driver[i].nom);
	    L3ChoixDriver(driver[i].nom);
	    return;
	 }
      }
      /* arrive ici c'est que le terminal demande est inconnu */
      (void) fprintf(stderr, "\007L3QuestionDriver: Driver %s inconnu\n", nom);
      if (getenv("TERML3") != NULL) {
	 break;
      }
   } /* while */
   (void) fprintf(stderr, " Abandon Sanglier L3QuestionDriver sur EOF\n");
   exit(1);
} /* L3QuestionDriver */


/* Fin de Fichier */
