#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "L3Environnement.h"
#include "L3CartesDeCouleursP.h"
#include "L3AppelDrivers.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

static void computecolormap(nom_de_carte type, /* Nom de la carte de couleurs */
			  int *red, /* tableau des composantes rouges */
			  int *green, /* tableau des composantes vertes */
			  int *blue, /* tableau des composantes bleues */
			  int nc); /* Nombre de couleurs de la carte */
static void CreeCarteL3(int *red, int *green, int *blue,const int n);

static RGB_INT dlpp1_def[] = { 
			     {191, 255, 255},
			     {127, 255, 255},
			     {127, 191, 255},
			     {127, 127, 255},
			     {127, 127, 191},
			     {127, 127, 127},
			     {127, 191, 127},
			     {127, 255, 127},
			     {191, 191, 191},
			     {255, 127, 255},
			     {255, 127, 127},
			     {255, 127,   0},
			     {255,  63,   0},
			     {255,   0,   0},
			     {191,   0,   0}
			   }; 

static RGB_INT xcode_def[] = {
                             {163, 163, 245},
                             {255, 163, 163},
                             {163, 205, 163},
                             {207, 197, 127},
                             {217, 157, 217},
                             {147, 197, 217},
                             {193, 185, 193},
                             {123, 123, 235},
                             {255, 123, 123},
                             {123, 185, 123},
                             {177, 167,  87},
                             {217, 107, 217},
                             { 57, 177, 197},
                             {143, 135, 143},
                             { 83,  83, 225},
                             {230,  83,  83},
                             { 67, 158,  77},
                             {135, 132,  67},
                             {187,  37, 187},
                             { 27, 157, 177},
                             {103,  95, 103},
                             {223, 223, 255},
                             {255, 223, 223},
                             {223, 255, 223},
                             {247, 242, 207},
                             {247, 207, 247},
                             {207, 247, 247},
                             {235, 235, 235},
                             {127,   0, 205},
                             {255,   0,   0},
                             {  0, 119,  25},
                             {  0,   0,   0}
                           };

/*----------------------------------------------------------------------
 *                  Procedure Utilisateur  :  L3ChoixCartesDeCouleurs   
 *----------------------------------------------------------------------
 *      Parametres :                                                    
 *                                                                      
 *         nomcarte  :  nom generique de la carte choisie     ( <- )    
 *         nombredecouleurs : nombre de couleurs de la carte  ( <- )    
 *         fond  : couleur du fond ( Composantes r , v , b )  ( <- )    
 *         antifond : couleur de "l'anti-fond" (Composantes r , v , b)  ( <- )
 *                                                                            
 *     Valeur de retour :                                                     
 *         ok : entier indiquant si la carte demandee a ete trouvee ou pas (->)
 *                         0  :  La carte demandee a ete trouvee              
 *                         1  :  La carte demandee n'a pas ete trouvee        
 *--------------------------------------------------------------------------*/

int
L3ChoixCartesDeCouleurs(nom_de_carte nomcarte,
			    int nombredecouleurs,
			    RGB *fond,
			    RGB *antifond)
{
   int            *codel3;
   int             i, j;
   int             nc;
   int            *tempr,*tempg,*tempb;

   if(nombredecouleurs < 16)
   {
      fprintf(stderr,"L3ChoixCartesDeCouleurs : "
		      "Nombre de couleurs trop petit (16 min) \n");
      return 1;
   }
   else
   {
      NOMBREDECOULEURS = nombredecouleurs;
      CARTEDECOULEURS = PRESENTE;
   }

   switch(nomcarte)
   {
    case ABSOLUTE:
      NOMCARTECOULEUR = ABSOLUTE;
      break;
    case BINARY:
      NOMCARTECOULEUR = BINARY;
      break;
    case BLUEGREEN:
      NOMCARTECOULEUR = BLUEGREEN;
      break;
    case DEFAULT:
      NOMCARTECOULEUR = DEFAULT;
      break;
    case NEGATIVE:
      NOMCARTECOULEUR = NEGATIVE;
      break;
    case POSITIVE:
      NOMCARTECOULEUR = POSITIVE;
      break;
    case RAINBOW:
      NOMCARTECOULEUR = RAINBOW;
      break;
    case RAINBOWHLS:
      NOMCARTECOULEUR = RAINBOWHLS;
      break;
    case RANDOM:
      NOMCARTECOULEUR = RANDOM;
      break;
    case REDBLUE:
      NOMCARTECOULEUR = REDBLUE;
      break;
    case REDGREEN:
      NOMCARTECOULEUR = REDGREEN;
      break;
    case SHIFTED:
      NOMCARTECOULEUR = SHIFTED;
      break;
    case STANDARDL3:
      NOMCARTECOULEUR = STANDARDL3;
      break;
    case DLPP1:
      NOMCARTECOULEUR = DLPP1;
      break;
    case XCODE:
      NOMCARTECOULEUR = XCODE;
      break;
    case INVDEFAULT:
      NOMCARTECOULEUR = INVDEFAULT;
      break;
    default:
      fprintf(stderr,
	      "L3ChoixCartesDeCouleurs : Carte de couleurs inconnue\n");
      return 1;
   }

   codel3 = (int *) malloc((unsigned) (3 * (nombredecouleurs + 1) * sizeof(int)));
   codel3cmap[0] = codel3[0] = 3 * nombredecouleurs + 3;
   codel3cmap[1] = codel3[1] = CODEL3COLORMAP ;
   codel3cmap[2] = codel3[2] = nombredecouleurs;
   
   if (COMPOSANTEROUGE != NULL) {
      free((char *) COMPOSANTEROUGE);
      free((char *) COMPOSANTEVERTE);
      free((char *) COMPOSANTEBLEUE);
   }

   COMPOSANTEROUGE = (int *) malloc((unsigned) nombredecouleurs * sizeof(int));
   COMPOSANTEVERTE = (int *) malloc((unsigned) nombredecouleurs * sizeof(int));
   COMPOSANTEBLEUE = (int *) malloc((unsigned) nombredecouleurs * sizeof(int));

   /* Couleurs de 0 a 9 imposees
      0 : fond
      1 : antifond
      2 : bleu
      3 : vert
      4 : jaune
      5 : orange
      6 : rouge
      7 : violet
      8 : Noir
      9 : Blanc
      */

   if (fond != NULL) {
      COMPOSANTEROUGE[0] = fond->r;
      COMPOSANTEVERTE[0] = fond->g;
      COMPOSANTEBLEUE[0] = fond->b;
   } else {
      /* Fond et antifond n'ont aucune importance et sont device dependant */
      COMPOSANTEROUGE[0] = 255;
      COMPOSANTEVERTE[0] = 255;
      COMPOSANTEBLEUE[0] = 255;
   }

   if (antifond != NULL) {
      COMPOSANTEROUGE[1] = antifond->r;
      COMPOSANTEVERTE[1] = antifond->g;
      COMPOSANTEBLEUE[1] = antifond->b;
   } else {
      COMPOSANTEROUGE[1] = 0;
      COMPOSANTEVERTE[1] = 0;
      COMPOSANTEBLEUE[1] = 0;
   }

   COMPOSANTEROUGE[2] = 0;
   COMPOSANTEVERTE[2] = 0;
   COMPOSANTEBLEUE[2] = 255;	/* Bleu */
   
   COMPOSANTEROUGE[3] = 0;
   COMPOSANTEVERTE[3] = 255;
   COMPOSANTEBLEUE[3] = 20;	/* Vert */
   
   COMPOSANTEROUGE[4] = 255;
   COMPOSANTEVERTE[4] = 255;
   COMPOSANTEBLEUE[4] = 0;	/* Jaune */
   
   COMPOSANTEROUGE[5] = 255;
   COMPOSANTEVERTE[5] = 128;
   COMPOSANTEBLEUE[5] = 0;	/* Orange */
   
   COMPOSANTEROUGE[6] = 255;
   COMPOSANTEVERTE[6] =  0;
   COMPOSANTEBLEUE[6] = 0;	/* Rouge */

   COMPOSANTEROUGE[7] = 255;
   COMPOSANTEVERTE[7] = 0;
   COMPOSANTEBLEUE[7] = 255;	/* Violet */
   
   COMPOSANTEROUGE[8] = 0;
   COMPOSANTEVERTE[8] = 0;
   COMPOSANTEBLEUE[8] = 0;	/* Noir */
   
   COMPOSANTEROUGE[9] = 255;
   COMPOSANTEVERTE[9] = 255;
   COMPOSANTEBLEUE[9] = 255;	/* Blanc */
   
   nc = nombredecouleurs - NbColReserved;
   
   tempr = COMPOSANTEROUGE + NbColReserved;
   tempg = COMPOSANTEVERTE + NbColReserved;
   tempb = COMPOSANTEBLEUE + NbColReserved; 
   
   computecolormap(nomcarte, tempr , tempg , tempb , nc);  

   for (i = 0; i < nombredecouleurs ; i++) {
      j = 3 + i * 3;
      codel3cmap[j    ] = codel3[j    ] = COMPOSANTEROUGE[i];
      codel3cmap[j + 1] = codel3[j + 1] = COMPOSANTEVERTE[i];
      codel3cmap[j + 2] = codel3[j + 2] = COMPOSANTEBLEUE[i];
   }

   L3CMap = TRUE;
   L3AppelDrivers(codel3);
   L3Couleur(1); /* couleur par defaut ANTIFOND */

   free((char *) codel3);

   return 0;
} /* L3ChoixCartesDeCouleurs */

/*------------------------------------------------------------------------
 *                                                                        
 *                                                                        
 *                  Procedure Utilisateur  :  L3LitCartesDeCouleurs       
 *                                                                        
 *                                                                        
 *------------------------------------------------------------------------
 *                                                                        
 *      Parametres :                                                      
 *                                                                        
 *                                                                        
 *                                                                        
 *         nombredecouleurs : nombre de couleurs de la carte              
 *                                                                        
 *                                                                        
 *         rouge  : pointeur sur le vecteur des composantes rouges de la carte
 *                                                                            
 *         vert   : pointeur sur le vecteur des composantes vertes de la carte
 *                                                                            
 *         bleu   : pointeur sur le vecteur des composantes bleues de la carte
 *                                                                            
 *----------------------------------------------------------------------------*/

carte_de_couleurs
L3LitCartesDeCouleurs( void )
{
   carte_de_couleurs CarteCourante ;

   if (CARTEDECOULEURS == ABSENTE)
   {
      fprintf (stderr," L3LitCartesDeCouleurs : Erreur : "
		      "Il n'y a pas de carte de couleurs definie \n");
      abort();
   }
   
   CarteCourante.n = NOMBREDECOULEURS ;
   
   CarteCourante.r = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));
   CarteCourante.g = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));
   CarteCourante.b = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));
   
   memcpy((char *) CarteCourante.r, (const char *) COMPOSANTEROUGE,
	  CarteCourante.n * sizeof(int));
   memcpy((char *) CarteCourante.g, (const char *) COMPOSANTEVERTE,
	  CarteCourante.n * sizeof(int));
   memcpy((char *) CarteCourante.b, (const char *) COMPOSANTEBLEUE,
	  CarteCourante.n * sizeof(int));

   return(CarteCourante) ;
}
/*----------------------------------------------------------------------
 *                                                                      
 *                                                                      
 *                  Procedure Utilisateur  :  L3EcritCartesDeCouleurs   
 *                                                                      
 *                                                                      
 *----------------------------------------------------------------------
 *                                                                      
 *      Parametres :                                                    
 *                                                                      
 *                                                                      
 *                                                                      
 *         nombredecouleurs : nombre de couleurs de la carte            
 *                                                                      
 *                                                                      
 *         rouge  : pointeur sur le vecteur des composantes rouges de la carte
 *                                                                     
 *         vert   : pointeur sur le vecteur des composantes vertes de la carte
 *                                                                            
 *         bleu   : pointeur sur le vecteur des composantes bleues de la carte
 *                                                                            
 *----------------------------------------------------------------------------*/

void
L3EcritCartesDeCouleurs(carte_de_couleurs CarteCourante)
{
   int             i, j;
   int            *codel3;

   if (CarteCourante.n <= 0) {
      fprintf(stderr,"L3EcritCartesDeCouleurs : Nombre de couleurs <= 0 \n");
      return;
   }

   CARTEDECOULEURS = PRESENTE;
   NOMBREDECOULEURS = CarteCourante.n;

   if (COMPOSANTEROUGE) {
      free((char *) COMPOSANTEROUGE);
      free((char *) COMPOSANTEVERTE);
      free((char *) COMPOSANTEBLEUE);
   }

   COMPOSANTEROUGE = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));
   COMPOSANTEVERTE = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));
   COMPOSANTEBLEUE = (int *) malloc((unsigned) CarteCourante.n * sizeof(int));

   memcpy((char*) COMPOSANTEROUGE,
		   (char *) CarteCourante.r, CarteCourante.n * sizeof(int));
   memcpy((char*) COMPOSANTEVERTE,
		   (char *) CarteCourante.g, CarteCourante.n * sizeof(int));
   memcpy((char*) COMPOSANTEBLEUE,
		   (char *) CarteCourante.b, CarteCourante.n * sizeof(int));

   codel3 = (int*) malloc((unsigned) (3 * (NOMBREDECOULEURS + 1) * sizeof(int)));
   
   codel3cmap[0] = codel3[0] = 3 * (NOMBREDECOULEURS + 1);
   codel3cmap[1] = codel3[1] = CODEL3COLORMAP;
   codel3cmap[2] = codel3[2] = NOMBREDECOULEURS;

   for (i = 0; i < NOMBREDECOULEURS; i++)
   {
      j = 3 + i * 3;
      codel3cmap[j    ] = codel3[ j ]   = COMPOSANTEROUGE[i];
      codel3cmap[j + 1] = codel3[j + 1] = COMPOSANTEVERTE[i];
      codel3cmap[j + 2] = codel3[j + 2] = COMPOSANTEBLEUE[i];
   }

   L3CMap = TRUE;
   L3AppelDrivers(codel3);

   free((char *) codel3);

   L3Couleur(1);

   return ;
} /* L3EcritCartesDeCouleurs */

/*---------------------------------------------------------------------------
 *                  Procedure Utilisateur  :  L3SelectionCartesDeCouleurs    
 *---------------------------------------------------------------------------
 *      Parametres :                                                         
 *         nomcarte  :  nom generique de la carte choisie     ( <- )         
 *         nombredecouleurs : nombre de couleurs de la carte  ( <- ) 
 *         fond  : couleur du fond ( Composantes r , v , b )  ( <- ) 
 *         antifond : couleur de "l'anti-fond" ( Composantes r , v , b)  ( <- )
 *     Valeur de retour :                                                      
 *         Structure type carte_de_couleurs                                    
 *----------------------------------------------------------------------------*/

carte_de_couleurs
L3SelectionCartesDeCouleurs(nom_de_carte nomcarte,
			    int nombredecouleurs,
			    RGB* fond,
			    RGB* antifond)			    
{
   int            *tempr;
   int            *tempg;
   int            *tempb;
   int             nc;

   carte_de_couleurs CarteSelected;

   if (nombredecouleurs < 16)
   {
      fprintf(stderr,
		      " L3SelectionCartesDeCouleurs : "
		      "Nombre de couleurs trop petit ( 16 minimum ) \n");
      CarteSelected.n = 0;
      CarteSelected.r = NULL;
      CarteSelected.g = NULL;
      CarteSelected.b = NULL;

      return CarteSelected;
   }
   else
   {
      CarteSelected.n = nombredecouleurs;
      CarteSelected.r =
	  (int*) malloc((unsigned) (sizeof(int) * nombredecouleurs));
      CarteSelected.g =
	  (int*) malloc((unsigned) (sizeof(int) * nombredecouleurs));
      CarteSelected.b =
	  (int*) malloc((unsigned) (sizeof(int) * nombredecouleurs));

      if (fond)
      {
	 CarteSelected.r[0] = fond->r;
	 CarteSelected.g[0] = fond->g;
	 CarteSelected.b[0] = fond->b;
      }
      else
      {
	 /* Couleurs de 0 a 9 */
	 CarteSelected.r[0] = 0;
	 CarteSelected.g[0] = 0;
	 CarteSelected.b[0] = 0;
      }

      if (antifond)
      {
	 CarteSelected.r[1] = antifond->r;
	 CarteSelected.g[1] = antifond->g;
	 CarteSelected.b[1] = antifond->b;
      }
      else
      {
	 CarteSelected.r[1] = 255;
	 CarteSelected.g[1] = 255;
	 CarteSelected.b[1] = 255;
      }
      
      CarteSelected.r[2] = 0;
      CarteSelected.g[2] = 0;
      CarteSelected.b[2] = 255;	/* Bleu */
      
      CarteSelected.r[3] = 0;
      CarteSelected.g[3] = 255;
      CarteSelected.b[3] = 20;	/* Vert */
      
      CarteSelected.r[4] = 255;
      CarteSelected.g[4] = 255;
      CarteSelected.b[4] = 0;	/* Jaune */
      
      CarteSelected.r[5] = 255;
      CarteSelected.g[5] = 128;
      CarteSelected.b[5] = 0;	/* Orange */
      
      CarteSelected.r[6] = 255;
      CarteSelected.g[6] = 0;
      CarteSelected.b[6] = 0;	/* Rouge */
      
      CarteSelected.r[7] = 255;
      CarteSelected.g[7] = 0;
      CarteSelected.b[7] = 255;	/* Violet */
      
      CarteSelected.r[8] = 0;
      CarteSelected.g[8] = 0;
      CarteSelected.b[8] = 0;	/* Noir */
      
      CarteSelected.r[9] = 255;
      CarteSelected.g[9] = 255;
      CarteSelected.b[9] = 255;	/* Blanc */
      
      nc = nombredecouleurs - NbColReserved;
      
      tempr = CarteSelected.r + NbColReserved;
      tempg = CarteSelected.g + NbColReserved;
      tempb = CarteSelected.b + NbColReserved;
      
      computecolormap(nomcarte, tempr, tempg, tempb, nc);

      return CarteSelected;
   }

}

static void
computecolormapDefault(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (; i < n; i++)
	{
	    *r = *g = *b = (256 * i) / n;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapAbsolute(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	const int ns2 = n / 2;
	int i = 0;

	for (; i < ns2; i++)
	{
	    *r = *g = *b = (256 * i) / n << 1;
	    r++;
	    g++;
	    b++;
	}
      
	*r = *g = *b = 255 ;
	r++;
	g++;
	b++;

	for (i = ns2 + 1 ; i < n; i++)
	{
	    const int j = 256 - (256 * i) / n;

	    *r = *g = *b = j << 1;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapBinary(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    const int j = i % 8;

	    switch (j)
	    {
	      case 0:
		*r++ = 0;
		*g++ = 128;
		*b++ = 255;
		break;
	      case 1:
		*r++ = 0;
		*g++ = 0;
		*b++ = 255;
		break;
	      case 2:
		*r++ = 0;
		*g++ = 255;
		*b++ = 255;
		break;
	      case 3:
		*r++ = 0;
		*g++ = 255;
		*b++ = 0;
		break;
	      case 4:
		*r++ = 255;
		*g++ = 0;
		*b++ = 255;
		break;
	      case 5:
		*r++ = 255;
		*g++ = 0;
		*b++ = 0;
		break;
	      case 6:
		*r++ = 255;
		*g++ = 255;
		*b++ = 0;
		break;
	      case 7:
		*r++ = 255;
		*g++ = 127;
		*b++ = 0;
		break;
	    }

	}

}

static void
computecolormapRainbow(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    const int k = (256 * i) / n;

	    if(k < 25)
	    {
		*r = 191 + (((25 - k) << 6) / 25);
		*g = 0;
		*b = 255;
	    }
	    else if(k < 51)
	    {
		*r = (191 * (51 - k)) / 26;
		*g = 0;
		*b = 255;
	    }
	    else if(k < 76)
	    {
		*r = 0;
		*g = (191 * (k - 51)) / 25;
		*b = 255;
	    }
	    else if(k < 102)
	    {
		*r = 0;
		*g = 191 + (((k - 76) << 6) / 26);
		*b = 255;
	    }
	    else if(k < 127)
	    {
		*r = 0;
		*g = 255;
		*b = 191 + (((127 - k) << 6) / 25);
	    }
	    else if(k < 153)
	    {
		*r = 0;
		*g = 255;
		*b = (191 * (153 - k)) / 26;
	    }
	    else if(k < 178)
	    {
		*r = (191 * (k - 153)) / 26;
		*g = 255;
		*b = 0;
	    }
	    else if(k < 204)
	    {
		*r = 191 + (((k - 178) << 6) / 26);
		*g = 255;
		*b = 0;
	    }
	    else if(k < 229)
	    {
		*r = 255;
		*g = 191 + (((229 - k) << 6) / 25);
		*b = 0;
	    }
	    else if(k < 256)
	    {
		*r = 255;
		*g = (191 * (255 - k)) / 26;
		*b = 0;
	    }

	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapRainbowHLS(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	RGB rgb;
	HLS hls;
	int i = 0;

    /* On part du bleu vers le violet en pasasnt par le vert et rouge */
	const int h0 = 240;
	const float dh = (float) (360. - 60.) / (float)(n - 1);

	for (i = 0; i < n; i++)
	{
	    int h = h0 - i * dh;

	    if (h < 0) h = h + 360;

	    hls.h = h; hls.l = 0.5; hls.s = 1.;
	    rgb = L3HlsToRgb(hls);
	    *r = rgb.r; *g = rgb.g; *b = rgb.b;
	    r++; g++; b++;
	}

}

static void
computecolormapShifted(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	const int ns2 = n / 2;
	int i = 0;

	for (i = 0; i < ns2; i++)
	{
	    const int j = (256 * i) / n + 128;

	    *r = *g = *b = j;
	    r++;
	    g++;
	    b++;
	}

	for (i = ns2; i < n; i++)
	{
	    const int j = (256 * i) / n - 128;

	    *r = *g = *b = j;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapPositive(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	const int ns2 = n / 2;
	int i = 0;

	for (i = 0; i < ns2; i++)
	{
	    *r = *g = *b = (256 * i) / n << 1;
	    r++;
	    g++;
	    b++;
	}

	for (i = ns2; i < n; i++)
	{
	    *r = *g = *b = 0;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapNegative(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	const int ns2 = n / 2;
	int i = 0;

	for (i = 0; i < ns2; i++)
	{
	    *r = *g = *b = 0;
	    r++;
	    g++;
	    b++;
	}

	for (i = ns2; i < n; i++)
	{
	    const int j = ((255 - (256 * i) / n) << 1) + 1;

	    *r = *g = *b = j;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapRedBlue(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    *r = (256 * i) / n;
	    *b = 255 - (256 * i) / n;
	    *g = (int) sqrt((double) (2 * (*r) * (*b)));
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapBlueGreen(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    *r = 0;
	    *g = 255 - (256 * i) / n;
	    *b = (256 * i) / n;
	    *r = 0;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapRedGreen(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    *r = (256 * i) / n;
	    *g = 255 - (256 * i) / n;
	    *b = 0;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormapRandom(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    *r = ALEACOLOR(255);
	    *g = ALEACOLOR(255);
	    *b = ALEACOLOR(255);
	    r++; g++; b++;
	}

}

static void
computecolormapDlpp1(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    register const int j = i % 15;

	    *r++ = dlpp1_def[j].r;
	    *g++ = dlpp1_def[j].g;
	    *b++ = dlpp1_def[j].b;
	}

}

static void
computecolormapXcode(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	int i = 0;

	for (i = 0; i < n; i++)
	{
	    register const int j = i % 32;

	    *r++ = xcode_def[j].r;
	    *g++ = xcode_def[j].g;
	    *b++ = xcode_def[j].b;
	}

}

static void
computecolormapInvDefault(int* red,int* green,int* blue,int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	const int n = nc;
	const int n1 = n-1;
	int i = 0;

	for (; i < n; i++)
	{
	    *r = *g = *b = (256 * (n1-i)) / n;
	    r++;
	    g++;
	    b++;
	}

}

static void
computecolormap(nom_de_carte type, int *red, int *green, int *blue, int nc)
{

	switch (type)
	{
	  case DEFAULT:
	    computecolormapDefault(red,green,blue,nc);
	    break;
	  case ABSOLUTE:
	    computecolormapAbsolute(red,green,blue,nc);
	    break;
	  case BINARY:
	    computecolormapBinary(red,green,blue,nc);
	    break;
	  case RAINBOW:
	    computecolormapRainbow(red,green,blue,nc);
	    break;
	  case RAINBOWHLS:
	    computecolormapRainbowHLS(red,green,blue,nc);
	    break;
	  case SHIFTED:
	    computecolormapShifted(red,green,blue,nc);
	    break;
	  case POSITIVE:
	    computecolormapPositive(red,green,blue,nc);
	    break;
	  case NEGATIVE:
	    computecolormapNegative(red,green,blue,nc);
	    break;
	  case REDBLUE:
	    computecolormapRedBlue(red,green,blue,nc);
	    break;
	  case BLUEGREEN:
	    computecolormapBlueGreen(red,green,blue,nc);
	    break;
	  case REDGREEN:
	    computecolormapRedGreen(red,green,blue,nc);
	    break;
	  case RANDOM:
	    computecolormapRandom(red,green,blue,nc);
	    break;
	  case STANDARDL3:
	    CreeCarteL3(red, green, blue, nc);
	    break;
	  case DLPP1:
	    computecolormapDlpp1(red,green,blue,nc);
	    break;
	  case XCODE:
	    computecolormapXcode(red,green,blue,nc);
	    break;
	  case INVDEFAULT:
	    computecolormapInvDefault(red,green,blue,nc);
	}

}

/* Partie propre au L3 pour y voir clair */
static int rm[8] = {  0,   0, 255, 255, 255, 255,   0, 255};
static int gm[8] = {  0, 255, 255, 128,   0,   0, 255, 255};
static int bm[8] = {255,   0,   0,   0,   0, 255, 255, 255};
/*
   Fonction de conversion qui traduit une couleur et une brillance en un index
   dans la carte de couleurs L3 a 64 entrees
   */

int
cb_to_cmap(int coul, int b)
{
   int index = 0;
   
   if (b == 0)
   {
   /* brillance nulle : On passe en fond inconditionellement */
       index = 0;
   }
   else if ((b == 1024) && (coul == 0))
   {
   /* indication de l'antifond */
       index = 1;
   }
   else
   {
       int c = coul;

       c = (c > 7) ? 8: c;
       c = (c != 0) ? c - 1: 7;
       index = NbColReserved + c * 6 + (int) (5. * b / 2048.);
   }

   return index;
}

static void 
CreeCarteL3(int *red, int *green, int *blue,const int nc)
{
	int* r = red;
	int* g = green;
	int* b = blue;
	register int i;

	for (i = 0; i < nc; i++)
	{
	    register const int j = i % 48;
	    register const int k = j / 6;
	    register const int l = j % 6;

	    RGB rgb; HLS hls;
	    rgb.r = rm[k]; rgb.g = gm[k]; rgb.b = bm[k]; 
	    hls = L3RgbToHls(rgb);
	    hls.l = (5. - (5. - l)) / 5. * 0.5 + 0.30;
	    rgb = L3HlsToRgb(hls);
	    *r = rgb.r; *g = rgb.g; *b = rgb.b;
	    r++; g++; b++;
	}

}
