/*
 * Gestionnaire de la copie. La copie est un fantome de l'image sur lequel on
 * peut faire des marquages pour pouvoir retirer des blocs plus tard. On peut
 * auusi l'afficher sur le driver courant choisi.
 *
 * Dans cette version la copie est une liste doublement chainee de codes L3 qui
 * est conservee en memoire. Dans la plus part des cas ce n'est pas une
 * limitation car les images L3 sont rarement > a 2Mo. Cela evite aussi de
 * devoir gerer des fichiers disques temporaires, plus long d'acces que la
 * RAM. Cela permet surtout de faire des recherches arriere rapides pour
 * eliminer des blocs precedes par un marqueur. (La recherche arriere sur
 * disque est la pire des choses a faire).
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <malloc.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#include "L3Environnement.h"
#include "L3CopieDriver.h"

#ifndef lint
   static const char* const sccs_id = "%W% %G% DMA/AIM/LM";
#endif

#if defined(SUNOS) && defined(SUNOS_4)
   void bcopy(void* b1,void* b2,int length);
#  define memmove(a,b,c) bcopy((b),(a),(c))
#endif

static void AjouteL3(int codel3[]);

typedef struct _ChaineL3 ChaineL3;

struct _ChaineL3
{
	ChaineL3* prec;
	ChaineL3* suiv;
	int* code;
};

static ChaineL3* debut = NULL;
static ChaineL3* fin = NULL;
static int ExisteMarqueur = 0;

/*-
	HasToBeClosed sert quand on est mode copie et que l'on trace une
	deuxieme image alors la premiere est effacee de la copie.
	Si on veut le contenu de la copie elle ne contiendra que
	la derniere image avec son close et n'aura pas ete detruite par
	le close.  (Pour SAGACE)
*/

static int HasToBeClosed = FALSE;

/*---*/

void
L3CopieDriver(int codel3[])
{

   switch (codel3[1]) {
    case CODEL3DIALOGUE:
      switch (codel3[2]) {
       case SOUSCODEL3_MGRAPHCLEAR:
	 HasToBeClosed = TRUE;
       default:;
      }
      break;
    case CODEL3CLOSE:
      /* close retarde pour pouvoir relire la copie */
      HasToBeClosed = TRUE;
      AjouteL3(codel3);
      break;
    case CODEL3SEPARATEUR:
      ExisteMarqueur++;
/*-
				branche des autres cas
*/
    default:
      if (HasToBeClosed) {
	 HasToBeClosed = FALSE;
	 L3DetruitCopie();
      }
      AjouteL3(codel3);
   }
}


static void
AjouteL3(int codel3[])
{
	ChaineL3* ptr;

	if (!debut)
	{
	    debut = (ChaineL3*) malloc(sizeof(ChaineL3));
	    fin = debut;
	    debut->code = (int*) malloc((unsigned) (sizeof(int) * codel3[0]));
	    memmove(debut->code,codel3,sizeof(int)*codel3[0]);
	    debut->prec = NULL;
	    debut->suiv = NULL;
	}
	else
	{
	    ptr = (ChaineL3 *) malloc(sizeof(ChaineL3));
	    ptr->code = (int*) malloc((unsigned) (sizeof(int) * codel3[0]));
	    memmove(ptr->code,codel3,sizeof(int)*codel3[0]);
	    fin->suiv = ptr;
	    ptr->prec = fin;
	    ptr->suiv = NULL;
	    fin = ptr;
	}

}

void
L3DetruitCopie()
{
	ChaineL3* ptr;

	if (!debut) return;

	ptr = fin;

	while (ptr)
	{
	    ChaineL3* prec;

	    free(ptr->code);
	    prec = ptr->prec;
	    free(ptr);
	    ptr = prec;
	}

	debut = NULL;
	ExisteMarqueur = 0;
}

void
L3UndoCopie()
{
   ChaineL3       *ptr, *prec;
   int             lecode;

   if (debut == NULL)
      return;

   if (ExisteMarqueur <= 0)
      return;

   ptr = fin;
   while (ptr != NULL) {
      lecode = ptr->code[1];
      free(ptr->code);
      prec = ptr->prec;

      /* la fin remonte d'un cran */
      fin = prec;
      prec->suiv = NULL;

      free(ptr);
      ptr = prec;
      if (lecode == CODEL3SEPARATEUR)
	 break;
   }
   ExisteMarqueur--;
}

void
L3AfficheCopie()
{
   ChaineL3       *ptr;

   if (debut == NULL)
      return;

   L3SetModeCopie(FALSE);
   
   /* fermeture de l'image en cours */
   L3AppelDrivers(codel3close);
   
   ptr = debut;
   while (ptr != NULL) {
      L3AppelDrivers(ptr->code);
      ptr = ptr->suiv;
   }
   
   L3SetModeCopie(TRUE);
}

void
L3SetSeparateur()
{
   int             codel3[3];

   codel3[0] = 2;
   codel3[1] = CODEL3SEPARATEUR;
   L3AppelDrivers(codel3) /* inscrit un separateur dans l'image */ ;
   L3FlushBuffers();
}

/* Fin de fichier */
