/****************/
/* L3DefSpace.c */
/****************/


#include <stdio.h>
#include "L3Environnement.h"
#include "L3DefSpace.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% G. Colin de Verdiere";
#endif

/*******************************************************************************
 * Procedure utilisateur : L3DefSpace			      	               *
 *-----------------------------------------------------------------------------*
 * Parametres :							      	       *
 *   sousimage : numero de la sous-image concernee			       *
 *   xu,yu     : abcisse et ordonnee minima de l'espace utilisateur	       *
 *                 valeurs par defaut a l'initialisation :                     *
 *                         0.0 pour xu et 0.0 pour yu                          *
 *   xv,yv     : abcisse et ordonnee maxima de l'espace utilisateur	       *
 *                 valeurs par defaut a l'initialisation :                     *
 *                         1.0 pour xv et 1.0 pour yv                          *
 *							       		       *
 * Valeur de retour : aucune						       *
 *									       *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987        *
 *             "GRAF CIGALE"                                                   *
 *                                                                             *
 * Erreurs :                                                                   *
 *   espace de coordonnees incorrect                                           *
 *******************************************************************************/

void
L3DefSpace(int sousimage, REEL xu, REEL yu, REEL xv, REEL yv)
{

	if ((xu < xv) && (yu < yv)) 
	{
		L3DefSpaceLog(sousimage, xu, yu, xv, yv, FAUX, FAUX);
	}
	else
	{
		/*-------------------------------------------*/
		/* Erreur : espace de coordonnees incorrect. */
		/*-------------------------------------------*/

		fprintf(stderr,"L3DefSpace : ");
		fprintf(stderr,"espace de coordonnees incorrect.\n");
	}

}

/*******************************************************************************
 * Procedue interne : L3DefSpaceLog				               *
 *-----------------------------------------------------------------------------*
 * Parametres :							               *
 *   sousimage : numero de la sous-image concernee		               *
 *   xu,yu     : abcisse et ordonnee minima de l'espace utilisateur	       *
 *                 valeurs par defaut a l'initialisation :                     *
 *                       0.0 pour xu et 0.0 pour yu                            *
 *   xv,yv     : abcisse et ordonnee maxima de l'espace utilisateur	       *
 *                 valeurs par defaut a l'initialisation :                     *
 *                       1.0 pour xv et 1.0 pour yv                            *
 *   xcoordlog : indicateur de mode logarithmique en X		       	       *
 *   ycoordlog : indicateur de mode logarithmique en Y                         *
 *                                                                             *
 * Valeur de retour : aucune						       *
 *									       *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987        *
 *                  "GRAF CIGALE"                                              *
 *                                                                             *
 * Erreurs :                                                                   *
 *   espace de coordonnees incorrect                                           *
 *******************************************************************************/

void
L3DefSpaceLog(int sousimage, REEL xu, REEL yu, REEL xv, REEL yv,
		int xcoordlog, int ycoordlog)
{

	if ((xu < xv) && (yu < yv)
			&& ((xcoordlog == FAUX) ||
					((xcoordlog == VRAI) && (xu > 0.0)))
			&& ((ycoordlog == FAUX) ||
					((ycoordlog == VRAI) && (yu > 0.0))))
	{
	    SETSOUSIMAGE(sousimage);
	    XU = xu;
	    YU = yu;
	    XV = xv;
	    YV = yv;
	    XCOORDLOG = xcoordlog;
	    YCOORDLOG = ycoordlog;
	    L3CalculConvCoord();
	    VISU3D = FAUX;
	}
	else
	{
	/*-------------------------------------------*/
	/* Erreur : espace de coordonnees incorrect. */
	/*-------------------------------------------*/
	    fprintf(stderr, "L3DefSpaceLog : "
			    "espace de coordonnees incorrect.\n");
	}

}
