#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include "U8vararg.h"
#include "L3Formules.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% J.P. Nomine";
#endif

static char modnormal[2]	= {MODNORMAL,'\0'};
static char modtexte[2]		= {MODTEXTE,'\0'};
static char modgraphique[2]	= {MODGRAPHIQUE,'\0'};
static char modtextlocal[2]	= {MODTEXTLOCAL,'\0'};
static char modgraphlocal[2]	= {MODGRAPHLOCAL,'\0'};
static char modindice[2]	= {MODINDICE,'\0'};
static char modexposant[2]	= {MODEXPOSANT,'\0'};
static char modindiceinf[2]	= {MODINDICEINF,'\0'};
static char modexposantsup[2]	= {MODEXPOSANTSUP,'\0'};
static char modreduit[2]	= {MODREDUIT,'\0'};
static char modagrandi[2]	= {MODAGRANDI,'\0'};
static char moddecaleinf[2]	= {MODDECALEINF,'\0'};
static char moddecalesup[2]	= {MODDECALESUP,'\0'};
static char modrecul[2]		= {MODRECUL,'\0'};
static char modindprec[2]	= {MODINDPREC,'\0'};
static char modindprecrecul[2]	= {MODINDPRECRECUL,'\0'};
static char modnul[1]		= {'\0'};

FORMULE GenFormule(const int nombre, const FORMULE formule1, ...)
{
	FORMULE	result = NULL;
	FORMULE terme;
	int longueur, i;
	va_list	liste_arg;

	if (nombre < 1)
	{
	    printf("GenFormule : nombre doit etre > 1...\n");
	}
	else
	{
	    longueur = 1 + strlen(formule1);
	    va_start(liste_arg, formule1);

	    for (i=0; i<nombre-1; i++)
	    {
		terme = va_arg(liste_arg, FORMULE);
		longueur += strlen(terme);
	    }

	    va_end(liste_arg);
	    result = (FORMULE) malloc(longueur);
	    strcpy(result, formule1);
	    va_start(liste_arg, formule1);

	    for (i=0; i<nombre-1; i++)
	    {
		terme = va_arg(liste_arg, FORMULE);
		strcat(result, terme);
	    }

	    va_end(liste_arg);
	}
	
	return result;
}

FORMULE GenControle(const int mode)
{
 char* SequenceControle;
 
 switch(mode) {
    case MODNORMAL:
	SequenceControle = modnormal;
	break;
    case MODTEXTE:
	SequenceControle = modtexte;
	break;
    case MODGRAPHIQUE:
	SequenceControle = modgraphique;
	break;
    case MODTEXTLOCAL:
	SequenceControle = modtextlocal;
	break;
    case MODGRAPHLOCAL:
	SequenceControle = modgraphlocal;
	break;
    case MODINDICE:
	SequenceControle = modindice;
	break;
    case MODEXPOSANT:
	SequenceControle = modexposant;
	break;
    case MODINDICEINF:
	SequenceControle = modindiceinf;
	break;
    case MODEXPOSANTSUP:
	SequenceControle = modexposantsup;
	break;
    case MODREDUIT:
	SequenceControle = modreduit;
	break;
    case MODAGRANDI:
	SequenceControle = modagrandi;
	break;
    case MODDECALEINF:
	SequenceControle = moddecaleinf;
	break;
    case MODDECALESUP:
	SequenceControle = moddecalesup;
	break;
    case MODRECUL:
	SequenceControle = modrecul;
	break;
    case MODINDPREC:
	SequenceControle = modindprec;
	break;
    case MODINDPRECRECUL:
	SequenceControle = modindprecrecul;
	break;
    default:
        printf("GenControle : mode invalide = %o (octal)\n", mode);
	SequenceControle = modnul;
	break;
 }

 return SequenceControle;
}

FORMULE
GenBloc(const int mode, const FORMULE formule)
{
	FORMULE result;
	char modfin[3];
 
	switch(mode)
	{
	  case MODNORMAL:
	  case MODTEXTE:
	    result = (FORMULE)malloc(2+strlen(formule));
	    result[0] = (char)mode;
	    strcpy(result+1, formule);
	    break;
	  case MODGRAPHIQUE:
	    result = (FORMULE)malloc(3+strlen(formule));
	    result[0] = (char)mode;
	    strcpy(result+1, formule);
	    modfin[0] = (char)MODTEXTE;
	    modfin[1] = '\0';
	    strcat(result, modfin);
	    break;
	  case MODINDICE:
	  case MODEXPOSANT:
	  case MODINDICEINF:
	  case MODEXPOSANTSUP:
	  case MODREDUIT:
	  case MODAGRANDI:
	  case MODDECALEINF:
	  case MODDECALESUP:
	    result = (FORMULE)malloc(3+strlen(formule));
	    result[0] = (char)mode;
	    strcpy(result+1, formule);
	    modfin[0] = (char)MODNORMAL;
	    modfin[1] = '\0';
	    strcat(result, modfin);
	    break;
	  default:
	    printf("GenBloc : mode invalide = %o (octal)\n", mode);
	    result = (FORMULE)malloc(1+strlen(formule));
	    strcpy(result, formule);
	    break;
	}

	return result;
}

/* fonction interne */
static FORMULE
GenHabille (const type_habillage habillage, const FORMULE formule)
{
	FORMULE         result = NULL;
	static char     debut_habillage[3];
	static char     fin_habillage[2] = {MODFINHAB, '\0'};

	if (!formule || (strlen(formule) == 0))
	{
	    fprintf(stderr, "GenHabille : formule nulle ou vide\n");
	}
	else
	{
	    debut_habillage[0] = MODDEBHAB;
	    debut_habillage[1] = (char)habillage;
	    debut_habillage[2] = '\0';
	    result = (FORMULE ) malloc((4 + strlen(formule)) * sizeof(char));
	    strcpy(result, debut_habillage);
	    strcat(result, formule);
	    strcat(result, fin_habillage);
	}

	return result;
}

/*
 Fonction utilisateur : GenRacine
 
 Description : passe une formule en radicande

 radicande : pointeur sur la formule radicande 
 
 resultat : pointeur sur la chaine encapsulee
*/

FORMULE              
GenRacine(const FORMULE radicande)
{
	const type_habillage habillage = RACINE;
   
	return GenHabille(habillage, radicande);
}

/*
 Fonction utilisateur : GenFraction
 
 Description : fabrique une fraction a partir d'un numerateur
               et d'un denominateur

 numerateur   : pointeur sur la formule numerateur 
 denominateur : pointeur sur la formule denominateur 
 
 resultat : pointeur sur la chaine encapsulee
*/

FORMULE              
GenFraction(const FORMULE numerateur, const FORMULE denominateur)
{
	FORMULE         result = NULL;
	static char     num_den[2] = {MODNUMDEN, '\0'};
	static char     fin_fraction[2] = {MODFINFRACTION, '\0'};

	if(!numerateur || !strlen(numerateur))
	{
	    fprintf(stderr, "GenFraction : numerateur nul ou vide\n");
	}
	else if(!denominateur || !strlen(denominateur))
	{
	    fprintf(stderr, "GenFraction : denominateur nul ou vide\n");
	}
	else
	{
	    result = (FORMULE ) malloc((3 + strlen(numerateur) +
			    strlen(denominateur)) * sizeof(char));
	    result[0] = MODFRACTION;
	    strcpy(result + 1, numerateur);
	    strcat(result, num_den);
	    strcat(result, denominateur);
	    strcat(result, fin_fraction);
	}

	return result;
}

/*
 Fonction utilisateur : GenVecteur
 
 Description : ajoute une fleche au-dessus de la formule

 vecteur : pointeur sur la formule a habiller 
 
 resultat : pointeur sur la chaine encapsulee
*/
FORMULE              
GenVecteur(const FORMULE vecteur)
{
   const type_habillage habillage = VECTEUR;
   
   return GenHabille (habillage, vecteur);
}

/*
 Fonction utilisateur : GenSouligne
 
 Description : ajoute une barre au-dessous de la formule

 formule : pointeur sur la formule a habiller 
 
 resultat : pointeur sur la chaine encapsulee
*/
FORMULE              
GenSouligne(const FORMULE formule)
{
   const type_habillage habillage = SOULIGNE;
   
   return GenHabille (habillage, formule);
}

/*
 Fonction utilisateur : GenSurligne
 
 Description : ajoute une barre au-dessus de la formule

 formule : pointeur sur la formule a habiller 
 
 resultat : pointeur sur la chaine encapsulee
*/
FORMULE              
GenSurligne(const FORMULE formule)
{
   const type_habillage habillage = SURLIGNE;
   
   return GenHabille (habillage, formule);
}

FORMULE GenParentheses(
		const FORMULE		formule)
{
   const type_habillage habillage = PARENTHESE;
   
   return GenHabille (habillage, formule);
}

FORMULE GenAccolades(
		const FORMULE		formule)
{
   const type_habillage habillage = ACCOLADE;
   
   return GenHabille (habillage, formule);
}

FORMULE GenCrochets(
		const FORMULE		formule)
{
   const type_habillage habillage = CROCHET;
   
   return GenHabille (habillage, formule);
}

FORMULE GenCrochetsCarres(
		const FORMULE		formule)
{
   const type_habillage habillage = CROCHET_CARRE;
   
   return GenHabille (habillage, formule);
}

/* fonction interne */
FORMULE GenIntegre(
		const FORMULE		borne_inf,
		const FORMULE		borne_sup,
		const FORMULE		terme,
		const type_encapsulage	encapsulage,
		const rang_integrale	rang)
{
 FORMULE	result;
 static char	sep[2] = {MODSEPINTEG, '\0'};
 static char    fin[2] = {MODFININTEG, '\0'};
 static char    debut[4];
 
 debut[0] = MODDEBINTEG;
 debut[1] = (char)encapsulage;
 debut[2] = (char)rang;
 debut[3] = '\0';
 
 result = (FORMULE)malloc( ( 7 +
 		strlen(borne_inf)+strlen(borne_sup)+strlen(terme) )
 		* sizeof (char));
 (void)strcpy(result, debut);
 (void)strcat(result, borne_inf);
 (void)strcat(result, sep);
 (void)strcat(result, borne_sup);
 (void)strcat(result, sep);
 (void)strcat(result, terme);
 (void)strcat(result, fin);
 return result;
}

FORMULE GenIntegrale(
		const FORMULE		borne_inf,
		const FORMULE		borne_sup,
		const FORMULE		integrande)
{
 const	type_encapsulage	symbole	= INTEGRALE;
 const  rang_integrale		rang	= SIMPLE;

 return GenIntegre(
 				borne_inf,
 				borne_sup,
 				integrande,
 				symbole,
 				rang);
}

FORMULE GenSomme(
		const FORMULE		borne_inf,
		const FORMULE		borne_sup,
		const FORMULE		terme)
{
 const	type_encapsulage	symbole	= SOMME;
 const  rang_integrale		rang	= SIMPLE;

 return GenIntegre(
 				borne_inf,
 				borne_sup,
 				terme,
 				symbole,
 				rang);
}

FORMULE GenProduit(
		const FORMULE		borne_inf,
		const FORMULE		borne_sup,
		const FORMULE		terme)
{
 const	type_encapsulage	symbole	= PRODUIT;
 const  rang_integrale		rang	= SIMPLE;

 return GenIntegre(
 				borne_inf,
 				borne_sup,
 				terme,
 				symbole,
 				rang);
}


/* fin de fichier L3Formules.c */
