/*
 * Module interface qui permet d'appeler le C depuis FORTRAN
 */

/* Auteur G. Colin de Verdiere DMA/SIE/IG 11/24/92 */


/*-
	####### ####### ######  #     #   ###   #     #    #    #
	   #    #       #     # ##   ##    #    ##    #   # #   #
	   #    #       #     # # # # #    #    # #   #  #   #  #
	   #    #####   ######  #  #  #    #    #  #  # #     # #
	   #    #       #   #   #     #    #    #   # # ####### #
	   #    #       #    #  #     #    #    #    ## #     # #
	   #    ####### #     # #     #   ###   #     # #     # #######
*/

#ifndef lint
    static const char* const sccs_id = "%W% %G% G. Colin de Verdiere";
#endif

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#ifdef CRAY
#include <fortran.h>
#endif

#include "L3Environnement.h"
#include "L3Ftn.h"

#if defined(SUNOS) && defined(SUNOS_4)
    void bcopy(void* b1,void* b2,int length);
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

/* --------------------------------- MALPHA -------------------------- */
/*
 * void malpha(void) passe en mode alphanumerique sans clear
 */

void
#ifdef CRAY_PROTO
MALPHA()
#else
FUNCDECL(malpha)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "MALPHA\n");
   }
   L3ModeAlphanum();
}

/* --------------------------------- MGRAPH -------------------------- */
/*-
 *	void mgraph(int *i)
 *		i = 0 : passe en mode graphique sans clear
 *		i > 0 : passe en mode graphique sur l'ecran NeGS "i"
 *		Attention Pour creer le deuxieme ecran Negs on prend
 *		directement le numero de port socket suivant de celui qui
 *		a servi pour l'ecran de base.
 */
void
#ifdef CRAY_PROTO
MGRAPH(i)
#else
FUNCDECL(mgraph)(i)
#endif
   int            *i;
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "MGRAPH");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *i);
      }
      fprintf(FicTrace, "\n");
   }
   if (*i > 0)
      L3NegsEcran(*i - 1);
   if (*i == 0)
      L3ModeGraphique(*i);
}

/* --------------------------------- FLUSHIM -------------------------- */
/*
 * void flushim(void) force le vidage du buffer et transmission de cet ordre
 * a l'afficheur
 */
void
#ifdef CRAY_PROTO
FLUSHIM()
#else
FUNCDECL(flushim)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "FLUSHIM\n");
   }
   L3FlushBuffers();
}

/* --------------------------------- VIDTEK0 -------------------------- */
/*
 * void vidtek0(void) force le vidage du buffer et transmission de cet ordre
 * a l'afficheur
 */
void
#ifdef CRAY_PROTO
VIDTEK0()
#else
FUNCDECL(vidtek0)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "VIDTEK0\n");
   }
   L3FlushBuffers();
}

/* --------------------------------- VIDTEK -------------------------- */
/*
 * void vidtek(int i) i vaut 0 (zero) force le vidage du buffer et
 * transmission de cet ordre a l'afficheur
 */

void
#ifdef CRAY_PROTO
VIDTEK(i)
#else
FUNCDECL(vidtek)(i)
#endif
   int            *i;
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "VIDTEK");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *i);
      }
      fprintf(FicTrace, "\n");
   }
   L3FlushBuffers();
}

/* --------------------------------- EFFACE -------------------------- */
/*
 * */
void
#ifdef CRAY_PROTO
EFFACE()
#else
FUNCDECL(efface)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "EFFACE\n");
   }
   L3SautDePage();
}

/* --------------------------------- S3EFAC -------------------------- */
/*
 * void s3efac(void) efface l'ecran
 */
void
#ifdef CRAY_PROTO
S3EFAC()
#else
FUNCDECL(s3efac)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "S3EFAC\n");
   }
   L3SautDePage();
}

/* --------------------------------- CHTERM -------------------------- */
/*
 * void chterm(void) choix du terminal
 */

static char    *periph[] = {
   "sun",
   "negs",
   "pericom",
   "falco",
   "xterm",
   "tektronix",
   "t4014"
};

/* stockage des attributs du terminal selectionne */

static char     nomterm[17];
/*-
	nb de lignes en graphique : Nblgr
	nb de lignes en alphanum : Nblal
	nb de page gerables : Nbpag
	possibilite VT100 : Ivt100
*/

static int      Nblgr, Nblal, Nbpag, Ivt100;

/*---------------------------------- QTERM -------------------------- */
/*-
 * Donne les possibilites du terminal (edition plein ecran ...)
      entry qterm(nomt,nblgr,nblal,nbpag,ivt100)
c
c     permet de connaitre le nom du terminal choisi
c  selon la reponse donnee a CHTERM , ou le dernier appel de DTERM
c     donne aussi le nb. de pages ( 2 si alpha+graph , 1 sinon )
c  le nb. de lignes sur la page graphique, et sur la page alpha
c  la possibilite de VT100 ( 0=pas ,2=avec Answerback, 1=sans)
 */

void
#ifdef CRAY_PROTO
QTERM(nomt, nblgr, nblal, nbpag, ivt100)
   _fcd            nomt;
#else
FUNCDECL(qterm)(nomt, nblgr, nblal, nbpag, ivt100, lgt)
   char           *nomt;
   int             lgt;
#endif
   int            *ivt100, *nblgr, *nblal, *nbpag;
{
#ifdef CRAY_PROTO
   char           *origine = _fcdtocp(nomt);	/* pointeur sur la chaine FORTRAN */
   int             lgt = _fcdlen(nomt);	/* longueur de la chaine FORTRAN */
#else
   char           *origine = nomt;
#endif
   int             i;

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "QTERM");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %s %d %d %d %d",
		 nomterm, Ivt100, Nblgr, Nblal, Nbpag);
      }
      fprintf(FicTrace, "\n");
   }
   if (lgt <= 0) {
      fprintf(stderr, "Sanglier QTERM n'utilisez pas de Hollerith\n");
      lgt = 4;
   }
   if (lgt < (int) strlen(nomterm)) {
      fprintf(stderr, "Sanglier QTERM longueur de nom trop petite\n");
   }
   strcpy(origine, nomterm);

   for (i = strlen(nomterm); i < lgt; i++)
      origine[i] = ' ';

   *ivt100 = Ivt100;
   *nblgr = Nblgr;
   *nblal = Nblal;
   *nbpag = Nbpag;
}

static void
driverchoisi(texte)
   char           *texte;
{
   strcpy(nomterm, "AUTRE");
   Nblgr = 50;
   Nblal = 24;
   Nbpag = 1;
   Ivt100 = 0;

   if (strncmp(texte, "su", 2) == 0) {	/* SUN */
      strcpy(texte, "negs");

      strcpy(nomterm, "SUN");
      Nblgr = 50;
      Nblal = 24;
      Nbpag = 3;
      Ivt100 = 2;
      L3ChoixDriver(texte);
      return;
   }
   if (strncmp(texte, "ne", 2) == 0) {	/* NEGS */
      strcpy(texte, "negs");
      strcpy(nomterm, "SUN");
      Nblgr = 50;
      Nblal = 24;
      Nbpag = 3;
      Ivt100 = 2;
      L3ChoixDriver(texte);
      return;
   }
   if (strncmp(texte, "pe", 2) == 0) {	/* PERICOM */
      strcpy(texte, "T4014");
      strcpy(nomterm, "PERICOM");
      Nblgr = 59;
      Nblal = 24;
      Nbpag = 2;
      Ivt100 = 2;
      L3ChoixDriver(texte);
      return;
   }
   if (strncmp(texte, "fa", 2) == 0) {	/* FALCO */
      strcpy(texte, "T4014");
      strcpy(nomterm, "FALCO");
      Nblgr = 35;
      Nblal = 23;
      Nbpag = 2;
      Ivt100 = 1;
      L3ChoixDriver(texte);
      return;
   }
   if (strncmp(texte, "te", 2) == 0) {	/* TEKTRONIX */
      strcpy(texte, "T4014");
      strcpy(nomterm, "TEKTRO");
      Nblgr = 124;
      Nblal = 124;
      Nbpag = 1;
      Ivt100 = 0;
      L3ChoixDriver(texte);
      return;
   }
   if (strncmp(texte, "xt", 2) == 0) {	/* XTERM */
      strcpy(texte, "T4014");
      strcpy(nomterm, "XTERM");
      Nblgr = 59;
      Nblal = 24;
      Nbpag = 2;
      Ivt100 = 1;
      L3ChoixDriver(texte);
      return;
   }
}

void
#ifdef CRAY_PROTO
CHTERM()
#else
FUNCDECL(chterm)()
#endif
{
   int i;
   char nom[256];
   int nbdrivers = sizeof(periph) / sizeof(char *);


   if (L3DebugFlag == FALSE) {
      L3DebugFlag = (getenv("L3DEBUG") != NULL);

      if (L3DebugFlag) {
	 L3DebugFlag = atoi(getenv("L3DEBUG"));
	 if (L3DebugFlag == 0)
	    L3DebugFlag = 1;
	 fprintf(stderr, "Mode trace de Sanglier actif niveau %d\n",
		 L3DebugFlag);
	 FicTrace = fopen("l3.trace", "a");
	 fprintf(FicTrace, "Begin trace L3 ......\n");
      }
   }

   printf("Choix de votre terminal ? \n");
   printf("Vous pouvez choisir entre exit (ou ^D) et : \n");

   for (i = 0; i < nbdrivers; i++) {
      printf("\t%s\n", periph[i]);
   }

   while (!feof(stdin)) {

      printf("Votre reponse : ");
      fflush(stdout);

      fgets(nom, 256, stdin);

      for (i = 0; i < (int) strlen(nom); i++)
	 nom[i] = tolower(nom[i]);

      if (strncmp(nom, "ex", 2) == 0) {
	 fprintf(stderr, " Abandon Sanglier CHTERM sur exit\n");
	 exit(1);
      }

      for (i = 0; i < nbdrivers; i++) {
	 if (strncmp(nom, periph[i], 2) == 0) {
	    fprintf(stderr, " Driver %s choisi\n", periph[i]);
	    driverchoisi(nom);
	    return;
	 }
      }

   }

   fprintf(stderr, " Abandon Sanglier CHTERM sur EOF\n");
   exit(1);
}

/* --------------------------------- DTERM -------------------------- */
/*
 * Impose le choix du terminal
 */
void
#ifdef CRAY_PROTO
DTERM(t)
   _fcd            t;
#else
FUNCDECL(dterm)(t, l)
   char           *t;
   int             l;
#endif
{
#ifdef CRAY_PROTO
   int             lg = _fcdlen(t);	/* taille donnee par Fortran */
   char           *origine = _fcdtocp(t);	/* pointeur sur la chaine FORTRAN */
#else
   int             lg = l;	/* taille donnee par Fortran */
   char           *origine = t;	/* pointeur sur la chaine FORTRAN */
#endif
   char           *texte;
   unsigned        i = 0;

   if (L3DebugFlag == FALSE) {
      L3DebugFlag = (getenv("L3DEBUG") != NULL);

      if (L3DebugFlag) {
	 L3DebugFlag = atoi(getenv("L3DEBUG"));
	 if (L3DebugFlag == 0)
	    L3DebugFlag = 1;
	 fprintf(stderr, "Mode trace de Sanglier actif niveau %d\n",
		 L3DebugFlag);
	 FicTrace = fopen("l3.trace", "a");
	 fprintf(FicTrace, "Begin trace L3 ......\n");
      }
   }
   if (lg == 0) {
      /* Horreur c'est un Hollerith ou une erreur */
      fprintf(stderr,
	      "Sanglier DTERM : N'utilisez pas de Hollerith \n");
   }
   if (lg < 16)
      lg = 16;
   texte = malloc((unsigned) (lg + 1));
   if (texte == NULL) {
      fprintf(stderr, "*******************************************************\n");
      fprintf(stderr, "* DTERM allocation memoire impossible : plus de place \n");
      fprintf(stderr, "*******************************************************\n");
      abort();
   }

   memmove(texte,origine, lg);
   texte[lg] = '\0';

   /* passe le texte en minuscule */
   for (i = 0; i < strlen(texte); i++)
      if (isupper((int) texte[i]))
	 texte[i] = tolower(texte[i]);

   driverchoisi(texte);

   free(texte);
}

/* --------------------------------- DESTIN -------------------------- */
/*
 * void destin(char *) Choix de la destination des images
 */
void
#ifdef CRAY_PROTO
DESTIN(_fcd t)
#else
FUNCDECL(destin)(t, l)
   char           *t;
   int             l;
#endif
{
#ifdef CRAY_PROTO
   int             lg = _fcdlen(t);	/* taille donnee par Fortran */ 
   char           *origine = _fcdtocp(t);	/* pointeur sur la chaine FORTRAN */
#else
   int             lg = l;	/* taille donnee par Fortran */
   char           *origine = t;	/* pointeur sur la chaine FORTRAN */
#endif
   char            texte[256];
   unsigned        i = 0;


   if (L3DebugFlag >= 1) {
#ifdef CRAY_PROTO
	malloc_check(0);
#endif
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   
   if (L3DebugFlag == FALSE) {
      L3DebugFlag = (getenv("L3DEBUG") != NULL);

      if (L3DebugFlag) {
	 L3DebugFlag = atoi(getenv("L3DEBUG"));
	 if (L3DebugFlag == 0)
	    L3DebugFlag = 1;
	 fprintf(stderr, "Mode trace de Sanglier actif niveau %d\n",
		 L3DebugFlag);
	 FicTrace = fopen("l3.trace", "a");
	 fprintf(FicTrace, "Begin trace L3 ......\n");
      }
   }

   if (lg == 0) {
      /* Horreur c'est un Hollerith ou une erreur */
      fprintf(stderr,
	      "Sanglier DESTIN : N'utilisez pas de Hollerith \n");
#ifdef CRAY_PROTO
      lg = 8;
#else
      lg = 4;
#endif
   }
   
   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   
   memmove(texte,origine, lg);
   
   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   texte[lg] = '\0';

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "DESTIN");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %s", texte);
      }
      fprintf(FicTrace, "\n");
      fflush(FicTrace);
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   
   /* passe le texte en minuscule */
   for (i = 0; i < strlen(texte); i++)
      if (isupper((int) texte[i]))
	 texte[i] = tolower(texte[i]);
	 
   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
	
   if (!strncmp(texte, "su", 2)) {	/* SUN */
      L3ChoixDriver("negs");
   }

   if (!strncmp(texte, "ne", 2)) {	/* NeGS */
      L3ChoixDriver("negs");
   }

   if (!strncmp(texte, "vi", 2))	/* VISU */
      L3ChoixDriver("visu");

   if (!strncmp(texte, "co", 2))	/* COPIE */
      L3SetModeCopie(TRUE);

   if (!strncmp(texte, "po", 2))	/* POSTSCRIPT */
      L3ChoixDriver("post");

   if (!strncmp(texte, "ps", 2))	/* POSTSCRIPT */
      L3ChoixDriver("post");
      
   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   
   /* --- */
   
   {
      int             done = FALSE;
      if (!done && (strncmp(texte, "lwct", 4) == 0)) {	/* transp couleur */
	 L3ChoixDriver("lwct");
	 done = TRUE;
      }
      if (!done && (strncmp(texte, "lwc", 3) == 0)) {	/* papier couleur */
	 L3ChoixDriver("lwc");
	 done = TRUE;
      }
      if (!done && (strncmp(texte, "lw", 2) == 0)) {	/* papier N&B */
	 L3ChoixDriver("post");
	 done = TRUE;
      }
   }
   
/* --- Autres destinations ancienne mode NOS/VE => VISU */

   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
   if (strncmp(texte, "di", 2) == 0)	/* DIAPO */
      L3ChoixDriver("visu");
   if (strncmp(texte, "tr", 2) == 0)	/* TRAC */
      L3ChoixDriver("visu");
   if (strncmp(texte, "im", 2) == 0)	/* IMAGE */
      L3ChoixDriver("visu");
   if (strncmp(texte, "el", 2) == 0)	/* ELEC */
      L3ChoixDriver("visu");
   if (strncmp(texte, "gd", 2) == 0)	/* GDIAPO */
      L3ChoixDriver("visu");
   if (strncmp(texte, "du", 2) == 0)	/* DUMP */
      L3ChoixDriver("visu");

   if (L3DebugFlag >= 1) {
	fprintf(stderr, "SANGLIER : %d %s\n", __LINE__, __FILE__);
   }
}

/* --------------------------------- FINDES -------------------------- */
/*
 * annule un destination
 */
void
#ifdef CRAY_PROTO
FINDES(t)
   _fcd            t;
#else
FUNCDECL(findes)(t, l)
   char           *t;
   int             l;
#endif
{
#ifdef CRAY_PROTO
   int             lg = _fcdlen(t);	/* taille donnee par Fortran */
   char           *origine = _fcdtocp(t);	/* pointeur sur la chaine FORTRAN */
#else
   int             lg = l;	/* taille donnee par Fortran */
   char           *origine = t;	/* pointeur sur la chaine FORTRAN */
#endif
   char           *texte;
   unsigned        i = 0;

   if (lg == 0) {
      /* Horreur c'est un Hollerith ou une erreur */
      fprintf(stderr,
	      "Sanglier FINDES : N'utilisez pas de Hollerith \n");
   }
   if (lg < 16)
      lg = 16;
   texte = malloc((unsigned) (lg + 1));
   if (texte == NULL) {
      fprintf(stderr, "*******************************************************\n");
      fprintf(stderr, "* FINDES allocation memoire impossible : plus de place \n");
      fprintf(stderr, "*******************************************************\n");
      abort();
   }

   memmove(texte,origine, lg);
   texte[lg] = '\0';

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "FINDES");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %s", texte);
      }
      fprintf(FicTrace, "\n");
   }
   
   /* Termine l'image courante */
   L3SautDePage();
   
   /* passe le texte en minuscule */
   for (i = 0; i < strlen(texte); i++)
      if (isupper((int) texte[i]))
	 texte[i] = tolower(texte[i]);

   if (!strncmp(texte, "su", 2)) {	/* SUN */
      strcpy(texte, "negs");
      L3FinChoixDriver(texte);
   }

   if (!strncmp(texte, "ne", 2)) {	/* NeGS */
      L3FinChoixDriver(texte);
   }

   if (!strncmp(texte, "vi", 2))	/* VISU */
      L3FinChoixDriver("visu");

   if (!strncmp(texte, "co", 2))	/* COPIE */
      L3SetModeCopie(FALSE);

   if (!strncmp(texte, "po", 2))	/* POSTSCRIPT */
      L3FinChoixDriver("post");

   if (!strncmp(texte, "ps", 2))	/* POSTSCRIPT */
      L3FinChoixDriver("post");

   {
      int             done = FALSE;
      if (!done && (strncmp(texte, "lwct", 4) == 0)) {	/* transp couleur */
	 L3FinChoixDriver("lwct");
	 done = TRUE;
      }
      if (!done && (strncmp(texte, "lwc", 3) == 0)) {	/* papier couleur */
	 L3FinChoixDriver("lwc");
	 done = TRUE;
      }
      if (!done && (strncmp(texte, "lw", 2) == 0)) {	/* papier N&B */
	 L3FinChoixDriver("post");
	 done = TRUE;
      }
   }

   free(texte);
}

/* --------------------------------- DECRAN -------------------------- */
/*
 * void decran(int i) type de mise en page de l'ecran l'ecran fait 0-1023,
 * 0-1023 i=1 espace carre utilise X:239-1017, Y:0-778 i=2 espace carre
 * utilise X:0-1023, Y:0-1023 i=3 espace carre tronque X:0-1023, Y:0-778
 */
void
#ifdef CRAY_PROTO
DECRAN(i)
#else
FUNCDECL(decran)(i)
#endif
   int            *i;
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "DECRAN");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *i);
      }
      fprintf(FicTrace, "\n");
   }
   L3FenetreEcran(*i);
}

/* --------------------------------- CLOCHE -------------------------- */
/*
 * void cloche(void) emet un bip au terminal
 */
void
#ifdef CRAY_PROTO
CLOCHE()
#else
FUNCDECL(cloche)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "CLOCHE");
      fprintf(FicTrace, "\n");
   }
   fprintf(stderr, "\007");
   fflush(stderr);
}

/* Fin de Fichier */
