/*******************/
/* L3LigneBrisee.c */
/*******************/

#include <stdio.h>

#include "L3Environnement.h"
#include "L3LigneBrisee.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/**************************************************************************************
 * Procedure utilisateur : L3LigneBrisee                                              *
 *------------------------------------------------------------------------------------*
 * Parametres :                                                                       *
 *   sousimage : numero de la sous-image concernee                                    *
 *   x         : tableau des abcisses de la ligne brisee dans l'espace utilisateur    *
 *   y         : tableau des ordonnees de la ligne brisee dans l'espace utilisateur   *
 *   nlignes   : nombre de segments de la ligne brisee                                *
 *                                                                                    *
 * Valeur de retour : aucune                                                          *
 *                                                                                    *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987 "GRAF CIGALE" *
 *                                                                                    *
 * Erreurs :                                                                          *
 *   nombre de lignes negatif ou nul                                                  *
 **************************************************************************************/

void
L3LigneBrisee(sousimage, x, y, nlignes)
	int             sousimage;
	REEL            x[], y[];
	int             nlignes;

{
	int             i;
	int             xpage1, ypage1, xpage2, ypage2;
	int             codel3[6];
	int             ligneatracer;

	if (nlignes > 0) {

		codel3[0] = 6;
		codel3[1] = CODEL3SEGMENT;
		SETSOUSIMAGE(sousimage);
		xpage1 = XPAGECOURANT;
		ypage1 = YPAGECOURANT;

		for (i = 0; i < nlignes; i++) {
			L3ConversionCoordonnees(x[i], y[i], &xpage2, &ypage2);
			ligneatracer = L3ClippingSegment(xpage1, ypage1, xpage2, ypage2, &codel3[2]);
			if (ligneatracer == VRAI) {
				L3AppelDrivers(codel3);
				XPAGECOURANT = xpage2;
				YPAGECOURANT = ypage2;
			}
			xpage1 = xpage2;
			ypage1 = ypage2;
		}

	} else {
		/*-------------------------------------------*/
		/* Erreur : Nombre de lignes negatif ou nul. */
		/*-------------------------------------------*/
		(void) fprintf(stderr, "L3LigneBrisee : nombre de lignes negatif ou nul :%d\n", nlignes);
	}
}
