/****************/
/* L3Marqueur.c */
/****************/

#include <stdio.h>

#include "L3Environnement.h"
#include "L3Marqueur.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/**************************************************************************************
 * Procedure utilisateur : L3Marqueur                                                 *
 *------------------------------------------------------------------------------------*
 * Parametres :                                                                       *
 *   sousimage    : numero de la sous-image concernee                                 *
 *   x            : tableau des abcisses utilisateur des points a marquer             *
 *   y            : tableau des ordonnees utilisateur des points a marquer            *
 *   npoints      : nombre de points a marquer                                        *
 *   typemarqueur : type du marqueur a utiliser                                       *
 *                                                                                    *
 * Valeur de retour : aucune                                                          *
 *                                                                                    *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987 "GRAF CIGALE" *
 *                                                                                    *
 * Erreurs :                                                                          *
 *   nombre de points a marquer negatif ou nul                                        *
 *   type de marqueur incorrect                                                       *
 **************************************************************************************/

void
L3Marqueur(sousimage, x, y, npoints, typemarqueur)
	int             sousimage;
	REEL            x[], y[];
	int             npoints;
	int             typemarqueur;

{
	int             i;
	int             codel3[5];
	int             xl3, yl3;
	int             marqueuratracer;

	if (npoints > 0) {
		if ((typemarqueur >= 0) && (typemarqueur <= 127)) {
			codel3[0] = 5;
			codel3[1] = CODEL3MARQUEUR;
			if (typemarqueur >= 32)
				codel3[4] = typemarqueur * 256;
			else
				codel3[4] = typemarqueur;
			SETSOUSIMAGE(sousimage);

			for (i = 0; i < npoints; i++) {
				L3ConversionCoordonnees(x[i], y[i], &xl3, &yl3);
				marqueuratracer = L3ClippingPoint(xl3, yl3);
				if (marqueuratracer == VRAI) {
					codel3[2] = xl3;
					codel3[3] = yl3;
					L3AppelDrivers(codel3);
					XPAGECOURANT = xl3;
					YPAGECOURANT = yl3;
				}
			}

		} else {
			/*--------------------------------------*/
			/* Erreur : type de marqueur incorrect. */
			/*--------------------------------------*/
			(void) fprintf(stderr, "L3Marqueur : type de marqueur incorrect :%d\n", typemarqueur);
		}
	} else {
		/*-----------------------------------------------------*/
		/* Erreur : nombre de points a marquer negatif ou nul. */
		/*-----------------------------------------------------*/
		(void) fprintf(stderr, "L3Marqueur : nombre de points a marquer negatif ou nul :%d\n", npoints);
	}
}
