/*************/
/* L3Texte.c */
/*************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "L3Environnement.h"
#include "L3Texte.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/**************************************************************************************
 * Procedure utilisateur : L3Texte                                                    *
 *------------------------------------------------------------------------------------*
 * Parametres :                                                                       *
 *   sousimage : numero de la sous-image concernee                                    *
 *   texte     : pointeur vers la chaine de caractere (au format C) a afficher        *
 *                                                                                    *
 * Valeur de retour : aucune.                                                         *
 *                                                                                    *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987 "GRAF CIGALE" *
 *                                                                                    *
 * Erreurs :                                                                          *
 *   probleme d'allocation memoire                                                    *
 *   longueur de la chaine de caracteres incorrecte                                   *
 **************************************************************************************/

void
L3Texte(int sousimage,const char texte[])
{
   int            *codel3;
   int             longueur;
   int             longtextel3;
   int             i;
   REEL            xutil, yutil;

   longueur = strlen(texte);
   longtextel3 = longueur / 2;
   if (longtextel3 * 2 < longueur)
     longtextel3++;
   if ((longueur > 0) && (longueur <= LONGUEURTEXTEMAX)) {

      if (TYPEPOLICE == POLICESOFT) {
		
	 L3ConvCoordL3VersUtil(XPAGECOURANT, YPAGECOURANT, &xutil, &yutil);
	 L3GenTexte(sousimage, texte, xutil, yutil, 1.0, AspectTexte, ANGLETEXTE * 180.0 / L3Pi, FAUX);

      } else {

	 if ((codel3 = (int *) malloc((unsigned int) sizeof(int) * (longtextel3 + 7))) != NULL) {

	    SETSOUSIMAGE(sousimage);

	    codel3[0] = longtextel3 + 7;
	    codel3[1] = CODEL3TEXTE;
	    codel3[2] = XPAGECOURANT;
	    codel3[3] = YPAGECOURANT;
	    codel3[4] = NBCAR;
	    codel3[5] = (int) (1024.0 * ANGLETEXTE);
	    codel3[6] = longueur;

	    for (i = 0; i < longtextel3; i++)
	      codel3[7 + i] = (((int) texte[2 * i]) << 8) | ((int) texte[2 * i + 1]);

	    L3AppelDrivers(codel3);
	    free((char *) codel3);
				
				
	    XPAGECOURANT += (int) (LARGEURPAGEL3 / NBCAR * longueur * cos(ANGLETEXTE));
	    YPAGECOURANT += (int) (LARGEURPAGEL3 / NBCAR * longueur * sin(ANGLETEXTE));

	 } else {
	    fprintf(stderr,
			   "L3Texte : probleme d'allocation memoire.");
	 }
      }
   } else {

      if(longueur != 0) fprintf(stderr,
		     "L3Texte : texte de longueur incorrecte :%d\n", longueur);

   }
} /* L3Texte */


static float ReductionExposant = 4./3.;

void
L3TexteNumerique(int ssim, char* txt)
{
   int l = strlen(txt);
   SETSOUSIMAGE(ssim);
   
   if (TYPEPOLICE == POLICESOFT) {
      L3Texte(ssim, txt);
   } else {
      int oldnbcar;
      int p = 0;
      char *t = malloc(l + 1);
      strcpy(t, txt);
      while ((txt[p] != MODEXPOSANT) && (txt[p] != '\0')) {
	 p++;
      } /* while */
      
      t[p] = '\0';
      L3Texte(ssim, t);

      if (p < l) {
	 oldnbcar = NBCAR;
	 XPAGECOURANT -= (int) (LARGEURPAGEL3 / NBCAR * sin(ANGLETEXTE));
	 YPAGECOURANT += (int) (LARGEURPAGEL3 / NBCAR * cos(ANGLETEXTE));
	 NBCAR = (int)(ReductionExposant * oldnbcar);
	 L3Texte(ssim, &txt[p+1]);
	 XPAGECOURANT += (int) (LARGEURPAGEL3 / NBCAR *
				sin(ANGLETEXTE) * ReductionExposant);
	 YPAGECOURANT -= (int) (LARGEURPAGEL3 / NBCAR *
				cos(ANGLETEXTE) * ReductionExposant);
	 NBCAR = oldnbcar;
      } /* if */
      free(t);
   } /* if */
} /* L3TexteNumerique */

void
L3TailleTexteNumerique(int ssim, char *txt, int *dx, int *dy)
{
/*-
    Donne la taille du rectangle de la boite englobant le texte
    On ne tient pas compte d'une eventuelle inclinaison.
*/

   int oldnbcar, xp = 0, yp = 0;
   int p = 0, l = strlen(txt);
   char *t = malloc(l + 1);
   SETSOUSIMAGE(ssim);
   
   strcpy(t, txt);
   while ((txt[p] != MODEXPOSANT) && (txt[p] != '\0')) {
      p++;
   } /* while */
   
   t[p] = '\0';
   xp += (int) (LARGEURPAGEL3 / NBCAR) * p;
   yp += (int) (LARGEURPAGEL3 / NBCAR);
   
   if (p < l) {
      oldnbcar = NBCAR;
      NBCAR = (int)(ReductionExposant * oldnbcar);      
      xp += (int) (LARGEURPAGEL3 / NBCAR) * (l - p);
      yp += (int) (LARGEURPAGEL3 / NBCAR);
      NBCAR = oldnbcar;
   } /* if */
   free(t);

   /* Retour des valeurs */
   *dx = xp;
   *dy = yp;
} /* L3TailleTexteNumerique */
