/* identification du source */

#ifndef lint
    static const char* const sccs_id = " %W% %G% Pierre Brochard";
#endif

#include "SpxP.h"
#include "SpxVisuDri.h"
#include "nx.h"
#include "SpxRect3d.h"
#include "SpxOpenP.h"

/*********************************************************/
/* envoi de donnees de type 3d structure rectangulaire : */
/*       x(i), y(j), z(i,j)                              */
/*********************************************************/

static void
SpxGenRect3d(int nx,int ny,void* x,void* y,void* z,int dimz,
		int precision,SPXFILE* spx_file)
{
	FILE* file_fich = NULL;

	if(!spx_file) return;

	if(SpxDebut(spx_file)) return;

	file_fich = spx_file->file;

	if(spx_file->type != SPXFICHIERASCII)
	{
            int temp = 0;
            int i = 0;
            int* nxsto = NULL;
	    char* const zz = (char*) z;

	    SPXDEFICODLMOT(icod,lmot,precision);
            temp = -7;
            fnxwriti(file_fich,1,&temp);
            temp = ny;
            fnxwriti(file_fich,1,&temp);
    
            nxsto = (int*) malloc(ny*sizeof(int));
    
            if(nxsto == NULL)
            {
                fprintf(stderr," pas assez de place memoire ; fin lecture");
                return;
            }
    
            for(i=0;i<ny;i++)
            {
                nxsto[i] = nx;
            }
    
            fnxwriti(file_fich,ny,nxsto);

            free(nxsto);nxsto = NULL;

	    fnxwritt(file_fich,nx,x,icod,"sp");
    	    fnxwritt(file_fich,ny,y,icod,"sp");
    
            for(i=0;i<ny;i++)
            {
                fnxwritt(file_fich,nx,zz+i*dimz*lmot,icod,"sp");
            }
        
            temp = 0;
            fnxwriti(file_fich,1,&temp);
            fnxwrite(file_fich,"@",1);
	}
	else
	{
	    char temp[80];
	    int i = 0,j = 0;

	    for(;i<nx;i++)
	    {

		for(j=0;j<ny;j++)
		{
		    SPXASCII(precision,
				    sprintf(temp,"%g %g %g\n",
						    ((REEL4*) x)[i],
						    ((REEL4*) y)[j],
						    ((REEL4*) z)[i+j*dimz]),
				    sprintf(temp,"%g %g %g\n",
						    ((REEL_S*) x)[i],
						    ((REEL_S*) y)[j],
						    ((REEL_S*) z)[i+j*dimz]),
				    sprintf(temp,"%g %g %g\n",
						    ((REEL*) x)[i],
						    ((REEL*) y)[j],
						    ((REEL*) z)[i+j*dimz]));
		    fnxwrite(file_fich,temp,strlen(temp));
		}

		if(i != nx-1)
		{
		    sprintf(temp,"~e\n");
		    fnxwrite(file_fich,temp,strlen(temp));
		}
		else
		{
		    sprintf(temp,"~f\n");
		    fnxwrite(file_fich,temp,strlen(temp));
		}

	    }

	}

	fnxflush(file_fich);
	SpxFin(spx_file);
}

void
SpxRect3d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		SPXFILE* spx_file)
{
	SpxGenRect3d(nx,ny,x,y,z,dimz,SPXREAL,spx_file);
}

/* double precision (transmis en ieee 32 bits) */

void
SpxDRect3d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		SPXFILE* spx_file)
{
	SpxGenRect3d(nx,ny,x,y,z,dimz,SPXDOUBLEPRECISION,spx_file);
}

/* REAL*4 (transmis en ieee 32 bits) */

void
Spx4Rect3d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 z[],int dimz,
		SPXFILE* spx_file)
{
	SpxGenRect3d(nx,ny,x,y,z,dimz,SPXREAL4,spx_file);
}

/* REAL*8 (transmis en ieee 32 bits) */

void
Spx8Rect3d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 z[],int dimz,
		SPXFILE* spx_file)
{
#if defined(CRAY)
	SpxGenRect3d(nx,ny,x,y,z,dimz,SPXREAL,spx_file);
#else
	SpxGenRect3d(nx,ny,x,y,z,dimz,SPXDOUBLEPRECISION,spx_file);
#endif
}

/************************************************/
/* donnees C aux mailles X, Y, Z = 0 aux noeuds */
/************************************************/

static void
SpxGenMailleRect3d(int nx,int ny,void* x,void* y,void* c,int dimc,int precision,
		SPXFILE* spx_file)
{
	FILE* file_fich = NULL;

	if(!spx_file) return;

	if(SpxDebut(spx_file)) return;

	file_fich = spx_file->file;

	if(spx_file->type != SPXFICHIERASCII)
	{
            int temp = 0;
            register int i = 0;
            int* nxsto = NULL;
	    char* const cc = (char*) c;

	    SPXDEFICODLMOT(icod,lmot,precision);
            temp = -13;
            fnxwriti(file_fich,1,&temp);
            temp = ny;
            fnxwriti(file_fich,1,&temp);
    
            nxsto = (int*) malloc(ny*sizeof(int));
    
            if(!nxsto)
            {
                fprintf(stderr," pas assez de place memoire ; fin lecture");
                return;
            }
    
            for(i=0;i<ny;i++)
            {
                nxsto[i] = nx;
            }
    
            fnxwriti(file_fich,ny,nxsto);

	    free(nxsto);nxsto = NULL;

	    fnxwritt(file_fich,nx,x,icod,"sp");
    	    fnxwritt(file_fich,ny,y,icod,"sp");

            for(i=0;i<ny-1;i++)
            {
                fnxwritt(file_fich,nx-1,cc+i*dimc*lmot,icod,"sp");
            }
        
            temp = 0;
            fnxwriti(file_fich,1,&temp);
            fnxwrite(file_fich,"@",1);
	}
	else
	{
	    char temp[80];
	    register int i = 0,j = 0;
	    register int nx1 = nx-1;
	    register int ny1 = ny-1;

	    for(;i<nx1;i++)
	    {

		for(j=0;j<ny1;j++)
		{
		    SPXASCII(precision,
				    sprintf(temp,"%g %g %g\n",
						    ((REEL4*) x)[i],
						    ((REEL4*) y)[j],
						    ((REEL4*) c)[i+j*dimc]),
				    sprintf(temp,"%g %g %g\n",
						    ((REEL_S*) x)[i],
						    ((REEL_S*) y)[j],
						    ((REEL_S*) c)[i+j*dimc]),
				    sprintf(temp,"%g %g %g\n",
						    ((REEL*) x)[i],
						    ((REEL*) y)[j],
						    ((REEL*) c)[i+j*dimc]));
		    fnxwrite(file_fich,temp,strlen(temp));
		}

		SPXASCII(precision,
				sprintf(temp,"%g %g\n",
						((REEL4*) x)[i],
						((REEL4*) y)[ny1]),
				sprintf(temp,"%g %g\n",
						((REEL_S*) x)[i],
						((REEL_S*) y)[ny1]),
				sprintf(temp,"%g %g\n",
						((REEL*) x)[i],
						((REEL*) y)[ny1]));
		fnxwrite(file_fich,temp,strlen(temp));
		sprintf(temp,"~e\n");
		fnxwrite(file_fich,temp,strlen(temp));
	    }

	    for(j=0;j<ny;j++)
	    {
		SPXASCII(precision,
				sprintf(temp,"%g %g\n",
						((REEL4*) x)[nx1],
						((REEL4*) y)[j]),
				sprintf(temp,"%g %g\n",
						((REEL_S*) x)[nx1],
						((REEL_S*) y)[j]),
				sprintf(temp,"%g %g\n",
						((REEL*) x)[nx1],
						((REEL*) y)[j]));
		fnxwrite(file_fich,temp,strlen(temp));
	    }

	    sprintf(temp,"~f\n");
	    fnxwrite(file_fich,temp,strlen(temp));
	}

	fnxflush(file_fich);
	SpxFin(spx_file);
}

void
SpxMailleRect3d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S c[],int dimc,
		SPXFILE* spx_file)
{
	SpxGenMailleRect3d(nx,ny,x,y,c,dimc,SPXREAL,spx_file);
}

void
SpxDMailleRect3d(int nx,int ny,REEL x[],REEL y[],REEL c[],int dimc,
		SPXFILE* spx_file)
{
	SpxGenMailleRect3d(nx,ny,x,y,c,dimc,SPXDOUBLEPRECISION,spx_file);
}

void
Spx4MailleRect3d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 c[],int dimc,
		SPXFILE* spx_file)
{
	SpxGenMailleRect3d(nx,ny,x,y,c,dimc,SPXREAL4,spx_file);
}

void
Spx8MailleRect3d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 c[],int dimc,
		SPXFILE* spx_file)
{
#if defined(CRAY)
	SpxGenMailleRect3d(nx,ny,x,y,c,dimc,SPXREAL,spx_file);
#else
	SpxGenMailleRect3d(nx,ny,x,y,c,dimc,SPXDOUBLEPRECISION,spx_file);
#endif
}
