#ifndef NXP_H
#define NXP_H

/* %W% %G% DMA/AIM */
/* modifs Pierre Brochard */
/* etendu le 5/9/96 par Pierre Brochard pour autoriser les sockets AF_UNIX */
/* modifie 2/98 par Pierre Brochard : ajouts de macros et recodage des fonctions
   de lecture/ecriture specifiques machine */
/* si l'on compile avec -DNX_NOT_ALLOC les routines d'entree-sorties
   specifiques machines (dans nxie*, nxcray.c, nxvms.c, nxlittleend.c)
   n'utilisent pas malloc mais un buffer de taille fixe (NX_BUF_SIZE en
   octets defini ci-dessous). Sinon, malloc est utilise borne a NX_BUF_SIZE si
   on depasse une certaine taille (HEAP_ALLOC en octets defini ci-dessous) en
   dessous de laquelle, un buffer fixe est utilise. */
/* -DNX_NOT_ALLOC est surtout utile sur CRAY pour des raisons de performances.*/

/*************************************************************/
/* includes                                                  */
/*************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

#include "U8machcomp.h"

#ifdef NXSUNSRC
#   undef NXSUNSRC
#endif

#if defined(SUNOS) && !defined(SUNOS_4) && !defined(__cplusplus) && \
	!defined(__EXTENSIONS__)
#   define __EXTENSIONS__
#   define NXSUNSRC
#endif

/* defines autres machines */

#if defined(LINUX) && !defined(_POSIX_SOURCE)
#   define _POSIX_SOURCE
#endif

#if defined(HP_UX) && !defined(_INCLUDE_HPUX_SOURCE)
#   define _INCLUDE_HPUX_SOURCE
#endif

/* includes */

#include <errno.h>
#include <stdio.h>

#if defined (SUNOS) && defined(SCX_X) && SCX_X >= 0x500 && defined(_LP64)
#   include <stdio_ext.h>
#endif

#include <stdlib.h>
#include <values.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <malloc.h>

#ifdef SUNOS
#   undef _POSIX_SOURCE /* sinon option -Xc de acc SUNOS 4 ne passe pas */
#endif

#if defined(NXSUNSRC)
#   undef __EXTENSIONS__
#endif

#if defined(HP_UX) && !defined(_INCLUDE_POSIX_SOURCE) && \
	!defined(_INCLUDE_XOPEN_SOURCE)
#   define _INCLUDE_POSIX_SOURCE
#   define _INCLUDE_XOPEN_SOURCE
#endif

#if defined(HP_UX) && defined(MAXINT)
#   undef MAXINT
#endif

#include <sys/param.h>
#include <unistd.h>
#include "nxsystem.h"
#include "U8signal.h"

/*
  note :
  size_t est ANSI C
  ssize_t est POSIX 1003.1 (utile pour fonctions systemes read, write)
*/

#if !defined(_SSIZE_T) && !defined(_POSIX_SOURCE) && !defined(CRAY)
    typedef int ssize_t; /* used by functions which return a */
#endif

#if defined(SUNOS_4) && defined(SC1_0) && !defined(__GNUC__) && \
	defined(__STDC__) && __STDC__ == 1
    typedef u_long u_int32; /* sinon option -Xc de acc SUN ne passe pas */
#endif

#if defined(HP_UX) && !defined(HP_UX_11)
    typedef int enum_t;
    typedef int bool_t;
#   define _INCLUDE_HPUX_SOURCE
#   include <rpc/types.h>
#else
#   include <rpc/types.h>
#   include <rpc/xdr.h>
#   include <rpc/auth.h>
#   include <rpc/clnt.h>
#endif

#include <sys/socket.h>

#if defined(SUNOS_5) || (defined(EPIX) && defined(_SYSTYPE_SVR4)) ||\
		defined(HP_UX_11)
#   define PORTMAP
#endif

#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>

#include <rpc/rpc.h>
#include <pwd.h>
#include <netdb.h>

#if !defined (_NETINET_IN_H) && !defined (_BSD_NETINET_IN_)
#   include <netinet/in.h> /* pour CDC epix_2 */
#endif

#ifndef NXP_SYS_UN_H
#define NXP_SYS_UN_H

#if defined(EPIX) && defined(_SYSTYPE_SVR3)
#   include <bsd/sys/un.h> /* pour sockets du domaine unix (AF_UNIX) */
#else
#   include <sys/un.h> /* pour sockets du domaine unix (AF_UNIX) */
#endif

#endif /* NXP_SYS_UN_H */

#if defined(NX_SOCKET_BSD)
#   undef NX_SOCKET_BSD
#endif

#if defined(SUNOS_4) && (!defined(EPIX) || defined(_SYSTYPE_BSD43))
#   define NX_SOCKET_BSD
#endif

#ifdef CRAY
#   include <fortran.h>
#endif

#if defined(IRIX_6)
#   include <sgidefs.h>
#endif

/*#if (defined (IRIX_6) && _MIPS_SIM==_MIPS_SIM_ABI64) ||\
 	defined(__alpha) || defined(__mips64) ||\
 	(defined(SCX_X) && SCX_X >= 0x500 && defined(_LP64)) ||\
	(defined(AIX_XLC) && defined(__64BIT__))*/
/* Pour machines IEEE (les CRAY 64 bits non pas besoin de ce define) : */

#define NX_INT32_MAX 0x7FFFFFFFl

#if LONG_MAX > NX_INT32_MAX
#   define NX_LONG_IS_64
#endif

/* type du 3eme argument de getsockname : */

#ifdef AIX_4
#   define NX_GSN_SIZE unsigned
#elif defined(SUNOS) && defined(SCX_X) && SCX_X >= 0x500
#   define NX_GSN_SIZE socklen_t
#elif defined(__GNUC__) && ((__GNUC__ == 3 && __GNUC_MINOR__ >= 2) || __GNUC__ > 3)
#   define NX_GSN_SIZE socklen_t
#else
#   define NX_GSN_SIZE int
#endif

/*************************************************************/
/* defines utiles                                            */
/*************************************************************/

#ifndef CRAY
#   define IEEE 1
#elif defined(IEEE)
#   undef IEEE
#endif

/*************************************************************/
/* Notes :
   Des qu'un FILE* est utilise ne pas utiliser
   directement le fileno associe ou un autre mecanisme
   de bufferisation comme XDR*.
   Une routine comme getsockname est par exemple a proscrire
   des que l'on a lu ou ecrit sur un FILE*.
   fnxaccept, fnxaccept2 et fnxopen lisent volontairement
   le header de nx par read et write pour permettre a leur
   sortie le choix du mecanisme de buffer (FILE* ou
   XDR* ou ...).
   Attention pour la meme raison au shutdown.

   Sur les systemes comme epix_2, attention a rester
   entierement BSD4.3, SVR3 ou SVR4. Ne pas compiler en BSD4.3
   et faire l'edition de liens en SVR4 par exemple.
   les libs socket, rpcsvc et nsl sont SVR4. Ne pas les linker
   avec des libraries BSD43. Sinon pbs bizarres et normaux
   car les API BSD et SVR* peuvent avoir la meme signature
   mais etre incompatibles surtout en ce qui concerne les
   sockets et les signaux.
   */

/*************************************************************/

/*************************************************************/
/* prototypes fonctions libs standards                       */
/*************************************************************/

#if (defined(SUNOS_4) || defined(SUNOS_5)) && !defined(__GNUC__)
    int pclose(FILE* fd);
    int dup(int fd);
    unsigned sleep(unsigned seconds);

#   if defined(SC1_0) && defined(SUNOS_4) || defined(__STDC__) && \
		__STDC__ != 0 && !defined(NXSUNSRC)
        FILE* popen(char* command,char* type);
        FILE* fdopen(int fd,char* type);
#   endif

#   if defined(SC3_0) && defined(SUNOS_5) || defined(SUNOS_4)
        int socket(int domain,int type,int protocol);
        int connect(int s,struct sockaddr* name,int namelen);
        int bind(int s,struct sockaddr* name,int namelen);
        int listen(int s,int backlog);
        int accept(int s,struct sockaddr* addr,int* addrlen);
#   endif
#endif

#if defined(SUNOS_4) && !defined(__GNUC__)
    void bcopy(char* b1,char* b2,int length);
    void bzero(char* b,int length);
    int select (int width,fd_set* readfds,fd_set* writefds,
		fd_set* exceptfds,struct timeval* timeout);
    int shutdown(int s,int how);
    int ruserok(char* rhost,int superuser,char* ruser,char* luser);
    int getsockname(int s,struct sockaddr* name,int* namelen);
    void exit(int status);
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

#ifdef IRIX
    int ruserok(char *rhost, int superuser, char *ruser, char *luser);
#endif

#if defined(EPIX) && defined(_SYSTYPE_BSD43)
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

char* mktemp(char* templ);

/*************************************************************/
/* defines utiles                                            */
/*************************************************************/

#define FILE_MAX_LEN	1024
#define TYPE_MAX_LEN	10

#ifdef _CRAYMPP
#   define NX_MAX_DESC	10000
#else
#   define NX_MAX_DESC	4096
#endif

#define MAXBUF          1024

#define MAX_ERROR_CNT   20           /* limitation des messages d'erreurs  */
                                     /* ds les cas d'underflow et overflow */
#define NXP_PIPE	1
#define NXP_SOCKET	2
#define NXP_FILE        3
#define NXP_RPC		4

/* taille types nx locaux */
#define NX_SIZE_INT sizeof(int)
#define NX_SIZE_LONG sizeof(long)
#define NX_SIZE_LONGLONG sizeof(long long)
#define NX_SIZE_CFLT sizeof(float) /* type float C */
#define NX_SIZE_CDBLE sizeof(double) /* type double C */
#define NX_SIZE_FLT sizeof(int) /* type REAL FORTRAN */
#define NX_SIZE_DBLE 2*sizeof(int) /* type DOUBLE PRECISION FORTRAN */

/* taille types nx de format defini */
#define NX_SIZE_32 4
#define NX_SIZE_64 8
#define NX_SIZE_128 16

/* taille buffers */
#define NX_BUF_SIZE 64*1024
#define NX_BUF_SIZE_FLT ((int) (NX_BUF_SIZE/NX_SIZE_FLT))
#define NX_BUF_SIZE_DBLE ((int) (NX_BUF_SIZE/NX_SIZE_DBLE))
#define NX_BUF_SIZE_32 NX_BUF_SIZE/NX_SIZE_32
#define NX_BUF_SIZE_64 NX_BUF_SIZE/NX_SIZE_64
#define NX_BUF_SIZE_128 NX_BUF_SIZE/NX_SIZE_128

#ifdef NX_NOT_ALLOC
#   define HEAP_ALLOC_RED NX_BUF_SIZE/NX_SIZE_INT
#else
#   define HEAP_ALLOC_RED 512
#endif

#define HEAP_ALLOC HEAP_ALLOC_RED*NX_SIZE_INT

typedef struct
{
	union
	{
		FILE* fd;             /* descripteur cas fichier */
		CLIENT* clnt;         /* descripteur cas rpc, v. structure */
	} dp;                         /* client ds rpc.clnt.h              */
	short ds;                     /* indicateur du cas : file, pipe ...*/
} DESC;

#define SOCK_VERSION 2
#define L1 4
#define L2X 64
#define L3 9

struct Header_S
{
	char lg[L1];
	char version[L1];
	char host_name[L2X];
	char user_name[L2X];
	char port_name[L3];
	char handler_name[L3];
};

#define SIZ_HDX 2*(L1+L2X+L3)
#define SIZ_HDSL 2*L1
#define L2 13
#define SIZ_HDS 2*(L1+L2+L3)

struct NxTexte
{
	char* data;
	int lg;
};

#define RPCWRITE 10
#define STD_TO 600
#define RPCDIALOG 11
#define MAX_L3 66000

#ifndef MIN
#   define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#   define MAX(a,b) (((a)>(b))?(a):(b))
#endif

/* types de machines */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
#   define NX_MACH_INT_IE32 /* le type int ie32 existe */
#   define NX_MACH_INT_IE64 /* le type int ie64 existe */
#   define NX_MACH_FLT_IE32 /* le type float ie32 existe */
#   define NX_MACH_FLT_IE64 /* le type float ie64 existe */
#endif

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
#   define NX_MACH_INT_IE64 /* le type int ie64 existe */
#   define NX_MACH_FLT_IE64 /* le type float ie64 existe */
#   define NX_MACH_FLT_IE128 /* le type float ie128 existe */
#endif

#if defined(CRAY) && defined(_CRAYMPP)
#   define NX_MACH_INT_IE64 /* le type int ie64 existe */
#   define NX_MACH_FLT_IE32 /* le type float ie32 existe */
#   define NX_MACH_FLT_IE64 /* le type float ie64 existe */
#endif

/* macros pour fonctions d'acces specifiques machines */

#define NX_READ(fd,buff2,byte_nb,nb,mes,rc) \
	{\
	    size_t n2 = nb;						\
	    while (n2)							\
	    {								\
		const size_t n = fread(buff2, byte_nb, n2, fd);		\
		n2 -= n;buff2 += n * byte_nb;				\
                if(!n2 || errno == EINTR) {     \
		    clearerr(fd);errno = 0;}				\
		else							\
		    break;						\
	    }								\
	    if(n2)							\
	    { *rc = ferror(fd) ? -1 -errno : -1;nxpioerror(mes);}	\
	    else							\
	      { *rc = 0 ;}						\
	    buff2 -= (nb - n2)*byte_nb;}

#define NX_READ_NBLK(fd,buff2,byte_nb,nb,nblus,mes,rc)			\
    {									\
	    size_t n2 = nb;						\
	    while (n2)							\
	    {								\
		const size_t n = fread(buff2, byte_nb, n2, fd);		\
		n2 -= n;buff2 += n * byte_nb;				\
                if(!n2 || errno == EINTR) {     \
		    clearerr(fd);errno = 0;}				\
		else							\
		    break;						\
	    }								\
	    *nblus = nb - n2;						\
	    if(!*nblus)							\
	    { *rc = ferror(fd) ? -1 -errno : -1;nxpioerror(mes);}	\
	    else							\
	      { *rc = 0 ;}						\
	    buff2 -= (*nblus)*byte_nb;}

#define NX_WRITE(fd,buff2,byte_nb,nb,mes,rc) \
	{\
	    size_t n2 = nb;						\
	    while (n2)							\
	    {								\
		const size_t n = fwrite(buff2, byte_nb, n2, fd);	\
		n2 -= n;buff2 += n * byte_nb;				\
                if(!n2 || errno == EINTR) {     \
		    clearerr(fd);errno = 0;}				\
		else							\
		    break;						\
	    }								\
	    if(n2)							\
	      { *rc = ferror(fd) ? -1 -errno : -1;nxpioerror(mes);}	\
	    else							\
	      { *rc = 0 ;}						\
	    buff2 -= (nb - n2)*byte_nb;}				\

#define NX_WRITE_NBLK(fd,buff2,byte_nb,nb,nbecr,mes,rc)			\
    {									\
	    size_t n2 = nb;						\
	    while (n2)							\
	    {								\
		const size_t n = fwrite(buff2, byte_nb, n2, fd);	\
		n2 -= n;buff2 += n * byte_nb;				\
               if(!n2 || errno == EINTR) {     \
		    clearerr(fd);errno = 0;}				\
		else							\
		    break;						\
	    }								\
	    *nbecr = nb - n2;						\
	    if(!*nbecr)							\
	    { *rc = ferror(fd) ? -1 -errno : -1;nxpioerror(mes);}		\
	    else							\
	      { *rc = 0 ;}						\
	    buff2 -= (*nbecr)*byte_nb;}

/* nouvelles macros pour alloc bufferisee limitee a NX_BUF_SIZE caracteres */

#define NX_READBUF(fd,buff2,byte_nb,nb,buff,bytel_nb,nx_buf_size,fconv,mes,rc)\
	{\
	    size_t nb2 = nb;\
	    int nbt = MIN(nb,nx_buf_size);\
	    int nt = nbt;*rc = 0;\
	    while(nt > 0)\
	    {\
	        errno = 0;\
                {\
		    const int n = (int) fread(buff2,byte_nb,nt,fd);\
		    if (!n)\
		    {\
		        *rc = errno ? -1 -errno : -1;nxpioerror(mes);break;\
		    }\
	            else\
	    	    {\
		        *rc = 0;nt -= n;buff2 += n*byte_nb;\
		    }\
		    if(!nt)\
		    {\
		        buff2 -= nbt*byte_nb;\
		        fconv;errno = 0;\
		        buff += nbt*bytel_nb;\
			nb2 -= MIN(nb2,(size_t) nx_buf_size);\
			nt = nbt = (int) MIN(nb2,(size_t) nx_buf_size);\
		    }\
		}\
	    }\
	}

#define NX_READBUF_NBLK(fd,buff2,byte_nb,nb,buff,bytel_nb,nx_buf_size,fconv,\
	       	nblus,mes,rc)\
	{\
	    size_t nb2 = nb;\
	    int nbt = MIN(nb,nx_buf_size);\
	    int nt = nbt;*nblus = 0;*rc = 0;\
	    while(nt > 0)\
	    {\
	        errno = 0;\
                {\
		    const int n = (int) fread(buff2,byte_nb,nt,fd);\
		    if (!n)\
		    {\
		        *rc = errno ? -1 -errno : -1;nxpioerror(mes);\
		    }\
	            else\
	    	    {\
		        *rc = 0;nt -= n;fconv;*nblus += n;\
		    }\
		    if(!nt)\
		    {\
		        buff += nbt*bytel_nb;\
		        nb2 -= MIN(nb2,(size_t) nx_buf_size);\
		        nt = nbt = (int) MIN(nb2,(size_t) nx_buf_size);\
		    }\
		    else break;\
		}\
	    }\
	}

#define NX_WRITEBUF(fd,buff2,byte_nb,nb,buff,bytel_nb,nx_buf_size,fconv,mes,rc)\
	{\
	    size_t nb2 = nb;\
	    int nbt = MIN(nb,nx_buf_size);\
	    int nt = nbt;\
	    while(nt > 0)\
	    {\
	        if(nt == nbt) {fconv;}\
                errno = 0;\
		{\
		    const int n = (int) fwrite(buff2,byte_nb,nt,fd);\
		    if (!n && ferror(fd))\
		    {\
		        *rc = errno ? -1 -errno : -1;nxpioerror(mes);\
		    }\
		    else\
		    {\
		        *rc = 0;nt -= n;buff2 += n*byte_nb;\
		    }\
		    if(!nt)\
		    {\
		        buff2 -= nbt*byte_nb;\
		        buff += nbt*bytel_nb;\
		        nb2 -= MIN(nb2,(size_t) nx_buf_size);\
		        nt = nbt = (int) MIN(nb2,(size_t) nx_buf_size);\
		    }\
	        }\
	    }\
	}

#define NX_WRITEBUF_NBLK(fd,buff2,byte_nb,nb,buff,bytel_nb,nx_buf_size,fconv,\
		nbecrits,mes,rc)\
	{\
	    size_t nb2 = nb;\
	    int nbt = MIN(nb,nx_buf_size);\
	    int nt = nbt;\
	    while(nt > 0)\
	    {\
                fconv;errno = 0;\
		{\
		    const int n = (int) fwrite(buff2,byte_nb,nt,fd);\
		    if (!n && ferror(fd))\
		    {\
		        *rc = errno ? -1 -errno : -1;nxpioerror(mes);\
		    }\
		    else\
		    {\
		        *rc = 0;nt -= n;*nbecrits += n;\
		    }\
		    if(!nt)\
		    {\
		        buff += nbt*bytel_nb;\
		        nb2 -= MIN(nb2,(size_t) nx_buf_size);\
		        nt = nbt = (int) MIN(nb2,(size_t) nx_buf_size);\
		    }\
		    else break;\
	        }\
	    }\
	}

#ifdef NX_NOT_ALLOC
#   define NX_ALLOCBUF(buff2,buff3,u)
#   define NX_FREEBUFU(buff3)
#else /* !NX_NOT_ALLOC */
#   define NX_ALLOCBUF(buff2,buff3,u)\
	if((u) > HEAP_ALLOC)\
	{\
	    const size_t tai = MIN((u),NX_BUF_SIZE);\
	    errno = 0;buff3 = (char*) malloc(tai);\
	    if(!buff3) cant_alloc_memory(tai);\
	    buff2 = buff3;\
	}
#   define NX_FREEBUFU(buff3) if(buff3) free(buff3)

/* ----------------------------------------------------------------------- */
/*		====>    C A N T _ A L L O C _ M E M O R Y    <====	   */
/* ----------------------------------------------------------------------- */

static void
cant_alloc_memory(const size_t lg)
{
	fprintf(stderr, "nx : cant alloc memory %lu\n",(unsigned long) lg);
	exit(1);
}
#endif /* NX_NOT_ALLOC */

#if defined(__cplusplus)
}
#endif

#endif /* NXP_H */
