/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Pierre Brochard CEA/DAM-Ile de France";
#endif

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"

#define NXCONV_H_IMPLEMENT
#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT
#undef NXCONV_H_IMPLEMENT

/*******************************************************************/
/*******************************************************************/
/* --------------- Conversion functions     ---------------------- */
/*******************************************************************/
/*******************************************************************/

/*******************************************************************/
/* macros utiles pour les conversions vers et de machines VMS      */
/*******************************************************************/

#if !defined(CRAY) || defined(_CRAYMPP)
#   define NXCONV_GEN_LITTLE_BIG32(ss) {register nx_unsigned32 stemp = ss;\
		register nx_unsigned32 stemp1 = stemp >> 24 & 0xff;\
	        register nx_unsigned32 stemp2 = stemp >> 8  & 0xff00;\
	        register nx_unsigned32 stemp3 = stemp << 8  & 0xff0000;\
	        ss = stemp << 24 | stemp3 | stemp2 | stemp1;}
#   define NXCONV_GEN_VMS_BIG32(ss) {register nx_unsigned32 stemp = ss;\
		register nx_unsigned32 stemp1 = stemp >> 8  & 0xff0000;\
	        register nx_unsigned32 stemp2 = stemp << 8  & 0xff000000;\
	        register nx_unsigned32 stemp3 = stemp >> 8  & 0xff;\
	        ss = (stemp << 8 & 0xff00) | stemp3 | stemp2 | stemp1;}
#   define NXCONV_LGEN_VMS_BIG32(ss) {register nx_unsigned32 stemp = ss;\
		register nx_unsigned32 stemp1 = stemp >> 16 & 0xff00;\
	        register nx_unsigned32 stemp2 = stemp >> 16 & 0xff;\
	        register nx_unsigned32 stemp3 = stemp << 16 & 0xff000000;\
	        ss = (stemp << 16 & 0xff0000) | stemp3 | stemp2 | stemp1;}
#endif

#if defined(CRAY) && !defined(_CRAYMPP)
#   define NXCONV_BIG_VMS_BIG32(ss) {register unsigned stemp = ss;\
		register unsigned sst1 = stemp >> 8  & 0xff000000000000;\
		register unsigned sst2 = stemp << 8  & 0xff00000000000000;\
		register unsigned sst3 = stemp >> 8  & 0xff00000000;\
		register unsigned sst5 = stemp >> 8  & 0xff0000;\
		register unsigned sst6 = stemp << 8  & 0xff000000;\
		register unsigned sst7 = stemp >> 8  & 0xff;\
		register unsigned sst8 = stemp << 8  & 0xff00;\
		ss = (stemp << 8 & 0xff0000000000) | sst3 | sst2 | sst1 |\
		sst8 | sst7 | sst6 | sst5;}
#elif defined(VMS)
#   define NXCONV_VMS_BIG32(ss) NXCONV_GEN_VMS_BIG32(ss)
#   define NXCONV_BIG_VMS_BIG32(ss)
#elif !defined(NX_LITTLEENDIAN)
#   define NXCONV_VMS_BIG32(ss)
#   define NXCONV_BIG_VMS_BIG32(ss) NXCONV_GEN_VMS_BIG32(ss)
#else
#   define NXCONV_VMS_BIG32(ss) NXCONV_GEN_LITTLE_BIG32(ss)
#   define NXCONV_BIG_VMS_BIG32(ss) NXCONV_LGEN_VMS_BIG32(ss)
#endif

/*******************************************************************/
/*                    nxconv_real_vms32_to_ie32(S,D,NB)            */
/*                                                                 */
/* Conversion of VMS 32 bits reals to IEEE 32 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms32_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	int rc = 0,nxerror = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register nx_unsigned32 ss = s[i];

	    NXCONV_BIG_VMS_BIG32(ss);

	    if(ss)
	    { 
		register nx_int32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 23 & 0xff;
		register nx_int32 mant = ss & 0x7fffff;

		expn -= 2; /* (expn - 129) + 127 */

		if (expn <= 0) /* underflow */
		{
		    d[i] = 0;
		    nxerror = 76;
		    rc = -1;
		}
		else
		{
		    expn <<= 23;
		    expn &= 0xff800000;
		    d[i] = sign | expn | mant;
		    NXCONV_VMS_BIG32(d[i]);
		}

	    }
	    else
		d[i] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror;
}
#else
nxconv_real_vms32_to_ie32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	int rc = 0,nxerror = 0;
	register int i = 0;
	register const int nb2 = nb/2;
	register const int nb21 = (nb+1)/2;

	for(; i < nb21; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[i] >> 32;

	    NXCONV_BIG_VMS_BIG32(ss);

	    if (ss) 
	    {
		register int sign = (ss >> 31) << 63;
		register int expn = ss >> 23 & 0xff;
		register int mant = ss & 0x7fffff;

		expn -= 2; /* (expn - 129) + 127 */

		if (expn <= 0)  	/* underflow */
		{
		    nxerror = 77;
		    rc = -1;
		} 
		else 
		{
		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] = temp;
	}

	for(i=0; i < nb2; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[i] & 0xffffffff;

	    NXCONV_BIG_VMS_BIG32(ss);

	    if (ss) 
	    {
		register int sign = (ss >> 31) << 63;
		register int expn = ss >> 23 & 0xff;
		register int mant = ss & 0x7fffff;

		expn -= 2; /* (expn - 129) + 127 */

		if (expn <= 0)  	/* underflow */
		{
		    nxerror = 77;
		    rc = -1;
		} 
		else 
		{
		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    } 

	    d[i] |= temp >> 32;
	}

	*cr = rc;
	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie32_to_vms32(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE 32 bits reals to VMS 32 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_vms32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	int rc = 0,nxerror = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register nx_unsigned32 ss = s[i];

	    NXCONV_VMS_BIG32(ss);

	    if(ss)
	    { 
		register nx_int32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 23 & 0xff;
		register nx_int32 mant = ss & 0x7fffff;

		expn += 2; /* (expn - 127) + 129 */

		if (expn >= 255)                /* overflow  */
		{
		    d[i] = sign | 0x7f7fffff; /* max float ? */
		    nxerror = 78;
		    rc = -1;
		}
		else
		{
		    expn <<= 23;
		    expn &= 0xff800000;
		    d[i] = sign | expn | mant;
		}

		NXCONV_BIG_VMS_BIG32(d[i]);
	    }
	    else
		d[i] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror;
}
#else
nxconv_real_ie32_to_vms32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	int rc = 0,nxerror = 0;
	register int i = 0;
	register const int nb2 = nb/2;
	register const int nb21 = (nb+1)/2;

	for(; i < nb21; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[i] >> 32;

	    if (ss) 
	    {
		register int sign = (ss >> 31) << 63;
		register int expn = ss >> 23 & 0xff;
		register int mant = ss & 0x7fffff;

		expn += 2; /* (expn - 127) + 129 */

		if (expn >= 255)                /* overflow  */
		{
		    temp = sign | 0x7f7fffff; /* max float ? */
		    nxerror = 79;
		    rc = -1;
		}
		else 
		{
		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] = temp;
	}

	for(i=0; i < nb2; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[i] & 0xffffffff;

	    if (ss) 
	    {
		register int sign = (ss >> 31) << 63;
		register int expn = ss >> 23 & 0xff;
		register int mant = ss & 0x7fffff;

		expn += 2; /* (expn - 127) + 129 */

		if (expn >= 255)                /* overflow  */
		{
		    temp = sign | 0x7f7fffff; /* max float ? */
		    nxerror = 79;
		    rc = -1;
		}
		else 
		{
		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    } 

	    d[i] |= temp >> 32;
	    NXCONV_BIG_VMS_BIG32(d[i]);
	}

	*cr = rc;
	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie32_to_vms64(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE 32 bits reals to VMS 64 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_vms64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_ie32_to_vms64 : conversion non implemented\n");
}
#else
nxconv_real_ie32_to_vms64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_ie32_to_vms64 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_vms64_to_ie32(S,D,NB)            */
/*                                                                 */
/* Conversion of VMS 64 bits reals to IEEE 32 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms64_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms64_to_ie32 : conversion non implemented\n");
}
#else
nxconv_real_vms64_to_ie32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms64_to_ie32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie64_to_vms32(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE 64 bits reals to VMS 32 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_vms32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_ie64_to_vms32 : conversion non implemented\n");
}
#else
nxconv_real_ie64_to_vms32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_ie64_to_vms32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_vms32_to_ie64(S,D,NB)            */
/*                                                                 */
/* Conversion of VMS 32 bits reals to IEEE 64 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms32_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms32_to_ie64 : conversion non implemented\n");
}
#else
nxconv_real_vms32_to_ie64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms32_to_ie64 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie64_to_vms64(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE 64 bits reals to VMS 64 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_vms64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_ie64_to_vms64 : conversion non implemented\n");
}
#else
nxconv_real_ie64_to_vms64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_ie64_to_vms64 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_vms64_to_ie64(S,D,NB)            */
/*                                                                 */
/* Conversion of VMS 64 bits reals to IEEE 64 bits reals           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms64_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms64_to_ie64 : conversion non implemented\n");
}
#else
nxconv_real_vms64_to_ie64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms64_to_ie64 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie128_to_vms32(S,D,NB)           */
/*                                                                 */
/* Conversion of IEEE 128 bits reals to VMS 32 bits reals          */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_vms32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_ie128_to_vms32 : conversion non implemented\n");
}
#else
nxconv_real_ie128_to_vms32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_ie128_to_vms32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie128_to_vms64(S,D,NB)           */
/*                                                                 */
/* Conversion of IEEE 128 bits reals to VMS 64 bits reals          */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_vms64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_ie128_to_vms64 : conversion non implemented\n");
}
#else
nxconv_real_ie128_to_vms64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_ie128_to_vms64 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_vms32_to_ie128(S,D,NB)           */
/*                                                                 */
/* Conversion of VMS 32 bits reals to IEEE 128 bits reals          */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms32_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms32_to_ie128 : conversion non implemented\n");
}
#else
nxconv_real_vms32_to_ie128(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms32_to_ie128 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_vms64_to_ie128(S,D,NB)           */
/*                                                                 */
/* Conversion of VMS 64 bits reals to IEEE 128 bits reals          */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms64_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms64_to_ie128 : conversion non implemented\n");
}
#else
nxconv_real_vms64_to_ie128(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms64_to_ie128 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_li32_to_vms32(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE little endian 32 bits reals to VMS 32 bits   */
/* reals sur machines ou existe un type entier non signe 32 bits   */
/* en C ou sur CRAY CRI                                            */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li32_to_vms32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_li32_to_vms32 : conversion non implemented\n");
}
#else
nxconv_real_li32_to_vms32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_li32_to_vms32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_li32_to_vms64(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE little endian 32 bits reals to VMS 64 bits   */
/* reals sur machines ou existe un type entier non signe 32 bits   */
/* en C ou sur CRAY CRI                                            */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li32_to_vms64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_li32_to_vms64 : conversion non implemented\n");
}
#else
nxconv_real_li32_to_vms64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_li32_to_vms64 : conversion non implemented\n");
}
#endif

/***********************************************************************/
/*                    nxconv_real_vms32_to_li32(S,D,NB)                */
/*                                                                     */
/* Conversion of VMS 32 bits reals to IEEE 32 bits reals little endian */
/* sur machines ou existe un type entier non signe 32 bits en C        */
/* ou sur CRAY CRI                                                     */
/*                                                                     */
/***********************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms32_to_li32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms32_to_li32 : conversion non implemented\n");
}
#else
nxconv_real_vms32_to_li32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms32_to_li32 : conversion non implemented\n");
}
#endif

/***********************************************************************/
/*                    nxconv_real_vms64_to_li32(S,D,NB)                */
/*                                                                     */
/* Conversion of VMS 64 bits reals to IEEE 32 bits reals little endian */
/* sur machines ou existe un type entier non signe 32 bits en C        */
/* ou sur CRAY CRI                                                     */
/*                                                                     */
/***********************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms64_to_li32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms64_to_li32 : conversion non implemented\n");
}
#else
nxconv_real_vms64_to_li32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms64_to_li32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_li64_to_vms32(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE little endian 64 bits reals to VMS 32 bits   */
/* reals sur machines ou existe un type entier non signe 32 bits   */
/* en C ou sur CRAY CRI                                            */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li64_to_vms32(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_li64_to_vms32 : conversion non implemented\n");
}
#else
nxconv_real_li64_to_vms32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_li64_to_vms32 : conversion non implemented\n");
}
#endif

/*******************************************************************/
/*                    nxconv_real_li64_to_vms64(S,D,NB)            */
/*                                                                 */
/* Conversion of IEEE little endian 64 bits reals to VMS 64 bits   */
/* reals sur machines ou existe un type entier non signe 32 bits   */
/* en C ou sur CRAY CRI                                            */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li64_to_vms64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_li64_to_vms64 : conversion non implemented\n");
}
#else
nxconv_real_li64_to_vms64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_li64_to_vms64 : conversion non implemented\n");
}
#endif

/***********************************************************************/
/*                    nxconv_real_vms32_to_li64(S,D,NB)                */
/*                                                                     */
/* Conversion of VMS 32 bits reals to IEEE 64 bits reals little endian */
/* sur machines ou existe un type entier non signe 32 bits en C        */
/* ou sur CRAY CRI                                                     */
/*                                                                     */
/***********************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms32_to_li64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms32_to_li64 : conversion non implemented\n");
}
#else
nxconv_real_vms32_to_li64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms32_to_li64 : conversion non implemented\n");
}
#endif

/***********************************************************************/
/*                    nxconv_real_vms64_to_li64(S,D,NB)                */
/*                                                                     */
/* Conversion of VMS 64 bits reals to IEEE 64 bits reals little endian */
/* sur machines ou existe un type entier non signe 32 bits en C        */
/* ou sur CRAY CRI                                                     */
/*                                                                     */
/***********************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_vms64_to_li64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb,
		int* cr)
{
	printf("nxconv_real_vms64_to_li64 : conversion non implemented\n");
}
#else
nxconv_real_vms64_to_li64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	printf("nxconv_real_vms64_to_li64 : conversion non implemented\n");
}
#endif
