/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/* Modifs Pierre Brochard 1997 - 1998 */

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxfonc.h"
#include "nxfrmt.h"
#include "nxconv.h"

/* ------------------------------------------------------------ */
/* init global variables                                        */
/* ------------------------------------------------------------ */

DESC nx_desc_table[NX_MAX_DESC] = {0};

FILE* nx_fdin_table[NX_MAX_DESC] = {0};
FILE* nx_fdout_table[NX_MAX_DESC] = {0};

const ACCESS* nx_ptio[NX_MAX_DESC] = {0};

/* ============================================================ */
/*			====>     INIT_ACCES   <====		*/ 
/* ============================================================ */

int
nx_init_access(int nf,int fmt)
{
	const ACCESS* const pt_access_xdr = nx_init_access_xdr();

#if defined(CRAY) && defined(_CRAYMPP)
#       define NXMAX_ACCESS_TABLE 7
	const ACCESS* access_table[NXMAX_ACCESS_TABLE];
	const ACCESS* const pt_access_cr64 = nx_init_access_mpp_cr64();
	const ACCESS* const pt_access_ie32 = nx_init_access_mpp_ie32();
	const ACCESS* const pt_access_li32 = nx_init_access_mpp_li32();
	const ACCESS* const pt_access_ie64 = nx_init_access_mpp_ie64();
	const ACCESS* const pt_access_li64 = nx_init_access_mpp_li64();

	access_table[0] = pt_access_ie64;
	access_table[1] = pt_access_cr64;
	access_table[2] = pt_access_ie32;
	access_table[3] = pt_access_li32;
	access_table[4] = pt_access_ie64;
	access_table[5] = pt_access_li64;
#elif defined(CRAY) && !defined(_CRAYIEEE)
#       define NXMAX_ACCESS_TABLE 7
	const ACCESS* access_table[NXMAX_ACCESS_TABLE];
	const ACCESS* const pt_access_cr64 = nx_init_access_same();
	const ACCESS* const pt_access_ie32 = nx_init_access_cray_ie32();
	const ACCESS* const pt_access_li32 = nx_init_access_cray_li32();
	const ACCESS* const pt_access_ie64 = nx_init_access_cray_ie64();
	const ACCESS* const pt_access_li64 = nx_init_access_cray_li64();

	access_table[0] = pt_access_cr64;
	access_table[1] = pt_access_cr64;
	access_table[2] = pt_access_ie32;
	access_table[3] = pt_access_li32;
	access_table[4] = pt_access_ie64;
	access_table[5] = pt_access_li64;
#elif defined(CRAY)
#       define NXMAX_ACCESS_TABLE 7
	const ACCESS* access_table[NXMAX_ACCESS_TABLE];
	const ACCESS* const pt_access_cr64 = nx_init_access_cray_cr64();
	const ACCESS* const pt_access_ie32 = nx_init_access_cray_ie32();
	const ACCESS* const pt_access_li32 = nx_init_access_cray_li32();
	const ACCESS* const pt_access_ie64 = nx_init_access_same();
	const ACCESS* const pt_access_li64 = nx_init_access_cray_li64();

	access_table[0] = pt_access_ie64;
	access_table[1] = pt_access_cr64;
	access_table[2] = pt_access_ie32;
	access_table[3] = pt_access_li32;
	access_table[4] = pt_access_ie64;
	access_table[5] = pt_access_li64;
#elif defined(IEEE) && !defined(NX_LITTLEENDIAN)
#       define NXMAX_ACCESS_TABLE 8
	const ACCESS* access_table[NXMAX_ACCESS_TABLE];
	const ACCESS* const pt_access_cr64 = nx_init_access_ieee_cr64();
	const ACCESS* const pt_access_ie32 = nx_init_access_ieee_ie32();
	const ACCESS* const pt_access_li32 = nx_init_access_ieee_li32();
	const ACCESS* const pt_access_ie64 = nx_init_access_ieee_ie64();
	const ACCESS* const pt_access_li64 = nx_init_access_ieee_li64();
	const ACCESS* const pt_access_vms32 = nx_init_access_ieee_vms32();

	access_table[0] = pt_access_ie32;
	access_table[1] = pt_access_cr64;
	access_table[2] = pt_access_ie32;
	access_table[3] = pt_access_li32;
	access_table[4] = pt_access_ie64;
	access_table[5] = pt_access_li64;
	access_table[6] = pt_access_vms32;
#elif defined(NX_LITTLEENDIAN)
#       define NXMAX_ACCESS_TABLE 7
	const ACCESS* access_table[NXMAX_ACCESS_TABLE];
	const ACCESS* const pt_access_cr64 = nx_init_access_little_cr64();
	const ACCESS* const pt_access_ie32 = nx_init_access_little_ie32();
	const ACCESS* const pt_access_li32 = nx_init_access_little_li32();
	const ACCESS* const pt_access_ie64 = nx_init_access_little_ie64();
	const ACCESS* const pt_access_li64 = nx_init_access_little_li64();

	access_table[0] = pt_access_li32;
	access_table[1] = pt_access_cr64;
	access_table[2] = pt_access_ie32;
	access_table[3] = pt_access_li32;
	access_table[4] = pt_access_ie64;
	access_table[5] = pt_access_li64;
#endif

	access_table[NXMAX_ACCESS_TABLE-1] = pt_access_xdr;

	if(fmt < NXMAX_ACCESS_TABLE)
	{
	    nx_ptio[nf] = access_table[fmt];

	    return 0;
	}
	else
	{
	    NXERRNO = 75;
	    nx_ptio[nf] = access_table[0];

	    return -1;
	}

}

void
_nx_wfc_not_implemented(FILE* fd,int nb,const char* buff,int* rc)
{
        fprintf(stderr,"Specific write function not implemented\n");
        *rc = -1;
}

void
_nx_nblk_wfc_not_implemented(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
        fprintf(stderr,"Specific write function not implemented\n");
        *rc = -1;
}

void
_nx_rfc_not_implemented(FILE* fd,int nb,char* buff,int* rc)
{
        fprintf(stderr,"Specific read function not implemented\n");
        *rc = -1;
}

void
_nx_nblk_rfc_not_implemented(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
        fprintf(stderr,"Specific read nblk function not implemented\n");
        *rc = -1;
}
