/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/* Modifs Pierre Brochard 1997 */

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxfonc.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"

/* sources pour machines ieee 32/64 bits big endian :
   conversions vers et de machines CRAY CRI */

#include "nxconvP.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

/* sur machines IEEE 32 bits big endian :
   int est ie32
   type est ie32 ou ie64
   float est ie32 = NXFLT
   double est ie64 = NXDBLE
   long double est ie128
   */

#if defined(NX_MACH_INT_IE32)
static void nx_int_ie32_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_ie32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_ie32_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_ie32_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(NX_MACH_FLT_IE32)
static void nx_float_ie32_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_ie32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_ie32_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_ie32_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_cr64_cr128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,
		int nb,const char* icod,const char* ocod,int* nblus,int* rc);
#endif

#if defined(NX_MACH_FLT_IE64)
static void nx_float_ie64_from_cr64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_ie64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_ie64_to_cr64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_ie64_to_cr64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
static void nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_cr64_cr128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);
#endif

#if defined(CRAY) && defined(_CRAYMPP)
static void nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_cr64_cr128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);
#endif

/* ------------------------------------------------------------- */
/*  init_access for big endian ieee 32/64 bits machines */
/* ------------------------------------------------------------- */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
const ACCESS*
nx_init_access_ieee_cr64(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    nx_int_ie32_from_cr64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_ie32_from_cr64,
#endif
	    NULL,
	    nx_float_ie32_from_cr64,
	    nx_float_ie64_from_cr64,
	    nx_type_from_cr64_cr128,
	    nx_int_nblk_ie32_from_cr64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_ie32_from_cr64,
#endif
	    NULL,
	    nx_float_nblk_ie32_from_cr64,
	    nx_float_nblk_ie64_from_cr64,
	    nx_type_nblk_from_cr64_cr128,
	    NULL,
	    nx_int_ie32_to_cr64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_ie32_to_cr64,
#endif
	    NULL,
	    nx_float_ie32_to_cr64,
	    nx_float_ie64_to_cr64,
	    nx_type_to_cr64_cr128,
	    nx_int_nblk_ie32_to_cr64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_ie32_to_cr64,
#endif
	    NULL,
	    nx_float_nblk_ie32_to_cr64,
	    nx_float_nblk_ie64_to_cr64,
	    nx_type_nblk_to_cr64_cr128
	};

	return &my_access;
}
#endif

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
const ACCESS*
nx_init_access_cray_cr64(void)
{

	static  ACCESS my_access =
	{
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie64_from_cr64,
	    nx_float_ie64_from_cr64,
	    nx_type_from_cr64_cr128,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie64_from_cr64,
	    nx_float_nblk_ie64_from_cr64,
	    nx_type_nblk_from_cr64_cr128,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie64_to_cr64,
	    nx_float_ie64_to_cr64,
	    nx_type_to_cr64_cr128, 
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie64_to_cr64,
	    nx_float_nblk_ie64_to_cr64,
	    nx_type_nblk_to_cr64_cr128
	};

	return &my_access;
}
#endif

#if defined(CRAY) && defined(_CRAYMPP)
const ACCESS*
nx_init_access_mpp_cr64(void)
{

	static  ACCESS my_access =
	{
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie32_from_cr64,
	    nx_float_ie64_from_cr64,
	    nx_type_from_cr64_cr128,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie32_from_cr64,
	    nx_float_nblk_ie64_from_cr64,
	    nx_type_nblk_from_cr64_cr128,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie32_to_cr64,
	    nx_float_ie64_to_cr64,
	    nx_type_to_cr64_cr128, 
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie32_to_cr64,
	    nx_float_nblk_ie64_to_cr64,
	    nx_type_nblk_to_cr64_cr128
	};

	return &my_access;
}
#endif

/*               Specific access functions                    */
/*               -------------------------                    */

/* ----------------------------------------------------------------------- */
/* 		          IEEE 32 bits - CRAY CRI                          */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_IE32_FROM_CR64 <====                       */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE32)
static void
nx_int_ie32_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_ie32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_int_ie32_from_cr64"),
			"nx_int_ie32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE32_FROM_CR64 <====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_ie32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_int_nblk_ie32_from_cr64"),
			nblus,"nx_int_nblk_ie32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE32_FROM_CR64  <====                    */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE32)
static void
nx_float_ie32_from_cr64(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_ie32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_ie32_from_cr64"),
			"nx_float_ie32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_FROM_CR64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_ie32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_ie32_from_cr64"),
			nblus,"nx_float_nblk_ie32_from_cr64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_CR64_CR128   <====                   */
/* ----------------------------------------------------------------------- */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void
nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float ieee */
    /* conversion float cray - double ieee */
    /* conversion double cray - float ieee */
    /* conversion double cray - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie64(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_CR64_CR128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float ieee */
    /* conversion float cray - double ieee */
    /* conversion double cray - float ieee */
    /* conversion double cray - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie64(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_IE32_TO_CR64    <====                      */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE32)
static void
nx_int_ie32_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_int_ie32_to_cr64(buff,buff2,nbt),
			"nx_int_ie32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE32_TO_CR64    <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_int_ie32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_ie32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE32_TO_CR64    <====                    */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE32)
static void
nx_float_ie32_to_cr64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_real_ie32_to_cr64(buff,buff2,nbt),
			"nx_float_ie32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_TO_CR64    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,NX_BUF_SIZE_64,
			nxconv_real_ie32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_ie32_to_cr64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_CR64_CR128    <====                    */
/* ----------------------------------------------------------------------- */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void
nx_type_to_cr64_cr128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion double ieee - float cray */
    /* conversion float ieee - double cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie32_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_CR64_CR128    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion double ieee - float cray */
    /* conversion float ieee - double cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_real_ie64_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_ie32_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		         IEEE 64 bits - CRAY CRI                           */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_IE64_FROM_CR64    <====               */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE64)
static void 
nx_float_ie64_from_cr64(FILE *fd, int nb, char* buff, int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_ie64(buff2,buff,nbt),
			"nx_float_ie64_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_NBLK_IE64_FROM_CR64    <====          */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie64_from_cr64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_real_cr64_to_ie64(buff2,buff,n),
			nblus,"nx_float_nblk_ie64_from_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE64_TO_CR64  <=====                     */
/* ----------------------------------------------------------------------- */

static void 
nx_float_ie64_to_cr64(FILE *fd, int nb,const char *buff, int *rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_cr64(buff,buff2,nbt),
			"nx_float_ie64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE64_TO_CR64  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie64_to_cr64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = nb*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_ie64_to_cr64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_CR64_CR128   <====                   */
/* ----------------------------------------------------------------------- */

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
static void
nx_type_from_cr64_cr128(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod,int* rc)
{
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_cr64_to_ie64(buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie128(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie128(
							buff2,buff,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_CR64_CR128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_cr64_to_ie64(buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie128(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie128(
							buff2,buff,n),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_TYPE_TO_CR64_CR128  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_cr64_cr128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_ie64_to_cr64(buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie128_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_TYPE_NBLK_TO_CR64_CR128  <====                */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_cr64_cr128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					nxconv_real_ie64_to_cr64(buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_real_ie128_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_ie128_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		                      MPP - CRAY CRI                       */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_CR64_CR128   <====                   */
/* ----------------------------------------------------------------------- */

#if defined(CRAY) && defined(_CRAYMPP)
static void
nx_type_from_cr64_cr128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre cray - mpp :
       On lit en CRAY 64 bits ou 128 et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_cr64_cr128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie64(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_CR64_CR128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_cr64_cr128(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre cray - mpp :
       On lit en CRAY 64 bits ou 128 et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_cr64_cr128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_64,nxconv_real_cr64_to_ie64(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_64,NX_BUF_SIZE_128,
					nxconv_real_cr128_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_CR64_CR128    <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_cr64_cr128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre MPP - CRAY :
       On convertit du MPP et on ecrit en CRAY CRI
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc CRAY
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_cr64_cr128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_cr64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_ie32_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_CR64_CR128    <====               */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_cr64_cr128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre MPP - CRAY :
       On convertit du MPP et on ecrit en CRAY CRI
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc CRAY
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_cr64_cr128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_64,NX_BUF_SIZE_64,
					nxconv_real_ie64_to_cr64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_32,NX_BUF_SIZE_128,
					nxconv_real_ie32_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_64,NX_BUF_SIZE_128,
					nxconv_real_ie64_to_cr128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif
