#! /usr/bin/python3

import os;
from cffi import FFI
ffibuilder = FFI()

# cdef() expects a single string declaring the C types, functions and
# globals needed to use the shared object. It must be in valid C syntax.
ffibuilder.cdef("""
    typedef float REEL_S; /* 32 bits */
    typedef double REEL; /* 64 bits */
    typedef float REEL4; /* 32 bits */
    typedef double REEL8; /* 64 bits */
    typedef struct SPXFILE SPXFILE;

    SPXFILE* SpxOpen(char* nom_fichier);
    void SpxClose(SPXFILE* spx_file);
    void SpxSync(SPXFILE* spx_file);

    void SpxMonoCourbe(REEL_S x[],REEL_S y[],int nx,SPXFILE* spx_file);
    void SpxDMonoCourbe(REEL x[],REEL y[],int nx,SPXFILE* spx_file);
    void SpxMulCourbe(REEL_S x[],REEL_S y[],int nx[],int ny,SPXFILE* spx_file);
    void SpxDMulCourbe(REEL x[],REEL y[],int nx[],int ny,SPXFILE* spx_file);

    void SpxRect3d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		SPXFILE* spx_file);
    void SpxDRect3d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		SPXFILE* spx_file);

    void SpxRect4d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		REEL_S c[],int dimc,SPXFILE* spx_file);
    void SpxDRect4d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		REEL c[],int dimc,SPXFILE* spx_file);

    void SpxPrintSurf(int nx,int ny,REEL_S z[],int dimz,SPXFILE* spx_file);
    void SpxDPrintSurf(int nx,int ny,REEL z[],int dimz,SPXFILE* spx_file);

    void SpxParRect4d(int nx,int ny,int nz,REEL_S x[],REEL_S y[],REEL_S z[],
		REEL_S c[],int dimc1,int dimc2,SPXFILE* spx_file);
    void SpxDParRect4d(int nx,int ny,int nz,REEL x[],REEL y[],REEL z[],
		REEL c[],int dimc1,int dimc2,SPXFILE* spx_file);

    void SpxNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		const int mailles[],SPXFILE* spx_file);
    void SpxMailleNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		const int mailles[],SPXFILE* spx_file);
    void SpxBrasNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		const int mailles[],int nbras,const int bras[],
		SPXFILE* spx_file);

    void SpxDNonStruc4d(int npoints,const REEL x[],const REEL y[],const REEL z[],
		const REEL c[],int nmailles,int dimmailles,const int mailles[],
		SPXFILE* spx_file);
    void SpxDMailleNonStruc4d(int npoints,const REEL x[],const REEL y[],
		const REEL z[],const REEL c[],int nmailles,int dimmailles,
		const int mailles[],SPXFILE* spx_file);
    void SpxDBrasNonStruc4d(int npoints,const REEL x[],const REEL y[],
		const REEL z[],const REEL c[],int nmailles,int dimmailles,
		const int mailles[],int nbras,const int bras[],
		SPXFILE* spx_file);

    void SpxVPurgeArg(SPXFILE* spx_file);
    void SpxVSetValues(SPXFILE* spx_file);
    void SpxVSetArgInt(SPXFILE* spx_file,char* name,int value);
    void SpxVSetArgReel(SPXFILE* spx_file,char* name,REEL_S value);
    void SpxVSetArgChar(SPXFILE* spx_file,char* name,char* value);
    void SpxVSetArgTabInt(SPXFILE* spx_file,char* name,int value,int indice);
    void SpxVSetArgTabReel(SPXFILE* spx_file,char* name,REEL_S value,int indice);
    void SpxVSetArgTabChar(SPXFILE* spx_file,char* name,char* value,int indice);

    void SpxSetColor(const char* type,const char* tabcol,SPXFILE* spx_file);

""")

libpath = os.environ['PROJECT_HOME'] if 'PROJECT_HOME' in os.environ else ''
myarch = os.environ['ARCH'] if 'ARCH' in os.environ else ''
myopt = os.environ['OPT'] if 'OPT' in os.environ else ''
narlibs = libpath + '/lib/' + myarch + '/' + myopt;

# set_source() gives the name of the python extension module to
# produce, and some C source code as a string.  This C code needs
# to make the declarated functions, types and globals available,
# so it is often just the "#include".
ffibuilder.set_source(
    "_pi_narcisse",
    """
    #include "SpxInclude.h"   // the C header of the library
    """,
    libraries=['Xau','X11','Xt','xcb','SM','ICE','belier','nx'], # library name, for the linker
    library_dirs=[narlibs]
)

if __name__ == "__main__":
    ffibuilder.compile(verbose=True)
