#ifndef SPXOBJETS_H
#define SPXOBJETS_H

/* @(#)SpxObjets.h	1.3 11/18/10 Pierre Brochard */

#include "SpxOpen.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ox,oy,oz = coordonnees du centre du ou des cercles ou spheres, ou origine
   du ou des vecteurs

   ray rayon du ou des cercles ou spheres

   ox1,oy1,oz1 premier point definissant l'axe du cylindre
   ox2,oy2,oz2 deuxieme point definissant l'axe du cylindre

   vx,vy,vz direction du ou des vecteurs

   nval nombre de points pour les sous-programmes *Collection

   rmin,rmax parametres de renormalisation des rayons ou norme
   de direction de vecteurs (si rmin = rmax = 0., on ne renormalise pas)

   def nombre de noeuds pour les cercles (qui sont approches par des
   polygones) ou nombre de noeuds sur un meridien et la moitie
   du nombre de noeuds sur un parallele pour une sphere (qui est maillee)

   spx_file celui renvoye par SpxOpen

   pour SpxCollectionSpheres4d : oc est de dim 2*def*def*nval et donne une
   valeur a chaque noeud des spheres maillees generees 
   i varie d'abord de 0 a def-1, j ensuite de 0 2*def-1, k varie
   de 0 a nval-1 */

void SpxObjetSphere(REEL_S ox,REEL_S oy,REEL_S oz,REEL_S ray,
		REEL_S valc,unsigned def,SPXFILE* spx_file);
void SpxObjetCylindre(REEL_S ox1,REEL_S oy1,REEL_S oz1,REEL_S decal1,
		REEL_S ox2,REEL_S oy2,REEL_S oz2,REEL_S decal2,REEL_S ray,
		REEL_S valc,unsigned def,SPXFILE* spx_file);
void SpxObjetCylindre4d(REEL_S ox1,REEL_S oy1,REEL_S oz1,REEL_S decal1,
		REEL_S ox2,REEL_S oy2,REEL_S oz2,REEL_S decal2,REEL_S ray,
		const REEL_S* const valc,unsigned def,SPXFILE* spx_file);
void SpxCollectionCercles(const REEL_S* const ox,const REEL_S* const oy,
		const REEL_S* const oz,const REEL_S* const oc,
		unsigned nval,REEL_S rmin,REEL_S rmax,unsigned def,
		SPXFILE* spx_file);
void SpxCollectionVecteurs(const REEL_S* const ox,const REEL_S* const oy,
		const REEL_S* const oz,const REEL_S* const oc,
		const REEL_S* const vx,const REEL_S* const vy,
		const REEL_S* const vz,unsigned nval,REEL_S rmin,REEL_S rmax,
		SPXFILE* spx_file);
void SpxCollectionSpheres(const REEL_S* const ox,const REEL_S* const oy,
		const REEL_S* const oz,const REEL_S* const oc,
		const REEL_S* const ray,unsigned nval,REEL_S rmin,
		REEL_S rmax,unsigned def,SPXFILE* spx_file);
void SpxCollectionSpheres4d(const REEL_S* const ox,const REEL_S* const oy,
		const REEL_S* const oz,const REEL_S* const oc,
		const REEL_S* const ray,unsigned nval,REEL_S rmin,REEL_S rmax,
		unsigned def,SPXFILE* spx_file);

void Spx4ObjetSphere(REEL4 ox,REEL4 oy,REEL4 oz,REEL4 ray,
		REEL4 valc,unsigned def,SPXFILE* spx_file);
void Spx4ObjetCylindre(REEL4 ox1,REEL4 oy1,REEL4 oz1,REEL4 decal1,
		REEL4 ox2,REEL4 oy2,REEL4 oz2,REEL4 decal2,REEL4 ray,
		REEL4 valc,unsigned def,SPXFILE* spx_file);
void Spx4ObjetCylindre4d(REEL4 ox1,REEL4 oy1,REEL4 oz1,REEL4 decal1,
		REEL4 ox2,REEL4 oy2,REEL4 oz2,REEL4 decal2,REEL4 ray,
		const REEL4* const valc,unsigned def,SPXFILE* spx_file);
void Spx4CollectionCercles(const REEL4* const ox,const REEL4* const oy,
		const REEL4* const oz,const REEL4* const oc,unsigned nval,
		REEL4 rmin,REEL4 rmax,unsigned def,SPXFILE* spx_file);
void Spx4CollectionVecteurs(const REEL4* const ox,const REEL4* const oy,
		const REEL4* const oz,const REEL4* const oc,
		const REEL4* const vx,const REEL4* const vy,
		const REEL4* const vz,unsigned nval,REEL4 rmin,REEL4 rmax,
		SPXFILE* spx_file);
void Spx4CollectionSpheres(const REEL4* const ox,const REEL4* const oy,
		const REEL4* const oz,const REEL4* const oc,
		const REEL4* const ray,unsigned nval,REEL4 rmin,
		REEL4 rmax,unsigned def,SPXFILE* spx_file);
void Spx4CollectionSpheres4d(const REEL4* const ox,const REEL4* const oy,
		const REEL4* const oz,const REEL4* const oc,
		const REEL4* const ray,unsigned nval,REEL4 rmin,
		REEL4 rmax,unsigned def,SPXFILE* spx_file);

void Spx8ObjetSphere(REEL8 ox,REEL8 oy,REEL8 oz,REEL8 ray,
		REEL8 valc,unsigned def,SPXFILE* spx_file);
void Spx8ObjetCylindre(REEL8 ox1,REEL8 oy1,REEL8 oz1,REEL8 decal1,
		REEL8 ox2,REEL8 oy2,REEL8 oz2,REEL8 decal2,REEL8 ray,
		REEL8 valc,unsigned def,SPXFILE* spx_file);
void Spx8ObjetCylindre4d(REEL8 ox1,REEL8 oy1,REEL8 oz1,REEL8 decal1,
		REEL8 ox2,REEL8 oy2,REEL8 oz2,REEL8 decal2,REEL8 ray,
		const REEL8* const valc,unsigned def,SPXFILE* spx_file);
void Spx8CollectionCercles(const REEL8* const ox,const REEL8* const oy,
		const REEL8* const oz,const REEL8* const oc,
		unsigned nval,REEL8 rmin,REEL8 rmax,
		unsigned def,SPXFILE* spx_file);
void Spx8CollectionVecteurs(const REEL8* const ox,const REEL8* const oy,
		const REEL8* const oz,const REEL8* const oc,
		const REEL8* const vx,const REEL8* const vy,
		const REEL8* const vz,unsigned nval,REEL8 rmin,REEL8 rmax,
		SPXFILE* spx_file);
void Spx8CollectionSpheres(const REEL8* const ox,const REEL8* const oy,
		const REEL8* const oz,const REEL8* const oc,
		const REEL8* const ray,unsigned nval,REEL8 rmin,
		REEL8 rmax,unsigned def,SPXFILE* spx_file);
void Spx8CollectionSpheres4d(const REEL8* const ox,const REEL8* const oy,
		const REEL8* const oz,const REEL8* const oc,
		const REEL8* const ray,unsigned nval,REEL8 rmin,
		REEL8 rmax,unsigned def,SPXFILE* spx_file);

void SpxDObjetSphere(REEL ox,REEL oy,REEL oz,REEL ray,
		REEL valc,unsigned def,SPXFILE* spx_file);
void SpxDObjetCylindre(REEL ox1,REEL oy1,REEL oz1,REEL decal1,
		REEL ox2,REEL oy2,REEL oz2,REEL decal2,REEL ray,
		REEL valc,unsigned def,SPXFILE* spx_file);
void SpxDObjetCylindre4d(REEL ox1,REEL oy1,REEL oz1,REEL decal1,
		REEL ox2,REEL oy2,REEL oz2,REEL decal2,REEL ray,
		const REEL* const valc,unsigned def,SPXFILE* spx_file);
void SpxDCollectionCercles(const REEL* const ox,const REEL* const oy,
		const REEL* const oz,const REEL* const oc,unsigned nval,
		REEL rmin,REEL rmax,unsigned def,SPXFILE* spx_file);
void SpxDCollectionVecteurs(const REEL* const ox,const REEL* const oy,
		const REEL* const oz,const REEL* const oc,
		const REEL* const vx,const REEL* const vy,
		const REEL* const vz,unsigned nval,REEL rmin,REEL rmax,
		SPXFILE* spx_file);
void SpxDCollectionSpheres(const REEL* const ox,const REEL* const oy,
		const REEL* const oz,const REEL* const oc,
		const REEL* const ray,unsigned nval,REEL rmin,
		REEL rmax,unsigned def,SPXFILE* spx_file);
void SpxDCollectionSpheres4d(const REEL* const ox,const REEL* const oy,
		const REEL* const oz,const REEL* const oc,
		const REEL* const ray,unsigned nval,REEL rmin,
		REEL rmax,unsigned def,SPXFILE* spx_file);

#ifdef __cplusplus
}
#endif

#endif
