#ifndef C5CONFIG_H
#define C5CONFIG_H

#include "C5constant.h"

// @(#)C5config.h	1.7 10/12/10 Pierre Brochard

class C5Config
{
    protected:

	static int     _vide_int;
	static REEL    _vide_REEL;
	static char*   _vide_ptchar;
	static int*    _vide_ptint;
	static REEL_S* _vide_ptreel;
	static char**  _vide_ptptchar;
	char* _name;

    public:

	C5Config() ;
	C5Config(const char* const name) ;
	C5Config(const char* const ident,const char* const name) ;
	C5Config(const C5Config& c) ;
	virtual ~C5Config() ;

	char* Name() const {return _name;};
	void Name(char* name) {_name = name;};
	virtual int Val(void* const,const int,int&) {return 0;};

	virtual void Enleve(char*) ;

	virtual int     Int() const;
	virtual REEL    Reel() const;
	virtual char*   Char() const;
	virtual int*    TabInt() const;
	virtual REEL_S* TabReel() const;
	virtual char**  TabChar() const;

	virtual int     Taille() const;

	int operator <  (const C5Config& c);
	int operator == (const C5Config& c);

	virtual void operator = (const C5Config& c);

	virtual int&     IntA() {return _vide_int;};
	virtual REEL&    ReelA() {return _vide_REEL;};
	virtual char*&   CharA() {return _vide_ptchar;};
	virtual int*&    TabIntA() {return _vide_ptint;};
	virtual REEL_S*& TabReelA() {return _vide_ptreel;};
	virtual char**&  TabCharA() {return _vide_ptptchar;};

	virtual istream& operator >> (istream& ist);
	virtual ostream& operator << (ostream& ost) const;
};

class C5ConfigInt : public C5Config
{
    protected:

	int _val;

    public:

	C5ConfigInt() ;
	C5ConfigInt(const char* const name,const int val) ;
	C5ConfigInt(const char* const ident,const char* const name,
			const int val) ;
	C5ConfigInt(const C5ConfigInt& c) ;
	~C5ConfigInt() ;

	int   Val(void* const val,const int i,int& taille);

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c) ;
	int&  IntA()  {return _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

class C5ConfigReel : public C5Config 
{
    protected:

	REEL _val;

    public:

	C5ConfigReel() ;
	C5ConfigReel(const char* const name,const REEL val) ;
	C5ConfigReel(const char* const ident,const char* const name,
			const REEL val) ;
	C5ConfigReel(const C5ConfigReel& c) ;
	~C5ConfigReel() ;

	int   Val(void* const val,const int i,int& taille);

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c) ;
	REEL& ReelA()  {return _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

class C5ConfigChar : public C5Config
{
    protected:

	char* _val;

    public:

    // ***********************************************************
    //   classe generale : tout type de couple (motcle, valeur) 
    //   peut-etre lu dans un objet de type C5ConfigChar
    //   contrairement aux autres qui s'attendent a un type
    //   particulier de donnees
    //   attention le format fichier et la chaine lu dans
    //   _val ont un format different pour les tableaux :
    //   (les crochets ([ et ]) sont enleves) 
    //   et les chaines de caracteres (les " sont enleves)
    // ***********************************************************

	C5ConfigChar() ;
	C5ConfigChar(const char* const name,const char* const val) ;
	C5ConfigChar(const char* const ident,const char* const name,
			const char* const val) ;
	C5ConfigChar(const C5ConfigChar& c) ;
	~C5ConfigChar() ;

	int    Val(void* const val,const int i,int& taille);

	void Enleve(char* val) ;

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c) ;
	char*& CharA()  {return _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

class C5ConfigVoid : public C5Config
{
    protected:

	char* _val;

    public:

    // ***********************************************************
    //   tout ce qui n'est pas dans _name est pris dans le meme
    //   format que pour le fichier
    //   attention toutes les fonctions virtuelles ne
    //   sont pas surdefinies
    // ***********************************************************

	C5ConfigVoid() ;
	~C5ConfigVoid() ;

	char*& CharA()  {return _val;};

	istream& operator >> (istream& ist);

    private:

	int    Val(void* const,const int,int&) {return 0;};

	void Enleve(char*) {};

	int     Int() const {return 0;};
	REEL    Reel() const {return 0;};
	char*   Char() const {return NULL;};
	int*    TabInt() const {return NULL;};
	REEL_S* TabReel() const {return NULL;};
	char**  TabChar() const {return NULL;};

	int Taille() const {return 0;};

	void operator = (const C5Config&) {};

	ostream& operator << (ostream& ost) const {return ost;};
};

class C5ConfigTabInt : public C5Config
{
    protected:

	int  _taille;
	int* _val;

    public:

	C5ConfigTabInt() ;
	C5ConfigTabInt(const char* const name,const int taille,
			const int* const val) ;
	C5ConfigTabInt(const char* const ident,const char* const name,
			const int taille,const int* const val) ;
	C5ConfigTabInt(const C5ConfigTabInt& c) ;
	~C5ConfigTabInt() ;

	int&  Tai()  {return _taille;};
	int   Val(void* const val,const int i,int& taille);

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c);

    // modif taille du tableau avec conservation des anciennes valeurs :
	void operator()(int taille,int defaut);

	int& IntA() {return _taille;};
	int*& TabIntA()  {return _val;};
	char*& CharA() {return (char*&) _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

class C5ConfigTabReel : public C5Config
{
    protected:

	int     _taille;
	REEL_S* _val;

    public:

	C5ConfigTabReel() ;
	C5ConfigTabReel(const char* const name,const int taille,
			const REEL_S* const val) ;
	C5ConfigTabReel(const char* const ident,const char* const name,
			const int taille,const REEL_S* const val) ;
	C5ConfigTabReel(const C5ConfigTabReel& c) ;
	~C5ConfigTabReel() ;

	int&     Tai()  {return _taille;};
	int      Val(void* const val,const int i,int& taille);

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c) ;

    // modif taille du tableau avec conservation des anciennes valeurs :
	void operator()(int taille,REEL_S delta,REEL_S coef);

	int& IntA() {return _taille;};
	REEL_S*& TabReelA()  {return _val;};
	char*& CharA() {return (char*&) _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

class C5ConfigTabChar : public C5Config
{
    protected:

	int     _taille;
	char**  _val;

    public:

	C5ConfigTabChar() ;
	C5ConfigTabChar(const char* const name,const int taille,
			const char** const val) ;
	C5ConfigTabChar(const char* const ident,const char* const name,
			const int taille,const char** const val) ;
	C5ConfigTabChar(const C5ConfigTabChar& c) ;
	~C5ConfigTabChar() ;

	int&     Tai()  {return _taille;};
	int      Val(void* const val,const int i,int& taille);

	int     Int() const;
	REEL    Reel() const;
	char*   Char() const;
	int*    TabInt() const;
	REEL_S* TabReel() const;
	char**  TabChar() const;

	int     Taille() const;

	void operator = (const C5Config& c) ;

    // modif taille du tableau avec conservation des anciennes valeurs :
	void operator()(int taille,const char* defaut);

	int& IntA() {return _taille;};
	char**& TabCharA()  {return _val;};
	char*& CharA() {return (char*&) (void*&) _val;};

	istream& operator >> (istream& ist);
	ostream& operator << (ostream& ost) const;
};

istream& operator >> (istream& ist,C5Config& c) ;
ostream& operator << (ostream& ost,const C5Config& c) ;

typedef C5Config* ptrC5Config;

inline ptrC5Config P4Log(const ptrC5Config& s) {return s;}
inline ptrC5Config P4Cvmgt(const ptrC5Config& x,const ptrC5Config& y,
		const P4Boolean t) {return t == TRUE ? x : y;}
#endif


