#ifndef NXSTREAM_H
#define NXSTREAM_H

// @(#)nxstream.h	1.4 03/10/21 Pierre Brochard

#include "nx.h"
#include "nxstreamstdio.h"

typedef stdiobuf nxstreambuf;

class nxstream : public iostream
{
    public:

	nxstream(FILE* f);
	nxstream(const char* name,const char* mode);
	nxstream(nxstream* nx_serveur,int mode_ostream = 0);
	nxstream(nxstream& nx_serveur);
	nxstream(nxstream& nx_serveur,char* handler_name);
	~nxstream();

	void set_shutdown(const int sd)
		{_shutdown = sd;if(_parent) _parent->set_shutdown(sd);};

	nxstream* get_parent() const {return _parent;};

	nxstreambuf* rdbuf();
	FILE* stdiofile();
	FILE* stdiofiler();
	FILE* stdiofilew();
	int numfile();

	virtual int DataInBuffer();

	void format(const char* frmt);

	istream& read(int buffer[],int count);
	istream& read(NXFLT buffer[],int count);
	istream& read(NXDBLE buffer[],int count);
#if defined(CRAY) && !defined(_CRAYMPP)
	istream& read(double buffer[],int count);
#endif
	istream& read(char buffer[],int& count);
	istream& nxget(char& car);

	ostream& write(const char* buffer,int count);
	ostream& wflush();
	ostream& write(const int buffer[],int count);
	ostream& write(const NXFLT buffer[],int count);
	ostream& write(const NXDBLE buffer[],int count);
#if defined(CRAY) && !defined(_CRAYMPP)
	ostream& write(const double buffer[],int count);
#endif

#if defined(__GNUC__)
    // BUG on ne peut employer les fonctions ci-dessous avec istream GNU
    // on les surdefini donc.

	istream& get(char& car);
	istream& get(char* buffer,int limit,char delim = '\n');
	istream& getline(char* buffer,int limit,char delim = '\n');

	int gcount();
#endif

    private:

	static FILE* init(const char* name,const char* mode);
	static FILE* init(nxstream& nx_serveur);
	static FILE* init(nxstream& nx_serveur,char* handler_name);

	void myreaderror();
	void mywriteerror();

    private:

	nxstream* const _parent;
	nxstreambuf buf;

#ifdef __GNUC__
	int _my_count;
#endif

    // Si shutdown vaut -1 nx ne fera pas un shutdown sur la socket
    // utile pour fork.
	int _shutdown;
};

// fonction membres inline

inline nxstreambuf*
nxstream::rdbuf()
{
	return &buf;
}

inline int
nxstream::numfile()
{
	return fileno(buf.stdiofile());
}

inline FILE*
nxstream::stdiofile()
{
	return buf.stdiofile();
}

inline FILE*
nxstream::stdiofiler()
{
	return fnx_read_desc(buf.stdiofile());
}

inline FILE*
nxstream::stdiofilew()
{
	return fnx_write_desc(buf.stdiofile());
}

inline void
nxstream::format(const char* frmt)
{
	fnxchfrm(buf.stdiofile(),frmt);
}

inline istream&
nxstream::nxget(char& car)
{

	if(good())
	{
	    FILE* const file = stdiofiler();
	    int lu;

	    U8GETC(file,lu);

	    if(lu != EOF)
		car = (char) lu;
	    else
	    {

		if(errno)
		    clear(ios::failbit | rdstate());
		else
		    clear(ios::eofbit | rdstate());

	    }

	}

	return *this;
}

#ifdef __GNUC__
// bug on ne peut employer le get de istream GNU
inline istream&
nxstream::get(char& car)
{

	if(good())
	{
	    FILE* const file = stdiofiler();
	    int lu = getc(file);

	    if(lu != EOF)
		car = (char) lu;
	    else
	    {

		if(errno)
		    clear(ios::failbit | rdstate());
		else
		    clear(ios::eofbit | rdstate());

	    }

	}

	return *this;
}
#endif

#endif
