#ifndef lint
static char sccsid[] = "%W% %G%";
#endif

#include "defs.h"
#include <sys/stat.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>

extern char Makecall;

int meta();
int doshell();
int doexec();
int metas();
void doclose();
void touch();

/* routines de misc.c */

void u8setenv();
void fatal();
void fatal1();

int
dosys(comstring, nohalt)
register CHARSTAR comstring;
int nohalt;
{
	register CHARSTAR p;
	int status;

	p = comstring;

	while(  *p == BLANK || *p == TAB) p++;

	if(!*p) return(-1);

	if(IS_ON(NOEX) && Makecall == NO) return(0);

	if(metas(comstring))
	    status = doshell(comstring,nohalt);
	else
	    status = doexec(comstring);

	return(status);
}

int
metas(s)   /* Are there are any  Shell meta-characters? */
register CHARSTAR s;
{

	while(*s) if( funny[(int) (*s++)] & META) return(YES);

	return(NO);
}

int
doshell(comstring,nohalt)
register CHARSTAR comstring;
register int nohalt;
{
	register CHARSTAR shell;
	int waitpid;

	if((waitpid = fork()) == 0)
	{
/*** le texier */
#if defined(CRAY) || defined(cray)
	    enbint(SCTL_IGN);
#endif
	    doclose();
	    u8setenv();
	    shell = varptr("SHELL")->varval;
	    if(shell == 0 || shell[0] == CNULL) shell = SHELLCOM;
/* Le Texier : certains MAKE transforment la variable SHELL en la
   macro SHELL (comme pour les autres variables) ce qui n'est 
   pas conforme a la norme POSIX ...
   */
	    if (IS_ON(DBUG)) 
		printf(" doshell : shell = %s %s\n",shell,
				(nohalt ? "-c" : "-ce"));
	    execl(shell, "sh", (nohalt ? "-c" : "-ce"), comstring, 0);
	    fatal("Couldn't load Shell");
	}

	return( waitpid );
}

void
doclose()       /* Close open directory files before exec'ing */
{
	register OPENDIR od;

	for (od = firstod; od != 0; od = od->nextopendir)
	    if (od->dirfc != NULL) closedir(od->dirfc);
}

int
doexec(str)
register CHARSTAR str;
{
	register CHARSTAR t;
	register CHARSTAR *p;
	CHARSTAR argv[300];
	int waitpid;

	while( *str==BLANK || *str==TAB ) ++str;
	if( *str == CNULL ) return(-1);     /* no command */

	p = argv;
	for(t = str ; *t ; )
	{
	    *p++ = t;
	    while(*t!=BLANK && *t!=TAB && *t!=CNULL) ++t;
	    if(*t) for( *t++ = CNULL ; *t==BLANK || *t==TAB  ; ++t);
	}

	*p = NULL;

	if((waitpid = fork()) == 0)
	{
/*** le texier */
#if defined(CRAY) || defined(cray)
	    enbint(SCTL_IGN);
#endif
	    doclose();
	    u8setenv();
	    execvp(str, argv);
	    fatal1("Cannot load %s",str);
	}

	return( waitpid );
}

void
touch(force, name)
register int force;
register char *name;
{
	struct stat stbuff;
	char junk[1];
	int fd;

	if( stat(name,&stbuff) < 0)
	    if(force)
		goto create;
	    else
	    {
		fprintf(stderr,"touch: file %s does not exist.\n",name);
		return;
	    }
	if(stbuff.st_size == 0) goto create;
	if( (fd = open(name, 2)) < 0) goto bad;
	if( read(fd, junk, 1) < 1)
	{
	    close(fd);
	    goto bad;
	}
	lseek(fd, 0L, 0);
	if( write(fd, junk, 1) < 1 )
	{
	    close(fd);
	    goto bad;
	}
	close(fd);
	return;
bad:
        fprintf(stderr, "Cannot touch %s\n", name);
        return;
create:
        if( (fd = creat(name, 0666)) < 0) goto bad;
        close(fd);
}

NAMEBLOCK
dowait ()
{
	int status, pid, j;
	NAMEBLOCK p;
	CHARSTAR comstring;

	if (waits == 0) fatal ("About to wait for nonexistent child");

	pid = wait (&status);

	if (pid == -1) 
	{
	    waits = 0;
	    exit(1);
	}

	for (j = 0; j < waits; j++) if (waitt[j].pid == pid) break;

	if (j == waits) fatal ("Internal error in dowait()\n");

	p = waitt [j].p;
	comstring = waitt [j].cmd;

	if (IS_ON(DBUG))
	{
	    printf ("dowait: `%s' (pid %d) is done\n", comstring, pid);
	    fflush (stdout);
	}

	if (status)
	{
	    if (waitt [j].makeblock)
	    {
		if ((status & 0377) == 0 && status >> 8 != 2)
/*
 * Some horrible error occurred in a child 'make' forked off to run a block
 */
		    exit (1);
	    }
	    else if( status>>8 )
		printf("Make: \"%s\": Error code %d", comstring, status>>8 );
	    else
		printf("Make: \"%s\": Termination code %d", comstring, status );

	    if(p->nohalt)
	    {
		printf(" (ignored)\n");
		p->done = 2;
	    }
	    else
	    {
		printf("\n");
		p->done = 3;
		if (IS_OFF(KEEPGO)) proceed = 0;
	    }
	    fflush(stdout);
	}
	else
	    p->done = 2;

	waits--;
	for (; j < waits; j++)
	{
	    waitt [j].p = waitt [j+1].p;
	    waitt [j].makeblock = waitt [j+1].makeblock;
	    waitt [j].pid = waitt [j+1].pid;
	    waitt [j].cmd = waitt [j+1].cmd;
	}
	waitt [waits].p = (NAMEBLOCK)0;
	waitt [waits].makeblock = 0;
	waitt [waits].pid = 0;
	waitt [waits].cmd = (CHARSTAR)0;
	free (comstring);

	return (p);
}
