#
# Makefile servant a fabriquer les composantes (toutes ou en partie)
# dans l'espace de modifications
# make doit etre execute depuis le repertoire racine de
# l'espace de modifications (au-dessus de src,bin,cfg,...)
#
# Version originale : Pierre Brochard CEA/DAM-Ile de France
# sccs_ID           : Modification.mk 1.2 11/02/25
#

# On lance un Bourne-shell
SHELL = /bin/sh

include $(MODIF_HOME)/cfg/machines/Default_Options.mk
include $(ATLAS_HOME)/resources/Global_Macros.mk
include $(PROJECT_HOME)/cfg/machines/Global_Macros.mk
include $(PROJECT_HOME)/cfg/machines/$(ARCH)/macros.mk
include $(PROJECT_HOME)/cfg/machines/$(ARCH)/options$(OPT).mk

# fabrication des libraries avec eventuellement un main
debut:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) debut;\
         done

# supression du main des librairies
otemain:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) otemain;\
         done

# fabrication des libraries sans main (debut + otemain)
libs:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) lib;\
         done

# fabrication des libraries avec eventuellement un main et edition de liens
load:
	@$(MAKE) debut
	@$(MAKE) link

# edition de liens
link:
	@$(MAKE) otemain
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) link;\
         done

# creation de l'espace de modifications
create:
	-@echo "---> appel de make create";\
        if [ "$(ATLAS_HOME)" != "" ] ;then\
          if [ "`printenv PATH | grep $ATLAS_HOME/bin`" = "" ] ;then\
            PATH=${ATLAS_HOME}/bin:`printenv PATH`;export PATH;\
          fi;\
          create_modification_space;\
          PARAMETER_FILE=cfg/machines/$(ARCH)/remote_project.sh;\
          if [ -f $$PARAMETER_FILE ] ;then\
            . $$PARAMETER_FILE;\
          fi;\
          if [ "$(MACHCOMP)" != "" ] ;then\
            if [ "$(MACHCOMP)" = "NONE" ] ;then\
              REMOTE_MACH=;\
            else\
              REMOTE_MACH=$(MACHCOMP);\
            fi;\
          fi;\
          if [ -z "$$REMOTE_MACH" ] ;then\
            REMOTE_MACH="`atlas_name`";\
          fi;\
          if [ "$(DIRCOMP)" != "" ] ;then\
            if [ "$(DIRCOMP)" = "NONE" ] ;then\
              REMOTE_DIR=;\
            else\
              REMOTE_DIR=$(DIRCOMP);\
            fi;\
          fi;\
          create_remote_modification_space $(ARCH) $$REMOTE_MACH $$REMOTE_DIR;\
          IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
             cd $$IWD/src/$$COMPOS;\
             create_component_in_modification_space $$COMPOS;\
          done;\
        else\
          echo "Attention : ATLAS_HOME n'est pas defini : ";\
          echo "make create non fait";\
        fi
	@for DIR in exe \
                  exe/$(ARCH) \
                  exe/$(ARCH)/$(OPT) \
                  lib \
                  lib/$(ARCH)/$(OPT);\
          do\
            if [ ! -d $$DIR ] ;then\
              mkdir $$DIR; chmod g+w $$DIR;\
              echo "---> creation de $$DIR";\
            fi\
          done

# remise a zero des librairies
newlib:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) newlib;\
         done

# generation des dependances
depend:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) depend;\
         done

# purges
clean: 
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) clean;\
         done

# purges plus severes
mrproper:
	@IWD=`pwd`;for COMPOS in `cat cfg/composantes/Lmodification`;do\
           cd $$IWD/src/$$COMPOS;$(MAKE) mrproper;\
         done

# creation de l'espace de distribution
install:
