#
# Makefile servant a fabriquer la composante $(COMPOS)
# dans l'espace de Reference.
# make doit etre execute depuis le repertoire src/$(COMPOS)
#
# Version originale : Projet OURANOS
#
# Modifie par Pierre Brochard : CEA/DAM-Ile de France de 1992 a 2000
#
# sccs_ID           : # 1.5 23/01/08	
#

# On lance un Bourne-shell
SHELL = /bin/sh
PROJECT_PWD = .
EXTRACT_INCLUDE = Y

# Inclusion des fichiers de definition de macros (Attention a l'ordre !)
# Macros globales a l'ensemble des projets geres sous atlas
include $(ATLAS_HOME)/resources/Global_Macros.mk
# Macros globales au projet (pour toutes les architectures)
include $(PROJECT_HOME)/cfg/machines/Global_Macros.mk
# Macros specifiques de l'architecture de la cible
include $(PROJECT_HOME)/cfg/machines/$(ARCH)/macros.mk
# Macros specifiques a une option de compilation
include $(PROJECT_HOME)/cfg/machines/$(ARCH)/options$(OPT).mk
# Macros specifiques a la composante $(COMPOS)
include $(PROJECT_PWD)/.special_options.mk

# Definition de macros utiles
# LIBRAIRIES : Repertoire des bibliotheques d'archivage creees (format ar)
# INCLUDES Liste des repertoires d'include explores par l'outil de recherche
# des dependances
# .include contient les includes PRIVES extraits et ../include 
# les includes PUBLICS
# ALL_INCLUDES reprend tous les includes
# TEMPLATE_INCLUDES contient les definitions des templates C++ (.C de meme nom
# que les .h contenant les declarations)
# TEMPLATE_DIR est la racine du repertoire servant a instancier les
# templates (Templates.DB sur SUN par exemple). 
# EXECUTES : Repertoire des executables crees
# EXE : Nom de l'executable de la composante $(COMPOS)
# LIB : Nom de la bibliotheque d'archivage de la composante $(COMPOS) et
# noms des trois liens symboliques vers elle

PROJECTCFG = $(PROJECT_HOME)/cfg
LIBRAIRIES = $(PROJECT_HOME)/lib/$(ARCH)/$(OPT)
INCLUDES   = -I$(PROJECT_HOME)/src/$(COMPOS)/.include \
		-I$(PROJECT_HOME)/src/include
ALL_INCLUDES = $(INCLUDES) $(SPECIAL_INCLUDES)
TEMPLATE_INCLUDES = -I$(PROJECT_HOME)/src/include/templates
TEMPLATE_DIR = $(PROJECT_HOME)/lst/$(ARCH)/$(OPT)/$(COMPOS)
EXECUTES   = $(PROJECT_HOME)/exe/$(ARCH)/$(OPT)
EXE        = $(EXECUTES)/$(COMPOS)
LIB        = $(LIBRAIRIES)/lib$(COMPOS).a
LIBAR      = $(LIBRAIRIES)/lib$(COMPOS).a
LST        = $(PROJECT_HOME)/lst/$(ARCH)/$(OPT)/$(COMPOS)
LIBO       = ../../../lib/$(ARCH)/$(OPT)/lib$(COMPOS).a
LIBC       = $(LST)/.objc.a
LIBC++     = $(LST)/.objc++.a
LIBF       = $(LST)/.objf.a
LIBY       = $(LST)/.objy.a
# Nom des fichiers de dependances C, C++ et F77
FDEPA      = .depend

# Destruction des regles implicites du make
# (ne sont jamais les memes suivant les machines)
.h~.h :;
.f~.f :;
.c~.c :;
.C~.C :;
.y~.y :;
.f.a  :;
.c.a  :;
.C.a  :;
.y.a  :;
.f~.a :;
.c~.a :;
.C~.a :;
.y~.a :;

# Extraction des include et compilation 
debut:
	@$(MAKE) init EXTRACT_INCLUDE=$(EXTRACT_INCLUDE)
	-@$(MAKE) template
# Generation d'une makefile contenant les dependances C, C++ et F77
	@IWD=`pwd`;cd $(LST);$(TOUCH) $(FDEPA);$(RM) xx;\
         cat $$IWD/makefile $(FDEPA) > xx
# Lancement recursif de make sur l'ensemble des fichiers C de la 
# composante pour compilation
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.c 2> /dev/null|wc -c`" -ne "0" ] ;then \
           $(MAKE) $(LIBC) PROJECT_PWD=`pwd` OBJECTS="`(ls s.*.c) 2> /dev/null|\
            grep -v s.main.c|sed 's+^s\.+$(LIB)(+;s+\.c$$+.o)+'|\
            tr '\012' ' '`" -f $(LST)/xx;fi
# Lancement recursif de make sur l'ensemble des fichiers C++ de la 
# composante pour compilation
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.C 2> /dev/null|wc -c`" -ne "0" ] ;then \
           $(MAKE) $(LIBC++) PROJECT_PWD=`pwd` \
           OBJECTS="`(ls s.*.C) 2> /dev/null|grep -v s.main.C | sed \
           's+^s\.+$(LIB)(+;s+\.C$$+.o)+'|tr '\012' ' '`" -f $(LST)/xx;fi
# Lancement recursif de make sur l'ensemble des fichiers F77 de la composante 
# pour compilation
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.f 2> /dev/null|wc -c`" -ne "0" ] ;then \
           $(MAKE) $(LIBF) PROJECT_PWD=`pwd` OBJECTS="`(ls s.*.f) 2> /dev/null|\
           grep -v s.main.f | sed 's+^s\.+$(LIB)(+;s+\.f$$+.o)+'|\
           tr '\012' ' '`" -f $(LST)/xx;fi
# Lancement recursif de make sur l'ensemble des fichiers yacc de la composante 
# pour compilation
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.y 2> /dev/null|wc -c`" -ne "0" ] ;then \
           $(MAKE) $(LIBY) PROJECT_PWD=`pwd` OBJECTS="`(ls s.*.y) 2> /dev/null|\
           grep -v s.main.y | sed 's+^s\.+$(LIB)(+;s+\.y$$+.o)+'|\
           tr '\012' ' '`" -f $(LST)/xx;fi
# Stockage des .o dans la librarie
	@$(MAKE) ranlib

# Indication a make du caractere "precieux" des bibliotheques
# (il ne les detruit pas en cas de probleme)
.PRECIOUS : $(LIB) $(LIBC) $(LIBC++) $(LIBF) $(LIBY)

lib:
	@$(MAKE) debut EXTRACT_INCLUDE=$(EXTRACT_INCLUDE)
	@$(MAKE) otemain

# Compilation et edition de liens
load:
	@$(MAKE) debut
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ -f s.main.[cCf] ] ;then \
	$(MAKE) $(EXE) SRC=`ls s.main.[cCf] 2> /dev/null`;fi

# Edition de liens
link:
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ -f s.main.[cCf] ] ;then \
	$(MAKE) init;$(MAKE) $(EXE) SRC=`ls s.main.[cCf] 2> /dev/null`;fi

otemain:
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ -f $(LIB) -a "`$${NPREFIX}$(AR) t $(LIB)|\
		grep '^main.o$$' | wc -l`" -eq 1 ] ;then\
           $${NPREFIX}$(AR) d $(LIB) main.o;$(TOUCH) -c $(LIB);$(RANLIB) $(LIB);\
           echo "---> make otemain : main enleve";\
         else\
           echo "---> make otemain : pas de main trouve";\
         fi

# Generation de l'executable
$(EXE) : $(LIB) $(LIBS) $(SRC)
	@echo "---> link de : $@"
	-@$(RMEXE)
	@cd $(LST);$(RM) __* *.o;if [ -f $(LIB) ] ;then $(AR) x $(LIB);fi
	-@IWD=`pwd`;$(MAKEMODEXESUP)
	-@IWD=`pwd`;cd $(LST);if [ ! -f main.o ];then\
            echo "Fabrication du main";\
            $(RM) main.[cCf];$(GET) $$IWD/s.main.[cCf];\
            if [ -f main.f ] ;then\
              $(CPP) $(ALL_INCLUDES) main.f > main.f.temp;$(RM) main.f;\
                      mv main.f.temp main.f;$(FC) -c main.f;\
            elif [ -f main.c ] ;then\
              $(CC) $(ALL_INCLUDES) -c main.c;\
            elif [ -f main.C ] ;then\
              $(CC++) $(ALL_INCLUDES) -c main.C;\
            fi;\
          fi;\
          if [ -f main.o ];then\
            $(LD) -o $@ *.o $(LIBMOD) $(LIBS) $(ADDLIB);\
            $(TOUCH) -c $@;\
          else\
            echo "Executable non genere car pas de main.o";\
          fi
	-@chmod 750 $@;chgrp $(USER_GROUP) $@
	@cd $(LST);$(RM) *.o

# Generation de la bibliotheque d'archive pour les sources C et mise a
# jour du fichier des dependances C
$(LIBC) : $(OBJECTS)
	@echo "---> compilation de : $(?:.o=.c)"
	-@for fic in $(?:.o=.c);do \
           $(RM) $(LST)/$$fic;$(GET) -p s.$$fic > $(LST)/$$fic;done
	-@cd $(LST); \
          if [ -x $(MKDEP) ] ;then \
            $(MKDEP) -sccs $(ALL_INCLUDES) -L'$$(LIB)' -f $(FDEPA) $(?:.o=.c);\
          fi
	-@cd $(LST);$(CC) $(ALL_INCLUDES) -c $(?:.o=.c);$(TOUCH) -c $?

$(LIBC++) : $(OBJECTS)
	@echo "---> compilation de : $(?:.o=.C)"
	-@for fic in $(?:.o=.C);do \
           $(RM) $(LST)/$$fic;$(GET) -p s.$$fic > $(LST)/$$fic;done
	-@cd $(LST); \
          if [ -x $(MKDEP) ] ;then \
            $(MKDEP) -sccs $(ALL_INCLUDES) -L'$$(LIB)' -f $(FDEPA) $(?:.o=.C);\
          fi
	-@cd $(LST);$(CC++) $(ALL_INCLUDES) -c $(?:.o=.C);$(TOUCH) -c $?

# Generation de la bibliotheque d'archive pour les sources F77 et
# mise a jour du fichier des dependences F77
$(LIBF) : $(OBJECTS)
	@echo "---> compilation de : $(?:.o=.f)"
	-@for fic in $(?:.o=.f);do \
            $(RM) $(LST)/$$fic;$(GET) -p s.$$fic > $(LST)/$$fic; \
          done
	-@cd $(LST); \
          if [ -x $(MKDEP) ] ;then \
            $(MKDEP) -sccs $(ALL_INCLUDES) -L'$$(LIB)' -f $(FDEPA) $(?:.o=.f);\
          fi
	-@cd $(LST); \
          for fic in $(?:.o=.f);do \
            $(CPP) $(ALL_INCLUDES) $$fic > $$fic.tmp; \
            $(RM) $$fic; mv $$fic.tmp $$fic; \
          done;$(FC) -c $(?:.o=.f);$(TOUCH) -c $?

# Generation de la bibliotheque d'archive pour les sources yacc et
# mise a jour du fichier des dependences
$(LIBY) : $(OBJECTS)
	@echo "---> compilation de : $(?:.o=.y)"
	-@for fic in $(?:.o=.y);do \
            $(RM) $(LST)/$$fic;$(GET) -p s.$$fic > $(LST)/$$fic; \
          done
	-@cd $(LST); \
          if [ -x $(MKDEP) ] ;then \
            $(MKDEP) -sccs $(ALL_INCLUDES) -L'$$(LIB)' -f $(FDEPA) $(?:.o=.y);\
          fi
	-@cd $(LST); \
          for fic in $(?:.o=.y);do \
             $(RM) y.tab.c;$(AYACC) $$fic;ficc="`basename $$fic .y`.c";\
             $(RM) $$ficc;mv -f y.tab.c $$ficc;\
             $(CC) $(ALL_INCLUDES) -c $$ficc;\
          done;$(TOUCH) -c $?

create :
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	echo "---> appel de make create";\
        if [ "$(ATLAS_HOME)" != "" ] ;then\
          if [ "`printenv PATH | grep $ATLAS_HOME/bin`" = "" ] ;then\
            PATH=${ATLAS_HOME}/bin:`printenv PATH`;export PATH;\
          fi;\
          create_project;cd ../..;\
          PARAMETER_FILE=cfg/machines/$(ARCH)/remote_project.sh;\
          if [ -f $$PARAMETER_FILE ] ;then\
            . $$PARAMETER_FILE;\
          fi;\
          if [ "$(MACHCOMP)" != "" ] ;then\
            if [ "$(MACHCOMP)" = "NONE" ] ;then\
              REMOTE_MACH=;\
            else\
              REMOTE_MACH=$(MACHCOMP);\
            fi;\
          fi;\
          if [ -z "$$REMOTE_MACH" ] ;then\
            REMOTE_MACH="`atlas_name`";\
          fi;\
          if [ "$(DIRCOMP)" != "" ] ;then\
            if [ "$(DIRCOMP)" = "NONE" ] ;then\
              REMOTE_DIR=;\
            else\
              REMOTE_DIR=$(DIRCOMP);\
            fi;\
          fi;\
          create_remote_project $(ARCH) $$REMOTE_MACH $$REMOTE_DIR;\
          create_component $(COMPOS);\
        else\
          echo "Attention : ATLAS_HOME n'est pas defini : ";\
          echo "make create non fait";\
        fi
	@for DIR in \
          $(PROJECT_HOME)/lib \
	  $(PROJECT_HOME)/exe \
	  $(PROJECT_HOME)/lst \
	  $(PROJECT_HOME)/lib/$(ARCH) \
	  $(PROJECT_HOME)/exe/$(ARCH) \
	  $(PROJECT_HOME)/lst/$(ARCH) \
	  $(PROJECT_HOME)/lib/$(ARCH)/$(OPT) \
	  $(PROJECT_HOME)/exe/$(ARCH)/$(OPT) \
	  $(PROJECT_HOME)/lst/$(ARCH)/$(OPT) \
	  $(PROJECT_HOME)/lst/$(ARCH)/$(OPT)/$(COMPOS); \
	  do\
	    if [ ! -d $$DIR ] ;then\
              mkdir $$DIR; chmod g+w $$DIR;\
              echo "---> creation de $$DIR";\
	    fi\
          done
	-@$(RM) $(LIBC) $(LIBC++) $(LIBF) $(LIBY)
	-@if [ "$(OPT)" = "" ] ;then \
	    $(LN) $(LIBO) $(LIBC);$(LN) $(LIBO) $(LIBC++);\
            $(LN) $(LIBO) $(LIBF);$(LN) $(LIBO) $(LIBY); \
	  else \
	    $(LN) ../$(LIBO) $(LIBC);$(LN) ../$(LIBO) $(LIBC++);\
            $(LN) ../$(LIBO) $(LIBF);$(LN) ../$(LIBO) $(LIBY);fi

#   Creation de la librairie avec trois liens :
#   LIBC, LIBC++, LIBY et LIBF "pointent" sur LIB
newlib :
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	echo "---> appel de make newlib";TESTLIB=GOOD;\
          for DIR in \
            $(PROJECT_HOME)/src/include \
            $(PROJECT_HOME)/src/$(COMPOS)/.include \
            $(PROJECT_HOME)/lib \
	    $(PROJECT_HOME)/exe \
	    $(PROJECT_HOME)/lst \
	    $(PROJECT_HOME)/lib/$(ARCH) \
	    $(PROJECT_HOME)/exe/$(ARCH) \
	    $(PROJECT_HOME)/lst/$(ARCH) \
	    $(PROJECT_HOME)/lib/$(ARCH)/$(OPT) \
	    $(PROJECT_HOME)/exe/$(ARCH)/$(OPT) \
	    $(PROJECT_HOME)/lst/$(ARCH)/$(OPT)/$(COMPOS); \
	  do\
	    if [ ! -d $$DIR ] ;then\
              TESTLIB=BAD;\
	    fi\
          done;\
          if [ "$$TESTLIB" = "BAD" -o \
              ! -f $(PROJECT_HOME)/src/include/makefile -o \
              ! -f $(PROJECT_HOME)/src/$(COMPOS)/.include/makefile ] ;then\
            $(MAKE) create MACHCOMP=NONE;\
          fi
	@$(RM) $(LIBC) $(LIBC++) $(LIBF) $(LIBY) $(LIB) $(LST)/$(FDEPA) \
                $(LST)/$(FDEPA)[fc]
	@$(TOUCH) $(LST)/$(FDEPA)
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	  cd $(LST);echo "void atlas_ar_bidon(){}" > xyz.c;\
	  $${NPREFIX}$(CC) -c xyz.c;\
	  $${NPREFIX}$(AR) r $(LIB) $(LST)/xyz.o;$${NPREFIX}$(AR) d $(LIB) xyz.o;\
          $(TOUCH) -c $(LIB);$(RANLIB) $(LIB);$(RM) $(LST)/xyz.*
	@chmod 660 $(LIB)
	@if [ "$(OPT)" = "" ] ;then \
	    $(LN) $(LIBO) $(LIBC);$(LN) $(LIBO) $(LIBC++);\
            $(LN) $(LIBO) $(LIBF);$(LN) $(LIBO) $(LIBY); \
	  else \
	    $(LN) ../$(LIBO) $(LIBC);$(LN) ../$(LIBO) $(LIBC++);\
            $(LN) ../$(LIBO) $(LIBF);$(LN) ../$(LIBO) $(LIBY);fi
	@$(MAKE) clean
	-@$(MAKE) template
	-@if [ "$(EXTRACT_INCLUDE)" = "Y" ] ;then $(MAKE) extract_include;fi
	-@$(MAKE) depend

extract_include:
	-@echo "---> appel de make extract_include"
	-@$(MAKE) extract_include_public
	-@$(MAKE) extract_include_private

extract_include_public:
	@if [ ! -d $(PROJECT_HOME)/src/include -o \
           ! -f $(PROJECT_HOME)/src/include/makefile ] ;then\
             $(MAKE) create;\
         fi
	-@cd ../include;if [ ! -f .public_include_list ] ;then\
            $(TOUCH) .public_include_list; fi;\
          for i in `ls -d * 2> /dev/null` ;do \
           if [ -d "$$i" -a -d "$$i/$(SCCSDIR)" ] ;then \
            $(TOUCH) $$i/.public_include_list;fi;done;\
          $(MAKE) newlib;$(MAKE) debut

extract_include_private:
	@if [ ! -d $(PROJECT_HOME)/src/$(COMPOS)/.include -o \
           ! -f $(PROJECT_HOME)/src/$(COMPOS)/.include/makefile ] ;then\
             $(MAKE) create;\
         fi
	-@cd .include; if [ ! -f .private_include_list ] ;then\
            $(TOUCH) .private_include_list; fi;\
          $(MAKE) newlib;$(MAKE) debut

clean:
	@$(RM) $(LST)/*.o $(LST)/xx;$(RMEXE)

mrproper:
	-@$(RM) $(LST)/*.o $(LST)/xx;$(RMEXE)
	-@$(RM) -r $(PROJECT_HOME)/lst/$(ARCH)/*
	-@$(RM) -r $(PROJECT_HOME)/lib/$(ARCH)/*

#       Verification coherence fichiers .o et fichiers .f et .c
#    On peut rajouter dans la boucle, pour supprimer les objets perimes ...
#        ar dv $(LIB) $$r.o
coher:
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	  radic=`$${NPREFIX}$(AR) t $(LIB)|sed 's/\.o//;/__\.SYMDEF/d'`;\
	  for r in $$radic; do \
	    if [ ! -f "s.$$r.c" -a ! -f "s.$$r.C" -a ! -f "s.$$r.f" ] ; then\
	      echo "---> le source de $$r.o n'existe plus";\
              $${NPREFIX}$(AR) dv $(LIB) $$r.o;\
	    fi;\
	  done

coherlst:
	@source=`(ls $(LST)/*.[cyfC]) 2> /dev/null|sed 's+\$(LST)/++'`;\
	for f in $$source; do \
	  if [ ! -f "s.$$f" ] ; then\
	    echo "---> le source de $$f n'existe plus";\
            $(RM) $(LST)/$$f;\
	  fi;\
	done

depend :
	@echo "---> appel de make depend"
	@cd $(LST);$(RM) $(FDEPA);$(TOUCH) $(FDEPA)
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	srcc=`ls s.*.[cfyC] 2> /dev/null|sed 's+^s\.++'`;\
	if [ -x $(MKDEP) ] ;then \
	  IWD=`pwd`;cd $(LST);\
	  for f in $$srcc; do\
            if [ ! -f $$f ] ;then $(TOUCH) $$f;fi;\
            $(RM) $$f.temp;mv -f $$f $$f.temp;\
	    $(GET) -p $$IWD/s.$$f > $$f;\
            $(MKDEP) -sccs $(ALL_INCLUDES) -L'$$(LIB)' -f $(FDEPA) $$f;\
            $(RM) $$f;mv -f $$f.temp $$f;\
	  done; \
	else \
	  for f in `ls s.*.[cfyC] 2> /dev/null` ; do\
	    echo "`echo $$f 2> /dev/null |\
              sed 's+^s\.+# +'|tr -d '\012'`" >> $(LST)/$(FDEPA);\
	    echo "`echo $$f 2> /dev/null |\
              sed 's+^s\.+$$\(LIB\)(+;s+\.[cfyC]$$+.o)+'|\
              tr -d '\012'` : $$f" >> $(LST)/$(FDEPA);\
            echo '	@$$(MKDEP_ACTION)' >> $(LST)/$(FDEPA);\
	  done;\
	  if [ "$(COMPOS)" != mkdep ] ;then\
            echo "********** Attention : pas de mkdep **********";fi;\
	fi

init :
	@echo "---> Traitement de la composante $(COMPOS)"
# Initialisation des repertoires de travail et des fichiers de dependances
	@if [ ! -d $(PROJECT_HOME)/src/include -o \
           ! -f $(PROJECT_HOME)/src/include/makefile -o \
           ! -d $(PROJECT_HOME)/src/$(COMPOS)/.include -o \
           ! -f $(PROJECT_HOME)/src/$(COMPOS)/.include/makefile -o \
           ! -d $(LST) -o ! -f $(LIB) -o ! -f $(LIBC) -o \
           ! -f $(LIBC++) -o ! -f $(LIBF) -o ! -f $(LIBY) -o \
           ! -d $(PROJECT_HOME)/lib/$(ARCH)/$(OPT) -o \
           ! -d $(PROJECT_HOME)/exe/$(ARCH)/$(OPT) -o \
           "`echo \`cat $(LIB) 2>/dev/null| wc -c\``" -eq "0" ] ;then\
             $(MAKE) newlib;\
         else \
           if [ "$(EXTRACT_INCLUDE)" = "Y" ] ;then $(MAKE) extract_include;fi;\
           $(MAKE) depend;\
         fi
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "X$$XHOST" != "X" -a "X$$XCROSS" != "X" ] ;then \
	  echo "---> Copie des libs de cross compilation";\
          for j in `cat $(PROJECTCFG)/Lcross` ;do \
	    i=`basename $$j`;subd=`dirname $$j`;\
	    [ "$$subd" = '.' ] && subd= || subd="/$$subd";\
	    [ ! -d $(LIBRAIRIES)$$subd ] && mkdir $(LIBRAIRIES)$$subd || true;\
	    if [ "$$XCROSSLIB" = yes -o ! -f $(LIBRAIRIES)$$subd/$$i ] ;then \
	      HOMECROSS=$$XCROSS/$$XHOST/$$XHOST;\
	      if [ -f $$HOMECROSS/lib*/$${i} ] ;then \
		echo "       - $$i copiee";\
	      	cp -af $$HOMECROSS/lib*/$${i} $(LIBRAIRIES)$$subd;\
		CRLIB=$(LIBRAIRIES)$$subd/$${i};\
		OTDLIB=`ls $$HOMECROSS/lib*/$${i}*[0-9] 2> /dev/null|tail -1`;\
	        [ -f "$$OTDLIB" ] && \
	      	  cp -af $$HOMECROSS/lib*/$${i}*[0-9] $(LIBRAIRIES)$$subd;\
	        TDLIB=`ls $(LIBRAIRIES)$$subd/$${i}*[0-9] 2> /dev/null|tail -1`;\
	        if [ ! -h "$$TDLIB" -a -f "$$TDLIB" ] ;then \
		  if [ "$(OPT)" != DBG ] ; then \
	            cp -a "$$TDLIB" "$${TDLIB}.temp";\
		    $${NPREFIX}$(STRIP) "$${TDLIB}.temp";\
	            mv "$${TDLIB}.temp" "$$TDLIB";fi;\
		elif [ ! -h "$$CRLIB" -a -f "$$CRLIB" ] ;then \
		  chmod u+w "$${CRLIB}";\
	          cp -a "$$CRLIB" "$${CRLIB}.temp";\
		  chmod u+w "$${CRLIB}.temp";\
		  $${NPREFIX}$(STRIP) "$${CRLIB}.temp";\
	          mv "$${CRLIB}.temp" "$$CRLIB";fi;\
		fi;fi;done;fi

strip :
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	[ "$(OPT)" != DBG ] && $${NPREFIX}$(STRIP) $(EXE) || true

libdyna :
	@echo "---> creation lib dynamique :"
	@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	cd $(LST);\
          if [ "$${NPREFIX}$(LDY)" != "" ] ;then\
            if [ -f $(LIB) ] ;then\
              $${NPREFIX}$(AR) xv $(LIB);\
              $${NPREFIX}$(LDY) -o lib$(COMPOS)$(LDYSUFF) *.o;\
              mv lib$(COMPOS)$(LDYSUFF) $(LIBRAIRIES);\
	      [ "$(OPT)" != DBG ] && \
		$${NPREFIX}$(STRIP) $(LIBRAIRIES)/lib$(COMPOS)$(LDYSUFF);\
	      $(RM) *.o;\
            fi;\
          else \
            echo "Error : lib non cree : ";\
            echo "definissez LDY et LDYSUFF dans "\
                 "cfg/machines/$(ARCH)/macros.mk";\
	  fi

template :
	@echo "---> init pour template C++ :";IWD=`pwd`;\
         XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	 cd ../include/templates;$(RM) *;$(LN) ../*/*.C .;cd $$IWD;\
         $(TEMPLATE_ACT)

partiel :
#
# usage : make partiel SRC=liste de fichiers *.c *.C *.f *.y
#
	@$(MAKE) init EXTRACT_INCLUDE=$(EXTRACT_INCLUDE)
	-@$(MAKE) template
	@IWD=`pwd`;cd $(LST);$(TOUCH) $(FDEPA);$(RM) xx;\
         cat $$IWD/makefile $(FDEPA) > xx
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.c 2> /dev/null|wc -c`" -ne "0" ] ;then \
          $(MAKE) $(LIBC) PROJECT_PWD=`pwd` OBJECTS="`echo $(SRC) |\
          tr ' ' '\012' | sed 's+^+$(LIB)\(+;s+\.c$$+\.o\)+;s+^.*\.[Cfy]$$++'|\
          tr '\012' ' '`" -f $(LST)/xx;fi
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.C 2> /dev/null|wc -c`" -ne "0" ] ;then \
          $(MAKE) $(LIBC++) PROJECT_PWD=`pwd` OBJECTS="`echo $(SRC) |\
          tr ' ' '\012' | sed 's+^+$(LIB)\(+;s+\.C$$+\.o\)+;s+^.*\.[cfy]$$++'|\
          tr '\012' ' '`" -f $(LST)/xx;fi
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.f 2> /dev/null|wc -c`" -ne "0" ] ;then \
          $(MAKE) $(LIBF) PROJECT_PWD=`pwd` OBJECTS="`echo $(SRC) |\
          tr ' ' '\012' |sed 's+^+$(LIB)\(+;s+\.f$$+\.o\)+;s+^.*\.[cCy]$$++'|\
          tr '\012' ' '`" -f $(LST)/xx;fi
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	if [ "`ls s.*.y 2> /dev/null|wc -c`" -ne "0" ] ;then \
          $(MAKE) $(LIBY) PROJECT_PWD=`pwd` OBJECTS="`echo $(SRC) |\
          tr ' ' '\012' |sed 's+^+$(LIB)\(+;s+\.y$$+\.o\)+;s+^.*\.[cCf]$$++'|\
          tr '\012' ' '`" -f $(LST)/xx;fi
	@$(MAKE) ranlib

ranlib :
	-@XCOMP=yes;export XCOMP;. $(PROJECTCFG)/profile;\
	IWD=`pwd`;cd $(LST); \
          if [ "`ls *.o 2> /dev/null`" != "" ] ;then\
            $${NPREFIX}$(ARR) $(LIBAR) *.o;$(TOUCH) -c $(LIB);$(RANLIB) $(LIB);\
          fi
	@cd $(LST);$(RM) xx *.o;$(TOUCH) $(FDEPA)

include $(PROJECT_HOME)/cfg/machines/Target_Sup.mk
